package com.pallas.ogsa.client;

import java.net.Socket;
import java.net.URL;
import java.security.cert.Certificate;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.apache.axis.client.Service;
import org.gridforum.www.ogsi.LocatorType;
import org.unicore.AJOIdentifier;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.Reply;

import com.pallas.ogsa.extensions.ServiceReferenceType;
import com.pallas.ogsa.registry.ISiteRegistry;
import com.pallas.ogsa.registry.SiteRegistryStub;
import com.pallas.ogsa.services.caservice.CAServiceStub;
import com.pallas.ogsa.services.caservice.ICAService;
import com.pallas.ogsa.services.upl.IUPL;
import com.pallas.ogsa.services.upl.UPLStub;
import com.pallas.ogsa.util.ObjectWrapperBean;

/**
 *@author     Ralf Ratering
 *@created    6. Februar 2003
 *@version    $Id: Client.java,v 1.3 2003/07/03 06:49:45 rrate Exp $
 */
public class Client {
	private static Logger logger = Logger.getLogger("com.pallas.ogsa.client");

	/**
	 *  The main program for the Server class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		try {
			Client client = new Client(9000);
			//            client.testRegistry();
			client.testCAService();
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Client crashed.", e);
		}
	}
	private int port;
	private final String registryAddress =
		"http://localhost:9001/unicore/services/SiteRegistry?WSDL";
	private final QName registryName =
		new QName("http://services.ogsa.pallas.com", "SiteRegistry");
	private Socket socket;

	/**
	 *  Constructor for the Server object
	 */
	public Client() {
		this(9000);
	}

	/**
	 *  Constructor for the Server object
	 *
	 *@param  port  Description of the Parameter
	 */
	public Client(int port) {
		this.port = port;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  wsdlAddress    Description of the Parameter
	 *@param  serviceName    Description of the Parameter
	 *@exception  Exception  Description of the Exception
	 */
	public void consignJob(String wsdlAddress, QName serviceName)
		throws Exception {
		URL wsdlUrl = new URL(wsdlAddress);

		Service service = new Service(wsdlUrl, serviceName);

		// next line won't find the serializers
		//IUPL upl = (IUPL)service.getPort(new QName("http://services.ogsa.pallas.com", "IUPL"), IUPL.class);
		// ...use stub class instead
		IUPL upl = new UPLStub(wsdlUrl, service);

		logger.info("Consigning job...");
		ConsignJob job = new ConsignJob();
		//String job = "Hey, I am just a String!";
		ObjectWrapperBean wrappedJob = new ObjectWrapperBean(job);
		ObjectWrapperBean wrappedReply = upl.consign(wrappedJob);
		//ObjectWrapperBean wrappedReply = port.consign(job);
		Reply reply = (Reply) wrappedReply.deserialize();
		logger.info("Consign Job Reply: " + reply.getLastEntry().getComment());

		logger.info("Retrieving outcome...");
		AJOIdentifier id = new AJOIdentifier("My Test Job");
		ObjectWrapperBean wrappedId = new ObjectWrapperBean(id);
		wrappedReply = upl.retrieveOutcome(wrappedId);
		reply = (Reply) wrappedReply.deserialize();
		logger.info("Consign Job Reply: " + reply.getLastEntry().getComment());

		logger.info("Sending retrieve outcome acknowledge...");
		wrappedReply = upl.retrieveOutcomeAck(wrappedId);
		reply = (Reply) wrappedReply.deserialize();
		logger.info("Consign Job Reply: " + reply.getLastEntry().getComment());

		logger.info("FEEEERTIG!!!");
	}

	/**
	 *  Gets the firstLocatorFromRegistry attribute of the Client object
	 *
	 *@param  wsdlAddress    Description of the Parameter
	 *@param  serviceName    Description of the Parameter
	 *@return                The firstLocatorFromRegistry value
	 *@exception  Exception  Description of the Exception
	 */
	public LocatorType getFirstLocatorFromRegistry(
		String wsdlAddress,
		QName serviceName)
		throws Exception {
		logger.info("Retrieving site locators...");

		URL wsdlUrl = new URL(wsdlAddress);
		Service service = new Service(wsdlUrl, serviceName);
		ISiteRegistry registry = new SiteRegistryStub(wsdlUrl, service);

		return registry.getSites()[0];
	}

	/**
	 *  A unit test for JUnit
	 *
	 *@exception  Exception  Description of the Exception
	 */
	public void testCAService() throws Exception {
		URL wsdlUrl =
			new URL("http://localhost:9000/unicore/services/CAService?WSDL");
		Service service =
			new Service(
				wsdlUrl,
				new QName("http://services.ogsa.pallas.com", "CAService"));

		// next line won't find the serializers
		//IUPL upl = (IUPL)service.getPort(new QName("http://services.ogsa.pallas.com", "IUPL"), IUPL.class);
		// ...use stub class instead
		ICAService caService = new CAServiceStub(wsdlUrl, service);

		logger.info("Getting CA certificate...");

		ObjectWrapperBean wrappedCert = caService.getCACertificate();
		Certificate cert = (Certificate) wrappedCert.deserialize();
		logger.info("CA certificate: " + cert);

	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	public void testRegistry() throws Exception {
		LocatorType siteLocator =
			getFirstLocatorFromRegistry(registryAddress, registryName);
		ServiceReferenceType reference =
			(ServiceReferenceType) siteLocator.getServiceReference();
		consignJob(reference.getWsdlURL(), reference.getQName());
	}
}
