package com.pallas.ogsa.registry;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;

import org.gridforum.www.ogsi.HandleType;
import org.gridforum.www.ogsi.LocatorType;

import com.pallas.util.notification.Listener;

/**
 *@author     Ralf Ratering
 *@created    11. M?rz 2003
 *@version    $Id: SiteRegistry.java,v 1.4 2003/07/04 09:12:04 thomas Exp $
 */
public class SiteRegistry implements ISiteRegistry {

	private static ArrayList listeners = new ArrayList();
	private static Logger logger = Logger.getLogger("com.pallas.ogsa.registry");
	private static ArrayList sites = new ArrayList();

	/**
	 *  Adds a feature to the Listener attribute of the SiteRegistry object
	 *
	 *@param  listener  The feature to be added to the Listener attribute
	 */
	public static void addListener(Listener listener) {
		listeners.add(listener);
	}

	/**
	 *  Constructor for the SiteRegistry object
	 */
	public SiteRegistry() {
	}

	/**
	 *  FOR NOW: Simply add a new Locator
	 *
	 *@param  siteLocator          Description of the Parameter
	 *@exception  RemoteException  Description of the Exception
	 */
	public void add(LocatorType siteLocator) throws RemoteException {
		sites.add(siteLocator);
		logger.info("Registered Site: " + siteLocator.getServiceHandle().getValue()+
			" Reference: "+siteLocator.getServiceReference().toString());
		updateListeners();
	}

	/**
	 *  Gets the sites attribute of the SiteRegistry object
	 *
	 *@return    The sites value
	 */
	public LocatorType[] getSites() {
		return (LocatorType[]) sites.toArray(new LocatorType[sites.size()]);
	}

	/**
	 *  FOR NOW: Remove first occurence of Service Handle
	 *
	 *@param  serviceHandle        Description of the Parameter
	 *@exception  RemoteException  Description of the Exception
	 */
	public void remove(HandleType serviceHandle) throws RemoteException {
		Iterator iterator = sites.iterator();
		while (iterator.hasNext()) {
			LocatorType locator = (LocatorType) iterator.next();
			if (serviceHandle.equals(locator.getServiceHandle())) {
				sites.remove(locator);
				return;
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	private void updateListeners() {
		Iterator iterator = listeners.iterator();
		while (iterator.hasNext()) {
			Listener listener = (Listener) iterator.next();
			listener.update();
		}
	}

}
