package com.pallas.ogsa.registry;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;

import org.gridforum.www.ogsi.LocatorType;

import com.pallas.ogsa.server.ServerFrame;
import com.pallas.util.notification.Listener;

/**
 *@author     Ralf Ratering
 *@created    11. M?rz 2003
 *@version    $Id: SiteRegistryServerFrame.java,v 1.6 2003/07/08 09:52:42 thomas Exp $
 */
public class SiteRegistryServerFrame extends ServerFrame {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    20. M?rz 2003
	 */
	private class SiteListModel extends DefaultListModel implements Listener {
		private SiteRegistry registry = new SiteRegistry();
		/**
		 *  Constructor for the SiteListModel object
		 */
		public SiteListModel() {
		}

		/**
		 *  Description of the Method
		 */
		public void update() {
			logger.info("Updating site list: " + registry.getSites());
			removeAllElements();
			LocatorType[] sites = registry.getSites();
			for(int i=0; i< sites.length; i++) {
				logger.info("Adding site: " + sites[i]);
				addElement(sites[i]);
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    20. M?rz 2003
	 */
	private class SiteListRenderer extends DefaultListCellRenderer {
		/**
		 *  Gets the ListCellRendererComponent attribute of the FontRenderer
		 *  object
		 *
		 *@param  list          Description of Parameter
		 *@param  value         Description of Parameter
		 *@param  index         Description of Parameter
		 *@param  isSelected    Description of Parameter
		 *@param  cellHasFocus  Description of Parameter
		 *@return               The ListCellRendererComponent value
		 */
		public Component getListCellRendererComponent(
			JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus) {

			super.getListCellRendererComponent(
				list,
				value,
				index,
				isSelected,
				cellHasFocus);
			if (value instanceof LocatorType) {
				LocatorType locator = (LocatorType) value;
				setText(locator.getServiceHandle().getValue().toString());
			}
			return this;
		}
	}

	/**
	 *  The main program for the ServerFrame class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		SiteRegistryServerFrame serverFrame = new SiteRegistryServerFrame();
		serverFrame.setSize(800, 600);
		serverFrame.setVisible(true);
	}
	private SiteListModel siteListModel;
	private JScrollPane siteListScrollPane;

	/**
	 *  Constructor for the ServerFrame object
	 */
	public SiteRegistryServerFrame() {
		setTitle("UNICOREpro Site Registry");
	}

	/**
	 *  Adds a feature to the Components attribute of the SiteRegistryServerFrame
	 *  object
	 */
	protected void addComponents() {
		JSplitPane splitPane =
			new JSplitPane(
				JSplitPane.VERTICAL_SPLIT,
				siteListScrollPane,
				new JScrollPane(logArea));
		getContentPane().add(splitPane, BorderLayout.CENTER);
		getContentPane().add(controlPanel, BorderLayout.SOUTH);
	}

	/**
	 *  Description of the Method
	 */
	protected void buildComponents() {
		super.buildComponents();
		siteListModel = new SiteListModel();
		SiteRegistry.addListener(siteListModel);

		JList siteList = new JList(siteListModel);
		siteList.setCellRenderer(new SiteListRenderer());
		siteListScrollPane = new JScrollPane(siteList);
		siteListScrollPane.setBorder(new TitledBorder("Registered Sites"));
	}

	/**
	 *  Description of the Method
	 */
	protected void deployService() {
		/*
		Deployment deployment = new Deployment(server.getAxisServer());
		deployment.deploy(
			SiteRegistry.class.getName(),
			"SiteRegistry",
			"unicore/services",
			ISiteRegistry.class);
			*/
		httpServer.getDeployment().deploy(
			SiteRegistry.class.getName(),
			"SiteRegistry",
			"unicore/services",
			ISiteRegistry.class, "per-access");
	}
}
