package com.pallas.ogsa.registry;

import java.rmi.RemoteException;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.description.OperationDesc;
import org.apache.axis.description.ParameterDesc;
import org.apache.axis.encoding.XMLType;
import org.gridforum.www.ogsi.HandleType;
import org.gridforum.www.ogsi.LocatorType;

import com.pallas.ogsa.util.OGSIStub;
import com.pallas.ogsa.util.OGSITypeRegistration;


/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    21. Februar 2003
 *@version    $Id: SiteRegistryStub.java,v 1.3 2003/07/03 06:48:31 rrate Exp $
 */
public class SiteRegistryStub extends OGSIStub implements ISiteRegistry {
	/**
	 *@exception  AxisFault  Description of the Exception
	 */
	public SiteRegistryStub() throws AxisFault {
		this(null);
	}

	/**
	 *@param  endpointURL    Description of the Parameter
	 *@param  service        Description of the Parameter
	 *@exception  AxisFault  Description of the Exception
	 */
	public SiteRegistryStub(java.net.URL endpointURL, Service service)
		throws AxisFault {
		this(service);
		super.cachedEndpoint = endpointURL;
	}

	/**
	 *@param  service        Description of the Parameter
	 *@exception  AxisFault  Description of the Exception
	 */
	public SiteRegistryStub(Service service) throws AxisFault {
		if (service == null) {
			super.service = new org.apache.axis.client.Service();
		} else {
			super.service = service;
		}
		init();
		setPortName("ISiteRegistry");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  siteLocator          Description of the Parameter
	 *@exception  RemoteException  Description of the Exception
	 */
	public void add(LocatorType siteLocator) throws RemoteException {
		if (super.cachedEndpoint == null) {
			throw new org.apache.axis.NoEndPointException();
		}
		Call call = createCall();
		call.setOperation((OperationDesc) operations.get("add"));
		call.setOperationName(
			new QName("http://services.ogsa.pallas.com", "add"));

		call.invoke(new Object[] { siteLocator });
	}

	/**
	 *  Gets the sites attribute of the SiteRegistryStub object
	 *
	 *@return                      The sites value
	 *@exception  RemoteException  Description of the Exception
	 */
	public LocatorType[] getSites() throws RemoteException {
		if (super.cachedEndpoint == null) {
			throw new org.apache.axis.NoEndPointException();
		}
		Call call = createCall();
		call.setOperation((OperationDesc) operations.get("getSites"));
		call.setOperationName(
			new QName("http://services.ogsa.pallas.com", "getSites"));
		return (LocatorType[]) call.invoke(new Object[] {
		});
	}

	 protected void init() {

        operations.clear();

        // register
        OperationDesc desc = new OperationDesc();
        desc.setName("add");

        ParameterDesc param =
                new ParameterDesc(new QName("", "siteLocator"),
                ParameterDesc.IN,
                new QName(OGSITypeRegistration.OGSI_NAMESPACE, "LocatorType"));
        desc.addParameter(param);
        desc.setStyle(org.apache.axis.enum.Style.RPC);
        desc.setUse(org.apache.axis.enum.Use.LITERAL);
        desc.setReturnType(XMLType.AXIS_VOID);

        operations.put(desc.getName(), desc);

        // un-register
        desc = new OperationDesc();
        desc.setName("remove");

        param = new ParameterDesc(new QName("", "serviceHandle"),
                ParameterDesc.IN,
                new QName(OGSITypeRegistration.OGSI_NAMESPACE, "HandleType"));
        desc.addParameter(param);
        desc.setStyle(org.apache.axis.enum.Style.RPC);
        desc.setUse(org.apache.axis.enum.Use.LITERAL);
        desc.setReturnType(XMLType.AXIS_VOID);

        operations.put(desc.getName(), desc);

        // get registered sites
        desc = new OperationDesc();
        desc.setName("getSites");
        desc.setReturnType(new QName(OGSITypeRegistration.OGSI_NAMESPACE, "ArrayOf_LocatorType"));
        desc.setReturnClass(LocatorType[].class);
        desc.setReturnQName(new QName("", "getSitesReturn"));
        desc.setStyle(org.apache.axis.enum.Style.RPC);
        desc.setUse(org.apache.axis.enum.Use.LITERAL);

        operations.put(desc.getName(), desc);

    }

	/**
	 *  Description of the Method
	 *
	 *@param  serviceHandle        Description of the Parameter
	 *@exception  RemoteException  Description of the Exception
	 */
	public void remove(HandleType serviceHandle) throws RemoteException {
		if (super.cachedEndpoint == null) {
			throw new org.apache.axis.NoEndPointException();
		}
		Call call = createCall();
		call.setOperation((OperationDesc) operations.get("remove"));
		call.setOperationName(
			new QName("http://services.ogsa.pallas.com", "remove"));

		call.invoke(new Object[] { serviceHandle });
	}

}
