package com.pallas.ogsa.server;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.InputStream;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.axis.MessageContext;
import org.apache.axis.configuration.FileProvider;
import org.apache.axis.deployment.wsdd.WSDDConstants;
import org.apache.axis.deployment.wsdd.WSDDDeployment;
import org.apache.axis.deployment.wsdd.WSDDService;
import org.apache.axis.description.ServiceDesc;
import org.apache.axis.encoding.SerializationContextImpl;
import org.apache.axis.encoding.TypeMapping;
import org.apache.axis.encoding.TypeMappingRegistry;
import org.apache.axis.enum.Style;
import org.apache.axis.enum.Use;
import org.apache.axis.providers.java.RPCProvider;
import org.apache.axis.server.AxisServer;
import org.apache.axis.utils.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *  Description of the Class
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. Februar 2003
 *@version    $Id: Deployment.java,v 1.4 2003/07/08 09:10:10 thomas Exp $
 */
public class Deployment {

	/**
	 *  Description of the Field
	 */
	protected static Logger logger = Logger.getLogger("com.pallas.ogsa.server");
	private org.apache.axis.EngineConfiguration config;
	private WSDDDeployment deployment;
	private AxisServer server;
	private String targetNameSpace = "http://services.ogsa.pallas.com";

	/**
	 *  Constructor for the Deployment object
	 *
	 *@param  server  Description of the Parameter
	 */
	public Deployment(AxisServer server) {
		this.server = server;
		config = server.getConfig();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  deployable   Description of the Parameter
	 *@param  serviceName  Description of the Parameter
	 *@param  serviceBase  Description of the Parameter
	 *@param  portType     Description of the Parameter
	 */
	public void deploy(
		String deployableClassName,
		String serviceName,
		String serviceBase,
		Class portType,
		String accessSope) {

		FileProvider wsddProvider = (FileProvider) config;
		deployment = wsddProvider.getDeployment();
		WSDDService service =
			deployment.getWSDDService(new QName(serviceBase, serviceName));

		if (service == null) {
			service = new WSDDService();
		}

		service.setParameter("className", deployableClassName);
		service.setParameter("wsdlServiceElement", serviceName);
		service.setParameter("targetNameSpace", targetNameSpace);
		service.setParameter("wsdlServicePort", portType.getName());
		service.setParameter("wsdlPortType", portType.getName());
		
		// scope can be per-request per-request or singleton
		// see org/apache/axis/deployment/wsdd/WSDDDeployableItem.java
 
		service.setParameter("scope", accessSope);
		String handlerClass = RPCProvider.class.getName();
		service.setParameter("handlerClass", handlerClass);
		service.setProviderQName(WSDDConstants.QNAME_HANDLER_PROVIDER);
		service.setName(serviceName);
		service.setQName(new QName("", serviceName));

		ServiceDesc desc = service.getServiceDesc();
		String encodingStyle = "";
		TypeMappingRegistry registry = server.getTypeMappingRegistry();
		TypeMapping mapping =
			(TypeMapping) registry.getTypeMapping(encodingStyle);

		MessageContext context = MessageContext.getCurrentContext();
		if (context != null) {
			registry = context.getTypeMappingRegistry();
			mapping = context.getTypeMapping();
		}

		desc.setTypeMappingRegistry(registry);
		desc.loadServiceDescByIntrospection(portType, mapping);
		desc.setStyle(Style.RPC);
		//Style.WRAPPED);
		desc.setUse(Use.LITERAL);
		desc.setName(serviceName);
		desc.setDefaultNamespace(targetNameSpace);
		// service.desc = desc;

		logger.info("Deploying service: " + service + "...");
		deployment.deployService(service);

		String dump = "Deployed services:\n";
		try {
			Iterator iter = config.getDeployedServices();
			while (iter.hasNext()) {
				ServiceDesc sd = (ServiceDesc) iter.next();
				dump += "Name: "
					+ sd.getName()
					+ ", WSDL file:  "
					+ sd.getWSDLFile()
					+ ", Namespace: "
					+ sd.getDefaultNamespace()
					+ "\n";
			}
			logger.info(dump);
		} catch (org.apache.axis.ConfigurationException e) {
			logger.log(Level.WARNING, "Could not get services.", e);
		}

	}

	/**
	 *  Description of the Method
	 */
	public void listServices() {
		try {
			Iterator iter = config.getDeployedServices();
			while (iter.hasNext()) {
				logger.info(((ServiceDesc) iter.next()).getName());
			}
			logger.info("===================================");
		} catch (org.apache.axis.ConfigurationException e) {
			logger.log(Level.WARNING, "Could not get services.", e);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  serviceName    Description of the Parameter
	 *@exception  Exception  Description of the Exception
	 */
	public void loadService(String serviceName) throws Exception {
		InputStream input = null;
		Document doc = null;
		try {
			try {

				input = new FileInputStream(serviceName + ".wsdd");
				logger.info("loading service:" + serviceName + ".wsdd");
			} catch (FileNotFoundException fnf) {
				input =
					Deployment.class.getClassLoader().getResourceAsStream(
						serviceName + ".wsdd");
				logger.info(
					"loading service from resource stream:"
						+ serviceName
						+ ".wsdd");
			}
			if (input == null) {
				throw new Exception("File not found: " + serviceName + "wsdd");
			}
			try {
				doc = newDocument(new InputSource(input));
			} catch (Exception e) {
				throw new Exception(e);
			}

			NodeList serviceNodes =
				doc.getElementsByTagNameNS(
					"http://xml.apache.org/axis/wsdd/",
					"service");
			for (int serviceNode = 0;
				serviceNode < serviceNodes.getLength();
				serviceNode++) {
				Element serviceElement =
					(Element) serviceNodes.item(serviceNode);
				WSDDService service = new WSDDService(serviceElement);
				deployment.deployService(service);
				;
			}

		} finally {
			if (input != null) {
				try {
					input.close();
				} catch (Exception e) {
					throw new Exception(e);
				}
			}
		}
		server.saveConfiguration();
		listServices();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  inputSource    Description of the Parameter
	 *@return                Description of the Return Value
	 *@exception  Exception  Description of the Exception
	 */
	private Document newDocument(InputSource inputSource) throws Exception {
		DocumentBuilderFactory dbf;
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Cannot build DOM", e);
			throw new Exception(e);
		}
		try {
			DocumentBuilder db;
			synchronized (dbf) {
				db = dbf.newDocumentBuilder();
			}
			db.setErrorHandler(new XMLUtils.ParserErrorHandler());
			return db.parse(inputSource);
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Cannot read XML input", e);
			throw new Exception(e);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  serviceName    Description of the Parameter
	 *@exception  Exception  Description of the Exception
	 */
	public void saveService(String serviceName) throws Exception {

		FileProvider wsddProvider = (FileProvider) config;
		deployment = wsddProvider.getDeployment();
		WSDDService service =
			deployment.getWSDDService(new QName("", serviceName));
		if (service == null) {
			logger.log(Level.SEVERE, "Unknown service: " + serviceName);
			return;
		}
		FileWriter writer = new FileWriter(serviceName + ".wsdd");
		SerializationContextImpl context = new SerializationContextImpl(writer);
		service.writeToContext(context);
		writer.flush();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  serviceName  Description of the Parameter
	 */
	public void undeploy(String serviceName) {

		FileProvider wsddProvider = (FileProvider) config;
		deployment = wsddProvider.getDeployment();
		WSDDService service =
			deployment.getWSDDService(new QName("", serviceName));

		deployment.undeployService(new QName("", serviceName));

		if (service == null) {
			logger.log(Level.SEVERE, "Unknown service: " + serviceName);
			return;
		}
		server.saveConfiguration();

	}
}
