package com.pallas.ogsa.server;

import java.net.InetAddress;
import java.net.URL;
import java.util.Hashtable;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.apache.axis.client.Service;
import org.apache.axis.types.URI;
import org.gridforum.www.ogsi.HandleType;
import org.gridforum.www.ogsi.LocatorType;
import org.gridforum.www.ogsi.ReferenceType;

import com.pallas.ogsa.extensions.ServiceReferenceType;
import com.pallas.ogsa.registry.ISiteRegistry;
import com.pallas.ogsa.registry.SiteRegistry;
import com.pallas.ogsa.registry.SiteRegistryStub;

import com.pallas.ogsa.services.caservice.CAService;
import com.pallas.ogsa.services.caservice.ICAService;
import com.pallas.ogsa.services.uplstatic.IUPL;
import com.pallas.ogsa.services.uplstatic.UPLServiceSoapBindingImpl;
import com.pallas.ogsa.services.uplstatic.VsiteConnection;

import com.pallas.ogsa.services.vsiteregistry.*;

/**
* Non-Frame version of the server startup
*/

public class ServerStartup {
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.ogsa.service");
	private static Handler filePlainHandler = null;

	/**
	 *  Description of the Field
	 */
	protected LocatorType registryLocator;

	private IVsiteRegistry vsiteRegistryPort;
	private ISiteRegistry registry;

	private ThreadGroup servers = new ThreadGroup("Servers");
	private int runningThreads;
	private Hashtable runningServers = new Hashtable();

	private Server httpServer;
	private Server httpsServer;

	public ServerStartup() {
	}

	/**
	 *  Description of the Method
	 *
	 *@param  numberOfThreads  Description of the Parameter
	 *@return                  Description of the Return Value
	 */
	protected Server attachServer(String[] args, String name)
		throws Exception {
		Server nextServer = new Server(args, name);
		Thread nextThread = new Thread(servers, nextServer);
		runningServers.put(name, nextServer);
		runningThreads++;
		nextThread.start();
		logger.info("Created new server: " + name);
		return nextServer;
	}

	protected Server getServerByName(String name) {
		return (Server) runningServers.get(name);
	}

	/**
	 *  Sets the registry attribute of the ServerFrame object
	 *
	 *@param  registryAddress  The new registry value
	 *@param  registryQName    The new registry value
	 *@param  registryHandle   The new registry value
	 */
	public void setRegistry(
		String registryHandle,
		String registryAddress,
		QName registryQName) {
		registryLocator = new LocatorType();

		//      ArrayList handles = new ArrayList();
		//          handles.add(new HandleType(new URI(registryHandle)));
		try {
			registryLocator.setServiceHandle(
				new HandleType(new URI(registryHandle)));
		} catch (URI.MalformedURIException e) {
			logger.log(
				Level.SEVERE,
				"Malformed Handle URI: " + registryHandle,
				e);
		}

		ReferenceType reference =
			new ServiceReferenceType(registryAddress, registryQName);
		registryLocator.setServiceReference(reference);

	}

	/**
	 *  Sets the registryLocator attribute of the ServerFrame object
	 *
	 *@param  registryLocator  The new registryLocator value
	 */
	public void setRegistryLocator(LocatorType registryLocator) {
		this.registryLocator = registryLocator;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  locator        Description of the Parameter
	 *@exception  Exception  Description of the Exception
	 */
	public void registerService(LocatorType locator) throws Exception {
		HandleType handle = registryLocator.getServiceHandle();
		ServiceReferenceType reference =
			(ServiceReferenceType) registryLocator.getServiceReference();
		Service service =
			new Service(reference.getWsdlURL(), reference.getQName());
		if (registry == null) {
			registry =
				new SiteRegistryStub(new URL(reference.getWsdlURL()), service);
		}
		registry.add(locator);
	}

	public void setUpVsites() {

		// initialize services
		try {
			String hostname = InetAddress.getLocalHost().getHostName();
			logger.info("Resolving hostname: " + hostname);

			// set Registry

			setRegistry(
				"http://www.unicore.org/SiteRegistry",
				"http://"
					+ hostname
					+ ":"
					+ httpServer.getPort()
					+ "/unicore/services/SiteRegistry?WSDL",
				new QName("http://services.ogsa.pallas.com", "SiteRegistry"));

			// @TODO set up a HandleResolver grid service later to seperate handles and references

			VsiteConnection[] knownVsites = vsiteRegistryPort.getConnections();
			for (int i = 0; i < knownVsites.length; i++) {
				// set up service handle
				LocatorType locator = new LocatorType();
				// UPL service HTTPS
				String uplServiceName = "UPLService";
				String uplServiceDir = "unicore/services";

				String uriString =
					"https://"
						+ hostname
						+ ":"
						+ httpsServer.getPort()
						+ "/"
						+ uplServiceDir
						+ "/"
						+ uplServiceName;

				URI handleUri = new URI(uriString);

				// this has to be the Vsite name presenterd to the client
				handleUri.setUserinfo(knownVsites[i].getName());
				HandleType handle = new HandleType(handleUri);
				locator.setServiceHandle(handle);

				// set up service reference

				String url =
					"https://"
						+ hostname
						+ ":"
						+ httpsServer.getPort()
						+ "/"
						+ uplServiceDir
						+ "/"
						+ uplServiceName;
				QName uplQname =
					new QName("https://services.ogsa.pallas.com", "UPLService");

				ReferenceType reference =
					new ServiceReferenceType(url, uplQname);
				locator.setServiceReference(reference);

				registerService(locator);
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Cannot register services", e);
		}

	}

	private void setUpServers() {
		try {
			String[] args1 = { "../etc/httpconfig.xml" };
			httpServer = attachServer(args1, "HTTP_Connector_1");
			String[] args2 = { "../etc/httpsconfig.xml" };
			httpsServer = attachServer(args2, "HTTPS_Connector_1");

			// vsite registration HTTP singleton acces scope
			httpServer.getDeployment().deploy(
				com
					.pallas
					.ogsa
					.services
					.vsiteregistry
					.ServicesSoapBindingImpl
					.class
					.getName(),
				"VsiteRegistry",
				"unicore/services",
				com.pallas.ogsa.services.vsiteregistry.IVsiteRegistry.class,
				"singleton");

			// registry service  HTTP 
			httpServer.getDeployment().deploy(
				SiteRegistry.class.getName(),
				"SiteRegistry",
				"unicore/services",
				ISiteRegistry.class,
				"singleton");

			// CA service for testgrid HTTP
			String caServiceName = "CAService";
			String caServiceDir = "unicore/services";
			httpServer.getDeployment().deploy(
				CAService.class.getName(),
				caServiceName,
				caServiceDir,
				ICAService.class,
				"per-access");

			// UPL service HTTPS
			String uplServiceName = "UPLService";
			String uplServiceDir = "unicore/services";

			httpsServer.getDeployment().deploy(
				UPLServiceSoapBindingImpl.class.getName(),
				uplServiceName,
				uplServiceDir,
				IUPL.class,
				"per-access");

			// wait for the httpServer to come up
			Thread.currentThread().sleep(10000);

			String url =
				"http://"
					+ InetAddress.getLocalHost().getHostName()
					+ ":"
					+ httpServer.getPort()
					+ "/"
					+ uplServiceDir
					+ "/"
					+ "VsiteRegistry";
			initVsiteRegistryServices(url);

		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could not start server.", e);
		}
	}

	/**
	 *  The main program for the ServerFrame class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		ServerStartup serverStartup = new ServerStartup();
		serverStartup.setUpServers();
		serverStartup.setUpVsites();
	}

	private void initVsiteRegistryServices(String urlString) throws Exception {
		// create an rpc service
		IVsiteRegistryService service = new IVsiteRegistryServiceLocator();

		//get the deployed web service
		URL url = new URL(urlString);
		vsiteRegistryPort = service.getservices(url);
		vsiteRegistryPort.init();
	}

}
