package com.pallas.ogsa.server;

import java.io.OutputStream;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.rpc.encoding.TypeMappingRegistry;

import org.apache.axis.AxisFault;
import org.apache.axis.Constants;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.description.OperationDesc;
import org.apache.axis.description.ServiceDesc;
import org.apache.axis.encoding.Base64;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.message.SOAPFault;
import org.apache.axis.server.AxisServer;
import org.apache.axis.transport.http.HTTPConstants;
import org.apache.axis.transport.http.NonBlockingBufferedInputStream;
import org.apache.axis.utils.Messages;
import org.apache.axis.utils.XMLUtils;
import org.w3c.dom.Document;

import com.pallas.ogsa.util.OGSITypeRegistration;
import com.pallas.unicore.threadpool.GenericObservable;
import com.pallas.unicore.threadpool.IObservable;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.util.queues.IWork;
import com.pallas.util.queues.WorkerThread;

/**
 *  Description of the Class
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. Februar 2003
 *@version    $Id: ServiceRequest.java,v 1.11 2003/07/10 12:26:00 thomas Exp $
 */
public class ServiceRequest implements IWork, IObservable {
	// ASCII character mapping to lower case
	private final static byte[] toLower = new byte[256];

	static {
		for (int i = 0; i < 256; i++) {
			toLower[i] = (byte) i;
		}

		for (int lc = 'a'; lc <= 'z'; lc++) {
			toLower[lc + 'A' - 'a'] = (byte) lc;
		}
	}

	// mime header for soap action
	private final static byte actionHeader[] = "soapaction: ".getBytes();
	private final static int actionLen = actionHeader.length;

	// HTTP header for authentication
	private final static byte authHeader[] = "authorization: ".getBytes();
	private final static int authLen = authHeader.length;

	// "Basic" auth string
	private final static byte basicAuth[] = "basic ".getBytes();

	// buffer for IO
	private final static int BUFSIZ = 4096;
	
	//	Tiddly little response
	private final static String responseStr =
		 "<html><head><title>UnicoreAxisServer</title></head>"
			 + "<body><h1>UnicoreAxisServer</h1>"
			 + Messages.getMessage("reachedServer00")
			 + "</html>";
	private static byte cannedHTMLResponse[] = responseStr.getBytes();

	// mime header for cookie2
	private final static byte cookie2Header[] = "cookie2: ".getBytes();
	private final static int cookie2Len = cookie2Header.length;

	// mime header for cookie
	private final static byte cookieHeader[] = "cookie: ".getBytes();
	private final static int cookieLen = cookieHeader.length;

	// mime header for GET
	private final static byte getHeader[] = "GET".getBytes();

	// header ender
	private final static byte headerEnder[] = ": ".getBytes();

	// Standard MIME headers for HTML payload
	private static byte HTML_MIME_STUFF[] =
		("\r\nContent-Type: text/html; charset=utf-8\r\n" + "Content-Length: ")
			.getBytes();

	// HTTP prefix
	private static byte HTTP[] = "HTTP/1.0 ".getBytes();
	private static byte ISE[] =
		("500 " + Messages.getMessage("internalError01")).getBytes();

	// mime header for content length
	private final static byte lenHeader[] = "content-length: ".getBytes();
	private final static int lenLen = lenHeader.length;

	// mime header for content location
	private final static byte locationHeader[] =
		(HTTPConstants.HEADER_CONTENT_LOCATION.toLowerCase() + ": ").getBytes();
	private final static int locationLen = locationHeader.length;
	/**
	 *  Description of the Field
	 */
	protected static Logger logger = Logger.getLogger("com.pallas.ogsa.server");

	// HTTP status codes
	private static byte OK[] =
		("200 " + Messages.getMessage("ok00")).getBytes();

	// mime header for POST
	private final static byte postHeader[] = "POST".getBytes();
	

	// Mime/Content separator
	private static byte SEPARATOR[] = "\r\n\r\n".getBytes();


	// Axis specific constants
	private static String transportName = "SimpleHTTP";

	// mime header for content type
	private final static byte typeHeader[] =
		(HTTPConstants.HEADER_CONTENT_TYPE.toLowerCase() + ": ").getBytes();
	private final static int typeLen = typeHeader.length;
	private static byte UNAUTH[] =
		("401 " + Messages.getMessage("unauth00")).getBytes();

	// Standard MIME headers for XML payload
	private static byte XML_MIME_STUFF[] =
		("\r\nContent-Type: text/xml; charset=utf-8\r\n" + "Content-Length: ")
			.getBytes();

	/**
	 *  One method for all host name lookups.
	 *
	 *@return    The localHost value
	 */
	public static String getLocalHost() {
		// FIXME
		// This doesn't return anything but 0.0.0.0
		//  String hostname = serverSocket.getInetAddress().getHostAddress();
		// And this returns the hostname of the host on the other
		// end of the socket:
		//  String hostname = socket.getInetAddress().getHostName();
		// This works for 99% of the uses of SimpleAxisServer,
		// but is very stupid
		try {
			return InetAddress.getLocalHost().getHostAddress();
		} catch (UnknownHostException uhe) {
			return "localhost";
		}
	}

	private PrintStream logStream = null;
	private X509Certificate peerCertificate;
	private Server server;

	private Socket socket;

	/**
	*  Description of the Field
	*/
	private GenericObservable theGenericObservable = new GenericObservable();
	private String threadId = "UNKNOWN";

	/**
	 *  Constructor for the ServiceRequest object
	 *
	 *@param  socket  Description of the Parameter
	 *@param  server  Description of the Parameter
	 */
	public ServiceRequest(
		Socket socket,
		Server server,
		X509Certificate peerCertificate) {
		this.socket = socket;
		this.server = server;
		this.peerCertificate = peerCertificate;
	}

	/**
	 *  Adds a feature to the Observer attribute of the UpdateObject object
	 *
	 *@param  anObserver  The feature to be added to the Observer attribute
	 */
	public void addObserver(IObserver anObserver) {
		this.theGenericObservable.addObserver(anObserver);
	}

	/**
	 *  Description of the Method
	 */
	public void deleteAllObservers() {
		this.theGenericObservable.deleteAllObservers();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  anObserver  Description of Parameter
	 */
	public void deleteObserver(IObserver anObserver) {
		this.theGenericObservable.deleteObserver(anObserver);
	}

	/*
	 *  does tolower[buf] match the target byte array, up to the target's
	 *  length?
	 *
	 *@param  buf     Description of the Parameter
	 *@param  target  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public boolean matches(byte[] buf, byte[] target) {
		for (int i = 0; i < target.length; i++) {
			if (toLower[buf[i]] != target[i]) {
				return false;
			}
		}
		return true;
	}

	/**
	 *  Case-insensitive match of a target byte [] to a source byte [], starting
	 *  from a particular offset into the source.
	 *
	 *@param  buf     Description of the Parameter
	 *@param  bufIdx  Description of the Parameter
	 *@param  target  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public boolean matches(byte[] buf, int bufIdx, byte[] target) {
		for (int i = 0; i < target.length; i++) {
			if (toLower[buf[bufIdx + i]] != target[i]) {
				return false;
			}
		}
		return true;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  theObserved  Description of Parameter
	 *@param  changeCode   Description of Parameter
	 */
	public void notifyObservers(Object theObserved, Object changeCode) {
		theGenericObservable.notifyObservers(theObserved, changeCode);
	}

	/**
	 *  Read all mime headers, returning the value of Content-Length and
	 *  SOAPAction.
	 *
	 *@param  is                       InputStream to read from
	 *@param  contentType              The content type.
	 *@param  contentLocation          The content location
	 *@param  soapAction               StringBuffer to return the soapAction
	 *      into
	 *@param  httpRequest              StringBuffer for GET / POST
	 *@param  cookie                   first cookie header (if doSessions)
	 *@param  cookie2                  second cookie header (if doSessions)
	 *@param  buf                      Description of the Parameter
	 *@param  fileName                 Description of the Parameter
	 *@param  authInfo                 Description of the Parameter
	 *@return                          Content-Length
	 *@exception  java.io.IOException  Description of the Exception
	 */
	private int parseHeaders(
		NonBlockingBufferedInputStream is,
		byte buf[],
		StringBuffer contentType,
		StringBuffer contentLocation,
		StringBuffer soapAction,
		StringBuffer httpRequest,
		StringBuffer fileName,
		StringBuffer cookie,
		StringBuffer cookie2,
		StringBuffer authInfo)
		throws java.io.IOException {
		int n;
		int len = 0;

		// parse first line as GET or POST
		n = this.readLine(is, buf, 0, buf.length);
		logger.info("Server: read first line.");
		if (n < 0) {
			// nothing!
			throw new java.io.IOException(
				Messages.getMessage("unexpectedEOS00"));
		}

		// which does it begin with?
		httpRequest.delete(0, httpRequest.length());
		fileName.delete(0, fileName.length());
		contentType.delete(0, contentType.length());
		contentLocation.delete(0, contentLocation.length());

		if (buf[0] == getHeader[0]) {
			logger.info(threadId + "Server: found GET header");
			httpRequest.append("GET");
			for (int i = 0; i < n - 5; i++) {
				char c = (char) (buf[i + 5] & 0x7f);
				if (c == ' ') {
					break;
				}
				fileName.append(c);
			}
			logger.fine(
				threadId
					+ Messages.getMessage(
						"filename01",
						"SimpleAxisServer",
						fileName.toString()));
			return 0;
		} else if (buf[0] == postHeader[0]) {
			logger.info(threadId + "Server: found POST header");
			httpRequest.append("POST");
			for (int i = 0; i < n - 6; i++) {
				char c = (char) (buf[i + 6] & 0x7f);
				if (c == ' ') {
					break;
				}
				fileName.append(c);
			}
			logger.fine(
				threadId
					+ Messages.getMessage(
						"filename01",
						"SimpleAxisServer",
						fileName.toString()));
		} else {
			logger.severe(threadId + "Server: found bad header");
			throw new java.io.IOException(Messages.getMessage("badRequest00"));
		}

		while ((n = readLine(is, buf, 0, buf.length)) > 0) {

			if ((n <= 2) && (buf[0] == '\n' || buf[0] == '\r') && (len > 0)) {
				break;
			}

			// RobJ gutted the previous loggeric; it was too hard to extend for more headers.
			// Now, all it does is search forwards for ": " in the buf,
			// then do a length / byte compare.
			// Hopefully this is still somewhat efficient (Sam is watching!).

			// First, search forwards for ": "
			int endHeaderIndex = 0;
			while (endHeaderIndex < n
				&& toLower[buf[endHeaderIndex]] != headerEnder[0]) {
				endHeaderIndex++;
			}
			endHeaderIndex += 2;
			// endHeaderIndex now points _just past_ the ": ", and is
			// comparable to the various lenLen, actionLen, etc. values

			// convenience; i gets pre-incremented, so initialize it to one less
			int i = endHeaderIndex - 1;

			// which header did we find?
			if (endHeaderIndex == lenLen && matches(buf, lenHeader)) {
				// parse content length
				while ((++i < n) && (buf[i] >= '0') && (buf[i] <= '9')) {
					len = (len * 10) + (buf[i] - '0');
				}
			} else if (
				endHeaderIndex == actionLen && matches(buf, actionHeader)) {
				soapAction.delete(0, soapAction.length());
				// skip initial '"'
				i++;
				while ((++i < n) && (buf[i] != '"')) {
					soapAction.append((char) (buf[i] & 0x7f));
				}
			} else if (endHeaderIndex == authLen && matches(buf, authHeader)) {
				if (matches(buf, endHeaderIndex, basicAuth)) {
					i += basicAuth.length;
					while (++i < n && (buf[i] != '\r') && (buf[i] != '\n')) {
						if (buf[i] == ' ') {
							continue;
						}
						authInfo.append((char) (buf[i] & 0x7f));
					}
				} else {
					throw new java.io.IOException(
						Messages.getMessage("badAuth00"));
				}
			} else if (
				endHeaderIndex == locationLen
					&& matches(buf, locationHeader)) {
				while (++i < n && (buf[i] != '\r') && (buf[i] != '\n')) {
					if (buf[i] == ' ') {
						continue;
					}
					contentLocation.append((char) (buf[i] & 0x7f));
				}
			} else if (endHeaderIndex == typeLen && matches(buf, typeHeader)) {
				while (++i < n && (buf[i] != '\r') && (buf[i] != '\n')) {
					if (buf[i] == ' ') {
						continue;
					}
					contentType.append((char) (buf[i] & 0x7f));
				}
			}
		}
		logger.info(threadId + "Server: finished  parsing header");
		return len;
	}

	/**
	 *  Description of the Method
	 */
	public void performTask() {

		byte buf[] = new byte[BUFSIZ];
		// create an Axis server
		AxisServer engine = server.getAxisServer();
		threadId = "[" + Thread.currentThread().getName() + "]: ";

		//  attach the certificate that establihed the connection to the WorkerThread
		try {
			WorkerThread worker = (WorkerThread) Thread.currentThread();
			worker.setPeerCertificate(peerCertificate);
		} catch (ClassCastException ce) {
			logger.severe(
				"ServiceRequest must be invoked by WorkerThread class");
			return;
		}

		// create and initialize a message context
		MessageContext msgContext = new MessageContext(engine);
		Message requestMsg;

		// Reusuable, buffered, content length controlled, InputStream
		NonBlockingBufferedInputStream is =
			new NonBlockingBufferedInputStream();

		// buffers for the headers we care about
		StringBuffer soapAction = new StringBuffer();
		StringBuffer httpRequest = new StringBuffer();
		StringBuffer fileName = new StringBuffer();
		StringBuffer cookie = new StringBuffer();
		StringBuffer cookie2 = new StringBuffer();
		StringBuffer authInfo = new StringBuffer();
		StringBuffer contentType = new StringBuffer();
		StringBuffer contentLocation = new StringBuffer();

		Message responseMsg = null;

		// prepare request (do as much as possible while waiting for the
		// next connection).  Note the next two statements are commented
		// out.  Uncomment them if you experience any problems with not
		// resetting state between requests:
		//   msgContext = new MessageContext();
		//   requestMsg = new Message("", "String");
		try {
			msgContext.setTargetService(null);
		} catch (AxisFault fault) {
			logger.log(Level.WARNING, "Could not set target service.", fault);
		}
		msgContext.setResponseMessage(null);
		msgContext.reset();
		//msgContext.setProperty("transport", "HTTPTransport");
		msgContext.setTransportName(transportName);

		responseMsg = null;

		try {
			// assume the best
			byte[] status = OK;
			// assume we're not getting WSDL
			boolean doWsdl = false;
			// cookie for this session, if any
			String cooky = null;

			try {
				authInfo.delete(0, authInfo.length());
				// read headers
				is.setInputStream(socket.getInputStream());
				logger.info(threadId + "Server starts parsing header");
				// parse all headers into hashtable
				int contentLength =
					parseHeaders(
						is,
						buf,
						contentType,
						contentLocation,
						soapAction,
						httpRequest,
						fileName,
						cookie,
						cookie2,
						authInfo);
				is.setContentLength(contentLength);

				int paramIdx = fileName.toString().indexOf('?');
				if (paramIdx != -1) {
					// Got params
					String params = fileName.substring(paramIdx + 1);
					fileName.setLength(paramIdx);

					logger.info(
						threadId
							+ Messages.getMessage(
								"filename00",
								fileName.toString()));
					logger.info(
						threadId + Messages.getMessage("params00", params));

					if ("wsdl".equalsIgnoreCase(params)) {
						doWsdl = true;
					}
				}

				// Real and relative paths are the same for the
				// SimpleAxisServer
				msgContext.setProperty(
					Constants.MC_REALPATH,
					fileName.toString());
				msgContext.setProperty(
					Constants.MC_RELATIVE_PATH,
					fileName.toString());
				msgContext.setProperty(Constants.MC_JWS_CLASSDIR, "jwsClasses");

				// !!! Fix string concatenation
				String url =
					"http://"
						+ getLocalHost()
						+ ":"
						+ server.getServerSocket().getLocalPort()
						+ "/"
						+ fileName.toString();
				msgContext.setProperty(MessageContext.TRANS_URL, url);

				String filePart = fileName.toString();
				if (filePart.startsWith("unicore/services/")) {
					logger.info(
						threadId
							+ "Requesting new service: "
							+ filePart.substring(filePart.lastIndexOf("/") + 1));
					msgContext.setTargetService(
						filePart.substring(filePart.lastIndexOf("/") + 1));
				}

				if (authInfo.length() > 0) {
					// Process authentication info
					//authInfo = new StringBuffer("dXNlcjE6cGFzczE=");
					logger.info(threadId + "Server: processing authentication");
					byte[] decoded = Base64.decode(authInfo.toString());
					StringBuffer userBuf = new StringBuffer();
					StringBuffer pwBuf = new StringBuffer();
					StringBuffer authBuf = userBuf;
					for (int i = 0; i < decoded.length; i++) {
						if ((char) (decoded[i] & 0x7f) == ':') {
							authBuf = pwBuf;
							continue;
						}
						authBuf.append((char) (decoded[i] & 0x7f));
					}

					logger.info(
						threadId
							+ Messages.getMessage("user00", userBuf.toString()));

					msgContext.setUsername(userBuf.toString());
					msgContext.setPassword(pwBuf.toString());
				}

				// if get, then return simpleton document as response
				if (httpRequest.toString().equals("GET")) {
					logger.info(threadId + "Server: processing GET header");
					OutputStream out = socket.getOutputStream();
					out.write(HTTP);
					out.write(status);

					if (doWsdl) {
						logger.info(threadId + "Server: WSDL requested");
						engine.generateWSDL(msgContext);
						Document doc =
							(Document) msgContext.getProperty("WSDL");
						if (doc != null) {
							String response = XMLUtils.DocumentToString(doc);
							byte[] respBytes = response.getBytes();
							out.write(XML_MIME_STUFF);
							putInt(buf, out, respBytes.length);
							out.write(SEPARATOR);
							out.write(respBytes);
							out.flush();
							logger.info(
								threadId + "Server: processed WSDL, returning");
							return;
						}
					}

					StringBuffer sb = new StringBuffer();
					sb.append("<h2>And now... Some Services</h2>\n");
					Iterator i = engine.getConfig().getDeployedServices();
					out.write("<ul>\n".getBytes());
					while (i.hasNext()) {
						ServiceDesc sd = (ServiceDesc) i.next();
						sb.append("<li>\n");
						sb.append(sd.getName());
						sb.append(" <a href=\"../unicore/services/");
						sb.append(sd.getName());
						sb.append("?wsdl\"><i>(wsdl)</i></a></li>\n");
						ArrayList operations = sd.getOperations();
						if (!operations.isEmpty()) {
							sb.append("<ul>\n");
							for (Iterator it = operations.iterator();
								it.hasNext();
								) {
								OperationDesc desc = (OperationDesc) it.next();
								sb.append("<li>" + desc.getName());
							}
							sb.append("</ul>\n");
						}
					}
					sb.append("</ul>\n");

					byte[] bytes = sb.toString().getBytes();

					out.write(HTML_MIME_STUFF);
					putInt(buf, out, bytes.length);
					out.write(SEPARATOR);
					out.write(bytes);
					out.flush();
					logger.info(
						threadId + "Server: processed GET header, returning");
					return;
				}

				// this may be "" if either SOAPAction: "" or if no SOAPAction at all.
				// for now, do not complain if no SOAPAction at all
				String soapActionString = soapAction.toString();
				if (soapActionString != null) {
					msgContext.setUseSOAPAction(true);
					msgContext.setSOAPActionURI(soapActionString);
				}
				requestMsg =
					new Message(
						is,
						false,
						contentType.toString(),
						contentLocation.toString());
				msgContext.setRequestMessage(requestMsg);

				//  Register OGSI classes
				TypeMappingRegistry registry =
					msgContext.getTypeMappingRegistry();
				OGSITypeRegistration.registerAll(registry);

				if (logStream != null) {
					logStream.println("\n--------- Client Request -------\n");
					// msgContext.getRequestMessage().writeTo(logStream);
					System.err.println();
				}
				logger.info(threadId + "Server: processing SOAP request");
				// invoke the Axis engine
				engine.invoke(msgContext);

				// Retrieve the response from Axis
				responseMsg = msgContext.getResponseMessage();
				if (responseMsg == null) {
					throw new AxisFault(Messages.getMessage("nullResponse00"));
				}

				if (logStream != null) {
					logStream.println("\n--------- Server Response -------\n");
					// msgContext.getResponseMessage().writeTo(logStream);
					System.err.println();
				}

			} catch (Exception e) {
				AxisFault af;
				if (e instanceof AxisFault) {
					af = (AxisFault) e;
					logger.severe(
						threadId + Messages.getMessage("serverFault00")+ af);
					//, af);

					if ("Server.Unauthorized".equals(af.getFaultCode())) {
						status = UNAUTH;
						// SC_UNAUTHORIZED
					} else {
						status = ISE;
						// SC_INTERNAL_SERVER_ERROR
					}
				} else {
					status = ISE;
					// SC_INTERNAL_SERVER_ERROR
					af = AxisFault.makeFault(e);
				}

				// There may be headers we want to preserve in the
				// response message - so if it's there, just add the
				// FaultElement to it.  Otherwise, make a new one.
				responseMsg = msgContext.getResponseMessage();
				if (responseMsg == null) {
					responseMsg = new Message(af);
				} else {
					try {
						SOAPEnvelope env = responseMsg.getSOAPEnvelope();
						env.clearBody();
						env.addBodyElement(new SOAPFault((AxisFault) e));
					} catch (AxisFault fault) {
						// Should never reach here!
					}
				}
			}

			// Send it on its way...

			logger.info(threadId + "Server: sending response message");
			OutputStream out = socket.getOutputStream();
			out.write(HTTP);
			out.write(status);
			//out.write(XML_MIME_STUFF);
			out.write(
				("\r\n"
					+ HTTPConstants.HEADER_CONTENT_TYPE
					+ ": "
					+ responseMsg.getContentType(msgContext.getSOAPConstants()))
					.getBytes());
			// Writing the length causes the entire message to be decoded twice.
			//out.write(("\r\n" + HTTPConstants.HEADER_CONTENT_LENGTH + ": " + responseMsg.getContentLength()).getBytes());
			// putInt(out, response.length);

			out.write(SEPARATOR);
			responseMsg.writeTo(out);
			// out.write(response);
			out.flush();
		} catch (Exception e) {
			logger.severe(threadId + Messages.getMessage("exception00")+" 2");
			//, e);
		} finally {
			try {
				if (socket != null) {
					logger.info(threadId + "Server: close Connection");
					socket.close();
					notifyObservers(this, socket);
				}
			} catch (Exception e) {
			}
		}
		if (msgContext.getProperty(MessageContext.QUIT_REQUESTED) != null) {
			// why then, quit!
			try {
				server.stop();
			} catch (Exception e) {
			}
		}
	}

	/**
	 *  output an integer into the output stream
	 *
	 *@param  out                      OutputStream to be written to
	 *@param  value                    Integer value to be written.
	 *@param  buf                      Description of the Parameter
	 *@exception  java.io.IOException  Description of the Exception
	 */
	private void putInt(byte buf[], OutputStream out, int value)
		throws java.io.IOException {
		int len = 0;
		int offset = buf.length;

		// negative numbers
		if (value < 0) {
			buf[--offset] = (byte) '-';
			value = -value;
			len++;
		}

		// zero
		if (value == 0) {
			buf[--offset] = (byte) '0';
			len++;
		}

		// positive numbers
		while (value > 0) {
			buf[--offset] = (byte) (value % 10 + '0');
			value = value / 10;
			len++;
		}

		// write the result
		out.write(buf, offset, len);
	}

	/**
	 *  Read a single line from the input stream
	 *
	 *@param  is                       inputstream to read from
	 *@param  b                        byte array to read into
	 *@param  off                      starting offset into the byte array
	 *@param  len                      maximum number of bytes to read
	 *@return                          Description of the Return Value
	 *@exception  java.io.IOException  Description of the Exception
	 */
	private int readLine(
		NonBlockingBufferedInputStream is,
		byte[] b,
		int off,
		int len)
		throws java.io.IOException {
		int count = 0;
		int c;

		while ((c = is.read()) != -1) {
			if (c != '\n' && c != '\r') {
				b[off++] = (byte) c;
				count++;
			}
			if (count == len) {
				break;
			}
			if ('\n' == c) {
				int peek = is.peek();
				//If the next line begins with tab or space then this is a continuation.
				if (peek != ' ' && peek != '\t') {
					break;
				}
			}
		}
		// logger.info("Read line: " + new String(b, 0, count));
		return count > 0 ? count : -1;
	}

	/**
	 *  Sets the logStream attribute of the ServiceRequest object
	 *
	 *@param  logStream  The new logStream value
	 */
	public void setLogStream(PrintStream logStream) {
		this.logStream = logStream;
	}

}
