package com.pallas.ogsa.server;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

public class WatchDog {

	private class WatchDogTimerTask extends TimerTask {
		private double totalMem, maxMem, freeMemory, usedMemory;
		public WatchDogTimerTask() {
		}
		public void run() {
			totalMem = Runtime.getRuntime().totalMemory() / 1024.0 / 1024.0;
			maxMem = Runtime.getRuntime().maxMemory() / 1024.0 / 1024.0;
			freeMemory = Runtime.getRuntime().freeMemory() / 1024.0 / 1024.0;
			usedMemory = totalMem - freeMemory;
			logger.info(
				"Watchdog triggered at: "
					+ new Date(System.currentTimeMillis())
					+ "max: "
					+ maxMem
					+ "free: "
					+ freeMemory
					+ "total: "
					+ totalMem
					+ "used:	"
					+ usedMemory);
			try {
				if (mail) {
					reporter.sendMessage(
						"Server ["
							+ hostName
							+ "/"
							+ port
							+ "] alive: "
							+ new Date(System.currentTimeMillis()));
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	private static Logger logger = Logger.getLogger("com.pallas.ogsa.server");

	private static Timer timer = new Timer();

	private long delay;
	private String hostName = "server";
	private boolean mail;
	private long period;
	private int port;

	private Reporter reporter;

	private WatchDogTimerTask timerTask;

	public WatchDog(long delay, long period, boolean mail, Server server) {
		this.delay = delay;
		this.period = period;
		this.mail = mail;
		this.port = server.getPort();
		try {
			this.hostName = InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException e) {
		}
		reporter =
			new Reporter(
				"thomas.kentemich@pallas.com",
				"mailhost",
				hostName,
				port);
	}

	public void start() {
		timerTask = new WatchDogTimerTask();
		timer.scheduleAtFixedRate(timerTask, delay, period);
	}

	public void stop() {
		timer.cancel();
	}

}
