package com.pallas.ogsa.services.caservice;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERInputStream;
import org.bouncycastle.asn1.pkcs.CertificationRequestInfo;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.bouncycastle.jce.X509KeyUsage;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.jce.X509V3CertificateGenerator;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.helpers.XMLReaderFactory;

import com.pallas.ogsa.util.ObjectWrapperBean;
import com.pallas.security.KeyStoreManager;
import com.pallas.unicore.utility.Base64;

/**
 *@author     Ralf Ratering
 *@created    29. Januar 2003
 *@version    $Id: CAService.java,v 1.9 2003/06/30 06:51:09 rrate Exp $
 */
public class CAService implements ICAService {

	/**
	 *  Description of the Class
	 *
	 *@author     thomas
	 *@created    September 27, 2001
	 */
	private static class CaServiceSaxHandler extends DefaultHandler {
		Vector foundSites = new Vector();

		/**
		 *  Description of the Method
		 *
		 *@param  buf               Description of Parameter
		 *@param  offset            Description of Parameter
		 *@param  len               Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void characters(char[] buf, int offset, int len)
			throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@exception  SAXException  Description of Exception
		 */
		public void endDocument() throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  namespaceURI      Description of Parameter
		 *@param  sName             Description of Parameter
		 *@param  qName             Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void endElement(String namespaceURI, String sName, String qName)
			throws SAXException {
			// qualified name
		}

		//===========================================================
		// SAX DocumentHandler methods
		//===========================================================

		/**
		 *  Description of the Method
		 *
		 *@exception  SAXException  Description of Exception
		 */
		public void startDocument() throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  namespaceURI      Description of Parameter
		 *@param  lName             Description of Parameter
		 *@param  qName             Description of Parameter
		 *@param  attrs             Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void startElement(
			String namespaceURI,
			String lName,
			String qName,
			Attributes attrs)
			throws SAXException {
			String elementName = lName;
			if ("".equals(elementName)) {
				elementName = qName;

				// namespaceAware = false
			}

			// read keystore information

			if (elementName.equals("keystore")) {
				if (attrs != null) {
					String keystore = null;
					String provider = null;
					String type = null;

					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("file")) {
							keystore = attrs.getValue(i);
						}

						if (aName.equals("provider")) {
							provider = attrs.getValue(i);
						}

						if (aName.equals("type")) {
							type = attrs.getValue(i);
						}
					}

					if (keystore != null && provider != null && type != null) {
						try {
							keystoreFilename =
								keystore.replace('/', File.separatorChar);
							keystoreProvider = provider;
							keystoreType = type;
						} catch (Exception e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}

				}
			}

			// read ca alias
			if (elementName.equals("cacert")) {
				if (attrs != null) {
					String alias = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("alias")) {
							alias = attrs.getValue(i);
						}
					}

					if (alias != null) {
						try {
							caCertAlias = alias;
						} catch (Exception e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}

				}
			}

			// read key alias
			if (elementName.equals("key")) {
				if (attrs != null) {
					String alias = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("alias")) {
							alias = attrs.getValue(i);
						}
					}

					if (alias != null) {
						try {
							keyAlias = alias;
						} catch (Exception e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}

				}
			}

			// read log dir
			if (elementName.equals("certdir")) {
				if (attrs != null) {
					String certdir = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("file")) {
							certdir = attrs.getValue(i);
						}
					}

					if (certdir != null) {
						try {
							certDir = certdir.replace('/', File.separatorChar);
						} catch (Exception e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}

				}

			}

			// read log dir
			if (elementName.equals("logdir")) {
				if (attrs != null) {
					String logdir = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("file")) {
							logdir = attrs.getValue(i);
						}
					}

					if (logdir != null) {
						try {
							loggingDir =
								logdir.replace('/', File.separatorChar);
						} catch (Exception e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}

				}

			}

		}
	}
	private static String caCertAlias = "pallas test ca certificate";
	private static String certDir = ".." + File.separator + "caservice";
	private final static String CONFIG_FILE_NAME =
		".." + File.separator + "etc" + File.separator + "caservice.xml";
	private static FileHandler filePlainHandler = null;
	private static String keyAlias = "pallas test ca";
	private static String keystoreFilename =
		".." + File.separator + "server.keystore";
	private static String keystoreProvider = "SunJCE";
	private static String keystoreType = "JCEKS";
	private static Logger logger =
		Logger.getLogger("com.pallas.ogsa.services.caservice");
	private static String loggingDir = ".." + File.separator + "serverlog";
	private static String password = "changeit";

	private static X509V3CertificateGenerator v3CertGen =
		new X509V3CertificateGenerator();
	private ObjectWrapperBean caCertBean;
	private String logFile = "caservice.log";
	private KeyStoreManager manager;

	/**
	 *  Constructor for the CAService object
	 */
	public CAService() {
		try {
			readCaServiceConfiguration();
			checkLogDir();
			manager =
				KeyStoreManager.getInstance(
					keystoreProvider,
					keystoreType,
					keystoreFilename);
			manager.loadKeyStore(
				password.toCharArray(),
				password.toCharArray());

			Vector trustedAliases = manager.getCachedTrustedCertificates();
			Certificate[] trustedCerts = new Certificate[trustedAliases.size()];
			String nextAlias = null;
			for (int i = 0; i < trustedAliases.size(); i++) {
				nextAlias = (String) trustedAliases.elementAt(i);
				try {
					trustedCerts[i] =
						manager.getCertificateByAlias(nextAlias)[0];
				} catch (Exception e) {
					logger.log(
						Level.SEVERE,
						"Could not load CA certificate: " + nextAlias,
						e);
				}
			}
			caCertBean = new ObjectWrapperBean(trustedCerts);
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could open keystore: " + caCertAlias, e);
		}
	}

	private void checkLogDir() throws Exception {

		File logDir = new File(loggingDir);
		logger.warning("Logging directory is now " + loggingDir);
		if (!logDir.exists()) {
			logDir.mkdir();
		}

		if (!logDir.isDirectory()) {
			//hmm seems to be a file; reset dir  to .
			loggingDir = ".";
			logger.warning("Logging directory reset to [.]");
		}
		filePlainHandler =
			new FileHandler(
				loggingDir + File.separator + logFile,
				1048576,
				1,
				true);
		filePlainHandler.setFormatter(new SimpleFormatter());
		logger.addHandler(filePlainHandler);

	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public ObjectWrapperBean getCACertificate() {
		return caCertBean;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  filename       Description of the Parameter
	 *@return                Description of the Return Value
	 *@exception  Exception  Description of the Exception
	 */
	private Certificate loadCertificate(String filename) throws Exception {
		logger.info("Importing cert: " + filename);
		DataInputStream dis =
			new DataInputStream(new FileInputStream(filename));
		CertificateFactory cf = CertificateFactory.getInstance("X.509");
		byte[] bytes = new byte[dis.available()];
		dis.readFully(bytes);
		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
		Certificate cert = null;
		while (bais.available() > 0) {
			cert = cf.generateCertificate(bais);
		}
		return cert;
	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	private void readCaServiceConfiguration() throws Exception {
		XMLReader parser = null;
		try {
			parser =
				XMLReaderFactory.createXMLReader(
					"org.apache.xerces.parsers.SAXParser");
		} catch (Exception e) {
			Parser sax1Parser =
				ParserFactory.makeParser("org.apache.xerces.parsers.SAXParser");
			parser = new ParserAdapter(sax1Parser);
			logger.warning(
				"Features and Properties not supported by SAX1 parsers");
		}
		CaServiceSaxHandler handler = new CaServiceSaxHandler();
		parser.setContentHandler(handler);
		parser.setErrorHandler(handler);
		File configFile = new File(CONFIG_FILE_NAME);
		if (!configFile.exists()) {
			throw new FileNotFoundException(
				"Config file " + CONFIG_FILE_NAME + " not found.");
		}
		FileInputStream fis = new FileInputStream(configFile);

		InputSource source = new InputSource(fis);
		parser.parse(source);

	}

	/**
	 *  Description of the Method
	 *
	 *@param  csr  Description of the Parameter
	 *@return      Description of the Return Value
	 */
	public ObjectWrapperBean signRequest(byte[] csr) {
		X509Certificate cert = null;

		logger.info("Got CSR: " + csr);
		try {
			X509Certificate caCertificate =
				manager.getCertificateByAlias(caCertAlias)[0];
			DERInputStream stream =
				new DERInputStream(new ByteArrayInputStream(csr));
			PKCS10CertificationRequest pkcs10 =
				new PKCS10CertificationRequest(
					(ASN1Sequence) stream.readObject());

			if (pkcs10.verify()) {
				logger.info("Received a valid CSR");

				CertificationRequestInfo csrInfo =
					pkcs10.getCertificationRequestInfo();
				// give 5 minutes grace period here to avoid difficulties with not synchronized clocks
				long validFrom = System.currentTimeMillis() - 10 * 60 * 1000;
				long validTo =
					System.currentTimeMillis()
						+ (long) (31 * 24 * 60 * 60 * (long) 1000);
				if (validTo <= validFrom) {
					throw new IllegalArgumentException("Invalid lifetime for certifcate");
				}

				String newissuer = "";

				// The Sun JCE is buggy!!!
				// The ordering of the SubjectDN fields for key entries is reversed
				// We have to create a new certificate in order to match the issuerDN
				// in the proxy with the original user cert !!

				X509Principal principal =
					new X509Principal(csrInfo.getSubject());
				String issuer = principal.getName();
				/*               int start = -1;
				               if((start = issuer.indexOf("EMAILADDRESS")) >= 0) {
				                   issuer = issuer.substring(0, start) + "EmailAddress" +
				                           issuer.substring(start + "EmailAddress".length());
				               } */
				String mark = "";
				StringTokenizer st = new StringTokenizer(issuer, ",");
				while (st.hasMoreTokens()) {
					String next = st.nextToken();
					if (st.hasMoreTokens()) {
						mark = "," + next;
					} else {
						mark = next;
					}
					newissuer = mark + newissuer;
				}

				v3CertGen.reset();
				// Sun JCCE reverts subject DN when it was a key GRRRRRR....
				v3CertGen.setIssuerDN(
					new X509Name(true, caCertificate.getSubjectDN().getName()));
				logger.info(
					"Setting issuer to: "
						+ caCertificate.getSubjectDN().getName());
				v3CertGen.setSubjectDN(new X509Name(newissuer));
				v3CertGen.setSerialNumber(BigInteger.valueOf(1));
				v3CertGen.setNotBefore(new Date(validFrom));
				v3CertGen.setNotAfter(new Date(validTo));

				v3CertGen.setPublicKey(pkcs10.getPublicKey());
				v3CertGen.setSignatureAlgorithm(caCertificate.getSigAlgName());

				// NETSCAPES KEY-USAGE
				v3CertGen.addExtension(
					"2.5.29.15",
					false,
					new X509KeyUsage(
						X509KeyUsage.digitalSignature
							| X509KeyUsage.keyAgreement
							| X509KeyUsage.keyEncipherment));
				// NETSCPAE CERT-TYPE digitalSignature == SSL-Client
				v3CertGen.addExtension(
					"2.16.840.1.113730.1.1",
					false,
					new X509KeyUsage(X509KeyUsage.digitalSignature));

				cert =
					manager.signCertificate(
						v3CertGen,
						manager.getCertificateByAlias(keyAlias)[0]);
				writePEM(cert);
			} else {
				logger.info("Received an invalid CSR");
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could not sign CSR", e);
		}
		return new ObjectWrapperBean(cert);
	}

	public void writePEM(X509Certificate cert) throws Exception {
		String certStart = "-----BEGIN CERTIFICATE-----\n";
		String certEnd = "\n-----END CERTIFICATE-----\n";
		String certFile = certDir + File.separator + cert.hashCode() + ".pem";

		FileOutputStream fos = new FileOutputStream(certFile);

		// proxy certificate
		fos.write(certStart.getBytes());
		fos.write(Base64.encode(cert.getEncoded(), true));
		fos.write(certEnd.getBytes());
		fos.close();
	}
}
