package com.pallas.ogsa.util;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Stub;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    21. Februar 2003
 *@version    $Id: OGSIStub.java,v 1.2 2003/06/30 06:51:23 rrate Exp $
 */
public abstract class OGSIStub extends Stub {
	/**
	 *  Description of the Field
	 */
	protected Hashtable operations = new Hashtable();

	/**
	 *  Description of the Method
	 *
	 *@return                      Description of the Return Value
	 *@exception  RemoteException  Description of the Exception
	 */
	protected Call createCall() throws RemoteException {
		try {
			Call call = (Call) super.service.createCall();
			if (super.maintainSessionSet) {
				call.setMaintainSession(super.maintainSession);
			}
			if (super.cachedUsername != null) {
				call.setUsername(super.cachedUsername);
			}
			if (super.cachedPassword != null) {
				call.setPassword(super.cachedPassword);
			}
			if (super.cachedEndpoint != null) {
				call.setTargetEndpointAddress(super.cachedEndpoint);
			}
			if (super.cachedTimeout != null) {
				call.setTimeout(super.cachedTimeout);
			}
			if (super.cachedPortName != null) {
				call.setPortName(super.cachedPortName);
			}
			Enumeration keys = super.cachedProperties.keys();
			while (keys.hasMoreElements()) {
				String key = (String) keys.nextElement();
				call.setProperty(key, super.cachedProperties.get(key));
			}
			// All the type mapping information is registered
			// when the first call is made.
			// The type mapping information is actually registered in
			// the TypeMappingRegistry of the service, which
			// is the reason why registration is only needed for the first call.
			synchronized (this) {
				if (firstCall()) {
					// must set encoding style before registering serializers
					call.setEncodingStyle(null);
					OGSITypeRegistration.registerAll(call);
				}
			}

			// init stuff
			call.setUseSOAPAction(true);
			call.setSOAPActionURI("");
			call.setEncodingStyle(null);
			call.setProperty(Call.SEND_TYPE_ATTR, Boolean.FALSE);
			call.setProperty(
				org.apache.axis.AxisEngine.PROP_DOMULTIREFS,
				Boolean.FALSE);
			call.setSOAPVersion(
				org.apache.axis.soap.SOAPConstants.SOAP11_CONSTANTS);

			setRequestHeaders(call);
			setAttachments(call);

			return call;
		} catch (Throwable t) {
			throw new AxisFault("Failure trying to get the Call object", t);
		}
	}

	/**
	 *  Gets the response attribute of the IUPLSoapBindingStub object
	 *
	 *@param  call                 Description of the Parameter
	 *@param  resp                 Description of the Parameter
	 *@return                      The response value
	 *@exception  RemoteException  Description of the Exception
	 */
	protected ObjectWrapperBean getResponse(Call call, Object resp)
		throws RemoteException {
		try {
			System.err.println("--------- Server Response -------");
			call.getMessageContext().getResponseMessage().writeTo(System.err);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (resp instanceof RemoteException) {
			throw (RemoteException) resp;
		} else {
			getResponseHeaders(call);
			extractAttachments(call);
			try {
				return (ObjectWrapperBean) resp;
			} catch (Exception exception) {
				return (
					ObjectWrapperBean) org.apache.axis.utils.JavaUtils.convert(
					resp,
					ObjectWrapperBean.class);
			}
		}
	}

}
