package com.pallas.ogsa.util;

import java.util.HashMap;
import java.util.Iterator;

import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.TypeMapping;
import javax.xml.rpc.encoding.TypeMappingRegistry;

import org.apache.axis.client.Call;
import org.apache.axis.encoding.DeserializerFactory;
import org.apache.axis.encoding.SerializerFactory;
import org.apache.axis.encoding.ser.ArrayDeserializerFactory;
import org.apache.axis.encoding.ser.ArraySerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.gridforum.www.ogsi.HandleType;
import org.gridforum.www.ogsi.LocatorType;
import org.gridforum.www.ogsi.ReferenceType;

import com.pallas.ogsa.extensions.ServiceReferenceType;

/**
 *@author     Ralf Ratering
 *@created    17. Mrz 2003
 *@version    $Id: OGSITypeRegistration.java,v 1.5 2003/07/03 06:51:40 rrate Exp $
 */
public class OGSITypeRegistration {


	private static HashMap mappings = new HashMap();
	public static String OGSI_EXTENSIONS_NAMESPACE =
		"http://extensions.ogsa.pallas.com";
	public static String OGSI_NAMESPACE = "http://ogsi.www.gridforum.org";
	public static String UTIL_NAMESPACE = "http://util.ogsa.pallas.com";

	static {
		init();
	}
	
	/**
	 *  Description of the Method
	 *
	 *@param  cls  Description of the Parameter
	 *@return      Description of the Return Value
	 */
	public static OGSIMapping get(Class cls) {
		return (OGSIMapping) mappings.get(cls);
	}

	/**
	 *  Description of the Method
	 */
	private static void init() {
		// register OGSI classes
		Class cls = LocatorType.class;
		QName qname = new QName(OGSI_NAMESPACE, "LocatorType");
		register(
			cls,
			qname,
			new BeanSerializerFactory(cls, qname),
			new BeanDeserializerFactory(cls, qname));

		qname = new QName(OGSI_NAMESPACE, "ArrayOf_LocatorType");
		register(
			LocatorType[].class,
			qname,
			new ArraySerializerFactory(),
			new ArrayDeserializerFactory());

		cls = HandleType.class;
		qname = new QName(OGSI_NAMESPACE, "HandleType");
		register(
			cls,
			qname,
			new BeanSerializerFactory(cls, qname),
			new BeanDeserializerFactory(cls, qname));

		cls = ReferenceType.class;
		qname = new QName(OGSI_NAMESPACE, "ReferenceType");
		register(
			cls,
			qname,
			new BeanSerializerFactory(cls, qname),
			new BeanDeserializerFactory(cls, qname));

		cls = ServiceReferenceType.class;
		qname = new QName(OGSI_EXTENSIONS_NAMESPACE, "ServiceReferenceType");
		register(
			cls,
			qname,
			new BeanSerializerFactory(cls, qname),
			new BeanDeserializerFactory(cls, qname));

		// register wrapper bean
		cls = ObjectWrapperBean.class;
		qname = new QName(UTIL_NAMESPACE, "ObjectWrapperBean");
		register(
			cls,
			qname,
			new BeanSerializerFactory(cls, qname),
			new BeanDeserializerFactory(cls, qname));
	}

	/**
	 *  Description of the Method
	 *
	 *@param  cls                  Description of the Parameter
	 *@param  qname                Description of the Parameter
	 *@param  serializerFactory    Description of the Parameter
	 *@param  deserializerFactory  Description of the Parameter
	 */
	public static void register(
		Class cls,
		QName qname,
		SerializerFactory serializerFactory,
		DeserializerFactory deserializerFactory) {
		mappings.put(
			cls,
			new OGSIMapping(
				cls,
				qname,
				serializerFactory,
				deserializerFactory));
	}

	/**
	 *  Description of the Method
	 *
	 *@param  call  Description of the Parameter
	 */
	public static void registerAll(Call call) {
		Iterator iterator = mappings.keySet().iterator();
		while (iterator.hasNext()) {
			OGSIMapping mapping = (OGSIMapping) mappings.get(iterator.next());
			call.registerTypeMapping(
				mapping.getMappedClass(),
				mapping.getQName(),
				mapping.getSerializerFactory(),
				mapping.getDeserializerFactory());
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  registry  Description of the Parameter
	 */
	public static void registerAll(TypeMappingRegistry registry) {
		Iterator iterator = mappings.keySet().iterator();
		while (iterator.hasNext()) {
			TypeMapping typeMapping = registry.getDefaultTypeMapping();
			OGSIMapping mapping = (OGSIMapping) mappings.get(iterator.next());
			typeMapping.register(
				mapping.getMappedClass(),
				mapping.getQName(),
				mapping.getSerializerFactory(),
				mapping.getDeserializerFactory());
		}
	}

	/**
	 *  Constructor for the OGSITypeRegistration object
	 */
	private OGSITypeRegistration() {
	}

}
