package com.pallas.ogsa.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *@author     Ralf Ratering
 *@created    20. Februar 2003
 *@version    $Id: ObjectWrapperBean.java,v 1.2 2003/06/30 06:51:24 rrate Exp $
 */
public class ObjectWrapperBean implements Serializable {
	private static Logger logger = Logger.getLogger("com.pallas.ogsa.util");
	private byte[] bytes;
	private Object equalsCalc;
	private boolean hashCodeCalc = false;

	/**
	 *  Constructor for the ObjectWrapperBean object
	 */
	public ObjectWrapperBean() {
	}

	/**
	 *  Constructor for the ObjectWrapperBean object
	 *
	 *@param  object  Description of the Parameter
	 */
	public ObjectWrapperBean(Object object) {
		serialize(object);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public Object deserialize() {
		try {
			ObjectInputStream stream =
				new ObjectInputStream(new ByteArrayInputStream(bytes));
			Object object = stream.readObject();
			return object;
		} catch (Exception e) {
			logger.log(Level.WARNING, "Could not deserialize object.", e);
		}
		return null;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  obj  Description of the Parameter
	 *@return      Description of the Return Value
	 */
	public synchronized boolean equals(Object obj) {
		if (!(obj instanceof ObjectWrapperBean)) {
			return false;
		}
		ObjectWrapperBean other = (ObjectWrapperBean) obj;
		if (obj == null) {
			return false;
		}
		if (this == obj) {
			return true;
		}
		if (equalsCalc != null) {
			return (equalsCalc == obj);
		}
		equalsCalc = obj;
		boolean equals =
			true
				&& ((bytes == null && other.getBytes() == null)
					|| (bytes != null && Arrays.equals(bytes, other.getBytes())));
		equalsCalc = null;
		return equals;
	}

	/**
	 *  Gets the bytes attribute of the ObjectWrapperBean object
	 *
	 *@return    The bytes value
	 */
	public byte[] getBytes() {
		return bytes;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public synchronized int hashCode() {
		if (hashCodeCalc) {
			return 0;
		}
		hashCodeCalc = true;
		int hashCode = 1;
		if (getBytes() != null) {
			for (int i = 0; i < Array.getLength(getBytes()); i++) {
				Object obj = Array.get(getBytes(), i);
				if (obj != null && !obj.getClass().isArray()) {
					hashCode += obj.hashCode();
				}
			}
		}
		hashCodeCalc = false;
		return hashCode;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 */
	public void serialize(Object object) {
		try {
			ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
			ObjectOutputStream objectOut = new ObjectOutputStream(byteOut);
			objectOut.writeObject(object);
			objectOut.flush();
			byteOut.flush();
			objectOut.close();
			bytes = byteOut.toByteArray();
		} catch (Exception e) {
			// delete previous contents
			bytes = null;
			logger.log(
				Level.WARNING,
				"Could not serialize object: " + object,
				e);
		}
	}

	/**
	 *  Sets the bytes attribute of the ObjectWrapperBean object
	 *
	 *@param  bytes  The new bytes value
	 */
	public void setBytes(byte[] bytes) {
		this.bytes = bytes;
	}
}
