package com.pallas.security;


import java.io.*;
import java.lang.reflect.*;
import java.text.SimpleDateFormat;
import java.security.PublicKey;
import java.security.cert.*;
import java.util.*;
import java.util.logging.*;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.*;
import org.apache.xml.security.c14n.*;
import org.apache.xml.security.exceptions.XMLSecurityException;
import org.apache.xml.security.signature.*;
import org.apache.xml.security.keys.*;
import org.apache.xml.security.keys.content.*;
import org.apache.xml.security.keys.content.x509.*;
import org.apache.xml.security.keys.keyresolver.*;
import org.apache.xml.security.keys.storage.*;
import org.apache.xml.security.keys.storage.implementations.*;
import org.apache.xml.security.utils.*;
import org.apache.xml.security.Init;


/**
 *
 *
 *
 *
 * @author $Author: thomas $
 *
 */
public class Clear {

	private static Logger logger = Logger.getLogger("com.pallas.security");

	private static Clear instance = new Clear();
	private final static int GRACE = 1000*60*60*24*14;

	private Clear() {}

	public static Clear getInstance() {
		return instance;
		}


	public static void main(String no[]) {
		Clear next = Clear.getInstance();
		try {
			next.verify();
			} catch (LicenseExpiredException le) {
			logger.fine("License about to expire. Continuing");
			}
		catch (LicenseException e) {
			// e.printStackTrace();
			}
		}


	public void verify() throws LicenseExpiredException, LicenseException {

		boolean schemaValidate = false;
		final String signatureSchemaFile = "data/xmldsig-core-schema.xsd";
		// String file = "data/ie/baltimore/merlin-examples/merlin-xmldsig-fifteen/signature-enveloping-rsa.xml";
		String file = "signedlicense.xml";

		if (schemaValidate) {
			// System.out.println("We do schema-validation");
			}

		javax.xml.parsers.DocumentBuilderFactory dbf =
		        javax.xml.parsers.DocumentBuilderFactory.newInstance();


		if (schemaValidate) {
			dbf.setAttribute("http://apache.org/xml/features/validation/schema",
			                 Boolean.TRUE);
			dbf.setAttribute(
			        "http://apache.org/xml/features/dom/defer-node-expansion",
			        Boolean.TRUE);
			dbf.setValidating(true);
			dbf.setAttribute("http://xml.org/sax/features/validation",
			                 Boolean.TRUE);
			}

		dbf.setNamespaceAware(true);
		dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);

		if (schemaValidate) {
			dbf.setAttribute(
			        "http://apache.org/xml/properties/schema/external-schemaLocation",
			        Constants.SignatureSpecNS + " " + signatureSchemaFile);
			}

		org.w3c.dom.Document doc = null;
		XMLSignature signature = null;
		KeyInfo ki = null;
		X509Certificate cert = null;

		try {

			File f = new File(file);

			logger.fine("Try to verify " + f.toURL().toString());

			javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();

			db.setErrorHandler(new org.apache.xml.security.utils
			                   .IgnoreAllErrorHandler());

			if (schemaValidate) {
				db.setEntityResolver(new org.xml.sax.EntityResolver() {

					                     public org.xml.sax.InputSource resolveEntity(
					                             String publicId, String systemId)
					                     throws org.xml.sax.SAXException {

						                     if (systemId.endsWith("xmldsig-core-schema.xsd")) {
							                     try {
								                     return new org.xml.sax.InputSource(
								                                    new FileInputStream(signatureSchemaFile));
								                     } catch (FileNotFoundException ex) {
								                     throw new org.xml.sax.SAXException(ex);
								                     }
							                     } else {
							                     return null;
							                     }
						                     }
					                     }
				                    );
				}

			doc = db.parse(new java.io.FileInputStream(f));
			Element nscontext = XMLUtils.createDSctx(doc, "ds",
			                    Constants.SignatureSpecNS);
			Element sigElement = (Element) XPathAPI.selectSingleNode(doc,
			                     "//ds:Signature[1]", nscontext);

			signature = new XMLSignature(sigElement,
			                             f.toURL().toString());

			signature.addResourceResolver(new OfflineResolver());

			//XMLUtils.outputDOMc14nWithComments(signature.getElement(), System.out);
			ki = signature.getKeyInfo();


			} catch (Exception ex) {
			throw new LicenseException(ex);
			}

		if (ki != null) {
			if (!ki.containsX509Data()) {
				logger.fine("Could find a properly signed license. Please contact Pallas for a valid one");
				throw new LicenseException("Could find a properly signed license. Please contact Pallas for a valid one");
				}

			try {
				cert = signature.getKeyInfo().getX509Certificate();
				} catch (KeyResolverException xmle) {
				throw new LicenseException(xmle);
				}


			if (cert != null) {
				logger.fine("Checking certificate..");
				try {
					cert.checkValidity();

					} catch (CertificateExpiredException cee) {
					logger.fine("Could find a properly signed license.\n certificate expired\nPlease contact Pallas for a valid one");
					throw new LicenseException("Could find a properly signed license.\n certificate expired\nPlease contact Pallas for a valid one");
					}
				catch (CertificateNotYetValidException cnyve) {
					logger.fine("Could find a properly signed license.\n certificate not yet valid\nPlease contact Pallas for a valid one");
					throw new LicenseException("Could find a properly signed license.\n certificate expired\nPlease contact Pallas for a valid one");
					}
				logger.fine("Checking signauture..");

				try {
					if(!signature.checkSignatureValue(cert)) {
						logger.fine("License has been tampered with!");
						throw new LicenseException("License has been tampered with!");
						}
					} catch (XMLSignatureException xmle) {
					throw new LicenseException(xmle);
					}

				try {
					logger.fine("License-ID:\t"+getId(doc));
					logger.fine("Issued:\t"+getFrom(doc));
					logger.fine("Valid to:\t"+getValidTo(doc));
					logger.fine("Processors:\t"+getProcessors(doc));
					logger.fine("Sites:\t"+getVsites(doc));
					String [] hosts = getHostIds(doc);
					for(int i=0; i< hosts.length; i++) {
						logger.fine("HostId["+i+"]:\t"+ hosts[i]);
						}
					} catch (java.text.ParseException pe) {
					throw new LicenseException(pe);
					}

				logger.fine("Checking dates...");
				try {
					Date validTo = getValidTo(doc);
					Date now = new Date(System.currentTimeMillis());
					if(now.compareTo(validTo) > 0) {

						Date grace = new Date(validTo.getTime()+GRACE);
						if(now.compareTo(grace) > 0) {
							logger.fine("License expired on "+grace);
							throw new LicenseException("License expired on "+grace);
							} else {
							logger.fine("License has expired on "+validTo);
							throw new LicenseExpiredException("License about to expire on "+grace);
							}
						}
					} catch (java.text.ParseException pe) {
					logger.fine("Invalid license format!");
					throw new LicenseException("Invalid license format!");
					}


				} else {
				logger.fine("Invalid license found. Please obtain a proper one.");
				throw new LicenseException("Invalid license found. Please obtain a proper one.");
				}

			} else {
			logger.fine("Could find a properly signed license. Please contact Pallas for a valid one");
			throw new LicenseException("Could find a properly signed license. Please contact Pallas for a valid one");

			}




		}

	private  Date getValidTo(Document doc) throws java.text.ParseException {
		Date validToDate = null;
		NodeList serviceNodes
		= doc.getElementsByTagNameNS("",
		                             "valid-to");
		for(int serviceNode = 0; serviceNode < serviceNodes.getLength();
		        serviceNode++) {
			Element serviceElement = (Element)serviceNodes.item(serviceNode);
			String validTo = serviceElement.getAttribute("date");
			SimpleDateFormat df = new SimpleDateFormat("MMM dd, yyyy hh:mm:ss aa");
			validToDate = df.parse(validTo);
			// expect only one date
			break;
			}
		return validToDate;
		}

	private  Date getFrom(Document doc) throws java.text.ParseException {
		Date validFromDate = null;
		NodeList serviceNodes
		= doc.getElementsByTagNameNS("",
		                             "issued");
		for(int serviceNode = 0; serviceNode < serviceNodes.getLength();
		        serviceNode++) {
			Element serviceElement = (Element)serviceNodes.item(serviceNode);
			String validTo = serviceElement.getAttribute("date");
			SimpleDateFormat df = new SimpleDateFormat("MMM dd, yyyy hh:mm:ss aa");
			validFromDate = df.parse(validTo);
			// expect only one date
			break;
			}
		return validFromDate;
		}

	private  String getId(Document doc) throws java.text.ParseException {
		String id = null;
		NodeList serviceNodes
		= doc.getElementsByTagNameNS("",
		                             "license-id");
		for(int serviceNode = 0; serviceNode < serviceNodes.getLength();
		        serviceNode++) {
			Element serviceElement = (Element)serviceNodes.item(serviceNode);
			id = serviceElement.getAttribute("id");
			break;
			}
		return id;
		}

	private  String getProcessors(Document doc) throws java.text.ParseException {
		String procs = null;
		NodeList serviceNodes
		= doc.getElementsByTagNameNS("",
		                             "processors");
		for(int serviceNode = 0; serviceNode < serviceNodes.getLength();
		        serviceNode++) {
			Element serviceElement = (Element)serviceNodes.item(serviceNode);
			procs = serviceElement.getAttribute("number");
			break;
			}
		return procs;
		}

	private  String getVsites(Document doc) throws java.text.ParseException {
		String vsites = null;
		NodeList serviceNodes
		= doc.getElementsByTagNameNS("",
		                             "vsites");
		for(int serviceNode = 0; serviceNode < serviceNodes.getLength();
		        serviceNode++) {
			Element serviceElement = (Element)serviceNodes.item(serviceNode);
			vsites = serviceElement.getAttribute("number");
			break;
			}
		return vsites;
		}

	private  String [] getHostIds(Document doc) throws java.text.ParseException {
		NodeList serviceNodes
		= doc.getElementsByTagNameNS("",
		                             "hostid");
		String [] hosts = new String[serviceNodes.getLength()];
		for(int serviceNode = 0; serviceNode < serviceNodes.getLength();
		        serviceNode++) {
			Element serviceElement = (Element)serviceNodes.item(serviceNode);
			hosts[serviceNode] = serviceElement.getAttribute("identifier");
			}
		return hosts;
		}

	static {
		org.apache.xml.security.Init.init();
		}
	}
