package com.pallas.unicore.client;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.pallas.unicore.utility.UserMessages;

/**
 *  Define, load and save system default values
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: SystemDefaults.java,v 1.4 2003/06/30 06:51:55 rrate Exp $
 */
public class SystemDefaults extends Properties {

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client");
	/**
	 *  String constants defining property keys
	 */
	public final static String USITE_URL_KEY = "systemdefault.usite.url",
		CRL_URL_KEY = "systemdefault.crl.url";
	/**
	 *  Description of the Field
	 */
	public final String fallbackCRL = "";

	/**
	 *  Description of the Field
	 */
	public final String fallbackURL = "http://www.unicore.de/unicoreSites.xml";

	/**
	 *  Constructor for the SystemDefaults object
	 */
	public SystemDefaults() {
		super();
		// override with command line settings
		if (System.getProperty(USITE_URL_KEY) != null) {
			setProperty(USITE_URL_KEY, System.getProperty(USITE_URL_KEY));
		} else {
			setProperty(USITE_URL_KEY, fallbackURL);
		}
		if (System.getProperty(CRL_URL_KEY) != null) {
			setProperty(CRL_URL_KEY, System.getProperty(CRL_URL_KEY));
		} else {
			setProperty(CRL_URL_KEY, fallbackCRL);
		}
	}

	/**
	 *  Gets the CrlUrl attribute of the SystemDefaults object
	 *
	 *@return    The CrlUrl value
	 */
	public String getCrlURL() {
		return getProperty(CRL_URL_KEY);
	}

	/**
	 *  Overwrite getProperty method to prevent return value null and return ""
	 *  instead. This is important to prevent null pointer execptions from
	 *  faulty ScriptDefaults files.
	 *
	 *@param  prop  Description of Parameter
	 *@return       The Property value
	 */
	public String getProperty(String prop) {
		String value = super.getProperty(prop);

		if (value == null) {
			UserMessages.warning(
				"Could not read system default property: " + prop);
			return "";
		} else {
			return value;
		}
	}

	/**
	 *  Gets the ScriptDirectory attribute of the ScriptDefaults object
	 *
	 *@return    The ScriptDirectory value
	 */
	public String getUsiteURL() {
		return getProperty(USITE_URL_KEY);
	}

	/**
	 *  Load system default values from file LIBDIR/SystemDefaults.txt
	 *
	 *@param  filename  Description of the Parameter
	 *@return           true if loading was successful, else false
	 */
	public boolean loadFromFile(String filename) {
		boolean retVal = false;
		boolean urlSet = false;
		logger.info("Trying to load: " + filename);
		File defs = new File(filename);
		if (defs.exists()) {
			// try to load file
			try {
				load(new FileInputStream(defs));
				retVal = true;
			}
			// loading failed
			catch (Exception e) {
				// no error message here, but in calling method
				logger.log(Level.SEVERE, "", e);
				retVal = false;
			}
		}

		// no LIBDIR/SystemDefaults.txt found
		else {
			retVal = false;
		}

		// override with command line settings
		if (System.getProperty(USITE_URL_KEY) != null) {
			setProperty(USITE_URL_KEY, System.getProperty(USITE_URL_KEY));
			urlSet = true;
		}
		if (System.getProperty(CRL_URL_KEY) != null) {
			setProperty(CRL_URL_KEY, System.getProperty(CRL_URL_KEY));
			retVal = true && urlSet;
		}

		// loading was successful
		return retVal;
	}

}
