package com.pallas.unicore.client;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.io.File;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.plaf.metal.DefaultMetalTheme;

import org.unicore.User;
import org.unicore.Vsite;

import com.pallas.unicore.client.themes.LookAndFeelManager;
import com.pallas.unicore.client.themes.UserTheme;
import com.pallas.unicore.client.util.Defaults;
import com.pallas.unicore.connection.ProxyManager;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Define, load and save user default values
 *
 *@author     Ralf Ratering
 *@created    14. August 2001
 */
public class UserDefaults extends Defaults {
    /**
     *  Description of the Field
     */
    public final static String
            BINARY_FORMAT = "serialized",
            XML_FORMAT = "XML",
            PLAIN_FORMAT = "PLAIN";

    /**
     *  Description of the Field
     */
    public final static String DEFAULT = "default::";

    private final static Logger logger = Logger.getLogger("com.pallas.unicore.client");

    // property string constants
    private final static String USITEURL = "unicore.usiteurl";
    private final static String CRLURL = "unicore.crl";
    private final static String PROXY_HOST = "unicore.proxy.host";
    private final static String PROXY_PORT = "unicore.proxy.port";
	private final static String NO_PROXY = "unicore.proxy.noproxy";
    private final static String PROXY_ENABLED = "unicore.proxy.enabled";
    private final static String PLUGINDIR = "unicore.plugin.directory";
    private final static String JOBDIR = "unicore.job.directory";
    private final static String RESOURCEDIR = "unicore.resource.directory";
    private final static String IMPORTDIR = "unicore.import.directory";
    private final static String EXPORTDIR = "unicore.export.directory";
    private final static String LOGDIR = "unicore.log.directory";
    private final static String KEYSTOREDIR = "unicore.keystore.path";
    private final static String TMPDIR = "unicore.keystore.tmp";
    private final static String REDIRECT = "unicore.redirect.stdout";
    private final static String USEREMAIL = "unicore.email";
    private final static String XLOGIN = "unicore.xlogin";
    private final static String PROJECT = "unicore.project";
    private final static String DEFAULTIDENTITY = "unicore.identity";
    private final static String LOOKANDFEEL = "unicore.plaf";
    private final static String JOBSTORAGEFORMAT = "unicore.jobformat";
    private final static String AUTOFETCHOUTCOME = "unicore.autofetch.outcome";
    private final static String DETAILEDJOBSTATUS = "unicore.detailed.jobstatus";
    private final static String STARTOFFLINE = "unicore.startoffline";
    private final static String SOCKET_TIMEOUT = "unicore.sockettimeout";
    private final static String THEME = "unicore.theme";
    private final static String THEME_PRIMARY1 = "unicore.theme.primary1";
    private final static String THEME_PRIMARY2 = "unicore.theme.primary2";
    private final static String THEME_PRIMARY3 = "unicore.theme.primary3";
    private final static String THEME_SECONDARY1 = "unicore.theme.secondary1";
    private final static String THEME_SECONDARY2 = "unicore.theme.secondary2";
    private final static String THEME_SECONDARY3 = "unicore.theme.secondary3";
    private final static String THEME_WHITE = "unicore.theme.white";
    private final static String THEME_BLACK = "unicore.theme.black";
    private final static String THEME_FONT_NAME = "unicore.theme.font.name";
    private final static String THEME_FONT_SIZE = "unicore.theme.font.size";
    private final static String WINDOW_SIZE_X = "unicore.window.size.x";
    private final static String WINDOW_SIZE_Y = "unicore.window.size.y";
    private final static String WINDOW_POS_X = "unicore.window.pos.x";
    private final static String WINDOW_POS_Y = "unicore.window.pos.y";
    private final static String LOG_LEVEL = "unicore.log.level";
    private final static String LOG_FORMAT = "unicore.log.format";
    private final static String PREEMPTIVE_FILESELECTOR_MODE = "unicore.preemptive.mode";
    private final static String IDENTITY = "unicore.identity.";

    // user home directory
    static ResourceBundle res = ResourceBundle.getBundle("com.pallas.unicore.client.ResourceStrings");



    /**
     *  Sets the Theme attribute of the UserDefaults object
     *
     *@param  theme  The new Theme value
     */
    public void setTheme(DefaultMetalTheme theme) {
        put(THEME, theme.getName());

        if(theme instanceof UserTheme) {
            UserTheme userTheme = (UserTheme)theme;
            put(THEME_PRIMARY1, colorString(userTheme.getPrimary1Color()));
            put(THEME_PRIMARY2, colorString(userTheme.getPrimary2Color()));
            put(THEME_PRIMARY3, colorString(userTheme.getPrimary3Color()));
            put(THEME_SECONDARY1, colorString(userTheme.getSecondary1Color()));
            put(THEME_SECONDARY2, colorString(userTheme.getSecondary2Color()));
            put(THEME_SECONDARY3, colorString(userTheme.getSecondary3Color()));
            put(THEME_BLACK, colorString(userTheme.getBlackColor()));
            put(THEME_WHITE, colorString(userTheme.getWhiteColor()));
            put(THEME_FONT_NAME, userTheme.getControlTextFont().getName());
            put(THEME_FONT_SIZE,
                    new Integer(userTheme.getControlTextFont().getSize()).toString());
        }
    }


    /**
     *  Sets the identitiy attribute of the UserDefaults object
     *
     *@param  vsite
     *@param  value  The new identitiy value
     */
    public void setIdentitiy(Vsite vsite, String value) {
        String key = IDENTITY + vsite.getAddress() + "." + vsite.toString();
        if(value.equals(this.DEFAULT)) {
            String identity = (String)get(key);
            if(identity == null) {
                return;
            }
            if(!identity.equals("")) {
                this.remove(key);
            }
            return;
        }
        put(key, value);
    }


    /**
     *  Sets the identitiy attribute of the UserDefaults object
     *
     *@param  value  The new identitiy value
     *@param  usite  The new identitiy value
     */
    public void setIdentitiy(Usite usite, String value) {
        String key = IDENTITY + usite.toString();
        if(value.equals(this.DEFAULT)) {
            String identity = (String)get(key);
            if(identity == null) {
                return;
            }
            if(!identity.equals("")) {
                this.remove(key);
            }
            return;
        }
        put(key, value);
    }


    /**
     *  Sets the LookAndFeel attribute of the UserDefaults object
     *
     *@param  value  The new LookAndFeel value
     */
    public void setLookAndFeel(String value) {
        put(LOOKANDFEEL, value);
    }


    /**
     *  Set URL where available usites can be requested
     *
     *@param  url  The new UsiteURL value
     */
    public void setUsiteURL(String url) {
        put(USITEURL, url);
    }


    /**
     *  Set URL where the CRL can be requested
     *
     *@param  url  The new UsiteURL value
     */
    public void setCrlURL(String url) {
        put(CRLURL, url);
    }


    /**
     *  Sets the logLevel attribute of the UserDefaults object
     *
     *@param  level  The new logLevel value
     */
    public void setLogLevel(String level) {
        put(LOG_LEVEL, level);
    }


    /**
     *  Sets the preemptiveMode attribute of the UserDefaults object
     *
     *@param  mode  The new preemptiveMode value
     */
    public void setPreemptiveFileSelectorMode(boolean mode) {
        put(PREEMPTIVE_FILESELECTOR_MODE, new Boolean(mode).toString());
    }


    /**
     *  Sets the logFormat attribute of the UserDefaults object
     *
     *@param  format  The new logFormat value
     */
    public void setLogFormat(String format) {
        put(LOG_FORMAT, format);
    }


    /**
     *  Set default email address
     *
     *@param  str  email address
     */
    public void setUserEmail(String str) {
        put(USEREMAIL, str);
        try {
            ResourceManager.updateDefaultIdentity();
        }
        catch(Exception e) {
            UserMessages.print("Setting User Email failed.", e.getMessage());
        }
    }


    /**
     *  Set default xlogin
     *
     *@param  str  xlogin
     */
    public void setXlogin(String str) {
        put(XLOGIN, str);
        try {
            ResourceManager.updateDefaultIdentity();
        }
        catch(Exception e) {
            UserMessages.print("Setting Xlogin failed.", e.getMessage());
        }
    }


    /**
     *  Set default project
     *
     *@param  str  project
     */
    public void setProject(String str) {
        put(PROJECT, str);
        try {
            ResourceManager.updateDefaultIdentity();
        }
        catch(Exception e) {
            UserMessages.print("Setting Project failed.", e.getMessage());
        }
    }


    /**
     *  Set default identity
     *
     *@param  str  identity
     */
    public void setDefaultIdentity(String str) {
        put(DEFAULTIDENTITY, str);
    }


    /**
     *  Set directory that is scanned for plugins
     *
     *@param  dirname  The new PluginDirectory value
     */
    public void setPluginDirectory(String dirname) {
        put(PLUGINDIR, dirname);
    }


    /**
     *  Set directory where stdout/stderr and logs are stored
     *
     *@param  dirname  The new LogDirectory value
     */
    public void setLogDirectory(String dirname) {
        put(LOGDIR, dirname);
    }


    /**
     *  Set directory where tmp files are stored
     *
     *@param  dirname  The new TmpDirectory value
     */
    public void setTmpDirectory(String dirname) {
        put(TMPDIR, dirname);
    }


    /**
     *  Set name for key store
     *
     *@param  s  The new KeyStoreName value
     */
    public void setKeyStoreName(String s) {
        put(KEYSTOREDIR, s);
    }


    /**
     *  Set path for job directory
     *
     *@param  s  The new JobDirectory value
     */
    public void setJobDirectory(String s) {
        put(JOBDIR, s);
    }


    /**
     *  Sets the resourceDirectory attribute of the UserDefaults object
     *
     *@param  s  The new resourceDirectory value
     */
    public void setResourceDirectory(String s) {
        put(RESOURCEDIR, s);
    }


    /**
     *  Sets the ImportDirectory attribute of the UserDefaults object
     *
     *@param  s  The new ImportDirectory value
     */
    public void setImportDirectory(String s) {
        put(IMPORTDIR, s);
    }


    /**
     *  Sets the ExportDirectory attribute of the UserDefaults object
     *
     *@param  s  The new ExportDirectory value
     */
    public void setExportDirectory(String s) {
        put(EXPORTDIR, s);
    }


    /**
     *  Sets the Redirect attribute of the UserDefaults object
     *
     *@param  b  The new Redirect value
     */
    public void setRedirect(boolean b) {
        put(REDIRECT, new Boolean(b).toString());
    }


    /**
     *  Sets the startOffline attribute of the UserDefaults object
     *
     *@param  b  The new startOffline value
     */
    public void setStartOffline(boolean b) {
        put(STARTOFFLINE, new Boolean(b).toString());
    }


    /**
     *  Sets the ProxyEnabled attribute of the UserDefaults object
     *
     *@param  enabled  The new ProxyEnabled value
     */
    public void setProxyEnabled(boolean enabled) {
        put(PROXY_ENABLED, new Boolean(enabled).toString());
    }


    /**
     *  Sets the ProxyHost attribute of the UserDefaults object
     *
     *@param  host  The new ProxyHost value
     */
    public void setProxyHost(String host) {
        setProperty(PROXY_HOST, host);
    }


    /**
     *  Sets the ProxyPort attribute of the UserDefaults object
     *
     *@param  port  The new ProxyPort value
     */
    public void setProxyPort(int port) {
        put(PROXY_PORT, new Integer(port).toString());
    }
    
	/**
	 *  Sets host thta should be contacted without proxy
	 *
	 *@param  noProxy  List of hosts 
	 */
	public void setNoProxy(String noProxy) {
		setProperty(NO_PROXY, noProxy);
	}


    /**
     *  Sets the JobStorageFormat attribute of the UserDefaults object
     *
     *@param  what  The new JobStorageFormat value
     */
    public void setJobStorageFormat(String what) {
        put(JOBSTORAGEFORMAT, what);
    }


    /**
     *  Sets the automatic fetching of outcome
     *
     *@param  what  The new JobStorageFormat value
     */
    public void setAutofetchOutcome(boolean what) {
        put(AUTOFETCHOUTCOME, new Boolean(what).toString());
    }


    /**
     *  Sets the detailed job status display
     *
     *@param  what  The new detail status
     */
    public void setDetailedJobStatus(boolean what) {
        put(DETAILEDJOBSTATUS, new Boolean(what).toString());
    }


    /**
     *  Sets the windowSize attribute of the UserDefaults object
     *
     *@param  dim  The new windowSize value
     */
    public void setWindowSize(Dimension dim) {
        put(WINDOW_SIZE_X, new Integer(dim.width).toString());
        put(WINDOW_SIZE_Y, new Integer(dim.height).toString());
    }


    /**
     *  Sets the windowSize attribute of the UserDefaults object
     *
     *@param  pos  The new windowSize value
     */
    public void setWindowPos(Point pos) {
        put(WINDOW_POS_X, new Integer(pos.x).toString());
        put(WINDOW_POS_Y, new Integer(pos.y).toString());
    }



    /**
     *  Sets the timeout value for connect and read operations
     *
     *@param  timeout  The new timeout value
     */
    public void setSocketTimeout(int timeout) {
        put(SOCKET_TIMEOUT, new Integer(timeout).toString());
    }


    /**
     *  Gets the Theme attribute of the UserDefaults object
     *
     *@return    The Theme value
     */
    public DefaultMetalTheme getTheme() {
        String themeName = (String)get(THEME);
        if(themeName == null) {
            return new DefaultMetalTheme();
        }

        DefaultMetalTheme theme = LookAndFeelManager.getThemeByName(themeName);

        if(theme instanceof UserTheme) {
            UserTheme userTheme = (UserTheme)theme;
            userTheme.setPrimary1Color(colorValue((String)get(THEME_PRIMARY1)));
            userTheme.setPrimary2Color(colorValue((String)get(THEME_PRIMARY2)));
            userTheme.setPrimary3Color(colorValue((String)get(THEME_PRIMARY3)));
            userTheme.setSecondary1Color(colorValue((String)get(THEME_SECONDARY1)));
            userTheme.setSecondary2Color(colorValue((String)get(THEME_SECONDARY2)));
            userTheme.setSecondary3Color(colorValue((String)get(THEME_SECONDARY3)));
            userTheme.setBlackColor(colorValue((String)get(THEME_BLACK)));
            userTheme.setWhiteColor(colorValue((String)get(THEME_WHITE)));

            String fontName = (String)get(THEME_FONT_NAME);
            int fontSize = intValue(THEME_FONT_SIZE);
            userTheme.setFont(new Font(fontName, Font.PLAIN, fontSize));
        }
        return theme;
    }


    /**
     *  Gets the LookAndFeel attribute of the UserDefaults object
     *
     *@return    The LookAndFeel value
     */
    public String getLookAndFeel() {
        String lookAndFeel = (String)get(LOOKANDFEEL);
        if(lookAndFeel == null) {
            lookAndFeel = "javax.swing.plaf.metal.MetalLookAndFeel";
            put(LOOKANDFEEL, lookAndFeel);
        }
        return lookAndFeel;
    }


    /**
     *  Get URL where available usites can be requested
     *
     *@return    String containing URL
     */
    public String getUsiteURL() {
        return getProperty(USITEURL);
    }


    /**
     *  Get URL where CRL can be requested
     *
     *@return    String containing URL
     */
    public String getCrlURL() {
        return getProperty(CRLURL);
    }


    /**
     *  Get default email address
     *
     *@return    default email address
     */
    public String getUserEmail() {
        return getProperty(USEREMAIL);
    }


    /**
     *  Get default xlogin
     *
     *@return    default xlogin
     */
    public String getXlogin() {
        return getProperty(XLOGIN);
    }


    /**
     *  Get default project
     *
     *@return    default project
     */
    public String getProject() {
        return getProperty(PROJECT);
    }


    /**
     *  Get default identity. This one's always lower case, because the keystore
     *  will only handle lower case aliases.
     *
     *@return    default identity
     */
    public String getDefaultIdentity() {
        return getProperty(DEFAULTIDENTITY).toLowerCase();
    }


    /**
     *  Get directory that is scanned for plugins
     *
     *@return    plugin directory
     */
    public String getPluginDirectory() {
        return getProperty(PLUGINDIR);
    }


    /**
     *  Get directory where stdout/stderr and logs are stored
     *
     *@return    log directory
     */
    public String getLogDirectory() {
        String logDir = getProperty(LOGDIR);
        if(logDir == null || logDir.equals("")) {
            logDir = ".";
        }
        return logDir;
    }


    /**
     *  Get directory where tmp files are stored
     *
     *@return    log directory
     */
    public String getTmpDirectory() {
        return getProperty(TMPDIR);
    }


    /**
     *  Get name for key store
     *
     *@return    default key store name
     */
    public String getKeyStoreName() {
        return getProperty(KEYSTOREDIR);
    }


    /**
     *  Get path to job directory
     *
     *@return    String containing last job path
     */
    public String getJobDirectory() {
        return getProperty(JOBDIR);
    }


    /**
     *  Gets the resourceDirectory attribute of the UserDefaults object
     *
     *@return    The resourceDirectory value
     */
    public String getResourceDirectory() {
        return getProperty(RESOURCEDIR);
    }


    /**
     *  Gets the ImportDirectory attribute of the UserDefaults object
     *
     *@return    The ImportDirectory value
     */
    public String getImportDirectory() {
        return getProperty(IMPORTDIR);
    }


    /**
     *  Gets the ExportDirectory attribute of the UserDefaults object
     *
     *@return    The ExportDirectory value
     */
    public String getExportDirectory() {
        return getProperty(EXPORTDIR);
    }


    /**
     *  Get default value if stdout will be redirected
     *
     *@return    true if stdout is redirected, else false
     */
    public boolean getRedirect() {
        return booleanValue(REDIRECT);
    }


    /**
     *  Gets the startOffline attribute of the UserDefaults object
     *
     *@return    The startOffline value
     */
    public boolean getStartOffline() {
        return booleanValue(STARTOFFLINE);
    }


    /**
     *  Get default value if stdout will be fetched automatically
     *
     *@return    true if stdout is fetched
     */
    public boolean getAutofetchOutcome() {
        return booleanValue(AUTOFETCHOUTCOME);
    }


    /**
     *  Get default value for job status display
     *
     *@return    true if detailed status is enabled
     */
    public boolean getDetailedJobStatus() {
        return booleanValue(DETAILEDJOBSTATUS);
    }


    /**
     *  Gets the ProxyEnabled attribute of the UserDefaults object
     *
     *@return    The ProxyEnabled value
     */
    public boolean isProxyEnabled() {
        return booleanValue(PROXY_ENABLED);
    }


    /**
     *  Gets the ProxyHost attribute of the UserDefaults object
     *
     *@return    The ProxyHost value
     */
    public String getProxyHost() {
        return getProperty("unicore.proxy.host");
    }


    /**
     *  Gets the ProxyPort attribute of the UserDefaults object
     *
     *@return    The ProxyPort value
     */
    public int getProxyPort() {
        return intValue(PROXY_PORT);
    }


	/**
	 *  Gets the list of host that are contacted without the proxy
	 *
	 *@return    The ProxyHost value
	 */
	public String getNoProxy() {
		return getProperty("unicore.proxy.noproxy");
	}

    /**
     *  Gets the logLevel attribute of the UserDefaults object
     *
     *@return    The logLevel value
     */
    public String getLogLevel() {
        String level = (String)get(LOG_LEVEL);
        if(level == null) {
            level = Level.INFO.toString();
            put(LOG_LEVEL, level);
        }
        return level;
    }


    /**
     *  Gets the identity attribute of the UserDefaults object
     *
     *@param  vsite
     *@return        The identity value
     */
    public String getIdentity(Vsite vsite) {
        String key = IDENTITY + vsite.getAddress() + "." + vsite.toString();
        String identity = (String)get(key);
        if(identity == null || identity.equals("")) {
            // look for identity of corresponding Usite
            Usite usite = ResourceManager.getUsite(vsite);
            identity = getIdentity(usite);
        }
        return identity;
    }



    /**
     *  Gets the defaultIdentity attribute of the UserDefaults object
     *
     *@param  usite  Description of the Parameter
     *@return        The defaultIdentity value
     */
    public String getDefaultIdentity(Usite usite) {
        String key = IDENTITY + usite.toString();
        String identity = (String)get(key);
        if(identity == null || identity.equals("")) {
            identity = "";
            String cert = this.getDefaultIdentity();
            try {
                if(!ResourceManager.getKeystoreManager().keyStoreContainsAlias(cert)) {
                    return identity;
                }
            }
            catch(Exception ex) {
                logger.warning("Keystore cannot be loaded");
            }
            identity = cert + ":" + this.getXlogin() + ":" + this.getProject();
        }
        return identity;
    }


    /**
     *  Gets the identity attribute of the UserDefaults object
     *
     *@param  usite  Description of the Parameter
     *@return        The identity value
     */
    public String getIdentity(Usite usite) {
        String key = IDENTITY + usite.toString();
        String identity = (String)get(key);
        if(identity == null || identity.equals("")) {
            identity = this.DEFAULT;
        }
        return identity;
    }


    /**
     *  Description of the Method
     *
     *@param  usite  Description of the Parameter
     *@return        The user value
     */
    public User getUser(Usite usite) {
        String identity = getIdentity(usite);
        if(DEFAULT.equals(identity)) {
            identity = getDefaultIdentity();
        }
        User userUsite = getUser(identity);
        if(userUsite == null) {
            logger.severe("user " + identity + " of the usite: " + usite + " is not available");
        }
        return userUsite;
    }


    /**
     *  Description of the Method
     *
     *@param  vsite  Description of the Parameter
     *@return        The user value
     */
    public User getUser(Vsite vsite) {
        String identity = getIdentity(vsite);
        if(DEFAULT.equals(identity)) {
            identity = getDefaultIdentity();
        }
        User userVsite = getUser(identity);
        if(userVsite == null) {
            logger.severe("user " + identity + " of the vsite: " + vsite + " is not available");
        }
        return userVsite;
    }


    /**
     *  Gets the logFormat attribute of the UserDefaults object
     *
     *@return    The logFormat value
     */
    public String getLogFormat() {
        String format = (String)get(LOG_FORMAT);
        if(format == null) {
            format = PLAIN_FORMAT;
            put(LOG_FORMAT, format);
        }
        return format;
    }


    /**
     *  Gets the windowSize attribute of the UserDefaults object
     *
     *@return    The windowSize value
     */
    public Dimension getWindowSize() {
        return new Dimension(intValue(WINDOW_SIZE_X), intValue(WINDOW_SIZE_Y));
    }


    /**
     *  Gets the windowPos attribute of the UserDefaults object
     *
     *@return    The windowPos value
     */
    public Point getWindowPos() {
        return new Point(intValue(WINDOW_POS_X), intValue(WINDOW_POS_Y));
    }


    /**
     *  Get default value for job storage format XML or Serialized
     *
     *@return    default storage format "xml" or "serialized"
     */
    public String getJobStorageFormat() {
        String format = getProperty(JOBSTORAGEFORMAT);
        if(format == null) {
            format = BINARY_FORMAT;
            put(JOBSTORAGEFORMAT, format);
        }
        return format;
    }


    /**
     *  Gets the preemptiveFileSelectorMode attribute of the UserDefaults object
     *
     *@return    The preemptiveFileSelctorMode value
     */
    public boolean getPreemptiveFileSelectorMode() {
        return booleanValue(PREEMPTIVE_FILESELECTOR_MODE);
    }


    /**
     *  Gets the socket timeout value
     *
     *@return    The socket timeout
     */
    public int getSocketTimeout() {
        String timeout = getProperty(SOCKET_TIMEOUT);
        if(timeout == null || timeout.equals("")) {
            timeout = "60000";
            put(SOCKET_TIMEOUT, timeout);
        }
        return Integer.parseInt(timeout);
    }


    /**
     *  Gets a user attribute of the UserDefaults
     *
     *@param  identity  - a string format of "identity:xLogin:project"
     *@return           a user value
     */
    private User getUser(String identity) {
        String alias = null;
        String project = "";
        String xLogin = "";
        final String delimiter = ":";

        int endIndex = identity.length();

        int beginIndex = identity.lastIndexOf(delimiter);
        if(beginIndex > -1) {
            project = identity.substring(beginIndex + 1);
            endIndex = beginIndex;

            beginIndex = identity.lastIndexOf(delimiter, endIndex - 1);
            if(beginIndex > -1) {
                xLogin = identity.substring(beginIndex + 1, endIndex);
                endIndex = beginIndex;
            }
        }

        try {
            alias = identity.substring(0, endIndex);
            User returnedUser = null;

            X509Certificate[] x509 = ResourceManager.getKeystoreManager().getCertificateByAlias(alias);

            if(x509 != null) {
                returnedUser = new User(x509[0], null);

                if(!xLogin.equals("")) {
                    returnedUser.setXlogin(xLogin);
                }

                if(!project.equals("")) {
                    returnedUser.setProject(project);
                }
            }
            return returnedUser;
        }
        catch(Exception ex) {
            logger.log(Level.SEVERE, "Could not read certificate " + alias +
                    " from keystore.", ex);
            return null;
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public boolean writeToFile() {
        // always overwrite old ${HOME}/.unicoreXX/UserDefaults.txt
        return writeToFile(unicoreDir + File.separator + "UserDefaults.txt");
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public boolean loadFromFile() {
        // look for  ${HOME}/.unicoreXX/UserDefaults.txt
        return loadFromFile(unicoreDir + File.separator + "UserDefaults.txt");
    }


    /**
     *  Fill properties with default values
     */
    protected void genSystemDefaults() {

        // look for home directory
        String home = "." + File.separator;
        try {
            home = System.getProperty("user.home");
        }
        catch(SecurityException se) {
            UserMessages.print(res.getString("NO_PERMISSION"));
        }

        String exampleDir =
            ResourceManager.getJarDirectory()
            + File.separator + ".." + File.separator + "examples";

        put(JOBDIR, exampleDir);
        put(RESOURCEDIR, exampleDir);
        put(IMPORTDIR, home);
        put(EXPORTDIR, home);
        put(LOGDIR, home);
        put(TMPDIR, unicoreDir);
        put(USEREMAIL, "");
        put(XLOGIN, "");
        put(PROJECT, "");
        put(USITEURL, ResourceManager.getSystemDefaults().getUsiteURL());
        put(CRLURL, ResourceManager.getSystemDefaults().getCrlURL());
        put(KEYSTOREDIR, unicoreDir + File.separator + "keystore");
        put(REDIRECT, Boolean.TRUE.toString());
        put(AUTOFETCHOUTCOME, Boolean.TRUE.toString());
        put(DETAILEDJOBSTATUS, Boolean.FALSE.toString());

        put(PROXY_ENABLED, Boolean.FALSE.toString());
        put(PROXY_HOST, "");
        put(PROXY_PORT, "8080");
        put(NO_PROXY, "");

        put(PLUGINDIR, home + File.separator + "plugins");
        put(LOOKANDFEEL, "javax.swing.plaf.metal.MetalLookAndFeel");
        put(THEME, LookAndFeelManager.getStandardTheme());
        put(JOBSTORAGEFORMAT, BINARY_FORMAT);
        put(STARTOFFLINE, Boolean.FALSE.toString());
        put(SOCKET_TIMEOUT, "60000");

        put(WINDOW_SIZE_X, "1024");
        put(WINDOW_SIZE_Y, "768");
        put(WINDOW_POS_X, "0");
        put(WINDOW_POS_Y, "0");

        put(LOG_LEVEL, Level.INFO.toString());
        put(LOG_FORMAT, PLAIN_FORMAT);
        put(PREEMPTIVE_FILESELECTOR_MODE, "false");
    }


    /**
     *  Description of the Method
     *
     *@param  prop  Description of the Parameter
     *@return       Description of the Return Value
     */
    protected int defaultIntValue(String prop) {

        if(prop.equals(WINDOW_SIZE_X)) {
            return 800;
        }
        else if(prop.equals(WINDOW_SIZE_Y)) {
            return 800;
        }
        else if(prop.equals(WINDOW_POS_X)) {
            return 0;
        }
        else if(prop.equals(WINDOW_POS_Y)) {
            return 0;
        }

        return 0;
    }


    /**
     *  Description of the Method
     */
    public void update() {
        if(isProxyEnabled()) {
            ProxyManager.useProxy(getProxyHost(), getProxyPort(), getNoProxy());
        }
        else {
            ProxyManager.noProxy();
        }
    }

}

