package com.pallas.unicore.client.controls;

import java.awt.Component;
import java.awt.Font;
import java.awt.GraphicsEnvironment;

import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.UIManager;

/**
 *  CombobBox for selecting one of all available fonts. If preview is switched on every font uses
 *  itself to render the comboBox entry.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FontSelectionComboBox.java,v 1.2 2003/06/30 06:53:11 rrate Exp $
 */
public class FontSelectionComboBox extends JComboBox {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class FontRenderer extends DefaultListCellRenderer {
		/**
		 *  Gets the ListCellRendererComponent attribute of the FontRenderer object
		 *
		 *@param  list          Description of Parameter
		 *@param  value         Description of Parameter
		 *@param  index         Description of Parameter
		 *@param  isSelected    Description of Parameter
		 *@param  cellHasFocus  Description of Parameter
		 *@return               The ListCellRendererComponent value
		 */
		public Component getListCellRendererComponent(
			JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus) {

			setComponentOrientation(list.getComponentOrientation());

			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			if (value instanceof Icon) {
				setIcon((Icon) value);
				setText("");
			} else {
				setIcon(null);
				setText((value == null) ? "" : value.toString());
			}

			setEnabled(list.isEnabled());

			String selectedFont = (String) value;

			if (preview) {
				for (int i = 0; i < fonts.length; i++) {
					if (selectedFont.equals(fonts[i].getFontName())) {
						setFont(
							new Font(fonts[i].getFontName(), Font.PLAIN, 12));
						continue;
					}
				}
			} else {
				setFont(list.getFont());
			}
			setBorder(
				(cellHasFocus)
					? UIManager.getBorder("List.focusCellHighlightBorder")
					: noFocusBorder);

			return this;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  args  Description of Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("Test font selector");
		frame.getContentPane().add(new FontSelectionComboBox());
		frame.pack();
		frame.show();
	}
	private Font[] fonts =
		GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
	private boolean preview = false;

	/**
	 *  Constructor for the FontSelectionComboBox object
	 */
	public FontSelectionComboBox() {
		for (int i = 0; i < fonts.length; i++) {
			addItem(fonts[i].getFontName());
		}

		setRenderer(new FontRenderer());
	}

	/**
	 *  Constructor for the FontSelectionComboBox object
	 *
	 *@param  preview  Description of Parameter
	 */
	public FontSelectionComboBox(boolean preview) {
		this();
		this.preview = preview;
	}
}
