package com.pallas.unicore.client.controls;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: GenericFileFilter.java,v 1.4 2003/06/30 06:52:11 rrate Exp $
 */
public class GenericFileFilter extends FileFilter {

	/**
	 *  Get the extension of a file.
	 *
	 *@param  f  Description of Parameter
	 *@return    The Extension value
	 */
	public static String getExtension(File f) {
		String name = f.getName();
		int index = name.lastIndexOf('.');
		if (index < 0) {
			return "";
		}
		return name.substring(index, name.length());
	}

	private String ending, description;

	/**
	 *  Constructor for the GenericFileFilter object
	 */
	public GenericFileFilter() {
		this("");
	}

	/**
	 *  Constructor for the GenericFileFilter object
	 *
	 *@param  ending  Description of Parameter
	 */
	public GenericFileFilter(String ending) {
		this.ending = ending;
		description = "All files";
		if (!ending.equals("")) {
			description += " ending with " + ending;
		}
	}

	/**
	 *Constructor for the GenericFileFilter object
	 *
	 *@param  ending       Description of the Parameter
	 *@param  description  Description of the Parameter
	 */
	public GenericFileFilter(String ending, String description) {
		this.ending = ending;
		this.description = description;
	}

	/**
	 *  If f is a directory or ending matches file return true
	 *
	 *@param  f  Description of Parameter
	 *@return    true if file ending matches or file is directory
	 */
	public boolean accept(File f) {
		if (ending.equals("")) {
			return true;
		}
		return f.isDirectory() || ending.equals(getExtension(f));
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public boolean equals(Object object) {
		GenericFileFilter filter2 = (GenericFileFilter) object;
		if (filter2.ending.equals(this.ending)
			&& filter2.description.equals(this.description)) {
			return true;
		}
		return false;
	}

	/**
	 *  Gets the Description attribute of the GenericFileFilter object
	 *
	 *@return    The Description value
	 */
	public String getDescription() {
		return description;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public int hashCode() {
		int result = 17;
		result = 37 * result + ending.hashCode();
		result = 37 * result + description.hashCode();
		return result;
	}

	/**
	 *  Sets the Description attribute of the GenericFileFilter object
	 *
	 *@param  description  The new Description value
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		if (ending.equals("")) {
			return getDescription();
		}
		return getDescription() + " (*" + ending + ")";
	}

}
