package com.pallas.unicore.client.controls;

import java.awt.Toolkit;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 *  Document class only for int values
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: IntDocument.java,v 1.2 2003/06/30 06:52:10 rrate Exp $
 *@see        IntTextField
 */
class IntDocument extends PlainDocument {

	private int min, max;

	/**
	 *  Constructor initializes with min and max
	 *
	 *@param  min  minimal value
	 *@param  max  maximal value
	 */
	public IntDocument(int min, int max) {
		this.min = min;
		this.max = max;
	}

	/**
	 *  Overwrites the function "insertString" of class PlainDocument for int only input.
	 *
	 *@param  offset                    >=0 where String should be inserted
	 *@param  str                       String to insert
	 *@param  a                         attributes for inserted content
	 *@exception  BadLocationException  Description of Exception
	 */
	public void insertString(int offset, String str, AttributeSet a)
		throws BadLocationException {

		boolean valid = true;

		String tmp =
			getText(0, offset) + str + getText(offset, getLength() - offset);

		if (min >= 0 || (!tmp.equals("-"))) {
			try {
				int i = Integer.parseInt(tmp);
			} catch (NumberFormatException e) {
				valid = false;
			}
		}

		if (valid) {
			// if all right put the digit on screen
			super.insertString(offset, str, a);
		} else {
			// beep if wrong input
			Toolkit.getDefaultToolkit().beep();
		}
	}

	/**
	 *  Set Maximum
	 *
	 *@param  max  maximum
	 */
	public void setMaximum(int max) {
		this.max = max;
	}

	/**
	 *  Set Minimum
	 *
	 *@param  min  minimum
	 */
	public void setMinimum(int min) {
		this.min = min;
	}
}
