package com.pallas.unicore.client.controls;

import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 *  Text field only for int. A beep indicates a wrong input try.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: IntTextField.java,v 1.3 2003/06/30 06:52:09 rrate Exp $
 */
public class IntTextField extends JTextField {

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    1. August 2002
	 */
	private class FieldFocusAdapter extends FocusAdapter {
		private IntTextField field;

		/**
		 *  Constructor for the FieldFocusAdapter object
		 *
		 *@param  field  Description of the Parameter
		 */
		public FieldFocusAdapter(IntTextField field) {
			this.field = field;
		}

		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void focusLost(FocusEvent ev) {
			if (ev.getSource() == field) {
				if (field.getText().equals("")) {
					field.setText("0");
				}
			}
		}
	}

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		Box panel = new Box(BoxLayout.Y_AXIS);
		panel.add(new IntTextField(-10, 200));
		panel.add(new IntTextField("10", 0, 10000));

		JPanel subPanel = new JPanel();
		subPanel.add(panel);

		JFrame frame = new JFrame();
		frame.getContentPane().add(subPanel);
		frame.setSize(400, 400);
		frame.show();
	}

	private int min, max;

	/**
	 *  Constructor sets length of textfield according to min and max value.
	 *  This class does not perform a range check!
	 *
	 *@param  min  minimal value
	 *@param  max  maximal value
	 */
	public IntTextField(int min, int max) {
		this(null, min, max);
	}

	/**
	 *  Constructor sets length of textfield according to min and max value and
	 *  initializes it with an integer value This class does not perform a range
	 *  check!
	 *
	 *@param  start  initial integer value
	 *@param  min    minimal value
	 *@param  max    maximal value
	 */
	public IntTextField(int start, int min, int max) {

		super(
			new IntDocument(min, max),
			new Integer(start).toString(),
			Math.max(("" + min).length(), ("" + max).length()));

		this.min = min;
		this.max = max;

		this.addFocusListener(new FieldFocusAdapter(this));
	}

	/**
	 *  Constructor sets length of textfield according to min and max value and
	 *  initializes it with a String. This class does not perform a range check!
	 *
	 *@param  text  String that should contain a integer value
	 *@param  min   minimal value
	 *@param  max   maximal value
	 */
	public IntTextField(String text, int min, int max) {

		super(
			new IntDocument(min, max),
			text,
			Math.max(("" + min).length(), ("" + max).length()));

		this.min = min;
		this.max = max;

		this.addFocusListener(new FieldFocusAdapter(this));
	}

	/**
	 *  Get integer value from textfield
	 *
	 *@return    String from textfield as integer value
	 */
	public int getValue() {
		if (getText().equals("")) {
			setValue(0);
		}
		return Integer.parseInt(getText());
	}

	/**
	 *  Set maximal value and adjust textfield length
	 *
	 *@param  max  new maximal value
	 */
	public void setMaximum(int max) {
		this.max = max;

		// convert min and max to String by adding ""
		setColumns(Math.max(("" + min).length(), ("" + max).length()));
		((IntDocument) getDocument()).setMaximum(max);
	}

	/**
	 *  Set minimal value and adjust textfield length
	 *
	 *@param  min  new minimal value
	 */
	public void setMinimum(int min) {
		this.min = min;

		// convert min and max to String by adding ""
		setColumns(Math.max(("" + min).length(), ("" + max).length()));
		((IntDocument) getDocument()).setMinimum(min);
	}

	/**
	 *  Set textfield to an integer value
	 *
	 *@param  value  new integer value
	 */
	public void setValue(int value) {
		setText((new Integer(value)).toString());
	}
}
