package com.pallas.unicore.client.controls;

import java.awt.GridBagConstraints;
import java.awt.Insets;

/**
 *  Some layout utitlities
 *
 *@author     Ralf Ratering
 *@created    2. Mai 2003
 *@version    $Id: LayoutTools.java,v 1.2 2003/06/30 06:52:09 rrate Exp $
 */
public class LayoutTools {

	/**
	 *  Creates a GridBagConstraints object with all of its fields set to the
	 *  passed-in arguments.
	 *
	 *@param  x          - x specifies the cell at the left of the component's
	 *      display area, where the leftmost cell has gridx=0.
	 *@param  y          - y specifies the cell at the top of the component's
	 *      display area, where the topmost cell has gridy=0.
	 *@param  gridwidth  - gridwith specifies the number of cells in a row for
	 *      the component's display area.
	 *@param  weightx    - weigty specifies how to distribute extra horizontal
	 *      space.
	 *@param  scalable   - true, if the component resize horizontally; false, if
	 *      the component do not resize.
	 *@return            - a GridbagConstraints objects with initial values
	 */
	public static GridBagConstraints makegbc(
		int x,
		int y,
		int gridwidth,
		int weightx,
		boolean scalable) {
		GridBagConstraints gbc = new GridBagConstraints();

		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = gridwidth;
		gbc.gridheight = 1;
		gbc.weightx = weightx;

		gbc.anchor = GridBagConstraints.WEST;

		//specifies the external padding of the component, the minimum amount
		//of space between the component and the edges of its display area.
		gbc.insets = new Insets(5, 5, 5, 5);

		if (scalable) {
			gbc.fill = GridBagConstraints.HORIZONTAL;
		} else {
			gbc.fill = GridBagConstraints.NONE;
		}

		return gbc;
	}

	/**
	 *  Constructor for the LayoutTools object
	 */
	private LayoutTools() {
	}

}
