package com.pallas.unicore.client.controls;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JList;

import org.unicore.Vsite;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;

import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@author     Kirsten Foerster
 *@author     Klaus-Dieter Oertel
 *@created    17. Juli 2002
 *@version    $Id: StorageComboBox.java,v 1.2 2003/06/30 06:52:08 rrate Exp $
 *      Combo box which keeps track of an unselectable storage i.e. a storage
 *      which was available at a different site but not at the current one. An
 *      unselectable storage will be remembered as unselectableEntry and added
 *      (temporarely) to the combo box model. A listener will remove it
 *      on-the-fly from the model if the combo box is opened.
 */
public class StorageComboBox extends JComboBox {

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    9. Dezember 2002
	 */
	private class ComboBoxListener implements ItemListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}

			if (e.getStateChange() == ItemEvent.SELECTED) {
				Object item = e.getItem();
				if (unselectableEntry != null
					&& unselectableEntry.equals(getSelectedItem())) {
					entries.remove(item);
				}
				((StorageComboBox) e.getSource()).setForeground(Color.BLACK);
			}
		}
	}

	/**
	 *  ComboBoxRenderer shows NOT_AVAILABLE if no Vsite has been selected up to
	 *  now i.e. the model has only a null value.
	 *
	 *@author     kfoerst
	 *@created    9. Dezember 2002
	 */
	class ComboBoxRenderer extends DefaultListCellRenderer {
		private StorageComboBox comboBox;

		/**
		 *  Constructor for the ComboBoxRenderer object
		 *
		 *@param  comboBox  Description of the Parameter
		 */
		public ComboBoxRenderer(StorageComboBox comboBox) {
			super();
			this.comboBox = comboBox;
		}

		/**
		 *  Gets the listCellRendererComponent attribute of the ComboBoxRenderer
		 *  object
		 *
		 *@param  list          Description of the Parameter
		 *@param  value         Description of the Parameter
		 *@param  index         Description of the Parameter
		 *@param  isSelected    Description of the Parameter
		 *@param  cellHasFocus  Description of the Parameter
		 *@return               The listCellRendererComponent value
		 */
		public Component getListCellRendererComponent(
			JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus) {
			if (value == null) {
				value = NOT_AVAILABLE;
			}

			super.getListCellRendererComponent(
				list,
				value,
				index,
				isSelected,
				cellHasFocus);
			list.setForeground(Color.BLACK);
			return this;
		}
	}
	/**
	 *  Description of the Field
	 */
	public final static String NOT_AVAILABLE = "Not Available";
	private Vector entries;
	private DefaultComboBoxModel model;
	private Object unselectableEntry = null;

	/**
	 *  Constructor for the StorageComboBox object
	 */
	public StorageComboBox() {
		entries = new Vector();
		model = new DefaultComboBoxModel(entries);
		setModel(model);

		this.addItemListener(new ComboBoxListener());
		this.setRenderer(new ComboBoxRenderer(this));
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public boolean browsableItem() {
		Object selectedItem = getSelectedItem();
		if (selectedItem == null || !(selectedItem instanceof ResourceEntry)) {
			return false;
		}

		ResourceEntry entry = (ResourceEntry) selectedItem;
		if (entry.equals(unselectableEntry)
			|| entry.getResource() instanceof USpace) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  storage  Description of the Parameter
	 *@return          Description of the Return Value
	 */
	private ResourceEntry findResourceEntry(Storage storage) {
		for (int i = 0; i < entries.size(); i++) {
			if (!(entries.elementAt(i) instanceof ResourceEntry)) {
				continue;
			}
			ResourceEntry entry = (ResourceEntry) entries.elementAt(i);
			if (entry.getResource().toString().equals(storage.toString())) {
				return entry;
			}
		}
		return null;
	}

	/**
	 *  Gets the selectedItem attribute of the StorageComboBox object
	 *
	 *@return    The selectedItem value
	 */
	public Object getSelectedItem() {
		Object item = super.getSelectedItem();
		return item;
	}

	/**
	 *  Gets the selectedStorage attribute of the StorageComboBox object
	 *
	 *@return    The selectedStorage value
	 */
	public Object getSelectedStorage() {
		return getStorageAt(getSelectedIndex());
	}

	/**
	 *  Gets the storageAt attribute of the StorageComboBox object
	 *
	 *@param  i  Description of the Parameter
	 *@return    The storageAt value
	 */
	public Object getStorageAt(int i) {
		Object item = this.getItemAt(i);
		if (item instanceof ResourceEntry) {
			ResourceEntry entry = (ResourceEntry) item;
			return entry.getResource();
		}
		return item;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  storage  Description of the Parameter
	 */
	public void removeStorage(Storage storage) {
		removeItem(findResourceEntry(storage));
	}

	// replace old unselectable (might be null i.e. not existing) item by new one
	/**
	 *  Description of the Method
	 *
	 *@param  item  Description of the Parameter
	 */
	private void replaceUnselectableEntry(ResourceEntry item) {
		unselectableEntry = item;
	}

	//
	// Selection
	//

	/**
	 *  Sets the selectedItem attribute of the StorageComboBox object
	 *
	 *@param  anObject  The new selectedItem value
	 */
	public void setSelectedItem(Object anObject) {
		super.setSelectedItem(anObject);
	}

	/**
	 *  Sets the selectedStorage attribute of the StorageComboBox object
	 *
	 *@param  storage  The new selectedStorage value
	 */
	public void setSelectedStorage(Storage storage) {
		if (storage == null) {
			return;
		}
		ResourceEntry entry = findResourceEntry(storage);
		if (entry == null) {
			// storage not found ==> add this unselectable entry to the model by
			// replacing the previous one (if this exists)
			replaceUnselectableEntry(new ResourceEntry(storage));
		} else if (!(entry.equals(unselectableEntry))) {
			// remove the unselectable entry from the model if a different one was found
			model.removeElement(unselectableEntry);
			unselectableEntry = null;
		}
		setSelectedItem(findResourceEntry(storage));
	}

	/**
	 *  Description of the Method
	 *
	 *@param  vsite  Description of the Parameter
	 */
	public void update(Vsite vsite) {
		Object selectedItem = model.getSelectedItem();

		// get site resources
		NamedResourceSet siteResources = ResourceManager.getResourceSet(vsite);

		// put site storages into storages vector
		Vector storages = new Vector();
		siteResources.addStorageResources(storages);

		// clear previous entries
		model.removeAllElements();

		// build vector with resource entries
		entries.clear();
		for (int i = 0; i < storages.size(); i++) {
			ResourceEntry entry =
				new ResourceEntry((Storage) storages.elementAt(i));
			entries.addElement(entry);
		}

		//this.updateUI();
		// try to re-select item
		if (selectedItem != null) {
			// also handles empty entries
			for (int i = 0; i < entries.size(); i++) {
				if (entries
					.elementAt(i)
					.toString()
					.equals(selectedItem.toString())) {
					model.setSelectedItem(entries.elementAt(i));
					this.setForeground(Color.BLACK);
					return;
				}
			}

			if (entries.size() == 0) {
				entries.addElement(NOT_AVAILABLE);
			}
			// could not find previous selected entry ==> add it as unselectable to the model
			replaceUnselectableEntry((ResourceEntry) selectedItem);
			model.setSelectedItem(selectedItem);
			this.setForeground(Color.RED);
		} else {
			if (entries.size() > 0) {
				// no item selected, choose first one
				setSelectedIndex(0);
				this.setForeground(Color.BLACK);
			}
		}
	}

}
