package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Ralf Ratering
 *@created    29. August 2001
 *@version    $Id: AddToKeystoreDialog.java,v 1.3 2003/06/30 06:53:31 rrate Exp $
 */

public class AddToKeystoreDialog extends JDialog {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    29. August 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == detailsButton) {
				showCertificate();
				return;
			} else if (e.getSource() == onlyForSessionButton) {
				result = ONLY_FOR_SESSION;
			} else if (e.getSource() == acceptButton) {
				result = ACCEPT;
			} else if (e.getSource() == rejectButton) {
				result = REJECT;
			}
			dispose();
		}
	}
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.dialogs");
	/**
	 *  Description of the Field
	 */
	public final static int ONLY_FOR_SESSION = 0, ACCEPT = 1, REJECT = 2;

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 *  The main program for the AddToKeystoreDialog class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		/*
		 *  try {
		 *  AddToKeystoreDialog dialog =
		 *  new AddToKeystoreDialog(new URL("http://www.gmd.de"));
		 *  dialog.show();
		 *  }
		 *  catch (Exception e) {
		 *  logger.log(Level.SEVERE, "" , e);
		 *  }
		 */
	}

	private X509Certificate certificate;

	private JButton detailsButton,
		acceptButton,
		rejectButton,
		onlyForSessionButton;
	private JFrame detailsFrame;

	private String issuer;

	private URL pluginURL;
	private int result;

	/**
	 *  Constructor for the AddToKeystoreDialog object
	 *
	 *@param  pluginURL    Description of Parameter
	 *@param  parent       Description of the Parameter
	 *@param  certificate  Description of the Parameter
	 */
	public AddToKeystoreDialog(
		Frame parent,
		URL pluginURL,
		X509Certificate certificate) {
		super(parent, res.getString("UNKNOWN_TITLE"), true);
		this.pluginURL = pluginURL;
		this.certificate = certificate;
		String all = certificate.getSubjectDN().toString();
		int in1 = all.indexOf("CN=") + 3;
		issuer = all.substring(in1, all.indexOf(',', in1));

		buildComponents();
		pack();

		setLocationRelativeTo(parent);
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		JTextArea textArea = new JTextArea();
		String message = res.getString("LOADING_PLUGIN") + pluginURL + ">\n";
		message += res.getString("SIGNED_BY") + issuer + "\n";
		message += res.getString("ADDING_SIGNER");
		textArea.setText(message);

		ButtonListener buttonListener = new ButtonListener();

		detailsButton = new JButton(res.getString("SHOW_DETAILS"));
		detailsButton.setToolTipText(res.getString("SHOW_DETAILS_EXPLICIT"));
		detailsButton.addActionListener(buttonListener);

		onlyForSessionButton = new JButton(res.getString("ACCEPT_FOR_SESSION"));
		onlyForSessionButton.setToolTipText(res.getString("ACCEPT_TIP"));
		onlyForSessionButton.addActionListener(buttonListener);

		acceptButton = new JButton(res.getString("ADD"));
		acceptButton.setToolTipText(res.getString("ADD_TIP"));
		acceptButton.addActionListener(buttonListener);

		rejectButton = new JButton(res.getString("NOT_ACCEPT"));
		rejectButton.setToolTipText(res.getString("NOT_ACCEPT_TIP"));
		rejectButton.addActionListener(buttonListener);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(detailsButton);
		buttonPanel.add(onlyForSessionButton);
		buttonPanel.add(acceptButton);
		buttonPanel.add(rejectButton);

		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(textArea, BorderLayout.CENTER);

		// window closing = pressing cancel
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			/**
			 *  Description of the Method
			 *
			 *@param  we  Description of Parameter
			 */
			public void windowClosing(WindowEvent we) {
				result = REJECT;
				dispose();
			}
		});

	}

	/**
	 *  Gets the Result attribute of the AddToKeystoreDialog object
	 *
	 *@return    The Result value
	 */
	public int getResult() {
		return result;
	}

	/**
	 *  Gets the certificateFromAlias attribute of the KeyStoreEditor object
	 */
	private void showCertificate() {
		String subject = "";

		try {
			subject = subject + certificate.getSubjectDN().toString();
			if (subject == null) {
				return;
			}
			int indexFrom = subject.indexOf("CN=");
			if (indexFrom != -1) {
				int indexTo = subject.indexOf(",", indexFrom);
				subject = subject.substring(indexFrom + 3, indexTo);
				System.err.println("*******subject: " + subject);
			}
			if (certificate != null) {
				KeyStoreDetailsDialog dialog =
					new KeyStoreDetailsDialog(
						certificate,
						(JFrame) getParent(),
						subject,
						true);
				dialog.show();
			}
		} catch (Exception ex) {
			logger.log(Level.SEVERE, "", ex);
			UserMessages.error(
				"Could not read certificate " + subject + " from keystore.",
				ex.getMessage());
		}
	}
}
