package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.client.util.ResourceCountryCodes;

/**
 *  A dialog to generate a certification request.
 *
 *@author     Kirsten Foerster
 *@created    22. Juli 2002
 *@version    $Id: CSRDialog.java,v 1.5 2003/06/30 06:53:30 rrate Exp $1.0$
 */

public class CSRDialog extends JDialog {

	/**
	 *  Creates a GridBagConstraints object with all of its fields set to the
	 *  passed-in arguments
	 *
	 *@author           rrate
	 *@created          2. Mai 2003
	 *@param  x         - the initial gridx value
	 *@param  y         - the initial gridy value
	 *@param  weightx   - the initial weightx value
	 *@param  scalable  - true, if the component resize horizontally; false, if
	 *      the component do not resize.
	 *@return           - a GridbagConstraints objects with initial values
	 */

	/**
	 *  An ActionListener for the buttons
	 *
	 *@author     kfoerst
	 *@created    24. Juli 2002
	 */
	private class ButtonListener implements ActionListener {

		/**
		 *  Invoked when the okButton or cancelButton is an action occurs
		 *
		 *@param  ev
		 */
		public void actionPerformed(ActionEvent ev) {
			if (ev.getSource() == okButton) {
				if (applyValues()) {
					dispose();
				}
			}
			//else: cancelButton
			else {
				resetValues();
				dispose();
			}
		}
	}

	/**
	 *  A FocusListener for the buttons.
	 *
	 *@author     kfoerst
	 *@created    24. Juli 2002
	 */
	private class FocusButtonListener implements FocusListener {

		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void focusGained(FocusEvent ev) {
			if (ev.getSource() == cancelButton) {
				getRootPane().setDefaultButton(cancelButton);
			}
		}

		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void focusLost(FocusEvent ev) {
			if (ev.getSource() == cancelButton) {
				getRootPane().setDefaultButton(okButton);
			}
		}
	}
	static ResourceCountryCodes countryRes =
		(ResourceCountryCodes) ResourceBundle.getBundle(
			"com.pallas.unicore.client.util.ResourceCountryCodes");
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	private JPanel buttonPanel;
	private JButton cancelButton;

	private JTextField cnField;

	private JLabel cnLabel;

	private JComboBox countryComboBox;
	private JLabel countryLabel;

	private String dnString;
	private JTextField emailField;
	private JLabel emailLabel;
	private JPanel individualPanel;
	private JTextField localField;
	private JPanel locatedPanel;

	private JButton okButton;
	private JTextField orgField;
	private JLabel orgLabel;
	private JTextField orgUnitField;
	private JTextField stateField;

	/**
	 *  Constructor for the CSRDialog2 object
	 *
	 *@param  parent
	 */
	public CSRDialog(JFrame parent, String title) {
		super(parent, title, true);

		this.getContentPane().setLayout(new BorderLayout(0, 25));

		this.buildIndividualPanel();
		this.buildLocatedPanel();
		this.buildButtonPanel();

		this.getContentPane().add(individualPanel, BorderLayout.NORTH);
		this.getContentPane().add(locatedPanel, BorderLayout.CENTER);
		this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		Border matteBorder =
			BorderFactory.createMatteBorder(
				15,
				20,
				0,
				20,
				this.getContentPane().getBackground());
		Border loweredBevelBorder = BorderFactory.createLoweredBevelBorder();
		Border raisedBevel =
			BorderFactory.createLineBorder(
				this.getContentPane().getBackground(),
				1);
		Border compound =
			BorderFactory.createCompoundBorder(raisedBevel, loweredBevelBorder);

		((JPanel) this.getContentPane()).setBorder(
			BorderFactory.createCompoundBorder(compound, matteBorder));

		this.pack();
	}

	/**
	 *  The distinguished name of an individual represents differernt fields.
	 *
	 *@return    true, if the essential fields are filled in by the user;
	 *      otherwise false
	 */
	public boolean applyValues() {

		//check if the user fill in the essential fields
		if (emailField.getText().equals("")
			|| cnField.getText().equals("")
			|| orgField.getText().equals("")
			|| ((String) countryComboBox.getSelectedItem()).equals("--")) {

			this.resetValues();
			this.setErrorColor();

			JOptionPane.showMessageDialog(
				this,
				"the (*) - fields  can not be empty",
				"ERROR",
				JOptionPane.ERROR_MESSAGE);
			//UserMessages.error("the (*) - fields  can not be empty");
			return false;
		}

		dnString =
			"EmailAddress="
				+ emailField.getText().trim()
				+ ",CN="
				+ cnField.getText().trim();

		if (!orgUnitField.getText().equals("")) {
			dnString += ",OU=" + orgUnitField.getText().trim();
		}

		dnString += ",O=" + orgField.getText().trim();

		if (!localField.getText().equals("")) {
			dnString += ",L=" + localField.getText().trim();
		}

		if (!stateField.getText().equals("")) {
			dnString += ",ST=" + stateField.getText().trim();
		}

		dnString += ",C=" + countryComboBox.getSelectedItem().toString();

		return true;
	}

	/**
	 *  This method builds the buttons of this panel
	 */
	private void buildButtonPanel() {
		buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		JPanel gridPanel = new JPanel(new GridLayout(1, 2, 7, 0));
		okButton = new JButton("OK");
		cancelButton = new JButton("Cancel");
		gridPanel.add(okButton);
		gridPanel.add(cancelButton);
		buttonPanel.add(gridPanel);

		ButtonListener buttonListener = new ButtonListener();
		okButton.addActionListener(buttonListener);
		cancelButton.addActionListener(buttonListener);

		FocusButtonListener focusListener = new FocusButtonListener();
		okButton.addFocusListener(focusListener);
		cancelButton.addFocusListener(focusListener);
	}

	/**
	 *  This method builds the first part of the components of this panel.
	 */
	private void buildIndividualPanel() {
		individualPanel = new JPanel(new GridLayout(2, 1, 0, 25));

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc;

		//fields for the common name and the email adress of the individual
		JPanel panel1 = new JPanel(gbl);

		cnLabel = new JLabel("Common Name: *");
		gbc = LayoutTools.makegbc(0, 0, 1, 0, false);
		gbc.insets = new Insets(15, 20, 7, 22);
		gbl.setConstraints(cnLabel, gbc);
		panel1.add(cnLabel);

		cnField = new JTextField("", 20);
		gbc = LayoutTools.makegbc(1, 0, 1, 100, true);
		gbc.insets = new Insets(15, 0, 7, 20);
		gbl.setConstraints(cnField, gbc);
		panel1.add(cnField);

		emailLabel = new JLabel("Email: *");
		gbc = LayoutTools.makegbc(0, 1, 1, 0, false);
		gbc.insets = new Insets(0, 20, 20, 22);
		gbl.setConstraints(emailLabel, gbc);
		panel1.add(emailLabel);

		emailField = new JTextField("", 20);
		gbc = LayoutTools.makegbc(1, 1, 1, 100, true);
		gbc.insets = new Insets(0, 0, 20, 20);
		gbl.setConstraints(emailField, gbc);
		panel1.add(emailField);

		//fields for the unit/organization the individual is associated with
		JPanel panel2 = new JPanel(gbl);

		orgLabel = new JLabel("Organization: *");
		gbc = LayoutTools.makegbc(0, 0, 1, 0, false);
		gbc.insets = new Insets(15, 20, 7, 10);
		gbl.setConstraints(orgLabel, gbc);
		panel2.add(orgLabel);

		orgField = new JTextField("", 20);
		gbc = LayoutTools.makegbc(1, 0, 1, 100, true);
		gbc.insets = new Insets(15, 0, 7, 20);
		gbl.setConstraints(orgField, gbc);
		panel2.add(orgField);

		JLabel orgUnitLabel = new JLabel("Organizational Unit:");
		gbc = LayoutTools.makegbc(0, 1, 1, 0, false);
		gbc.insets = new Insets(0, 20, 20, 10);

		gbl.setConstraints(orgUnitLabel, gbc);
		panel2.add(orgUnitLabel);

		orgUnitField = new JTextField("", 20);
		gbc = LayoutTools.makegbc(1, 1, 1, 100, true);
		gbc.insets = new Insets(0, 0, 20, 20);
		gbl.setConstraints(orgUnitField, gbc);
		panel2.add(orgUnitField);

		Border loweredBevel = BorderFactory.createLoweredBevelBorder();
		Border etched = BorderFactory.createEtchedBorder();
		TitledBorder title1 =
			BorderFactory.createTitledBorder(etched, "individual");
		TitledBorder title2 =
			BorderFactory.createTitledBorder(etched, "is associated with");

		title1.setTitleFont(new Font("Dialog", Font.ITALIC, 12));
		title2.setTitleFont(new Font("Dialog", Font.ITALIC, 12));
		panel1.setBorder(title1);
		panel2.setBorder(title2);

		individualPanel.add(panel1);
		individualPanel.add(panel2);
	}

	/**
	 *  This method builds the second part of the components of this panel.
	 */
	private void buildLocatedPanel() {
		GridBagLayout gbl = new GridBagLayout();
		locatedPanel = new JPanel(gbl);
		GridBagConstraints gbc;

		JLabel localLabel = new JLabel("Locality:");
		gbc = LayoutTools.makegbc(0, 0, 1, 0, false);
		gbc.insets = new Insets(15, 20, 7, 73);
		gbl.setConstraints(localLabel, gbc);
		locatedPanel.add(localLabel, gbc);

		localField = new JTextField("", 20);
		gbc = LayoutTools.makegbc(1, 0, 1, 100, true);
		gbc.insets = new Insets(15, 0, 7, 20);
		gbl.setConstraints(localField, gbc);
		locatedPanel.add(localField, gbc);

		JLabel stateLabel = new JLabel("State:");
		gbc = LayoutTools.makegbc(0, 1, 1, 0, false);
		gbc.insets = new Insets(0, 20, 7, 73);
		gbl.setConstraints(stateLabel, gbc);
		locatedPanel.add(stateLabel, gbc);

		stateField = new JTextField("", 20);
		gbc = LayoutTools.makegbc(1, 1, 1, 100, true);
		gbc.insets = new Insets(0, 0, 7, 20);
		gbl.setConstraints(stateField, gbc);
		locatedPanel.add(stateField, gbc);

		countryLabel = new JLabel("country(shortcut) *");
		gbc = LayoutTools.makegbc(0, 2, 1, 0, false);
		gbc.insets = new Insets(0, 20, 20, 5);
		gbl.setConstraints(countryLabel, gbc);
		locatedPanel.add(countryLabel, gbc);

		Vector items = new Vector();
		items.addElement("--");

		for (Enumeration el = countryRes.getCountryCodes();
			el.hasMoreElements();
			) {
			items.addElement((String) el.nextElement());
		}

		countryComboBox = new JComboBox(items);

		countryComboBox.setEditable(false);
		countryComboBox.setSize(5, 40);
		gbc = LayoutTools.makegbc(1, 2, 1, 0, false);
		gbc.insets = new Insets(0, 0, 20, 20);
		gbl.setConstraints(countryComboBox, gbc);
		locatedPanel.add(countryComboBox, gbc);

		Border etched = BorderFactory.createEtchedBorder();

		TitledBorder title =
			BorderFactory.createTitledBorder(etched, "is located");
		title.setTitleFont(
			new Font(title.getTitleFont().getName(), Font.ITALIC, 12));

		locatedPanel.setBorder(title);
	}

	/**
	 *  Gets the dNString attribute of the CSRDialog object which represents a
	 *  principal
	 *
	 *@return    The dNString value
	 */
	public String getDNString() {
		return dnString;
	}

	/**
	 *  the value of the dnString variable will be set to null
	 */
	public void resetValues() {
		dnString = null;
	}

	/**
	 *  Sets the errorColor attribute for the essential fields which don't fill
	 *  in by the user
	 */
	private void setErrorColor() {
		if (cnField.getText().equals("")) {
			cnLabel.setForeground(Color.RED);
		} else {
			cnLabel.setForeground(Color.BLACK);
		}

		if (emailField.getText().equals("")) {
			emailLabel.setForeground(Color.RED);
		} else {
			emailLabel.setForeground(Color.BLACK);
		}

		if (orgField.getText().equals("")) {
			orgLabel.setForeground(Color.RED);
		} else {
			orgLabel.setForeground(Color.BLACK);
		}

		if (((String) countryComboBox.getSelectedItem()).equals("--")) {
			countryLabel.setForeground(Color.RED);
		} else {
			countryLabel.setForeground(Color.BLACK);
		}
	}

	/**
	 *  Displays this dialog.
	 */
	public void show() {
		getRootPane().setDefaultButton(okButton);
		//this.pack();
		setLocationRelativeTo(getParent());
		super.show();
	}
}
