package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.security.Principal;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.Border;

import com.pallas.unicore.client.util.DistinguishedNameFields;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.CertificateUtility;

/**
 *  This dialog gives information about the place where the user can get the ca
 *  certifacate which he need for a plugin.
 *
 *@author     Kirsten Foerster
 *@created    13. Januar 2003
 *@version    $Id: CertificateGeneralDialog.java,v 1.2 2003/06/30 06:53:31 rrate Exp $
 */

public class CertificateGeneralDialog extends JDialog {

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    14. Januar 2003
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent ev) {
			if (ev.getSource() == okButton) {
				dispose();
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    13. Januar 2003
	 */
	private class DNTextArea extends JTextArea {

		private String text = "";

		/**
		 *  Constructor for the DNTextArea object
		 */
		public DNTextArea() {
			this.setLineWrap(true);
			this.setWrapStyleWord(true);
		}

		/**
		 *  This method is essential for setting this table not to be editable.
		 *
		 *@return    - false
		 */
		public boolean isEditable() {
			return false;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    13. Januar 2003
	 */
	private class LinePanel extends JPanel {
		/**
		 *  Description of the Method
		 *
		 *@param  g
		 */
		public void paint(Graphics g) {
			g.setColor(Color.BLACK);
			g.drawLine(
				0,
				this.getHeight() / 2,
				this.getWidth(),
				this.getHeight() / 2);
			g.drawLine(
				0,
				this.getHeight() / 2 + 1,
				this.getWidth(),
				this.getHeight() / 2 + 1);
		}
	}
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.dialogs");
	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	private X509Certificate[] chain = null;
	private DistinguishedNameFields issuer;
	private JButton okButton;
	private String plugin;
	private Principal principal = null;

	/**
	 *  Constructor for the CertificateGeneralDialog object
	 *
	 *@param  principal Description of the Parameter
	 *@param  title     Description of the Parameter
	 */
	public CertificateGeneralDialog(Principal principal, String title) {
		super((JFrame) ResourceManager.getCurrentInstance(), title, true);
		this.principal = principal;
		this.issuer = new DistinguishedNameFields(this.principal);
		int index = title.lastIndexOf("/") + 1;
		if (index != -1) {
			this.plugin = title.substring(index, title.length());
		} else {
			this.plugin = title;
		}
		String end = ".jar";
		if (this.plugin.endsWith(end)) {
			this.plugin =
				this.plugin.substring(0, this.plugin.length() - end.length());
		}
	}

	/**
	 *  Constructor for the CertificateGeneralDialog object
	 *
	 *@param  x509   Description of the Parameter
	 *@param  title  Description of the Parameter
	 */
	public CertificateGeneralDialog(X509Certificate x509, String title) {
		this(x509.getSubjectDN(), title);
	}

	/**
	 *  Constructor for the CertificateGeneralDialog object
	 *
	 *@param  x509   Description of the Parameter
	 *@param  title  Description of the Parameter
	 */
	public CertificateGeneralDialog(X509Certificate[] chain, String title) {
		this(chain[0].getIssuerDN(), title);
		this.chain = chain;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	private JPanel buildComponents() {
		Font font1 = new Font("Dialog", Font.BOLD, 12);
		Font font2 = new Font("Dialog", Font.PLAIN, 12);

		JPanel textPanel = new JPanel();
		textPanel.setLayout(new BorderLayout(10, 10));

		JPanel northPanel = new JPanel(new GridLayout(5, 1));
		JLabel label1 = new JLabel(plugin + ":", JLabel.CENTER);
		label1.setFont(new Font(label1.getFont().getName(), Font.BOLD, 14));
		JLabel label2 =
			new JLabel(res.getString("NO_PLUGIN_CERT"), JLabel.LEFT);
		label2.setFont(font2);
		JLabel label3 =
			new JLabel(res.getString("OPEN_KEYSTORE_EDITOR"), JLabel.LEFT);
		label3.setFont(font2);
		northPanel.setBorder(
			BorderFactory.createMatteBorder(20, 30, 0, 30, Color.WHITE));

		northPanel.add(label1);
		northPanel.add(label2);
		northPanel.add(label3);
		northPanel.add(new JLabel(""));
		northPanel.add(new LinePanel());

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel centerPanel = new JPanel(gbl);

		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 100;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;

		gbc.gridx = 0;
		if (this.chain != null && chain.length != 0) {
			JLabel arrow =
				new JLabel(
					ResourceManager.getIcon(ResourceManager.ARROW),
					JLabel.LEFT);
			String issuerInfos = "";
			for (int i = 0; i < chain.length; i++) {
				X509Certificate x509 = chain[i];
				if (!CertificateUtility.isSelfSigned(x509)) {
					if (i == 0) {
						issuerInfos = this.getDNForGeneral(issuer);
					} else {
						gbc.gridy = 2 * i - 1;
						gbc.insets = new Insets(0, 120, 15, 0);
						centerPanel.add(arrow, gbc);
						issuerInfos =
							this.getDNForGeneral(
								new DistinguishedNameFields(
									x509.getIssuerDN()));
						gbc.insets = new Insets(0, 0, 0, 0);
					}
					gbc.gridy = 2 * i;

					DNTextArea dNArea1 = new DNTextArea();
					dNArea1.setFont(font1);
					dNArea1.setText("issued by:  ");

					DNTextArea dNArea2 = new DNTextArea();
					dNArea2.setText(issuerInfos);
					dNArea2.setFont(font2);
					dNArea2.setSize(300, 100);
					dNArea2.setRows(dNArea2.getLineCount());

					JPanel issuerPanel = new JPanel();
					issuerPanel.setLayout(
						new BoxLayout(issuerPanel, BoxLayout.X_AXIS));
					issuerPanel.add(dNArea1);
					issuerPanel.add(dNArea2);
					centerPanel.add(issuerPanel, gbc);
				}
			}
		}

		centerPanel.setBackground(Color.WHITE);

		centerPanel.setBorder(
			BorderFactory.createMatteBorder(10, 30, 10, 30, Color.WHITE));
		JScrollPane scrollPane =
			new JScrollPane(
				centerPanel,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		okButton = new JButton("OK");
		okButton.addActionListener(new ButtonListener());
		buttonPanel.add(okButton);

		northPanel.setBackground(Color.WHITE);
		centerPanel.setBackground(Color.WHITE);

		textPanel.add(northPanel, BorderLayout.NORTH);
		textPanel.add(scrollPane, BorderLayout.CENTER);

		Color color = textPanel.getBackground();
		textPanel.setBackground(Color.WHITE);
		Border matteBorder =
			BorderFactory.createMatteBorder(15, 15, 8, 15, color);
		Border loweredBevelBorder = BorderFactory.createLoweredBevelBorder();
		Border raisedBevel = BorderFactory.createLineBorder(color, 1);
		Border compound =
			BorderFactory.createCompoundBorder(loweredBevelBorder, raisedBevel);
		textPanel.setBorder(
			BorderFactory.createCompoundBorder(matteBorder, compound));

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(textPanel, BorderLayout.CENTER);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		panel.setBorder(BorderFactory.createMatteBorder(7, 7, 7, 7, color));

		return panel;
	}

	/**
	 *  Gets the dNForGeneral attribute of the CertificateGeneralDialog object
	 *
	 *@param  dN  Description of the Parameter
	 *@return     The dNForGeneral value
	 */
	private String getDNForGeneral(DistinguishedNameFields dN) {
		String text = dN.getCommonName(false) + "\n";

		String partialText = dN.getOrg(false);
		if (!partialText.equals("")) {
			text += partialText;
		}

		partialText = dN.getOrgUnit(false);
		if (!partialText.equals("")) {
			text += ", ";
		}
		text += partialText + "\n";

		partialText = dN.getCountry(false);
		if (!partialText.equals("")) {
			text += partialText;
		}

		partialText = dN.getLocation(false);
		if (!partialText.equals("")) {
			text += "-";
		}
		text += partialText + "\n";

		partialText = dN.getEmail(false);
		if (!partialText.equals("")) {
			text += partialText;
		}

		return text;
	}

	public void show() {
		JPanel panel = this.buildComponents();
		this.getContentPane().add(panel);

		this.getRootPane().setDefaultButton(okButton);
		//this.setSize(new Dimension(450, 340));
		this.pack();
		this.setResizable(true);
		this.setLocationRelativeTo(getParent());

		super.show();
	}
}
