package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

/**
 *  Dialog to login with alias and password.
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: ChangeAliasDialog.java,v 1.1 2003/06/27 12:26:31 thomas Exp $
 */
public class ChangeAliasDialog extends GenericDialog {
	static ResourceBundle res = ResourceBundle.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
	private JTextField oldAliasField = new JTextField(15);
	private String alias;


	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title   dialog title
	 *@param  parent  dialog parent - important for modal dialogs
	 */
	public ChangeAliasDialog(JFrame parent, String title, String oldAlias) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);
		alias = oldAlias;
		buildComponents();
		pack();
	}


	/**
	 *  Get the entered user name
	 *
	 *@return    name as string
	 */
	public String getAlias() {
		return alias;
	}



	/**
	 *  Put focus into password field and make okButton default
	 */
	public void show() {
		oldAliasField.requestFocus();
		super.show();
	}


	/**
	 *  Sets password to null
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		return true;
	}


	/**
	 *  Takes current textfield entry as password
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean applyValues() {
		alias = oldAliasField.getText();
		return true;
	}


	/**
	 *  Add a password field to the parent dialog
	 */
	private void buildComponents() {
		JLabel aliasLabel = new JLabel(res.getString("CHANGE_ALIAS"));

		JPanel aliasPanel = new JPanel();
		aliasPanel.add(aliasLabel);
		aliasPanel.add(oldAliasField);
		oldAliasField.setText(alias);

		JPanel subPanel = new JPanel(new GridLayout(1, 1, 10, 10));
		subPanel.add(aliasPanel);

        // add some spacing around
        subPanel.setBorder(new EmptyBorder(5, 20, 5, 20));

		getContentPane().add(subPanel, BorderLayout.CENTER);
	}


	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		ChangeAliasDialog dialog = new ChangeAliasDialog(frame, "TEST: PasswordDialog","test");
		dialog.show();
	}
}
