package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;

import com.pallas.unicore.utility.UserMessages;

/**
 *  Dialog to enter and verify a new password fo the first time
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: DefinePasswordDialog.java,v 1.2 2003/06/30 06:53:29 rrate Exp $
 */
public class DefinePasswordDialog extends GenericDialog {
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		DefinePasswordDialog dialog =
			new DefinePasswordDialog(frame, "TEST: DefinePasswordDialog");
		dialog.show();
	}

	private char[] newPassword;
	private JPasswordField newPasswordField = new JPasswordField(15),
		verifyPasswordField = new JPasswordField(15);

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title   dialog title
	 *@param  parent  dialog parent - important for modal dialogs
	 */
	public DefinePasswordDialog(JFrame parent, String title) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);

		buildComponents();
		pack();
	}

	/**
	 *  Takes current textfield entry as new password and compares it to
	 *  verified password
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean applyValues() {
		// test new password
		newPassword = newPasswordField.getPassword();
		String verifyNewString = new String(verifyPasswordField.getPassword());
		if (!(new String(newPassword)).equals(verifyNewString)) {
			setStatusMessage(res.getString("VERIFICATION_FAILED"));
			return false;
		}
		if (!ChangePasswordDialog.checkEnforcement(newPassword)) {
			UserMessages.warning(res.getString("WEAK_PASSWORD"));
		}
		// tests passed return ok
		return true;
	}

	/**
	 *  Add password fields to the parent dialog
	 */
	private void buildComponents() {
		JLabel newLabel = new JLabel(res.getString("NEW_PASSWORD"));
		JLabel verifyLabel = new JLabel(res.getString("VERIFY_PASSWORD"));

		JPanel labelPanel = new JPanel(new GridLayout(2, 1, 20, 20));
		labelPanel.add(newLabel);
		labelPanel.add(verifyLabel);

		JPanel textPanel = new JPanel(new GridLayout(2, 1, 20, 20));
		textPanel.add(newPasswordField);
		textPanel.add(verifyPasswordField);

		JPanel centerPanel = new JPanel();
		centerPanel.add(labelPanel);
		centerPanel.add(textPanel);
		centerPanel.setBorder(new EmptyBorder(20, 5, 10, 5));

		getContentPane().add(centerPanel, BorderLayout.CENTER);
	}

	/**
	 *  Sets new password to null
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		newPassword = null;
		return true;
	}

	/**
	 *  Get the result from the dialog
	 *
	 *@return    null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getPassword() {
		return newPassword;
	}

	/**
	 *  Put focus into old password field and make okButton default
	 */
	public void show() {
		newPasswordField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}
}
