package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;

import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.util.DistinguishedNameFields;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.Hex;

/**
 *  This dialog displays the details of a certification.
 *
 *@author     Kirsten Foerster
 *@created    10. Juli 2002
 *@version    $1.0$
 */
public class KeyStoreDetailsDialog extends JDialog {

	/**
	 *  An ActionListener for the buttons.
	 *
	 *@author     kfoerst
	 *@created    11. Juli 2002
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent ev) {
			if (ev.getSource() == okButton) {
				dispose();
			} else if (ev.getSource() == saveButton) {
				saveLog();
			}
		}

	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    15. August 2002
	 */
	private class CellRenderer implements TableCellRenderer {

		/**
		 *  Gets the tableCellRendererComponent attribute of the CellRenderer
		 *  object
		 *
		 *@param  table       Description of the Parameter
		 *@param  value       Description of the Parameter
		 *@param  isSelected  Description of the Parameter
		 *@param  hasFocus    Description of the Parameter
		 *@param  row         Description of the Parameter
		 *@param  column      Description of the Parameter
		 *@return             The tableCellRendererComponent value
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {

			JLabel label = (JLabel) value;
			((JLabel) value).setFont(
				new Font(
					label.getFont().getName(),
					Font.PLAIN,
					label.getFont().getSize()));

			if (isSelected) {
				((JLabel) value).setBackground(table.getSelectionBackground());
			} else {
				((JLabel) value).setBackground(Color.WHITE);
			}

			((JLabel) value).setOpaque(true);

			return (JLabel) value;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    24. Juli 2002
	 */
	private class DetailsScrollPane extends JScrollPane {

		/**
		 *  Constructor for the DetailsScrollPane object
		 *
		 *@param  detailsTable
		 */
		public DetailsScrollPane(DetailsTable detailsTable) {
			super(detailsTable);

			Border matteBorder =
				BorderFactory.createMatteBorder(
					15,
					15,
					15,
					15,
					Color.LIGHT_GRAY);
			Border loweredBevelBorder =
				BorderFactory.createLoweredBevelBorder();
			Border raisedBevel =
				BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1);
			Border compound =
				BorderFactory.createCompoundBorder(
					loweredBevelBorder,
					raisedBevel);
			this.setBorder(
				BorderFactory.createCompoundBorder(matteBorder, compound));
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    12. Juli 2002
	 */
	private class DetailsTable extends JTable {
		/**
		 *  Constructor for the DetailsTable object
		 *
		 *@param  data       - - the data of the table
		 *@param  coleheads  - the coleheads of the columns
		 */
		public DetailsTable(Object[][] data, String[] coleheads) {
			super(data, coleheads);

			this.setShowGrid(false);
			this.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
			this.setColumnSelectionAllowed(false);
			this.setRowSelectionAllowed(true);
			this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			this.getColumnModel().getColumn(0).setCellRenderer(
				new CellRenderer());
			this.getSelectionModel().addListSelectionListener(
				new ListListener());
		}

		/**
		 *  This method is essential for setting this table not to be editable.
		 *
		 *@param  row     - a row number of the table
		 *@param  column  - a column number of the table
		 *@return         - false
		 */
		public boolean isCellEditable(int row, int column) {
			return false;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    12. Juli 2002
	 */
	private class DNTextArea extends JTextArea {

		private String text = "";

		/**
		 *  Constructor for the DetailsTextArea object
		 */
		public DNTextArea() {
			this.setLineWrap(true);
			this.setWrapStyleWord(true);
		}

		/**
		 *  This method is essential for setting this table not to be editable.
		 *
		 *@return    - false
		 */
		public boolean isEditable() {
			return false;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    24. Juli 2002
	 */
	private class LinePanel extends JPanel {
		/**
		 *  Description of the Method
		 *
		 *@param  g
		 */
		public void paint(Graphics g) {
			g.setColor(Color.BLACK);
			g.drawLine(
				15,
				this.getHeight() / 2,
				this.getWidth() - 15,
				this.getHeight() / 2);
			g.drawLine(
				15,
				this.getHeight() / 2 + 1,
				this.getWidth() - 15,
				this.getHeight() / 2 + 1);
		}
	}

	/**
	 *  Display detailed description of the selected part of the information
	 *  about the certification.
	 *
	 *@author     kfoerst
	 *@created    11. Juli 2002
	 */
	private class ListListener implements ListSelectionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e
		 */
		public void valueChanged(ListSelectionEvent e) {
			if (e.getValueIsAdjusting()) {
				return;
			}

			int row = detailsTable.getSelectedRow();
			int column = detailsTable.getSelectedColumn();

			Object partKey = detailsTable.getValueAt(row, column);
			String key;
			if (column == 0) {
				key =
					((JLabel) partKey).getText()
						+ (String) detailsTable.getValueAt(row, 1);
			} else {
				key =
					((JLabel) detailsTable.getValueAt(row, 0)).getText()
						+ (String) partKey;
			}

			String text = "" + (String) hashTable.get(key);

			if (!text.equals("")) {
				area.setText(text);
			}
		}
	}

	/**
	 *  A ItemListener for the comboBox.
	 *
	 *@author     kfoerst
	 *@created    24. Juli 2002
	 */
	private class SelectionListener implements ItemListener {
		/**
		 *  If the user selects a item, the table displays all or a part of the
		 *  information of the certification.
		 *
		 *@param  e
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}

			fieldElements.clear();
			valueElements.clear();
			hashTable.clear();
			String item = (String) comboBox.getSelectedItem();
			if (item.equals("<all>")) {
				buildAllTableElements();
			}
			if (item.equals("version-1-fields")) {
				buildVersion1FieldsTableElements();
			}
			if (item.equals("extensions")) {
				buildExtensionTableElements();
			}
			if (item.equals("critical extensions")) {
				buildCritExtensionTableElements();
			}

			detailsPanel.remove(scrollPane);
			buildTable();
			scrollPane = new DetailsScrollPane(detailsTable);
			detailsPanel.add(scrollPane, BorderLayout.CENTER);
			detailsPanel.validate();
		}
	}
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.dialogs");
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");
	private String alias;
	private DNTextArea area;

	private JPanel buttonPanel;
	private Vector coleheads;

	private JComboBox comboBox;

	private Vector data;
	private JPanel detailsPanel;
	private DetailsTable detailsTable;

	private Vector fieldElements = new Vector();
	private JPanel generalPanel;
	private Hashtable hashTable = new Hashtable();
	private DistinguishedNameFields issuer;

	private String log;
	private JButton okButton;
	private JButton saveButton;
	private DetailsScrollPane scrollPane;
	private DistinguishedNameFields subject;

	private JTabbedPane tabbedPane;
	private Vector valueElements = new Vector();
	private X509Certificate x509;

	/**
	 *  Constructor for the KeyStoreDetailsDialog object
	 *
	 *@param  x509
	 *@param  parentFrame  Description of the Parameter
	 *@param  title        Description of the Parameter
	 *@param  modal        Description of the Parameter
	 */
	public KeyStoreDetailsDialog(
		X509Certificate x509,
		Frame parentFrame,
		String alias,
		boolean modal) {
		super(parentFrame, "certificate: " + alias, modal);
		this.x509 = x509;
		this.log = x509.toString();
		this.alias = alias;
		this.issuer = new DistinguishedNameFields(this.x509.getIssuerDN());
		this.subject = new DistinguishedNameFields(this.x509.getSubjectDN());

		this.buildPanel();
		this.setSize(new Dimension(400, 503));
		this.setResizable(false);
		this.setLocationRelativeTo(parentFrame);
		this.setBackground(Color.LIGHT_GRAY);
	}

	/**
	 *  This method builds the complete fields for the table of the details
	 *  register.
	 */
	private void buildAllTableElements() {
		this.buildVersion1FieldsTableElements();
		this.buildExtensionTableElements();
	}

	/**
	 *  This method builds the panel for the buttons.
	 */
	private void buildButtonPanel() {
		buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		ButtonListener buttonListener = new ButtonListener();

		saveButton = new JButton("SAVE");

		saveButton.addActionListener(buttonListener);
		saveButton.setBackground(Color.LIGHT_GRAY);
		saveButton.setToolTipText(res.getString("SAVE_CERT"));

		okButton = new JButton("OK");
		okButton.setBackground(Color.LIGHT_GRAY);

		okButton.addActionListener(buttonListener);

		JPanel gridPanel = new JPanel(new GridLayout(1, 2, 7, 0));
		gridPanel.add(saveButton);
		gridPanel.add(okButton);

		buttonPanel.add(gridPanel);
		buttonPanel.setBackground(Color.LIGHT_GRAY);
	}

	/**
	 *  This method builds only the critical extensions fields for the table of
	 *  the details register.
	 */
	private void buildCritExtensionTableElements() {
		String text;

		JLabel label =
			new JLabel(
				"critical extension",
				ResourceManager.getIcon(ResourceManager.X509CRITEXTENSION),
				JLabel.LEFT);
		Set critSet = x509.getCriticalExtensionOIDs();

		if (critSet != null && !critSet.isEmpty()) {
			for (Iterator i = critSet.iterator(); i.hasNext();) {
				fieldElements.addElement(label);
				text = (String) i.next();
				valueElements.addElement(text);
				hashTable.put(label.getText() + text, text);
			}
		}
	}

	/**
	 *  This method builds the second register of the JTabbedPane object.
	 */
	private void buildDetailsPanel() {
		detailsPanel = new JPanel(new BorderLayout());

		JPanel northPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		northPanel.setBackground(Color.LIGHT_GRAY);
		JLabel label = new JLabel("list:  ");
		Vector items = new Vector();
		items.addElement("<all>");
		items.addElement("version-1-fields");
		items.addElement("extensions");
		items.addElement("critical extensions");
		comboBox = new JComboBox(items);
		comboBox.setEditable(false);
		comboBox.setBackground(Color.WHITE);
		comboBox.addItemListener(new SelectionListener());
		comboBox.setSize(5, 40);
		northPanel.add(label);
		northPanel.add(comboBox);

		this.buildAllTableElements();
		this.buildTable();

		scrollPane = new DetailsScrollPane(detailsTable);

		area = new DNTextArea();
		area.setRows(11);

		Border matteBorder =
			BorderFactory.createMatteBorder(0, 15, 15, 15, Color.LIGHT_GRAY);
		Border loweredBevelBorder = BorderFactory.createLoweredBevelBorder();
		Border raisedBevel =
			BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1);
		Border compound =
			BorderFactory.createCompoundBorder(loweredBevelBorder, raisedBevel);

		area.setBorder(
			BorderFactory.createCompoundBorder(matteBorder, compound));

		detailsPanel.add(northPanel, BorderLayout.NORTH);
		detailsPanel.add(scrollPane, BorderLayout.CENTER);
		detailsPanel.add(area, BorderLayout.SOUTH);
		detailsPanel.setBorder(
			BorderFactory.createMatteBorder(10, 0, 0, 0, Color.LIGHT_GRAY));
	}

	/**
	 *  This method builds all extensions fields for the table of the details
	 *  register.
	 */
	private void buildExtensionTableElements() {
		String text;

		JLabel label =
			new JLabel(
				"non critical extension",
				ResourceManager.getIcon(ResourceManager.X509NONCRITEXTENSION),
				JLabel.LEFT);
		Set nonCritSet = x509.getNonCriticalExtensionOIDs();

		if (nonCritSet != null && !nonCritSet.isEmpty()) {
			for (Iterator i = nonCritSet.iterator(); i.hasNext();) {
				fieldElements.addElement(label);
				text = (String) i.next();
				valueElements.addElement(text);
				hashTable.put(label.getText() + text, text);
			}
		}
		this.buildCritExtensionTableElements();
	}

	/**
	 *  This method builds the first register of the JTabbedPane object.
	 */
	private void buildGeneralPanel() {
		generalPanel = new JPanel(new BorderLayout(10, 10));

		JPanel northPanel = new JPanel(new GridLayout(4, 1));
		JLabel label1 =
			new JLabel(
				"Information about this certificate   ",
				null,
				JLabel.CENTER);
		label1.setFont(new Font(label1.getFont().getName(), Font.BOLD, 14));

		northPanel.add(new JLabel(""));
		northPanel.add(label1);
		northPanel.add(new JLabel(""));
		northPanel.add(new LinePanel());

		Font font1 = new Font("Dialog", Font.BOLD, 12);
		Font font2 = new Font("Dialog", Font.PLAIN, 12);

		String subjectInfos = this.getDNForGeneral(subject);
		DNTextArea dNArea2 = new DNTextArea();
		dNArea2.setFont(font2);
		dNArea2.setText(subjectInfos);
		dNArea2.setRows(dNArea2.getLineCount());
		dNArea2.setSize(200, 100);

		DNTextArea dNArea1 = new DNTextArea();
		dNArea1.setFont(font1);
		dNArea1.setText("     issued for: ");

		String issuerInfos = this.getDNForGeneral(issuer);
		DNTextArea dNArea4 = new DNTextArea();
		dNArea4.setFont(font2);
		dNArea4.setText(issuerInfos);
		dNArea4.setFont(font2);
		dNArea4.setSize(200, 100);

		DNTextArea dNArea3 = new DNTextArea();
		dNArea3.setFont(font1);
		dNArea3.setText("     issued by:  ");
		JLabel label2 = new JLabel("    valid from");
		label2.setFont(font1);

		String date = x509.getNotBefore().toString();
		int indexFrom = date.indexOf(" ");
		int indexTo = date.indexOf(" ", indexFrom + 1);
		indexTo = date.indexOf(" ", indexTo + 1);
		String partialDate = date.substring(indexFrom + 1, indexTo);
		indexFrom = date.indexOf(" ", indexTo + 2);
		indexFrom = date.indexOf(" ", indexFrom + 1);
		partialDate += date.substring(indexFrom, date.length());
		JLabel label3 = new JLabel(partialDate);
		label3.setFont(font2);

		JLabel label4 = new JLabel("valid to");
		label4.setFont(font1);

		date = x509.getNotAfter().toString();
		indexFrom = date.indexOf(" ");
		indexTo = date.indexOf(" ", indexFrom + 1);
		indexTo = date.indexOf(" ", indexTo + 1);
		partialDate = date.substring(indexFrom + 1, indexTo);
		indexFrom = date.indexOf(" ", indexTo + 2);
		indexFrom = date.indexOf(" ", indexFrom + 1);
		partialDate += date.substring(indexFrom, date.length());
		JLabel label5 = new JLabel(partialDate);
		label5.setFont(font2);

		JPanel centerPanel = new JPanel(new GridLayout(2, 1));

		JPanel box = new JPanel();
		box.setLayout(new BoxLayout(box, BoxLayout.X_AXIS));
		box.setBackground(Color.WHITE);
		box.add(dNArea1);
		box.add(dNArea2);
		centerPanel.add(box);
		box = new JPanel();
		box.setLayout(new BoxLayout(box, BoxLayout.X_AXIS));
		box.setBackground(Color.WHITE);
		box.add(dNArea3);
		box.add(dNArea4);
		centerPanel.add(box);
		JPanel southPanel = new JPanel(new GridLayout(2, 1));
		JPanel flow = new JPanel(new FlowLayout(FlowLayout.LEFT));
		flow.setBackground(Color.WHITE);
		flow.add(label2);
		flow.add(label3);
		flow.add(label4);
		flow.add(label5);
		southPanel.add(flow);

		JLabel label6 = new JLabel();
		if (alias != null) {
			try {
				if (ResourceManager.getKeystoreManager().isKeyEntry(alias)) {
					label6.setText(
						"You own a private key for this certificate.");
					label6.setIcon(
						ResourceManager.getIcon(ResourceManager.KEY));
				} else {
					label6.setText(
						"You don't have a private key for this certificate.");
					label6.setIcon(
						ResourceManager.getIcon(ResourceManager.NOKEY));
				}
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Open KeyStoreManager failed", e);
			}
			label6.setFont(font2);
			flow = new JPanel(new FlowLayout(FlowLayout.LEFT));
			flow.setBackground(Color.WHITE);
			flow.add(new JLabel("   "));
			flow.add(label6);
			southPanel.add(flow);
		}

		northPanel.setBackground(Color.WHITE);
		centerPanel.setBackground(Color.WHITE);
		southPanel.setBackground(Color.WHITE);
		southPanel.setBorder(
			BorderFactory.createMatteBorder(0, 0, 20, 0, Color.WHITE));

		generalPanel.add(northPanel, BorderLayout.NORTH);
		generalPanel.add(centerPanel, BorderLayout.CENTER);
		generalPanel.add(southPanel, BorderLayout.SOUTH);

		generalPanel.setBackground(Color.WHITE);
		Border matteBorder =
			BorderFactory.createMatteBorder(15, 15, 15, 15, Color.LIGHT_GRAY);
		Border loweredBevelBorder = BorderFactory.createLoweredBevelBorder();
		Border raisedBevel =
			BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1);
		Border compound =
			BorderFactory.createCompoundBorder(loweredBevelBorder, raisedBevel);
		generalPanel.setBorder(
			BorderFactory.createCompoundBorder(matteBorder, compound));
	}

	/**
	 *  This method builds this panel.
	 */
	private void buildPanel() {
		this.buildTabbedPane();
		this.buildButtonPanel();

		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(
			BorderFactory.createMatteBorder(7, 7, 7, 7, Color.LIGHT_GRAY));
		panel.add(tabbedPane, BorderLayout.CENTER);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		this.getContentPane().add(panel);
		panel.setBackground(Color.LIGHT_GRAY);
	}

	/**
	 *  This method builds the register panel.
	 */
	private void buildTabbedPane() {
		tabbedPane = new JTabbedPane();

		this.buildGeneralPanel();
		tabbedPane.add(generalPanel, "general");

		this.buildDetailsPanel();
		tabbedPane.add(detailsPanel, "details");
	}

	/**
	 *  This method builds the table of the details register.
	 */
	private void buildTable() {
		Object[][] data = new Object[fieldElements.size()][2];

		for (int i = 0; i < fieldElements.size(); i++) {
			data[i][0] = fieldElements.elementAt(i);
			data[i][1] = valueElements.elementAt(i);
		}

		String[] coleheads = { "field", "value" };

		detailsTable = new DetailsTable(data, coleheads);
	}

	/**
	 *  This method builds the version-1-fields for the table of the details
	 *  register.
	 */
	private void buildVersion1FieldsTableElements() {
		String text;
		String partialText;

		ImageIcon icon = ResourceManager.getIcon(ResourceManager.X509VERSION1);

		JLabel label1 = new JLabel("version", icon, JLabel.LEFT);
		fieldElements.addElement(label1);
		text = new Integer((x509.getVersion())).toString();
		valueElements.addElement(text);
		hashTable.put(label1.getText() + text, text);

		JLabel label2 = new JLabel("serial number", icon, JLabel.LEFT);
		fieldElements.addElement(label2);
		text = x509.getSerialNumber().toString();
		valueElements.addElement(text);
		hashTable.put(label2.getText() + text, text);

		JLabel label3 = new JLabel("signature algorithm", icon, JLabel.LEFT);
		fieldElements.addElement(label3);
		text = x509.getSigAlgName();
		valueElements.addElement(text);
		hashTable.put(label3.getText() + text, text);

		JLabel label4 = new JLabel("issuer", icon, JLabel.LEFT);
		fieldElements.addElement(label4);
		partialText = this.getPartialDNForDetails(issuer);
		text = this.getDNForDetails(issuer);
		valueElements.addElement(partialText);
		hashTable.put(label4.getText() + partialText, text);

		JLabel label5 = new JLabel("valid from", icon, JLabel.LEFT);
		fieldElements.addElement(label5);
		text = x509.getNotBefore().toString();
		valueElements.addElement(text);
		hashTable.put(label5.getText() + text, text);

		JLabel label6 = new JLabel("valid to", icon, JLabel.LEFT);
		fieldElements.addElement(label6);
		text = x509.getNotAfter().toString();
		valueElements.addElement(text);
		hashTable.put(label6.getText() + text, text);

		JLabel label7 = new JLabel("subject", icon, JLabel.LEFT);
		fieldElements.addElement(label7);
		partialText = this.getPartialDNForDetails(subject);
		text = this.getDNForDetails(subject);
		valueElements.addElement(partialText);
		hashTable.put(label7.getText() + partialText, text);

		JLabel label8 = new JLabel("public key", icon, JLabel.LEFT);
		fieldElements.addElement(label8);
		partialText = x509.getPublicKey().getAlgorithm();
		byte[] textByte = x509.getPublicKey().getEncoded();
		String zk = Hex.encode(textByte);
		text = "";
		for (int i = 0; i < (zk.length() - 3);) {
			text += zk.substring(i, i + 4) + " ";
			i += 4;
		}
		valueElements.addElement(partialText);
		hashTable.put(label8.getText() + partialText, text);
	}

	/**
	 *  Returns information of a distinguished name for the details panel.
	 *
	 *@param  dN
	 *@return     The dNForDetails value
	 */
	private String getDNForDetails(DistinguishedNameFields dN) {
		String text = dN.getCommonName(true);
		if (!text.equals("")) {
			text += "\n";
		}

		String partialText = dN.getOrgUnit(true);
		if (!partialText.equals("")) {
			text += partialText + "\n";
		}

		partialText = dN.getOrg(true);
		if (!partialText.equals("")) {
			text += partialText + "\n";
		}

		partialText = dN.getLocation(true);
		if (!partialText.equals("")) {
			text += partialText + "\n";
		}

		partialText = dN.getState(true);
		if (!partialText.equals("")) {
			text += partialText + "\n";
		}

		partialText = dN.getCountry(true);
		if (!partialText.equals("")) {
			text += partialText + "\n";
		}

		partialText = dN.getEmail(true);
		if (!partialText.equals("")) {
			text += partialText;
		}

		return text;
	}

	/**
	 *  Returns information of a distinguished name for the general panel.
	 *
	 *@param  dN
	 *@return     The dNForGeneral value
	 */
	private String getDNForGeneral(DistinguishedNameFields dN) {
		String text = dN.getCommonName(false) + "\n";

		String partialText = dN.getOrg(false);
		if (!partialText.equals("")) {
			text += partialText;
		}

		partialText = dN.getOrgUnit(false);
		if (!partialText.equals("")) {
			text += ", ";
		}
		text += partialText + "\n";

		partialText = dN.getCountry(false);
		if (!partialText.equals("")) {
			text += partialText;
		}

		partialText = dN.getLocation(false);
		if (!partialText.equals("")) {
			text += "-";
		}
		text += partialText + "\n";

		partialText = dN.getEmail(false);
		if (!partialText.equals("")) {
			text += partialText;
		}

		return text;
	}

	/**
	 *  Returns information of a part of a distinguished name for the details
	 *  panel.
	 *
	 *@param  dN
	 *@return     The partialDNForDetails value
	 */
	private String getPartialDNForDetails(DistinguishedNameFields dN) {
		String text = "";
		text += dN.getCommonName(false);
		String partialText = dN.getOrgUnit(false);
		if (!text.equals("") && !partialText.equals("")) {
			text += ",";
		}
		text += partialText;

		partialText = dN.getLocation(false);
		if (!text.equals("") && !partialText.equals("")) {
			text += ",";
		}
		text += partialText;

		return text;
	}

	/**
	 *  Save the information of the certification.
	 */
	private void saveLog() {
		String logDir = ResourceManager.getUserDefaults().getLogDirectory();
		SelectorDialog d =
			ResourceManager.getSelectorDialog(new File(logDir), null);
		d.setTitle(res.getString("SAVE_TITLE"));
		int result = d.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			PrintWriter pw;
			try {
				pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
				pw.println(log);
				pw.close();
			} catch (IOException ioe) {
				System.out.println("I/O error");
				logger.log(Level.SEVERE, "", ioe);
			}
		}
	}
}
