package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ProgressMonitor;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.requests.GetSpooledFiles;
import com.pallas.unicore.threadpool.IObserver;

/**
 *  Display and select files that will be exported to nspace.
 *
 *@author      Thomas Kentemich
 *@author      Ralf Ratering
 *@created     September 5, 2001
 *@version:    $Id: OutputFilesDialog.java,v 1.12 2001/09/24 15:07:10 rrate Exp
 *      $
 */
public class OutputFilesDialog extends GenericDialog implements IObserver {

	/**
	 *  Listener class for list selections
	 *
	 *@author     thomas
	 *@created    September 5, 2001
	 */
	private class ListListener implements ListSelectionListener {

		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void valueChanged(ListSelectionEvent event) {

			// check if this is one of multiple selection events
			if (event.getValueIsAdjusting()) {
				return;
			}
			Object source = event.getSource();
			if (source == fromList) {
			}
		}
	}
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");
	private int fetched = 0;
	private JList fromList;
	private DefaultListModel fromListModel;
	private JobContainer jobContainer;
	private ContainerNode jobNode;
	private JFrame parent;
	private ProgressMonitor progress;
	private Vector spooledFiles;
	private JTable table;
	private DefaultTableModel tableModel;
	private int toDo = 0;
	private Vector transferJobs;

	/**
	 *  Consturctor builds gui and fetches output files
	 *
	 *@param  parent        reference to main frame
	 *@param  jobNode       selected node in jmc
	 *@param  spooledFiles  Description of the Parameter
	 */
	public OutputFilesDialog(
		JFrame parent,
		ContainerNode jobNode,
		Vector spooledFiles) {
		super(
			parent,
			res.getString("EXPORT_TITLE"),
			true,
			GenericDialog.OK_CANCEL);
		this.parent = parent;
		this.jobNode = jobNode;
		this.jobContainer = (JobContainer) jobNode.getUserObject();
		this.spooledFiles = spooledFiles;
		buildComponents();

		pack();
	}

	/**
	 *  If we pressed ok, the selected files will be exported to nspace.
	 *
	 *@return    true always
	 */
	protected boolean applyValues() {
		Object[] objects = fromList.getSelectedValues();
		if (objects.length > 0) {
			jobNode.setState(ContainerNode.STATE_UPDATING);
		}
		toDo = 0;
		boolean overwriteAll = false;
		transferJobs = new Vector();
		for (int i = 0; i < objects.length; i++) {
			FileExport fe = (FileExport) objects[i];
			GetSpooledFiles gsf = new GetSpooledFiles(jobNode, fe);
			gsf.addObserver(this);
			fe.setUnSpool(gsf);
			transferJobs.add(gsf);
			toDo++;
		}
		return true;
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		ListListener listListener = new ListListener();
		fromListModel = new DefaultListModel();
		fromList = new JList(fromListModel);
		fromList.addListSelectionListener(listListener);
		fromList.setSelectionMode(
			ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		fromList.setToolTipText(res.getString("CHOOSE_FILES"));
		for (int j = 0; j < spooledFiles.size(); j++) {
			fromListModel.addElement(
				((GetSpooledFiles) spooledFiles.elementAt(j)).getFileExport());
		}
		fromList.setSelectedIndex(0);
		JScrollPane fromScrollPane = new JScrollPane(fromList);
		fromScrollPane.setBorder(
			new TitledBorder(res.getString("SELECT_FILES")));
		getContentPane().add(fromScrollPane, BorderLayout.CENTER);
	}

	/**
	 *  If we cancel the dialog nothing will happen
	 *
	 *@return    tru always
	 */
	protected boolean cancelValues() {
		return true;
	}

	/**
	 *  Gets the transferJobs attribute of the OutputFilesDialog object
	 *
	 *@return    The transferJobs value
	 */
	public Vector getTransferJobs() {
		return this.transferJobs;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  theObserved  Description of Parameter
	 *@param  changeCode   Description of Parameter
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {
	}
}
