package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;

/**
 *  Dialog to enter a password.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: PasswordDialog.java,v 1.3 2003/06/30 06:53:27 rrate Exp $
 */
public class PasswordDialog extends GenericDialog {
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		PasswordDialog dialog =
			new PasswordDialog(frame, "TEST: PasswordDialog");
		dialog.show();
	}
	private char[] password;
	private JPasswordField passwordField = new JPasswordField(15);

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title   dialog title
	 *@param  parent  dialog parent - important for modal dialogs
	 */
	public PasswordDialog(JFrame parent, String title) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);
		buildComponents();
		pack();
	}

	/**
	 *  Takes current textfield entry as password
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean applyValues() {
		password = passwordField.getPassword();
		return true;
	}

	/**
	 *  Add a password field to the parent dialog
	 */
	private void buildComponents() {
		JLabel passwordLabel = new JLabel(res.getString("ENTER_PASSWORD"));

		JPanel passwordPanel = new JPanel();
		passwordPanel.add(passwordLabel);
		passwordPanel.add(passwordField);
		passwordPanel.setBorder(new EmptyBorder(20, 5, 10, 5));

		getContentPane().add(passwordPanel, BorderLayout.CENTER);
	}

	/**
	 *  Sets password to null
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		password = null;
		return true;
	}

	/**
	 *  Get the result from the dialog
	 *
	 *@return    null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getPassword() {
		return password;
	}

	/**
	 *  Put focus into password field
	 */
	public void show() {
		// focus request does not work for windows, see jdk bug 4199374
		passwordField.requestFocus();
		super.show();
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public int showDialog() {
		passwordField.requestFocus();
		return super.showDialog();
	}
}
