package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.security.cert.X509Certificate;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import org.unicore.User;
import org.unicore.Vsite;

import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.KeyStoreManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Set different user certifictaes for usites or vsites
 *
 *@author     Kirsten Foerster
 *@version    $Id: SetIdentityDialog.java,v 1.6 2003/07/09 08:48:04 rrate Exp $
 */
public class SetIdentityDialog extends GenericDialog {
    public final static String ALL = "<all>";
    private final static Logger logger = Logger.getLogger("com.pallas.unicore.client.dialogs");
    static ResourceBundle res = ResourceBundle.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");

    private boolean showAllIdentities = true;
    private Vector uIdColheads = null;
    private String selectedItem = "";

    private Vector certList;
    private Vector vIdColheads;

    private Vector vIdData;
    private Vector uIdData;

    private JCheckBox checkBox;
    private JComboBox comboBox;
    private JTabbedPane tabbedPane;
    private JPanel uIdPanel;
    private JPanel vIdPanel;
    private JScrollPane vIdScrollPane;
    private JScrollPane uIdScrollPane;
    private DetailsTable vsitesIdTable;
    private DetailsTable usitesIdTable;
    private ComboCellEditor comboCellEditor;

    private UserDefaults userDefaults;
    private Hashtable vIdentity;
    private Hashtable uIdentity;
    private Hashtable usitesTable;

    private SelectionListener selectionListener;


    /**
     *  Constructor for the SetIdentityDialog object
     *
     *@param  parentFrame    Description of the Parameter
     *@exception  Exception  Description of the Exception
     */
    public SetIdentityDialog(JFrame parentFrame) throws Exception {
        super(parentFrame, res.getString("SETIDENTITY_TITLE"), true, OK_CANCEL);

        // get reference to user defaults from static resource manager
        userDefaults = ResourceManager.getUserDefaults();

        selectionListener = new SelectionListener();
        this.buildUsitesTable();
        this.certList = this.buildCertElements();
        this.buildVIdentity();
        this.buildUIdentity();
        this.buildColheads(false);
        this.buildComponents(false);
        this.okButton.removeActionListener(buttonListener);
        this.okButton.addActionListener(new NewButtonListener());
        this.setSize(500, 600);
    }


    /**
     *  Sets the users attribute of the SetIdentityDialog object
     *
     *@param  table  The new users value
     */
    private void setUsers(Hashtable table) {
        Enumeration e = table.keys();
        while(e.hasMoreElements()) {
            Object key = e.nextElement();
            String identity = (String)table.get(key);
            if(identity.equals(UserDefaults.DEFAULT)) {
                Usite usite = null;
                if(key instanceof Vsite) {
                    Vsite vsite = (Vsite)key;
                    usite = (Usite)this.usitesTable.get(vsite);
                }
                else {
                    usite = (Usite)key;
                }
                identity = userDefaults.getDefaultIdentity(usite);
                if(identity == null || identity.equals("")) {
                    logger.log(Level.SEVERE, "default identity is not set -> user could not be created for " + key.toString());
                    continue;
                }
            }

            int beginIndex = 0;
            int endIndex = identity.lastIndexOf(":");
            endIndex = identity.lastIndexOf(":", endIndex - 1);

            String alias = identity.substring(beginIndex, endIndex);

            try {
                X509Certificate[] x509 = ResourceManager.getKeystoreManager().getCertificateByAlias(alias);
                if(x509 != null) {
                    User user = new User(x509[0], null);

                    beginIndex = endIndex + 1;
                    endIndex = identity.indexOf(":", beginIndex);
                    String xLogin = identity.substring(beginIndex, endIndex);
                    if(!xLogin.equals("")) {
                        user.setXlogin(xLogin);
                    }

                    beginIndex = endIndex + 1;
                    endIndex = identity.length();
                    String project = identity.substring(beginIndex, endIndex);
                    if(!project.equals("")) {
                        user.setProject(project);
                    }

                    if(key instanceof Usite) {
                        ResourceManager.setUser(user, (Usite)key);
                    }
                    else {
                        ResourceManager.setUser(user, (Vsite)key);
                    }
                }
            }
            catch(Exception ex) {
                logger.log(Level.SEVERE, "Could not read certificate " + alias +
                        " from keystore.", ex);
            }
        }
    }


    /**
     *  Makes the Dialog visible.
     */
    public void show() {
        getRootPane().setDefaultButton(okButton);
        if(!showAllIdentities) {
            JOptionPane.showMessageDialog(this, res.getString("NO_DEFAULT_CERT"),
                    "WARNING", JOptionPane.WARNING_MESSAGE);
        }
        super.show();
    }


    /**
     *  When cancel was pressed reload user defaults from file.
     *
     *@return    if canceling was successful
     */
    protected boolean cancelValues() {
        resetValues();
        return true;
    }


    /**
     *  Fill text fields with current class member values
     */
    protected void resetValues() {
        super.resetValues();
    }


    /**
     *  Apply values will be called from parent class
     *
     *@return    true if applying was successful
     */
    protected boolean applyValues() {
        this.buildNewVIdentity(this.checkBox.isSelected());
        Enumeration e = this.vIdentity.keys();
        while(e.hasMoreElements()) {
            Vsite vsite = (Vsite)e.nextElement();
            Usite usite = (Usite)this.usitesTable.get(vsite);
            String value = (String)this.vIdentity.get(vsite);
            userDefaults.setIdentitiy(vsite, value);
        }
        this.buildNewUIdentity(false);
        e = this.uIdentity.keys();
        while(e.hasMoreElements()) {
            Usite usite = (Usite)e.nextElement();
            String value = (String)this.uIdentity.get(usite);
            userDefaults.setIdentitiy(usite, value);
        }

        // write defaults to file
        if(!userDefaults.writeToFile()) {
            UserMessages.error(res.getString("COULD_NOT_WRITE"));
            setStatusMessage(res.getString("COULD_NOT_WRITE"));
        }
        else {
            setStatusMessage(res.getString("SUCESSFULLY_STORED"));
        }

        this.setUsers(this.vIdentity);
        this.setUsers(this.uIdentity);

        // returning true closes dialog if ok was pressed
        return true;
    }


    /**
     *  Builds the components of this dialog.
     *
     *@param  flag  Description of the Parameter
     */
    private void buildVsiteIdPanel(boolean flag) {
        this.vIdPanel = new JPanel(new BorderLayout(0, 20));
        JLabel label = new JLabel("UNICORE Site: ");
        Vector items = new Vector();
        items.addElement(ALL);
        Vector usites = ResourceManager.getUsites();
        for(int i = 0; i < usites.size(); i++) {
            items.addElement(usites.elementAt(i));
        }

        comboBox = new JComboBox(items);
        comboBox.setEditable(false);
        comboBox.setName("usitesList");
        comboBox.setBackground(Color.WHITE);
        comboBox.addItemListener(selectionListener);
        comboBox.setSize(5, 40);

        JPanel northPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
        northPanel.add(label);
        northPanel.add(comboBox);
        this.vIdPanel.add(northPanel, BorderLayout.NORTH);
        this.vIdData = this.buildAllVsiteIdElements(flag);
        this.buildVsitesIdTable();
        this.vIdPanel.setBorder(BorderFactory.createMatteBorder(30, 10, 10, 10, this.vIdPanel.getBackground()));
    }


    private void buildUsiteIdPanel() {
        this.uIdPanel = new JPanel(new BorderLayout(15, 0));
        this.uIdData = this.buildAllUsiteIdElements(false);
        this.buildUsitesIdTable();
        this.uIdPanel.setBorder(BorderFactory.createMatteBorder(30, 10, 10, 10, this.uIdPanel.getBackground()));
    }

    private void buildComponents(boolean flag) {
        JPanel panel = new JPanel(new BorderLayout());

        tabbedPane = new JTabbedPane();
        this.buildVsiteIdPanel(flag);
        this.buildUsiteIdPanel();
        TableFocusListener focusListener = new TableFocusListener();
        this.vIdPanel.addFocusListener(focusListener);
        this.uIdPanel.addFocusListener(focusListener);
        tabbedPane.add(this.vIdPanel, "Virtual Sites");
        tabbedPane.add(this.uIdPanel, "UNICORE Sites");
        tabbedPane.setBorder(BorderFactory.createMatteBorder(7, 7, 7, 7, this.vIdPanel.getBackground()));
        panel.add(tabbedPane, BorderLayout.CENTER);

        JPanel flow = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        checkBox = new JCheckBox("expert mode");
        checkBox.setSelected(flag);
        checkBox.addItemListener(selectionListener);
        flow.add(checkBox);
        flow.setBorder(BorderFactory.createMatteBorder(15, 0, 15, 0, flow.getBackground()));
        panel.add(flow, BorderLayout.SOUTH);

        this.getContentPane().add(panel);
    }


    /**
     *  Builds the vector of all identities.
     *
     *@return    Returns the identity
     */
    private Vector buildCertElements() {
        Vector cert = new Vector();
        KeyStoreManager manager = ResourceManager.getKeystoreManager();
        cert.addAll(manager.getCachedKeyEntries());
        cert.insertElementAt("default", 0);
        return cert;
    }


    private void buildVsitesIdTable() {
        DetailsTableModel model = new DetailsTableModel();
        model.setDataVector(vIdData, vIdColheads);
        this.vsitesIdTable = new DetailsTable(model, 2);
        this.vsitesIdTable.changeSelection(0, 0, false, false);
        this.vIdScrollPane = new JScrollPane(this.vsitesIdTable);

        Border matteBorder = BorderFactory.createMatteBorder(0, 15, 15, 15, this.vIdPanel.getBackground());
        Border loweredBevelBorder = BorderFactory.createLoweredBevelBorder();
        Border raisedBevel = BorderFactory.createLineBorder(this.vIdPanel.getBackground(), 1);
        Border compound = BorderFactory.createCompoundBorder(loweredBevelBorder, raisedBevel);
        this.vIdScrollPane.setBorder(BorderFactory.createCompoundBorder(matteBorder, compound));

        this.vIdPanel.add(this.vIdScrollPane, BorderLayout.CENTER);
    }


    private void buildUsitesIdTable() {
        DetailsTableModel model = new DetailsTableModel();
        model.setDataVector(uIdData, uIdColheads);
        this.usitesIdTable = new DetailsTable(model, 1);
        this.usitesIdTable.setModel(model);
        this.usitesIdTable.changeSelection(0, 0, false, false);
        this.uIdScrollPane = new JScrollPane(this.usitesIdTable);

        Border matteBorder = BorderFactory.createMatteBorder(0, 15, 15, 15, this.uIdPanel.getBackground());
        Border loweredBevelBorder = BorderFactory.createLoweredBevelBorder();
        Border raisedBevel = BorderFactory.createLineBorder(this.uIdPanel.getBackground(), 1);
        Border compound = BorderFactory.createCompoundBorder(loweredBevelBorder, raisedBevel);
        this.uIdScrollPane.setBorder(BorderFactory.createCompoundBorder(matteBorder, compound));

        this.uIdPanel.add(this.uIdScrollPane, BorderLayout.CENTER);
    }


    /**
     *  Builds the columns headings of the JTable component.
     *
     *@param  flag  Description of the Parameter
     */
    private void buildColheads(boolean flag) {
        this.vIdColheads = new Vector();
        this.vIdColheads.add("UNICORE Site");
        this.vIdColheads.addElement("Virtual Site");
        this.vIdColheads.addElement("Certificate");
        if(flag) {
            this.vIdColheads.addElement("XLogin");
            this.vIdColheads.addElement("Project");
        }

        if(this.uIdColheads == null) {
            this.uIdColheads = new Vector();

            this.uIdColheads.add("UNICORE Site");
            this.uIdColheads.addElement("Certificate");
        }
    }


    /**
     *  Builds one row of the JTable.
     *
     *@param  usite
     *@param  vsite
     *@param  flag   Description of the Parameter
     *@return        Returns one row vector of the JTable data
     */
    private Vector buildRowElements(Usite usite, Vsite vsite, boolean flag) {
        Vector row = new Vector();
        String cert = "";
        String xLogin = "";
        String project = "";

        String identity = (String)this.vIdentity.get(vsite);

        if(identity == null || identity.equals("")) {
            return null;
        }

        int beginIndex = 0;
        int endIndex = identity.lastIndexOf(":");
        endIndex = identity.lastIndexOf(":", endIndex - 1);

        cert += identity.substring(beginIndex, endIndex);

        row.addElement(usite);
        row.addElement(vsite);
        row.addElement(cert);

        if(flag) {
            beginIndex = endIndex + 1;
            endIndex = identity.indexOf(":", beginIndex);
            xLogin += identity.substring(beginIndex, endIndex);

            beginIndex = endIndex + 1;
            endIndex = identity.length();
            project += identity.substring(beginIndex, endIndex);

            row.addElement(xLogin);
            row.addElement(project);
        }

        return row;
    }


    private Vector buildRowElements(Usite usite, boolean flag) {
        Vector row = new Vector();
        String cert = "";
        String xLogin = "";
        String project = "";

        String identity = (String)this.uIdentity.get(usite);

        if(identity == null || identity.equals("")) {
            return null;
        }

        int beginIndex = 0;
        int endIndex = identity.lastIndexOf(":");
        endIndex = identity.lastIndexOf(":", endIndex - 1);
        cert += identity.substring(beginIndex, endIndex);

        row.addElement(usite);
        row.addElement(cert);

        if(flag) {
            beginIndex = endIndex + 1;
            endIndex = identity.indexOf(":", beginIndex);
            xLogin += identity.substring(beginIndex, endIndex);

            beginIndex = endIndex + 1;
            endIndex = identity.length();
            project += identity.substring(beginIndex, endIndex);

            row.addElement(xLogin);
            row.addElement(project);
        }

        return row;
    }


    /**
     *  Builds all data of the JTable
     *
     *@param  flag  Description of the Parameter
     *@return       Returns a JTable data vector for all usites
     */
    private Vector buildAllVsiteIdElements(boolean flag) {
        Vector data = new Vector();
        Vector usites = ResourceManager.getUsites();
        for(int i = 0; i < usites.size(); i++) {
            Usite usite = (Usite)usites.elementAt(i);
            Vector vsites = ResourceManager.getVsites(usite);
            for(int j = 0; j < vsites.size(); j++) {
                Vsite vsite = (Vsite)vsites.elementAt(j);
                Vector vector = this.buildRowElements(usite, vsite, flag);
                if(vector != null) {
                    data.addElement(vector);
                }
                else {
                    showAllIdentities = false;
                }
            }
        }

        return data;
    }


    private Vector buildAllUsiteIdElements(boolean flag) {
        Vector data = new Vector();
        Vector usites = ResourceManager.getUsites();
        for(int i = 0; i < usites.size(); i++) {
            Usite usite = (Usite)usites.elementAt(i);
            Vector vector = this.buildRowElements(usite, flag);
            if(vector != null) {
                data.addElement(vector);
            }
            else {
                showAllIdentities = false;
            }
        }
        return data;
    }

    private void buildUsitesTable() {
        this.usitesTable = new Hashtable();
        Vector usites = ResourceManager.getUsites();
        for(int i = 0; i < usites.size(); i++) {
            Vector vsites = ResourceManager.getVsites((Usite)usites.elementAt(i));
            for(int j = 0; j < vsites.size(); j++) {
                this.usitesTable.put((Vsite)vsites.elementAt(j), (Usite)usites.elementAt(i));
            }
        }
    }


    /**
     *  Builds the data of the JTable for one usite
     *
     *@param  usite
     *@param  flag   Description of the Parameter
     *@return        Returns a JTable data vector for one usite
     */
    private Vector buildVsiteIdElements(Usite usite, boolean flag) {
        Vector data = new Vector();
        Vector vsites = ResourceManager.getVsites(usite);
        for(int i = 0; i < vsites.size(); i++) {
            Vector vector = this.buildRowElements(usite, (Vsite)vsites.elementAt(i), flag);
            if(vector != null) {
                data.addElement(vector);
            }
            else {
                showAllIdentities = false;
            }
        }

        return data;
    }

    private void buildNewVIdentity(boolean flag) {
        int rowCount = this.vsitesIdTable.getModel().getRowCount();
        int columnCount = this.vsitesIdTable.getModel().getColumnCount();

        for(int i = 0; i < vsitesIdTable.getModel().getRowCount(); i++) {
            Vsite vsite = (Vsite)vsitesIdTable.getModel().getValueAt(i, 1);
            String value = "";
            for(int j = 2; j < columnCount; j++) {
                value += vsitesIdTable.getModel().getValueAt(i, j).toString().trim();
                if(j != columnCount - 1) {
                    value += ":";
                }
            }
            if(!flag) {
                String oldValue = (String)this.vIdentity.get(vsite);
                int beginIndex = 0;
                int endIndex = oldValue.lastIndexOf(":");
                endIndex = oldValue.lastIndexOf(":", endIndex - 1);
                String cert = oldValue.substring(beginIndex, endIndex);
                if(cert.equals(this.vsitesIdTable.getModel().getValueAt(i, 2).toString())) {
                    beginIndex = endIndex;
                    endIndex = oldValue.length();
                    value += oldValue.substring(beginIndex, endIndex);
                }
                else {
                    value += "::";
                }
            }
            this.vIdentity.put(vsite, value);
        }
    }

    private void buildNewUIdentity(boolean flag) {
        int rowCount = this.usitesIdTable.getModel().getRowCount();
        int columnCount = this.usitesIdTable.getColumnCount();

        for(int i = 0; i < rowCount; i++) {
            Usite usite = (Usite)this.usitesIdTable.getModel().getValueAt(i, 0);
            String value = "";
            for(int j = 1; j < columnCount; j++) {
                value += this.usitesIdTable.getModel().getValueAt(i, j).toString().trim();
                if(j != columnCount - 1) {
                    value += ":";
                }
            }
            if(!flag) {
                String oldValue = (String)this.uIdentity.get(usite);
                int beginIndex = 0;
                int endIndex = oldValue.lastIndexOf(":");
                endIndex = oldValue.lastIndexOf(":", endIndex - 1);
                String cert = oldValue.substring(beginIndex, endIndex);
                if(cert.equals(this.usitesIdTable.getModel().getValueAt(i, 1).toString())) {
                    beginIndex = endIndex;
                    endIndex = oldValue.length();
                    value += oldValue.substring(beginIndex, endIndex);
                }
                else {
                    value += "::";
                }

            }
            this.uIdentity.put(usite, value);
        }
    }


    /**
     *  Description of the Method
     *
     *@param  identity  Description of the Parameter
     *@param  usite     ""-string, if a usite identity will be checked; if a
     *      vsite will be checked you need the belonging usite.
     *@return           Description of the Return Value
     */
    private String checkIdentitiy(String identity, Usite usite) {
        if(identity.equals("") || identity == null) {
            return null;
        }

        String value = "";

        int beginIndex = 0;
        int endIndex = identity.lastIndexOf(":");
        endIndex = identity.lastIndexOf(":", endIndex - 1);
        String cert = identity.substring(beginIndex, endIndex);

        KeyStoreManager manager = ResourceManager.getKeystoreManager();

        try {
            if(!manager.keyStoreContainsAlias(cert)) {
                if(usite == null) {
                    if(userDefaults.getDefaultIdentity() == null || ResourceManager.getUserDefaults().getDefaultIdentity().equals("")) {
                        return value;
                    }
                }
                else {
                    if(this.userDefaults.getIdentity(usite).equals("")) {
                        return value;
                    }
                    return this.checkIdentitiy(this.userDefaults.getIdentity(usite), null);
                }
            }
        }
        catch(Exception ex) {
            logger.warning("Keystore does not contain this alias " + cert);
        }

        int index = -1;

        for(int i = 0; i < certList.size(); i++) {
            if(certList.elementAt(i).toString().equals(cert)) {
                index = i;
                break;
            }
        }

        //if the saved identity is removed from the keystore the usite will be
        //obtained the default identity
        if(index == -1) {
            if(usite == null) {
                if(userDefaults.getDefaultIdentity() == null || userDefaults.getDefaultIdentity().equals("")) {
                    return value;
                }
                String defaultStr = UserDefaults.DEFAULT;
                defaultStr = defaultStr.substring(0,defaultStr.indexOf(":"));
                index = certList.indexOf(defaultStr);
            }
            else {
                if(this.userDefaults.getIdentity(usite).equals("")) {
                    return value;
                }
                return this.checkIdentitiy(this.userDefaults.getIdentity(usite), null);
            }
        }

        beginIndex = endIndex;

        if(index == -1) {
            return value;
        }
        identity = certList.get(index).toString() + identity.substring(beginIndex);

        return identity;
    }


    private void buildVIdentity() {
        this.vIdentity = new Hashtable();
        Enumeration e = this.usitesTable.keys();
        Vsite vsite = null;
        while(e.hasMoreElements()) {
            vsite = (Vsite)e.nextElement();
            Usite usite = (Usite)this.usitesTable.get(vsite);

            String oldIdentity = this.userDefaults.getIdentity(vsite);
            String identity = this.checkIdentitiy(oldIdentity, usite);

			if(!oldIdentity.equals(identity)) {
				UserMessages.warning("Identity: "+oldIdentity+" no longer valid.\nReset to default");
			}
            if(identity.equals("")) {
                this.showAllIdentities = false;
                continue;
            }
            this.vIdentity.put(vsite, identity);
        }
    }


    private void buildUIdentity() {
        this.uIdentity = new Hashtable();
        Vector usites = ResourceManager.getUsites();
        for(int i = 0; i < usites.size(); i++) {
            Usite usite = (Usite)usites.elementAt(i);
            String oldIdentity = this.userDefaults.getIdentity(usite);
            String identity = this.checkIdentitiy(oldIdentity, null);
            
			if(!oldIdentity.equals(identity)) {
				UserMessages.warning("Identity: "+oldIdentity+" no longer valid.\nReset to default");
			}
            
            if(identity.equals("")) {
                this.showAllIdentities = false;
                continue;
            }
            this.uIdentity.put(usite, identity);
        }
    }


    private class DetailsTableModel extends DefaultTableModel {

        /**
         *  Gets the CellEditable attribute of the InfoTableModel object
         *
         *@param  row  Description of Parameter
         *@param  col  Description of Parameter
         *@return      The CellEditable value
         */
        public boolean isCellEditable(int row, int col) {
            Object entry = getValueAt(row, col);
            return (!(entry instanceof Usite) && !(entry instanceof Vsite));
        }
    }


    private class DetailsTable extends JTable {


        /**
         *  Constructor for the DetailsTable object
         *
         *@param  data      Description of the Parameter
         *@param  colheads  Description of the Parameter
         *@param  column    Description of the Parameter
         */
        public DetailsTable(DetailsTableModel model, int comboColumn) {
            super(model);


            this.setShowGrid(true);
            this.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
            this.setColumnSelectionAllowed(false);
            this.setRowSelectionAllowed(true);
            this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

            JComboBox comboBox = new JComboBox(certList);
            comboBox.setName("certList");
            comboBox.setBackground(Color.WHITE);
            comboBox.addItemListener(selectionListener);
            this.setRowHeight(21);
            this.setFont(new Font(this.getName(), Font.PLAIN, 11));
            getColumnModel().getColumn(comboColumn).setCellEditor(new ComboCellEditor(comboBox));
            getColumnModel().getColumn(comboColumn).setCellRenderer(new ComboRenderer(certList));
            TableFocusListener tableFocusListener = new TableFocusListener();
            this.addFocusListener(tableFocusListener);

        }
    }

    private class ComboRenderer extends JComboBox implements TableCellRenderer {
        /**
         *  Constructor for the ComboRenderer object
         *
         *@param  vector  Description of the Parameter
         */
        public ComboRenderer(Vector vector) {
            super(vector);
        }


        /**
         *  Gets the tableCellRendererComponent attribute of the ComboRenderer
         *  object
         *
         *@param  table       Description of the Parameter
         *@param  value       Description of the Parameter
         *@param  isSelected  Description of the Parameter
         *@param  hasFocus    Description of the Parameter
         *@param  row         Description of the Parameter
         *@param  column      Description of the Parameter
         *@return             The tableCellRendererComponent value
         */
        public Component getTableCellRendererComponent(JTable table, Object value,
                boolean isSelected, boolean hasFocus,
                int row, int column) {

            this.setOpaque(true);
            setFont(table.getFont());

            if(isSelected) {
                this.setBackground(table.getSelectionBackground());
            }
            else {

                this.setBackground(Color.WHITE);
            }
            this.setSelectedItem(value);
            return this;
        }
    }

    private class ComboCellEditor extends DefaultCellEditor {
        public ComboCellEditor(JComboBox comboBox) {
            super(comboBox);
        }


        /**
         *  Implements the <code>TableCellEditor</code> interface.
         *
         *@param  table       Description of the Parameter
         *@param  value       Description of the Parameter
         *@param  isSelected  Description of the Parameter
         *@param  row         Description of the Parameter
         *@param  column      Description of the Parameter
         *@return             The tableCellEditorComponent value
         */
        public Component getTableCellEditorComponent(JTable table, Object value,
                boolean isSelected,
                int row, int column) {
            /*
             *  Select the current row. For some reason the ComboBoxListener is executed
             *  before the ListSelectionListener, i.e. the JTable.getSelectedRow() would
             *  be wrong in ComboBoxListener.itemChanged() without the changed selection.
             */
            this.delegate.setValue(value.toString());
            table.changeSelection(row, column, false, false);
            return editorComponent;
        }
    }


    /**
     *  This listener is used to get the value of the last edited cell of the
     *  JTable
     *
     *@author     kfoerst
     */
    private class TableFocusListener extends FocusAdapter {
        public void focusLost(FocusEvent e) {
            DetailsTable table = null;
            if(e.getSource() instanceof DetailsTable) {
                table = (DetailsTable)e.getSource();
            }

            if(e.getSource() == tabbedPane) {
                if((JPanel)tabbedPane.getSelectedComponent() == vIdPanel) {
                    table = usitesIdTable;
                }
                else {
                    table = vsitesIdTable;
                }
            }

            if(table != null && table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn()) != null) {
                if(table.editCellAt(table.getSelectedRow(), table.getSelectedColumn())) {
                    table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn()).stopCellEditing();
                }
            }
        }

        public void focusGained(FocusEvent e) {
            if(e.getSource() instanceof JComboBox) {
                selectedItem = (String)((JComboBox)e.getSource()).getSelectedItem();
            }
        }
    }



    /**
     *  Returns the identities of one or all usites
     *
     *@author     kfoerst
     */
    private class SelectionListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if(e.getSource() instanceof JComboBox && ((JComboBox)e.getSource()).getName().equals("certList")) {
                if(e.getStateChange() == ItemEvent.DESELECTED) {
                    return;
                }
                if(checkBox.isSelected() && (JPanel)tabbedPane.getSelectedComponent() == vIdPanel) {
                    int selectedRow = vsitesIdTable.getSelectedRow();
                    if(!((String)((JComboBox)e.getSource()).getSelectedItem()).equals(selectedItem)) {
                        vsitesIdTable.setValueAt("", selectedRow, 3);
                        vsitesIdTable.setValueAt("", selectedRow, 4);
                    }
                }
                return;
            }

            DetailsTable table;
            if((JPanel)tabbedPane.getSelectedComponent() == vIdPanel) {
                table = vsitesIdTable;
            }
            else {
                table = usitesIdTable;
            }
            if(table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn()) != null) {
                if(table.editCellAt(table.getSelectedRow(), table.getSelectedColumn())) {
                    table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn()).stopCellEditing();
                }
            }

            if(e.getSource() instanceof JCheckBox) {
                if(e.getStateChange() == ItemEvent.SELECTED) {
                    buildColheads(true);
                    buildNewVIdentity(false);
                    vIdData.removeAllElements();
                    try {
                        if(comboBox.getSelectedItem().toString().equals(ALL)) {
                            vIdData = buildAllVsiteIdElements(true);
                        }
                        else {
                            vIdData = buildVsiteIdElements((Usite)comboBox.getSelectedItem(), true);
                        }
                    }
                    catch(Exception ex) {
                        logger.log(Level.WARNING, "Vsite identities cannot be loaded.", ex);
                    }
                }

                if(e.getStateChange() == ItemEvent.DESELECTED) {
                    buildColheads(false);
                    buildNewVIdentity(true);
                    vIdData.removeAllElements();
                    try {
                        if(comboBox.getSelectedItem().toString().equals(ALL)) {
                            vIdData = buildAllVsiteIdElements(false);
                        }
                        else {
                            vIdData = buildVsiteIdElements((Usite)comboBox.getSelectedItem(), false);
                        }
                    }
                    catch(Exception ex) {
                        logger.log(Level.WARNING, "Vsite identities cannot be loaded.", ex);
                    }
                }
                vIdPanel.remove(vIdScrollPane);
                buildVsitesIdTable();
                vIdPanel.validate();
                return;
            }

            if(e.getStateChange() == ItemEvent.DESELECTED) {
                return;
            }

            if(e.getSource() instanceof JComboBox && ((JComboBox)e.getSource()).getName().equals("usitesList")) {
                buildNewVIdentity(checkBox.isSelected());
                vIdData.removeAllElements();
                try {
                    if(((JComboBox)e.getSource()).getSelectedItem().toString().equals(ALL)) {
                        vIdData = buildAllVsiteIdElements(true);
                    }
                    else {
                        vIdData = buildVsiteIdElements((Usite)((JComboBox)e.getSource()).getSelectedItem(), true);
                    }
                }
                catch(Exception ex) {
                    logger.log(Level.WARNING, "Vsite identities cannot be loaded.", ex);
                }
                vIdPanel.remove(vIdScrollPane);
                buildVsitesIdTable();
                vIdPanel.validate();
                return;
            }
        }
    }


    private class NewButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DetailsTable table;
            if((JPanel)tabbedPane.getSelectedComponent() == vIdPanel) {
                table = vsitesIdTable;
            }
            else {
                table = usitesIdTable;
            }
            if(table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn()) != null) {
                if(table.editCellAt(table.getSelectedRow(), table.getSelectedColumn())) {
                    table.getCellEditor(table.getSelectedRow(), table.getSelectedColumn()).stopCellEditing();
                }
            }

            // ok = apply and close
            if(e.getSource() == okButton) {
                if(applyValues()) {
                    dispose();
                }
            }
        }
    }
}
