package com.pallas.unicore.client.dialogs;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.Principal;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Dialog that displays logging information from JMC
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ShowCertDialog.java,v 1.3 2003/06/30 06:53:25 rrate Exp $
 */
public class ShowCertDialog extends JDialog {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  evt  Description of Parameter
		 */
		public void actionPerformed(ActionEvent evt) {
			Object source = evt.getSource();

			if (source == closeButton) {
				dispose();
			} else if (source == saveButton) {
				saveLog();
			}
		}
	}
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.dialogs");

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");
	private JButton closeButton = new JButton(res.getString("CLOSE"));
	private String log;
	private JButton saveButton = new JButton(res.getString("SAVE"));
	private X509Certificate x509;

	/**
	 *@param  parentFrame  Description of Parameter
	 *@param  x509         Description of Parameter
	 */
	public ShowCertDialog(JFrame parentFrame, X509Certificate x509) {

		// build a modal dialog
		super(parentFrame, res.getString("CERTIFICATE"), true);
		this.x509 = x509;
		this.log = x509.toString();

		buildComponents();

		pack();
		setLocationRelativeTo(parentFrame);
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		/*
		 *  JTextArea textArea = new JTextArea(15, 40);
		 *  textArea.append(log);
		 *  JScrollPane scrollPane = new JScrollPane(textArea);
		 */
		int in1;
		/*
		 *  JTextArea textArea = new JTextArea(15, 40);
		 *  textArea.append(log);
		 *  JScrollPane scrollPane = new JScrollPane(textArea);
		 */
		int in2;

		JPanel subjectPanel = new JPanel(new GridLayout(5, 1, 5, 5));
		subjectPanel.setBorder(new TitledBorder(res.getString("SUBJECT")));
		Principal subjectDN = x509.getSubjectDN();
		String all = subjectDN.toString();
		in1 = all.indexOf("CN=");
		in2 = all.indexOf(',', in1);
		JLabel text1 = new JLabel(all.substring(in1 + 3, in2));
		in1 = all.indexOf("O=");
		in2 = all.indexOf(',', in1);
		JLabel text2 = new JLabel(all.substring(in1 + 2, in2));
		in1 = all.indexOf("OU=");
		in2 = all.indexOf(',', in1);
		JLabel text3 = new JLabel(all.substring(in1 + 3, in2));
		in1 = all.indexOf("L=");
		in2 = all.indexOf(',', in1);
		JLabel text4 = new JLabel(all.substring(in1 + 2, in2));
		in1 = all.indexOf(res.getString("EMAIL"));
		in2 = all.indexOf(',', in1);
		JLabel text5 = new JLabel(all.substring(in1 + 13, in2));
		subjectPanel.add(text1);
		subjectPanel.add(text2);
		subjectPanel.add(text3);
		subjectPanel.add(text4);
		subjectPanel.add(text5);

		JPanel issuerPanel = new JPanel(new GridLayout(5, 1, 5, 5));
		issuerPanel.setBorder(new TitledBorder("Issuer"));

		Principal issuerDN = x509.getIssuerDN();
		all = issuerDN.toString();
		in1 = all.indexOf("CN=");
		in2 = all.indexOf(',', in1);
		JLabel text6 = new JLabel(all.substring(in1 + 3, in2));
		in1 = all.indexOf("O=");
		in2 = all.indexOf(',', in1);
		JLabel text7 = new JLabel(all.substring(in1 + 2, in2));
		in1 = all.indexOf("OU=");
		in2 = all.indexOf(',', in1);
		JLabel text8 = new JLabel(all.substring(in1 + 3, in2));
		in1 = all.indexOf("L=");
		in2 = all.indexOf(',', in1);
		JLabel text9 = new JLabel(all.substring(in1 + 2, in2));
		in1 = all.indexOf(res.getString("EMAIL"));
		in2 = all.indexOf(',', in1);
		JLabel text10 = new JLabel(all.substring(in1 + 13, in2));
		issuerPanel.add(text6);
		issuerPanel.add(text7);
		issuerPanel.add(text8);
		issuerPanel.add(text9);
		issuerPanel.add(text10);

		JPanel validPanel = new JPanel(new GridLayout(2, 1, 5, 5));
		validPanel.setBorder(new TitledBorder(res.getString("INFO")));

		JLabel text11 =
			new JLabel(res.getString("SERIAL") + x509.getSerialNumber());
		JLabel text12 =
			new JLabel(
				res.getString("VALID")
					+ x509.getNotBefore()
					+ res.getString("TO")
					+ x509.getNotAfter());

		validPanel.add(text11);
		validPanel.add(text12);

		ButtonListener buttonListener = new ButtonListener();
		closeButton.addActionListener(buttonListener);
		saveButton.addActionListener(buttonListener);

		JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 20, 20));
		buttonPanel.add(saveButton);
		buttonPanel.add(closeButton);

		JPanel southPanel = new JPanel();
		southPanel.add(buttonPanel);

		JPanel northPanel = new JPanel(new GridLayout(2, 1, 10, 10));
		northPanel.add(subjectPanel);
		northPanel.add(issuerPanel);

		JPanel centerPanel = new JPanel(new GridLayout(2, 1, 10, 10));
		centerPanel.add(validPanel);

		getContentPane().add(northPanel, "North");
		getContentPane().add(centerPanel, "Center");
		getContentPane().add(southPanel, "South");

		//    getContentPane().add(scrollPane, "Center");
	}

	/**
	 *  Description of the Method
	 */
	private void saveLog() {
		String logDir = ResourceManager.getUserDefaults().getLogDirectory();
		JFileChooser d = new JFileChooser(new File(logDir));
		d.setFileHidingEnabled(false);
		d.setDialogTitle(res.getString("SAVE_TITLE"));
		int result = d.showSaveDialog(this);
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			PrintWriter pw;
			try {
				pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
				pw.println(log);
				pw.close();
			} catch (IOException ioe) {
				System.out.println("I/O error");
				logger.log(Level.SEVERE, "", ioe);
			}
		}
	}
}
