package com.pallas.unicore.client.dialogs;

import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Dialog that displays logging information from JMC
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ShowLogDialog.java,v 1.4 2003/06/30 06:53:25 rrate Exp $
 */
public class ShowLogDialog extends JDialog {

	/**
	 *  Listen to button clicks
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  evt  Description of Parameter
		 */
		public void actionPerformed(ActionEvent evt) {
			Object source = evt.getSource();

			if (source == closeButton) {
				dispose();
			} else if (source == saveButton) {
				saveLog();
			}
		}
	}
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.dialogs");

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");
	private JButton closeButton = new JButton(res.getString("CLOSE"));
	private String log;
	private JButton saveButton = new JButton(res.getString("SAVE"));
	private JTextArea textArea = new JTextArea();

	/**
	 *  Constructor builds gui
	 *
	 *@param  parentFrame  reference to main frame
	 *@param  log          Description of Parameter
	 *@param  title        Description of the Parameter
	 */
	public ShowLogDialog(JFrame parentFrame, String log, String title) {
		// build a modal dialog
		super(parentFrame, title, false);

		this.log = log;

		buildComponents();

		pack();
	}

	/**
	 *  Build gui elements
	 */
	private void buildComponents() {
		textArea.setLineWrap(false);
		textArea.setText(log);
		JScrollPane scrollPane =
			new JScrollPane(
				textArea,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.scrollRectToVisible(new Rectangle(new Point(0, 0)));

		ButtonListener buttonListener = new ButtonListener();
		closeButton.addActionListener(buttonListener);
		saveButton.addActionListener(buttonListener);

		JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 20, 20));
		buttonPanel.add(saveButton);
		buttonPanel.add(closeButton);

		JPanel southPanel = new JPanel();
		southPanel.add(buttonPanel);

		getContentPane().add(southPanel, "South");
		getContentPane().add(scrollPane, "Center");
	}

	/**
	 *  Display file chooser and write logging info to file
	 */
	private void saveLog() {
		String logDir = ResourceManager.getUserDefaults().getLogDirectory();

		SelectorDialog d =
			ResourceManager.getSelectorDialog(new File(logDir), null);
		d.setTitle("UNICOREpro: Save text information");
		int result = d.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			PrintWriter pw;
			try {
				pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
				pw.println(log);
				pw.close();
			} catch (IOException ioe) {
				logger.log(Level.SEVERE, "I/O error", ioe);
			}
		}
	}

	/**
	 *  Sets the editable attribute of the ShowLogDialog object
	 *
	 *@param  editable  The new editable value
	 */
	public void setEditable(boolean editable) {
		textArea.setEditable(editable);
	}

	/**
	 *  Sets the textFont attribute of the ShowLogDialog object
	 *
	 *@param  font  The new textFont value
	 */
	public void setTextFont(Font font) {
		textArea.setFont(font);
	}
}
