package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

/**
 *  Dialog to enter text in a textfield
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: TextfieldDialog.java,v 1.2 2003/06/30 06:53:24 rrate Exp $
 */
public class TextfieldDialog extends GenericDialog {

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		TextfieldDialog dialog =
			new TextfieldDialog(
				frame,
				"Enter some text, if you want: ",
				"TEST: TextfieldDialog",
				true);
		dialog.show();

	}
	private String text;
	private JTextField textField = new JTextField(20);
	private JLabel textLabel;

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title      dialog title
	 *@param  parent     dialog parent - important for modal dialogs
	 *@param  modal      true if dialog is modal
	 *@param  labelText  Description of Parameter
	 */
	public TextfieldDialog(
		JFrame parent,
		String labelText,
		String title,
		boolean modal) {
		super(parent, title, modal, OK_CANCEL);
		textLabel = new JLabel(labelText);
		buildComponents();
		pack();
	}

	/**
	 *  Takes current textfield entry as text
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean applyValues() {
		text = textField.getText();
		return true;
	}

	/**
	 *  Add a password field to the parent dialog
	 */
	private void buildComponents() {
		okButton.setEnabled(false);

		textField.addCaretListener(new CaretListener() {
			/**
			 *  Description of the Method
			 *
			 *@param  e  Description of Parameter
			 */
			public void caretUpdate(CaretEvent e) {
				caretMoved();
			}
		});

		JPanel textPanel = new JPanel();
		textPanel.add(textLabel);
		textPanel.add(textField);
		textPanel.setBorder(new EmptyBorder(20, 5, 10, 5));

		getContentPane().add(textPanel, BorderLayout.CENTER);
	}

	/**
	 *  Sets text to null
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		text = null;
		return true;
	}

	/**
	 *  Enable okButton only when textfield is not empty. Overwrite this method if you want to allow
	 *  empty inputs.
	 */
	protected void caretMoved() {
		if (textField.getText().equals("")) {
			okButton.setEnabled(false);
		} else {
			okButton.setEnabled(true);
			getRootPane().setDefaultButton(okButton);
		}
	}

	/**
	 *  Get the result from the dialog
	 *
	 *@return    null if cancel was pressed and a String if ok was pressed
	 */
	public String getText() {
		return text;
	}

	/**
	 *  Set label for textfield from outside
	 *
	 *@param  label  label for textfield
	 */
	public void setLabel(String label) {
		textLabel.setText(label);
	}

	/**
	 *  Put focus into password field and make okButton default
	 */
	public void show() {
		textField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}
}
