package com.pallas.unicore.client.dialogs;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ResourceBundle;

import javax.swing.JFrame;

/**
 *  Dialog to enter URL in a textfield and check it syntactically
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: URLDialog.java,v 1.2 2003/06/30 06:53:24 rrate Exp $
 */
public class URLDialog extends TextfieldDialog {
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");

	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();

		URLDialog dialog =
			new URLDialog(frame, "Enter Url: ", "TEST: URLDialog", true);
		dialog.show();

	}
	private URL url;

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title      dialog title
	 *@param  parent     dialog parent - important for modal dialogs
	 *@param  modal      true if dialog is modal
	 *@param  labelText  Description of Parameter
	 */
	public URLDialog(
		JFrame parent,
		String labelText,
		String title,
		boolean modal) {
		super(parent, labelText, title, modal);
	}

	/**
	 *  Takes current textfield entry and tries to build a URL out of it
	 *
	 *@return    true if URL was syntactically correct
	 */
	protected boolean applyValues() {
		try {
			url = new URL(getText());
		} catch (MalformedURLException e) {
			setStatusMessage(res.getString("SYNTAX_ERROR"));
			url = null;
			return false;
		}
		return true;
	}

	/**
	 *  Constantly check URL for syntax
	 */
	protected void caretMoved() {
		try {
			URL test = new URL(getText());
			setStatusMessage("URL ok.");
			okButton.setEnabled(true);
			getRootPane().setDefaultButton(okButton);
		} catch (MalformedURLException e) {
			setStatusMessage(res.getString("SYNTAX_ERROR"));
			okButton.setEnabled(false);
		}
	}

	/**
	 *  Get the resulting URL from the dialog
	 *
	 *@return    null if cancel was pressed and a URL if ok was pressed
	 */
	public URL getURL() {
		return url;
	}
}
