package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

/**
 *  Fetches a password and an alias for importing a user certificate to the UNICOREpro keystore
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: UserCertImportDialog.java,v 1.3 2003/06/30 06:53:24 rrate Exp $
 */
public class UserCertImportDialog extends GenericDialog {

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.dialogs.ResourceStrings");
	private String alias;
	private JTextField aliasField = new JTextField(15);
	private char[] keyPassword;
	private JPasswordField keyPasswordField = new JPasswordField(15);

	private char[] newPassword;
	private JPasswordField newPasswordField = new JPasswordField(15);

	/**
	 *  Constructor builds gui and sets a title
	 *
	 *@param  title   dialog title
	 *@param  parent  dialog parent - important for modal dialogs
	 */
	public UserCertImportDialog(JFrame parent, String title) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);

		buildComponents();
		pack();
	}

	/**
	 *  Takes current textfield entry as new password and compares it to verified password
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean applyValues() {
		// test new password
		newPassword = newPasswordField.getPassword();
		keyPassword = keyPasswordField.getPassword();

		alias = aliasField.getText().toLowerCase();

		if (newPassword.length == 0 || (alias.compareTo("") == 0)) {
			setStatusMessage(res.getString("PASSWORD_ALIAS"));
			return false;
		}

		// tests passed return ok
		return true;
	}

	/**
	 *  Add password fields to the parent dialog
	 */
	private void buildComponents() {
		JLabel newLabel = new JLabel(res.getString("ENTER_PASSWORD_DETAILED"));
		JLabel aliasLabel = new JLabel(res.getString("ENTER_ALIAS"));
		JLabel keyLabel = new JLabel(res.getString("OPTIONAL_PASSOWRD"));

		aliasField.setToolTipText(res.getString("ALIASES_LOWER_CASE"));

		JPanel labelPanel = new JPanel(new GridLayout(3, 1, 20, 20));
		labelPanel.add(newLabel);
		//	labelPanel.add(keyLabel);
		labelPanel.add(aliasLabel);

		JPanel textPanel = new JPanel(new GridLayout(3, 1, 20, 20));
		textPanel.add(newPasswordField);
		//	textPanel.add(keyPasswordField);
		textPanel.add(aliasField);

		JPanel centerPanel = new JPanel();
		centerPanel.add(labelPanel);
		centerPanel.add(textPanel);
		centerPanel.setBorder(new EmptyBorder(20, 5, 10, 5));

		getContentPane().add(centerPanel, BorderLayout.CENTER);
	}

	/**
	 *  Sets new password to null
	 *
	 *@return    true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		newPassword = null;
		alias = null;
		return true;
	}

	/**
	 *  Gets the Alias attribute of the UserCertImportDialog object
	 *
	 *@return    The Alias value
	 */
	public String getAlias() {
		return alias;
	}

	/**
	 *  Gets the KeyPassword attribute of the UserCertImportDialog object
	 *
	 *@return    The KeyPassword value
	 */
	public char[] getKeyPassword() {
		return keyPassword;
	}

	/**
	 *  Get the results from the dialog
	 *
	 *@return    null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getPassword() {
		return newPassword;
	}

	/**
	 *  Put focus into password field and make okButton default
	 */
	public void show() {
		newPasswordField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}

}
