package com.pallas.unicore.client.editor;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.logging.Level;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import org.unicore.Vsite;

import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.connection.DirectConnection;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.StringTools;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    6. Dezember 2001
 *@version    $Id: RemoteTextEditor.java,v 1.5 2003/06/30 06:53:41 rrate Exp $
 */
public class RemoteTextEditor extends TextEditor {

	/**
	 *  Thread to load a file into the text storage model
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class RemoteFileLoaderThread extends Thread {

		private RemoteFile newFile;

		/**
		 *  Constructor for the FileLoader object
		 *
		 *@param  newFile  Description of the Parameter
		 */
		RemoteFileLoaderThread(RemoteFile newFile) {
			this.newFile = newFile;
		}

		/**
		 *  Main processing method for the FileLoader object
		 */
		public void run() {
			String oldMessage = getStatusMessage();
			try {
				setStatusMessage(
					"Loading remote file: " + newFile.getAbsolutePath());

				DirectConnection connection = new DirectConnection(vsite);
				byte[] contents = connection.getRemoteFile(newFile);
				if (contents == null) {
					// if loading failed, get out
					throw new Exception(
						"Loading file "
							+ newFile.getAbsolutePath()
							+ " failed.");
				}

				// remove old document
				Document oldDoc = getEditor().getDocument();
				if (oldDoc != null) {
					oldDoc.removeUndoableEditListener(undoHandler);
				}
				getEditor().setDocument(new PlainDocument());

				textArea.setText(new String(contents));

				textArea.getDocument().addUndoableEditListener(undoHandler);

				resetUndoManager();
				file = newFile;
				setStatusMessage("File: " + file.getAbsolutePath());
			} catch (Exception e) {
				setStatusMessage(oldMessage);
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class RemoteFileWriterThread extends Thread {

		private Document doc;
		private RemoteFile file;

		/**
		 *  Constructor for the FileLoader object
		 *
		 *@param  doc   Description of Parameter
		 *@param  file  Description of Parameter
		 */
		RemoteFileWriterThread(RemoteFile file, Document doc) {
			this.file = file;
			this.doc = doc;
		}

		/**
		 *  Main processing method for the FileLoader object
		 */
		public void run() {
			boolean success = false;
			try {
				setStatusMessage("Saving file: " + file.getAbsolutePath());

				DirectConnection connection = new DirectConnection(vsite);
				success = connection.sendRemoteFile(file, getText().getBytes());
				connection.close();

			} catch (Exception e) {
				logger.log(
					Level.WARNING,
					"Saving file " + file.getAbsolutePath() + " failed!",
					e);
				success = false;
			}
			if (success) {
				setStatusMessage(
					"File: " + file.getAbsolutePath() + " successfully saved.");
			} else {
				setStatusMessage(
					"Saving file " + file.getAbsolutePath() + " failed!");
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class RemoteOpenAction extends OpenAction {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			initFileChooser();

			fileChooser.setTitle("UNICOREpro: Open file");
			int result = fileChooser.showDialog();
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}

			// load file
			File newFile = fileChooser.getSelectedFile();
			if (newFile instanceof RemoteFile) {
				RemoteFile remoteFile = (RemoteFile) newFile;
				Thread loader = new RemoteFileLoaderThread(remoteFile);
				loader.start();
			} else {
				if (newFile.exists()) {
					Thread loader = new FileLoaderThread(newFile);
					loader.start();
				}
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class RemoteSaveAction extends SaveAction {
		/**
		 *  Constructor for the RemoteSaveAsAction object
		 */
		public RemoteSaveAction() {
			this("Save");
		}

		/**
		 *  Constructor for the RemoteSaveAction object
		 *
		 *@param  name  Description of the Parameter
		 */
		public RemoteSaveAction(String name) {
			super(name);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			if (file == null || file.isDirectory()) {
				new RemoteSaveAsAction().actionPerformed(e);
				return;
			}
			save();
		}

		/**
		 *  Description of the Method
		 */
		protected void save() {
			if (file instanceof RemoteFile) {
				RemoteFile remoteFile = (RemoteFile) file;
				Thread saveThread =
					new RemoteFileWriterThread(
						remoteFile,
						textArea.getDocument());
				saveThread.start();
			} else {
				Thread saveThread =
					new FileWriterThread(file, textArea.getDocument());
				saveThread.start();
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class RemoteSaveAsAction extends RemoteSaveAction {
		/**
		 *  Constructor for the RemoteSaveAsAction object
		 */
		RemoteSaveAsAction() {
			super("Save As...");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			initFileChooser();
			fileChooser.setTitle("UNICOREpro: Save file as");
			int result = fileChooser.showDialog();
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}

			file = fileChooser.getSelectedFile();

			save();
		}
	}
	private File defaultDir;

	private SelectorDialog fileChooser;
	private Vsite vsite;

	/**
	 *  Description of the Method
	 */
	protected void buildActions() {
		newAction = new NewAction();
		openAction = new RemoteOpenAction();
		saveAction = new RemoteSaveAction();
		saveAsAction = new RemoteSaveAsAction();
		cutAction = new CutAction();
		copyAction = new CopyAction();
		pasteAction = new PasteAction();
		undoAction = new UndoAction();
		redoAction = new RedoAction();

	}

	/**
	 *  Gets the DefaultDir attribute of the RemoteTextEditor object
	 *
	 *@return    The DefaultDir value
	 */
	public File getDefaultDir() {
		return defaultDir;
	}

	/**
	 *  Gets the text from the text area
	 *
	 *@return    The text
	 */
	public String getText() {
		return textArea.getText();
	}

	/**
	 *  Gets the text from the text area without windows/dos style characters
	 *  Take care: Using getText() will perform much better for large files.
	 *
	 *@return        The unix style text
	 *@deprecated    Performance is too bad for large files
	 */
	public String getUnixStyleText() {
		return StringTools.dos2Unix(textArea.getText());
	}

	/**
	 *  Gets the Vsite attribute of the RemoteTextEditor object
	 *
	 *@return    The Vsite value
	 */
	public Vsite getVsite() {
		return vsite;
	}

	/**
	 *  Description of the Method
	 */
	private void initFileChooser() {
		File dir = file;
		if (dir == null) {
			dir = defaultDir;
		}

		// always build a new file chooser to update vsite
		fileChooser = ResourceManager.getSelectorDialog(dir, vsite);
		fileChooser.removeFileFilters();
		for (int i = 0; i < fileFilters.size(); i++) {
			fileChooser.addFileFilter((FileFilter) fileFilters.elementAt(i));
		}
		if (selectedFileFilter != null) {
			fileChooser.setSelectedFileFilter(selectedFileFilter);
		}
	}

	/**
	 *  Sets the DefaultDir attribute of the RemoteTextEditor object
	 *
	 *@param  defaultDir  The new DefaultDir value
	 */
	public void setDefaultDir(File defaultDir) {
		this.defaultDir = defaultDir;
	}

	/**
	 *  Sets the Vsite attribute of the RemoteTextEditor object
	 *
	 *@param  vsite  The new Vsite value
	 */
	public void setVsite(Vsite vsite) {
		this.vsite = vsite;
	}
}
