package com.pallas.unicore.client.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Reader;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    6. Dezember 2001
 *@version    $Id: TextEditor.java,v 1.5 2003/06/30 06:53:41 rrate Exp $
 */
public class TextEditor extends JPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Mai 2002
	 */
	private class ActionChangedListener implements PropertyChangeListener {
		JMenuItem menuItem;

		/**
		 *  Constructor for the ActionChangedListener object
		 *
		 *@param  mi  Description of Parameter
		 */
		ActionChangedListener(JMenuItem mi) {
			super();
			this.menuItem = mi;
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void propertyChange(PropertyChangeEvent e) {
			String propertyName = e.getPropertyName();
			if (e.getPropertyName().equals(Action.NAME)) {
				String text = (String) e.getNewValue();
				menuItem.setText(text);
			} else if (propertyName.equals("enabled")) {
				Boolean enabledState = (Boolean) e.getNewValue();
				menuItem.setEnabled(enabledState.booleanValue());
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class CopyAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public CopyAction() {
			super("Copy");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			textArea.copy();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class CutAction extends AbstractAction {
		/**
		 *  Constructor for the CutAction object
		 */
		public CutAction() {
			super("Cut");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			textArea.cut();
		}
	}

	/**
	 *  Thread to load a file into the text storage model
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class FileLoaderThread extends Thread {

		private File newFile;

		/**
		 *  Constructor for the FileLoader object
		 *
		 *@param  newFile  Description of the Parameter
		 */
		FileLoaderThread(File newFile) {
			this.newFile = newFile;
		}

		/**
		 *  Main processing method for the FileLoader object
		 */
		public void run() {
			String oldMessage = getStatusMessage();
			try {
				setStatusMessage("Loading file: " + newFile.getAbsolutePath());

				// remove old document
				Document oldDoc = getEditor().getDocument();
				if (oldDoc != null) {
					oldDoc.removeUndoableEditListener(undoHandler);
				}
				getEditor().setDocument(new PlainDocument());

				Document doc = textArea.getDocument();

				// try to start reading
				Reader in = new FileReader(newFile);
				char[] buff = new char[4096];
				int nch;
				while ((nch = in.read(buff, 0, buff.length)) != -1) {
					doc.insertString(
						doc.getLength(),
						new String(buff, 0, nch),
						null);
				}

				// we are done... get rid of progressbar
				doc.addUndoableEditListener(undoHandler);

				resetUndoManager();
				file = newFile;
				setStatusMessage("File: " + file.getAbsolutePath());

			} catch (Exception e) {
				setStatusMessage(oldMessage);
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class FileWriterThread extends Thread {

		private Document doc;
		private File file;

		/**
		 *  Constructor for the FileLoader object
		 *
		 *@param  doc   Description of Parameter
		 *@param  file  Description of Parameter
		 */
		FileWriterThread(File file, Document doc) {
			this.file = file;
			this.doc = doc;
		}

		/**
		 *  Main processing method for the FileLoader object
		 */
		public void run() {
			try {
				setStatusMessage("Saving file: " + file.getAbsolutePath());
				BufferedWriter bw = new BufferedWriter(new FileWriter(file));
				bw.write(textArea.getText());
				bw.close();
				setStatusMessage(
					"File: " + file.getAbsolutePath() + " successfully saved.");
			} catch (Exception e) {
				setStatusMessage(
					"Saving file "
						+ file.getAbsolutePath()
						+ " failed! Reason: "
						+ e.getMessage());
			}

		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class NewAction extends AbstractAction {

		/**
		 *  Constructor for the NewAction object
		 */
		NewAction() {
			super("New");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			Document oldDoc = getEditor().getDocument();
			if (oldDoc != null) {
				oldDoc.removeUndoableEditListener(undoHandler);
			}
			getEditor().setDocument(new PlainDocument());
			getEditor().getDocument().addUndoableEditListener(undoHandler);
			resetUndoManager();
			file = null;
			setStatusMessage("New File");

			revalidate();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class OpenAction extends AbstractAction {

		/**
		 *  Constructor for the OpenAction object
		 */
		OpenAction() {
			super("Open...");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			initFileChooser();

			int result = fileChooser.showOpenDialog(getFrame());
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}

			File newFile = fileChooser.getSelectedFile();
			if (newFile.exists()) {
				Thread loader = new FileLoaderThread(newFile);
				loader.start();
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class PasteAction extends AbstractAction {
		/**
		 *  Constructor for the PasteAction object
		 */
		public PasteAction() {
			super("Paste");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			textArea.paste();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    3. Juni 2002
	 */
	private class PopupMouseListener extends MouseAdapter {

		/**
		 *  Description of the Method
		 *
		 *@param  evt  Description of Parameter
		 */
		public void mouseClicked(MouseEvent evt) {
			testShowPopup(evt);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  evt  Description of Parameter
		 */
		public void mousePressed(MouseEvent evt) {
			testShowPopup(evt);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  evt  Description of Parameter
		 */
		public void mouseReleased(MouseEvent evt) {
			testShowPopup(evt);
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class RedoAction extends AbstractAction {
		/**
		 *  Constructor for the RedoAction object
		 */
		public RedoAction() {
			super("Redo");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				undo.redo();
			} catch (CannotRedoException ex) {
				logger.log(Level.WARNING, "Unable to redo.", ex);
			}
			update();
			undoAction.update();
		}

		/**
		 *  Description of the Method
		 */
		protected void update() {
			if (undo.canRedo()) {
				setEnabled(true);
				putValue(Action.NAME, undo.getRedoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Redo");
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class SaveAction extends AbstractAction {

		/**
		 *  Constructor for the OpenAction object
		 */
		SaveAction() {
			this("Save");
		}

		/**
		 *  Constructor for the SaveAction object
		 *
		 *@param  name  Description of Parameter
		 */
		SaveAction(String name) {
			super(name);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			if (file == null || file.isDirectory()) {
				new SaveAsAction().actionPerformed(e);
				return;
			}
			if (!file.getParentFile().exists()) {
				setStatusMessage(
					"Directory for file "
						+ file.getAbsolutePath()
						+ " does not exist!");
				return;
			}
			save();
		}

		/**
		 *  Description of the Method
		 */
		protected void save() {
			Thread saveThread =
				new FileWriterThread(file, textArea.getDocument());
			saveThread.start();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class SaveAsAction extends SaveAction {

		/**
		 *  Constructor for the OpenAction object
		 */
		SaveAsAction() {
			super("Save As...");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			initFileChooser();

			int result = fileChooser.showSaveDialog(getFrame());
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}

			file = fileChooser.getSelectedFile();

			save();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	class StatusBar extends JPanel {
		private JLabel label = new JLabel();

		/**
		 *  Constructor for the StatusBar object
		 */
		public StatusBar() {
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		}

		/**
		 *  Gets the text attribute of the StatusBar object
		 *
		 *@return    The text value
		 */
		public String getText() {
			return label.getText();
		}

		/**
		 *  Sets the Text attribute of the StatusBar object
		 *
		 *@param  text  The new Text value
		 */
		public void setText(String text) {
			label.setText(text);
			label.setToolTipText(text);
			removeAll();
			add(label);
			revalidate();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class UndoAction extends AbstractAction {
		/**
		 *  Constructor for the UndoAction object
		 */
		public UndoAction() {
			super("Undo");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				undo.undo();
			} catch (CannotUndoException ex) {
				logger.log(Level.WARNING, "Unable to undo.", ex);
			}
			update();
			redoAction.update();
		}

		/**
		 *  Description of the Method
		 */
		protected void update() {
			if (undo.canUndo()) {
				setEnabled(true);
				putValue(Action.NAME, undo.getUndoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Undo");
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Dezember 2001
	 */
	class UndoHandler implements UndoableEditListener {

		/**
		 *  Messaged when the Document has created an edit, the edit is added to
		 *  <code>undo</code> , an instance of UndoManager.
		 *
		 *@param  e  Description of Parameter
		 */
		public void undoableEditHappened(UndoableEditEvent e) {
			undo.addEdit(e.getEdit());
			undoAction.update();
			redoAction.update();
		}
	}
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.editor");

	/**
	 *  The main program for the TextEditor class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setTitle("Text Editor");
		frame.setBackground(Color.lightGray);
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add("Center", new TextEditor());
		frame.addWindowListener(new WindowAdapter() {
			/**
			 *  Description of the Method
			 *
			 *@param  e  Description of Parameter
			 */
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.pack();
		frame.setSize(500, 600);
		frame.show();
	}
	/**
	 *  Description of the Field
	 */
	protected CopyAction copyAction;
	/**
	 *  Description of the Field
	 */
	protected CutAction cutAction;

	/**
	 *  edit menu entries
	 */
	protected JMenuItem cutItem, copyItem, pasteItem, undoItem, redoItem;

	/**
	 *  Description of the Field
	 */
	protected File file;

	/**
	 *  Description of the Field
	 */
	protected JFileChooser fileChooser;

	/**
	 *  Description of the Field
	 */
	protected Vector fileFilters = new Vector();

	/**
	 *  Description of the Field
	 */
	protected NewAction newAction;

	/**
	 *  file menu entries
	 */
	protected JMenuItem newItem, openItem, saveItem, saveAsItem;
	/**
	 *  Description of the Field
	 */
	protected OpenAction openAction;
	/**
	 *  Description of the Field
	 */
	protected PasteAction pasteAction;

	private JPopupMenu popup;
	/**
	 *  Description of the Field
	 */
	protected RedoAction redoAction;
	/**
	 *  Description of the Field
	 */
	protected SaveAction saveAction;
	/**
	 *  keep saveAs of type save
	 */
	protected SaveAction saveAsAction;

	/**
	 *  Description of the Field
	 */
	protected FileFilter selectedFileFilter;

	private StatusBar statusBar = new StatusBar();

	/**
	 *  Description of the Field
	 */
	protected JTextArea textArea;

	/**
	 *  UndoManager that we add edits to.
	 */
	protected UndoManager undo = new UndoManager();
	/**
	 *  Description of the Field
	 */
	protected UndoAction undoAction;
	/**
	 *  Listener for the edits on the current document.
	 */
	protected UndoableEditListener undoHandler = new UndoHandler();

	/**
	 *  Constructor for the TextEditor object
	 */
	public TextEditor() {
		buildActions();

		buildComponents();

		setStatusMessage("UNICOREpro TextEditor");
	}

	/**
	 *  Sets the fileFilter attribute of the TextEditor object
	 *
	 *@param  filter  The new fileFilter value
	 */
	public void addFileFilter(FileFilter filter) {
		fileFilters.add(filter);
	}

	/**
	 *  Constructor for the buildActions object
	 */
	protected void buildActions() {
		newAction = new NewAction();
		openAction = new OpenAction();
		saveAction = new SaveAction();
		saveAsAction = new SaveAsAction();
		cutAction = new CutAction();
		copyAction = new CopyAction();
		pasteAction = new PasteAction();
		undoAction = new UndoAction();
		redoAction = new RedoAction();

	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {

		// create the embedded JTextComponent
		textArea = new JTextArea();
		setEditorFont(new Font("monospaced", Font.PLAIN, 12));
		// Add this as a listener for undoable edits.
		textArea.getDocument().addUndoableEditListener(undoHandler);
		textArea.addMouseListener(new PopupMouseListener());

		JScrollPane scrollPane = new JScrollPane(textArea);

		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEtchedBorder());
		add(buildMenubar(), BorderLayout.NORTH);
		add(scrollPane, BorderLayout.CENTER);
		add(statusBar, BorderLayout.SOUTH);

		buildPopupMenu();

	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected JMenuBar buildMenubar() {
		// file menu
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');

		newItem = new JMenuItem(newAction);
		newItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		fileMenu.add(newItem);

		openItem = new JMenuItem(openAction);
		openItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		fileMenu.add(openItem);

		saveItem = new JMenuItem(saveAction);
		saveItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		fileMenu.add(saveItem);

		saveAsItem = new JMenuItem(saveAsAction);
		fileMenu.add(saveAsItem);

		// edit menu
		JMenu editMenu = new JMenu("Edit");
		editMenu.setMnemonic('E');

		cutItem = new JMenuItem(cutAction);
		cutItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		editMenu.add(cutItem);

		copyItem = new JMenuItem(copyAction);
		copyItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		editMenu.add(copyItem);

		pasteItem = new JMenuItem(pasteAction);
		pasteItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
		editMenu.add(pasteItem);

		editMenu.addSeparator();

		undoItem = new JMenuItem(undoAction);
		undoItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK));
		editMenu.add(undoItem);

		redoItem = new JMenuItem(redoAction);
		redoItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK));
		editMenu.add(redoItem);

		JMenuBar menubar = new JMenuBar();
		menubar.add(fileMenu);
		menubar.add(editMenu);

		return menubar;
	}

	/**
	 *  Description of the Method
	 */
	private void buildPopupMenu() {
		popup = new JPopupMenu();
		popup.setInvoker(textArea);
		popup.add(cutAction);
		popup.add(copyAction);
		popup.add(pasteAction);
		popup.addSeparator();
		popup.add(undoAction);
		popup.add(redoAction);
	}

	/**
	 *  Fetch the editor contained in this panel
	 *
	 *@return    The Editor value
	 */
	protected JTextComponent getEditor() {
		return textArea;
	}

	/**
	 *  Gets the editorFont attribute of the TextEditor object
	 *
	 *@return    The editorFont value
	 */
	public Font getEditorFont() {
		return textArea.getFont();
	}

	/**
	 *  Gets the File attribute of the TextEditor object
	 *
	 *@return    The File value
	 */
	public File getFile() {
		return file;
	}

	/**
	 *  Find the hosting frame, for the file-chooser dialog.
	 *
	 *@return    The Frame value
	 */
	protected Frame getFrame() {
		for (Container p = getParent(); p != null; p = p.getParent()) {
			if (p instanceof Frame) {
				return (Frame) p;
			}
		}
		return null;
	}

	/**
	 *  Gets the lineWrap attribute of the TextEditor object
	 *
	 *@return    The lineWrap value
	 */
	public boolean getLineWrap() {
		return textArea.getLineWrap();
	}

	/**
	 *  Gets the statusMessage attribute of the TextEditor object
	 *
	 *@return    The statusMessage value
	 */
	protected String getStatusMessage() {
		return statusBar.getText();
	}

	/**
	 *  Gets the Text attribute of the TextEditor object
	 *
	 *@return    The Text value
	 */
	public String getText() {
		return textArea.getText();
	}

	/**
	 *  Description of the Method
	 */
	private void initFileChooser() {
		// only build the file chooser once...
		if (fileChooser == null) {
			// if we have a filename already use it's directory to start
			if (file != null) {
				if (file.exists()) {
					fileChooser = new JFileChooser(file.getParent());
				}
			}
			fileChooser = new JFileChooser();
			fileChooser.setMultiSelectionEnabled(false);
			fileChooser.setFileHidingEnabled(false);
			for (int i = 0; i < fileFilters.size(); i++) {
				fileChooser.addChoosableFileFilter(
					(FileFilter) fileFilters.elementAt(i));
			}
			if (selectedFileFilter != null) {
				fileChooser.setFileFilter(selectedFileFilter);
			}
		}
	}

	/**
	 *  Helper method initializes menu items
	 *
	 *@param  item    Description of Parameter
	 *@param  action  Description of Parameter
	 */
	protected void initMenuItem(JMenuItem item, Action action) {
		item.addActionListener(action);
		action.addPropertyChangeListener(new ActionChangedListener(item));
	}

	/**
	 *  Helper method initializing menu item including a CTRL-Shortcut
	 *
	 *@param  item     Description of Parameter
	 *@param  keyCode  Description of Parameter
	 *@param  action   Description of Parameter
	 */
	protected void initMenuItem(JMenuItem item, Action action, int keyCode) {
		item.setAccelerator(
			KeyStroke.getKeyStroke(keyCode, ActionEvent.CTRL_MASK));
		initMenuItem(item, action);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  filter  Description of the Parameter
	 */
	public void removeFileFilter(FileFilter filter) {
		fileFilters.remove(filter);
	}

	/**
	 *  Resets the undo manager.
	 */
	protected void resetUndoManager() {
		undo.discardAllEdits();
		undoAction.update();
		redoAction.update();
	}

	/**
	 *  Sets the EditorFont attribute of the TextEditor object
	 *
	 *@param  font  The new EditorFont value
	 */
	public void setEditorFont(Font font) {
		textArea.setFont(font);
	}

	/**
	 *  Sets the File attribute of the TextEditor object
	 *
	 *@param  file  The new File value
	 */
	public void setFile(File file) {
		this.file = file;
		if (file != null && file.exists() && !file.isDirectory()) {
			setStatusMessage("File: " + file.getAbsolutePath());
		} else {
			setStatusMessage("New File");
		}
	}

	/**
	 *  Sets the fileFilter attribute of the TextEditor object
	 *
	 *@param  selectedFileFilter  The new fileFilter value
	 */
	public void setFileFilter(FileFilter selectedFileFilter) {
		this.selectedFileFilter = selectedFileFilter;
	}

	/**
	 *  Sets the lineWrapping attribute of the TextEditor object
	 *
	 *@param  wrap  The new lineWrapping value
	 */
	public void setLineWrap(boolean wrap) {
		textArea.setLineWrap(wrap);
	}

	/**
	 *  Sets the statusMessage attribute of the TextEditor object
	 *
	 *@param  message  The new statusMessage value
	 */
	protected void setStatusMessage(String message) {
		statusBar.setText(message);
	}

	/**
	 *  Sets the Text attribute of the TextEditor object
	 *
	 *@param  text  The new Text value
	 */
	public void setText(String text) {
		textArea.setText(text);
	}

	/**
	 *@param  evt  Description of Parameter
	 */
	private void testShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			popup.show(textArea, evt.getX(), evt.getY());
		}
	}

}
