package com.pallas.unicore.client.explorer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.util.Arrays;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: DirNode.java,v 1.2 2003/06/30 06:53:51 rrate Exp $
 */
public class DirNode extends DefaultMutableTreeNode implements Transferable {
	private static DataFlavor flavor;
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.explorer");

	/**
	 *  Gets the Flavor attribute of the PanelNode class
	 *
	 *@return    The Flavor value
	 */
	public static DataFlavor getFlavor() {
		return flavor;
	}
	private Icon icon;
	private File[] listing;
	private boolean listingEnabled = false;
	private DataFlavor[] supportedFlavors = { flavor };
	private boolean upToDate = false;
	private FileSystemView view;

	/**
	 *  Constructor for the DirNode object
	 *
	 *@param  file   Description of the Parameter
	 *@param  _view  Description of the Parameter
	 */
	public DirNode(File file, FileSystemView _view) {
		super(file);
		view = _view;

		// per default do not list floppy drives
		listingEnabled = !view.isFloppyDrive(file);

		icon = view.getSystemIcon(file);
		try {
			flavor =
				new DataFlavor(
					Class.forName("com.pallas.unicore.client.explorer.DirNode"),
					"DirNode");
		} catch (ClassNotFoundException e) {
			logger.log(Level.SEVERE, "Did not find DirNode class.", e);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public boolean equals(Object object) {
		DirNode node2 = (DirNode) object;
		return getUserObject().equals(node2.getUserObject());
	}

	/**
	 *  Gets the file attribute of the DirNode object
	 *
	 *@return    The file value
	 */
	public File getDir() {
		return (File) getUserObject();
	}

	/**
	 *  Gets the icon attribute of the DirNode object
	 *
	 *@return    The icon value
	 */
	public Icon getIcon() {
		return icon;
	}

	/**
	 *  Gets the listing attribute of the ExplorerTree object
	 *
	 *@return    The listing value
	 */
	public File[] getListing() {
		if (listing == null) {
			File file = (File) userObject;
			if (listingEnabled) {
				listing = view.getFiles(file, false);
				Arrays.sort(listing);
			} else {
				logger.info("NO LISTING FOR: " + file);
				listing = new File[0];
			}
		}
		return listing;
	}

	/**
	 *  Transferable interface:
	 *
	 *@param  flavor                          Description of Parameter
	 *@return                                 The TransferData value
	 *@exception  UnsupportedFlavorException  Description of Exception
	 */
	public synchronized Object getTransferData(DataFlavor flavor)
		throws UnsupportedFlavorException {

		if (flavor.equals(flavor)) {
			return this;
		} else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	/**
	 *  Transferable interface:
	 *
	 *@return    The TransferDataFlavors value
	 */
	public synchronized DataFlavor[] getTransferDataFlavors() {
		return (supportedFlavors);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public int hashCode() {
		return getUserObject().hashCode();
	}

	/**
	 *  Transferable interface:
	 *
	 *@param  flavor  Description of Parameter
	 *@return         The DataFlavorSupported value
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return (flavor.equals(flavor));
	}

	/**
	 *  Gets the listingEnabled attribute of the DirNode object
	 *
	 *@return    The listingEnabled value
	 */
	public boolean isListingEnabled() {
		return listingEnabled;
	}

	/**
	 *  Gets the upToDate attribute of the DirNode object
	 *
	 *@return    The upToDate value
	 */
	public boolean isUpToDate() {
		return upToDate;
	}

	/**
	 *  Sets the listing attribute of the DirNode object
	 *
	 *@param  listing  The new listing value
	 */
	public void setListing(File[] listing) {
		this.listing = listing;
	}

	/**
	 *  Sets the listing attribute of the DirNode object
	 *
	 *@param  listVector  The new listing value
	 */
	public void setListing(Vector listVector) {
		this.listing = new File[listVector.size()];
		for (int i = 0; i < listVector.size(); i++) {
			listing[i] = (File) listVector.elementAt(i);
		}
	}

	/**
	 *  Constructor for the setListingEnabled object
	 *
	 *@param  listingEnabled  Description of the Parameter
	 */
	public void setListingEnabled(boolean listingEnabled) {
		this.listingEnabled = listingEnabled;
	}

	/**
	 *  Sets the upToDate attribute of the DirNode object
	 *
	 *@param  upToDate  The new upToDate value
	 */
	public void setUpToDate(boolean upToDate) {
		this.upToDate = upToDate;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		return view.getSystemDisplayName(getDir());
	}

}
