package com.pallas.unicore.client.explorer;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import javax.swing.table.*;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: ExplorerPanel.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class ExplorerPanel extends JPanel {
    private JTextField nameField;
    private ExplorerTree tree;
    private ExplorerTable table;
    private StatusBar statusBar;

    private JSplitPane splitPane;
    private File currentDir;


    /**
     *  Constructor for the ExplorerPanel object
     */
    public ExplorerPanel() {
        this(null);
    }


    /**
     *  Constructor for the ExplorerPanel object
     *
     *@param  currentDir  Description of the Parameter
     */
    public ExplorerPanel(File currentDir) {
        this.currentDir = currentDir;
        buildComponents();
    }


    /**
     *  Sets the dividerLocation attribute of the ExplorerPanel object
     *
     *@param  value  The new dividerLocation value
     */
    public void setDividerLocation(double value) {
        splitPane.setDividerLocation(value);
    }


    /**
     *  Description of the Method
     */
    private void buildComponents() {
        // build tree
        tree = new ExplorerTree(currentDir);
        tree.addTreeSelectionListener(new TreeListener());

        // get initial node from tree
        DirNode currentNode = tree.getSelectedNode();
        File[] listing = currentNode.getListing();

        // build table
        table = new ExplorerTable();
        table.getTableHeader().addMouseListener(new TableHeaderListener());
        table.addMouseListener(new TableListener());
        table.setListing(listing);

        JScrollPane tablePane = new JScrollPane(table);
        // build status bar
        statusBar = new StatusBar();
        statusBar.setText("Objects: " + listing.length);

        JPanel rightPanel = new JPanel(new BorderLayout());
        rightPanel.add(tablePane, BorderLayout.CENTER);
        rightPanel.add(statusBar, BorderLayout.SOUTH);

        // build name field
        nameField = new JTextField(currentNode.getDir().getAbsolutePath());

        JPanel southPanel = new JPanel(new BorderLayout());
        southPanel.add(new JLabel("Filename:"), BorderLayout.WEST);
        southPanel.add(nameField, BorderLayout.CENTER);

        JScrollPane leftPane = new JScrollPane(tree);

        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPane, rightPanel);
        setLayout(new BorderLayout());
        add(splitPane, BorderLayout.CENTER);
        add(southPanel, BorderLayout.SOUTH);
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    16. Mai 2002
     */
    private class TreeListener implements TreeSelectionListener {
        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void valueChanged(final TreeSelectionEvent e) {
            if(tree.isDragging()) {
                return;
            }
            final JRootPane rootPane = getRootPane();
            final DirNode node = tree.getSelectedNode();
            String path = node.getDir().getAbsolutePath();
            nameField.setText(path);

            if(rootPane != null) {
                if(node.isUpToDate()) {
                    statusBar.setText("Building file table for: " + path);
                }
                else {
                    statusBar.setText("Scanning directory " + path);
                }
                rootPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

            }
            Runnable runner =
                new Runnable() {
                    public void run() {
                        File[] listing = node.getListing();
                        table.setListing(listing);
                        if(rootPane != null) {
                            rootPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                        }
                        statusBar.setText("Objects: " + listing.length);
                    }
                };

            SwingUtilities.invokeLater(runner);

        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    21. Mai 2002
     */
    private class TableListener extends MouseAdapter {
        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void mouseClicked(MouseEvent e) {
            File file = table.getSelectedFile();
            if(e.getClickCount() == 1) {
                nameField.setText(file.getAbsolutePath());
            }
            else if(e.getClickCount() > 1) {
                if(file.isDirectory()) {
                    tree.setSelectedObject(file);
                }
            }
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    22. Mai 2002
     */
    private class TableHeaderListener extends MouseAdapter {
        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void mouseClicked(MouseEvent e) {
            TableColumnModel columnModel = table.getColumnModel();
            int viewColumn = columnModel.getColumnIndexAtX(e.getX());
            int column = table.convertColumnIndexToModel(viewColumn);
            table.sortByColumn(column);
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    6. Dezember 2001
     */
    private class StatusBar extends JPanel {
        private JLabel label = new JLabel();


        /**
         *  Constructor for the StatusBar object
         */
        public StatusBar() {
            buildComponents();
        }


        /**
         *  Sets the Text attribute of the StatusBar object
         *
         *@param  text  The new Text value
         */
        public void setText(String text) {
            label.setText(text);
            revalidate();
        }


        /**
         *  Constructor for the buildComponents object
         */
        private void buildComponents() {
            setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
            add(label);
        }
    }
}
