package com.pallas.unicore.client.explorer;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: ExplorerTree.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class ExplorerTree extends SelectorTree {
    private boolean dragging = false;
    private CutAction cutAction = new CutAction();
    private CopyAction copyAction = new CopyAction();
    private PasteAction pasteAction = new PasteAction();
    private RemoveAction removeAction = new RemoveAction();
    private JPopupMenu popup;


    /**
     *  Constructor for the ExplorerTree object
     */
   /* public ExplorerTree() {
        this(null);
    }*/


    /**
     *  Constructor for the ExplorerTree object
     *
     *@param  currentDir  Description of the Parameter
     */
    public ExplorerTree(File currentDir) {
        super(currentDir, true);
        setDragEnabled(true);

        setDropTarget(new DropTarget(this, new DropListener()));
        setTransferHandler(new TreeTransferHandler());
        buildPopupMenu();
        addMouseListener(new MouseListener());
    }


    /**
     *  Gets the dragging attribute of the ExplorerTree object
     *
     *@return    The dragging value
     */
    public boolean isDragging() {
        return dragging;
    }


    /**
     *  Description of the Method
     */
    private void buildPopupMenu() {
        popup = new JPopupMenu();
        popup.setInvoker(this);

        JMenuItem cutItem = new JMenuItem();
        cutItem.setAction(cutAction);
        popup.add(cutItem);

        JMenuItem copyItem = new JMenuItem();
        copyItem.setAction(copyAction);
        popup.add(copyItem);

        JMenuItem pasteItem = new JMenuItem();
        pasteItem.setAction(pasteAction);
        popup.add(pasteItem);

        JMenuItem removeItem = new JMenuItem();
        removeItem.setAction(removeAction);
        popup.add(removeItem);

    }


    /**
     *@param  evt  Description of Parameter
     */
    private void testShowPopup(MouseEvent evt) {
        if(evt.isPopupTrigger()) {
            popup.show(this, evt.getX(), evt.getY());
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    24. Mai 2002
     */
    class CutAction extends AbstractAction {
        /**
         *  Constructor for the CopyAction object
         */
        public CutAction() {
            super("Cut");
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void actionPerformed(ActionEvent e) {
            System.err.println("CUTTING ");
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    24. Mai 2002
     */
    class CopyAction extends AbstractAction {
        /**
         *  Constructor for the CopyAction object
         */
        public CopyAction() {
            super("Copy");
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void actionPerformed(ActionEvent e) {
            System.err.println("COPYING ");
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    24. Mai 2002
     */
    class PasteAction extends AbstractAction {
        /**
         *  Constructor for the CopyAction object
         */
        public PasteAction() {
            super("Paste");
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void actionPerformed(ActionEvent e) {
            System.err.println("PASTING ");
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    24. Mai 2002
     */
    class RemoveAction extends AbstractAction {
        /**
         *  Constructor for the CopyAction object
         */
        public RemoveAction() {
            super("Remove");
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void actionPerformed(ActionEvent e) {
            System.err.println("REMOVING ");
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    May 31, 2002
     */
    private class MouseListener extends MouseAdapter {

        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void mousePressed(MouseEvent e) {
//            JComponent c = (JComponent)e.getSource();
            //          TransferHandler th = c.getTransferHandler();
            //        th.exportAsDrag(c, e, TransferHandler.MOVE);
            testShowPopup(e);
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void mouseClicked(MouseEvent e) {
            testShowPopup(e);
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    23. Mai 2002
     */
    private class DropListener extends DropTargetAdapter {
        private ExplorerTree tree;
        private TreePath currentPath;


        /**
         *  Gets the pathAt attribute of the DropListener object
         *
         *@param  p  Description of the Parameter
         *@return    The pathAt value
         */
        private TreePath getPathAt(Point p) {
            return tree.getClosestPathForLocation(p.x, p.y);
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void drop(DropTargetDropEvent e) {
            dragging = false;
            tree = (ExplorerTree)e.getDropTargetContext().getComponent();
            Transferable transferable = (Transferable)e.getTransferable();
            TransferHandler th = tree.getTransferHandler();
            System.err.println("TH: " + th.getCopyAction());
            DirNode node = (DirNode)currentPath.getLastPathComponent();
            System.err.println("YOU DROPPED SOMETHING ONTO: " + node);
        }


        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void dragOver(DropTargetDragEvent e) {
            dragging = true;

            tree = (ExplorerTree)e.getDropTargetContext().getComponent();
            TreePath path = getPathAt(e.getLocation());
            //System.err.println("YOU'RE DRAGGING OVER: " + currentPath);
            if(!path.equals(currentPath)) {
                tree.setSelectionPath(path);
                currentPath = path;
            }
        }


        /**
         *  Description of the Method
         *
         *@param  dtde  Description of the Parameter
         */
        public void dragEnter(DropTargetDragEvent e) {
            System.err.println("DRAG ENTERED: " + e);
        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    3. Juni 2002
     */
    private class TreeTransferHandler extends TransferHandler {

        /**
         *  Gets the visualRepresentation attribute of the TreeTransferHandler
         *  object
         *
         *@param  t  Description of the Parameter
         *@return    The visualRepresentation value
         */
        public Icon getVisualRepresentation(Transferable t) {
            DirNode node = (DirNode)t;
            System.err.println("NOOOOOODE: " + node);
            return node.getIcon();
        }


        /**
         *  Description of the Method
         *
         *@param  comp    Description of the Parameter
         *@param  e       Description of the Parameter
         *@param  action  Description of the Parameter
         */
        public void exportAsDrag(JComponent comp,
                InputEvent e,
                int action) {

            System.err.println("EXPORT AS DRAG COMP: " + comp);
            //          System.err.println("EXPORT AS DRAG EVT : " + e);
            //        System.err.println("EXPORT AS DRAG ACT : " + action);

        }


        /**
         *  Description of the Method
         *
         *@param  c  Description of the Parameter
         *@return    Description of the Return Value
         */
        protected Transferable createTransferable(JComponent c) {
            return getSelectedNode();
        }
    }

}
