package com.pallas.unicore.client.explorer;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.SoftBevelBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;
import javax.swing.table.TableColumnModel;

import com.pallas.unicore.client.controls.GenericFileFilter;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: SelectorPanel.java,v 1.3 2003/06/30 06:53:48 rrate Exp $
 */
public class SelectorPanel extends JPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. August 2002
	 */
	class FilterListener implements ItemListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			table.setFileFilter((FileFilter) filterBox.getSelectedItem());
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    July 22, 2002
	 */
	class RefreshAction extends AbstractAction {
		/**
		 *  Constructor for the RefreshAction object
		 */
		RefreshAction() {
			super("Refresh");
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			refresh();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    6. Dezember 2001
	 */
	protected class StatusBar extends JPanel {
		private JLabel label = new JLabel();

		/**
		 *  Constructor for the StatusBar object
		 */
		public StatusBar() {
			buildComponents();
		}

		/**
		 *  Constructor for the buildComponents object
		 */
		private void buildComponents() {
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			add(label);
		}

		/**
		 *  Sets the Text attribute of the StatusBar object
		 *
		 *@param  text  The new Text value
		 */
		public void setText(String text) {
			label.setText(text);
			super.revalidate();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    22. Mai 2002
	 */
	protected class TableHeaderListener extends MouseAdapter {
		/**
		 *  Constructor for the TableHeaderListener object
		 */
		public TableHeaderListener() {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			TableColumnModel columnModel = table.getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			int column = table.convertColumnIndexToModel(viewColumn);
			table.sortByColumn(column);
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    21. Mai 2002
	 */
	protected class TableMouseListener extends MouseAdapter {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			selectedFile = table.getSelectedFile();
			if (e.getClickCount() > 1) {
				if (selectedFile.isDirectory()) {
					tree.setSelectedObject(selectedFile);
				} else {
					firePropertyChange(
						FILE_SELECTED_PROPERTY,
						null,
						selectedFile);
				}
			}
		}
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void mousePressed(MouseEvent e) {
			selectedFile = table.getSelectedFile();
			if (selectedFile instanceof RemoteFile) {
				nameField.setText(
					((RemoteFile) selectedFile).getSubspacePath());
			} else {
				nameField.setText(selectedFile.getAbsolutePath());
			}
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mouseReleased(MouseEvent e) {
			selectedFile = table.getSelectedFile();
			if (selectedFile instanceof RemoteFile) {
				nameField.setText(
					((RemoteFile) selectedFile).getSubspacePath());
			} else {
				nameField.setText(selectedFile.getAbsolutePath());
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    7. August 2002
	 */
	protected class TreeListener implements TreeSelectionListener {
		/**
		 *  Constructor for the TreeListener object
		 */
		public TreeListener() {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void valueChanged(TreeSelectionEvent e) {
			updateNodeSelection();
		}
	}
	/**
	 *  Description of the Field
	 */
	public final static GenericFileFilter DEFAULT_FILTER =
		new GenericFileFilter();

	/**
	 *  Description of the Field
	 */
	public final static String FILE_SELECTED_PROPERTY = "FILE_SELECTED";
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.explorer");
	private JComboBox filterBox;
	private JPanel leftPanel;
	private JTextField nameField;
	private boolean preemptiveMode = false;
	private File selectedFile;
	private JSplitPane splitPane;
	private StatusBar statusBar;
	private SelectorTable table;
	private SelectorTree tree;
	private FileSystemView view;

	/**
	 *  Constructor for the ExplorerPanel object
	 *
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorPanel(boolean preemptiveMode) {
		this(null, FileSystemView.getFileSystemView(), preemptiveMode);
	}

	/**
	 *  Constructor for the SelectorPanel object
	 *
	 *@param  currentDir      Description of the Parameter
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorPanel(File currentDir, boolean preemptiveMode) {
		this(currentDir, FileSystemView.getFileSystemView(), preemptiveMode);
	}

	/**
	 *  Constructor for the ExplorerPanel object
	 *
	 *@param  view            Description of the Parameter
	 *@param  selectedFile    Description of the Parameter
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorPanel(
		File selectedFile,
		FileSystemView view,
		boolean preemptiveMode) {
		this.preemptiveMode = preemptiveMode;
		if (selectedFile == null) {
			selectedFile = view.getDefaultDirectory();
		}
		this.selectedFile = selectedFile;
		this.view = view;
		buildComponents();
	}

	/**
	 *  Constructor for the SelectorPanel object
	 *
	 *@param  view            Description of the Parameter
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorPanel(FileSystemView view, boolean preemptiveMode) {
		this(null, view, preemptiveMode);
	}

	/**
	 *  Sets the fileFilter attribute of the SelectorPanel object
	 *
	 *@param  filter  The new fileFilter value
	 */
	public void addFileFilter(FileFilter filter) {
		for (int i = 0; i < filterBox.getItemCount(); i++) {
			FileFilter existing = (FileFilter) filterBox.getItemAt(i);
			if (existing.getDescription().equals(filter.getDescription())) {
				return;
			}
		}
		filterBox.addItem(filter);
	}

	/**
	 *  Description of the Method
	 */
	protected void buildComponents() {

		// build tree
		tree = new SelectorTree(selectedFile, view, preemptiveMode);
		tree.addTreeSelectionListener(new TreeListener());

		// get initial node from tree
		DirNode currentNode = tree.getSelectedNode();
		File[] listing = currentNode.getListing();

		// build table
		table = new SelectorTable(view, true);
		table.getTableHeader().addMouseListener(new TableHeaderListener());
		table.addMouseListener(new TableMouseListener());
		table.setListing(listing);

		JScrollPane tablePane = new JScrollPane(table);
		// build status bar
		statusBar = new StatusBar();
		statusBar.setText("Objects: " + listing.length);
		statusBar.setBorder(new SoftBevelBorder(SoftBevelBorder.LOWERED));

		filterBox = new JComboBox();
		filterBox.addItem(DEFAULT_FILTER);
		filterBox.addItemListener(new FilterListener());
		JLabel filterLabel = new JLabel("Filter: ");
		filterLabel.setLabelFor(filterBox);

		JButton refreshButton = new JButton(new RefreshAction());

		JPanel filterPanel = new JPanel();
		filterPanel.setLayout(new BoxLayout(filterPanel, BoxLayout.X_AXIS));
		filterPanel.add(filterLabel);
		filterPanel.add(filterBox);
		filterPanel.add(refreshButton);

		// use a sub panel to prevent filter box from resizing
		JPanel southPanel = new JPanel(new BorderLayout());
		southPanel.add(filterPanel, BorderLayout.CENTER);
		southPanel.add(statusBar, BorderLayout.SOUTH);

		JPanel rightPanel = new JPanel(new BorderLayout());
		rightPanel.add(tablePane, BorderLayout.CENTER);
		rightPanel.add(southPanel, BorderLayout.SOUTH);

		// build name field
		nameField = new JTextField(currentNode.getDir().getAbsolutePath());
		JLabel nameLabel = new JLabel("Filename: ");
		nameLabel.setLabelFor(nameField);

		JScrollPane treePane = new JScrollPane(tree);

		leftPanel = new JPanel(new BorderLayout());
		leftPanel.add(treePane, BorderLayout.CENTER);

		splitPane =
			new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, rightPanel);

		JPanel namePanel = new JPanel();
		namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.X_AXIS));
		namePanel.add(nameLabel);
		namePanel.add(nameField);

		setLayout(new BorderLayout());
		add(splitPane, BorderLayout.CENTER);
		add(namePanel, BorderLayout.SOUTH);

		if (selectedFile.isFile()) {
			table.setSelectedObject(selectedFile);
		}
		if (selectedFile instanceof RemoteFile) {
			nameField.setText(((RemoteFile) selectedFile).getSubspacePath());
		} else {
			nameField.setText(selectedFile.getAbsolutePath());
		}

	}

	/**
	 *  Gets the dividerLocation attribute of the SelectorPanel object
	 *
	 *@return    The dividerLocation value
	 */
	public int getDividerLocation() {
		return splitPane.getDividerLocation();
	}

	/**
	 *  Gets the fileFilters attribute of the SelectorPanel object
	 *
	 *@return    The fileFilters value
	 */
	public Vector getFileFilters() {
		Vector filters = new Vector();
		for (int i = 0; i < filterBox.getItemCount(); i++) {
			filters.add(filterBox.getItemAt(i));
		}
		return filters;
	}

	/**
	 *  Gets the selectedFile attribute of the SelectorPanel object
	 *
	 *@return    The selectedFile value
	 */
	public File getSelectedFile() {
		if (selectedFile instanceof RemoteFile) {
			RemoteFile remoteFile = (RemoteFile) selectedFile;
			// check if we picked an existing file
			if (remoteFile
				.getSubspacePath()
				.equals(nameField.getText().trim())) {
				return remoteFile;
			}
			// we want to build a new remote file
			else {
				return new RemoteFile(
					nameField.getText().trim(),
					remoteFile.getStorage(),
					remoteFile.getFileSystem());
			}
		} else {
			if (selectedFile
				.getAbsolutePath()
				.equals(nameField.getText().trim())) {
				return selectedFile;
			} else {
				return new File(nameField.getText().trim());
			}
		}
	}

	/**
	 *  Gets the selectedFileFilter attribute of the SelectorPanel object
	 *
	 *@return    The selectedFileFilter value
	 */
	public FileFilter getSelectedFileFilter() {
		return (FileFilter) filterBox.getSelectedItem();
	}

	/**
	 *  Gets the selectorTable attribute of the SelectorPanel object
	 *
	 *@return    The selectorTable value
	 */
	public SelectorTable getSelectorTable() {
		return this.table;
	}

	/**
	 *  Description of the Method
	 */
	public void refresh() {
		tree.refresh(tree.getSelectedNode());
		updateNodeSelection();

	}

	public void removeFileFilter(FileFilter filter) {
		filterBox.removeItem(filter);
	}

	/**
	 *  Description of the Method
	 */
	public void removeFileFilters() {
		filterBox.removeAllItems();
		filterBox.addItem(DEFAULT_FILTER);
	}

	/**
	 *  Sets the dividerLocation attribute of the ExplorerPanel object
	 *
	 *@param  value  The new dividerLocation value
	 */
	public void setDividerLocation(double value) {
		splitPane.setDividerLocation(value);
	}

	/**
	 *  Sets the dividerLocation attribute of the SelectorPanel object
	 *
	 *@param  value  The new dividerLocation value
	 */
	public void setDividerLocation(int value) {
		splitPane.setDividerLocation(value);
	}

	/**
	 *  Sets the filename attribute of the SelectorPanel object
	 *
	 *@param  filename  The new filename value
	 */
	public void setFilename(String filename) {
		File currentDir = tree.getSelectedNode().getDir();
		if (currentDir instanceof RemoteFile) {
			nameField.setText(
				((RemoteFile) currentDir).getSubspacePath()
					+ RemoteFile.separatorChar
					+ filename);
		} else {
			nameField.setText(currentDir + File.separator + filename);
		}
	}

	/**
	 *  Sets the selectedFile attribute of the ExplorerPanel object
	 *
	 *@param  selectedFile               The new selectedFile value
	 *@exception  FileNotFoundException  Description of the Exception
	 */
	public void setSelectedFile(final File selectedFile)
		throws FileNotFoundException {
		this.selectedFile = selectedFile;
		if (selectedFile.isDirectory()) {
			tree.setCurrentDirectory(selectedFile);
			table.setSelectedObject(null);
		} else {
			tree.setCurrentDirectory(selectedFile.getParentFile());
			// make sure table is up to date now
			updateNodeSelection();
			// wait until table is up to date, befor selecting new entry
			Runnable runner = new Runnable() {
				public void run() {
					table.setSelectedObject(selectedFile);
					updateNameField();
				}
			};
			SwingUtilities.invokeLater(runner);
		}
	}

	/**
	 *  Sets the selectedFilter attribute of the SelectorPanel object
	 *
	 *@param  filter  The new selectedFilter value
	 */
	public void setSelectedFileFilter(FileFilter filter) {
		filterBox.setSelectedItem(filter);
	}

	/**
	 *  Description of the Method
	 */
	private void updateNameField() {
		if (selectedFile instanceof RemoteFile) {
			nameField.setText(((RemoteFile) selectedFile).getSubspacePath());
		} else {
			nameField.setText(selectedFile.getAbsolutePath());
		}
	}

	/**
	 *  Description of the Method
	 */
	private void updateNodeSelection() {
		final JRootPane rootPane = getRootPane();
		final DirNode node = tree.getSelectedNode();
		if (node == null) {
			return;
		}
		selectedFile = node.getDir();

		updateNameField();

		String path = selectedFile.getAbsolutePath();
		if (rootPane != null) {
			if (node.isUpToDate()) {
				statusBar.setText("Building file table for: " + path);
			} else {
				statusBar.setText("Scanning directory " + path);
			}
			rootPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		}
		Runnable runner = new Runnable() {
			public void run() {
				long start = System.currentTimeMillis();
				// if node is not up to date
				// a complete scan is running in the background
				if (!node.isUpToDate()) {
					tree.addNextLevel(node);
				}
				File[] listing = node.getListing();
				table.setListing(listing);
				if (rootPane != null) {
					rootPane.setCursor(
						Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				}
				statusBar.setText("Objects: " + listing.length);
				long end = System.currentTimeMillis();
				logger.fine("TIME(s): " + (end - start) / 1000.0);
			}
		};
		SwingUtilities.invokeLater(runner);
	}

}
