package com.pallas.unicore.client.explorer;

import java.awt.Component;
import java.io.File;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.DateTableCellRenderer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.VectorSorter;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: SelectorTable.java,v 1.2 2003/06/30 06:53:48 rrate Exp $
 */
public class SelectorTable extends AdjustTable {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. Mai 2002
	 */
	private class FileCellRenderer extends DefaultTableCellRenderer {
		/**
		 *  Gets the tableCellRendererComponent attribute of the
		 *  FileCellRenderer object
		 *
		 *@param  table       Description of the Parameter
		 *@param  value       Description of the Parameter
		 *@param  isSelected  Description of the Parameter
		 *@param  hasFocus    Description of the Parameter
		 *@param  row         Description of the Parameter
		 *@param  column      Description of the Parameter
		 *@return             The tableCellRendererComponent value
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {

			JLabel label =
				(JLabel) super.getTableCellRendererComponent(
					table,
					value,
					isSelected,
					hasFocus,
					row,
					column);

			FileEntry fileEntry = (FileEntry) value;
			try {
				Icon icon = view.getSystemIcon(fileEntry.getFile());
				label.setIcon(icon);
			} catch (Exception e) {
				logger.log(Level.INFO, "Could not display file icon.", e);
			}
			return label;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. Mai 2002
	 */
	protected class FileTableModel extends DefaultTableModel {

		/**
		 *  Gets the CellEditable attribute of the InfoTableModel object
		 *
		 *@param  row  Description of Parameter
		 *@param  col  Description of Parameter
		 *@return      The CellEditable value
		 */
		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    5. Dezember 2002
	 */
	private class TableHeaderRenderer extends DefaultTableCellRenderer {
		/**
		 *  Gets the tableCellRendererComponent attribute of the
		 *  TableHeaderRenderer object
		 *
		 *@param  table       Description of the Parameter
		 *@param  value       Description of the Parameter
		 *@param  isSelected  Description of the Parameter
		 *@param  hasFocus    Description of the Parameter
		 *@param  row         Description of the Parameter
		 *@param  column      Description of the Parameter
		 *@return             The tableCellRendererComponent value
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {

			if (column == sortColumn) {
				if (ascending) {
					this.setIcon(up);
				} else {
					this.setIcon(down);
				}
			} else {
				this.setIcon(null);
			}
			this.setHorizontalTextPosition(JLabel.LEFT);

			JTableHeader header = table.getTableHeader();
			if (header != null) {
				this.setForeground(header.getForeground());
				this.setBackground(header.getBackground());
				this.setFont(header.getFont());
			}

			this.setText((value == null) ? "" : value.toString());
			this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));

			return this;
		}
	}
	private final static int COL_DATE = 3;
	private final static int COL_FILE = 0;
	private final static int COL_MEMORY = 1;
	private final static int COL_TYPE = 2;
	private static TableCellRenderer dateCellRenderer =
		new DateTableCellRenderer(ResourceManager.getCompleteFormatter());
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.explorer");
	private boolean ascending = true;
	private Vector columnIdentifiers;
	private boolean displayIcons;
	private final ImageIcon down =
		ResourceManager.getIcon(ResourceManager.DOWN);
	private FileCellRenderer fileCellRenderer = new FileCellRenderer();
	private FileFilter filter;

	private boolean initialized;
	private int lastSortColumn = -1;
	private File[] listing;
	private FileTableModel model;
	private int sortColumn = 0;

	private final ImageIcon up = ResourceManager.getIcon(ResourceManager.UP);

	/**
	 *  Description of the Field
	 */
	protected FileSystemView view;

	/**
	 *  Constructor for the ExplorerTable object
	 */
	public SelectorTable() {
		this(FileSystemView.getFileSystemView(), true);
	}

	/**
	 *  Constructor for the SelectorTable object
	 *
	 *@param  displayIcons  Description of the Parameter
	 */
	public SelectorTable(boolean displayIcons) {
		this(FileSystemView.getFileSystemView(), displayIcons);
	}

	/**
	 *  Constructor for the SelectorTable object
	 *
	 *@param  view  Description of the Parameter
	 */
	public SelectorTable(FileSystemView view) {
		this(view, true);
	}

	/**
	 *  Constructor for the ExplorerTable object
	 *
	 *@param  displayIcons  Description of the Parameter
	 *@param  view          Description of the Parameter
	 */
	public SelectorTable(FileSystemView view, boolean displayIcons) {
		this.view = view;
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setColumnSelectionAllowed(false);
		setDragEnabled(true);
		buildComponents();
		setDisplayIcons(displayIcons);
		getColumnModel().getColumn(3).setCellRenderer(dateCellRenderer);
		//        ignoreDataWidth(COL_TYPE);
		//        ignoreDataWidth(COL_DATE);
		initialized = false;
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		columnIdentifiers = new Vector();
		columnIdentifiers.add("Name");
		columnIdentifiers.add("Size");
		columnIdentifiers.add("Type");
		columnIdentifiers.add("Last Modified");

		model = new FileTableModel();
		model.setDataVector(null, columnIdentifiers);
		setModel(model);
		this.getTableHeader().setDefaultRenderer(new TableHeaderRenderer());
	}

	/**
	 *  Description of the Method
	 *
	 *@param  file  Description of the Parameter
	 *@return       The row value
	 */
	private RowData buildRow(File file) {
		return new RowData(
			file,
			view.getSystemDisplayName(file),
			view.getSystemTypeDescription(file));
	}

	/**
	 *  Description of the Method
	 */
	private void clear() {
		int rowCount;
		for (int i = 0; i < (rowCount = getRowCount()); i++) {
			model.removeRow(rowCount - 1);
		}
	}

	/**
	 *  Gets the selectedElement attribute of the ExplorerTable object
	 *
	 *@return    The selectedElement value
	 */
	public File getSelectedFile() {
		int selectedRow = getSelectedRow();
		if (selectedRow < 0) {
			return null;
		}
		RowData rowData =
			(RowData) model.getDataVector().elementAt(selectedRow);
		return rowData.getFile();
	}

	/**
	 *  Description of the Method
	 */
	public void resetValues() {
		this.ascending = true;
		this.sortColumn = 0;
	}

	/**
	 *  Sets the displayIcons attribute of the ExplorerTable object
	 *
	 *@param  displayIcons  The new displayIcons value
	 */
	public void setDisplayIcons(boolean displayIcons) {
		this.displayIcons = displayIcons;
		TableColumn fileColumn = columnModel.getColumn(0);
		if (displayIcons) {
			fileColumn.setCellRenderer(fileCellRenderer);
		} else {
			fileColumn.setCellRenderer(null);
		}
	}

	/**
	 *  Sets the fileFilter attribute of the SelectorTable object. Only updates
	 *  table if description of new filter is different from old description.
	 *
	 *@param  filter  The new fileFilter value
	 */
	public void setFileFilter(FileFilter filter) {
		FileFilter oldFilter = this.filter;
		this.filter = filter;
		if (oldFilter == null
			|| !oldFilter.getDescription().equals(filter.getDescription())) {
			setListing(listing);
		}
	}

	/**
	 *  Sets the fileSystemView attribute of the SelectorTable object
	 *
	 *@param  view  The new fileSystemView value
	 */
	public void setFileSystemView(FileSystemView view) {
		this.view = view;
	}

	/**
	 *  Sets the listing attribute of the ExplorerTable object
	 *
	 *@param  listing  The new listing value
	 */
	public synchronized void setListing(File[] listing) {
		if (listing == null) {
			return;
		}
		this.listing = listing;

		// remove old rows
		while (model.getRowCount() > 0) {
			model.removeRow(0);
		}

		// add new rows
		for (int i = 0; i < listing.length; i++) {
			if (filter == null || filter.accept(listing[i])) {
				model.addRow(buildRow(listing[i]));
			}
		}

		// find nice column widths
		if (!initialized) {
			initialized = true;
			adjustWidths();
		}
	}

	/**
	 *  Sets the selectedObject attribute of the SelectorTable object
	 *
	 *@param  file  The new selectedObject value
	 */
	public void setSelectedObject(File file) {
		if (file == null) {
			selectionModel.clearSelection();
			return;
		}
		Vector dataVector = model.getDataVector();
		for (int i = 0; i < dataVector.size(); i++) {
			RowData rowData = (RowData) dataVector.elementAt(i);
			if (rowData.getFile().equals(file)) {
				selectionModel.setSelectionInterval(i, i);
				return;
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  column  Description of the Parameter
	 */
	public void sortByColumn(int sortColumn) {
		this.sortColumn = sortColumn;
		if (lastSortColumn == sortColumn) {
			ascending = !ascending;
		} else {
			ascending = true;
		}
		RowData.setCompareIndex(sortColumn);
		Vector dataVector = model.getDataVector();
		VectorSorter.sort(dataVector, ascending);

		lastSortColumn = sortColumn;
	}

}
