package com.pallas.unicore.client.explorer;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.filechooser.FileSystemView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystemView;
import com.pallas.unicore.requests.GetListings;
import com.pallas.unicore.threadpool.IObserver;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: SelectorTree.java,v 1.4 2003/06/30 06:53:47 rrate Exp $
 */
public class SelectorTree extends JTree implements IObserver {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    13. August 2002
	 */
	private class ClickListener extends MouseAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() >= 2) {
				DirNode node = getSelectedNode();
				logger.finest("Double click on " + node);
				if (view.isFloppyDrive(node.getDir())) {
					refresh(node);
				}
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. Mai 2002
	 */
	private class ExpandListener implements TreeExpansionListener {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void treeCollapsed(TreeExpansionEvent e) {
			TreePath path = e.getPath();
			if (path.isDescendant(new TreePath(getSelectedNode().getPath()))) {
				setSelectionPath(path);
			}
		}
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void treeExpanded(TreeExpansionEvent e) {
			TreeNode node = (TreeNode) e.getPath().getLastPathComponent();
			logger.finest("Tree-expanded event detected: " + node);
			if (node instanceof DirNode) {
				DirNode dirNode = (DirNode) node;
				buildNextLevel(dirNode);
			}
		}
	}
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.explorer");
	private TreeCellRenderer cellRenderer = new FileCellRenderer();
	private ExpandListener expandListener = new ExpandListener();
	private Object lock = new Object();
	private DefaultTreeModel model;
	private boolean preemptiveMode = false;
	private DefaultMutableTreeNode root = new DefaultMutableTreeNode("Root");
	private Vector rootVector;
	/**
	 *  Description of the Field
	 */
	protected FileSystemView view;

	/**
	 *  Constructor for the ExplorerTree object
	 *
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorTree(boolean preemptiveMode) {
		this(null, FileSystemView.getFileSystemView(), preemptiveMode);
	}

	/**
	 *  Constructor for the SelectorTree object
	 *
	 *@param  currentDir      Description of the Parameter
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorTree(File currentDir, boolean preemptiveMode) {
		this(currentDir, FileSystemView.getFileSystemView(), preemptiveMode);
	}

	/**
	 *  Constructor for the ExplorerTree object
	 *
	 *@param  currentDir      Description of the Parameter
	 *@param  view            Description of the Parameter
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorTree(
		File currentDir,
		FileSystemView view,
		boolean preemptiveMode) {
		this.view = view;
		this.preemptiveMode = preemptiveMode;
		initialize();

		// try to get default dir
		if (currentDir == null
			|| !currentDir.exists()
			|| (!(currentDir instanceof RemoteFile)
				&& (view instanceof RemoteFileSystemView))) {
			currentDir = view.getDefaultDirectory();
		}

		try {
			setCurrentDirectory(currentDir);
		} catch (FileNotFoundException e) {
			logger.log(
				Level.INFO,
				"Did not find start directory: " + currentDir,
				e);
		}

		getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);

		buildNextLevel(root);

	}

	/**
	 *  Constructor for the SelectorTree object
	 *
	 *@param  view            Description of the Parameter
	 *@param  preemptiveMode  Description of the Parameter
	 */
	public SelectorTree(FileSystemView view, boolean preemptiveMode) {
		this(null, view, preemptiveMode);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void addNextLevel(DirNode node) {
		logger.fine("Adding next level to node " + node);
		if (node.isUpToDate()) {
			return;
		}
		logger.fine("Proceeding adding next level to node " + node);
		File[] listing = node.getListing();
		// add hidden next level
		for (int i = 0; i < listing.length; i++) {
			if (listing[i].isDirectory()) {
				DirNode listNode = new DirNode(listing[i], view);
				addNode(listNode, node);
			}
		}
		logger.finest("Listing for node " + node + ":\n" + listing);
		node.setUpToDate(true);
	}

	/**
	 *  Adds a feature to the Node attribute of the ExplorerTree object
	 *
	 *@param  parent  The feature to be added to the Node attribute
	 *@param  node    The feature to be added to the Node attribute
	 */
	private void addNode(DirNode node, MutableTreeNode parent) {
		synchronized (lock) {
			model.insertNodeInto(node, parent, parent.getChildCount());
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	private void buildNextLevel(MutableTreeNode node) {
		if (!preemptiveMode) {
			return;
		}
		if (view instanceof RemoteFileSystemView) {
			buildNextRemoteLevel(node);
		} else {
			buildNextLocalLevel(node);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	private void buildNextLocalLevel(MutableTreeNode node) {
		Enumeration children = node.children();
		while (children.hasMoreElements()) {
			final DirNode next = (DirNode) children.nextElement();
			if (!next.isUpToDate()) {
				Thread thread = new Thread() {
					public void run() {
						addNextLevel(next);
					}
				};
				thread.start();
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	private void buildNextRemoteLevel(MutableTreeNode node) {
		Vector dirs = new Vector();
		Vector dirNodes = new Vector();
		Enumeration children = node.children();
		while (children.hasMoreElements()) {
			DirNode next = (DirNode) children.nextElement();
			if (!next.isUpToDate()) {
				dirs.add(next.getDir());
				dirNodes.add(next);
			}
		}

		if (dirs.isEmpty()) {
			return;
		}

		RemoteFile file = (RemoteFile) dirs.firstElement();
		GetListings request = new GetListings(dirs, dirNodes, file.getVsite());
		request.start();
		request.addObserver(this);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  dir         Description of the Parameter
	 *@param  parentNode  Description of the Parameter
	 *@return             Description of the Return Value
	 */
	public DirNode findNode(File dir, TreeNode parentNode) {
		logger.finest("Trying to find " + dir + " at " + parentNode);
		String dirname = dir.getAbsolutePath();
		// make windows drives appear upper case always
		if (dirname.indexOf(":") == 1) {
			dirname =
				dirname.substring(0, 1).toUpperCase() + dirname.substring(1);
		}

		for (int i = 0; i < parentNode.getChildCount(); i++) {
			DirNode child = (DirNode) parentNode.getChildAt(i);
			String childname = child.getDir().getAbsolutePath();
			// make windows drives appear upper case always
			if (childname.indexOf(":") == 1) {
				childname =
					childname.substring(0, 1).toUpperCase()
						+ childname.substring(1);
			}
			logger.finest("Comparing " + dir + " to " + child.getDir());
			if (dir.equals(child.getDir()) || dirname.equals(childname)) {
				logger.finest(
					"Comparison between "
						+ dir
						+ " and "
						+ child.getDir()
						+ "successful.");
				return child;
			}
		}
		return null;
	}

	/**
	 *  Gets the selectedNode attribute of the ExplorerTree object
	 *
	 *@return    The selectedNode value
	 */
	public DirNode getSelectedNode() {
		synchronized (lock) {
			if (getSelectionPath() == null) {
				return null;
			}
			return (DirNode) getSelectionPath().getLastPathComponent();
		}
	}

	/**
	 *  Description of the Method
	 */
	public void initialize() {
		setInvokesStopCellEditing(true);
		setShowsRootHandles(true);
		addTreeExpansionListener(expandListener);
		addMouseListener(new ClickListener());
		model = new DefaultTreeModel(root);
		setModel(model);
		setRootVisible(false);
		setCellRenderer(cellRenderer);

		// add roots
		File[] roots = view.getRoots();
		rootVector = new Vector();
		for (int i = 0; i < roots.length; i++) {
			logger.finest("Found root: " + roots[i]);
			rootVector.add(roots[i]);
			DirNode node = new DirNode(roots[i], view);
			addNode(node, root);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  observable  Description of the Parameter
	 *@param  changeCode  Description of the Parameter
	 */
	public void observableUpdate(
		final Object observable,
		final Object changeCode) {
		Runnable runner = new Runnable() {
			public void run() {
				GetListings request = (GetListings) observable;
				Hashtable table = (Hashtable) changeCode;
				Vector dirs = request.getDirectories();
				Vector dirNodes = request.getDirectoryNodes();

				// add hidden next level
				for (int i = 0; i < dirs.size(); i++) {
					RemoteFile dir = (RemoteFile) dirs.elementAt(i);
					DirNode dirNode = (DirNode) dirNodes.elementAt(i);
					// if node has been updated via manual selection
					// in between, continue
					logger.finest(
						"Returned from polling thread, adding nodes to "
							+ dirNode);
					if (dirNode.isUpToDate()) {
						continue;
					}
					logger.finest("Proceeding adding nodes to " + dirNode);
					Vector listing = (Vector) table.get(dir);
					dirNode.setListing(listing);
					dirNode.setUpToDate(true);
					for (int j = 0; j < listing.size(); j++) {
						RemoteFile listFile = (RemoteFile) listing.elementAt(j);
						if (listFile.isDirectory()) {
							DirNode listNode = new DirNode(listFile, view);
							addNode(listNode, dirNode);
						}
					}
				}
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public synchronized void refresh(DirNode node) {
		synchronized (lock) {
			logger.fine("Refreshing " + node);
			// explicitly enable listing in case we have selected a floppy drive
			node.setListingEnabled(true);
			node.setListing((File[]) null);
			node.removeAllChildren();
			model.reload(node);
			node.setUpToDate(false);
		}
		addNextLevel(node);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	private void selectNode(DefaultMutableTreeNode node) {
		synchronized (lock) {
			TreePath path = new TreePath(node.getPath());
			setSelectionPath(path);
			scrollPathToVisible(path);
		}
	}

	/**
	 *  Sets the currentDirectory attribute of the ExplorerTree object
	 *
	 *@param  dir                        The new currentDirectory value
	 *@exception  FileNotFoundException  Description of the Exception
	 */
	void setCurrentDirectory(File dir) throws FileNotFoundException {
		if (dir == null) {
			return;
		}
		if (!dir.exists()) {
			throw new FileNotFoundException(
				"Current directory " + dir + " does not exist.");
		}
		synchronized (lock) {
			// collect ancestors
			Vector ancestors = new Vector();
			// start with file itself
			File ancestor = dir;
			try {
				// relative path names do not work in the level building loop below
				ancestor = ancestor.getCanonicalFile();
			} catch (IOException ioe) {
				logger.log(Level.SEVERE, "", ioe);
				ancestor = dir;
			}
			// if current dir is not a directory choose the parent
			if (ancestor.isFile()) {
				ancestor = view.getParentDirectory(ancestor);
			}
			// follow ancestors until file system root is reached
			while (ancestor != null && !rootVector.contains(ancestor)) {
				ancestors.add(ancestor);
				ancestor = view.getParentDirectory(ancestor);
			}
			// also add file system root
			ancestors.add(ancestor);

			// start with tree root (parent to file system roots)
			DefaultMutableTreeNode node = root;

			// loop from root to current dir and build levels in tree
			for (int i = ancestors.size() - 1; i >= 0; i--) {
				// find node belonging to current ancestor
				File file = (File) ancestors.elementAt(i);
				if (file.isDirectory()) {
					TreeNode parent = node;
					node = findNode(file, node);
					// if we could not find node, refresh parent and re-try
					if (node == null) {
						refresh((DirNode) parent);
						node = findNode(file, parent);
					}
					addNextLevel((DirNode) node);
				}
			}
			// select current dir
			if (node != null) {
				selectNode(node);
			}
		}
	}

	/**
	 *  Sets the fileSystemView attribute of the SelectorTree object
	 *
	 *@param  view  The new fileSystemView value
	 */
	public void setFileSystemView(FileSystemView view) {
		this.view = view;
	}

	/**
	 *  Sets the selectedObject attribute of the ExplorerTree object
	 *
	 *@param  file  The new selectedObject value
	 */
	public void setSelectedObject(File file) {
		synchronized (lock) {
			DirNode parent =
				(DirNode) getSelectionPath().getLastPathComponent();
			if (parent == null) {
				return;
			}
			DirNode dirNode = findNode(file, parent);
			setSelectionPath(new TreePath(dirNode.getPath()));
		}
	}
}
