package com.pallas.unicore.client.graphics;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.Point2D;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 *  Shape that draws an arrow as polygon
 *
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: Arrow.java,v 1.2 2003/06/30 06:53:59 rrate Exp $
 */
public class Arrow extends Polygon {

	/**
	 *  The main program for the Arrow class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame("Arrow Test");
		frame.setSize(400, 400);

		JPanel drawPanel = new JPanel() {
			/**
			 *  Description of the Method
			 *
			 *@param  g  Description of Parameter
			 */
			public void paint(Graphics g) {
				super.paint(g);
				Point2D start = new Point2D.Double(20, 20);
				Point2D end = new Point2D.Double(300, 200);
				Arrow arrow = new Arrow(start, end);
				Graphics2D g2 = (Graphics2D) g;
				g2.draw(arrow);

			}
		};

		frame.getContentPane().add(drawPanel);
		frame.show();

	}
	private int headSize = 6;

	/**
	 *  Constructor for the Arrow object
	 */
	public Arrow() {
		super();
	}

	/**
	 *  Constructor for the Arrow object
	 *
	 *@param  headSize  size of arrow head
	 */
	public Arrow(int headSize) {
		this.headSize = headSize;
	}

	/**
	 *  Constructor for the Arrow object
	 *
	 *@param  start  start point
	 *@param  end    end point
	 */
	public Arrow(Point2D start, Point2D end) {
		this();
		npoints = 6;
		xpoints = new int[npoints];
		ypoints = new int[npoints];
		setLine(start, end);
	}

	/**
	 *  Constructor for the Arrow object
	 *
	 *@param  start     start point
	 *@param  end       end point
	 *@param  headSize  size of arrow head
	 */
	public Arrow(Point2D start, Point2D end, int headSize) {
		this(start, end);
		this.headSize = headSize;
		setLine(start, end);
	}

	/**
	 *  Sets the HeadSize attribute of the Arrow object
	 *
	 *@param  headSize  The new HeadSize value
	 */
	public void setHeadSize(int headSize) {
		this.headSize = headSize;
	}

	/**
	 *  Sets the Line attribute of the Arrow object
	 *
	 *@param  start  The new Line value
	 *@param  end    The new Line value
	 */
	public void setLine(Point2D start, Point2D end) {

		// get vector v from start to end
		double vx = end.getX() - start.getX();
		double vy = end.getY() - start.getY();

		// normalize v -> vn
		double v_2 = vx * vx + vy * vy;
		double v_length = Math.sqrt(v_2);
		if (v_length < headSize) {
			for (int i = 0; i < this.npoints; i++) {
				xpoints[i] = 0;
				ypoints[i] = 0;
			}
			return;
		}
		double v_norm = v_length / v_2;
		double vnx = v_norm * vx;
		double vny = v_norm * vy;

		// find p1
		double p1x = start.getX() + (v_length - headSize) * vnx;
		double p1y = start.getY() + (v_length - headSize) * vny;

		// find p2
		// rotate vn by 90 deg
		double halfHeadSize = headSize / 2.0;
		double p2x = p1x - vny * halfHeadSize;
		double p2y = p1y + vnx * halfHeadSize;

		// find p2
		// rotate vn by -90 deg
		double p4x = p1x + vny * halfHeadSize;
		double p4y = p1y - vnx * halfHeadSize;

		xpoints[0] = (int) start.getX();
		ypoints[0] = (int) start.getY();

		xpoints[1] = (int) p1x;
		ypoints[1] = (int) p1y;

		xpoints[2] = (int) p2x;
		ypoints[2] = (int) p2y;

		xpoints[3] = (int) end.getX();
		ypoints[3] = (int) end.getY();

		xpoints[4] = (int) p4x;
		ypoints[4] = (int) p4y;

		xpoints[5] = (int) p1x;
		ypoints[5] = (int) p1y;
	}
}
