package com.pallas.unicore.client.graphs;

import java.util.Vector;

/**
 *  Sorts a vector of comparable vertexs belonging to a graph in ascending
 *  order. The ordering depends on the deph and the affiliations of their
 *  belonging vertexs.
 *
 *@author     Kirsten Foerster
 *@created    18. Juni 2002
 *@version    $Id: TreeSorter.java,v 1.2 2003/06/30 06:54:06 rrate Exp $
 */
public class TreeSorter {

	private ITreeComparable treeComparable;

	/**
	 *  Constructor for the TreeSorter object
	 *
	 *@param  treeComparable  Description of the Parameter
	 */
	public TreeSorter(ITreeComparable treeComparable) {
		this.treeComparable = treeComparable;
	}

	/**
	 *  Sorts a vector of comparable vertexs belonging to a graph in ascending
	 *  order.
	 *
	 *@param  unsortVector  - a unsorted vector
	 *@return               - the sorted vector
	 */
	public Vector sort(Vector unsortVector) {
		if (unsortVector.isEmpty()) {
			return null;
		}

		Vector sortVector = new Vector();

		Object element = unsortVector.firstElement();
		sortVector.addElement(element);

		boolean inserted = false;
		int comparison;
		int maxPrioritaet;
		int pos = 0;
		boolean flag = false;

		for (int i = 1; i < unsortVector.size(); i++) {

			element = unsortVector.elementAt(i);
			if (this.treeComparable.noChildrens(element)
				&& this.treeComparable.noParents(element)) {
				sortVector.addElement(element);
				continue;
			}

			maxPrioritaet = 0;

			for (int j = 0; j < sortVector.size(); j++) {
				comparison =
					this.treeComparable.compareTo(
						element,
						sortVector.elementAt(j));
				if (comparison == VertexsTreeComparable.IMMEDIATE_INSERTING) {
					sortVector.insertElementAt(element, j);
					inserted = true;
					break;
				}

				if (comparison == VertexsTreeComparable.POSITION_MARKER) {
					if (!flag) {
						pos = j;
						flag = true;
					}
					continue;
				}

				//the computed value of comparison is between
				//NEXT_POSITION_MARKER_LOWEST_PRIORITY and
				//NEXT_POSITION_MAKRER_TOP_PRIORITY
				if (comparison > 0) {
					if (comparison >= maxPrioritaet) {
						pos = j + 1;
						maxPrioritaet = comparison;
						flag = false;
					}
				}
			}

			if (!inserted) {
				sortVector.insertElementAt(element, pos);
			}

			inserted = false;
			flag = false;
			maxPrioritaet = 0;
		}

		return sortVector;
	}

}
