package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DateFormat;
import java.text.ParsePosition;
import java.util.Date;

import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@created    17. Juli 2002
 *@version    $Id: DatePanel.java,v 1.3 2003/06/30 06:54:44 rrate Exp $
 */

public class DatePanel extends JPanel {

	/**
	 */
	private class CheckBoxListener implements ItemListener {
		/**
		 *  Constructor for the itemStateChanged object
		 *
		 *@param  e  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent e) {
			if (todayBox.isSelected()) {
				localDateField.setText(dateFormatter.format(new Date()));
				localDateField.setEnabled(false);
			} else {
				localDateField.setEnabled(true);
			}
		}

	}

	/**
	 */
	private class TextFieldFocusListener extends FocusAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void focusLost(FocusEvent e) {
			updateUTCTime();
		}
	}

	/**
	 */
	private class TextFieldListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			updateUTCTime();
		}
	}
	/**
	 *  Description of the Field
	 */
	protected static DateFormat dateFormatter =
		ResourceManager.getDateFormatter(),
		timeFormatter = ResourceManager.getTimeFormatter(),
		completeFormatter = ResourceManager.getCompleteFormatter(),
		utcFormatter = ResourceManager.getUTCFormatter();

	/**
	 *
	 *@param  args  Description of the Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("Date Panel Test");
		frame.getContentPane().add(new DatePanel(), BorderLayout.CENTER);
		frame.pack();
		frame.setVisible(true);

	}

	private Component additionalComponent1, additionalComponent2;
	private JFormattedTextField localDateField, localTimeField;
	private JCheckBox todayBox;
	private JLabel waitLabel,
		localTimeLabel,
		localDateLabel,
		utcLabel,
		utcTimeLabel;

	/**
	 *  Constructor for the DatePanel object
	 */
	public DatePanel() {
		this(null, null);
	}

	/**
	 *  Constructor for the DatePanel object
	 *
	 *@param  additionalComponent1  Description of the Parameter
	 *@param  additionalComponent2  Description of the Parameter
	 */
	public DatePanel(
		Component additionalComponent1,
		Component additionalComponent2) {
		this.additionalComponent1 = additionalComponent1;
		this.additionalComponent2 = additionalComponent2;
		buildComponents();
		setEnabled(false);
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		GridBagLayout g = new GridBagLayout();
		setLayout(g);
		GridBagConstraints c = new GridBagConstraints();

		int row = 0;
		if (additionalComponent1 != null) {
			c = LayoutTools.makegbc(0, row, 1, 0, false);
			add(additionalComponent1, c);
		}
		if (additionalComponent2 != null) {
			c = LayoutTools.makegbc(1, row, 1, 0, true);
			add(additionalComponent2, c);
		}

		row++;
		localTimeLabel = new JLabel("Time (hh:mm:ss): ");
		localTimeLabel.setToolTipText("Time at local workstation.");
		c = LayoutTools.makegbc(0, row, 1, 0, false);
		add(localTimeLabel, c);

		TextFieldListener actionListener = new TextFieldListener();
		TextFieldFocusListener focusListener = new TextFieldFocusListener();

		localTimeField = new JFormattedTextField(timeFormatter);
		localTimeField.addActionListener(actionListener);
		localTimeField.addFocusListener(focusListener);
		localTimeField.setValue(new Date());
		localTimeField.setToolTipText("Insert time at local workstation.");
		c = LayoutTools.makegbc(1, row, 2, 100, true);
		add(localTimeField, c);

		row++;
		localDateLabel = new JLabel("Date (mm/dd/yyyy): ");
		localDateLabel.setToolTipText("Date at local workstation.");
		c = LayoutTools.makegbc(0, row, 1, 0, false);
		add(localDateLabel, c);

		localDateField = new JFormattedTextField(dateFormatter);
		localDateField.addActionListener(actionListener);
		localDateField.addFocusListener(focusListener);
		localDateField.setToolTipText("Insert date at local workstation.");
		localDateField.setValue(new Date());
		c = LayoutTools.makegbc(1, row, 1, 100, true);
		add(localDateField, c);

		todayBox = new JCheckBox("Today");
		todayBox.addItemListener(new CheckBoxListener());
		todayBox.setToolTipText("Always use today as date.");
		c = LayoutTools.makegbc(2, row, 1, 50, false);
		add(todayBox, c);

		row++;
		utcLabel = new JLabel("UTC Time:");
		c = LayoutTools.makegbc(0, row, 1, 0, false);
		add(utcLabel, c);

		utcTimeLabel = new JLabel();
		utcTimeLabel.setToolTipText("Local time converted to UTC.");
		c = LayoutTools.makegbc(1, row, 1, 0, true);
		add(utcTimeLabel, c);
	}

	/**
	 *  Gets the date attribute of the HoldJobTestPanel object
	 *
	 *@return    The date value
	 */
	public Date getDate() {
		String dateString =
			localTimeField.getText().trim()
				+ " "
				+ localDateField.getText().trim();
		ParsePosition pos = new ParsePosition(0);
		return completeFormatter.parse(dateString, pos);
	}

	/**
	 *  Gets the useToday attribute of the DatePanel object
	 *
	 *@return    The useToday value
	 */
	public boolean getUseToday() {
		return todayBox.isSelected();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  date  The new date value
	 */
	public void setDate(Date date) {
		localDateField.setText(dateFormatter.format(date));
		localTimeField.setText(timeFormatter.format(date));
	}

	/**
	 *  Sets the enabled attribute of the HoldJobTestPanel object
	 *
	 *@param  enabled  The new enabled value
	 */
	public void setEnabled(boolean enabled) {
		localTimeLabel.setEnabled(enabled);
		localTimeField.setEnabled(enabled);
		localDateLabel.setEnabled(enabled);
		localDateField.setEnabled(enabled && !todayBox.isSelected());
		todayBox.setEnabled(enabled);
		utcLabel.setEnabled(enabled);
		utcTimeLabel.setEnabled(enabled);
	}

	/**
	 *  Sets the useToday attribute of the DatePanel object
	 *
	 *@param  useToday  The new useToday value
	 */
	public void setUseToday(boolean useToday) {
		todayBox.setSelected(useToday);
	}

	/**
	 *  Description of the Method
	 */
	private void updateUTCTime() {
		Date date = getDate();
		if (date != null) {
			utcTimeLabel.setText(utcFormatter.format(date));
		}
	}

	/**
	 *  Description of the Method
	 */
	public void updateValues() {
		if (todayBox.isSelected()) {
			localDateField.setText(dateFormatter.format(new Date()));
		}
		updateUTCTime();
	}
}
