package com.pallas.unicore.client.panels;

import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.pallas.unicore.container.ControlContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.extensions.TestType;

/**
 *@author     Ralf Ratering
 *@author     Sven Haubold
 *@author     Kirsten Foerster
 *@created    21. August 2001
 *@version    $Id: DoRepeatPanel.java,v 1.2 2003/06/30 06:54:43 rrate Exp $
 */
public class DoRepeatPanel extends GroupDependencyPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    2. August 2002
	 */
	private class SelectionListener implements ItemListener {

		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent ev) {
			if (ev.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}

			JComboBox testBox = (JComboBox) ev.getSource();
			TestType test = (TestType) testBox.getSelectedItem();

			if (ev.getSource() == returnCodeTestPanel.getTestBox()) {
				returnCodeTestPanel.setEnabled(test.isEnabled());
				if (test.isEnabled()) {
					fileTestPanel.setEnabled(false);
					timeTestPanel.setEnabled(false);
				}
			} else if (ev.getSource() == fileTestPanel.getTestBox()) {
				fileTestPanel.setEnabled(test.isEnabled());
				if (test.isEnabled()) {
					returnCodeTestPanel.setEnabled(false);
					timeTestPanel.setEnabled(false);
				}
			} else if (ev.getSource() == timeTestPanel.getTestBox()) {
				timeTestPanel.setEnabled(test.isEnabled());
				if (test.isEnabled()) {
					returnCodeTestPanel.setEnabled(false);
					fileTestPanel.setEnabled(false);
				}
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  args  Description of the Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("Do Repeat Panel");
		frame.getContentPane().add(
			new DoRepeatPanel(new JFrame(), new DoRepeatContainer()));
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent ev) {
				System.exit(0);
			}
		});

		frame.pack();
		frame.setVisible(true);
	}
	private FileTestPanel fileTestPanel;

	private ReturnCodeTestPanel returnCodeTestPanel;
	private TimeTestPanel timeTestPanel;

	/**
	 *  Constructor
	 *
	 *@param  parentFrame  reference to main JFrame
	 *@param  container    Description of the Parameter
	 */
	public DoRepeatPanel(JFrame parentFrame, DoRepeatContainer container) {
		super(parentFrame, container);
	}

	/**
	 *  Apply values from gui to JobContainer
	 */
	public void applyValues() {
		logger.fine("Aplying do repeat panel " + container);
		fileTestPanel.applyValues();
		timeTestPanel.applyValues();
		returnCodeTestPanel.applyValues();
		super.applyValues();
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected JPanel buildControlPanel() {
		JPanel panel = new JPanel(new GridLayout(3, 1, 10, 10));

		ControlContainer controlContainer = (ControlContainer) container;
		returnCodeTestPanel = new ReturnCodeTestPanel(controlContainer, true);
		fileTestPanel = new FileTestPanel(controlContainer, true);
		timeTestPanel = new TimeTestPanel(controlContainer, true);

		SelectionListener selectionListener = new SelectionListener();
		returnCodeTestPanel.getTestBox().addItemListener(selectionListener);
		fileTestPanel.getTestBox().addItemListener(selectionListener);
		timeTestPanel.getTestBox().addItemListener(selectionListener);

		setTitledBorder("Do Repeat");

		panel.add(returnCodeTestPanel);
		panel.add(fileTestPanel);
		panel.add(timeTestPanel);

		return panel;
	}

	/**
	 *  Fill values from JobContainer into gui
	 */
	public void resetValues() {
		logger.fine("Resetting do repeat panel " + container);
		DoRepeatContainer doRepeatContainer = (DoRepeatContainer) container;
		fileTestPanel.resetValues();
		timeTestPanel.resetValues();
		returnCodeTestPanel.resetValues();
		super.resetValues();
	}

	/**
	 *  Update dependency display
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public void updateValues(boolean vsiteChanged) {
		logger.fine(
			"Updating do repeat panel "
				+ container
				+ ". Vsite changed: "
				+ vsiteChanged);
		returnCodeTestPanel.updateValues(vsiteChanged);
		fileTestPanel.updateValues(vsiteChanged);
		timeTestPanel.updateValues(vsiteChanged);
		super.updateValues(vsiteChanged);
	}
}
