package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.pallas.unicore.container.FileExportContainer;

/**
 *  JPAPanel to define a file export from Uspace to Nspace or Xspace. This panel can be plugged into
 *  the clients data area and can be connected to a jpa tree node
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ExportJPAPanel.java,v 1.2 2003/06/30 06:54:42 rrate Exp $
 */
public class ExportJPAPanel extends JPAPanel {
	// text field for task name
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	// container to be modified
	private FileExportContainer container;

	// panel containing export table
	private FileExportPanel exportPanel;
	private JTextField nameTextField;

	/**
	 *  Constructor builds GUI
	 *
	 *@param  parentFrame      parent JFrame
	 *@param  container  container to be modified
	 */
	public ExportJPAPanel(JFrame parentFrame, FileExportContainer container) {
		super(parentFrame, container);

		this.container = container;

		buildComponents();

	}

	/**
	 *  Overwrite abstract method to apply the current GUI values to the container
	 */
	public void applyValues() {
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		exportPanel.applyValues();
	}

	/**
	 *  Build GUI
	 */
	private void buildComponents() {

		exportPanel = new FileExportPanel(container);

		nameTextField = new JTextField(20);
		nameTextField.setToolTipText(res.getString("ENTER_IDENTIFIER"));

		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel(res.getString("NAME")));
		namePanel.add(nameTextField);

		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.add(namePanel, BorderLayout.NORTH);
		subPanel.add(exportPanel, BorderLayout.CENTER);

		setTitledBorder(res.getString("EXPORT_BORDER"));
		add(subPanel, BorderLayout.CENTER);

		revalidate();
	}

	/**
	 *  Overwrite abstract method to reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		nameTextField.setText(container.getName());
		exportPanel.resetValues();
	}

	/**
	 *  Overwrite abstract method that will be called whenever the panel becomes visible
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
		exportPanel.updateValues();
	}

}
