package com.pallas.unicore.client.panels;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.Date;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;

import org.unicore.resources.Storage;
import org.unicore.resources.USpace;

import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystem;
import com.pallas.unicore.client.tables.ComboBoxCellRenderer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Panel for defining a file import from Nspace or Xspace into Uspace.
 *
 *@author     Ralf Ratering
 *@created    11. September 2001
 *@version    $Id: FileImportPanel.java,v 1.3 2003/06/30 06:54:41 rrate Exp $
 */
public class FileImportPanel extends FileStoragePanel {
	final static int COL_BINARY = 4;
	final static int COL_OVERWRITE = 3;
	final static int COL_SOURCE = 1;

	final static int COL_STORAGE = 0;
	final static int COL_TARGET = 2;

	private JComboBox storagesEditorComboBox;
	private ComboBoxCellRenderer storagesRendererComboBox;

	/**
	 *  Constructor
	 *
	 *@param  container  Description of Parameter
	 */
	public FileImportPanel(TaskContainer container) {
		this(container, null);
	}

	/**
	 *  Constructor for the FileImportPanel object
	 *
	 *@param  container         Description of the Parameter
	 *@param  initialDirectory  Description of the Parameter
	 */
	public FileImportPanel(TaskContainer container, String initialDirectory) {
		super(container, initialDirectory);
		buildComponents();
	}

	/**
	 *  Add and initialize a new empty row to the table
	 */
	protected void addEmptyRow() {
		Vector lastRowData = null;
		int selectedRow = table.getSelectedRow();
		if (selectedRow >= 0) {
			lastRowData =
				(Vector) tableModel.getDataVector().elementAt(selectedRow);
		}
		Vector rowData = new Vector();
		if (lastRowData == null) {
			rowData.add(FileStorage.NSPACE_STRING);
			rowData.add("");
			rowData.add("");
			rowData.add(Boolean.FALSE);
			rowData.add(Boolean.FALSE);
		} else {
			rowData.add(lastRowData.elementAt(COL_STORAGE));
			rowData.add("");
			rowData.add("");
			rowData.add(lastRowData.elementAt(COL_OVERWRITE));
			rowData.add(lastRowData.elementAt(COL_BINARY));
		}

		tableModel.addRow(rowData);

		selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 *  Apply current GUI entries to container
	 */
	public void applyValues() {
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}

		// loop over rows and fill in missing destination names
		int nrOfRows = table.getRowCount();
		FileImport[] fileImports = new FileImport[nrOfRows];
		for (int i = 0; i < nrOfRows; i++) {
			FileStorage fileop = (FileStorage) getFileOperationAt(i);
			/*
			 *  commented this out: empty destination means the Uspace
			 *  if(!fileop.getSourceName().equals("") && fileop.getDestinationName().equals("")) {
			 *  String destName = fileop.getSourceFileName();
			 *  fileop.setDestinationName(destName);
			 *  tableModel.setValueAt(destName, i, COL_TARGET);
			 *  }
			 */
			// put file op into array
			fileImports[i] = (FileImport) fileop;
		}

		// check if imports have changed
		updateModifiedTime(fileImports);

		// apply stuff to container
		container.setFileImports(fileImports);
		container.checkContents();
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		// minimize checkbox and combobox columns
		int storageChoice = 70;
		table.getColumnModel().getColumn(COL_STORAGE).setPreferredWidth(
			storageChoice);
		//        table.getColumnModel().getColumn(COL_STORAGE).setMinWidth(storageChoice);
		table.getColumnModel().getColumn(COL_STORAGE).setMaxWidth(
			storageChoice);
		int overwriteChoice = 115;
		table.getColumnModel().getColumn(COL_OVERWRITE).setPreferredWidth(
			overwriteChoice);
		//        table.getColumnModel().getColumn(COL_OVERWRITE).setMinWidth(overwriteChoice);
		table.getColumnModel().getColumn(COL_OVERWRITE).setMaxWidth(
			overwriteChoice);
		int binaryChoice = 55;
		table.getColumnModel().getColumn(COL_BINARY).setPreferredWidth(
			binaryChoice);
		//        table.getColumnModel().getColumn(COL_BINARY).setMinWidth(binaryChoice);
		table.getColumnModel().getColumn(COL_BINARY).setMaxWidth(binaryChoice);

		// build the combobox editors and renderers in tables
		storages = new Vector();
		storages.add(FileStorage.NSPACE_STRING);
		storagesEditorComboBox = new JComboBox(storages);
		storagesRendererComboBox = new ComboBoxCellRenderer(storages);
		table.getColumnModel().getColumn(COL_STORAGE).setCellEditor(
			new DefaultCellEditor(storagesEditorComboBox));
		table.getColumnModel().getColumn(COL_STORAGE).setCellRenderer(
			storagesRendererComboBox);
	}

	/**
	 *  Gets the columnNames attribute of the FileImportPanel object
	 *
	 *@return    The columnNames value
	 */
	protected String[] getColumnNames() {
		String[] columnNames =
			{
				res.getString("SOURCE"),
				res.getString("FILE_AT_SOURCE"),
				res.getString("USPACE_NAME"),
				res.getString("OVERWRITE"),
				res.getString("BINARY")};
		return columnNames;
	}

	/**
	 *  Gets the dataFlavor attribute of the FileImportPanel object
	 *
	 *@return    The dataFlavor value
	 */
	protected DataFlavor getDataFlavor() {
		return FileImport.getFlavor();
	}

	/**
	 *  Build a FileStorage object out of a certain table row
	 *
	 *@param  row  nr of row
	 *@return      new FileStorage object
	 */
	protected FileOperation getFileOperationAt(int row) {
		Object storageItem = getStorageItem(row);
		String sourceName = (String) tableModel.getValueAt(row, COL_SOURCE);
		String destinationName =
			(String) tableModel.getValueAt(row, COL_TARGET);
		boolean overwrite =
			((Boolean) tableModel.getValueAt(row, COL_OVERWRITE))
				.booleanValue();
		boolean isAscii =
			!((Boolean) tableModel.getValueAt(row, COL_BINARY)).booleanValue();
		return new FileImport(
			storageItem,
			sourceName,
			destinationName,
			overwrite,
			isAscii);
	}

	/**
	 *  Gets the storageItem attribute of the FileExportPanel object
	 *
	 *@param  row  Description of the Parameter
	 *@return      The storageItem value
	 */
	protected Object getStorageItem(int row) {
		return tableModel.getValueAt(row, COL_STORAGE);
	}

	/**
	 *  Reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}

		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}

		clearTable();

		FileImport[] fileImports = container.getFileImports();
		for (int i = 0; i < fileImports.length; i++) {
			addFileOperation(fileImports[i]);
		}
	}

	/**
	 *  Fill an existing table row with a certain FileExport object. If row
	 *  index is too high add a new row and use that one.
	 *
	 *@param  index   nr of table row
	 *@param  fileOp  The new fileOperationAt value
	 */
	protected void setFileOperationAt(FileOperation fileOp, int index) {
		FileImport fileImport = (FileImport) fileOp;

		String target = fileImport.getDestinationName();
		Boolean overwrite = new Boolean(fileImport.getOverwritePolicy());
		Boolean binary = new Boolean(!fileImport.getFileMode());

		// the table is edited
		if (index < table.getRowCount()) {
			String oldSource =
				(String) tableModel.getValueAt(index, COL_SOURCE);
			String oldTarget =
				(String) tableModel.getValueAt(index, COL_TARGET);

			// if table entry already contains a target name, keep it
			if (!oldTarget.equals("")) {
				target = oldTarget;
			}

			// if table entry already contains a source or target name, keep the flags
			if (!oldSource.equals("") || !oldTarget.equals("")) {
				overwrite =
					(Boolean) tableModel.getValueAt(index, COL_OVERWRITE);
				binary = (Boolean) tableModel.getValueAt(index, COL_BINARY);
			}
		}

		if (index > table.getRowCount() - 1) {
			tableModel.addRow(new Vector());
		}

		tableModel.setValueAt(fileImport.getStorageItem(), index, COL_STORAGE);
		tableModel.setValueAt(fileImport.getSourceName(), index, COL_SOURCE);
		tableModel.setValueAt(target, index, COL_TARGET);
		tableModel.setValueAt(overwrite, index, COL_OVERWRITE);
		tableModel.setValueAt(binary, index, COL_BINARY);
	}

	/**
	 *  Popup the remote file chooser
	 */
	protected void showBrowser() {
		File selectedFile = null;
		if (table.getSelectedRow() >= 0) {
			FileImport fileImport =
				(FileImport) getFileOperationAt(table.getSelectedRow());
			logger.fine("Selected file import: " + fileImport);
			if (fileImport != null) {
				Object storageItem = fileImport.getStorageItem();
				String filename = fileImport.getSourceName();
				if (storageItem.equals(FileStorage.NSPACE_STRING)) {
					selectedFile = new File(filename);
				} else {
					selectedFile =
						new RemoteFile(
							filename,
							(Storage) storageItem,
							new RemoteFileSystem(container.getVsite()));
				}
			}
		}
		logger.fine("Selected file: " + selectedFile);
		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(
				selectedFile,
				container.getVsite());
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.setTitle("UNICOREpro: Choose source for import");
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			// for now only single selection is supported
			File[] files = { chooser.getSelectedFile()};
			for (int i = 0; i < files.length; i++) {
				selectedFile = files[i];
				Object storageObject = FileStorage.NSPACE_STRING;
				String path = selectedFile.getPath();
				String name = selectedFile.getName();

				if (selectedFile instanceof RemoteFile) {
					RemoteFile file = (RemoteFile) selectedFile;
					storageObject = file.getStorage();
					path = file.getSubspacePath();
					// always add a slash to path for remote directories
					if (selectedFile.isDirectory()) {
						path += "/";
						name += "/";
					}
				} else {
					// for local directories add platform specific separator
					if (selectedFile.isDirectory()) {
						path += File.separator;
						name += File.separator;
					}
				}
				FileImport fileImport =
					new FileImport(storageObject, path, name);

				// if we have selected multiple files we want to add them
				// if we selected one table row and one file we wanted to edit
				if (table.getSelectedRowCount() == 1 && files.length == 1) {
					setFileOperationAt(fileImport, table.getSelectedRow());
				} else {
					addFileOperation(fileImport);
				}
			}
		}
	}

	/**
	 *@param  newImports  Description of Parameter
	 */
	private void updateModifiedTime(FileImport[] newImports) {
		FileImport[] oldImports = container.getFileImports();

		// first a simple check...
		if (oldImports.length != newImports.length) {
			container.setModifiedTime(new Date());
			return;
		}

		// ... now try to find matching imports
		// loop over old imports
		for (int i = 0; i < oldImports.length; i++) {
			boolean found = false;
			// loop over new imports
			for (int j = 0; j < newImports.length; j++) {
				// try to find matching new import
				if (newImports[j].equals(oldImports[i])) {
					// match was found
					found = true;
				}
			}
			if (!found) {
				// if we get here one import was new...
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 *  Update storage resource entries
	 */
	public void updateValues() {

		// get storage resources from currently selected vsite
		updateStorageResources();

		// replace existing storage resources
		replaceStorageResources(COL_STORAGE);

		for (int i = 0; i < storages.size(); i++) {
			if (storages.elementAt(i) instanceof USpace) {
				storages.remove(storages.elementAt(i));
				break;
			}
		}

		// update combo boxes
		storagesEditorComboBox.setModel(new DefaultComboBoxModel(storages));
		storagesRendererComboBox.setModel(new DefaultComboBoxModel(storages));
	}

}
