package com.pallas.unicore.client.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Date;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.tables.ComboBoxCellEditor;
import com.pallas.unicore.client.tables.ComboBoxCellRenderer;
import com.pallas.unicore.client.tables.FilePermission;
import com.pallas.unicore.client.tables.FilePermissionCellEditor;
import com.pallas.unicore.client.tables.FilePermissionPanel;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Panel to define a file operations in a certain file space (Uspace or Xspace)
 *
 *@author     Ralf Ratering
 *@author     Klaus-Dieter Oertel
 *@created    9. August 2002
 *@version    $Id: FileOperationPanel.java,v 1.3 2003/06/30 06:54:41 rrate Exp $
 */
public class FileOperationPanel extends FileStoragePanel {

	/**
	 *  Description of the Class
	 *
	 *@author     Klaus-Dieter Oertel
	 *@created    9. August 2002
	 */
	private class ComboBoxListener implements ItemListener {
		/**
		 *  Initialise the last column based on the operation choice
		 *
		 *@param  e  Description of Parameter
		 */

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				//int rows[] = table.getSelectedRows();
				//for (int i=0; i<rows.length; i++) { System.err.println("DESELECTED "+e.getItem()+" rows["+i+" = "+rows[i]);}
				return;
			}
			if (e.getSource() == operationComboBox) {
				//int rows[] = table.getSelectedRows();
				//for (int i=0; i<rows.length; i++) { System.err.println("  SELECTED "+e.getItem()+" rows["+i+" = "+rows[i]);}
				int row = table.getSelectedRow();
				if (e.getItem().equals(FileSystemOperation.CHMOD)) {
					tableModel.setValueAt(
						new FilePermission(),
						row,
						COL_TARGET);
				} else if (
					e.getItem().equals(FileSystemOperation.DELETE)
						|| e.getItem().equals(FileSystemOperation.MAKEDIR)) {
					tableModel.setValueAt(new String(""), row, COL_TARGET);
				} else {
					// What was the reason for the following line?
					// Object value = tableModel.getValueAt(row, COL_TARGET);
					Object value = tableModel.getValueAt(row, COL_OPERATION);
					if (FileSystemOperation.CHMOD.equals(value)
						|| FileSystemOperation.DELETE.equals(value)
						|| FileSystemOperation.MAKEDIR.equals(value)) {
						tableModel.setValueAt(new String(""), row, COL_TARGET);
					}
				}
				tableModel.fireTableRowsUpdated(row, row);
			}
		}
	}

	/**
	 *  Table with cell dependent renderers/editors
	 *
	 *@author     Klaus-Dieter Oertel
	 *@created    1. August 2002
	 */
	class FileOperationTable extends JTable {
		/**
		 *  Constructor for the FileOperationTable object
		 *
		 *@param  tableModel  Description of the Parameter
		 */
		public FileOperationTable(FileTableModel tableModel) {
			super(tableModel);
		}

		/**
		 *  Gets the TableCellEditor for the current cell
		 *
		 *@param  row  Description of Parameter
		 *@param  col  Description of Parameter
		 *@return      The TableCellEditor value
		 */

		public TableCellEditor getCellEditor(int row, int col) {
			/*
			 *  if (this.convertColumnIndexToModel(col) == COL_TARGET) {
			 *  / the editor choice is based on the class of the cell's value
			 *  return super.getDefaultEditor(table.getValueAt(row,col).getClass());
			 *  }
			 *  * alternative implementation without using the value's class
			 */
			if (convertColumnIndexToModel(col) == COL_TARGET) {
				if (tableModel
					.getValueAt(row, COL_OPERATION)
					.equals(FileSystemOperation.CHMOD)) {
					return permissionsEditor;
				} else {
					return stringEditor;
				}
			} else {
				return super.getCellEditor(row, col);
			}
		}

		/**
		 *  Gets the TableCellRenderer for the current cell
		 *
		 *@param  row  Description of Parameter
		 *@param  col  Description of Parameter
		 *@return      The CellRenderer value
		 */
		public TableCellRenderer getCellRenderer(int row, int col) {
			if (convertColumnIndexToModel(col) == COL_TARGET) {
				if (tableModel
					.getValueAt(row, COL_OPERATION)
					.equals(FileSystemOperation.CHMOD)) {
					return permissionsRenderer;
				} else {
					return stringRenderer;
				}
			} else {
				return super.getCellRenderer(row, col);
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     Ralf Ratering
	 *@created    21. August 2001
	 */
	private class FileOperationTableCellRenderer
		extends DefaultTableCellRenderer {
		/**
		 *  Gets the TableCellRendererComponent attribute of the
		 *  FileOperationTableCellRenderer object
		 *
		 *@param  table       Description of Parameter
		 *@param  value       Description of Parameter
		 *@param  isSelected  Description of Parameter
		 *@param  hasFocus    Description of Parameter
		 *@param  row         Description of Parameter
		 *@param  column      Description of Parameter
		 *@return             The TableCellRendererComponent value
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
			Component component =
				super.getTableCellRendererComponent(
					table,
					value,
					isSelected,
					hasFocus,
					row,
					column);

			if (column == COL_TARGET) {
				String selectedItem =
					(String) tableModel.getValueAt(row, COL_OPERATION);
				if (selectedItem.equals(FileSystemOperation.DELETE)
					|| selectedItem.equals(FileSystemOperation.MAKEDIR)) {
					component.setBackground(Color.LIGHT_GRAY);
				} else {
					if (isSelected) {
						component.setBackground(table.getSelectionBackground());
					} else {
						component.setBackground(Color.WHITE);
					}
				}
			}
			return component;
		}
	}

	/**
	 *  Table model for non file op table
	 *
	 *@author     Ralf Ratering
	 *@created    21. August 2001
	 */
	class FileOperationTableModel extends FileTableModel {
		/**
		 *  Constructor for the FileOperationTableModel object
		 *
		 *@param  columnNames  Description of Parameter
		 *@param  numRows      Description of Parameter
		 */
		public FileOperationTableModel(String[] columnNames, int numRows) {
			super(columnNames, numRows);
		}

		/**
		 *  Gets the CellEditable attribute of the FileOperationTableModel
		 *  object
		 *
		 *@param  row  Description of Parameter
		 *@param  col  Description of Parameter
		 *@return      The CellEditable value
		 */
		public boolean isCellEditable(int row, int col) {
			if (col != COL_TARGET) {
				return true;
			}
			String selectedItem =
				(String) tableModel.getValueAt(row, COL_OPERATION);
			if (selectedItem.equals(FileSystemOperation.DELETE)
				|| selectedItem.equals(FileSystemOperation.MAKEDIR)) {
				return false;
			} else {
				return true;
			}
		}
	}

	final static int COL_OPERATION = 0;
	final static int COL_SOURCE = 2;
	final static int COL_STORAGE = 1;
	final static int COL_TARGET = 3;

	private JComboBox operationComboBox;
	private Vector operations;
	private FilePermissionCellEditor permissionsEditor;
	private FilePermissionPanel permissionsRenderer;

	private JComboBox storagesEditorComboBox;
	private ComboBoxCellRenderer storagesRendererComboBox;
	private DefaultCellEditor stringEditor;
	private FileOperationTableCellRenderer stringRenderer;

	/**
	 *  Constructor
	 *
	 *@param  container  to be modified
	 */
	public FileOperationPanel(TaskContainer container) {
		this(container, null);
	}

	/**
	 *  Constructor for the FileOperationPanel object
	 *
	 *@param  container         Description of the Parameter
	 *@param  initialDirectory  Description of the Parameter
	 */
	public FileOperationPanel(
		TaskContainer container,
		String initialDirectory) {
		super(container, initialDirectory);
		operations = new Vector();
		String[] ops = FileSystemOperation.getOperations();
		for (int i = 0; i < ops.length; i++) {
			operations.add(ops[i]);
		}

		// set combo box of file system operations
		ComboBoxListener comboBoxListener = new ComboBoxListener();
		operationComboBox = new JComboBox(operations);
		operationComboBox.addItemListener(comboBoxListener);
		table.getColumnModel().getColumn(COL_OPERATION).setCellEditor(
			new ComboBoxCellEditor(operationComboBox));
		table.getColumnModel().getColumn(COL_OPERATION).setCellRenderer(
			new ComboBoxCellRenderer(operations));

		stringRenderer = new FileOperationTableCellRenderer();
		stringEditor = new DefaultCellEditor(new JTextField());
		permissionsRenderer = new FilePermissionPanel();
		permissionsEditor =
			new FilePermissionCellEditor(new FilePermissionPanel());
		/*
		 *  The documentation of javax.swing.table.TableColumn promises that a renderer/editor selection based on the
		 *  classes of cell values is possible. Actually this is not true because JTable.getCellRenderer(row,col) evaluates
		 *  the class only from the column by calling getColumnClass(col) and NOT by getValueAt(row,col).getClass().
		 *  Therefore JTable.getCellRenderer(row,col) has to be overwritten in FileOperationTable.
		 *  In the hope of future releases of JTable.getCellRenderer(row,col) we keep the approach of mapping the
		 *  class values to certain renderers in the following (which is evaluated then in
		 *  FileOperationTable.getCellRenderer(row,col) where we could set the Renderer directly, however.)
		 */
		/*
		 *  table.setDefaultRenderer(String.class,   stringRenderer);
		 *  table.setDefaultRenderer(FilePermission.class, permissionsRenderer);
		 *  table.setDefaultEditor  (FilePermission.class, permissionsEditor);
		 */

		buildComponents();
	}

	/**
	 *  Add an empty row at the table end
	 */
	protected void addEmptyRow() {
		Vector rowData = new Vector();
		rowData.add(operations.elementAt(0));
		rowData.add(storages.elementAt(0));
		// storages contains at least res.getString("NONE")
		rowData.add("");
		if (FileSystemOperation.CHMOD.equals(operations.elementAt(0))) {
			rowData.add(new FilePermission());
		} else {
			rowData.add("");
		}

		tableModel.addRow(rowData);
		int selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 *  Apply gui values to container
	 */
	public void applyValues() {
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}

		// loop over rows
		int nrOfRows = table.getRowCount();
		FileSystemOperation[] fileOps = new FileSystemOperation[nrOfRows];
		for (int i = 0; i < nrOfRows; i++) {
			fileOps[i] = (FileSystemOperation) getFileOperationAt(i);
		}

		updateModifiedTime(fileOps);

		container.setFileSystemOperations(fileOps);
		container.checkContents();
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		// minimize combobox columns
		int operationChoice = 135;
		table.getColumnModel().getColumn(COL_OPERATION).setPreferredWidth(
			operationChoice);
		//        table.getColumnModel().getColumn(COL_OPERATION).setMinWidth(operationChoice);
		table.getColumnModel().getColumn(COL_OPERATION).setMaxWidth(
			operationChoice);
		int storageChoice = 90;
		table.getColumnModel().getColumn(COL_STORAGE).setPreferredWidth(
			storageChoice);
		//        table.getColumnModel().getColumn(COL_STORAGE).setMinWidth(storageChoice);
		table.getColumnModel().getColumn(COL_STORAGE).setMaxWidth(
			storageChoice);

		// build the combobox editors and renderers in tables
		storages = new Vector();
		storages.add(FileStorage.NSPACE_STRING);
		storagesEditorComboBox = new JComboBox(storages);
		storagesRendererComboBox = new ComboBoxCellRenderer(storages);
		table.getColumnModel().getColumn(COL_STORAGE).setCellEditor(
			new DefaultCellEditor(storagesEditorComboBox));
		table.getColumnModel().getColumn(COL_STORAGE).setCellRenderer(
			storagesRendererComboBox);
	}

	/**
	 *  Allow overwrite in sub-classes Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected JTable createTable() {
		return new FileOperationTable(tableModel);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected FileTableModel createTableModel() {
		columnNames = getColumnNames();
		return new FileOperationTableModel(columnNames, 0);
	}

	/**
	 *  Gets the columnNames attribute of the FileOperationPanel object
	 *
	 *@return    The columnNames value
	 */
	protected String[] getColumnNames() {
		String[] columnNames =
			{ "Operation", "File System", "Filename", "Second Parameter" };
		return columnNames;
	}

	/**
	 *  Gets the dataFlavor attribute of the FileOperationPanel object
	 *
	 *@return    The dataFlavor value
	 */
	protected DataFlavor getDataFlavor() {
		return FileSystemOperation.getFlavor();
	}

	/**
	 *  Get file operation from a certain row
	 *
	 *@param  row  index of row
	 *@return      file operation at selected row
	 */
	protected FileOperation getFileOperationAt(int row) {
		String operation = (String) tableModel.getValueAt(row, COL_OPERATION);
		Object storageItem = tableModel.getValueAt(row, COL_STORAGE);
		if (storageItem.equals(res.getString("NONE"))) {
			storageItem = null;
		}
		String sourceName = (String) tableModel.getValueAt(row, COL_SOURCE);
		// replace FilePermission by String representation
		String destinationName =
			tableModel.getValueAt(row, COL_TARGET).toString();
		return new FileSystemOperation(
			sourceName,
			operation,
			destinationName,
			storageItem);
	}

	/**
	 *  Reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}

		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}

		clearTable();

		FileSystemOperation[] fileOps = container.getFileSystemOperations();
		for (int i = 0; i < fileOps.length; i++) {
			addFileOperation(fileOps[i]);
		}
	}

	/**
	 *  Fill an existing table row with a certain FileExport object. If row
	 *  index is too high add a new row and use that one.
	 *
	 *@param  index   nr of table row
	 *@param  fileOp  The new fileOperationAt value
	 */
	protected void setFileOperationAt(FileOperation fileOp, int index) {
		FileSystemOperation fileSysOp = (FileSystemOperation) fileOp;

		String operation = fileSysOp.getOperation();
		Object target = fileSysOp.getDestinationName();

		// the table is edited
		if (index < table.getRowCount()) {
			String oldSource =
				(String) tableModel.getValueAt(index, COL_SOURCE);

			// if table entry already contains a source, keep the operation and target
			if (!oldSource.equals("")) {
				operation =
					(String) tableModel.getValueAt(index, COL_OPERATION);
				target = tableModel.getValueAt(index, COL_TARGET);
			}
		}

		if (index > table.getRowCount() - 1) {
			tableModel.addRow(new Vector());
		}

		tableModel.setValueAt(operation, index, COL_OPERATION);
		tableModel.setValueAt(fileSysOp.getStorageItem(), index, COL_STORAGE);
		tableModel.setValueAt(fileSysOp.getSourceName(), index, COL_SOURCE);
		if (operation.equals(FileSystemOperation.CHMOD)
			&& !(target instanceof FilePermission)) {
			FilePermission permission = new FilePermission();
			permission.setPermission((String) target);
			tableModel.setValueAt(permission, index, COL_TARGET);
		} else {
			tableModel.setValueAt(target, index, COL_TARGET);
		}
	}

	/**
	 *  Description of the Method
	 */
	protected void showBrowser() {
		if (container.getVsite() == null) {
			UserMessages.info("Please select a virtual site first.");
			return;
		}
		if (!ResourceManager.isVsiteAvailable(container.getVsite())) {
			UserMessages.info(
				"Virtual site " + container.getVsite() + " is not available.");
			return;
		}
		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(null, container.getVsite());
		chooser.setTitle("UNICOREpro: Choose source for file operation");
		chooser.setLocalEnabled(false);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			// for now only single selection is supported
			File[] files = { chooser.getSelectedFile()};
			for (int i = 0; i < files.length; i++) {
				FileSystemOperation fileOp = null;
				if (files[i] instanceof RemoteFile) {
					RemoteFile file = (RemoteFile) files[i];
					fileOp =
						new FileSystemOperation(
							file.getSubspacePath(),
							FileSystemOperation.COPY,
							file.getSubspacePath(),
							file.getStorage());
				}

				// if we have selected multiple files we want to add them
				// if we selected one table row and one file we wanted to edit
				if (table.getSelectedRowCount() == 1 && files.length == 1) {
					setFileOperationAt(fileOp, table.getSelectedRow());
				} else {
					addFileOperation(fileOp);
				}
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  newOps  Description of Parameter
	 */
	private void updateModifiedTime(FileSystemOperation[] newOps) {
		FileSystemOperation[] oldOps = container.getFileSystemOperations();

		// first a simple check...
		if (oldOps.length != newOps.length) {
			container.setModifiedTime(new Date());
			return;
		}

		// ... now try to find matching ops
		// loop over old ops
		for (int i = 0; i < oldOps.length; i++) {
			boolean found = false;
			// loop over new ops
			for (int j = 0; j < newOps.length; j++) {
				// try to find matching new import
				if (newOps[j].equals(oldOps[i])) {
					// match was found
					found = true;
				}
			}
			if (!found) {
				// if we get here one op was new...
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 *  Get current storage resources from ResourceManager
	 */
	public void updateStorageResources() {
		//storages.update(container.getVsite());
		storages = new Vector();
		// local?
		//storages.add(FileStorage.NSPACE_STRING);

		NamedResourceSet resourceSet =
			ResourceManager.getResourceSet(container.getVsite());
		resourceSet.addStorageResources(storages);

		if (storages.size() == 0) {
			storages.add(res.getString("NONE"));
			return;
		}

		// exclude Spool and AlternativeUspace
		Vector cloned = (Vector) storages.clone();
		for (int i = 0; i < cloned.size(); i++) {
			if (cloned.elementAt(i) instanceof org.unicore.resources.Spool
				|| cloned.elementAt(i)
					instanceof org.unicore.resources.AlternativeUspace) {
				storages.remove(cloned.elementAt(i));
			}
		}
		cloned.clear();
	}

	/**
	 *  Update storage resources
	 */
	public void updateValues() {
		updateStorageResources();

		replaceStorageResources(COL_STORAGE);

		// update combo boxes
		storagesEditorComboBox.setModel(new DefaultComboBoxModel(storages));
		storagesRendererComboBox.setModel(new DefaultComboBoxModel(storages));
	}
}
