package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import com.pallas.unicore.client.controls.SimpleToolBar;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.TransferableVector;

/**
 *  Base class for import and export panels
 *
 *@author     Ralf Ratering
 *@created    21. August 2001
 *@version    $Id: FilePanel.java,v 1.30 2001/11/20 12:40:54 rrate Exp
 */
public abstract class FilePanel extends JPanel implements ClipboardOwner {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    4. Juni 2002
	 */
	class AddAction extends AbstractAction {
		/**
		 *  Constructor for the AddAction object
		 */
		public AddAction() {
			super("Add", ResourceManager.getIcon(ResourceManager.ADD));
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			if (table.getCellEditor() != null) {
				table.getCellEditor().stopCellEditing();
			}
			addEmptyRow();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class CopyAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public CopyAction() {
			super("Copy", ResourceManager.getIcon(ResourceManager.COPY));
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			copy();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class CutAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public CutAction() {
			super("Cut", ResourceManager.getIcon(ResourceManager.CUT));
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			cut();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    21. August 2001
	 */
	class FileTableModel extends DefaultTableModel {
		/**
		 *  Constructor for the FileTableModel object
		 *
		 *@param  names  Description of Parameter
		 *@param  rows   Description of Parameter
		 */
		public FileTableModel(Object[] names, int rows) {
			super(names, rows);
		}

		/**
		 *  JTable uses this method to determine the default renderer editor for
		 *  each cell. If we didn't implement this method, then the last column
		 *  would contain text ("true"/"false"), rather than a check box.
		 *
		 *@param  c  Description of Parameter
		 *@return    The ColumnClass value
		 */
		public Class getColumnClass(int c) {
			Object columnObject = getValueAt(0, c);
			if (columnObject == null) {
				String dummy = new String();
				return dummy.getClass();
			} else {
				return columnObject.getClass();
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    19. August 2002
	 */
	private class MyKeyListener extends KeyAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void keyPressed(KeyEvent e) {
			int keyCode = e.getKeyCode();
			int modifiers = e.getModifiers();
			// increase id
			eventId++;
			if (keyCode == KeyEvent.VK_N && modifiers == KeyEvent.CTRL_MASK) {
				addAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (
				keyCode == KeyEvent.VK_X && modifiers == KeyEvent.CTRL_MASK) {
				cutAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (
				keyCode == KeyEvent.VK_C && modifiers == KeyEvent.CTRL_MASK) {
				copyAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (
				keyCode == KeyEvent.VK_V && modifiers == KeyEvent.CTRL_MASK) {
				pasteAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (keyCode == KeyEvent.VK_DELETE && modifiers == 0) {
				removeAction.actionPerformed(
					new ActionEvent(this, eventId, ""));
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class PasteAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public PasteAction() {
			super("Paste", ResourceManager.getIcon(ResourceManager.PASTE));
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			paste();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    5. Juni 2002
	 */
	class PopupMouseListener extends MouseAdapter {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			testShowPopup(e);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mousePressed(MouseEvent e) {
			testShowPopup(e);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mouseReleased(MouseEvent e) {
			testShowPopup(e);
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    4. Juni 2002
	 */
	class RemoveAction extends AbstractAction {
		/**
		 *  Constructor for the RemoveAction object
		 */
		public RemoveAction() {
			super("Remove", ResourceManager.getIcon(ResourceManager.REMOVE));
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			remove();
		}
	}

	/**
	 *  Description of the Field
	 */
	protected static Clipboard clipboard = new Clipboard("filePanelClipboard");
	// id for key listener actions
	private static int eventId = 0;
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	/**
	 *  The main program for the FilePanel class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(new JTable(0, 5), BorderLayout.CENTER);
		panel.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				System.err.println("MOUSIIIIII!");
			}
		});
		panel.setPreferredSize(new Dimension(400, 400));

		JFrame app = new JFrame();
		app.getContentPane().add(panel);
		app.pack();
		app.show();

	}

	private AddAction addAction = new AddAction();
	/**
	 *  Description of the Field
	 */
	protected String[] columnNames;

	/**
	 *  container to be modified
	 */
	protected TaskContainer container;
	private CopyAction copyAction = new CopyAction();
	private CutAction cutAction = new CutAction();
	private MyKeyListener keyListener = new MyKeyListener();
	private PasteAction pasteAction = new PasteAction();

	/**
	 *  Description of the Field
	 */
	protected JPopupMenu popup;

	private PopupMouseListener popupMouseListener = new PopupMouseListener();
	private RemoveAction removeAction = new RemoveAction();
	/**
	 *  table containing file operations
	 */
	protected JTable table;

	/**
	 *  Description of the Field
	 */
	protected FileTableModel tableModel;

	/**
	 *  Description of the Field
	 */
	protected SimpleToolBar toolbar;

	/**
	 *  Constructor for the FilePanel object
	 *
	 *@param  container  Description of the Parameter
	 */
	public FilePanel(TaskContainer container) {
		this.container = container;
		buildTable();
		buildPopupMenu();
		buildToolBar();
	}

	/**
	 *  Add an empty row to the table
	 */
	protected abstract void addEmptyRow();

	/**
	 *  Add a new table row and fill it with a file import
	 *
	 *@param  fileOperation  The feature to be added to the FileOperation
	 *      attribute
	 */
	public void addFileOperation(FileOperation fileOperation) {
		setFileOperationAt(fileOperation, tableModel.getRowCount());
	}

	/**
	 *  Apply gui values to container
	 */
	public abstract void applyValues();

	/**
	 *  Description of the Method
	 */
	protected void buildPopupMenu() {
		popup = new JPopupMenu();
		popup.setInvoker(this);
		popup.add(addAction);
		popup.add(cutAction);
		popup.add(copyAction);
		popup.add(pasteAction);
		popup.add(removeAction);
	}

	/**
	 *  Description of the Method
	 */
	protected void buildTable() {
		tableModel = createTableModel();

		table = createTable();
		table.setRowHeight(20);
		table.addMouseListener(popupMouseListener);
		table.addKeyListener(keyListener);

		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.addMouseListener(popupMouseListener);

		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);

	}

	/**
	 *  Description of the Method
	 */
	protected void buildToolBar() {
		toolbar = new SimpleToolBar();
		toolbar.add(addAction);
		toolbar.add(removeAction);
		add(toolbar, BorderLayout.NORTH);
	}

	/**
	 *  Remove all table entries
	 */
	protected void clearTable() {
		int nrOfRows = table.getRowCount();
		while (nrOfRows > 0) {
			tableModel.removeRow(nrOfRows - 1);
			nrOfRows = table.getRowCount();
		}
	}

	/**
	 *  Description of the Method
	 */
	protected void copy() {
		int[] rows = table.getSelectedRows();
		if (rows.length <= 0) {
			return;
		}
		TransferableVector vector = new TransferableVector(getDataFlavor());
		for (int i = 0; i < rows.length; i++) {
			FileOperation fileOp = getFileOperationAt(rows[i]);
			try {
				vector.add((Transferable) fileOp.clone());
			} catch (UnsupportedFlavorException e) {
				logger.log(
					Level.WARNING,
					"Wrong type of file operation copied.",
					e);
			}
		}
		clipboard.setContents(vector, this);
	}

	/**
	 *  Allow overwrite in sub-classes Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected JTable createTable() {
		return new JTable(tableModel);
	}

	/**
	 *  Allow overwrite in sub-classes Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected FileTableModel createTableModel() {
		columnNames = getColumnNames();
		return new FileTableModel(columnNames, 0);
	}

	/**
	 *  Description of the Method
	 */
	protected void cut() {
		copy();
		remove();
	}

	/**
	 *  Gets the columnNames attribute of the FilePanel object
	 *
	 *@return    The columnNames value
	 */
	protected abstract String[] getColumnNames();

	/**
	 *  Gets the dataFlavor attribute of the FilePanel object
	 *
	 *@return    The dataFlavor value
	 */
	protected abstract DataFlavor getDataFlavor();

	/**
	 *  Get file storage object at a certain row
	 *
	 *@param  row  nr of row
	 *@return      The FileStorageAt value
	 */
	protected abstract FileOperation getFileOperationAt(int row);

	/**
	 *  Description of the Method
	 *
	 *@param  clipboard     Description of the Parameter
	 *@param  transferable  Description of the Parameter
	 */
	public void lostOwnership(Clipboard clipboard, Transferable transferable) {
		logger.warning(
			"FilePanel Clipboard " + clipboard.getName() + " lost ownership!");
	}

	/**
	 *  Description of the Method
	 */
	protected void paste() {
		Transferable clipboardContent = clipboard.getContents(this);
		try {
			// get node from clipboard
			TransferableVector vector =
				(TransferableVector) clipboardContent.getTransferData(
					getDataFlavor());
			for (int i = 0; i < vector.size(); i++) {
				FileOperation fileOp = (FileOperation) vector.elementAt(i);
				addFileOperation((FileOperation) fileOp.clone());
			}
		} catch (Exception e) {
			logger.log(Level.WARNING, "Could not paste file operation.", e);
		}

	}

	/**
	 *  Description of the Method
	 */
	protected void remove() {
		while (table.getSelectedRow() > -1) {
			tableModel.removeRow(table.getSelectedRow());
		}
		int selectedRow = table.getRowCount() - 1;
		if (selectedRow >= 0) {
			table.setRowSelectionInterval(selectedRow, selectedRow);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  fileOperation  Description of the Parameter
	 */
	public void removeFileOperation(FileOperation fileOperation) {
		for (int i = 0; i < tableModel.getRowCount(); i++) {
			if (getFileOperationAt(i).equals(fileOperation)) {
				tableModel.removeRow(i);
				// decrease i, because of deleted row
				i--;
			}
		}
	}

	/**
	 *  Apply container entries to gui elements
	 */
	public abstract void resetValues();

	/**
	 *  Sets the FileStorageAt attribute of the FilePanel object
	 *
	 *@param  row     The new FileStorageAt value
	 *@param  fileOp  The new fileOperationAt value
	 */
	protected abstract void setFileOperationAt(FileOperation fileOp, int row);

	/**
	 *@param  evt  Description of Parameter
	 */
	private void testShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			Transferable vector = clipboard.getContents(this);
			pasteAction.setEnabled(
				vector != null
					&& vector.isDataFlavorSupported(getDataFlavor()));
			popup.show(this, evt.getX(), evt.getY());
		}
	}

	/**
	 *  Update storage resource entries and build dependent GUI elements
	 *  like combobox editors and renderers
	 */
	public abstract void updateValues();

}
