package com.pallas.unicore.client.panels;

import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ResourceBundle;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.pallas.unicore.client.controls.FontSelectionComboBox;

/**
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FontSelectionPanel.java,v 1.2 2003/06/30 06:54:38 rrate Exp $
 */
public class FontSelectionPanel extends JPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class SelectionListener implements ItemListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			firePropertyChange("FONT_CHANGED", null, getSelectedFont());
		}
	}
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	/**
	 *  Description of the Method
	 *
	 *@param  args  Description of Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("Test font selector");
		frame.getContentPane().add(new FontSelectionPanel());
		frame.pack();
		frame.show();
	}
	private FontSelectionComboBox fontComboBox;
	private SelectionListener listener;
	private JComboBox sizeComboBox;
	private JLabel sizeLabel;

	/**
	 *  Constructor for the FontSelectionPanel object
	 */
	public FontSelectionPanel() {
		this(false);
	}

	/**
	 *  Constructor for the FontSelectionPanel object
	 *
	 *@param  preview  Description of Parameter
	 */
	public FontSelectionPanel(boolean preview) {
		fontComboBox = new FontSelectionComboBox(preview);
		sizeComboBox = new JComboBox();
		listener = new SelectionListener();
		buildComponents();
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		for (int i = 8; i < 33; i++) {
			sizeComboBox.addItem(new Integer(i));
		}

		fontComboBox.addItemListener(listener);
		sizeComboBox.addItemListener(listener);

		JPanel subPanel = new JPanel();
		subPanel.add(fontComboBox);
		sizeLabel = new JLabel(res.getString("SIZE"));
		subPanel.add(sizeLabel);
		subPanel.add(sizeComboBox);

		add(subPanel);
	}

	/**
	 *  Gets the fontComboBox attribute of the FontSelectionPanel object
	 *
	 *@return    The fontComboBox value
	 */
	public FontSelectionComboBox getFontComboBox() {
		return this.fontComboBox;
	}

	/**
	 *  Gets the SelectedFont attribute of the FontSelectionPanel object
	 *
	 *@return    The SelectedFont value
	 */
	public Font getSelectedFont() {
		String fontName = (String) fontComboBox.getSelectedItem();
		Integer size = (Integer) sizeComboBox.getSelectedItem();
		return new Font(fontName, Font.PLAIN, size.intValue());
	}

	/**
	 *  Gets the sizeComboBox attribute of the FontSelectionPanel object
	 *
	 *@return    The sizeComboBox value
	 */
	public JComboBox getSizeComboBox() {
		return this.sizeComboBox;
	}

	/**
	 *  Gets the sizeLabel attribute of the FontSelectionPanel object
	 *
	 *@return    The sizeLabel value
	 */
	public JLabel getSizeLabel() {
		return this.sizeLabel;
	}

	/**
	 *  Sets the Enabled attribute of the FontSelectionPanel object
	 *
	 *@param  enabled  The new Enabled value
	 */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		fontComboBox.setEnabled(enabled);
		sizeComboBox.setEnabled(enabled);
	}

	/**
	 *  Sets the SelectedFont attribute of the FontSelectionPanel object
	 *
	 *@param  font  The new SelectedFont value
	 */
	public void setSelectedFont(Font font) {
		fontComboBox.setSelectedItem(font.getFontName());
		sizeComboBox.setSelectedItem(new Integer(font.getSize()));
		repaint();
	}
}
