package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.Date;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.pallas.unicore.container.GroupContainer;

/**
 *  Class for control group panels
 *
 *@author     Ralf Ratering
 *@created    21. August 2001
 *@version    $Id: GroupPanel.java,v 1.2 2003/06/30 06:54:37 rrate Exp $
 */
public class GroupPanel extends JPAPanel {

	/**
	 *  container to be modified
	 */
	protected GroupContainer container;

	private boolean editableNameField;

	/**
	 *  name for group
	 */
	protected JTextField groupName;

	/**
	 *  Constructor for the GroupPanel object
	 *
	 *@param  parentFrame  Description of the Parameter
	 *@param  container    Description of the Parameter
	 */
	public GroupPanel(JFrame parentFrame, GroupContainer container) {
		this(parentFrame, container, true);
	}

	/**
	 *  Constructor
	 *
	 *@param  parentFrame        reference to main JFrame
	 *@param  container          Description of the Parameter
	 *@param  editableNameField  Description of the Parameter
	 */
	public GroupPanel(
		JFrame parentFrame,
		GroupContainer container,
		boolean editableNameField) {
		super(parentFrame, container);
		this.container = container;
		this.editableNameField = editableNameField;
		// build gui
		buildComponents();
	}

	/**
	 *  Apply values from gui to JobContainer
	 */
	public void applyValues() {
		// remove spaces from text field
		groupName.setText(groupName.getText().trim().replace(' ', '_'));
		container.setName(groupName.getText());
		//container.checkContents();
		updateModifiedTime();
	}

	/**
	 *  Build GUI
	 */
	protected void buildComponents() {

		add(buildNamePanel(), BorderLayout.NORTH);

		// split pane between lists and dependency display
		JPanel controlPanel = buildControlPanel();
		if (controlPanel != null) {
			add(controlPanel, BorderLayout.CENTER);
		}

	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected JPanel buildControlPanel() {
		return null;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected JPanel buildNamePanel() {

		groupName = new JTextField(15);
		groupName.setToolTipText("Name for group");
		groupName.setEnabled(editableNameField);

		JPanel groupNamePanel = new JPanel();
		groupNamePanel.add(new JLabel("Name"));
		groupNamePanel.add(groupName);

		return groupNamePanel;
	}

	/**
	 *  Gets the groupName attribute of the GroupPanel object
	 *
	 *@return    The groupName value
	 */
	public String getGroupName() {
		return groupName.getText();
	}

	/**
	 *  Fill values from JobContainer into gui
	 */
	public void resetValues() {
		String name = container.getName();
		if (name != null) {
			groupName.setText(name);
		}
	}

	/**
	 *  Description of the Method
	 */
	protected void updateModifiedTime() {
		if (container.getName() == null
			|| !container.getName().equals(groupName.getText())) {
			container.setModifiedTime(new Date());
			return;
		}
	}

	/**
	 *  recurively inform child panels about vsite change
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public void updateValues(boolean vsiteChanged) {

	}

}
