package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.container.HoldJobContainer;

/**
 *@author     Ralf Ratering
 *@author     Sven Haubold
 *@created    21. August 2001
 *@version    $Id: HoldJobPanel.java,v 1.2 2003/06/30 06:54:37 rrate Exp $
 */
public class HoldJobPanel extends JPAPanel {
	private HoldJobContainer container;
	// text fields for script filename and task name
	private JTextField nameTextField;

	private HoldJobTestPanel testPanel;

	/**
	 *  Constructor
	 *
	 *@param  parentFrame  reference to main JFrame
	 *@param  container    Description of the Parameter
	 */
	public HoldJobPanel(JFrame parentFrame, HoldJobContainer container) {
		super(parentFrame, container);
		this.container = container;
		buildComponents();
	}

	/**
	 *  Apply values from gui to JobContainer
	 */
	public void applyValues() {
		// replace spaces in script name with _
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		testPanel.applyValues();
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		nameTextField = new JTextField(15);
		nameTextField.setToolTipText("Enter name for hold job task");

		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel("Name: "));
		namePanel.add(nameTextField);

		testPanel = new HoldJobTestPanel(container);
		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.setBorder(
			BorderFactory.createMatteBorder(
				20,
				100,
				0,
				100,
				this.getBackground()));
		subPanel.add(testPanel, BorderLayout.NORTH);
		add(namePanel, BorderLayout.NORTH);
		add(subPanel, BorderLayout.CENTER);

		setBorder(new TitledBorder("Hold Job"));

	}

	/**
	 *  Fill values from JobContainer into gui
	 */
	public void resetValues() {
		testPanel.resetValues();
		nameTextField.setText(container.getName());
	}

	/**
	 *  Description of the Method
	 *
	 *@param  vsiteChanged  Description of the Parameter
	 */
	public void updateValues(boolean vsiteChanged) {
		testPanel.updateValues(vsiteChanged);
	}

}
