package com.pallas.unicore.client.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JPanel;

/**
 *  Panel displaying an Image
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ImagePanel.java,v 1.5 2003/07/11 09:04:55 rrate Exp $
 */
public class ImagePanel extends JPanel {
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");
	final static long serialVersionUID = 1384825511689110745L;

	// image width in relation to image height
	private double aspectRatio;

	// image to be displayed
	private Image image;

	// image height
	private int imgHeight;

	// image width
	private int imgWidth;

	// fit image to panel or not
	private boolean resizeOn = true;

	// use tracker to wait for image loading
	private MediaTracker tracker;

	
	/**
	 *  Constructor for the ImagePanel object
	 *
	 *@param  resizeOn  Description of Parameter
	 */
	public ImagePanel(boolean resizeOn) {
		this.resizeOn = resizeOn;
		tracker = new MediaTracker(this);
		setBackground(Color.white);
	}

	/**
	 *  Constructor taking image as input
	 *
	 *@param  image     Image to be displayed
	 *@param  resizeOn  Description of Parameter
	 */
	public ImagePanel(Image image, boolean resizeOn) {
		this(resizeOn);
		setImage(image);
	}

	/**
	 *  Draw image centered into panel
	 *
	 *@param  g  graphics context
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		if (image == null) {
			return;
		}

		Dimension d = getSize();
		int width = d.width;
		int height = d.height;

		int centerX = (int) (width / 2.0);
		int centerY = (int) (height / 2.0);

		// shrink logo to panel size
		if (resizeOn) {
			g.drawImage(
				image,
				centerX - width / 2,
				centerY - (int) (width / aspectRatio) / 2,
				width,
				(int) (width / aspectRatio),
				this);
		} else {
			g.drawImage(
				image,
				centerX - imgWidth / 2,
				centerY - imgHeight / 2,
				this);
		}
	}

	/**
	 *  Sets the Image attribute of the ImagePanel object
	 *
	 *@param  image  The new Image value
	 */
	public void setImage(Image image) {
		// wait for image to be loaded
		tracker.addImage(image, 0);
		try {
			tracker.waitForID(0);
		}
		catch(InterruptedException e) {
			logger.log(Level.SEVERE, "Tracker interrupted.", e);
		}

		this.image = image;
		imgWidth = image.getWidth(this);
		imgHeight = image.getHeight(this);
		aspectRatio = imgWidth / imgHeight;
		if (!resizeOn) {
			setPreferredSize(new Dimension(imgWidth, imgHeight));
		}

	}

	/**
	 *  Set flag if image is resized to panel size
	 *
	 *@param  resizeOn  true, if image should be resized
	 */
	public void setResizeOn(boolean resizeOn) {
		this.resizeOn = resizeOn;
		repaint();
	}
}
