package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.pallas.unicore.container.FileImportContainer;

/**
 *  JPAPanel to define a file import from Nspace or Xspace into Uspace This panel can be plugged
 *  into the clients data area and can be connected to a jpa tree node
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ImportJPAPanel.java,v 1.2 2003/06/30 06:54:35 rrate Exp $
 */
public class ImportJPAPanel extends JPAPanel {
	// text field for task name
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	// container to be modified
	private FileImportContainer container;

	// panel containing file imports
	private FileImportPanel importPanel;
	private JTextField nameTextField;

	/**
	 *  Constructor
	 *
	 *@param  parentFrame  parent JFrame
	 *@param  container    Description of Parameter
	 */
	public ImportJPAPanel(JFrame parentFrame, FileImportContainer container) {
		super(parentFrame, container);

		this.container = container;

		buildComponents();

	}

	/**
	 *  Overwrite abstract method to apply the current GUI values to the container
	 */
	public void applyValues() {
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		importPanel.applyValues();
	}

	/**
	 *  Build GUI
	 */
	private void buildComponents() {
		importPanel = new FileImportPanel(container);

		nameTextField = new JTextField(20);
		nameTextField.setToolTipText(res.getString("ENTER_IDENTIFIER"));

		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel(res.getString("NAME")));
		namePanel.add(nameTextField);

		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.add(namePanel, BorderLayout.NORTH);
		subPanel.add(importPanel, BorderLayout.CENTER);

		setTitledBorder(res.getString("IMPORT_BORDER"));
		add(subPanel, BorderLayout.CENTER);

		revalidate();
	}

	/**
	 *  Overwrite abstract method to reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		nameTextField.setText(container.getName());
		importPanel.resetValues();
	}

	/**
	 *  Overwrite abstract method that will be called whenever the panel becomes visible
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
		importPanel.updateValues();
	}

}
