package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.unicore.Vsite;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.JobContainer;

/**
 *  Abstract base class for all panels that corresponds to a node in the jpa
 *  tree.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: JPAPanel.java,v 1.3 2003/07/09 14:56:26 rrate Exp $
 */

public abstract class JPAPanel extends JPanel implements Applyable {
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");
	/**
	 *  Description of the Field
	 */
	protected static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	private ActionContainer container;

	/**
	 *  reference to parent JFrame
	 */
	protected JFrame parentFrame;

	private Vsite vsite;

	/**
	 *  Simple constructor sets a BorderLayout and builds southern panel
	 *
	 *@param  parentFrame  reference to parent JFrame for modal dialogs
	 *@param  container    Description of the Parameter
	 */
	public JPAPanel(JFrame parentFrame, ActionContainer container) {
		this.parentFrame = parentFrame;
		this.container = container;
		JobContainer parent = container.getParentJobContainer();
		if (parent == null) {
			parent = (JobContainer) container;
		}
		// this.vsite = parent.getVsite();
		setLayout(new BorderLayout());
	}

	/**
	 *  Apply the current GUI values to the container
	 */
	public abstract void applyValues();

	/**
	 *  Get a reference to the parent frame
	 *
	 *@return    parentFrame
	 */
	public JFrame getParentFrame() {
		return parentFrame;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public boolean hasVsiteChanged() {
		JobContainer parent = container.getParentJobContainer();
		if (parent == null) {
			parent = (JobContainer) container;
		}
		logger.finer(
			"Check vsite change for : "
				+ container
				+ ", New Vsite : "
				+ vsite
				+ ", Old Vsite: "
				+ parent.getVsite());
		boolean changed =
			((vsite == null && parent.getVsite() != null)
				|| vsite != null
				&& !vsite.equals(parent.getVsite()));
		logger.finer("Vsite Changed: " + changed);
		vsite = parent.getVsite();
		return changed;
	}

	/**
	 *  Reset current GUI entries to last container contents.
	 */
	public abstract void resetValues();

	/**
	 *  Sets the titledBorder attribute of the JPAPanel object
	 *
	 *@param  title  The new titledBorder value
	 */
	public void setTitledBorder(String title) {
		setBorder(new TitledBorder(title));
	}

	/**
	 *  Description of the Method
	 */
	public void updateValues() {
		updateValues(hasVsiteChanged());
	}

	/**
	 *  Method that will be called whenever the panel becomes visible
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public abstract void updateValues(boolean vsiteChanged);

}
