package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import org.unicore.Vsite;

import com.pallas.unicore.client.controls.GenericFileFilter;
import com.pallas.unicore.client.controls.SimpleToolBar;
import com.pallas.unicore.client.dialogs.ResourceEditDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.tables.ListTable;
import com.pallas.unicore.client.tables.NamedObject;
import com.pallas.unicore.client.xml.XMLObjectReader;
import com.pallas.unicore.client.xml.XMLObjectWriter;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.resourcemanager.ResourceTray;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Panel to define a resources for job group tasks
 *
 *@author     Ralf Ratering
 *@created    11. Oktober 2001
 *@version    $Id: JobResourcePanel.java,v 1.3 2003/06/30 06:54:33 rrate Exp $
 */
public class JobResourcePanel extends JPanel {

	/**
	 *  Listen to toolbar buttons
	 *
	 *@author     Ralf Ratering
	 *@created    21. August 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  called when button is pressed
		 *
		 *@param  event
		 */
		public void actionPerformed(ActionEvent event) {
			if (resourceList.getCellEditor() != null) {
				resourceList.getCellEditor().stopCellEditing();
			}

			Object src = event.getSource();
			if (src.equals(newButton)) {
				addNewResourceSet();
			} else if (src.equals(openButton)) {
				openResourceSet();
			} else if (src.equals(saveButton)) {
				saveResourceSet();
			} else if (src.equals(saveAsButton)) {
				saveResourceSetAs();
			} else if (src.equals(editButton)) {

				editResourceSet();
			} else if (src.equals(removeButton)) {
				removeResourceSet();
			} else if (src.equals(selectAllButton)) {
				// first deselect to make sure change event will be fired
				taskList.clearSelection();
				taskList.setSelectionInterval(0, taskModel.getSize() - 1);
			} else if (src.equals(clearButton)) {
				Object[] selectedTasks = taskList.getSelectedValues();
				for (int i = 0; i < selectedTasks.length; i++) {
					TaskListEntry listEntry = (TaskListEntry) selectedTasks[i];
					// set to site default
					listEntry.setResourceSet(new NamedResourceSet());
				}
				taskList.clearSelection();
			}
		}
	}

	/**
	 *  Listener class listens to list selections from resource and task list
	 *
	 *@author     Ralf Ratering
	 *@created    21. August 2001
	 */
	private class ListListener implements ListSelectionListener {

		/**
		 *@param  evt  Description of Parameter
		 */
		public void valueChanged(ListSelectionEvent evt) {
			// check if this is one of multiple selection events
			if (evt.getValueIsAdjusting()) {
				return;
			}

			updateResourceButtons();

			NamedResourceSet resourceSet =
				(NamedResourceSet) resourceList.getSelectedValue();
			if (resourceSet == null) {
				return;
			}

			// resource list selected
			if (evt.getSource().equals(resourceList.getSelectionModel())) {
				taskList.clearSelection();
			}
			// task list selected
			else if (evt.getSource().equals(taskList)) {
				updateTaskButtons();
				Object[] selectedTasks = taskList.getSelectedValues();
				if (selectedTasks != null) {
					for (int i = 0; i < selectedTasks.length; i++) {
						TaskListEntry listEntry =
							(TaskListEntry) selectedTasks[i];
						listEntry.setResourceSet(resourceSet);
					}
				}
			}
			parent.updateJPATree();
		}
	}

	/**
	 *  Define a JTextField with special colors as cell editor
	 *
	 *@author     Ralf Ratering
	 *@created    17. Oktober 2001
	 */
	private class ResourceCellEditor extends DefaultCellEditor {
		private JTextField textField;

		/**
		 *  Constructor
		 */
		public ResourceCellEditor() {
			super(new JTextField());
			// get text field back from super class
			textField = (JTextField) editorComponent;
		}

		/**
		 *  get cell editor and set appropriate selection colors
		 *
		 *@param  table
		 *@param  value
		 *@param  isSelected
		 *@param  row
		 *@param  column
		 *@return             cell editor component
		 */
		public Component getTableCellEditorComponent(
			JTable table,
			Object value,
			boolean isSelected,
			int row,
			int column) {
			Component editor =
				super.getTableCellEditorComponent(
					table,
					value,
					isSelected,
					row,
					column);

			if (isSelected) {
				editor.setForeground(table.getSelectionForeground());
				editor.setBackground(table.getSelectionBackground());
			}
			return editor;
		}
	}

	/**
	 *  Listen if a resource set in the resource table changed its name. In this
	 *  case change the corresponding entries in the task table
	 *
	 *@author     Ralf Ratering
	 *@created    22. Oktober 2001
	 */
	private class ResourceChangeListener implements TableModelListener {
		/**
		 *  method is fired when resource table changed
		 *
		 *@param  e  event
		 */
		public void tableChanged(TableModelEvent e) {
			if ((e.getType() == 0)) {
				NamedResourceSet resourceSet =
					(NamedResourceSet) resourceList.getSelectedValue();
				// update task table entries
				for (int i = 0; i < taskModel.size(); i++) {
					TaskListEntry entry =
						(TaskListEntry) taskModel.getElementAt(i);
					if (entry.getResourceSet().equals(resourceSet)) {
						entry.setResourceSet(resourceSet);
						taskModel.setElementAt(entry, i);
					}
				}
			}
		}
	}

	/**
	 *  A list entry containing a container and a resource set
	 *
	 *@author     Ralf Ratering
	 *@created    8. Oktober 2001
	 */
	private class TaskListEntry {
		private TaskContainer container;

		/**
		 *  Constructor for the TaskListEntry object
		 *
		 *@param  container
		 *@param  resourceSet
		 */
		public TaskListEntry(
			TaskContainer container,
			NamedResourceSet resourceSet) {
			this.container = container;
			setResourceSet(resourceSet);
		}

		/**
		 *  Gets the Container attribute of the TaskListEntry object
		 *
		 *@return    The Container value
		 */
		public TaskContainer getContainer() {
			return container;
		}

		/**
		 *  Gets the ResourceSet attribute of the TaskListEntry object
		 *
		 *@return    The ResourceSet value, an empty resource set indicates site
		 *      defaults
		 */
		public NamedResourceSet getResourceSet() {
			ResourceTray tray = jobContainer.getResourceTray();
			return (NamedResourceSet) tray.get(container.getIdentifier());
		}

		/**
		 *  Sets the ResourceSet attribute of the TaskListEntry object
		 *
		 *@param  resourceSet  The new ResourceSet value
		 */
		public void setResourceSet(NamedResourceSet resourceSet) {
			ResourceTray tray = jobContainer.getResourceTray();
			if (!resourceSet.isSiteDefault()) {
				tray.put(container.getIdentifier(), resourceSet);
			} else {
				tray.remove(container.getIdentifier());
			}
		}

		/**
		 *  String representation of list entry
		 *
		 *@return    String representation
		 */
		public String toString() {
			ResourceTray tray = jobContainer.getResourceTray();
			NamedResourceSet set =
				(NamedResourceSet) tray.get(container.getIdentifier());
			return container.toString() + " (" + set.toString() + ")";
		}

	}

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");
	private int counter = 0;
	private JobContainer jobContainer;

	private JButton newButton,
		editButton,
		openButton,
		saveButton,
		saveAsButton,
		removeButton,
		clearButton,
		selectAllButton;
	private JobGroupPanel parent;
	private ListTable resourceList;
	private NamedResourceSet siteResources;
	private JList taskList;
	private DefaultListModel taskModel;

	/**
	 *  Constructor
	 *
	 *@param  jobContainer  container that this panel represents
	 *@param  parent        Description of the Parameter
	 */
	public JobResourcePanel(JobContainer jobContainer, JobGroupPanel parent) {
		this.jobContainer = jobContainer;
		this.parent = parent;
		// default site resources are empty
		this.siteResources = NamedResourceSet.getEmptyResourceSet();

		// build gui
		buildComponents();

		updateResourceButtons();
		updateTaskButtons();
	}

	/**
	 *  Add a new resource set to the list
	 */
	private void addNewResourceSet() {
		counter++;
		addResourceSet(new NamedResourceSet("New_ResourceSet" + counter));
	}

	/**
	 *  Add an existing resource set to the list (e.g. read from file)
	 *
	 *@param  resourceSet  The feature to be added to the ResourceSet attribute
	 */
	private void addResourceSet(NamedResourceSet resourceSet) {
		resourceList.addElement(resourceSet);
		resourceList.revalidate();
		int index = resourceList.getNrOfElements() - 1;
		updateResourceSet(index);
		resourceList.setSelectedRow(index);
		parent.updateJPATree();
	}

	/**
	 *  Apply values from gui to JobContainer
	 */
	public void applyValues() {
		logger.fine("Applying resource panel" + jobContainer);
		updateModifiedTime();
	}

	/**
	 *  Build GUI
	 */
	private void buildComponents() {
		resourceList = new ListTable();
		resourceList.setToolTipText("Select a resource set");
		//resourceList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		resourceList.addListSelectionListener(new ListListener());
		resourceList.setCellEditor(new ResourceCellEditor());
		resourceList.addTableModelListener(new ResourceChangeListener());

		JScrollPane resourceScrollPane = new JScrollPane(resourceList);
		resourceScrollPane.getViewport().setView(resourceList);

		ButtonListener buttonListener = new ButtonListener();

		// resource toolbar for buttons
		newButton = new JButton(ResourceManager.getIcon(ResourceManager.NEW));
		newButton.setToolTipText("Add a new resource set");
		newButton.addActionListener(buttonListener);

		openButton = new JButton(ResourceManager.getIcon(ResourceManager.OPEN));
		openButton.setToolTipText("Load a resource set from file");
		openButton.addActionListener(buttonListener);

		saveButton = new JButton(ResourceManager.getIcon(ResourceManager.SAVE));
		saveButton.setToolTipText("Save a resource set to file");
		saveButton.addActionListener(buttonListener);

		saveAsButton =
			new JButton(ResourceManager.getIcon(ResourceManager.SAVEAS));
		saveAsButton.setToolTipText("Save a resource set using a new filename");
		saveAsButton.addActionListener(buttonListener);

		editButton = new JButton(ResourceManager.getIcon(ResourceManager.EDIT));
		editButton.setToolTipText("Edit selected resource set");
		editButton.addActionListener(buttonListener);

		removeButton =
			new JButton(ResourceManager.getIcon(ResourceManager.REMOVE));
		removeButton.setToolTipText("Delete selected resource set from list");
		removeButton.addActionListener(buttonListener);

		SimpleToolBar resourceToolBar = new SimpleToolBar();
		resourceToolBar.setVertical(true);
		resourceToolBar.add(newButton);
		resourceToolBar.add(openButton);
		resourceToolBar.add(saveButton);
		resourceToolBar.add(saveAsButton);
		resourceToolBar.add(editButton);
		resourceToolBar.add(removeButton);

		JPanel resourcePanel = new JPanel(new BorderLayout());
		resourcePanel.setBorder(new TitledBorder("Resource Sets"));
		resourcePanel.add(resourceScrollPane, BorderLayout.CENTER);
		resourcePanel.add(resourceToolBar, BorderLayout.EAST);

		taskModel = new DefaultListModel();
		taskList = new JList(taskModel);
		taskList.setToolTipText(
			"Click onto task to apply selected resource set");
		taskList.setSelectionMode(
			ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		taskList.addListSelectionListener(new ListListener());

		JScrollPane taskScrollPane = new JScrollPane(taskList);
		taskScrollPane.getViewport().setView(taskList);

		// task toolbar for buttons
		selectAllButton =
			new JButton(ResourceManager.getIcon(ResourceManager.SELECTALL));
		selectAllButton.setToolTipText("Select all tasks");
		selectAllButton.addActionListener(buttonListener);

		clearButton =
			new JButton(ResourceManager.getIcon(ResourceManager.CLEAR));
		clearButton.setToolTipText(
			"Set selected tasks to site default resources");
		clearButton.addActionListener(buttonListener);

		SimpleToolBar taskToolBar =
			new SimpleToolBar(selectAllButton, clearButton);
		taskToolBar.setVertical(true);

		JPanel taskPanel = new JPanel(new BorderLayout());
		taskPanel.setBorder(new TitledBorder("Tasks"));
		taskPanel.add(taskScrollPane, BorderLayout.CENTER);
		taskPanel.add(taskToolBar, BorderLayout.EAST);

		JPanel listPanel = new JPanel(new GridLayout(2, 1));
		listPanel.add(resourcePanel);
		listPanel.add(taskPanel);

		setLayout(new BorderLayout());
		add(listPanel, BorderLayout.CENTER);
	}

	/**
	 *  Edit selected resource set
	 */
	private void editResourceSet() {
		if (resourceList.getNrOfElements() == 0) {
			addNewResourceSet();
		}

		logger.info(
			"EDITING: "
				+ ((NamedResourceSet) resourceList.getSelectedValue())
					.toStringDetailed());

		ResourceEditDialog editor =
			new ResourceEditDialog(
				ResourceManager.getCurrentInstance(),
				(NamedResourceSet) resourceList.getSelectedValue(),
				siteResources);
		editor.show();
		updateResourceSet(resourceList.getSelectedRow());
		parent.updateJPATree();
	}

	/**
	 *  Load a resource set from file
	 */
	private void openResourceSet() {
		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(
				new File(
					ResourceManager.getUserDefaults().getResourceDirectory()),
				null);
		chooser.setTitle("UNICOREpro: Open Resource Set");

		GenericFileFilter filter = new GenericFileFilter(".xml");
		filter.setDescription(res.getString("UNICORE_RESOURCES_XML"));
		chooser.setSelectedFileFilter(filter);
		int result = chooser.showDialog();

		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = chooser.getSelectedFile().getAbsolutePath();
			try {
				XMLObjectReader din = new XMLObjectReader(new File(filename));
				Object o = din.readObjectXML();

				if (o.getClass().equals(NamedResourceSet.class)) {
					NamedResourceSet resourceSet = (NamedResourceSet) o;
					addResourceSet(resourceSet);
				} else {
					// if we got here something went wrong
					throw new Exception(
						o.getClass()
							+ " different from "
							+ NamedResourceSet.class);
				}
			} catch (FileNotFoundException e) {
				UserMessages.error(
					"Could not find resource file: " + filename,
					e.getMessage());
			} catch (Exception e) {
				UserMessages.error(
					"Reading resource file <" + filename + "> failed.",
					e.getMessage());
			}
		}
	}

	/**
	 *  Remove selected entry from resource list
	 */
	private void removeResourceSet() {

		int index = resourceList.getSelectedRow();
		NamedResourceSet resourceSet =
			(NamedResourceSet) resourceList.getSelectedValue();
		resourceList.removeElement(resourceSet);
		if (index > 0) {
			resourceList.setRowSelectionInterval(index - 1, index - 1);
		} else if (index == 0 && resourceList.getNrOfElements() > 0) {
			resourceList.setRowSelectionInterval(0, 0);
		}
		for (int i = 0; i < taskModel.getSize(); i++) {
			TaskListEntry listEntry = (TaskListEntry) taskModel.elementAt(i);
			if (listEntry.getResourceSet() == resourceSet) {
				listEntry.setResourceSet(new NamedResourceSet());
			}
			taskModel.setElementAt(listEntry, i);
		}
		taskList.clearSelection();
		updateResourceButtons();
		parent.updateJPATree();
	}

	/**
	 *  Fill values from JobContainer into gui
	 */
	public void resetValues() {
		logger.fine("Updating resource panel" + jobContainer);
		ResourceTray tray = jobContainer.getResourceTray();
		Vector resourceSets = tray.getAllResourceSets();
		for (int i = 0; i < resourceSets.size(); i++) {
			resourceList.addElement((NamedObject) resourceSets.elementAt(i));
		}
		updateValues(false);
		if (resourceList.getRowCount() > 0) {
			resourceList.setSelectedRow(0);
		}
		updateResourceSets();
		updateResourceButtons();
		updateTaskButtons();
	}

	/**
	 *  Description of the Method
	 */
	private void saveResourceSet() {
		NamedResourceSet resourceSet =
			(NamedResourceSet) resourceList.getSelectedValue();
		String filename = resourceSet.getFilename();
		if (filename == null) {
			saveResourceSetAs();
			return;
		}
		try {
			logger.fine(
				"JobResourcePanel: Writing "
					+ resourceSet.getName()
					+ " to file: "
					+ filename);
			FileOutputStream f = new FileOutputStream(filename);
			XMLObjectWriter dout = new XMLObjectWriter(f);
			dout.writeObjectXML(resourceSet);
			f.flush();
			f.close();
		} catch (Exception e) {
			UserMessages.error(
				"Writing resource set " + resourceSet.getName() + " failed.",
				e.getMessage());
		}
	}

	/**
	 *  Write a resource set to file
	 */
	private void saveResourceSetAs() {
		NamedResourceSet resourceSet =
			(NamedResourceSet) resourceList.getSelectedValue();
		String filename = resourceSet.getFilename();
		if (filename == null) {
			filename = ResourceManager.getUserDefaults().getResourceDirectory();
		}

		File selectedFile = new File(filename);
		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(selectedFile, null);
		chooser.setTitle(
			"UNICOREpro: Save Resource Set <" + resourceSet.getName() + ">");
		GenericFileFilter filter = new GenericFileFilter(".xml");
		filter.setDescription(res.getString("UNICORE_RESOURCES_XML"));
		chooser.addFileFilter(filter);
		chooser.setSelectedFileFilter(filter);
		// if we are in default directory add resource name as default
		// filename in text field
		if (selectedFile.isDirectory()) {
			chooser.setFilename(resourceSet.getName());
		}

		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			resourceSet.setFilename(
				chooser.getSelectedFile().getAbsolutePath());
			saveResourceSet();
		}
	}

	/**
	 *  Description of the Method
	 */
	private void updateModifiedTime() {
		ResourceTray tray = jobContainer.getResourceTray();

		// compare entries
		for (int i = 0; i < taskModel.getSize(); i++) {
			TaskListEntry listEntry = (TaskListEntry) taskModel.elementAt(i);
			// compare entries for containers that have their own resource sets
			if (!listEntry.getResourceSet().isSiteDefault()) {
				Object oldResourceSet =
					tray.get(listEntry.getContainer().getIdentifier());
				if (!oldResourceSet.equals(listEntry.getResourceSet())) {
					jobContainer.setModifiedTime(new Date());
					return;
				}
			}
			// check site default entries
			else {
				NamedResourceSet oldResourceSet =
					(NamedResourceSet) tray.get(
						listEntry.getContainer().getIdentifier());
				if (!oldResourceSet.isSiteDefault()) {
					jobContainer.setModifiedTime(new Date());
					return;
				}
			}
		}
	}

	/**
	 *  Description of the Method Description of the Method Description of the
	 *  Method Description of the Method Description of the Method Enable or
	 *  disable toolbar buttons for resource list
	 */
	private void updateResourceButtons() {
		boolean enabled = resourceList.getSelectedValue() != null;
		saveButton.setEnabled(enabled);
		saveAsButton.setEnabled(enabled);
		removeButton.setEnabled(enabled);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  i  Description of the Parameter
	 */
	private void updateResourceSet(int i) {
		NamedResourceSet resourceSet =
			(NamedResourceSet) resourceList.getElement(i);
		if (resourceSet.matchesSiteResources(siteResources)) {
			resourceList.setElementColor(i, Color.green);
		} else {
			resourceList.setElementColor(i, Color.red);
		}
		resourceList.repaint();
	}

	/**
	 *  Description of the Method
	 */
	private void updateResourceSets() {
		for (int i = 0; i < resourceList.getNrOfElements(); i++) {
			updateResourceSet(i);
		}
	}

	/**
	 *  Enable or disable toolbar buttons for task list
	 */
	private void updateTaskButtons() {
		selectAllButton.setEnabled(taskModel.size() > 0);

		boolean enabled = taskList.getSelectedValue() != null;
		clearButton.setEnabled(enabled);
	}

	/**
	 *  Update task list. If a new task has been added to job, add it to task
	 *  list.
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public void updateValues(boolean vsiteChanged) {
		logger.fine("Updating resource panel" + jobContainer);
		Vector tasks = jobContainer.getExecutionTasks();
		ResourceTray tray = jobContainer.getResourceTray();

		// remove old tasks
		for (int i = 0; i < taskModel.size(); i++) {
			TaskListEntry entry = (TaskListEntry) taskModel.elementAt(i);
			if (!tasks.contains(entry.getContainer())) {
				taskModel.removeElement(entry);
			}
		}

		// loop over job tasks
		for (int i = 0; i < tasks.size(); i++) {
			if (tasks.elementAt(i) instanceof TaskContainer) {
				TaskContainer container = (TaskContainer) tasks.elementAt(i);
				// loop over task list
				boolean found = false;
				for (int j = 0; j < taskModel.size(); j++) {
					TaskListEntry entry =
						(TaskListEntry) taskModel.elementAt(j);
					if (container.equals(entry.getContainer())) {
						found = true;
						continue;
					}
				}

				// if job task is not in list, add it
				if (!found) {
					NamedResourceSet resourceSet =
						(NamedResourceSet) tray.get(container.getIdentifier());
					TaskListEntry listEntry =
						new TaskListEntry(container, resourceSet);
					taskModel.addElement(listEntry);
				}
			}

		}
	}

	/**
	 *  Sets the Vsite attribute of the JobResourcePanel object
	 */
	/*
	 *  public void setVsite(Vsite vsite) {
	 *  this.vsite = vsite;
	 *  siteResources = ResourceManager.getResourceSet(vsite);
	 *  updateResourceSets();
	 *  parent.updateJPATree();
	 *  }
	 */
	/**
	 *  if vsite information has been reloaded, call this method
	 */
	public void updateVsite() {
		Vsite vsite = jobContainer.getVsite();
		if (vsite != null) {
			siteResources = ResourceManager.getResourceSet(vsite);
			updateResourceSets();

			parent.updateJPATree();
		}
	}

}
