package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.pallas.unicore.container.FileOperationContainer;

/**
 *  JPAPanel to define a file operations in a certain file space (Uspace or Xspace) This panel can
 *  be plugged into the clients data area and can be connected to a jpa tree node
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: OperationJPAPanel.java,v 1.2 2003/06/30 06:54:32 rrate Exp $
 */
public class OperationJPAPanel extends JPAPanel {
	// text field for task name
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	// container to be modified
	private FileOperationContainer container;
	private JTextField nameTextField;

	// panel containing file operation table
	private FileOperationPanel operationPanel;

	/**
	 *  Constructor
	 *
	 *@param  parentFrame  parent JFrame
	 *@param  container    Description of Parameter
	 */
	public OperationJPAPanel(
		JFrame parentFrame,
		FileOperationContainer container) {
		super(parentFrame, container);

		this.container = container;

		buildComponents();
	}

	/**
	 *  Overwrite abstract method to apply the current GUI values to the container
	 */
	public void applyValues() {
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		operationPanel.applyValues();
	}

	/**
	 *  Build GUI
	 */
	private void buildComponents() {
		operationPanel = new FileOperationPanel(container);

		nameTextField = new JTextField(20);
		nameTextField.setToolTipText(res.getString("ENTER_IDENTIFIER"));

		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel(res.getString("NAME")));
		namePanel.add(nameTextField);

		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.add(namePanel, BorderLayout.NORTH);
		subPanel.add(operationPanel, BorderLayout.CENTER);

		add(subPanel, BorderLayout.CENTER);
		setTitledBorder(res.getString("OPERATION_BORDER"));

		revalidate();
	}

	/**
	 *  Overwrite abstract method to reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		nameTextField.setText(container.getName());
		operationPanel.resetValues();
	}

	/**
	 *  Overwrite abstract method that will be called whenever the panel becomes visible
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
		operationPanel.updateValues();
	}

}
