package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import org.unicore.ajo.Option;
import org.unicore.sets.OptionEnumeration;
import org.unicore.sets.OptionSet;

import com.pallas.unicore.container.UserContainer;

/**
 *  Panel to define options for executable commands
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: OptionPanel.java,v 1.2 2003/06/30 06:54:32 rrate Exp $
 */
public class OptionPanel extends JPanel {

	/**
	 *  container to be modified
	 */
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");
	/**
	 *  Description of the Field
	 */
	protected UserContainer commandContainer;

	// some check boxes
	private JCheckBox redirectStdoutCheckBox,
		redirectStderrCheckBox,
		redirectStdinCheckBox,
		timeBox,
		versionBox,
		debugBox,
		profileBox,
		verboseBox;

	// some text fields
	private JTextField stdoutTextField,
		stderrTextField,
		stdinTextField,
		commandLine,
		envVariables;

	/**
	 *  Constructor builds GUI
	 *
	 *@param  container  to be modified
	 */
	public OptionPanel(UserContainer container) {
		this.commandContainer = container;
		buildComponents();
	}

	/**
	 *  Helper method to handle GridBagLayout
	 *
	 *@param  panel  The feature to be added to the ToPanel attribute
	 *@param  l      The feature to be added to the ToPanel attribute
	 *@param  r      The feature to be added to the ToPanel attribute
	 *@param  g      The feature to be added to the ToPanel attribute
	 *@param  c      The feature to be added to the ToPanel attribute
	 */
	private void addToPanel(
		JPanel panel,
		Component l,
		Component r,
		GridBagLayout g,
		GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);

		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}

	/**
	 *  Apply gui entries to container
	 */
	public void applyValues() {
		commandContainer.setCommandLine(commandLine.getText());
		commandContainer.setEnv(getOptionSetFromString(envVariables.getText()));
		commandContainer.setDebug(debugBox.isSelected());
		commandContainer.setProfile(profileBox.isSelected());
		commandContainer.setMeasureTime(timeBox.isSelected());

		commandContainer.setVersion(versionBox.isSelected());
		commandContainer.setVerbose(verboseBox.isSelected());

		commandContainer.setRedirectStdout(redirectStdoutCheckBox.isSelected());
		commandContainer.setRedirectStdin(redirectStdinCheckBox.isSelected());
		commandContainer.setRedirectStderr(redirectStderrCheckBox.isSelected());

		commandContainer.setStdout(stdoutTextField.getText());
		commandContainer.setStderr(stderrTextField.getText());
		commandContainer.setStdin(stdinTextField.getText());

	}

	/**
	 *  Build gui elements
	 */
	private void buildComponents() {
		// edit area
		commandLine = new JTextField(30);
		commandLine.setToolTipText(res.getString("ARG_TIP"));
		envVariables = new JTextField(30);
		envVariables.setToolTipText(res.getString("ENV_TIP"));

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(5, 5, 5, 5);
		c.anchor = GridBagConstraints.WEST;

		JPanel subPanel1 = new JPanel(gridbag);
		subPanel1.setBorder(new TitledBorder(res.getString("ARGUMENTS")));
		addToPanel(
			subPanel1,
			new JLabel(res.getString("CMD_LINE")),
			commandLine,
			gridbag,
			c);
		addToPanel(
			subPanel1,
			new JLabel(res.getString("ENVIRONMENT")),
			envVariables,
			gridbag,
			c);

		debugBox = new JCheckBox(res.getString("DEBUG"));
		profileBox = new JCheckBox(res.getString("PROFILING"));
		timeBox = new JCheckBox(res.getString("TIMING"));
		versionBox = new JCheckBox(res.getString("SHOW_VERSION"));
		verboseBox = new JCheckBox(res.getString("VERBOSE"));

		JPanel gridPanel = new JPanel(new GridLayout(3, 2, 10, 5));
		gridPanel.add(debugBox);
		gridPanel.add(profileBox);
		gridPanel.add(timeBox);
		gridPanel.add(versionBox);
		gridPanel.add(verboseBox);

		JPanel subPanel2 = new JPanel();
		subPanel2.setBorder(new TitledBorder(res.getString("OPTIONS")));
		subPanel2.add(gridPanel);

		redirectStdoutCheckBox =
			new JCheckBox(res.getString("REDIRECT_STDOUT"));
		redirectStderrCheckBox =
			new JCheckBox(res.getString("REDIRECT_STDERR"));
		redirectStdinCheckBox = new JCheckBox(res.getString("REDIRECT_STDIN"));
		stdoutTextField = new JTextField(25);
		stdoutTextField.setToolTipText(res.getString("OUTNAME"));
		stderrTextField = new JTextField(25);
		stderrTextField.setToolTipText(res.getString("ERRNAME"));
		stdinTextField = new JTextField(25);
		stdinTextField.setToolTipText(res.getString("INNAME"));

		JPanel subPanel3 = new JPanel(gridbag);
		subPanel3.setBorder(new TitledBorder(res.getString("OUT_IN_ERR")));
		addToPanel(
			subPanel3,
			redirectStdoutCheckBox,
			stdoutTextField,
			gridbag,
			c);
		addToPanel(
			subPanel3,
			redirectStderrCheckBox,
			stderrTextField,
			gridbag,
			c);
		addToPanel(
			subPanel3,
			redirectStdinCheckBox,
			stdinTextField,
			gridbag,
			c);

		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(subPanel1, BorderLayout.NORTH);
		mainPanel.add(subPanel2, BorderLayout.CENTER);
		mainPanel.add(subPanel3, BorderLayout.SOUTH);

		JScrollPane scrollPane = new JScrollPane(mainPanel);

		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);

	}

	/**
	 *  Parse a string in the following format and return an OptionSet: 1.)
	 *  Items are separated by white space or tab 2.) Items contain a key/value
	 *  pair separated by "="
	 *
	 *@param  in  Description of Parameter
	 *@return     The OptionSetFromString value
	 */
	private OptionSet getOptionSetFromString(String in) {
		OptionSet optionSet = new OptionSet();
		StringTokenizer st = new StringTokenizer(in, " \t");
		int count = st.countTokens();
		//	System.out.println("Found "+count+" tokens");
		if (count < 1) {
			return null;
		}

		while (st.hasMoreElements()) {
			String token = st.nextToken();
			StringTokenizer st2 = new StringTokenizer(token, "=");
			int count2 = st2.countTokens();
			/*
			 *  if( count != 2) {
			 *  something wrong
			 *  continue;
			 *  }
			 */
			String key = st2.nextToken();
			String value = st2.nextToken();
			optionSet.add(new Option(key, value));
		}

		return optionSet;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  options  Description of Parameter
	 *@return          Description of the Returned Value
	 */
	private String optionSetToString(OptionSet options) {
		String result = new String();
		if (options == null) {
			return result;
		}
		OptionEnumeration enum = options.elements();

		while (enum.hasMoreElements()) {
			Option next = enum.nextElement();
			result = result + next.getToken() + "=" + next.getValue() + " ";
		}
		return result;
	}

	/**
	 *  Fill gui elements with container values
	 */
	public void resetValues() {
		commandLine.setText(commandContainer.getCommandLine());

		debugBox.setSelected(commandContainer.getDebug());
		profileBox.setSelected(commandContainer.getProfile());
		timeBox.setSelected(commandContainer.getMeasureTime());
		verboseBox.setSelected(commandContainer.isVerboseOn());
		versionBox.setSelected(commandContainer.isVersionOn());

		redirectStdoutCheckBox.setSelected(
			commandContainer.getRedirectStdout());
		redirectStdinCheckBox.setSelected(commandContainer.getRedirectStdin());
		redirectStderrCheckBox.setSelected(
			commandContainer.getRedirectStderr());

		stdoutTextField.setText(commandContainer.getStdout());
		stderrTextField.setText(commandContainer.getStderr());
		stdinTextField.setText(commandContainer.getStdin());

		envVariables.setText(optionSetToString(commandContainer.getEnv()));
	}
}
