package com.pallas.unicore.client.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.unicore.outcome.AbstractActionStatus;

import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.DateTableCellRenderer;
import com.pallas.unicore.resourcemanager.DetailsEntry;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.VectorSorter;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    16. Mai 2002
 *@version    $Id: OutcomeDetailsTable.java,v 1.2 2003/06/30 06:54:31 rrate Exp $
 */
public class OutcomeDetailsTable extends AdjustTable {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. Mai 2002
	 */
	protected class DetailsTableModel extends DefaultTableModel {

		/**
		 *  Gets the CellEditable attribute of the InfoTableModel object
		 *
		 *@param  row  Description of Parameter
		 *@param  col  Description of Parameter
		 *@return      The CellEditable value
		 */
		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    16. Oktober 2002
	 */
	protected static class StatusTableCellRenderer
		implements TableCellRenderer {
		/**
		 *  Gets the tableCellRendererComponent attribute of the
		 *  StatusTableCellRenderer object
		 *
		 *@param  table       Description of the Parameter
		 *@param  value       Description of the Parameter
		 *@param  isSelected  Description of the Parameter
		 *@param  hasFocus    Description of the Parameter
		 *@param  row         Description of the Parameter
		 *@param  column      Description of the Parameter
		 *@return             The tableCellRendererComponent value
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
			JLabel label = new JLabel(value.toString());
			label.setFont(table.getFont());
			if (isSelected) {
				label.setBackground(table.getSelectionBackground());
			} else {
				label.setBackground(Color.WHITE);
			}

			if (value instanceof AbstractActionStatus) {
				AbstractActionStatus status = (AbstractActionStatus) value;
				if (status.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
					label.setForeground(Color.RED);
				} else if (
					status.isEquivalent(AbstractActionStatus.SUCCESSFUL)) {
					label.setForeground(Color.GREEN);
				}
			}

			label.setOpaque(true);

			return label;
		}

	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    23. September 2002
	 */
	protected class TableHeaderListener extends MouseAdapter {
		private OutcomeDetailsTable table;

		/**
		 *  Constructor for the TableHeaderListener object
		 */
		public TableHeaderListener() {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			TableColumnModel columnModel = getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			sortColumn = convertColumnIndexToModel(viewColumn);
			sortByColumn(sortColumn);
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    19. November 2002
	 */
	private class TableHeaderRenderer extends DefaultTableCellRenderer {

		/**
		 *  Gets the tableCellRendererComponent attribute of the
		 *  TableHeaderRenderer object
		 *
		 *@param  table       Description of the Parameter
		 *@param  value       Description of the Parameter
		 *@param  isSelected  Description of the Parameter
		 *@param  hasFocus    Description of the Parameter
		 *@param  row         Description of the Parameter
		 *@param  column      Description of the Parameter
		 *@return             The tableCellRendererComponent value
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {

			if (column == sortColumn) {
				if (ascending) {
					this.setIcon(up);
				} else {
					this.setIcon(down);
				}
			} else {
				this.setIcon(null);
			}
			this.setHorizontalTextPosition(JLabel.LEFT);

			JTableHeader header = table.getTableHeader();
			if (header != null) {
				this.setForeground(header.getForeground());
				this.setBackground(header.getBackground());
				this.setFont(header.getFont());
			}

			this.setText((value == null) ? "" : value.toString());
			this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));

			return this;
		}
	}

	private final static String[] columnIdentifiers =
		{
			"Time Stamp",
			"Name",
			"Description",
			"Status",
			"Reason",
			"Iteration" };
	private static TableCellRenderer dateCellRenderer =
		new DateTableCellRenderer(ResourceManager.getCompleteFormatter());

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");
	private static TableCellRenderer statusCellRenderer =
		new StatusTableCellRenderer();
	private boolean ascending;
	private final ImageIcon down =
		ResourceManager.getIcon(ResourceManager.DOWN);

	private boolean initialized = false;
	private int lastSortColumn = -1;
	private DetailsTableModel model;
	private int sortColumn = 0;

	private final ImageIcon up = ResourceManager.getIcon(ResourceManager.UP);

	/**
	 *  Constructor for the ExplorerTable object
	 */
	public OutcomeDetailsTable() {
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		buildComponents();
		getTableHeader().setDefaultRenderer(new TableHeaderRenderer());
		getTableHeader().addMouseListener(new TableHeaderListener());
	}

	/**
	 *  Description of the Method
	 *
	 *@param  rows  Description of the Parameter
	 */
	private void adjustWidthsAndLookForIterations(Vector rows) {
		adjustWidths();
		boolean hasIteration = false;
		for (int i = 0; i < rows.size(); i++) {
			DetailsEntry entry = (DetailsEntry) rows.elementAt(i);
			if (!entry.getIteration().equals("")) {
				hasIteration = true;
				break;
			}
		}
		if (!hasIteration) {
			TableColumn column = getColumn("Iteration");
			column.setMinWidth(0);
			column.setMaxWidth(0);
			column.setPreferredWidth(0);
			column.setResizable(false);
		}
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		model = new DetailsTableModel();
		model.setColumnIdentifiers(columnIdentifiers);
		setModel(model);
		getColumnModel().getColumn(0).setCellRenderer(dateCellRenderer);
		getColumnModel().getColumn(3).setCellRenderer(statusCellRenderer);
	}

	/**
	 *  Description of the Method
	 */
	public void clear() {
		int rowCount = getRowCount();
		for (int i = 0; i < rowCount; i++) {
			model.removeRow(0);
		}
	}

	/**
	 *  Gets the selectedEntry attribute of the OutcomeDetailsTable object
	 *
	 *@return    The selectedEntry value
	 */
	public DetailsEntry getSelectedEntry() {
		return (DetailsEntry) model.getDataVector().elementAt(getSelectedRow());
	}

	/**
	 *  Description of the Method
	 */
	public void resort() {
		ascending = !ascending;
		sortByColumn(lastSortColumn);
	}

	/**
	 *  Sets the dataVector attribute of the OutcomeDetailsTable object
	 *
	 *@param  rows  The new dataVector value
	 */
	public void setDataVector(Vector rows) {
		this.clear();
		for (int i = 0; i < rows.size(); i++) {
			model.addRow((Vector) rows.elementAt(i));
		}
		resort();
		if (rows.size() > 0 && !initialized) {
			adjustWidthsAndLookForIterations(rows);
			initialized = true;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  column  Description of the Parameter
	 */
	public void sortByColumn(int column) {
		if (column < 0) {
			return;
		}
		if (lastSortColumn == column) {
			ascending = !ascending;
		} else {
			ascending = true;
		}
		DetailsEntry.setCompareIndex(column, 0);
		VectorSorter.sort(model.getDataVector(), ascending);

		lastSortColumn = column;
	}
}
