package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;

import org.unicore.outcome.AbstractActionStatus;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.dialogs.ShowLogDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.resourcemanager.DetailsEntry;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.VectorSorter;

/**
 *  Panel to display job output from job monitor
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@author     Klaus-Dieter Oertel
 *@created    August 30, 2001
 *@version    $Id: OutcomePanel.java,v 1.5 2003/06/30 06:54:31 rrate Exp $
 */
public class OutcomePanel extends JPanel {

	/**
	 *  Listener class listens to save button events
	 *
	 *@author     thomas
	 *@created    August 30, 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == stdoutSaveButton) {
				saveToFile(stdoutArea.getText(), res.getString("SAVE_OUT"));
			} else if (event.getSource() == stderrSaveButton) {
				saveToFile(stderrArea.getText(), res.getString("SAVE_ERR"));
			} else if (event.getSource() == logSaveButton) {
				saveToFile(logArea.getText(), res.getString("SAVE_MESSAGES"));
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    26. September 2002
	 */
	private class DetailsTableListener extends MouseAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() >= 2) {
				DetailsEntry entry = detailsTable.getSelectedEntry();
				ShowLogDialog logDialog =
					new ShowLogDialog(
						ResourceManager.getCurrentInstance(),
						detailsTable.getSelectedEntry().toString(),
						"UNICOREpro: Details about " + entry.getContainer());
				Font font = logDialog.getFont();
				logDialog.setTextFont(
					new Font("monospaced", Font.PLAIN, font.getSize()));
				logDialog.setSize(800, 250);
				logDialog.setEditable(false);
				logDialog.setLocationRelativeTo(
					ResourceManager.getCurrentInstance());
				logDialog.setVisible(true);
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. September 2002
	 */
	private class TabbedPaneListener implements ChangeListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void stateChanged(ChangeEvent e) {
			if (e.getSource() == tabbedPane.getModel()) {
				selectedIndex = tabbedPane.getModel().getSelectedIndex();
				for (int i = 0; i < additionalOutcomePanels.size(); i++) {
					if (additionalOutcomePanels.elementAt(i)
						instanceof AdditionalOutcomePanel) {
						AdditionalOutcomePanel panel =
							(
								AdditionalOutcomePanel) additionalOutcomePanels
									.elementAt(
								i);
						panel.updateValues();
					} else {
						Applyable panel =
							(Applyable) additionalOutcomePanels.elementAt(i);
						panel.updateValues();
					}
				}
			}
		}
	}
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	private static int selectedIndex = 0;

	private Vector additionalOutcomePanels = new Vector();

	// container to get outcome from
	private ActionContainer container;

	private OutcomeDetailsTable detailsTable;

	private Font fixedFont;

	// Copy of the original JPAPanel
	private JScrollPane jpaPane;

	// parent frame
	private Client parentFrame;

	// text areas
	private JTextArea stdoutArea, stderrArea, logArea, detailsArea;

	// some buttons
	private JButton stdoutSaveButton, stderrSaveButton, logSaveButton;
	private JTabbedPane tabbedPane;

	/**
	 *  Constructor builds gui
	 *
	 *@param  container  to get outcome from
	 */
	public OutcomePanel(ActionContainer container) {
		this.container = container;
		buildComponents();
		tabbedPane.getModel().setSelectedIndex(selectedIndex);
	}

	/**
	 *  Constructor builds gui
	 *
	 *@param  container  to get outcome from
	 *@param  parent     Description of Parameter
	 */
	public OutcomePanel(Client parent, ActionContainer container) {
		this.container = container;
		this.parentFrame = parent;
		buildComponents();
		tabbedPane.getModel().setSelectedIndex(selectedIndex);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  area  Description of the Parameter
	 *@param  file  Description of the Parameter
	 */
	private void appendToTextArea(File file, JTextArea area) {
		// task did not produce stderr/stdout
		if (file == null) {
			return;
		}
		// file should be there but is not...
		if (!file.exists()) {
			logger.warning(
				"Could not read outcome file: " + file.getAbsolutePath());
			return;
		}

		try {
			DefaultEditorKit kit = new DefaultEditorKit();
			Document doc = area.getDocument();
			Reader reader = new FileReader(file);
			try {
				kit.read(reader, doc, doc.getLength());
				logger.finer(
					"Reading outcome file: "
						+ file.getAbsolutePath()
						+ " successful.");
			} catch (BadLocationException e) {
				logger.log(Level.SEVERE, "Error appending to text area", e);
			}
		} catch (IOException e) {
			logger.log(Level.WARNING, "Could not read outcome file", e);
		}
	}

	/**
	 *  Build GUI
	 */
	private void buildComponents() {
		//
		// stdout panel
		//
		stdoutArea = new JTextArea();
		JScrollPane stdoutScrollPane = new JScrollPane(stdoutArea);

		stdoutSaveButton = new JButton(res.getString("SAVE"));
		stdoutSaveButton.addActionListener(new ButtonListener());

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(stdoutSaveButton);

		JPanel stdoutPanel = new JPanel(new BorderLayout());
		stdoutPanel.add(stdoutScrollPane, BorderLayout.CENTER);
		stdoutPanel.add(buttonPanel, BorderLayout.SOUTH);

		//
		// stderr panel
		//
		stderrArea = new JTextArea();
		JScrollPane stderrScrollPane = new JScrollPane(stderrArea);

		stderrSaveButton = new JButton(res.getString("SAVE"));
		stderrSaveButton.addActionListener(new ButtonListener());

		JPanel stderrButtonPanel = new JPanel();
		stderrButtonPanel.add(stderrSaveButton);

		JPanel stderrPanel = new JPanel(new BorderLayout());
		stderrPanel.add(stderrScrollPane, BorderLayout.CENTER);
		stderrPanel.add(stderrButtonPanel, BorderLayout.SOUTH);

		// details panel
		detailsArea = new JTextArea();
		JScrollPane detailsTextScrollPane = new JScrollPane(detailsArea);
		detailsTextScrollPane.setBorder(new TitledBorder("Summary"));

		String tooltip =
			"Double click on row will display textual overview for "
				+ "selected entry.";
		detailsTable = new OutcomeDetailsTable();
		detailsTable.setToolTipText(tooltip);
		detailsTable.addMouseListener(new DetailsTableListener());
		JScrollPane detailsTableScrollPane = new JScrollPane(detailsTable);
		detailsTableScrollPane.setBorder(new TitledBorder("History"));
		detailsTableScrollPane.setToolTipText(tooltip);

		JPanel detailsPanel = new JPanel(new BorderLayout());
		detailsPanel.add(detailsTextScrollPane, BorderLayout.NORTH);
		detailsPanel.add(detailsTableScrollPane, BorderLayout.CENTER);
		detailsPanel.setToolTipText(tooltip);

		// logs
		logArea = new JTextArea();
		JScrollPane logScrollPane = new JScrollPane(logArea);
		JPanel logPanel = new JPanel(new BorderLayout());
		logSaveButton = new JButton(res.getString("SAVE"));
		logSaveButton.addActionListener(new ButtonListener());

		JPanel logButtonPanel = new JPanel();
		logButtonPanel.add(logSaveButton);
		logPanel.add(logScrollPane, BorderLayout.CENTER);
		logPanel.add(logButtonPanel, BorderLayout.SOUTH);

		jpaPane = new JScrollPane();

		tabbedPane = new JTabbedPane();
		tabbedPane.add(stdoutPanel, res.getString("STDOUT"));
		tabbedPane.add(stderrPanel, res.getString("STDERR"));
		tabbedPane.add(detailsPanel, res.getString("DETAILS"));
		tabbedPane.add(logPanel, res.getString("LOG"));
		// add additional outcome panels from plugins
		if (container instanceof IPanelProvider) {
			IPanelProvider panelProvider = (IPanelProvider) container;
			additionalOutcomePanels.clear();
			for (int i = 0; i < panelProvider.getNrOfPanels(); i++) {
				tabbedPane.add(
					new JScrollPane(panelProvider.getPanel(i)),
					panelProvider.getPanelTitle(i));
				if (panelProvider.getPanel(i) instanceof AdditionalOutcomePanel
					|| panelProvider.getPanel(i) instanceof Applyable) {
					additionalOutcomePanels.add(panelProvider.getPanel(i));
				}
			}
		}

		tabbedPane.getModel().addChangeListener(new TabbedPaneListener());
		setLayout(new BorderLayout());
		add(tabbedPane, BorderLayout.CENTER);
	}

	/**
	 *  Gets the selectedIndex attribute of the OutcomePanel object
	 *
	 *@return    The selectedIndex value
	 */
	public int getSelectedIndex() {
		return selectedIndex;
	}

	/**
	 *  Fill outcome into text areas
	 */
	public void resetValues() {

		// if font has changed set it to back to a monospaced one
		if (fixedFont != stdoutArea.getFont()) {
			updateFont();
		}

		stdoutArea.setText("");
		stderrArea.setText("");
		logArea.setText("");

		Vector sortedEntries =
			OutcomeManager.getOutcomeEntries(container.getActionGroup());
		VectorSorter.sort(sortedEntries);
		for (int i = 0; i < sortedEntries.size(); i++) {
			OutcomeEntry entry = (OutcomeEntry) sortedEntries.elementAt(i);
			appendToTextArea(entry.getStdoutFile(), stdoutArea);
			appendToTextArea(entry.getStderrFile(), stderrArea);
			if (entry.getOutcome() != null) {
				String header = "Task: " + entry.getOutcome().getId().getName();
				if (!entry.getIteration().equals("")) {
					header += ", Iteration: " + entry.getIteration();
				}
				logArea.append(header + "\n" + entry.getLog() + "\n");
			}
		}

		Vector details = OutcomeManager.getDetails(container);
		VectorSorter.sort(details);

		detailsArea.setText(container.getName() + ": " + container.getStatus());
		if (container.getStatus() != null
			&& container.getStatus().isEquivalent(
				AbstractActionStatus.NOT_SUCCESSFUL)) {
			for (int i = 0; i < details.size(); i++) {
				DetailsEntry entry = (DetailsEntry) details.elementAt(i);
				if (entry
					.getStatus()
					.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)
					&& !entry.getReason().equals("")) {
					detailsArea.append("\nReason: " + entry.getReason());
				}
			}
		}
		detailsTable.clear();
		detailsTable.setDataVector(details);
		for (int i = 0; i < additionalOutcomePanels.size(); i++) {
			if (additionalOutcomePanels.elementAt(i)
				instanceof AdditionalOutcomePanel) {
				AdditionalOutcomePanel panel =
					(AdditionalOutcomePanel) additionalOutcomePanels.elementAt(
						i);
				panel.resetValues();
			} else {
				Applyable panel =
					(Applyable) additionalOutcomePanels.elementAt(i);
				panel.resetValues();
			}
		}
	}

	/**
	 *  Save stdout or stderr to file
	 *
	 *@param  text   text from stdout or stderr area
	 *@param  title  title of file chooser dialog
	 */
	private void saveToFile(String text, String title) {

		SelectorDialog d =
			ResourceManager.getSelectorDialog(
				new File(ResourceManager.getUserDefaults().getLogDirectory()),
				null);
		d.setTitle(title);

		int result = d.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			String filename = d.getSelectedFile().getAbsolutePath();
			File file = new File(filename);
			try {
				PrintWriter pw =
					new PrintWriter(new BufferedWriter(new FileWriter(file)));
				pw.println(text);
				pw.close();
			} catch (IOException ioe) {
				UserMessages.error(
					"Saving file " + filename + " failed.",
					ioe.getMessage());
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	private void updateFont() {
		Font font =
			ResourceManager.getUserDefaults().getTheme().getControlTextFont();
		int fontSize = font.getSize();
		fixedFont = new Font("monospaced", Font.PLAIN, fontSize);
		stdoutArea.setFont(fixedFont);
		stderrArea.setFont(fixedFont);

		detailsTable.setFont(new Font(font.getName(), font.getStyle(), 10));
	}

	/**
	 *  Description of the Method
	 */
	public void updateValues() {
		if (tabbedPane.getTabCount() > selectedIndex) {
			tabbedPane.getModel().setSelectedIndex(selectedIndex);
		} else {
			tabbedPane.getModel().setSelectedIndex(
				tabbedPane.getTabCount() - 1);
		}
	}

}
