package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.client.controls.IntTextField;
import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.ControlContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.extensions.ReturnCodeTest;
import com.pallas.unicore.extensions.TestType;

/**
 *@author     Ralf Ratering
 *@author     Kirsten Foerster
 *@created    17. Juli 2002
 *@version    $Id: ReturnCodeTestPanel.java,v 1.3 2003/06/30 06:54:28 rrate Exp $
 */
public class ReturnCodeTestPanel extends AbstractTestPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    15. August 2002
	 */
	private class TestBoxListener implements ItemListener {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			updateControls();
		}
	}

	private static int rightInsets = 24;

	/**
	 *  only for testing
	 *
	 *@param  args  Description of the Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("Return Code Test");
		frame.getContentPane().add(
			new ReturnCodeTestPanel(new DoRepeatContainer(), true),
			BorderLayout.CENTER);
		frame.pack();
		frame.setVisible(true);
	}

	private ControlContainer controlContainer;

	private boolean repeatMode;
	private JComboBox taskBox;
	private DefaultComboBoxModel taskBoxModel;

	private JLabel testLabel, valueLabel, taskLabel;
	private IntTextField valueField;

	/**
	 *  Constructor for the ReturnCodeTestPanel object
	 *
	 *@param  controlContainer  Description of the Parameter
	 *@param  repeatMode        Description of the Parameter
	 */
	public ReturnCodeTestPanel(
		ControlContainer controlContainer,
		boolean repeatMode) {
		super(controlContainer);
		this.controlContainer = controlContainer;
		this.repeatMode = repeatMode;
		buildComponents();
		setEnabled(false);
	}

	/**
	 *  Apply values from this gui to ReturnCodeTest
	 */
	public void applyValues() {
		ReturnCodeTest test = controlContainer.getReturnCodeTest();
		test.setType((TestType) testBox.getSelectedItem());
		test.setValue(new Integer(valueField.getText().trim()).intValue());
		test.setTestContainer((ActionContainer) taskBox.getSelectedItem());
	}

	/**
	 *  Build all components of this panel
	 */
	private void buildComponents() {
		GridBagLayout g = new GridBagLayout();
		setLayout(g);

		GridBagConstraints c = new GridBagConstraints();

		if (repeatMode) {
			testLabel = new JLabel("Repeat Until: ");
			testLabel.setToolTipText(
				"Repeat the job task until the status of the return code is achieved.");
		} else {
			testLabel = new JLabel("If: ");
			testLabel.setToolTipText(
				"Interrogate the status of the return code.");
		}
		c = LayoutTools.makegbc(0, 0, 1, 0, false);
		c.insets = new Insets(5, 5, 5, rightInsets);
		add(testLabel, c);

		testBox = new JComboBox(ReturnCodeTest.getTypes());
		testBox.addItemListener(new TestBoxListener());
		if (repeatMode) {
			testBox.setToolTipText(
				"Repeat the job task until the status of the return code is achieved.");
		} else {
			testBox.setToolTipText(
				"Interrogate the status of the return code.");
		}
		c = LayoutTools.makegbc(1, 0, 2, 100, true);
		add(testBox, c);

		valueLabel = new JLabel("Value: ");
		valueLabel.setToolTipText("the value of the return code");
		c = LayoutTools.makegbc(0, 1, 1, 0, false);
		c.insets = new Insets(5, 5, 5, rightInsets);
		add(valueLabel, c);

		valueField = new IntTextField(0, 100);
		valueField.setToolTipText("the value of the return code");
		c = LayoutTools.makegbc(1, 1, 2, 100, true);
		add(valueField, c);

		taskLabel = new JLabel("Task: ");
		taskLabel.setToolTipText("Choose a task.");
		c = LayoutTools.makegbc(0, 2, 1, 0, false);
		c.insets = new Insets(5, 5, 5, rightInsets);
		add(taskLabel, c);

		taskBoxModel = new DefaultComboBoxModel();
		taskBox = new JComboBox();
		taskBox.setModel(taskBoxModel);
		taskBox.setToolTipText("Choose a task.");

		c = LayoutTools.makegbc(1, 2, 2, 100, true);
		add(taskBox, c);

		setBorder(new TitledBorder("Return Code Test"));
	}

	/**
	 *  Fill values from ReturnCodeTest into this gui.
	 */
	public void resetValues() {
		ReturnCodeTest test = controlContainer.getReturnCodeTest();
		testBox.setSelectedItem(test.getType());

		valueField.setText(new Integer(test.getValue()).toString());

		if (test.getTestContainer() != null) {
			taskBoxModel.setSelectedItem(test.getTestContainer());
		}
	}

	/**
	 *  Sets the enabled attribute of the ReturnCodeTestPanel object
	 *
	 *@param  enabled  The new enabled value
	 */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		taskLabel.setEnabled(enabled);
		taskBox.setEnabled(enabled);
		if (enabled) {
			updateControls();
		} else {
			valueLabel.setEnabled(false);
			valueField.setEnabled(false);
		}
	}

	/**
	 *  Description of the Method
	 */
	private void updateControls() {
		TestType type = (TestType) testBox.getSelectedItem();
		boolean enabled =
			type.isEnabled()
				&& !type.equals(ReturnCodeTest.SUCCESSFUL)
				&& !type.equals(ReturnCodeTest.NOT_SUCCESSFUL);
		valueLabel.setEnabled(enabled);
		valueField.setEnabled(enabled);
	}

	/**
	 *  Update values from this panel.
	 *
	 *@param  vsiteChanged  Description of the Parameter
	 */
	public void updateValues(boolean vsiteChanged) {
		logger.finest(
			"Updating values in return code test "
				+ controlContainer
				+ ", vsite changed: "
				+ vsiteChanged);
		Object selectedTask = taskBoxModel.getSelectedItem();

		taskBoxModel.removeAllElements();
		Vector tasks = null;
		if (repeatMode) {
			tasks = controlContainer.getTasks();
		} else {
			tasks = controlContainer.getParentContainer().getTasks();
		}
		for (int i = 0; i < tasks.size(); i++) {
			if (!tasks.elementAt(i).equals(controlContainer)) {
				taskBoxModel.addElement(tasks.elementAt(i));
			}
		}

		if (valueField.getText().equals("")) {
			valueField.setText("0");
		}

		if (tasks.contains(selectedTask)) {
			taskBoxModel.setSelectedItem(selectedTask);
		}

		if (taskBoxModel.getSelectedItem() == null
			&& taskBoxModel.getSize() > 0) {
			taskBox.setSelectedIndex(0);
		}
	}

}
