package com.pallas.unicore.client.panels;

import java.text.NumberFormat;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    April 18, 2002
 *@version    $Id: StatusPanel.java,v 1.5 2003/06/30 06:54:28 rrate Exp $
 */
public class StatusPanel extends JPanel {
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.panels");
	private Icon connectedIcon =
		new ImageIcon(ResourceManager.getImage("estab_conn.gif"));
	private JLabel connectionStatus;
	private JLabel identity;
	private JProgressBar memoryBar;
	private String memoryString = "Java heap memory: ";
	private JTextField message;
	private Icon notConnectedIcon =
		new ImageIcon(ResourceManager.getImage("no_conn.gif"));
	private JProgressBar progressBar;

	/**
	 *  Constructor for the StatusPanel object
	 */
	public StatusPanel() {
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		buildComponents();
	}

	/**
	 *  Adds a feature to the ToStatusArea attribute of the StatusPanel object
	 *
	 *@param  component  The feature to be added to the ToStatusArea attribute
	 */
	public void addToStatusArea(JComponent component) {

		/*
		 *  this.removeAll();
		 *  this.add(component, BorderLayout.WEST);
		 */
		//        this.revalidate();
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		Border border = BorderFactory.createLoweredBevelBorder();
		connectionStatus = new JLabel();
		connectionStatus.setIcon(notConnectedIcon);
		add(connectionStatus);
		identity = new JLabel("No identity");
		identity.setBorder(border);
		identity.setToolTipText("Current identity");
		add(identity);
		message = new JTextField();
		message.setBorder(border);
		message.setToolTipText("Status messages");
		message.setEditable(false);
		add(message);
		add(Box.createHorizontalGlue());
		progressBar = new JProgressBar(0, 1);
		progressBar.setBorder(border);
		add(progressBar);
		memoryBar = new JProgressBar(0, 100);
		memoryBar.setBorder(border);
		memoryBar.setStringPainted(true);
		startMemoryUpdate();
		add(memoryBar);
	}

	/**
	 *  Gets the progressBar attribute of the StatusPanel object
	 *
	 *@return    The progressBar value
	 */
	public JProgressBar getProgressBar() {
		return progressBar;
	}

	/**
	 *  Sets the connected attribute of the StatusPanel object
	 *
	 *@param  connected  The new connected value
	 */
	public void setConnected(final boolean connected) {
		Runnable runner = new Runnable() {
			public void run() {
				if (connected) {
					connectionStatus.setIcon(connectedIcon);
					connectionStatus.setToolTipText(
						"Client is in online mode.");
				} else {
					connectionStatus.setIcon(notConnectedIcon);
					connectionStatus.setToolTipText(
						"Client is in offline mode. Job can be prepared, but not submitted.");
				}
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Sets the identity attribute of the StatusPanel object
	 *
	 *@param  myIdentity  The new identity value
	 */
	public void setIdentity(final String myIdentity) {
		Runnable runner = new Runnable() {
			public void run() {
				identity.setText(myIdentity);
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Sets the progressMaxValue attribute of the StatusPanel object
	 *
	 *@param  max  The new progressMaxValue value
	 */
	public void setProgressMaxValue(int max) {
		progressBar.setMaximum(max);
	}

	/**
	 *  Sets the progressString attribute of the StatusPanel object
	 *
	 *@param  in  The new progressString value
	 */
	public synchronized void setProgressString(final String in) {
		Runnable runner = new Runnable() {
			public void run() {
				progressBar.setStringPainted(true);
				progressBar.setString(in);
				progressBar.revalidate();
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Sets the progressValue attribute of the StatusPanel object
	 *
	 *@param  value  The new progressValue value
	 */
	public synchronized void setProgressValue(final int value) {

		// progressBar.setStringPainted(true);
		Runnable runner = new Runnable() {
			public void run() {
				progressBar.setValue(value);
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Sets the statusMessage attribute of the StatusPanel object
	 *
	 *@param  messageString  The new statusMessage value
	 */
	public synchronized void setStatusMessage(final String messageString) {
		Runnable runner = new Runnable() {
			public void run() {
				message.setText(messageString);
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Description of the Method
	 */
	private void startMemoryUpdate() {
		Thread thread = new Thread() {
			private double freeMemory;
			private double maxMem;
			private NumberFormat nf;

			/**
			 *  Main processing method for the CustomProgressMonitor object
			 */
			private double totalMem;
			private double usedMemory;

			public void run() {
				nf = NumberFormat.getNumberInstance();
				nf.setMaximumFractionDigits(2);
				nf.setMaximumIntegerDigits(3);
				nf.setMinimumFractionDigits(2);
				nf.setMinimumIntegerDigits(1);
				while (true) {
					totalMem =
						Runtime.getRuntime().totalMemory() / 1024.0 / 1024.0;
					maxMem = Runtime.getRuntime().maxMemory() / 1024.0 / 1024.0;
					freeMemory =
						Runtime.getRuntime().freeMemory() / 1024.0 / 1024.0;
					usedMemory = totalMem - freeMemory;

					// memoryBar.setValue((int)(usedMemory / totalMem * 100));
					Runnable runner = new Runnable() {
						public void run() {
							memoryBar.setToolTipText(
								"Allocated: "
									+ nf.format(usedMemory)
									+ "Mb Total: "
									+ nf.format(totalMem)
									+ "Mb Max: "
									+ nf.format(maxMem)
									+ "Mb");
							memoryBar.setString(
								nf.format(usedMemory)
									+ "Mb/"
									+ nf.format(totalMem)
									+ "Mb");
							memoryBar.setValue(
								(int) ((usedMemory / totalMem) * 100.0));
						}
					};
					SwingUtilities.invokeLater(runner);
					try {
						Thread.sleep(20000);
					} catch (InterruptedException e) {
						logger.log(Level.SEVERE, "", e);
					}
				}
			}
		};
		thread.start();
	}

	/**
	 *  Description of the Method
	 */
	public void startProgressBar() {
		Runnable runner = new Runnable() {
			public void run() {
				progressBar.setStringPainted(true);
				progressBar.setValue(0);
				progressBar.setIndeterminate(true);
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Description of the Method
	 */
	public void startWaiting() {
		progressBar.setIndeterminate(true);
	}

	/**
	 *  Description of the Method
	 */
	public void stopProgressBar() {
		Runnable runner = new Runnable() {
			public void run() {
				progressBar.setStringPainted(false);
				progressBar.setValue(0);
				progressBar.setIndeterminate(false);
			}
		};
		SwingUtilities.invokeLater(runner);
	}

	/**
	 *  Description of the Method
	 */
	public void stopWaiting() {
		progressBar.setIndeterminate(false);
	}
}
