package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.metal.DefaultMetalTheme;

import com.pallas.unicore.client.themes.UserTheme;

/**
 *  Panel to define all capacity resource requests
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ThemeEditorPanel.java,v 1.2 2003/06/30 06:54:27 rrate Exp $
 */
public class ThemeEditorPanel extends JPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source.equals(primary1Button)) {
				selectColor(res.getString("PRIMARY1"), primary1Button);
			} else if (source.equals(primary2Button)) {
				selectColor(res.getString("PRIMARY2"), primary2Button);
			} else if (source.equals(primary3Button)) {
				selectColor(res.getString("PRIMARY3"), primary3Button);
			} else if (source.equals(secondary1Button)) {
				selectColor(res.getString("SECONDARY1"), secondary1Button);
			} else if (source.equals(secondary2Button)) {
				selectColor(res.getString("SECONDARY2"), secondary2Button);
			} else if (source.equals(secondary3Button)) {
				selectColor(res.getString("SECONDARY3"), secondary3Button);
			} else if (source.equals(blackButton)) {
				selectColor(res.getString("TEXT_COLOR"), blackButton);
			} else if (source.equals(whiteButton)) {
				selectColor(res.getString("BACK_COLOR"), whiteButton);
			}

			updateTheme();
		}
	}
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	/**
	 *  Description of the Method
	 *
	 *@param  args  Description of Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("Test Theme Editor");
		frame.getContentPane().add(new ThemeEditorPanel());
		frame.pack();
		frame.show();
	}

	private FontSelectionPanel fontPanel;
	private JButton primary1Button,
		primary2Button,
		primary3Button,
		secondary1Button,
		secondary2Button,
		secondary3Button,
		blackButton,
		whiteButton;

	private DefaultMetalTheme theme;

	/**
	 *  Constructor for the ThemeEditorPanel object
	 */
	public ThemeEditorPanel() {
		this(new UserTheme());
	}

	/**
	 *  Constructor for the ThemeEditorPanel object
	 *
	 *@param  theme  Description of Parameter
	 */
	public ThemeEditorPanel(DefaultMetalTheme theme) {
		buildComponents();
		setTheme(theme);
	}

	/**
	 *  Description of the Method
	 */
	protected void buildComponents() {
		ButtonListener listener = new ButtonListener();

		primary1Button = new JButton(" ");
		primary1Button.addActionListener(listener);
		primary1Button.setToolTipText(res.getString("PRIMARY1_TIP"));

		primary2Button = new JButton(" ");
		primary2Button.addActionListener(listener);
		primary2Button.setToolTipText(res.getString("PRIMARY2_TIP"));

		primary3Button = new JButton(" ");
		primary3Button.addActionListener(listener);
		primary3Button.setToolTipText(res.getString("PRIMARY3_TIP"));

		secondary1Button = new JButton(" ");
		secondary1Button.addActionListener(listener);
		secondary1Button.setToolTipText(res.getString("SECONDARY1_TIP"));

		secondary2Button = new JButton(" ");
		secondary2Button.addActionListener(listener);
		secondary2Button.setToolTipText(res.getString("SECONDARY2_TIP"));

		secondary3Button = new JButton(" ");
		secondary3Button.addActionListener(listener);
		secondary3Button.setToolTipText(res.getString("SECONDARY3_TIP"));

		blackButton = new JButton(" ");
		blackButton.addActionListener(listener);
		blackButton.setToolTipText(res.getString("BLACK_TIP"));

		whiteButton = new JButton(" ");
		whiteButton.addActionListener(listener);
		whiteButton.setToolTipText(res.getString("WHITE_TIP"));

		JPanel buttonPanel = new JPanel(new GridLayout(4, 2));
		buttonPanel.setBorder(new TitledBorder(res.getString("COLORS")));
		buttonPanel.add(primary1Button);
		buttonPanel.add(secondary1Button);

		buttonPanel.add(primary2Button);
		buttonPanel.add(secondary2Button);

		buttonPanel.add(primary3Button);
		buttonPanel.add(secondary3Button);

		buttonPanel.add(blackButton);
		buttonPanel.add(whiteButton);

		fontPanel = new FontSelectionPanel();

		fontPanel.addPropertyChangeListener(new PropertyChangeListener() {
			/**
			 *  Description of the Method
			 *
			 *@param  e  Description of Parameter
			 */
			public void propertyChange(PropertyChangeEvent e) {
				if (e.getPropertyName().equals("FONT_CHANGED")) {
					if (theme instanceof UserTheme) {
						((UserTheme) theme).setFont((Font) e.getNewValue());
					}
				}
			}
		});

		JPanel subPanel = new JPanel();
		subPanel.setBorder(new TitledBorder("Font"));
		subPanel.add(fontPanel);

		setLayout(new BorderLayout());
		add(buttonPanel, BorderLayout.CENTER);
		add(subPanel, BorderLayout.EAST);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  title   Description of Parameter
	 *@param  button  Description of Parameter
	 */
	private void selectColor(String title, JButton button) {
		Color color = JColorChooser.showDialog(this, title, button.getBackground());
		if (color == null) {
			return;
		}
		button.setBackground(color);
	}

	/**
	 *  Sets the Enabled attribute of the ThemeEditorPanel object
	 *
	 *@param  enabled  The new Enabled value
	 */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		primary1Button.setEnabled(enabled);
		primary2Button.setEnabled(enabled);
		primary3Button.setEnabled(enabled);
		secondary1Button.setEnabled(enabled);
		secondary2Button.setEnabled(enabled);
		secondary3Button.setEnabled(enabled);
		whiteButton.setEnabled(enabled);
		blackButton.setEnabled(enabled);

		fontPanel.setEnabled(enabled);
	}

	/**
	 *  Sets the Theme attribute of the ThemeEditorPanel object
	 *
	 *@param  theme  The new Theme value
	 */
	public void setTheme(DefaultMetalTheme theme) {
		this.theme = theme;
		primary1Button.setBackground(theme.getSystemTextColor());
		primary2Button.setBackground(theme.getMenuSelectedBackground());
		primary3Button.setBackground(theme.getWindowTitleBackground());
		secondary1Button.setBackground(theme.getControlDarkShadow());
		secondary2Button.setBackground(theme.getInactiveControlTextColor());
		secondary3Button.setBackground(theme.getMenuBackground());
		blackButton.setBackground(theme.getControlTextColor());
		whiteButton.setBackground(theme.getWindowBackground());
		fontPanel.setSelectedFont(theme.getControlTextFont());
		setEnabled(theme instanceof UserTheme);
	}

	/**
	 *  Description of the Method
	 */
	private void updateTheme() {
		if (theme instanceof UserTheme) {
			UserTheme userTheme = (UserTheme) theme;
			userTheme.setPrimary1Color(primary1Button.getBackground());
			userTheme.setPrimary2Color(primary2Button.getBackground());
			userTheme.setPrimary3Color(primary3Button.getBackground());
			userTheme.setSecondary1Color(secondary1Button.getBackground());
			userTheme.setSecondary2Color(secondary2Button.getBackground());
			userTheme.setSecondary3Color(secondary3Button.getBackground());
			userTheme.setBlackColor(blackButton.getBackground());
			userTheme.setWhiteColor(whiteButton.getBackground());

			userTheme.setFont(fontPanel.getSelectedFont());
		}
	}

}
