package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.container.ControlContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.extensions.TestType;
import com.pallas.unicore.extensions.TimeTest;

/**
 *@author     Ralf Ratering
 *@author     Kirsten Foerster
 *@created    17. Juli 2002
 *@version    $Id: TimeTestPanel.java,v 1.2 2003/06/30 06:54:27 rrate Exp $
 */
public class TimeTestPanel extends AbstractTestPanel {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    9. August 2002
	 */
	private class SelectionListener implements ItemListener {

		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent ev) {
			if (ev.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}

			if (ev.getSource() == testBox) {
				JComboBox comboBox = (JComboBox) ev.getSource();
				TestType test = ((TestType) comboBox.getSelectedItem());
				datePanel.setEnabled(test.isEnabled());
				datePanel.updateValues();
			}
		}
	}

	/**
	 *  only for testing
	 *
	 *@param  args  Description of the Parameter
	 */
	public static void main(String args[]) {
		JFrame frame = new JFrame("TimeTest");
		TimeTestPanel panel = new TimeTestPanel(new DoRepeatContainer(), true);
		panel.applyValues();
		panel.resetValues();
		frame.getContentPane().add(panel, BorderLayout.CENTER);
		frame.pack();
		frame.setVisible(true);
	}
	private ControlContainer controlContainer;
	private DatePanel datePanel;
	private boolean repeatMode;

	/**
	 *  Constructor for the TimeTestPanel object
	 *
	 *@param  controlContainer  Description of the Parameter
	 *@param  repeatMode        Description of the Parameter
	 */
	public TimeTestPanel(
		ControlContainer controlContainer,
		boolean repeatMode) {
		super(controlContainer);
		this.controlContainer = controlContainer;
		this.repeatMode = repeatMode;
		buildComponents();
	}

	/**
	 *  Apply values from this gui to TimeTest.
	 */
	public void applyValues() {
		TimeTest test = controlContainer.getTimeTest();
		test.setDate(datePanel.getDate());
		test.setUseToday(datePanel.getUseToday());
		test.setType((TestType) testBox.getSelectedItem());
	}

	/**
	 *  Build all components of this panel.
	 */
	private void buildComponents() {

		JLabel testLabel = new JLabel();
		if (repeatMode) {
			testLabel.setText("Repeat Until: ");
			testLabel.setToolTipText(
				"Repeat group until the choosen date has expired.");
		} else {
			testLabel.setText("If: ");
			testLabel.setToolTipText(
				"Take branch dependent on execution date.");
		}

		testBox = new JComboBox(TimeTest.getTypes());
		testBox.addItemListener(new SelectionListener());
		if (repeatMode) {
			testBox.setToolTipText(
				"Repeat group until the choosen date has expired.");
		} else {
			testBox.setToolTipText("Take branch dependent on execution date.");
		}

		datePanel = new DatePanel(testLabel, testBox);
		setBorder(new TitledBorder("Time Test"));
		setLayout(new BorderLayout());
		add(datePanel, BorderLayout.CENTER);

	}

	/**
	 *  Fill values from TimeTest into this gui.
	 */
	public void resetValues() {
		TimeTest test = controlContainer.getTimeTest();
		testBox.setSelectedItem(test.getType());
		datePanel.setDate(test.getDate());
		datePanel.setUseToday(test.getUseToday());
	}

	/**
	 *  Description of the Method
	 *
	 *@param  vsiteChanged  Description of the Parameter
	 */
	public void updateValues(boolean vsiteChanged) {
		datePanel.updateValues();
	}

}
