package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.pallas.unicore.container.FileTransferContainer;

/**
 *  JPAPanel to define a transfer between two uspaces This panel can be plugged
 *  into the clients data area and can be connected to a jpa tree node
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: TransferJPAPanel.java,v 1.2 2003/06/30 06:54:27 rrate Exp $
 */
public class TransferJPAPanel extends JPAPanel {

	// additional text fields
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.panels.ResourceStrings");

	/**
	 *  Test main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {

		JFrame app = new JFrame();
		app.getContentPane().add(
			new TransferJPAPanel(app, new FileTransferContainer(null)));
		app.pack();
		app.show();

	}
	private FileTransferContainer container;
	private JTextField nameTextField;

	private FileTransferPanel transferPanel;

	/**
	 *  Constructor
	 *
	 *@param  parentFrame        parent JFrame
	 *@param  container  container to be modified
	 */
	public TransferJPAPanel(
		JFrame parentFrame,
		FileTransferContainer container) {
		super(parentFrame, container);
		this.container = container;
		buildComponents();
	}

	/**
	 *  Overwrite abstract method to apply the current GUI values to the
	 *  container
	 */
	public void applyValues() {
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());
		transferPanel.applyValues();
	}

	/**
	 *  Build GUI
	 */
	private void buildComponents() {

		transferPanel = new FileTransferPanel(container);

		nameTextField = new JTextField(20);
		nameTextField.setToolTipText(res.getString("ENTER_IDENTIFIER"));

		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel(res.getString("NAME")));
		namePanel.add(nameTextField);

		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.add(namePanel, BorderLayout.NORTH);
		subPanel.add(transferPanel, BorderLayout.CENTER);

		setTitledBorder(res.getString("TRANSFER_BORDER"));
		add(subPanel, BorderLayout.CENTER);

		revalidate();
	}

	/**
	 *  Overwrite abstract method to reset current GUI entries to last container
	 *  contents.
	 */
	public void resetValues() {
		nameTextField.setText(container.getName());
		transferPanel.resetValues();
	}

	/**
	 *  Overwrite abstract method that will be called whenever the panel becomes
	 *  visible
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public void updateValues(boolean vsiteChanged) {
		transferPanel.updateValues();
	}

}
