package com.pallas.unicore.client.plugins.autoupdate;

import java.awt.Color;
import java.awt.Component;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import com.pallas.unicore.client.trees.PanelNode;

/**
 *  Cell renderer for displaying the different icons in a table
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: WatchTableCellRenderer.java,v 1.2 2003/06/30 06:55:06 rrate Exp $
 */

public class WatchTableCellRenderer extends DefaultTableCellRenderer {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.plugins.autoupdate");

	/**
	 *  Constructor for the WatchTableCellRenderer object
	 */
	public WatchTableCellRenderer() {
	}

	/**
	 *  Sets the value of the current table cell to value.
	 *
	 *@param  table      is the JTable the receiver is being configured for
	 *@param  object     is the current column
	 *@param  selected   if true, the cell will be drawn as if selected.
	 *@param  row
	 *@param  column
	 *@param  isFocused  Description of Parameter
	 *@return            the Component that the renderer uses to draw the value
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object object,
		boolean selected,
		boolean isFocused,
		int row,
		int column) {

		Component c =
			super.getTableCellRendererComponent(
				table,
				object,
				selected,
				isFocused,
				row,
				column);

		if (!(c instanceof JLabel)) {
			logger.warning("Table component is not a JLabel, what else is it?");
			return c;
		}

		JLabel label = (JLabel) c;

		// if we have a PanelNode get icon from container
		// else choose a default icon
		if (object instanceof PanelNode) {
			PanelNode node = (PanelNode) object;
			ImageIcon icon = node.getCurrentIcon();
			if (icon != null) {
				label.setIcon(icon);
			}
			if (node.isUpToDate()) {
				label.setForeground(Color.black);
			} else {
				label.setForeground(Color.gray);
			}
		}

		//	label.setText("");
		label.setHorizontalAlignment(JLabel.CENTER);

		return c;
	}

}
