package com.pallas.unicore.client.plugins.command;

import java.io.File;
import java.util.ResourceBundle;

import org.unicore.resources.Resource;
import org.unicore.resources.Storage;

import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Container needed to import and execute a command on a vsite.
 *
 *@author     Thomas Kentemich
 *@created    21. August 2001
 *@version    $Id: CommandContainer.java,v 1.2 2003/06/30 06:55:15 rrate Exp $
 */
public class CommandContainer extends UserContainer {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "COMMANDDIRECTORY", "commandDirectory" }
	};

	// local command directory
	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.plugins.command.ResourceStrings");

	final static long serialVersionUID = -8765069416469853521L;
	private String commandDirectory;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  Description of Parameter
	 */
	public CommandContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		if (getVsite() == null) {
			return err;
		}

		if ((softwareResource == null) && (commandImport == null)) {
			err.add(new UError(getIdentifier(), res.getString("NO_SELECTION")));
		} else if (commandImport != null) {
			if (commandImport.getSourceName().equals("")) {
				err.add(
					new UError(
						getIdentifier(),
						res.getString("NO_EXECUTABLE")));
			} else if (
				commandImport.getStorageItem().equals(
					FileStorage.NSPACE_STRING)) {
				File commandFile = new File(commandImport.getSourceName());
				if (!commandFile.exists()) {
					err.add(
						new UError(
							getIdentifier(),
							res.getString("MISSING_LOCALIMPORT")
								+ commandImport.getSourceName()));
				}
			} else {
				// search for StorageResource in site resources
				NamedResourceSet siteResources =
					ResourceManager.getResourceSet(getVsite());
				Storage storage = (Storage) commandImport.getStorageItem();
				Resource resource = siteResources.findResource(storage);
				if (resource == null) {
					err.add(
						new UError(
							id,
							res.getString("STORAGE_RESOURCE")
								+ storage
								+ res.getString("NOT_AVAILABLE")
								+ getVsite()));
				}
			}
		} else if (softwareResource != null) {
			// search for SoftwareResource in site resources
			NamedResourceSet siteResources =
				ResourceManager.getResourceSet(getVsite());
			Resource resource = siteResources.findResource(softwareResource);
			if (resource == null) {
				err.add(
					new UError(
						id,
						res.getString("SOFTWARE_RESOURCE")
							+ softwareResource
							+ res.getString("NOT_AVAILABLE")
							+ getVsite()));
			}

		}

		setErrors(err);

		return err;
	}

	/**
	 *  Gets the CommandDirectory attribute of the CommandContainer object
	 *
	 *@return    The CommandDirectory value
	 */
	public String getCommandDirectory() {
		return commandDirectory;
	}

	/**
	 *  Sets the CommandDirectory attribute of the CommandContainer object
	 *
	 *@param  commandDirectory  The new CommandDirectory value
	 */
	public void setCommandDirectory(String commandDirectory) {
		this.commandDirectory = commandDirectory;
	}

}
