package com.pallas.unicore.client.plugins.command;

import java.io.File;

import com.pallas.unicore.client.util.Defaults;

/**
 *  Define, load and save command plugin default values
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: CommandDefaults.java,v 1.3 2003/06/30 06:55:15 rrate Exp $
 */
public class CommandDefaults extends Defaults {

	/**
	 *  String constants defining property keys
	 */
	public final static String COMMANDDIRECTORY_KEY =
		"plugin.command.commanddirectory";

	/**
	 *  Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(COMMANDDIRECTORY_KEY, userHome);
	}

	/**
	 *  Gets the CommandDirectory attribute of the CommandDefaults object
	 *
	 *@return    The CommandDirectory value
	 */
	public String getCommandDirectory() {
		return getProperty(COMMANDDIRECTORY_KEY);
	}

	/**
	 *  Load command default values from file ${HOME}/.unicoreXX/CommandDefaults.txt
	 *
	 *@return    true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		// look for  ${HOME}/.unicoreXX/CommandDefaults.txt
		return loadFromFile(
			unicoreDir + File.separator + "CommandDefaults.txt");
	}

	/**
	 *  Sets the CommandDirectory attribute of the CommandDefaults object
	 *
	 *@param  s  The new CommandDirectory value
	 */
	public void setCommandDirectory(String s) {
		put(COMMANDDIRECTORY_KEY, s);
	}

	/**
	 *  Try to write script default values to file ${HOME}/.unicoreXX/CommandDefaults.txt
	 *
	 *@return    true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicoreXX/CommandDefaults.txt
		return writeToFile(unicoreDir + File.separator + "CommandDefaults.txt");
	}

}
