package com.pallas.unicore.client.plugins.command;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import org.unicore.resources.Resource;
import org.unicore.resources.SoftwareResource;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;

import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.panels.FileExportPanel;
import com.pallas.unicore.client.panels.FileImportPanel;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.panels.OptionPanel;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystem;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Panel that represents Plugin Container in the Client GUI's data area
 *
 *@author     Thomas Kentemich
 *@created    20. August 2001
 *@version    $Id: CommandJPAPanel.java,v 1.3 2003/06/30 06:55:14 rrate Exp $
 */
public class CommandJPAPanel extends JPAPanel {

	/**
	 *  Listener class listens to Button events
	 *
	 *@author     rrate
	 *@created    20. August 2001
	 */
	private class ButtonListener implements ActionListener {
		private Object lock = new Object();

		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void actionPerformed(ActionEvent event) {
			synchronized (lock) {
				Object source = event.getSource();
				if (source == load) {
					showBrowser();
				}
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    20. August 2001
	 */
	private class ComboBoxListener implements ItemListener {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			if (e.getStateChange() == ItemEvent.SELECTED) {
				Object item = e.getItem();
				if (item instanceof SoftwareResource) {
					if (!softwareResources.contains(item)) {
						sources.remove(item);
					}
				} else {
					if (!storageResources.contains(item)) {
						sources.remove(item);
					}

					if (item instanceof USpace) {
						load.setEnabled(false);
					} else {
						load.setEnabled(true);
					}
				}
			}
		}
	}

	/**
	 *  Toggles the contents of the ComboBox according to the setting of the
	 *  radio buttons that select software or storage resources
	 *
	 *@author     rrate
	 *@created    July 22, 2002
	 */
	private class RadioButtonListener implements ItemListener {
		/**
		 *  Description of the Method
		 *
		 *@param  evt  Description of the Parameter
		 */
		public void itemStateChanged(ItemEvent evt) {
			if (evt.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}

			if (evt.getSource().equals(softwareResButton)) {
				copyVector(softwareResources, sources);
				filenameTextField.setText("");
				filenameTextField.setEditable(false);
				load.setEnabled(false);
			} else {
				copyVector(storageResources, sources);
				filenameTextField.setEditable(true);
				load.setEnabled(true);
			}

			if (sources.size() > 0) {
				sourceComboBox.setSelectedIndex(0);
				if (sourceComboBox.getSelectedItem() instanceof USpace) {
					load.setEnabled(false);
				}
			} else {
				sourceComboBox.setSelectedItem(null);
			}

			sourceComboBox.repaint();
		}

	}
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.plugins.command");
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.plugins.command.ResourceStrings");
	private ButtonListener buttonListener = new ButtonListener();
	private CommandContainer container;
	private String defaultDirectory;
	private FileExportPanel exportPanel;
	private JTextField filenameTextField;
	private FileImportPanel importPanel;
	private JButton load;
	private JTextField nameTextField;
	private JPanel optionsArea;
	private OptionPanel optionsPanel;
	private JRadioButton softwareResButton;
	private Vector softwareResources = new Vector();
	private JComboBox sourceComboBox;
	private Vector sources = new Vector();
	private JRadioButton storageButton;
	private Vector storageResources = new Vector();
	private UserDefaults userDefaults = ResourceManager.getUserDefaults();

	/**
	 *  Constructor
	 *
	 *@param  parentFrame       reference to main JFrame for modal sub dialogs
	 *@param  container  Description of Parameter
	 */
	public CommandJPAPanel(JFrame parentFrame, CommandContainer container) {
		super(parentFrame, container);
		this.container = container;

		// build gui
		buildComponents();

		// init sources
		initValues();
	}

	/**
	 *  Description of the Method
	 */
	public void applyValues() {

		// replace spaces in script name with _
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
		container.setName(nameTextField.getText());

		// TODO: implement modification time
		//container.setModifiedTime(new Date(System.currentTimeMillis()));
		Object selectedItem = sourceComboBox.getSelectedItem();
		if (selectedItem instanceof SoftwareResource) {
			container.setPreinstalledSoftware((Resource) (selectedItem));
			container.setCommand(null);
		} else if (selectedItem != null) {

			// build an import using name as uspace name for command
			String path = filenameTextField.getText();

			// If the selectedItem is the Uspace then the command has been imported
			// already elsewhere, possibly into a sub directory of the Uspace
			// ==> take the full relative path as destination
			FileImport commandImport = new FileImport(selectedItem, path, path);
			if (!(selectedItem instanceof USpace)) {
				// the command has not been imported already elsewhere
				// ==> take the basename as destination
				commandImport.setDestinationName(
					commandImport.getDestinationBasename());
			}
			container.setCommand(commandImport);
			container.setPreinstalledSoftware(null);
		} else {
			logger.warning(
				"No Software Resource selected in Command JPA PAnel!");
		}
		optionsPanel.applyValues();
		importPanel.applyValues();
		exportPanel.applyValues();
		/*
		    FileImport commandImport = container.getCommand();
		    if(commandImport != null) {
		
		        // select storage resource if available
		        softwareResButton.setSelected(false);
		        storageButton.setSelected(true);    // triggers copyVector(storageResources, sources)
		        sourceComboBox.setSelectedItem(commandImport.getStorageItem());
		    }
		    Resource softwareResource = container.getPreinstalledSoftware();
		    if(softwareResource != null) {
		        softwareResButton.setSelected(true);    // triggers copyVector(softwareResources, sources)
		        storageButton.setSelected(false);
		        sourceComboBox.setSelectedItem(softwareResource);
		    }
		*/
	}

	/**
	 *  Description of the Method
	 *
	 *@param  box   Description of Parameter
	 *@param  item  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	private boolean boxContainsItem(JComboBox box, Object item) {
		boolean contained = false;
		int itemCount = box.getItemCount();
		for (int i = 0; i < itemCount; i++) {
			Object next = box.getItemAt(i);
			if (next.equals(item)) {
				contained = true;
				break;
			}
		}
		return contained;
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		nameTextField = new JTextField(20);
		nameTextField.setToolTipText(res.getString("ID_TIP"));
		JPanel namePanel = new JPanel();
		namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.X_AXIS));
		namePanel.add(new JLabel("Task Name: "));
		namePanel.add(nameTextField);

		sourceComboBox = new JComboBox(sources);
		sourceComboBox.addItemListener(new ComboBoxListener());
		//	sourceComboBox.setEditable(true);
		ButtonGroup grp = new ButtonGroup();
		softwareResButton = new JRadioButton("Software Resource");
		storageButton = new JRadioButton("Command");
		grp.add(softwareResButton);
		grp.add(storageButton);
		RadioButtonListener listener = new RadioButtonListener();
		softwareResButton.addItemListener(listener);
		storageButton.addItemListener(listener);
		JPanel radioPanel = new JPanel();
		radioPanel.setLayout(new BoxLayout(radioPanel, BoxLayout.X_AXIS));
		radioPanel.add(softwareResButton);
		radioPanel.add(storageButton);
		softwareResources = new Vector();

		/*
		 *  softwareResourceComboBox = new JComboBox(softwareResources);
		 *  softwareResourceComboBox.addItemListener(new ComboBoxListener());
		 */
		filenameTextField = new JTextField(30);
		filenameTextField.setToolTipText(res.getString("FILENAME_TIP"));
		filenameTextField.setEditable(true);
		load = new JButton(res.getString("BROWSE"));
		load.addActionListener(buttonListener);
		JPanel sourcePanel = new JPanel(new GridLayout(2, 2, 10, 10));
		sourcePanel.setBorder(new TitledBorder("Command locations"));
		sourcePanel.add(sourceComboBox);
		sourcePanel.add(radioPanel);
		sourcePanel.add(filenameTextField);
		sourcePanel.add(load);
		importPanel =
			new FileImportPanel(container, userDefaults.getImportDirectory());
		exportPanel =
			new FileExportPanel(container, userDefaults.getExportDirectory());
		optionsPanel = new OptionPanel(container);
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add(optionsPanel, res.getString("OPTIONS"));
		tabbedPane.add(importPanel, res.getString("IMPORTS"));
		tabbedPane.add(exportPanel, res.getString("EXPORTS"));

		JPanel centerPanel = new JPanel(new BorderLayout());
		centerPanel.add(sourcePanel, BorderLayout.NORTH);
		centerPanel.add(tabbedPane, BorderLayout.CENTER);

		add(namePanel, BorderLayout.NORTH);
		add(centerPanel, BorderLayout.CENTER);

		setBorder(new TitledBorder(res.getString("BORDER")));
		revalidate();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  in   Description of the Parameter
	 *@param  out  Description of the Parameter
	 */
	private void copyVector(Vector in, Vector out) {
		out.clear();
		out.addAll(in);
	}

	/**
	 *  Description of the Method
	 */
	private void initValues() {
		updateSources();
		defaultDirectory = container.getCommandDirectory();
		storageButton.setSelected(false);
		softwareResButton.setSelected(true);
		// triggers copyVector(softwareResources, sources)
	}

	/**
	 *  Description of the Method
	 */
	public void resetValues() {

		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}

		nameTextField.setText(container.getName());

		FileImport commandImport = container.getCommand();
		Resource softwareResource = container.getPreinstalledSoftware();
		if (softwareResource != null) {
			softwareResButton.setSelected(true);
			// triggers copyVector(softwareResources, sources)
			storageButton.setSelected(false);
			if (!sources.contains(softwareResource)) {
				sources.add(softwareResource);
			}
			sourceComboBox.setSelectedItem(softwareResource);
		} else if (commandImport != null) {
			String filename = commandImport.getSourceName();
			if (filename != null) {
				filenameTextField.setText(filename);
			}

			// select storage resource if available
			softwareResButton.setSelected(false);
			storageButton.setSelected(true);
			// triggers copyVector(storageResources, sources)
			Object storageItem = commandImport.getStorageItem();
			if (!sources.contains(storageItem)) {
				sources.add(storageItem);
			}
			sourceComboBox.setSelectedItem(storageItem);
		}

		optionsPanel.resetValues();
		importPanel.resetValues();
		exportPanel.resetValues();
	}

	/**
	 *  Description of the Method
	 */
	private void showBrowser() {
		File selectedFile = null;
		Object storageItem = sourceComboBox.getSelectedItem();
		String filename = filenameTextField.getText();
		if (storageItem.equals(FileStorage.NSPACE_STRING)) {
			selectedFile = new File(filename);
		} else {
			selectedFile =
				new RemoteFile(
					filename,
					(Storage) storageItem,
					new RemoteFileSystem(container.getVsite()));
		}
		logger.fine("Selected file: " + selectedFile);

		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(
				selectedFile,
				container.getVsite());
		chooser.setTitle("UNICOREpro: Choose command");
		chooser.setLocalEnabled(true);
		chooser.setFileSelectionMode(SelectorDialog.FILES_ONLY);

		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile();
			if (file instanceof RemoteFile) {
				RemoteFile rFile = (RemoteFile) file;
				filenameTextField.setText(rFile.getSubspacePath());
				sourceComboBox.setSelectedItem(rFile.getStorage());
			} else {
				filenameTextField.setText(file.getPath());
				sourceComboBox.setSelectedItem(FileStorage.NSPACE_STRING);
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	private void updateSources() {
		NamedResourceSet siteResources =
			ResourceManager.getResourceSet(container.getVsite());
		softwareResources.clear();
		storageResources.clear();

		// always add local
		storageResources.add(FileStorage.NSPACE_STRING);
		siteResources.addStorageResources(storageResources);

		//siteResources.addHomeResources(sources);
		siteResources.addSoftwareResources(softwareResources);

		// filter out Context's
		Vector contexts = siteResources.addContextResources(new Vector());
		for (int i = 0; i < contexts.size(); i++) {
			softwareResources.remove(contexts.elementAt(i));
		}
		Vector cloned = (Vector) storageResources.clone();
		for (int i = 0; i < cloned.size(); i++) {
			if (cloned.elementAt(i) instanceof org.unicore.resources.Spool
				|| cloned.elementAt(i)
					instanceof org.unicore.resources.AlternativeUspace) {
				storageResources.remove(cloned.elementAt(i));
			}
		}
		cloned.clear();

		Object selectedItem = sourceComboBox.getSelectedItem();
		if (softwareResButton.isSelected()) {
			copyVector(softwareResources, sources);

			Resource softwareResource = container.getPreinstalledSoftware();
			if (softwareResource != null) {
				if (!sources.contains(softwareResource)) {
					sources.add(softwareResource);
				}
			}
		} else if (storageButton.isSelected()) {
			copyVector(storageResources, sources);

			FileImport commandImport = container.getCommand();
			if (commandImport != null) {
				Object storageItem = commandImport.getStorageItem();
				if (!sources.contains(storageItem)) {
					sources.add(storageItem);
				}
			}
		} else {
			sources.clear();
		}
		// Trick: select null first to trigger a selected item changed event
		//        otherwise sourceComboBox does not get the new list
		sourceComboBox.setSelectedItem(null);
		if (selectedItem != null) {
			sourceComboBox.setSelectedItem(selectedItem);
		} else if (sources.size() > 0) {
			sourceComboBox.setSelectedIndex(0);
		}
	}

	/**
	 *  Method will be called whenever this panel is activated
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public void updateValues(boolean vsiteChanged) {

		// if we are online get current storage resources from server
		if (container.getVsite() != null) {
			if (vsiteChanged) {
				updateSources();

				// update storage resources for sub panels
				importPanel.updateValues();
				exportPanel.updateValues();
			}
		}
	}
}
