package com.pallas.unicore.client.plugins.command;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 *  This class manages the execute command plugin
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: CommandPlugin.java,v 1.2 2003/06/30 06:55:13 rrate Exp $
 */
public class CommandPlugin extends TaskPlugable {

	/**
	 *  Listener class for settings menu entry
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class SettingsListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void actionPerformed(ActionEvent event) {
			showDefaultDialog();
		}
	}

	// plugin in specific user defaults
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.plugins.command.ResourceStrings");

	private CommandDefaults commandDefaults;

	/**
	 *  Return a new container for this plugin
	 *
	 *@param  parentContainer  Description of Parameter
	 *@return                  new initialized instance of plugin container
	 */
	public ActionContainer getContainerInstance(GroupContainer parentContainer) {

		// build new plugin container
		CommandContainer container = new CommandContainer(parentContainer);
		container.setName(res.getString("INITIAL_NAME") + counter);
		container.setIcon(new ImageIcon(getIconImage()));

		// apply defaults
		container.setCommandDirectory(commandDefaults.getCommandDirectory());

		// count instance
		counter++;

		return container;
	}

	/**
	 *  Gets the iconPath attribute of the CommandPlugin object
	 *
	 *@return    The iconPath value
	 */
	public String getIconPath() {
		return "com/pallas/unicore/client/plugins/command/command.gif";
	}

	/**
	 *@return    "Command" String
	 */
	public String getName() {
		return "Command";
	}

	/**
	 *  Build a new panel that manages the settings of a plugin container
	 *
	 *@param  tCont  plugin container to build panel for
	 *@return        JPAPanel that represents container settings in GUI
	 */
	public JPAPanel getPanelInstance(ActionContainer tCont) {
		// build new plugin panel
		CommandJPAPanel panel =
			new CommandJPAPanel(getClient(), (CommandContainer) tCont);

		return panel;
	}

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return res.getString("INFO_STRING");
	}

	/**
	 *  Gets the SettingsItem attribute of the CommandPlugin object
	 *
	 *@return    The SettingsItem value
	 */
	public JMenuItem getSettingsItem() {
		// handling setting menu events must be done by the plugin writers
		// as this is an optional feature
		JMenuItem settingsItem =
			new JMenuItem(res.getString("DEFAULT_ITEM"), 'C');
		settingsItem.addActionListener(new SettingsListener());
		return settingsItem;
	}

	/**
	 *  Show a dialog that configures plugin defaults
	 */
	private void showDefaultDialog() {
		CommandDefaultsDialog commandDefaultsDialog =
			new CommandDefaultsDialog(getClient(), commandDefaults);
		commandDefaultsDialog.show();
	}

	/**
	 *  Called to inititialize the plugin
	 */
	public void startPlugin() {
		commandDefaults = new CommandDefaults();
	}

	/**
	 *  Called when terminating
	 */
	public void stopPlugin() {
		// do nothing
	}
}
