package com.pallas.unicore.client.plugins.compile;

import java.io.*;

import com.pallas.unicore.client.util.Defaults;

/**
 *  Define, load and save command plugin default values
 *
 *@author     Thomas Kentemich
 *@created    23. April 2002
 *@version    $Id: CompileDefaults.java,v 1.1.1.1 2002/04/22 08:30:25 thomas Exp
 *      $
 */
public class CompileDefaults extends Defaults {

    /**
     *  String constants defining property keys
     */
    public final static String
            COMMANDDIRECTORY_KEY = "plugin.command.commanddirectory";


    /**
     *  Sets the commandDirectory attribute of the CompileDefaults object
     *
     *@param  s  The new commandDirectory value
     */
    public void setCommandDirectory(String s) {
        put(COMMANDDIRECTORY_KEY, s);
    }

    /**
     *  Gets the commandDirectory attribute of the CompileDefaults object
     *
     *@return    The commandDirectory value
     */
    public String getCommandDirectory() {
        return getProperty(COMMANDDIRECTORY_KEY);
    }


    /**
     *  Load command default values from file ${HOME}/.unicoreXX/CommandDefaults.txt
     *
     *@return    true if loading was successful, else false
     */
    public boolean loadFromFile() {
        // look for  ${HOME}/.unicoreXX/CommandDefaults.txt
        return loadFromFile(unicoreDir + File.separator + "CompileDefaults.txt");
    }


    /**
     *  Try to write script default values to file ${HOME}/.unicoreXX/CommandDefaults.txt
     *
     *@return    true if saving was successful, else false
     */
    public boolean writeToFile() {
        // always overwrite old ${HOME}/.unicoreXX/CommandDefaults.txt
        return writeToFile(unicoreDir + File.separator + "CompileDefaults.txt");
    }


    /**
     *  Fill properties with default values
     */
    protected void genSystemDefaults() {
        put(COMMANDDIRECTORY_KEY, userHome);
    }

}
