package com.pallas.unicore.client.plugins.compile;
import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;

import org.unicore.ajo.ScriptType;
import org.unicore.resources.*;
import org.unicore.sets.*;

import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.dialogs.TextfieldDialog;
import com.pallas.unicore.client.panels.*;
import com.pallas.unicore.client.remotefilechooser.*;
import com.pallas.unicore.connection.*;
import com.pallas.unicore.container.*;
import com.pallas.unicore.extensions.*;
import com.pallas.unicore.resourcemanager.*;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Panel that represents Plugin Container in the Client GUI's data area
 *
 *@author     Thomas Kentemich
 *@created    3. Juni 2002
 *@version    $Id: CompileJPAPanel.java,v 1.1.1.1 2002/04/22 08:30:25 thomas Exp
 *      $
 */

public class CompileJPAPanel extends JPAPanel {
    private UserDefaults userDefaults = ResourceManager.getUserDefaults();

    private ButtonListener buttonListener = new ButtonListener();

    private FortranContainer container;

    private FileImportPanel importPanel;
    private FileExportPanel exportPanel;
    private CompileTaskOptionPanel optionsPanel;
    private CompilerOptionPanel compilerOptionPanel;
    private LinkerOptionPanel linkerOptionPanel;
    private SourcesPanel sourcePanel;
    private ObjectsPanel objectPanel;

    private JButton
            load;

    private Vector
            sources;

    private JTextField
            nameTextField;

    private JPanel optionsArea;

    private String defaultDirectory;


    /**
     *  Constructor
     *
     *@param  parentFrame       reference to main JFrame for modal sub dialogs
     *@param  container  Description of the Parameter
     */
    public CompileJPAPanel(JFrame parentFrame, FortranContainer container) {
        super(parentFrame, container);
        this.container = container;

        // build gui
        buildComponents();

        // init sources
        initValues();
    }


    /**
     *  Description of the Method
     */
    public void resetValues() {
        // check if container has id and name
        if(container.getIdentifier() == null) {
            return;
        }
        nameTextField.setText(container.getName());

        optionsPanel.resetValues();
        compilerOptionPanel.resetValues();
        sourcePanel.resetValues();
        objectPanel.resetValues();
        linkerOptionPanel.resetValues();
        importPanel.resetValues();
        exportPanel.resetValues();

    }


    /**
     *  Method will be called whenever this panel is activated
     *
     *@param  vsiteChanged  true, if vsite has changed since last update or
     *      reset
     */
    public void updateValues(boolean vsiteChanged) {
        // if we are online get current storage resources from server
        if(container.getVsite() != null) {
            if(vsiteChanged) {
                // update storage resources for sub panels
                importPanel.updateValues();
                exportPanel.updateValues();
            }
        }
    }


    /**
     *  Description of the Method
     */
    public void applyValues() {
        // replace spaces in script name with _
        nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));
        container.setName(nameTextField.getText());

        container.setModifiedTime(new Date());

        optionsPanel.applyValues();
        importPanel.applyValues();
        exportPanel.applyValues();
        sourcePanel.applyValues();
        objectPanel.applyValues();
        compilerOptionPanel.applyValues();
        linkerOptionPanel.applyValues();

    }


    /**
     *  Description of the Method
     */
    private void buildComponents() {
        nameTextField = new JTextField(20);
        nameTextField.setToolTipText("Enter identifier for this command");

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.anchor = GridBagConstraints.EAST;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        JPanel labelTextPanel = new JPanel(gridbag);
        addToGridBag(new JLabel("Task Name: "), nameTextField, constraints, gridbag, labelTextPanel);

        JPanel controlPanel = new JPanel();
        controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));
        controlPanel.add(labelTextPanel);

        optionsPanel = new CompileTaskOptionPanel(container);
        compilerOptionPanel = new CompilerOptionPanel(container);
        sourcePanel = new SourcesPanel(container);
        objectPanel = new ObjectsPanel(container, sourcePanel);
        linkerOptionPanel = new LinkerOptionPanel(container);
        importPanel = new FileImportPanel(container);
        exportPanel = new FileExportPanel(container);

        JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.add(sourcePanel, "Sources");
        tabbedPane.add(objectPanel, "Objects");
        tabbedPane.add(compilerOptionPanel, "Compiler Options");
        tabbedPane.add(linkerOptionPanel, "Linker Options");
        tabbedPane.add(importPanel, "File Imports");
        tabbedPane.add(exportPanel, "File Exports");
        tabbedPane.add(optionsPanel, "Compile Task Options");

        add(controlPanel, BorderLayout.NORTH);
        add(tabbedPane, BorderLayout.CENTER);

        setBorder(new TitledBorder("Compile and Link"));

        revalidate();
    }


    /**
     *  Description of the Method
     */
    private void initValues() {
//	  defaultDirectory = container.getCommandDirectory();
    }


    /**
     *  Adds a feature to the ToGridBag attribute of the CompileJPAPanel object
     *
     *@param  label        The feature to be added to the ToGridBag attribute
     *@param  component    The feature to be added to the ToGridBag attribute
     *@param  constraints  The feature to be added to the ToGridBag attribute
     *@param  gridbag      The feature to be added to the ToGridBag attribute
     *@param  panel        The feature to be added to the ToGridBag attribute
     */
    private void addToGridBag(JLabel label, JComponent component,
            GridBagConstraints constraints, GridBagLayout gridbag,
            JPanel panel) {
        constraints.gridwidth = GridBagConstraints.RELATIVE;
        gridbag.setConstraints(label, constraints);
        panel.add(label);
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        gridbag.setConstraints(component, constraints);
        panel.add(component);
    }



    /**
     *  Listener class listens to Button events
     *
     *@author     rrate
     *@created    3. Juni 2002
     */
    private class ButtonListener implements ActionListener {
        private Object lock = new Object();


        /**
         *  Description of the Method
         *
         *@param  event  Description of the Parameter
         */
        public void actionPerformed(ActionEvent event) {

        }
    }

}
