package com.pallas.unicore.client.plugins.compile;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.Vector;
import javax.help.HelpSet;
import javax.swing.*;

import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.resources.*;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.panels.*;
import com.pallas.unicore.client.util.*;
import com.pallas.unicore.container.*;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  This class manages the execute Compile plugin
 *
 *@author     Thomas Kentemich
 *@created    10. April 2002
 *@version    $Id: CompilePlugin.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class CompilePlugin extends TaskPlugable {

    // plugin in specific user defaults
    private CompileDefaults compileDefaults;


    /**
     *  Get a short info message for the plugin
     *
     *@return    String with info message
     */
    public String getPluginInfo() {
        return "Execute Compile Plugin V1.0\n" +
                "(C)2001 Pallas GmbH";
    }


    /**
     *  Gets the name attribute of the CompilePlugin object
     *
     *@return    The name value
     */
    public String getName() {
        return "Compile";
    }



    /**
     *  Gets the settingsItem attribute of the CompilePlugin object
     *
     *@return    The settingsItem value
     */
    public JMenuItem getSettingsItem() {
        // handling setting menu events must be done by the plugin writers
        // as this is an optional feature
        JMenuItem settingsItem = new JMenuItem("Compile Defaults", 'C');
        settingsItem.addActionListener(new SettingsListener());
        return settingsItem;
    }


    /**
     *  Return a new container for this plugin
     *
     *@param  parentContainer  Description of the Parameter
     *@return                  new initialized instance of plugin container
     */
    public ActionContainer getContainerInstance(GroupContainer parentContainer) {

        // build new plugin container
        FortranContainer container = new FortranContainer(parentContainer);
        container.setName("New_Compile" + counter);
        container.setIcon(new ImageIcon(getIconImage()));

        // count instance
        counter++;
        return container;
    }


    /**
     *  Build a new panel that manages the settings of a plugin container
     *
     *@param  tCont  plugin container to build panel for
     *@return        JPAPanel that represents container settings in GUI
     */
    public JPAPanel getPanelInstance(ActionContainer tCont) {
        // build new plugin panel
        CompileJPAPanel panel =
                new CompileJPAPanel(getClient(), (FortranContainer)tCont);

        return panel;
    }


    /**
     *  Gets the iconPath attribute of the CompilePlugin object
     *
     *@return    The iconPath value
     */
    public String getIconPath() {
        return "com/pallas/unicore/client/plugins/compile/compile.gif";
    }


    /**
     *  Called to inititialize the plugin
     */
    public void startPlugin() {
        compileDefaults = new CompileDefaults();
    }


    /**
     *  Called when terminating
     */
    public void stopPlugin() {
        // do nothing
    }


    /**
     *  Show a dialog that configures plugin defaults
     */
    private void showDefaultDialog() {
        CompileDefaultsDialog compileDefaultsDialog
                 = new CompileDefaultsDialog(getClient(), compileDefaults);
        compileDefaultsDialog.show();
    }


    /**
     *  Listener class for settings menu entry
     *
     *@author     rrate
     *@created    10. April 2002
     */
    private class SettingsListener implements ActionListener {
        /**
         *  Description of the Method
         *
         *@param  event  Description of the Parameter
         */
        public void actionPerformed(ActionEvent event) {
            showDefaultDialog();
        }
    }
}
