package com.pallas.unicore.client.plugins.compile;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.StringTokenizer;

import org.unicore.sets.OptionSet;
import org.unicore.ajo.Option;
import com.pallas.unicore.container.*;

/**
 * Panel to define options for executable commands
 *
 * @author Thomas Kentemich
 * @version $Id: CompileTaskOptionPanel.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class CompileTaskOptionPanel extends JPanel {

  // container to be modified
  protected CompileContainer compileContainer;

  // some text fields
  private JTextField
	stdoutTextField,
	stderrTextField,
	stdinTextField,
	commandLine,
	envVariables;

  // some check boxes
  private JCheckBox
	redirectStdoutCheckBox,
	redirectStderrCheckBox,
	redirectStdinCheckBox,
	versionBox,
	verboseBox;

  /**
   * Constructor builds GUI
   *
   * @param container to be modified
   */
  public CompileTaskOptionPanel(CompileContainer container) {
	 this.compileContainer = container;
	 buildComponents();
  }

  /**
   * Apply gui entries to container
   */
  public void applyValues() {
	compileContainer.setCommandLine(commandLine.getText());
	compileContainer.setEnv(getOptionSetFromString(envVariables.getText()));
	compileContainer.setVersion(versionBox.isSelected());
	compileContainer.setVerbose(verboseBox.isSelected());

	compileContainer.setRedirectStdout(redirectStdoutCheckBox.isSelected());
	compileContainer.setRedirectStdin(redirectStdinCheckBox.isSelected());
	compileContainer.setRedirectStderr(redirectStderrCheckBox.isSelected());

	compileContainer.setStdout(stdoutTextField.getText());
	compileContainer.setStderr(stderrTextField.getText());
	compileContainer.setStdin(stdinTextField.getText());

  }

  /**
   * Fill gui elements with container values
   */
  public void resetValues() {
	commandLine.setText(compileContainer.getCommandLine());

	verboseBox.setSelected(compileContainer.isVerboseOn());
	versionBox.setSelected(compileContainer.isVersionOn());

	redirectStdoutCheckBox.setSelected(compileContainer.getRedirectStdout());
	redirectStdinCheckBox.setSelected(compileContainer.getRedirectStdin());
	redirectStderrCheckBox.setSelected(compileContainer.getRedirectStderr());

	stdoutTextField.setText(compileContainer.getStdout());
	stderrTextField.setText(compileContainer.getStderr());
	stdinTextField.setText(compileContainer.getStdin());
  }

  /**
   * Build gui elements
   */
  private void buildComponents() {
	// edit area
	commandLine = new  JTextField(30);
	commandLine.setToolTipText("Enter command line arguments");
	envVariables = new  JTextField(30);
	envVariables.setToolTipText("Enter environment variables (key=value)");

	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(5, 5, 5, 5);
	c.anchor=GridBagConstraints.WEST;

	JPanel subPanel1 = new JPanel(gridbag);
	subPanel1.setBorder(new TitledBorder("Arguments"));
	addToPanel(subPanel1, new JLabel("Command line: "), commandLine, gridbag, c);
	addToPanel(subPanel1, new JLabel("Environment: "), envVariables, gridbag, c);

	versionBox = new JCheckBox("Show executable version");
	verboseBox = new JCheckBox("Verbose execution");

	JPanel gridPanel = new JPanel(new  GridLayout(1, 2, 10, 5));
	gridPanel.add(versionBox);
	gridPanel.add(verboseBox);

	JPanel subPanel2 = new JPanel();
	subPanel2.setBorder(new TitledBorder("Runtime options"));
	subPanel2.add(gridPanel);

	redirectStdoutCheckBox = new JCheckBox("Redirect stdout to:");
	redirectStderrCheckBox = new JCheckBox("Redirect stderr to:");
	redirectStdinCheckBox = new JCheckBox("Redirect stdin from:");
	stdoutTextField = new  JTextField(25);
	stdoutTextField.setToolTipText("Enter filename for stdout");
	stderrTextField = new  JTextField(25);
	stderrTextField.setToolTipText("Enter filename for stderr");
	stdinTextField = new  JTextField(25);
	stdinTextField.setToolTipText("Enter filename for stdin");

	JPanel subPanel3 = new JPanel(gridbag);
	subPanel3.setBorder(new TitledBorder("stdout/stderr/stdin"));
	addToPanel(subPanel3, redirectStdoutCheckBox, stdoutTextField, gridbag, c);
	addToPanel(subPanel3, redirectStderrCheckBox, stderrTextField, gridbag, c);
	addToPanel(subPanel3, redirectStdinCheckBox, stdinTextField, gridbag, c);

	JPanel mainPanel = new JPanel(new GridLayout(3, 1, 10, 5));
/*	mainPanel.add(subPanel1, BorderLayout.NORTH);
	mainPanel.add(subPanel2, BorderLayout.CENTER);
	mainPanel.add(subPanel3, BorderLayout.SOUTH); */

	mainPanel.add(subPanel1);
	mainPanel.add(subPanel2);
	mainPanel.add(subPanel3);

	JScrollPane scrollPane = new JScrollPane(mainPanel);

	setLayout(new BorderLayout());
	add(scrollPane, BorderLayout.CENTER);


  }

  /**
   *  Parse a string in the following format and return an OptionSet:
   *  1.) Items are separated by white space or tab
   *  2.) Items contain a key/value pair separated by "="
   *
   */
  private OptionSet getOptionSetFromString(String in) {
	OptionSet optionSet = new OptionSet();
	StringTokenizer st = new StringTokenizer(in, " \t");
	int count = st.countTokens();
//	System.out.println("Found "+count+" tokens");
	if(count < 1) {
	  return null;
	}

	while(st.hasMoreElements()) {
	  String token = st.nextToken();
	  StringTokenizer st2 = new StringTokenizer(token,"=");
	  int count2 = st2.countTokens();
//	  System.out.println("Found "+count+" tokens");
/*	  if( count != 2) {
		// something wrong
		continue;
	  }*/
	  String key = st2.nextToken();
	  String value = st2.nextToken();
	  System.out.println("Found Option: "+key+" "+value);
	  optionSet.add(new Option(key, value));
	}

	return optionSet;
  }

 /**
  * Helper method to handle GridBagLayout
  */
  private void addToPanel(JPanel panel,
						  Component l,
						  Component r,
						  GridBagLayout g,
						  GridBagConstraints c) {
	c.gridwidth=GridBagConstraints.RELATIVE;
	g.setConstraints(l, c);
	panel.add(l);

	c.anchor=GridBagConstraints.WEST;
	c.gridwidth=GridBagConstraints.REMAINDER;
	g.setConstraints(r, c);
	panel.add(r);
  }
}
