package com.pallas.unicore.client.plugins.compile;

import java.util.logging.*;
import javax.swing.ImageIcon;

import org.unicore.ajo.*;
import org.unicore.resources.*;
import org.unicore.sets.*;

import com.pallas.unicore.container.*;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.*;

/**
 *  Title: UNICOREpro Description: Copyright: Copyright (c) 2001 Company:
 *
 *@author     Thomas Kentemich
 *@created    August 16, 2001
 *@version    $ID:$
 */

public class FortranContainer extends CompileContainer {

    final static long serialVersionUID = 4983481768961721914L;

    private boolean checkArrayBounds;
    private boolean checkArguments;
    private PreprocessLevel preprocessLevel;
    private IntegerLength integerLength;
    private RealLength realLength;
    private DoubleLength doubleLength;
    private FortranSourceForm sourceForm;
    private OptionSet defines;
    private OptionSet undefines;


    /**
     *  Constructors
     *
     *@param  parentContainer  Description of Parameter
     */
    public FortranContainer(GroupContainer parentContainer) {
        super(parentContainer);
    }


    /**
     *  Sets the CheckArrayBounds attribute of the FortranContainer object
     *
     *@param  check  The new CheckArrayBounds value
     */
    public void setCheckArrayBounds(boolean check) {
        this.checkArrayBounds = check;
    }


    /**
     *  Sets the CheckArguments attribute of the FortranContainer object
     *
     *@param  check  The new CheckArguments value
     */
    public void setCheckArguments(boolean check) {
        this.checkArguments = check;
    }


    /**
     *  Sets the PreprocessLevel attribute of the FortranContainer object
     *
     *@param  level  The new PreprocessLevel value
     */
    public void setPreprocessLevel(PreprocessLevel level) {
        this.preprocessLevel = level;
    }


    /**
     *  Sets the IntegerLength attribute of the FortranContainer object
     *
     *@param  len  The new IntegerLength value
     */
    public void setIntegerLength(IntegerLength len) {
        this.integerLength = len;
    }


    /**
     *  Sets the RealLength attribute of the FortranContainer object
     *
     *@param  len  The new RealLength value
     */
    public void setRealLength(RealLength len) {
        this.realLength = len;
    }


    /**
     *  Sets the DoubleLength attribute of the FortranContainer object
     *
     *@param  len  The new DoubleLength value
     */
    public void setDoubleLength(DoubleLength len) {
        this.doubleLength = len;
    }


    /**
     *  Sets the SourceForm attribute of the FortranContainer object
     *
     *@param  form  The new SourceForm value
     */
    public void setSourceForm(FortranSourceForm form) {
        this.sourceForm = form;
    }


    /**
     *  Sets the Defines attribute of the FortranContainer object
     *
     *@param  defines  The new Defines value
     */
    public void setDefines(OptionSet defines) {
        this.defines = defines;
    }


    /**
     *  Sets the Undefines attribute of the FortranContainer object
     *
     *@param  undefines  The new Undefines value
     */
    public void setUndefines(OptionSet undefines) {
        this.undefines = undefines;
    }


    /**
     *  Gets the CheckArrayBounds attribute of the FortranContainer object
     *
     *@return    The CheckArrayBounds value
     */
    public boolean getCheckArrayBounds() {
        return checkArrayBounds;
    }


    /**
     *  Gets the CheckArguments attribute of the FortranContainer object
     *
     *@return    The CheckArguments value
     */
    public boolean getCheckArguments() {
        return checkArguments;
    }


    /**
     *  Gets the PreprocessLevel attribute of the FortranContainer object
     *
     *@return    The PreprocessLevel value
     */
    public PreprocessLevel getPreprocessLevel() {
        return preprocessLevel;
    }


    /**
     *  Gets the IntegerLength attribute of the FortranContainer object
     *
     *@return    The IntegerLength value
     */
    public IntegerLength getIntegerLength() {
        return integerLength;
    }


    /**
     *  Gets the RealLength attribute of the FortranContainer object
     *
     *@return    The RealLength value
     */
    public RealLength getRealLength() {
        return realLength;
    }


    /**
     *  Gets the DoubleLength attribute of the FortranContainer object
     *
     *@return    The DoubleLength value
     */
    public DoubleLength getDoubleLength() {
        return doubleLength;
    }


    /**
     *  Gets the SourceForm attribute of the FortranContainer object
     *
     *@return    The SourceForm value
     */
    public FortranSourceForm getSourceForm() {
        return sourceForm;
    }


    /**
     *  Gets the Defines attribute of the FortranContainer object
     *
     *@return    The Defines value
     */
    public OptionSet getDefines() {
        return defines;
    }


    /**
     *  Gets the Undefines attribute of the FortranContainer object
     *
     *@return    The Undefines value
     */
    public OptionSet getUndefines() {
        return undefines;
    }


    /**
     *  Description of the Method
     */
    protected void buildExecuteGroup() {

        if(getSources() == null) {
            return;
        }

        executeGroup = new ActionGroup(getName() + "_EXECUTION");

        // get task resources from job container
        NamedResourceSet taskResourceSet = getResourceSet();
        ResourceSet uspaceResourceSet = new ResourceSet(taskResourceSet.getUspace());

        MakePortfolio makePortfolio = new MakePortfolio(getName() + "_Sources");
        makePortfolio.setFiles(getSources());

        String[] includes = getIncludes();
        MakePortfolio includePortfolio = null;
        PortfolioList pList = null;
        if(includes != null && includes.length > 0) {
            includePortfolio = new MakePortfolio(getName() + "_Includes");
            includePortfolio.setFiles(includes);
            pList = new PortfolioList(includePortfolio.getPortfolio());
        }

        FortranTask compileTask =
                new FortranTask(getName(),
        // name
                null,
        // extra information
                taskResourceSet.getResourceSet(),
        // resource limits
                getEnv(),
        // environment variables
                getCommandLine(),
        // command line string
                null,
        // stdin
                getRedirectStdout(),
        // redirect_stdout
                getRedirectStderr(),
        // redirect stderr
                isVerboseOn(),
        // verbose
                isVersionOn(),
        // version
                new PortfolioSet(makePortfolio.getPortfolio()),
        // sources to compile
                getListingLevel(),
        // listing
                getOptimisationLevel(),
        // optimize
                pList,
        // include files
                getCompileForDebug(),
        // debug level
                getCompileForProfiling(),
        // profiling level
                getCheckArrayBounds(),
        // array bounds check
                getCheckArguments(),
        //argument checking
                getIntegerLength(),
        // int length
                getRealLength(),
        // real length
                getDoubleLength(),
        //double length
                getSourceForm(),
        // source code format
                getPreprocessLevel(),
        // preprocessor
                getDefines(),
        // preprocessor defines
                getUndefines());
        // proprocessor undef


        this.setObjectsPortfolio(compileTask.getObjects());
        executeGroup.add(makePortfolio);
        executeGroup.add(compileTask);
        try {
            if(includes != null && includes.length > 0) {
                executeGroup.add(includePortfolio);
                executeGroup.addDependency(new Dependency(includePortfolio, compileTask));
            }

            executeGroup.addDependency(new Dependency(makePortfolio, compileTask));

            if(getRedirectStdout()) {
                compileTask.setStdoutFileName(this.getStdout());
                // delete portfolio in case of loops
                DeletePortfolio delStdoutPortfolio =
                        new DeletePortfolio(getName() + "_DELETE_STDOUT_"
                         + ResourceManager.getNextObjectIdentifier());
                delStdoutPortfolio.setPortfolio(compileTask.getStdout().getId());

                executeGroup.add(delStdoutPortfolio);
                executeGroup.addDependency(new Dependency(compileTask, delStdoutPortfolio));
            }
            if(getRedirectStderr()) {
                compileTask.setStderrFileName(this.getStderr());
                // delete portfolio in case of loops
                DeletePortfolio delStderrPortfolio =
                        new DeletePortfolio(getName() + "_DELETE_STDERR_"
                         + ResourceManager.getNextObjectIdentifier());
                delStderrPortfolio.setPortfolio(compileTask.getStderr().getId());

                executeGroup.add(delStderrPortfolio);
                executeGroup.addDependency(new Dependency(compileTask, delStderrPortfolio));
            }
            MakePortfolio stdinPortfolio = null;

            if(getRedirectStdin()) {
                String[] stdinfiles = {this.getStdin()};
                stdinPortfolio =
                        new MakePortfolio(getName() + "_MP_STDIN_" + ResourceManager.getNextObjectIdentifier(),
                // name String name,
                        null,
                // extra information
                        uspaceResourceSet,
                // storage resource
                        stdinfiles);
                // execute
                compileTask.setStdin(stdinPortfolio.getPortfolio());
                executeGroup.add(stdinPortfolio);
                executeGroup.addDependency(new Dependency(stdinPortfolio, compileTask));
            }

            if(this.getObjects() != null) {
                MakePortfolio makeObjectPortfolio = new MakePortfolio(getName() + "_Objects");
                PortfolioEnumeration pEnum = getObjects().elements();
                while(pEnum.hasMoreElements()) {
                    makeObjectPortfolio.addFile(pEnum.nextElement().getName());
                }
                // perform link step
                LinkTask linkTask = new LinkTask(
                        getName(),
                // name
                        null,
                // extra information
                        taskResourceSet.getResourceSet(),
                // resource limits
                        getEnv(),
                // environment variables
                        getCommandLine(),
                // command line string
                        null,
                // stdin
                        getRedirectStdout(),
                // redirect_stdout
                        getRedirectStderr(),
                // redirect stderr
                        isVerboseOn(),
                // verbose
                        isVersionOn(),
                // version
                //getObjects(),
                        new PortfolioList(makeObjectPortfolio.getPortfolio()),
                // objects
                        getLibraries(),
                // libs
                        getLinkMapLevel(),
                // link map
                        getLinkForDebug(),
                // debug level
                        getLinkForProfiling()
                // profiling level
                        );

                if(getRedirectStdout()) {
                    linkTask.setStdoutFileName(this.getStdout());
                }
                if(getRedirectStderr()) {
                    linkTask.setStderrFileName(this.getStderr());
                }
                if(getRedirectStdin()) {
                    linkTask.setStdin(stdinPortfolio.getPortfolio());
                }

                executeGroup.add(makeObjectPortfolio);
                executeGroup.add(linkTask);
                executeGroup.addDependency(new Dependency(compileTask, makeObjectPortfolio));
                executeGroup.addDependency(new Dependency(makeObjectPortfolio, linkTask));
            }
        }
        catch(ActionGroup.InvalidDependencyException ex) {
           logger.log(Level.SEVERE, "Could not add dependency.", ex);
        }

    }

}
