package com.pallas.unicore.client.plugins.compile;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;


import org.unicore.sets.*;
import org.unicore.ajo.*;
import com.pallas.unicore.container.*;

/**
 * Panel to define options for executable commands
 *
 * @author Thomas Kentemich
 * @version $Id: LinkerOptionPanel.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class LinkerOptionPanel extends JPanel {

  // container to be modified
  protected CompileContainer compileContainer;


  //some ComboBoxes
  private JComboBox
    linkMapLevel;

  // some check boxes
  private JCheckBox
	linkForDebug,
	linkForProfiling;

  /**
   * Constructor builds GUI
   *
   * @param container to be modified
   */
  public LinkerOptionPanel(CompileContainer container) {
	 this.compileContainer = container;
	 buildComponents();
  }

  /**
   * Apply gui entries to container
   */
  public void applyValues() {
		compileContainer.setLinkForDebug(linkForDebug.isSelected());
		compileContainer.setLinkForProfile(linkForProfiling.isSelected());
		compileContainer.setLinkMapLevel((LinkMapLevel) linkMapLevel.getSelectedItem());
  }

  /**
   * Fill gui elements with container values
   */
  public void resetValues() {
		linkForDebug.setSelected(compileContainer.getLinkForDebug());
		linkForProfiling.setSelected(compileContainer.getLinkForProfiling());
		if (compileContainer.getLinkMapLevel() != null) {
			linkMapLevel.setSelectedItem(compileContainer.getLinkMapLevel());
		}
  }

  /**
   * Build gui elements
   */
  private void buildComponents() {
	// edit area

	Vector items = new Vector();
	Enumeration enum = LinkMapLevel.elements();
	while(enum.hasMoreElements()) {
		items.add(enum.nextElement());
	}
	linkMapLevel = new JComboBox(items);
	linkMapLevel.setSelectedItem(LinkMapLevel.NONE);


	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(5, 5, 5, 5);
	c.anchor=GridBagConstraints.WEST;


	JPanel gridPanel = new JPanel(new  GridLayout(2, 2, 10, 5));
	gridPanel.add(new JLabel("Linkmap level:"));
	gridPanel.add(linkMapLevel);

	linkForDebug = new JCheckBox("Generate debug output");
	linkForProfiling = new JCheckBox("Generate profiling output");
	gridPanel.add(linkForDebug);
	gridPanel.add(linkForProfiling);

	JPanel subPanel1 = new JPanel();
	subPanel1.setBorder(new TitledBorder("Runtime Options"));
	subPanel1.add(gridPanel);

	JPanel mainPanel = new JPanel(new BorderLayout());
	mainPanel.add(subPanel1, BorderLayout.CENTER);


	JScrollPane scrollPane = new JScrollPane(mainPanel);

	setLayout(new BorderLayout());
	add(scrollPane, BorderLayout.CENTER);


  }



 /**
  * Helper method to handle GridBagLayout
  */
  private void addToPanel(JPanel panel,
						  Component l,
						  Component r,
						  GridBagLayout g,
						  GridBagConstraints c) {
	c.gridwidth=GridBagConstraints.RELATIVE;
	g.setConstraints(l, c);
	panel.add(l);

	c.anchor=GridBagConstraints.WEST;
	c.gridwidth=GridBagConstraints.REMAINDER;
	g.setConstraints(r, c);
	panel.add(r);
  }
}
