package com.pallas.unicore.client.plugins.compile;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.unicore.sets.*;
import org.unicore.ajo.*;
import com.pallas.unicore.container.*;

/**
 *  Panel to define options for executable commands
 *
 *@author     Thomas Kentemich
 *@created    August 22, 2001
 *@version    $Id: SourcesPanel.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class SourcesPanel extends JPanel {

	// container to be modified
	/**
	 *  Description of the Field
	 */
	protected CompileContainer compileContainer;

	private JButton
			sourceAddButton,
			sourceRemoveButton,
			sourceTransferButton,
			includeAddButton,
			includeRemoveButton;

	private JTable sourceTable;
	private JTable includeTable;
	private DefaultTableModel sourceTableModel;
	private DefaultTableModel includeTableModel;


	/**
	 *  Constructor builds GUI
	 *
	 *@param  container  to be modified
	 */
	public SourcesPanel(CompileContainer container) {
		this.compileContainer = container;
		buildComponents();
	}


	/**
	 *  Apply gui entries to container
	 */
	public void applyValues() {
		applySourceFiles();
	}


	/**
	 *  Fill gui elements with container values
	 */
	public void resetValues() {
		String [] files = compileContainer.getSources();
		if (files != null) {
			for(int i=0; i< files.length; i++) {
				insertNewEntry(sourceTable, files[i]);
			}
		}

		files = compileContainer.getIncludes();
		if (files != null) {
			for(int i=0; i< files.length; i++) {
				insertNewEntry(includeTable, files[i]);
			}
		}

		includeRemoveButton.setEnabled(includeTable.getRowCount() > 0);
		sourceRemoveButton.setEnabled(sourceTable.getRowCount() > 0);
		sourceTransferButton.setEnabled(sourceTable.getRowCount() > 0);
	}


	/**
	 *  Gets the CompleteRows attribute of the SourcesPanel object
	 *
	 *@param  table  Description of Parameter
	 *@return        The CompleteRows value
	 */
	private Vector getCompleteRows(JTable table) {
		// loop over rows
		int nrOfRows = table.getRowCount();
		DefaultTableModel tableModel = (DefaultTableModel) table.getModel();
		Vector complete = new Vector();
		for (int i = 0; i < nrOfRows; i++) {
			String sourceName = (String) tableModel.getValueAt(i, 0);
			if (sourceName != null) {
				complete.add(tableModel.getValueAt(i, 0));
			}
		}
		return complete;
	}


	/**
	 *  Build gui elements
	 */
	private void buildComponents() {
		ButtonListener buttonListener = new ButtonListener();
		// edit area

		String[] columnNames;
		columnNames = new String[1];
		columnNames[0] = "Source file";
		sourceTableModel = new DefaultTableModel(columnNames, 0);
		sourceTable =
			new JTable(sourceTableModel) {
				/**
				 *  Gets the CellEditable attribute of the SourcesPanel object
				 *
				 *@param  row  Description of Parameter
				 *@param  col  Description of Parameter
				 *@return      The CellEditable value
				 */
				public boolean isCellEditable(int row, int col) {
					return true;
				}
			};

		sourceTable.setRowHeight(20);

		JScrollPane sourceScrollPane = new JScrollPane(sourceTable);

		JPanel sourceButtonPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		sourceAddButton = new JButton("Add");
		sourceRemoveButton = new JButton("Remove");
		sourceTransferButton = new JButton("Pass");
		sourceAddButton.setToolTipText("Add a new source file");
		sourceRemoveButton.setToolTipText("Remove a source file from the list");
		sourceTransferButton.setToolTipText("Transfer file names to the linker");
		sourceRemoveButton.setEnabled(false);
		sourceTransferButton.setEnabled(false);
		sourceButtonPanel.add(sourceAddButton);
		sourceButtonPanel.add(sourceRemoveButton);
		sourceButtonPanel.add(sourceTransferButton);
		sourceAddButton.addActionListener(buttonListener);
		sourceRemoveButton.addActionListener(buttonListener);
		sourceTransferButton.addActionListener(buttonListener);

		JPanel subPanel1 = new JPanel(new BorderLayout());
		subPanel1.setBorder(new TitledBorder(""));
		subPanel1.add(sourceScrollPane, BorderLayout.CENTER);
		subPanel1.add(sourceButtonPanel, BorderLayout.SOUTH);

		columnNames = new String[1];
		columnNames[0] = "Include path";
		includeTableModel = new DefaultTableModel(columnNames, 0);
		includeTable =
			new JTable(includeTableModel) {
				/**
				 *  Gets the CellEditable attribute of the SourcesPanel object
				 *
				 *@param  row  Description of Parameter
				 *@param  col  Description of Parameter
				 *@return      The CellEditable value
				 */
				public boolean isCellEditable(int row, int col) {
					return true;
				}
			};

		includeTable.setRowHeight(20);

		JScrollPane includeScrollPane = new JScrollPane(includeTable);

		JPanel includeButtonPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		includeAddButton = new JButton("Add");
		includeRemoveButton = new JButton("Remove");
		includeAddButton.setToolTipText("Add a new include file");
		includeRemoveButton.setToolTipText("Remove an include file from the list");
		includeRemoveButton.setEnabled(false);
		includeButtonPanel.add(includeAddButton);
		includeButtonPanel.add(includeRemoveButton);
		includeAddButton.addActionListener(buttonListener);
		includeRemoveButton.addActionListener(buttonListener);

		JPanel subPanel2 = new JPanel(new BorderLayout());
		subPanel2.setBorder(new TitledBorder(""));
		subPanel2.add(includeScrollPane, BorderLayout.CENTER);
		subPanel2.add(includeButtonPanel, BorderLayout.SOUTH);

		JPanel mainPanel = new JPanel(new GridLayout(1, 2, 10, 10));
		mainPanel.add(subPanel1);
		mainPanel.add(subPanel2);

		setLayout(new BorderLayout());
		add(mainPanel, BorderLayout.CENTER);

	}


	/**
	 *  Description of the Method
	 *
	 *@param  table  Description of Parameter
	 *@param  name   Description of Parameter
	 */
	private void insertNewEntry(JTable table, String name) {
		// loop over rows
		int nrOfRows = table.getRowCount();
		DefaultTableModel tableModel = (DefaultTableModel) table.getModel();
		Vector complete = new Vector();
		for (int i = 0; i < nrOfRows; i++) {
			String sourceName = (String) tableModel.getValueAt(i, 0);
			if (name.equals(sourceName)) {
				// already contained
				return;
			}
		}
		addRow(table, name);
	}


	/**
	 *  Helper method to handle GridBagLayout
	 *
	 *@param  panel  The feature to be added to the ToPanel attribute
	 *@param  l      The feature to be added to the ToPanel attribute
	 *@param  r      The feature to be added to the ToPanel attribute
	 *@param  g      The feature to be added to the ToPanel attribute
	 *@param  c      The feature to be added to the ToPanel attribute
	 */
	private void addToPanel(JPanel panel,
			Component l,
			Component r,
			GridBagLayout g,
			GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);

		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}


	/**
	 *  add a row to the table
	 *
	 *@param  theTable  The feature to be added to the Row attribute
	 *@param  data      The feature to be added to the Row attribute
	 */
	private void addRow(JTable theTable, Object data) {
		Vector rowData = new Vector();
		rowData.add(data);

		((DefaultTableModel) theTable.getModel()).addRow(rowData);
		int selectedRow = theTable.getRowCount() - 1;
		theTable.setRowSelectionInterval(selectedRow, selectedRow);
	}


	/**
	 *  Description of the Method
	 */
	private void applySourceFiles() {
		// stop editing
		if (sourceTable.getCellEditor() != null) {
			sourceTable.getCellEditor().stopCellEditing();
		}

		if (includeTable.getCellEditor() != null) {
			includeTable.getCellEditor().stopCellEditing();
		}

		// get vector with complete sources
		Vector complete = getCompleteRows(sourceTable);
		String [] files = new String [complete.size()];
		for (int i = 0; i < complete.size(); i++) {
			files [i] = (String) complete.elementAt(i);
		}

	        compileContainer.setSources(files);

		complete = getCompleteRows(includeTable);
		files = new String [complete.size()];
		for (int i = 0; i < complete.size(); i++) {
			files [i] = (String) complete.elementAt(i);
		}

		compileContainer.setIncludes(files);

	}


	/**
	 *  remove a row from the table
	 *
	 *@param  theTable  Description of Parameter
	 */
	private void removeRow(JTable theTable) {
		while (theTable.getSelectedRow() > -1) {
			((DefaultTableModel) theTable.getModel()).removeRow(theTable.getSelectedRow());
		}
		int selectedRow = theTable.getRowCount() - 1;
		if (selectedRow >= 0) {
			theTable.setRowSelectionInterval(selectedRow, selectedRow);
		}
	}


	/**
	 *  Description of the Class
	 *
	 *@author     thomas
	 *@created    August 22, 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(java.awt.event.ActionEvent e) {
			Object src = e.getSource();
			if (src == sourceAddButton) {
				addRow(sourceTable, new String());
			}
			else if (src == sourceRemoveButton) {
				// remove an entry from the list
				removeRow(sourceTable);
				applySourceFiles();
				firePropertyChange("sourceTableRemove", null, null);
			}
			else if (src == sourceTransferButton) {
				// pass filename to the linker
				applySourceFiles();
				firePropertyChange("sourceTableTransfer", null, null);
			}
			else if (src == includeAddButton) {
				addRow(includeTable, new String());
			}
			else if (src == includeRemoveButton) {
				removeRow(includeTable);
			}
			includeRemoveButton.setEnabled(includeTable.getRowCount() > 0);
			sourceRemoveButton.setEnabled(sourceTable.getRowCount() > 0);
			sourceTransferButton.setEnabled(sourceTable.getRowCount() > 0);
		}
	}
}
