package com.pallas.unicore.client.plugins.minimal;

import javax.swing.*;

import org.unicore.ajo.*;
import org.unicore.resources.*;
import org.unicore.sets.*;

import com.pallas.unicore.client.panels.IPanelProvider;
import com.pallas.unicore.container.*;
import com.pallas.unicore.container.errorspec.*;
import com.pallas.unicore.extensions.*;
import com.pallas.unicore.resourcemanager.*;

/**
 *  A minimal example container
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: MinimalContainer.java,v 1.2 2003/06/13 15:55:23 rrate Exp $
 */
public class MinimalContainer extends UserContainer implements IPanelProvider {
    private MinimalOutcomePanel outcomePanel = new MinimalOutcomePanel();


    /**
     *  Constructor
     *
     *@param  parentContainer  parent job group
     */
    public MinimalContainer(GroupContainer parentContainer) {
        super(parentContainer);
    }


    /**
     *  Gets the panel attribute of the MinimalContainer object
     *
     *@param  i  Description of the Parameter
     *@return    The panel value
     */
    public JPanel getPanel(int i) {
        return outcomePanel;
    }


    /**
     *  Gets the panelTitle attribute of the MinimalContainer object
     *
     *@param  i  Description of the Parameter
     *@return    The panelTitle value
     */
    public String getPanelTitle(int i) {
        return "Minimal Outcome";
    }


    /**
     *  Gets the nrOfPanels attribute of the MinimalContainer object
     *
     *@return    The nrOfPanels value
     */
    public int getNrOfPanels() {
        return 1;
    }


    /**
     *  Description of the Method
     */
    public void finalizePanel() { }


    /**
     *  Add container specific errors to error set of super classes and select
     *  appropriate icon for JPA display.
     *
     *@return    ErrorSet set containing all errors including those from super
     *      classes
     */
    public ErrorSet checkContents() {
        ErrorSet err = super.checkContents();

        if(id == null || getName().equals("")) {
            err.add(new UError(this.getIdentifier(), "No task name specified"));
        }

        // checkContents
        checkSoftwareResource(err);

        setErrors(err);
        return err;
    }



    /**
     *  Build the AJO ActionGroup that will be plugged into parent job group.
     */
    protected void buildExecuteGroup() {
        System.err.println("BUILDING EXECUTE GROUP!");

        executeGroup = new ActionGroup(getName() + "_EXECUTION");

        UserTask executeTask =
                new UserTask(getName(), null, getResourceSet().getResourceSet(), getEnv(),
                getCommandLine(), null, getRedirectStdout(), getRedirectStderr(),
                isVerboseOn(), isVersionOn(), null,
                getMeasureTime(), getDebug(), getProfile());
        executeGroup.add(executeTask);

    }


    // look for software resource
    /**
     *  Description of the Method
     *
     *@param  err  Description of the Parameter
     */
    private void checkSoftwareResource(ErrorSet err) {

        // search for SoftwareResource in site resources
        boolean softwareAvailable = false;
        ResourceSet siteResources = ResourceManager.getResourceSet(getVsite()).getResourceSet();
        ResourceEnumeration siteEnum = siteResources.elements();
        while(siteEnum.hasMoreElements()) {
            Resource siteResource = (Resource)siteEnum.nextElement();
            if(siteResource instanceof SoftwareResource) {
                SoftwareResource siteSoftware = (SoftwareResource)siteResource;
                if(siteSoftware.getName().toUpperCase().startsWith("GAUSSIAN")) {
                    softwareAvailable = true;
                    // set software resource
                    setPreinstalledSoftware(siteSoftware);
                }
            }
        }

        if(!softwareAvailable) {
            String vsiteName = "Unknown";
            if(getVsite() != null) {
                vsiteName = getVsite().getName();
            }
            err.add(new UError(getIdentifier(),
                    "Minimal plugin is not supported at site " + vsiteName));
        }

    }

}
