package com.pallas.unicore.client.plugins.minimal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

import org.unicore.resources.*;
import org.unicore.sets.*;

import com.pallas.unicore.client.panels.*;

/**
 * Just an empty implementation of a JPAPanel
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: MinimalJPAPanel.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class MinimalJPAPanel extends JPAPanel {
    // panel to define file imports and exports
	private FileImportPanel importPanel;
	private FileExportPanel exportPanel;

	// textfield for task name
	private JTextField nameTextField;

	// container that this panel represents
	private MinimalContainer container;


	/**
	 *  Constructor
	 *
	 *@param  parentFrame  reference to main JFrame for modal sub dialogs
	 *@param  container    container that this panel represents
	 */
	public MinimalJPAPanel(JFrame parentFrame, MinimalContainer container) {
		super(parentFrame, container);
		this.container = container;

		// build gui
		buildComponents();
	}


	/**
	 *  Apply values from GUI to container
	 */
	public void applyValues() {
	    System.err.println("APPLYING VALUES!");

		// replace spaces in name with _
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));

		container.setName(nameTextField.getText());

        // applyValues
        importPanel.applyValues();
        exportPanel.applyValues();


	}


	/**
	 *  Fill values from container into gui
	 */
	public void resetValues() {
	    System.err.println("RESETTING VALUES!");
		if (container.getIdentifier()!=null && container.getName() != null) {
    		nameTextField.setText(container.getName());
        }
        else {
        	nameTextField.setText("");
        }

        // resetValues
        importPanel.resetValues();
        exportPanel.resetValues();

	}


	/**
	 *  This method is executed whenever the panel becomes visible. Here can be checked for instance if
	 *  resources have changed
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or reset
	 */
	public void updateValues(boolean vsiteChanged) {
	    System.err.println("UPDATING VALUES!");
        // updateValues
        if (vsiteChanged) {
            importPanel.updateValues();
            exportPanel.updateValues();
        }

	}


	/**
	 *  Build GUI
	 */
	private void buildComponents() {
		nameTextField = new JTextField(20);
		nameTextField.setToolTipText("Enter identifier for this task");

		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel("Name: "));
		namePanel.add(nameTextField);

		add(namePanel, BorderLayout.NORTH);

        // buildComponents
        importPanel = new FileImportPanel(container);
        importPanel.setBorder(new TitledBorder("File Imports"));

        exportPanel = new FileExportPanel(container);
        exportPanel.setBorder(new TitledBorder("File Exports"));

        JPanel importExportPanel = new JPanel(new GridLayout(2, 1, 10, 10));
        importExportPanel.add(importPanel);
        importExportPanel.add(exportPanel);

        add(importExportPanel, BorderLayout.CENTER);

        importPanel.revalidate();
        exportPanel.revalidate();


		revalidate();
	}
}

/* EXAMPLE2







    */
