package com.pallas.unicore.client.plugins.minimal;

import javax.swing.*;

import com.pallas.unicore.client.panels.*;
import com.pallas.unicore.client.util.*;
import com.pallas.unicore.container.*;

/**
 *  A minimal plugin class
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: MinimalPlugin.java,v 1.1.1.1 2003/04/08 09:03:17 rrate Exp $
 */
public class MinimalPlugin extends TaskPlugable {

    /**
     *  Get a short info message for the plugin
     *
     *@return    String with info message
     */
    public String getPluginInfo() {
        return "Tutorial Example 1: A minimal plugin\n" +
                "(C)2001 Pallas GmbH";
    }


    /**
     *  Gets the name attribute of the MinimalPlugin object
     *
     *@return    The name value
     */
    public String getName() {
        return "Minimal";
    }


    /**
     *  Return a new container for this plugin
     *
     *@param  parentContainer  job that this task belongs to
     *@return                  new initialized instance of plugin container
     */
    public ActionContainer getContainerInstance(GroupContainer parentContainer) {
        // build new plugin container
        MinimalContainer container = new MinimalContainer(parentContainer);
        container.setName("Minimal_Task");
        return container;
    }


    /**
     *  Build a new panel that manages the parameters of a plugin container
     *
     *@param  container  Description of the Parameter
     *@return            JPAPanel that represents container parameters in GUI
     */
    public JPAPanel getPanelInstance(ActionContainer container) {
        // return new instance of plugin JPAPanel
        return new MinimalJPAPanel(getClient(), (MinimalContainer)container);
    }


    /**
     *  Gets the iconPath attribute of the MinimalPlugin object
     *
     *@return    The iconPath value
     */
    public String getIconPath() {
        return "com/pallas/unicore/client/plugins/minimal/minimal.gif";
    }


    /**
     *  Called to inititialize the plugin
     */
    public void startPlugin() { }


    /**
     *  Called when terminating
     */
    public void stopPlugin() {
        // do nothing
    }

}
