package com.pallas.unicore.client.plugins.povray;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.util.logging.Level;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.controls.DoubleTextField;
import com.pallas.unicore.client.controls.GenericFileFilter;
import com.pallas.unicore.client.controls.IntTextField;
import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.client.editor.RemoteTextEditor;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.panels.FileExportPanel;
import com.pallas.unicore.client.panels.FileImportPanel;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.panels.OptionPanel;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: PovrayJPAPanel.java,v 1.8 2003/06/30 06:55:24 rrate Exp $
 */
public class PovrayJPAPanel extends JPAPanel {

	private class ChooseDirectoryAction extends AbstractAction {
		/**
		 *  Constructor for the OpenAction object
		 */
		public ChooseDirectoryAction() {
			super("", ResourceManager.getIcon(ResourceManager.OPEN));
		}

		/**
		 *  Description of the Method
		 *
		 *@param  ev  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent ev) {
			SelectorDialog dialog =
				ResourceManager.getSelectorDialog(
					new File(outputDirField.getText()),
					null);
			dialog.setFileSelectionMode(SelectorDialog.DIRECTORIES_ONLY);
			int result = dialog.showDialog();
			if (result == SelectorDialog.APPROVE_OPTION) {
				outputDirField.setText(
					dialog.getSelectedFile().getAbsolutePath());
			}
		}
	}
	private DoubleTextField aaField;

	// the container this panel represents
	private PovrayContainer container;

	// panel to define file exports
	private FileExportPanel exportPanel;

	// panel to define file imports
	private FileImportPanel importPanel;

	// text field for task name
	private JTextField nameTextField;

	// panel for setting options
	private OptionPanel optionsPanel;

	private JTextField outputDirField;

	// defaults
	private PovrayDefaults povrayDefaults;

	// panel to edit the input file
	private RemoteTextEditor textEditor = new RemoteTextEditor();

	private IntTextField widthField, heightField;

	/**
	 *  Constructor
	 *
	 *@param  parentFrame  reference to main JFrame for modal sub dialogs
	 *@param  container    container that this panel represents
	 */
	public PovrayJPAPanel(JFrame parentFrame, PovrayContainer container) {
		super(parentFrame, container);
		this.container = container;

		// build gui
		buildComponents();
	}

	/**
	 *  Apply values from GUI to container
	 */
	public void applyValues() {
		// replace spaces in name with _
		nameTextField.setText(nameTextField.getText().trim().replace(' ', '_'));

		updateModifiedTime();

		// container id will change here.....
		container.setName(nameTextField.getText());
		container.setInputFile(textEditor.getFile());
		container.setInputString(textEditor.getText());
		container.setOutputDirectory(outputDirField.getText());
		container.setWidth(widthField.getValue());
		container.setHeight(heightField.getValue());
		container.setAntiAliasing(aaField.getValue());

		importPanel.applyValues();
		exportPanel.applyValues();
		optionsPanel.applyValues();
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	private JPanel buildAboutPanel() {
		JPanel panel = new JPanel(new BorderLayout());
		ClassLoader loader = getClass().getClassLoader();
		String aboutFilename =
			"com/pallas/unicore/client/plugins/povray/about.rtf";

		try {
			URL url = loader.getResource(aboutFilename);
			JEditorPane rtfPane = new JEditorPane();
			rtfPane.setContentType("text/rtf");
			//rtfPane.setPage(url);
			rtfPane.setText(readURL(url));
			rtfPane.setEditable(false);
			panel.add(new JScrollPane(rtfPane), BorderLayout.NORTH);
		} catch (IOException e) {
			logger.log(Level.WARNING, "Could not read " + aboutFilename, e);
		}

		String legalFilename =
			"com/pallas/unicore/client/plugins/povray/povlegal.rtf";
		try {
			URL url = loader.getResource(legalFilename);
			JEditorPane rtfPane = new JEditorPane();
			rtfPane.setContentType("text/rtf");
			// rtfPane.setPage(url);
			rtfPane.setText(readURL(url));
			rtfPane.setEditable(false);
			panel.add(new JScrollPane(rtfPane), BorderLayout.CENTER);
		} catch (IOException e) {
			logger.log(Level.WARNING, "Could not read " + legalFilename, e);
		}

		return panel;
	}

	/**
	 *  Build GUI components
	 */
	private void buildComponents() {
		nameTextField = new JTextField();
		nameTextField.setToolTipText("Enter Task Name");

		JPanel topPanel = new JPanel(new GridBagLayout());

		topPanel.add(
			new JLabel("Name:"),
			LayoutTools.makegbc(0, 0, 1, 0, false));
		GridBagConstraints c = LayoutTools.makegbc(1, 0, 1, 0, true);
		c.gridwidth = GridBagConstraints.REMAINDER;
		topPanel.add(nameTextField, c);

		widthField = new IntTextField(0, 9999);
		topPanel.add(
			new JLabel("Width:"),
			LayoutTools.makegbc(1, 1, 1, 0, false));
		topPanel.add(widthField, LayoutTools.makegbc(2, 1, 1, 0, true));

		heightField = new IntTextField(0, 9999);
		topPanel.add(
			new JLabel("Height:"),
			LayoutTools.makegbc(3, 1, 1, 0, false));
		topPanel.add(heightField, LayoutTools.makegbc(4, 1, 1, 0, true));

		aaField = new DoubleTextField(0.0, 1.0);
		topPanel.add(
			new JLabel("Anti Aliasing:"),
			LayoutTools.makegbc(5, 1, 1, 0, false));
		topPanel.add(aaField, LayoutTools.makegbc(6, 1, 1, 0, true));

		topPanel.add(
			new JLabel("Output Directory:"),
			LayoutTools.makegbc(0, 2, 1, 0, false));
		outputDirField = new JTextField();
		topPanel.add(outputDirField, LayoutTools.makegbc(1, 2, 5, 0, true));
		topPanel.add(
			new JButton(new ChooseDirectoryAction()),
			LayoutTools.makegbc(6, 2, 1, 0, false));

		// build import export panels
		UserDefaults userDefaults = ResourceManager.getUserDefaults();
		importPanel =
			new FileImportPanel(container, userDefaults.getImportDirectory());
		exportPanel =
			new FileExportPanel(container, userDefaults.getExportDirectory());
		optionsPanel = new OptionPanel(container);

		GenericFileFilter filter =
			new GenericFileFilter(".pov", "POV-Ray Input Files");
		textEditor.addFileFilter(filter);
		textEditor.setFileFilter(filter);

		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add(textEditor, "POV-Ray Input File");
		tabbedPane.add(optionsPanel, "Options");
		tabbedPane.add(importPanel, "Imports");
		tabbedPane.add(exportPanel, "Exports");
		tabbedPane.add(buildAboutPanel(), "About POV-Ray");

		add(topPanel, BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);

		setBorder(new TitledBorder("POV-Ray"));
	}

	/**
	  *  Reads the content from an URL and returns it as aString
	  *
	  *@param  url              URL to read from
	  *@return                  Content as String
	  *@exception  IOException  Oops
	  */
	private String readURL(URL url) throws IOException {
		URLConnection uc = url.openConnection();
		InputStream content = (InputStream) uc.getInputStream();
		BufferedReader in = new BufferedReader(new InputStreamReader(content));
		String line;
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		while ((line = in.readLine()) != null) {
			pw.println(line);
		}
		return sw.toString();
	}

	/**
	 *  Fill values from container into gui
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}

		// reset sub panels
		importPanel.resetValues();
		exportPanel.resetValues();
		optionsPanel.resetValues();

		// set task name
		nameTextField.setText(container.getName());

		// get input contents
		textEditor.setText(container.getInputString());
		textEditor.setFile(container.getInputFile());

		outputDirField.setText(container.getOutputDirectory());

		widthField.setValue(container.getWidth());
		heightField.setValue(container.getHeight());
		aaField.setValue(container.getAntiAliasing());
	}

	/**
	 *@param  povrayDefaults  The new defaults value
	 */
	public void setDefaults(PovrayDefaults povrayDefaults) {
		this.povrayDefaults = povrayDefaults;
		textEditor.setDefaultDir(new File(povrayDefaults.getInputDirectory()));
		widthField.setValue(povrayDefaults.getWidth());
		heightField.setValue(povrayDefaults.getHeight());
		aaField.setValue(povrayDefaults.getAntiAliasing());
		outputDirField.setText(povrayDefaults.getOutputDirectory());
	}

	/**
	 *  compare gui entries to container values
	 */
	private void updateModifiedTime() {
		// if an entry is missing update modified time and get out
		if (container.getName() == null
			|| container.getInputString() == null) {
			container.setModifiedTime(new Date());
			return;
		}

		if (!container.getName().equals(nameTextField.getText())
			|| !container.getInputString().equals(textEditor.getText())) {
			container.setModifiedTime(new Date());
			return;
		}

		if (container.getInputFile() != null
			&& !container.getInputFile().equals(textEditor.getFile())) {
			container.setModifiedTime(new Date());
			return;
		}

	}

	/**
	 *  Method will be called whenever this panel is activated
	 *
	 *@param  vsiteChanged  true, if vsite has changed since last update or
	 *      reset
	 */
	public void updateValues(boolean vsiteChanged) {
		// if we are online get current storage resources from server
		if (container.getVsite() != null) {
			if (vsiteChanged) {
				// pass vsite to remote editor
				textEditor.setVsite(container.getVsite());

				// update storage resources for sub panels
				importPanel.updateValues();
				exportPanel.updateValues();
			}
		}

		// update editor if neccessary
		if (!textEditor.getFont().equals(povrayDefaults.getEditorFont())
			|| textEditor.getLineWrap() != povrayDefaults.getLineWrap()) {
			textEditor.setEditorFont(povrayDefaults.getEditorFont());
			textEditor.setLineWrap(povrayDefaults.getLineWrap());
			textEditor.repaint();
		}
	}

}
