package com.pallas.unicore.client.plugins.povray;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 *  This class manages the plugin
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: PovrayPlugin.java,v 1.3 2003/06/30 06:55:24 rrate Exp $
 */
public class PovrayPlugin extends TaskPlugable {

	/**
	 *  Listener class for settings menu entry
	 *
	 *@author     rrate
	 *@created    27. August 2001
	 */
	private class SettingsListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void actionPerformed(ActionEvent event) {
			showDefaultDialog();
		}
	}

	private static PovrayDefaults povrayDefaults;

	/**
	 *  Return a new container for this plugin
	 *
	 *@param  parentContainer  parent GroupContainer
	 *@return                  new initialized instance of plugin container
	 */
	public ActionContainer getContainerInstance(GroupContainer parentContainer) {

		// build new plugin container
		PovrayContainer container = new PovrayContainer(parentContainer);
		container.setName("New_" + getName() + counter);
		container.setIcon(new ImageIcon(getIconImage()));

		// apply defaults
		container.setInputFile(new File(povrayDefaults.getInputDirectory()));
		container.setOutputDirectory(povrayDefaults.getOutputDirectory());
		container.setWidth(povrayDefaults.getWidth());
		container.setHeight(povrayDefaults.getHeight());
		container.setAntiAliasing(povrayDefaults.getAntiAliasing());

		// count instance
		counter++;

		return container;
	}

	/**
	 *  Gets the iconPath attribute of the ScriptPlugin object
	 *
	 *@return    The iconPath value
	 */
	public String getIconPath() {
		return "com/pallas/unicore/client/plugins/povray/povray.gif";
	}

	/**
	 *  Gets the name attribute of the ScriptPlugin object
	 *
	 *@return    The name value
	 */
	public String getName() {
		return "POV-Ray";
	}

	/**
	 *  Build a new panel that manages the settings of a plugin container
	 *
	 *@param  tCont  plugin container to build panel for
	 *@return        JPAPanel that represents container settings in GUI
	 */
	public JPAPanel getPanelInstance(ActionContainer tCont) {
		// build new plugin panel
		PovrayJPAPanel panel =
			new PovrayJPAPanel(getClient(), (PovrayContainer) tCont);
		panel.setDefaults(povrayDefaults);
		return panel;
	}

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return "POV-Ray Plugin 1.0\n"
			+ "Author: Ralf Ratering\n"
			+ "(C)2003 Pallas GmbH";
	}

	/**
	 *  Gets the SettingsItem attribute of the ScriptPlugin object
	 *
	 *@return    The SettingsItem value
	 */
	public JMenuItem getSettingsItem() {
		// handling setting menu events must be done by the plugin writers
		// as this is an optional feature
		JMenuItem settingsItem = new JMenuItem("POV-Ray Defaults", 'P');
		settingsItem.addActionListener(new SettingsListener());
		return settingsItem;
	}

	/**
	 *  Show a dialog that configures plugin defaults
	 */
	private void showDefaultDialog() {
		PovrayDefaultsDialog povrayDefaultsDialog =
			new PovrayDefaultsDialog(getClient(), povrayDefaults);
		povrayDefaultsDialog.show();
		// apply changes (e.g. editor font) to currently
		// selected panel, if it is a script plugin panel
		getClient().updateDataArea();
	}

	/**
	 *  Called to inititialize the plugin
	 */
	public void startPlugin() {
		povrayDefaults = new PovrayDefaults();
	}

	/**
	 *  Called when terminating
	 */
	public void stopPlugin() {
		// do nothing
	}
}
