package com.pallas.unicore.client.plugins.povray;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import com.pallas.unicore.client.panels.Applyable;
import com.pallas.unicore.client.panels.ImagePanel;
import com.sixlegs.image.png.PngImage;

/**
 *@author     Ralf Ratering
 *@created    30. April 2003
 *@version    $Id: RenderPanel.java,v 1.6 2003/06/30 06:55:23 rrate Exp $
 */
public class RenderPanel extends JPanel implements Applyable {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.plugins.povray");
	final static long serialVersionUID = -8104512832433094363L;

	private File file;
	private JTextField filenameTextField;
	private ImagePanel imagePanel;
	private boolean initialized = false;

	private long lastModified = -1;

	/**
	 *  Constructor for the RenderPanel object
	 */
	public RenderPanel() {
		buildComponents();
	}

	/**
	 *  This method gets executed whenever the panel contents will be applied to
	 *  an object
	 */
	public void applyValues() {
		logger.fine("Applying povray render panel...");
	}

	/**
	 */
	private void buildComponents() {
		setLayout(new BorderLayout());

		imagePanel = new ImagePanel(false);
		add(new JScrollPane(imagePanel), BorderLayout.CENTER);

		filenameTextField = new JTextField(40);
		JPanel northPanel = new JPanel();
		northPanel.add(new JLabel("Filename:"));
		northPanel.add(filenameTextField);

		add(northPanel, BorderLayout.NORTH);

	}

	/**
	 *  Description of the Method
	 *
	 *@exception  IOException  Description of the Exception
	 */
	private void loadImage() throws IOException {
		PngImage png = new PngImage(file.getAbsolutePath());
		Image img = Toolkit.getDefaultToolkit().createImage(png);
		imagePanel.setImage(img);
	}

	/**
	 *  This method will be executed whenever a panel is filled with values
	 */
	public void resetValues() {
		logger.fine("Resetting povray render panel...");
		try {
			loadImage();
		} catch (IOException e) {
			logger.info("POV-Ray output file " + file + " not available yet.");
		}
		repaint();
	}

	/**
	 *  Sets the filename attribute of the RenderPanel object
	 *
	 *@param  file  The new file value
	 */
	public void setFile(File file) {
		this.file = file;
		filenameTextField.setText(file.getAbsolutePath());
		revalidate();
	}

	/**
	 *  Description of the Method
	 */
	public void updateValues() {
		logger.fine("Updating povray render panel...");
		try {
			loadImage();
		} catch (IOException e) {
			logger.info("POV-Ray output file " + file + " not available yet.");
		}
		repaint();
	}
}
