package com.pallas.unicore.client.plugins.proxycertificate;

import java.io.*;

import com.pallas.unicore.client.util.Defaults;

/**
 *  Define, load and save command plugin default values
 *
 *@author     Thomas Kentemich
 *@created    23. April 2002
 *@version    $Id: CompileDefaults.java,v 1.1.1.1 2002/04/22 08:30:25 thomas Exp
 *      $
 */
public class ProxyCertificateDefaults extends Defaults {

    /**
     *  String constants defining property keys
     */
    public final static String
            KEYSIZE_KEY = "plugin.proxycertificate.keysize",
	    LIMITED_KEY = "plugin.proxycertificate.limited",
	    VALIDTIME_KEY = "plugin.proxycertificate.validtime";

	    
    /**
     *  Sets the keysize
     *
     *@param  keysize  The new keysize value
     */
    public void setKeysize(int keysize) {
        put(KEYSIZE_KEY, new Integer(keysize).toString());
    }

    /**
     *  Gets the keysize
     *
     *@return    The keysize value
     */
    public int getKeysize() {
	int keySize = intValue(KEYSIZE_KEY);
	if(keySize == 0) {
		keySize = 512;
	}
        return keySize;
    }

   /**
     *  Sets the limited flag
     *
     *@param  keysize  The new limited value
     */
    public void setLimited(boolean limited) {
        put(LIMITED_KEY, new Boolean(limited).toString());
    }

    /**
     *  Gets the limited flag
     *
     *@return    The limited value
     */
    public boolean getLimited() {
        return booleanValue(LIMITED_KEY);
    }    
    
    
    /**
     *  Sets the valid time
     *
     *@param  valid  The new valid time value
     */
    public void setValidTime(long valid) {
        put(VALIDTIME_KEY, new Long(valid).toString());
    }

    /**
     *  Gets the validTime
     *
     *@return    The valid Time value
     */
    public long getValidTime() {
	long valid = longValue(VALIDTIME_KEY);
	if(valid == 0) {
		valid = 43200000;
	}
        return valid;
    }

    /**
     *  Load command default values from file ${HOME}/.unicoreXX/ProxyCertificatesDefaults.txt
     *
     *@return    true if loading was successful, else false
     */
    public boolean loadFromFile() {
        return loadFromFile(unicoreDir + File.separator + "ProxyCertificateDefaults.txt");
    }


    /**
     *  Try to write script default values to file ${HOME}/.unicoreXX/ProxyCertificates.txt
     *
     *@return    true if saving was successful, else false
     */
    public boolean writeToFile() {
        // always overwrite old ${HOME}/.unicoreXX/ProxyCertificates.txt
        return writeToFile(unicoreDir + File.separator + "ProxyCertificateDefaults.txt");
    }


    /**
     *  Fill properties with default values
     */
    protected void genSystemDefaults() {
        put(KEYSIZE_KEY, new Integer(512).toString());
	put(LIMITED_KEY, new Boolean(false).toString());
	put(VALIDTIME_KEY, new Long(1000L * 60 * 60 * 12).toString());
    }

}
