package com.pallas.unicore.client.plugins.proxycertificate;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.panels.*;
import com.pallas.unicore.client.util.*;
import com.pallas.unicore.container.*;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.KeyStoreManager;
import com.pallas.unicore.utility.Base64;

import java.awt.event.*;

import java.io.*;

import java.lang.reflect.*;

import java.math.BigInteger;

import java.security.*;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.security.spec.*;

import java.util.logging.*;
import java.util.Vector;

import javax.help.HelpSet;

import javax.swing.*;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.pkcs.EncryptedPrivateKeyInfo;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
import org.bouncycastle.asn1.util.*;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;

import org.bouncycastle.crypto.*;
import org.bouncycastle.crypto.encodings.*;
import org.bouncycastle.crypto.engines.*;
import org.bouncycastle.crypto.params.*;

import org.unicore.Vsite;

import org.unicore.ajo.AbstractJob;


/**
 *  A minimal plugin class
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: ProxyCertificatePlugin.java,v 1.1 2002/04/30 15:00:00 thomas
 *      Exp $
 */
public class ProxyCertificatePlugin
    extends ExtensionPlugable {
    private final static Logger logger = Logger.getLogger("com.pallas.unicore.client.plugins.proxycertificate");
    private ProxyCertificateDefaults proxyCertificateDefaults;

    /**
     *  Constructor for the ProxyCertificatePlugin object
     */
    public ProxyCertificatePlugin() {
    }

    /**
     *  Return additional services provided by this plugin
     *
     *@param  vsite          Description of the Parameter
     *@param  job            Description of the Parameter
     *@return                The serviceClass value
     *@exception  Exception  Description of the Exception
     */
    public Object setupSpecialVsiteFeatures(Vsite vsite, AbstractJob job)
                                     throws Exception {
        final String globus = "Globus";
        final String proxyFile = System.getProperty("java.io.tmpdir") +
                                 File.separator + "proxy.pem";
        String vsiteType = vsite.getType().getDescription();
        if (vsite.getType().equals(Vsite.Type.NJS)) {
            return null;
        }
        if (vsiteType.equals(globus)) {

            // try generating a proxy certificate
            try {
                IProxyCertificate proxyCertificateFactory =
                        ProxyCertificate.getInstance(job.getEndorser().getCertificate(),
                                                     proxyCertificateDefaults.getLimited(),
                                                     proxyCertificateDefaults.getValidTime(),
                                                     proxyCertificateDefaults.getKeysize());
                X509Certificate proxyCert = proxyCertificateFactory.getProxyCertificate();
                X509Certificate[] userCert = {
                    proxyCertificateFactory.getUserCertificate()
                };
                FileOutputStream fos = new FileOutputStream(proxyFile);
                proxyCertificateFactory.writePEM(fos);
                fos.close();
                FileInputStream fis = new FileInputStream(proxyFile);
                byte[] in = new byte[fis.available()];
                fis.read(in);
                fis.close();
                if (System.getProperty(
                            "com.pallas.unicore.proxycertificate.store") != null) {
				    File proxy = new File(proxyFile);
				    proxy.deleteOnExit();
				    // proxy.delete();
                }

                //      System.out.println(">>>>" + new String(in));
                // finally set the SSO
                job.setSSO(in);
            } catch (Exception e) {
               logger.log(Level.SEVERE, "", e);
            }
        }
        return null;
    }


    /**
     *  Get a short info message for the plugin
     *
     *@return    String with info message
     */
    public String getPluginInfo() {
        return "Generate a Globus proxy certificate 1.2\n" +
               "(C)2002 Pallas GmbH";
    }


    /**
     *  No help set available yet
     *
     *@return    a javax.help.HelpSet
     */
    public HelpSet getHelpSet() {
        return null;
    }


    /**
     *  Gets item for JPA menu
     *
     *@return    a new menu item
     */
    public JMenuItem getJPAItem() {

        // event handling for jpa menus will be done in the JPATree,
        // so we do not care about that here
        // plugin does not need a menu entry
        return null;
    }


    /**
     *  Gets an item for settings menu
     *
     *@return    null
     */
    public JMenuItem getSettingsItem() {

        // handling setting menu events must be done by the plugin writers
        // as this is an optional feature
        JMenuItem settingsItem = new JMenuItem("Proxy defaults", 'P');
        settingsItem.addActionListener(new SettingsListener());
        return settingsItem;
    }


    /**
     *  Gets a sub menu for the extension menu
     *
     *@return    null
     */
    public JMenuItem getCustomMenu() {
        return null;
    }


    /**
     *  Return a new container for this plugin
     *
     *@param  parentContainer  job that this task belongs to
     *@return                  new initialized instance of plugin container
     */
    public ActionContainer getContainerInstance(GroupContainer parentContainer) {

        // build new plugin container
        // plugin does not need a container
        return null;
    }


    /**
     *  Build a new panel that manages the parameters of a plugin container
     *
     *@param  container  Description of the Parameter
     *@return            JPAPanel that represents container parameters in GUI
     */
    public JPAPanel getPanelInstance(ActionContainer container) {

        // return new instance of plugin JPAPanel
        return null;
    }


    /**
     *  Called to inititialize the plugin
     *
     *@param  client  reference to main frame
     */
    public void startPlugin() {
        proxyCertificateDefaults = new ProxyCertificateDefaults();
        Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
    }


    /**
     *  Called when terminating
     */
    public void stopPlugin() {

        // do nothing
    }


    /**
     *  Description of the Method
     *
     *@param  container  Description of the Parameter
     *@return            Description of the Returned Value
     */
    public boolean canProvideJPAPanel(ActionContainer container) {
        return (false);
    }


    /**
     *  Show a dialog that configures plugin defaults
     */
    private void showDefaultDialog() {
        ProxyCertificateDefaultsDialog proxyCertificateDefaultsDialog =
                new ProxyCertificateDefaultsDialog(getClient(),
                                                   proxyCertificateDefaults);
        proxyCertificateDefaultsDialog.show();
    }

    /**
     *  Listener class for settings menu entry
     *
     *@author     rrate
     *@created    17. September 2001
     */
    private class SettingsListener
        implements ActionListener {

        /**
         *  Description of the Method
         *
         *@param  event  Description of Parameter
         */
        public void actionPerformed(ActionEvent event) {
            showDefaultDialog();
        }
    }
}
