package com.pallas.unicore.client.plugins.script;

import java.awt.Font;
import java.io.File;
import java.util.Enumeration;

import org.unicore.ajo.ScriptType;

import com.pallas.unicore.client.util.Defaults;

/**
 *  Define, load and save script plugin default values
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: ScriptDefaults.java,v 1.2 2003/06/30 06:55:38 rrate Exp $
 */
public class ScriptDefaults extends Defaults {

	/**
	 *  String constants defining property keys
	 */
	public final static String SCRIPTDIRECTORY =
		"plugin.script.scriptdirectory",
		TYPE = "plugin.script.type",
		EDITORFONTNAME = "plugin.script.editor.fontname",
		EDITORFONTSIZE = "plugin.script.editor.fontsize",
		EDITORLINEWRAP = "plugin.script.editor.linewrap";

	/**
	 *  Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(SCRIPTDIRECTORY, userHome);
		put(TYPE, ScriptType.SH.toString());
		put(EDITORFONTNAME, "Default");
		put(EDITORFONTSIZE, "12");
		put(EDITORLINEWRAP, "true");
	}

	/**
	 *  Get the editor font
	 *
	 *@return    The editor font
	 */
	public Font getEditorFont() {
		String fontName = (String) get(EDITORFONTNAME);
		int fontSize = intValue(EDITORFONTSIZE);

		return new Font(fontName, Font.PLAIN, fontSize);
	}

	/**
	 *  Get the line wrap flag for editor
	 *
	 *@return    boolean line wrap value
	 */
	public boolean getLineWrap() {
		return booleanValue(EDITORLINEWRAP);
	}

	/**
	 *  Get the script directory
	 *
	 *@return    script directory value
	 */
	public String getScriptDirectory() {
		return getProperty(SCRIPTDIRECTORY);
	}

	/**
	 *  Get the script type
	 *
	 *@return    script type
	 */
	public ScriptType getType() {
		String typeString = getProperty(TYPE);
		Enumeration typeEnum = ScriptType.elements();
		while (typeEnum.hasMoreElements()) {
			ScriptType type = (ScriptType) typeEnum.nextElement();
			if (type.toString().equals(typeString)) {
				return type;
			}
		}
		return null;
	}

	/**
	 *  Get script type as String
	 *
	 *@return    script type as String
	 */
	public String getTypeString() {
		return getProperty(TYPE);
	}

	/**
	 *  Load script default values from file ${HOME}/.unicore/ScriptDefaults.txt
	 *
	 *@return    true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		// look for  ${HOME}/.unicoreXX/ScriptDefaults.txt
		return loadFromFile(unicoreDir + File.separator + "ScriptDefaults.txt");
	}

	/**
	 *  Set the editor font
	 *
	 *@param  font  The new editor font
	 */
	public void setEditorFont(Font font) {
		put(EDITORFONTNAME, font.getFontName());
		put(EDITORFONTSIZE, new Integer(font.getSize()).toString());
	}

	/**
	 *  Set the line wrap flag for editor
	 *
	 *@param  b  boolean line wrap flag
	 */
	public void setLineWrap(boolean b) {
		put(EDITORLINEWRAP, new Boolean(b).toString());
	}

	/**
	 *  Set the script directory
	 *
	 *@param  s  The new script directory
	 */
	public void setScriptDirectory(String s) {
		put(SCRIPTDIRECTORY, s);
	}

	/**
	 *  Set the script type as string
	 *
	 *@param  type  script type as string
	 */
	public void setTypeString(String type) {
		put(TYPE, type);
	}

	/**
	 *  Try to write script default values to file ${HOME}/.unicore/ScriptDefaults.txt
	 *
	 *@return    true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicoreXX/ScriptDefaults.txt
		return writeToFile(unicoreDir + File.separator + "ScriptDefaults.txt");
	}

}
