package com.pallas.unicore.client.plugins.script;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 *  This class manages the execute script plugin
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: ScriptPlugin.java,v 1.2 2003/06/30 06:55:37 rrate Exp $
 */
public class ScriptPlugin extends TaskPlugable {

	/**
	 *  Listener class for settings menu entry
	 *
	 *@author     rrate
	 *@created    27. August 2001
	 */
	private class SettingsListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void actionPerformed(ActionEvent event) {
			showDefaultDialog();
		}
	}

	// plugin in specific user defaults
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.plugins.script.ResourceStrings");

	private static ScriptDefaults scriptDefaults;

	/**
	 *  Return a new container for this plugin
	 *
	 *@param  parentContainer  parent GroupContainer
	 *@return                  new initialized instance of plugin container
	 */
	public ActionContainer getContainerInstance(GroupContainer parentContainer) {

		// build new plugin container
		ScriptContainer container = new ScriptContainer(parentContainer);
		container.setName("New_" + getName() + counter);
		container.setIcon(new ImageIcon(getIconImage()));

		// apply defaults
		container.setScriptFile(new File(scriptDefaults.getScriptDirectory()));
		container.setScriptType(scriptDefaults.getType(), false);

		// count instance
		counter++;

		return container;
	}

	/**
	 *  Gets the iconPath attribute of the ScriptPlugin object
	 *
	 *@return    The iconPath value
	 */
	public String getIconPath() {
		return "com/pallas/unicore/client/plugins/script/script.gif";
	}

	/**
	 *  Gets the name attribute of the ScriptPlugin object
	 *
	 *@return    The name value
	 */
	public String getName() {
		return "Script";
	}

	/**
	 *  Build a new panel that manages the settings of a plugin container
	 *
	 *@param  tCont  plugin container to build panel for
	 *@return        JPAPanel that represents container settings in GUI
	 */
	public JPAPanel getPanelInstance(ActionContainer tCont) {
		// build new plugin panel
		ScriptJPAPanel panel =
			new ScriptJPAPanel(getClient(), (ScriptContainer) tCont);
		panel.setDefaults(scriptDefaults);

		return panel;
	}

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return res.getString("PLUGIN_INFO");
	}

	/**
	 *  Gets the SettingsItem attribute of the ScriptPlugin object
	 *
	 *@return    The SettingsItem value
	 */
	public JMenuItem getSettingsItem() {
		// handling setting menu events must be done by the plugin writers
		// as this is an optional feature
		JMenuItem settingsItem =
			new JMenuItem(res.getString("DEFAULT_ITEM"), 'S');
		settingsItem.addActionListener(new SettingsListener());
		return settingsItem;
	}

	/**
	 *  Show a dialog that configures plugin defaults
	 */
	private void showDefaultDialog() {
		ScriptDefaultsDialog scriptDefaultsDialog =
			new ScriptDefaultsDialog(getClient(), scriptDefaults);
		scriptDefaultsDialog.show();
		// apply changes (e.g. editor font) to currently
		// selected panel, if it is a script plugin panel
		getClient().updateDataArea();
	}

	/**
	 *  Called to inititialize the plugin
	 */
	public void startPlugin() {
		scriptDefaults = new ScriptDefaults();
	}

	/**
	 *  Called when terminating
	 */
	public void stopPlugin() {
		// do nothing
	}
}
