package com.pallas.unicore.client.plugins.smallservice;

import java.io.File;

import javax.swing.ImageIcon;

import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Minimal implementation of the abstract UserContainer.
 *
 *@author     Ralf Ratering
 *@created    25. Juni 2002
 *@version: $Id: SmallServiceContainer.java,v 1.3 2003/06/30 06:55:45 rrate Exp $
 */
public class SmallServiceContainer extends UserContainer {
	/**
	 *  Constructor for the SmallServiceContainer object
	 *
	 *@param  parentContainer  Description of the Parameter
	 */
	public SmallServiceContainer(JobContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Overwrite method from super class to add custom exports.
	 *  For this example the invocation in the IDB should be something like:
	 *  INVOCATION smallservice-1.0 [echo "Executing small service..."; date > smallService.txt ]
	 *
	 */
	public void buildActionGroup() {
		String unicoreDir = ResourceManager.getUserDefaults().getUnicoreDir();
		String userHome;
		try {
			userHome = System.getProperty("user.home");
		} catch (SecurityException se) {
			userHome = unicoreDir;
		}

		String filename = userHome + File.separator + "smallServiceOutcome.txt";
		FileExport[] exports =
			{
				 new FileExport(
					this,
					FileStorage.NSPACE_STRING,
					"smallService.txt",
					filename,
					true,
					true)};
		setFileExports(exports);
		super.buildActionGroup();
	}

	/**
	 *  We do not need an icon here, because service tasks do not appear in the
	 *  gui.
	 *
	 *@return    null
	 */
	public ImageIcon getIcon() {
		return null;
	}
}
