package com.pallas.unicore.client.remotefilechooser;

import java.awt.Component;
import java.awt.Cursor;
import java.io.File;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;

import org.unicore.Vsite;

import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  FileSystemView on remote UNICOREpro file systems
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: RemoteFileSystemView.java,v 1.3 2003/06/30 06:55:53 rrate Exp $
 */
public class RemoteFileSystemView extends FileSystemView {

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.remotefilechooser");

	// current directory
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.remotefilechooser.ResourceStrings");
	private RemoteFile currentDir;

	// file system that does the work
	private RemoteFileSystem fs;

	// flag if chooser has been initialized
	private boolean initialized = false;

	// parent component needed to switch mouse cursor
	private Component parent;

	/**
	 *  Constructor
	 *
	 *@param  vsite   to connect to
	 *@param  parent  GUI component
	 */
	public RemoteFileSystemView(Vsite vsite, Component parent) {
		this.parent = parent;
		this.fs = new RemoteFileSystem(vsite);
	}

	/**
	 *  Build file object from directory and filename.
	 *
	 *@param  d             directory that contains file
	 *@param  subspacePath  Description of the Parameter
	 *@return               new file object
	 */
	public File createFileObject(File d, String subspacePath) {
		RemoteFile dir = (RemoteFile) d;
		return new RemoteFile(subspacePath, dir.getStorage(), fs);
	}

	/**
	 *  Not yet implemented
	 *
	 *@param  containingDir            Description of Parameter
	 *@return                          Description of the Returned Value
	 *@exception  java.io.IOException  Description of Exception
	 */
	public File createNewFolder(File containingDir)
		throws java.io.IOException {
		logger.warning("Cannot create directories yet!");
		return null;
	}

	/**
	 *  Gets the defaultDirectory attribute of the RemoteFileSystemView object
	 *
	 *@return    The defaultDirectory value
	 */
	public File getDefaultDirectory() {
		return getHomeDirectory();
	}

	/**
	 *  Get array of files for a certain directory
	 *
	 *@param  dir            directory to get contents from
	 *@param  useFileHiding  flag if we show hidden files or not
	 *@return                array of files
	 */
	public File[] getFiles(File dir, boolean useFileHiding) {
		if (dir instanceof sun.awt.shell.ShellFolder) {
			return new File[0];
		}
		currentDir = (RemoteFile) dir;

		if (parent != null && initialized) {
			parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		}

		File[] listing = fs.getListing((RemoteFile) dir);

		if (parent != null) {
			parent.setCursor(Cursor.getDefaultCursor());
		}

		initialized = true;

		return listing;
	}

	/**
	 *  Get file system home
	 *
	 *@return    The HomeDirectory value
	 */
	public File getHomeDirectory() {
		return fs.getHome();
	}

	/**
	 *  Get parent directory from a file
	 *
	 *@param  d  RemoteFile to get parent from
	 *@return    parent RemoteFile
	 */
	public File getParentDirectory(File d) {
		RemoteFile dir = (RemoteFile) d;
		return dir.getParentFile();
	}

	/**
	 *  Get file system roots
	 *
	 *@return    array containing root RemoteFiles
	 */
	public File[] getRoots() {
		return fs.getRoots();
	}

	/**
	 *@param  f  Description of the Parameter
	 *@return    The systemDisplayName value
	 */
	public String getSystemDisplayName(File file) {
		String name = "";
		try {
			name = file.getName();
		} catch (Exception e) {
			logger.log(
				Level.WARNING,
				"Could not get system display name for file " + file,
				e);
		}
		return name;
	}

	/**
	 *  Gets the systemIcon attribute of the RemoteFileSystemView object
	 *
	 *@param  file  Description of the Parameter
	 *@return       The systemIcon value
	 */
	public Icon getSystemIcon(File file) {
		if (file.isDirectory()) {
			return ResourceManager.getIcon(ResourceManager.OPEN);
		} else {
			return ResourceManager.getIcon(ResourceManager.NEW);
		}
	}

	/**
	 *  Gets the systemTypeDescription attribute of the RemoteFileSystemView
	 *  object
	 *
	 *@param  file  Description of the Parameter
	 *@return       The systemTypeDescription value
	 */
	public String getSystemTypeDescription(File file) {
		if (file.isDirectory()) {
			return "Directory";
		} else {
			return "File";
		}
	}

	/**
	 *@param  f  Description of Parameter
	 *@return    true if file is hidden
	 */
	public boolean isHiddenFile(File f) {
		return f.isHidden();
	}

	/**
	 *@param  f  Description of Parameter
	 *@return    true if a file is file system root
	 */
	public boolean isRoot(File f) {
		if (f instanceof sun.awt.shell.ShellFolder) {
			return false;
		}
		return fs.isRoot(f);
	}

	/**
	 *  Gets the traversable attribute of the RemoteFileSystemView object
	 *
	 *@param  file  Description of the Parameter
	 *@return       The traversable value
	 */
	public Boolean isTraversable(File file) {
		RemoteFile remoteFile = (RemoteFile) file;
		return Boolean.valueOf(remoteFile.isDirectory() || remoteFile.isRoot());
	}

}
