package com.pallas.unicore.client.tables;

import java.awt.Component;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;

/**
 *  Table cell editor which uses a combo box
 *
 *  The combo box performs a changeSelection call to get the "correct" order of Listeners
 *
 *@author     Klaus-Dieter Oertel
 *@created    1. August 2002
 *@version $Id: ComboBoxCellEditor.java,v 1.2 2003/06/30 06:56:03 rrate Exp $
 */
public class ComboBoxCellEditor extends DefaultCellEditor {

	public ComboBoxCellEditor(JComboBox comboBox) {
		super(comboBox);
	}

	/**
	 *  Implements the <code>TableCellEditor</code> interface.
	 *
	 *@param  table       Description of the Parameter
	 *@param  value       Description of the Parameter
	 *@param  isSelected  Description of the Parameter
	 *@param  row         Description of the Parameter
	 *@param  column      Description of the Parameter
	 *@return             The tableCellEditorComponent value
	 */
	public Component getTableCellEditorComponent(
		JTable table,
		Object value,
		boolean isSelected,
		int row,
		int column) {
		/**
		 *  Select the current row. For some reason the ComboBoxListener is executed
		 *  before the ListSelectionListener, i.e. the JTable.getSelectedRow() would
		 *  be wrong in ComboBoxListener.itemChanged() without the changed selection.
		 */
		table.changeSelection(row, column, false, false);

		/**
		 *  The call of DefaultCellEditor.getTableCellEditorComponent will perform setValue
		 *  which will trigger the ListSelectionListener once more!
		 */
		return super.getTableCellEditorComponent(
			table,
			value,
			isSelected,
			row,
			column);
	}
}
