package com.pallas.unicore.client.tables;

import java.awt.Color;
import java.awt.Component;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 *  Table cell renderer which shows a combo box
 *
 *@author     Kirsten Foerster
 *@author     Klaus-Dieter Oertel
 *@created    18. September 2002
 *@version $Id: ComboBoxCellRenderer.java,v 1.2 2003/06/30 06:56:03 rrate Exp $
 */
public class ComboBoxCellRenderer
	extends JComboBox
	implements TableCellRenderer {

	/**
	 *  Constructor for the ComboRenderer object
	 *
	 *@param  vector  Description of the Parameter
	 */
	public ComboBoxCellRenderer(Vector vector) {
		super(vector);
		setOpaque(true);
	}

	/**
	 *  Gets the tableCellRendererComponent attribute of the ComboRenderer
	 *  object
	 *
	 *@param  table       Description of the Parameter
	 *@param  value       Description of the Parameter
	 *@param  isSelected  Description of the Parameter
	 *@param  hasFocus    Description of the Parameter
	 *@param  row         Description of the Parameter
	 *@param  column      Description of the Parameter
	 *@return             The tableCellRendererComponent value
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		// set appropriate background colour
		Color background = table.getBackground();
		if (isSelected) {
			background = table.getSelectionBackground();
		}
		setBackground(background);

		// set combo box to value
		setSelectedItem(value);

		// if the previous setting failed return a different combo box
		// showing the invalid value
		if (value != null && !value.equals(getSelectedItem())) {
			JComboBox errorBox = new JComboBox(new Object[] { value });
			errorBox.setSelectedItem(value);

			// set appropriate background colour
			errorBox.setBackground(background);
			//  errorBox.setForeground(Color.RED);

			return errorBox;
		}

		return this;
	}
}
