package com.pallas.unicore.client.tables;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@author     Klaus-Dieter Oertel
 *@created    9. August 2002
 *@version $Id: FilePermissionPanel.java,v 1.2 2003/06/30 06:56:01 rrate Exp $
 */
public class FilePermissionPanel extends JPanel implements TableCellRenderer {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    19. Februar 2002
	 */
	private class CheckBoxListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			// FilePermission = new FilePermission();
			if (e.getSource().equals(readCheckBox)) {
				permissions.setReadPermission(readCheckBox.isSelected());
			}
			if (e.getSource().equals(writeCheckBox)) {
				permissions.setWritePermission(writeCheckBox.isSelected());
			}
			if (e.getSource().equals(executeCheckBox)) {
				permissions.setExecutePermission(executeCheckBox.isSelected());
			}
		}
	}
	private FilePermission permissions;
	private JCheckBox readCheckBox, writeCheckBox, executeCheckBox;

	/**
	 *  Constructor for the FilePermissionPanel object
	 */
	public FilePermissionPanel() {
		buildComponents();
	}

	/**
	 *  Adds a feature to the ActionListener attribute of the
	 *  FilePermissionPanel object
	 *
	 *@param  listener  The feature to be added to the ActionListener
	 *      attribute
	 */
	public void addActionListener(ActionListener listener) {
		readCheckBox.addActionListener(listener);
		writeCheckBox.addActionListener(listener);
		executeCheckBox.addActionListener(listener);
	}

	/**
	 *  Description of the Method
	 */
	private void buildComponents() {
		readCheckBox = new JCheckBox("r");
		writeCheckBox = new JCheckBox("w");
		executeCheckBox = new JCheckBox("x");

		CheckBoxListener listener = new CheckBoxListener();
		readCheckBox.addActionListener(listener);
		writeCheckBox.addActionListener(listener);
		executeCheckBox.addActionListener(listener);

		setLayout(new GridLayout(1, 5));
		add(readCheckBox);
		add(new JLabel());
		add(writeCheckBox);
		add(new JLabel());
		add(executeCheckBox);
	}

	/**
	 *  Sets the FilePermission attribute of the FilePermissionPanel object
	 *
	 *@return    Description of the Return Value
	 */
	protected FilePermission getFilePermission() {
		return permissions;
	}

	/**
	 *  Gets the tableCellRendererComponent attribute of the DefaultCellEditor
	 *  object
	 *
	 *@param  table       Description of the Parameter
	 *@param  value       Description of the Parameter
	 *@param  isSelected  Description of the Parameter
	 *@param  row         Description of the Parameter
	 *@param  column      Description of the Parameter
	 *@return             The tableCellEditorComponent value
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object permissions,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		if (permissions instanceof FilePermission) {
			setFilePermission((FilePermission) permissions);
		} else {
			// this should never happen ...
			setFilePermission(
				new FilePermission().setPermission(permissions.toString()));
		}
		return this;
	}

	/**
	 *  Sets the FilePermission attribute of the FilePermissionPanel object
	 *
	 *@param  FilePermission  The new FilePermission value
	 */
	protected void setFilePermission(FilePermission permissions) {
		this.permissions = permissions;
		updateCheckBoxes();
	}

	/**
	 *  Description of the Method
	 */
	protected void updateCheckBoxes() {
		readCheckBox.setSelected(permissions.getReadPermission());
		writeCheckBox.setSelected(permissions.getWritePermission());
		executeCheckBox.setSelected(permissions.getExecutePermission());
	}
}
