package com.pallas.unicore.client.tables;

import java.awt.Color;
import java.awt.Component;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

/**
 *@author     Ralf Ratering
 *@created    17. Oktober 2001
 *@version    $Id: ListTable.java,v 1.2 2003/06/30 06:56:01 rrate Exp $
 */
public class ListTable extends JTable {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. Oktober 2001
	 */
	private class MyCellRenderer extends DefaultTableCellRenderer {

		/**
		 *  Returns the default table cell renderer.
		 *
		 *@param  table       the <code>JTable</code>
		 *@param  value       the value to assign to the cell at <code>[row, column]</code>
		 *@param  isSelected  true if cell is selected
		 *@param  row         the row of the cell to render
		 *@param  column      the column of the cell to render
		 *@param  hasFocus    Description of Parameter
		 *@return             the default table cell renderer
		 */
		public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
			super.getTableCellRendererComponent(
				table,
				value,
				isSelected,
				hasFocus,
				row,
				column);

			Color color = (Color) elementColors.get(row);

			if (isSelected) {
				super.setForeground(color.darker());
				super.setBackground(table.getSelectionBackground());
			} else {
				super.setForeground(color);
				super.setBackground(table.getBackground());
			}
			return this;
		}
	}
	private Vector columnNames = new Vector();
	private Vector data = new Vector();
	private Vector elementColors = new Vector();
	private DefaultTableModel model;

	/**
	 *  Constructor for the ListTable object
	 */
	public ListTable() {
		columnNames.add("");
		model = new DefaultTableModel(data, columnNames);
		// neccessary to get renderer work
		setOpaque(false);
		setModel(model);
		tableHeader.setVisible(false);
		setDefaultRenderer(Object.class, new MyCellRenderer());
	}

	/**
	 *  Description of the Method
	 *
	 *@param  value  Description of Parameter
	 */
	public void addElement(NamedObject value) {
		Vector row = new Vector();
		row.add(value.getName());
		row.add(value);
		data.add(row);
		elementColors.add(getForeground());
	}

	/**
	 *  Adds a feature to the ListSelectionListener attribute of the ListTable
	 *  object
	 *
	 *@param  listener  The feature to be added to the ListSelectionListener
	 *      attribute
	 */
	public void addListSelectionListener(ListSelectionListener listener) {
		getSelectionModel().addListSelectionListener(listener);
	}

	/**
	 *  Adds a feature to the TableModelListener attribute of the ListTable
	 *  object
	 *
	 *@param  listener  The feature to be added to the TableModelListener
	 *      attribute
	 */
	public void addTableModelListener(TableModelListener listener) {
		model.addTableModelListener(listener);
	}

	/**
	 *  Gets the Element attribute of the ListTable object
	 *
	 *@param  i  Description of Parameter
	 *@return    The Element value
	 */
	public NamedObject getElement(int i) {
		Vector selectedRow = (Vector) data.elementAt(i);
		String name = (String) selectedRow.firstElement();
		NamedObject object = (NamedObject) selectedRow.elementAt(1);
		object.setName(name);
		return object;
	}

	/**
	 *  Gets the NrOfElements attribute of the ListTable object
	 *
	 *@return    The NrOfElements value
	 */
	public int getNrOfElements() {
		return model.getRowCount();
	}

	/**
	 *  Gets the SelectedValue attribute of the ListTable object
	 *
	 *@return    The SelectedValue value
	 */
	public NamedObject getSelectedValue() {
		if (getSelectedRow() < 0) {
			return null;
		}
		return getElement(getSelectedRow());
	}

	public void removeAllElements() {
		data.clear();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  value  Description of Parameter
	 */
	public void removeElement(NamedObject value) {
		for (int i = 0; i < data.size(); i++) {
			Vector row = (Vector) data.elementAt(i);
			NamedObject object = (NamedObject) row.elementAt(1);
			if (object.equals(value)) {
				model.removeRow(i);
				elementColors.remove(i);
				return;
			}
		}
	}

	/**
	 *  Sets the elementColor attribute of the ListTable object
	 *
	 *@param  row    The new elementColor value
	 *@param  color  The new elementColor value
	 */
	public void setElementColor(int row, Color color) {
		elementColors.setElementAt(color, row);
	}

	/**
	 *  Sets the selectedRow attribute of the ListTable object
	 *
	 *@param  nr  The new selectedRow value
	 */
	public void setSelectedRow(int nr) {
		getSelectionModel().setSelectionInterval(nr, nr);
	}

}
