/*
 * @(#)TableMap.java	1.7 99/04/23
 *
 * Copyright (c) 1997-1999 by Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.pallas.unicore.client.tables;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

/**
 * In a chain of data manipulators some behaviour is common. TableMap
 * provides most of this behavour and can be subclassed by filters
 * that only need to override a handful of specific methods. TableMap
 * implements TableModel by routing all requests to its model, and
 * TableModelListener by routing all events to its listeners. Inserting
 * a TableMap which has not been subclassed into a chain of table filters
 * should have no effect.
 *
 * @version 1.7 04/23/99
 * @author Philip Milne
 * @created    8. Oktober 2001
 */
public class TableMap
	extends AbstractTableModel
	implements TableModelListener {
	/**
	 *  Description of the Field
	 */
	protected TableModel model;

	/**
	 *  Gets the ColumnClass attribute of the TableMap object
	 *
	 *@param  aColumn  Description of Parameter
	 *@return          The ColumnClass value
	 */
	public Class getColumnClass(int aColumn) {
		return model.getColumnClass(aColumn);
	}

	/**
	 *  Gets the ColumnCount attribute of the TableMap object
	 *
	 *@return    The ColumnCount value
	 */
	public int getColumnCount() {
		return (model == null) ? 0 : model.getColumnCount();
	}

	/**
	 *  Gets the ColumnName attribute of the TableMap object
	 *
	 *@param  aColumn  Description of Parameter
	 *@return          The ColumnName value
	 */
	public String getColumnName(int aColumn) {
		return model.getColumnName(aColumn);
	}

	/**
	 *  Gets the Model attribute of the TableMap object
	 *
	 *@return    The Model value
	 */
	public TableModel getModel() {
		return model;
	}

	/**
	 *  Gets the RowCount attribute of the TableMap object
	 *
	 *@return    The RowCount value
	 */
	public int getRowCount() {
		return (model == null) ? 0 : model.getRowCount();
	}

	/**
	 *  By default, Implement TableModel by forwarding all messages to the model.
	 *
	 *@param  aRow     Description of Parameter
	 *@param  aColumn  Description of Parameter
	 *@return          The ValueAt value
	 */
	public Object getValueAt(int aRow, int aColumn) {
		return model.getValueAt(aRow, aColumn);
	}

	/**
	 *  Gets the CellEditable attribute of the TableMap object
	 *
	 *@param  row     Description of Parameter
	 *@param  column  Description of Parameter
	 *@return         The CellEditable value
	 */
	public boolean isCellEditable(int row, int column) {
		return model.isCellEditable(row, column);
	}

	/**
	 *  Sets the Model attribute of the TableMap object
	 *
	 *@param  model  The new Model value
	 */
	public void setModel(TableModel model) {
		this.model = model;
		model.addTableModelListener(this);
	}

	/**
	 *  Sets the ValueAt attribute of the TableMap object
	 *
	 *@param  aValue   The new ValueAt value
	 *@param  aRow     The new ValueAt value
	 *@param  aColumn  The new ValueAt value
	 */
	public void setValueAt(Object aValue, int aRow, int aColumn) {
		model.setValueAt(aValue, aRow, aColumn);
	}

	/**
	 *  Implementation of the TableModelListener interface, By default forward all events to all the
	 *  listeners.
	 *
	 *@param  e  Description of Parameter
	 */
	public void tableChanged(TableModelEvent e) {
		fireTableChanged(e);
	}
}
