package com.pallas.unicore.client.trees;

import java.awt.Color;
import java.awt.Component;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    29. August 2001
 *@version    $Id: ClientCellRenderer.java,v 1.2 2003/06/30 06:56:26 rrate Exp $
 */
public class ClientCellRenderer extends DefaultTreeCellRenderer {

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.trees");

	/**
	 *  Sets the value of the current tree cell to value.
	 *
	 *@param  tree      is the JTree the receiver is being configured for
	 *@param  object    is the current tree node
	 *@param  selected  if true, the cell will be drawn as if selected.
	 *@param  expanded  if true, the node is currently expanded
	 *@param  leaf      if true, the node represets a leaf
	 *@param  row
	 *@param  hasFocus  if true, the node currently has focus
	 *@return           the Component that the renderer uses to draw the value
	 */
	public Component getTreeCellRendererComponent(
		JTree tree,
		Object object,
		boolean selected,
		boolean expanded,
		boolean leaf,
		int row,
		boolean hasFocus) {

		Component c =
			super.getTreeCellRendererComponent(
				tree,
				object,
				selected,
				expanded,
				leaf,
				row,
				hasFocus);

		if (!(c instanceof JLabel)) {
			logger.warning("Tree component is not a JLabel, what else is it?");
			return c;
		}

		JLabel label = (JLabel) c;

		tree.setRowHeight(24);

		// if we have a PanelNode get icon from container
		// else choose a default icon
		if (object instanceof PanelNode) {
			PanelNode node = (PanelNode) object;
			ImageIcon icon = node.getCurrentIcon();
			if (icon != null) {
				label.setIcon(icon);
			}

			setBackgroundNonSelectionColor(tree.getBackground());

			Color fg = tree.getForeground();
			Color selectionBackground = Color.orange;
			Color selectionBorder = Color.black;
			if (fg != null
				&& fg.getRed() > 140
				&& fg.getGreen() > 140
				&& fg.getBlue() > 140) {
				selectionBackground = Color.blue.darker();
				selectionBorder = Color.white;
			}
			selectionBackground =
				new Color(
					selectionBackground.getRed(),
					selectionBackground.getGreen(),
					selectionBackground.getBlue(),
					100);

			setBackgroundSelectionColor(selectionBackground);
			setBorderSelectionColor(selectionBorder);

			if (node.isUpToDate()) {
				label.setForeground(fg);
			} else {
				// if background is too dark for gray, choose light gray
				Color bg = tree.getBackground();
				if (bg.getRed() < 170
					&& bg.getGreen() < 170
					&& bg.getBlue() < 170) {
					label.setForeground(Color.lightGray);
				} else {
					label.setForeground(Color.gray);
				}
			}
		}

		return this;
	}
}
