package com.pallas.unicore.client.trees;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Abstract base class for a panel that shows jobs for jpa/jmc as a tree
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ClientTree.java,v 1.3 2003/06/30 06:56:25 rrate Exp $
 */
public abstract class ClientTree extends JTree implements ClipboardOwner {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class CopyAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public CopyAction() {
			super("Copy", ResourceManager.getIcon(ResourceManager.COPY));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			copy();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class CutAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public CutAction() {
			super("Cut", ResourceManager.getIcon(ResourceManager.CUT));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			cut();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    7. August 2002
	 */
	private class MyKeyListener extends KeyAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void keyPressed(KeyEvent e) {
			int keyCode = e.getKeyCode();
			int modifiers = e.getModifiers();
			// increase id
			eventId++;
			if (keyCode == KeyEvent.VK_X && modifiers == KeyEvent.CTRL_MASK) {
				cutAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (
				keyCode == KeyEvent.VK_C && modifiers == KeyEvent.CTRL_MASK) {
				copyAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (
				keyCode == KeyEvent.VK_V && modifiers == KeyEvent.CTRL_MASK) {
				pasteAction.actionPerformed(new ActionEvent(this, eventId, ""));
			} else if (keyCode == KeyEvent.VK_DELETE && modifiers == 0) {
				removeAction.actionPerformed(
					new ActionEvent(this, eventId, ""));
			}
		}
	}

	/**
	 *  Listen to tree selections and call updateSelection method
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class MyTreeSelectionListener implements TreeSelectionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  event  Description of Parameter
		 */
		public void valueChanged(TreeSelectionEvent event) {
			updateSelection();
			enableDefaultActions();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class PasteAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public PasteAction() {
			super("Paste", ResourceManager.getIcon(ResourceManager.PASTE));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			paste();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mai 2002
	 */
	class RemoveAction extends AbstractAction {
		/**
		 *  Constructor for the CopyAction object
		 */
		public RemoveAction() {
			super("Remove", ResourceManager.getIcon(ResourceManager.REMOVE));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			remove();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    7. November 2001
	 */
	class TreeMouseListener extends MouseAdapter {

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void mouseClicked(MouseEvent e) {
			enableDefaultActions();
			testShowPopup(e);
		}
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void mousePressed(MouseEvent e) {
			if (!hasFocus()) {
				requestFocus();
				updateSelection();
			}
			testShowPopup(e);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void mouseReleased(MouseEvent e) {
			testShowPopup(e);
		}

	}

	/**
	 *  Description of the Field
	 */
	protected static Clipboard clipboard = new Clipboard("treeclipboard");

	// id for key listener actions
	private static int eventId = 0;

	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.trees");

	/**
	 *  reference to UNICOREpro client
	 */
	protected Client client;
	/**
	 *  Description of the Field
	 */
	protected CopyAction copyAction = new CopyAction();

	/**
	 *  Description of the Field
	 */
	protected ContainerNode currentGroupNode, currentTopLevelJobNode;

	/**
	 *  Description of the Field
	 */
	protected CutAction cutAction = new CutAction();

	/**
	 *  selected node that has focus
	 */
	protected PanelNode focussedNode;

	/**
	 *  object corresponding to focussed node
	 */
	protected Object focussedObject;

	/**
	 *  panel corresponding to focussed node
	 */
	protected JPanel focussedPanel;

	/**
	 *  tree model
	 */
	protected DefaultTreeModel model;
	/**
	 *  Description of the Field
	 */
	protected PasteAction pasteAction = new PasteAction();

	/**
	 *  Description of the Field
	 */
	protected JPopupMenu popup = new JPopupMenu();
	/**
	 *  Description of the Field
	 */
	protected RemoveAction removeAction = new RemoveAction();

	/**
	 *  root node
	 */
	protected PanelNode root;

	/**
	 *  Constructor inits tree
	 *
	 *@param  client  reference to UNICOREpro client
	 */
	public ClientTree(Client client) {
		this.client = client;

		root = new PanelNode("");

		model = new DefaultTreeModel(root);
		model.setAsksAllowsChildren(true);

		setModel(model);

		// some cosmetics
		setCellRenderer(new ClientCellRenderer());
		setEditable(false);
		//setBackground(Color.white);
		putClientProperty("JTree.lineStyle", "Angled");

		// listen to mouse clicks...
		addMouseListener(new TreeMouseListener());

		// ...and selections
		addTreeSelectionListener(new MyTreeSelectionListener());

		// ...and keyboard events
		addKeyListener(new MyKeyListener());

		// allow only selection of one node at a time
		getSelectionModel().setSelectionMode(
			TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);

	}

	/**
	 *  Description of the Method
	 */
	protected abstract void copy();

	/**
	 *  Description of the Method
	 */
	protected abstract void cut();

	/**
	 *  Description of the Method
	 */
	protected abstract void enableDefaultActions();

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	private TreeNode findJobNode(TreeNode node) {
		if (!(node instanceof ContainerNode)) {
			return null;
		}
		while (node.getParent() instanceof ContainerNode) {
			node = node.getParent();
			if (node == null) {
				return null;
			}
		}
		return node;
	}

	/**
	 *  Gets the copyAction attribute of the JPATree object
	 *
	 *@return    The copyAction value
	 */
	public AbstractAction getCopyAction() {
		return copyAction;
	}

	/**
	 *  Gets the currentJob attribute of the ClientTree object
	 *
	 *@return    The currentJob value
	 */
	public GroupContainer getCurrentGroup() {
		if (currentGroupNode == null) {
			return null;
		}
		return (GroupContainer) currentGroupNode.getActionContainer();
	}

	/**
	 *  Gets the currentJobNode attribute of the ClientTree object
	 *
	 *@return    The currentJobNode value
	 */
	public ContainerNode getCurrentGroupNode() {
		return currentGroupNode;
	}

	/**
	 *  Gets the currentJob attribute of the ClientTree object
	 *
	 *@return    The currentJob value
	 */
	public JobContainer getCurrentJob() {
		return (JobContainer) getCurrentJobNode().getActionContainer();
	}

	/**
	 *  Gets the currentJobNode attribute of the ClientTree object
	 *
	 *@return    The currentJobNode value
	 */
	public ContainerNode getCurrentJobNode() {
		ContainerNode node = getCurrentGroupNode();
		if (node == null) {
			return null;
		}
		while (!node.isJobGroup()) {
			node = (ContainerNode) node.getParent();
		}
		return node;
	}

	/**
	 *  Gets the currentTopLevelJob attribute of the ClientTree object
	 *
	 *@return    The currentTopLevelJob value
	 */
	public JobContainer getCurrentTopLevelJob() {
		if (currentTopLevelJobNode == null) {
			return null;
		}
		return (JobContainer) currentTopLevelJobNode.getActionContainer();
	}

	/**
	 *  Gets the currentTopLevelJobNode attribute of the ClientTree object
	 *
	 *@return    The currentTopLevelJobNode value
	 */
	public ContainerNode getCurrentTopLevelJobNode() {
		return currentTopLevelJobNode;
	}

	/**
	 *  Gets the cutAction attribute of the JPATree object
	 *
	 *@return    The cutAction value
	 */
	public AbstractAction getCutAction() {
		return cutAction;
	}

	/**
	 *  Gets the focussed node
	 *
	 *@return    The SelectedNode value
	 */
	public PanelNode getFocussedNode() {
		return focussedNode;
	}

	/**
	 *  Gets the object belonging to the focussed node
	 *
	 *@return    The SelectedObject value
	 */
	public Object getFocussedObject() {
		return focussedObject;
	}

	/**
	 *  Gets the pasteAction attribute of the JPATree object
	 *
	 *@return    The pasteAction value
	 */
	public AbstractAction getPasteAction() {
		return pasteAction;
	}

	/**
	 *  Gets the removeAction attribute of the JPATree object
	 *
	 *@return    The removeAction value
	 */
	public AbstractAction getRemoveAction() {
		return removeAction;
	}

	/**
	 *  Gets the rootSelected attribute of the ClientTree object
	 *
	 *@return    The rootSelected value
	 */
	public boolean isRootSelected() {
		return root.equals(focussedNode);
	}

	/**
	 *  Gets the topLevelJobNodeSelected attribute of the ClientTree object
	 *
	 *@return    The topLevelJobNodeSelected value
	 */
	public boolean isTopLevelJobNodeSelected() {
		return focussedNode == currentTopLevelJobNode;
	}

	/**
	 *  ClipboardOwner interface:
	 *
	 *@param  clipboard     Description of Parameter
	 *@param  transferable  Description of Parameter
	 */
	public void lostOwnership(Clipboard clipboard, Transferable transferable) {
		logger.warning("Clipboard " + clipboard.getName() + " lost ownership!");
	}

	/**
	 *  Description of the Method
	 */
	protected abstract void paste();

	/**
	 *  Description of the Method
	 */
	protected abstract void remove();

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public boolean selectionContainsContainerNode() {
		TreePath[] paths = getSelectionPaths();
		if (paths == null) {
			return false;
		}
		for (int i = 0; i < paths.length; i++) {
			if (paths[i].getLastPathComponent() instanceof ContainerNode) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void selectNode(PanelNode node) {
		synchronized (root) {
			setSelectionPath(new TreePath(node.getPath()));
		}
	}

	/**
	 *  Description of the Method
	 */
	public void selectRootNode() {
		selectNode(root);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void setFocussedNode(PanelNode node) {
		this.focussedNode = node;
	}

	/**
	 *@param  evt  Description of Parameter
	 */
	private void testShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			enableDefaultActions();
			popup.show(this, evt.getX(), evt.getY());
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void updateNode(TreeNode node) {
		synchronized (root) {
			model.nodeChanged(node);
		}
	}

	/**
	 *  Get selected node and its corresponding objects
	 */
	protected void updateSelection() {
		synchronized (root) {
			// if no node has been selected, automatically select last selected nodes
			// and if it's the first time, select root node
			if (isSelectionEmpty()) {
				focussedNode = root;
				setSelectionPath(new TreePath(focussedNode.getPath()));
			}

			// set focus to last selected node
			focussedNode =
				(PanelNode) getLeadSelectionPath().getLastPathComponent();
			focussedObject = focussedNode.getUserObject();
			focussedPanel = focussedNode.getPanel();

			currentTopLevelJobNode = (ContainerNode) findJobNode(focussedNode);

			if (focussedNode.isGroup()) {
				currentGroupNode = (ContainerNode) focussedNode;
			} else if (focussedNode.isTask()) {
				currentGroupNode = (ContainerNode) focussedNode.getParent();
			}
			// root selected
			else {
				currentGroupNode = null;
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void updateSubtree(TreeNode node) {
		model.nodeChanged(node);
		for (int i = 0; i < node.getChildCount(); i++) {
			updateSubtree(node.getChildAt(i));
		}
	}

}
