package com.pallas.unicore.client.trees;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.JobContainer;

/**
 *  Node that extends a panel node by storing an ActionContainer. Note, that the
 *  state of the node follows the container's state and also the node icon is
 *  taken from the container. This class implements the Clonable interface to
 *  allow copying to the clipboard.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ContainerNode.java,v 1.2 2003/06/30 06:56:25 rrate Exp $
 */
public class ContainerNode extends PanelNode {
	private int completedDownloads;

	private int totalDownloads;

	/**
	 *  Constructor
	 *
	 *@param  container  to be stored
	 *@param  panel      belonging to this node
	 */
	public ContainerNode(ActionContainer container, JPanel panel) {
		super(container, panel);
	}

	/**
	 *  Get a reference to the ActionContainer
	 *
	 *@return    stored ActionContainer
	 */
	public ActionContainer getActionContainer() {
		return (ActionContainer) userObject;
	}

	/**
	 *  Gets the completedDownloads attribute of the ContainerNode object
	 *
	 *@return    The completedDownloads value
	 */
	public int getCompletedDownloads() {
		return this.completedDownloads;
	}

	/**
	 *  Overwrite method to pipe through icon from container
	 *
	 *@return    The ImageIcon value
	 */
	public ImageIcon getCurrentIcon() {
		ActionContainer container = (ActionContainer) userObject;
		return container.getStatusIcon();
	}

	/**
	 *  Get the state of a JobContainer. All other containers are always up to
	 *  date.
	 *
	 *@return    one of the constants ActionContainer.STATE_UPDATING, etc.
	 */
	public int getState() {
		if (getActionContainer() instanceof JobContainer) {
			return ((JobContainer) getActionContainer()).getState();
		} else {
			return ActionContainer.STATE_UP_TO_DATE;
		}
	}

	/**
	 *  Gets the totalDownloads attribute of the ContainerNode object
	 *
	 *@return    The totalDownloads value
	 */
	public int getTotalDownloads() {
		return this.totalDownloads;
	}

	/**
	 *  Test if a container is currently updating
	 *
	 *@return    true, if container is updating
	 */
	public boolean isUpdating() {
		return (getState() == ActionContainer.STATE_UPDATING);
	}

	/**
	 *  Test if a container is up to date
	 *
	 *@return    true if container is up to date
	 */
	public boolean isUpToDate() {
		return (getState() == ActionContainer.STATE_UP_TO_DATE);
	}

	/**
	 *  Sets the completedDownloads attribute of the ContainerNode object
	 *
	 *@param  completedDownloads  The new completedDownloads value
	 */
	public void setCompletedDownloads(int completedDownloads) {
		this.completedDownloads = completedDownloads;
	}

	/**
	 *  Set state for a job container. In case node does not represent a job
	 *  group do nothing.
	 *
	 *@param  newState  The new state value
	 */
	public void setState(int newState) {
		if (userObject instanceof JobContainer) {
			((JobContainer) userObject).setState(newState);
		}
		super.setState(newState);
	}

	/**
	 *  Sets the totalDownloads attribute of the ContainerNode object
	 *
	 *@param  totalDownloads  The new totalDownloads value
	 */
	public void setTotalDownloads(int totalDownloads) {
		this.totalDownloads = totalDownloads;
	}

	/**
	 *  Set container state to STATE_UP_TO_DATE
	 */
	public void setUpToDate() {
		setState(ActionContainer.STATE_UP_TO_DATE);
	}
}
