package com.pallas.unicore.client.trees;

import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.tree.TreePath;

import org.unicore.Vsite;
import org.unicore.ajo.Control;
import org.unicore.outcome.AbstractActionStatus;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.dialogs.DependencyViewer;
import com.pallas.unicore.client.dialogs.ResourceInfoDialog;
import com.pallas.unicore.client.panels.JobGroupPanel;
import com.pallas.unicore.client.panels.OutcomePanel;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.HoldJobContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.TransferableVector;
import com.pallas.unicore.utility.UserMessages;

/**
 *  A tree that shows usites, vsites, jobs and tasks for the JMC
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    24. August 2001
 *@version    $Id: JMCTree.java,v 1.4 2003/06/30 06:56:25 rrate Exp $
 */
public class JMCTree extends ClientTree {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class AbortAction extends AbstractAction {
		/**
		 *  Constructor for the AbortAction object
		 */
		public AbortAction() {
			super("Abort Job");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			controlAction(Control.ABORT);
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     kfoerst
	 *@created    11. Dezember 2002
	 */
	private class CollapseAllAction extends AbstractAction {
		/**
		 *  Constructor for the CollapseAllAction object
		 */
		public CollapseAllAction() {
			super("Collapse All");
			this.setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			collapseAll();
		}

	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class FetchOutputAction extends AbstractAction {
		/**
		 *  Constructor for the FetchOutputAction object
		 */
		public FetchOutputAction() {
			super(
				"Fetch Output",
				ResourceManager.getIcon(ResourceManager.FETCHOUTPUT));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			fetchOutcome();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class HoldAction extends AbstractAction {
		/**
		 *  Constructor for the HoldAction object
		 */
		public HoldAction() {
			super("Hold Job");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			controlAction(Control.HOLD);
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class RefreshAction extends AbstractAction {
		/**
		 *  Constructor for the RefreshAction object
		 */
		public RefreshAction() {
			super("Refresh", ResourceManager.getIcon(ResourceManager.REFRESH));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			refresh();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class ResumeAction extends AbstractAction {
		/**
		 *  Constructor for the ResumeAction object
		 */
		public ResumeAction() {
			super("Resume Job");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			// check if job is held by Hold Task
			ContainerNode holdNode = null;
			if (getCurrentJobNode() != null) {
				holdNode = findExecutingHoldNode(getCurrentJobNode());
			}
			if (holdNode == null) {
				controlAction(Control.RESUME);
			} else {
				// if hold task is executing, abort it
				// to continue job execution
				controlAction(Control.ABORT, holdNode);
			}
		}

	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class ShowDependenciesAction extends AbstractAction {
		/**
		 *  Constructor for the ShowDependenciesAction object
		 */
		public ShowDependenciesAction() {
			super("Show Dependencies");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			// build dependency viewer for jmc
			DependencyViewer dependencyViewer =
				new DependencyViewer(
					client,
					(GroupContainer) currentGroupNode.getActionContainer());
			dependencyViewer.show();
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class ShowResourcesAction extends AbstractAction {
		/**
		 *  Constructor for the ShowResourcesAction object
		 */
		public ShowResourcesAction() {
			super("Show Resources");
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			ActionContainer container =
				((ContainerNode) focussedNode).getActionContainer();
			if (container instanceof TaskContainer) {
				TaskContainer task = (TaskContainer) container;

				ResourceInfoDialog resourceInfo =
					new ResourceInfoDialog(client, task.getResourceSet(), true);
				resourceInfo.show();
			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    10. Juni 2002
	 */
	class StopRefreshAction extends AbstractAction {
		/**
		 *  Constructor for the StopRefreshAction object
		 */
		public StopRefreshAction() {
			super(
				"Stop Refresh",
				ResourceManager.getIcon(ResourceManager.STOP));
			setEnabled(false);
		}

		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			interruptUpdate();
			// update menus
			enableDefaultActions();
		}
	}

	// updater starts threads and updates the tree via observer callbacks
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.trees.ResourceStrings");

	private AbortAction abortAction = new AbortAction();
	private CollapseAllAction collapseAllAction = new CollapseAllAction();
	private FetchOutputAction fetchOutputAction = new FetchOutputAction();
	private HoldAction holdAction = new HoldAction();
	private RefreshAction refreshAction = new RefreshAction();
	private ResumeAction resumeAction = new ResumeAction();

	private String rootMessage = "";
	private ShowDependenciesAction showDependenciesAction =
		new ShowDependenciesAction();
	private ShowResourcesAction showResourcesAction = new ShowResourcesAction();
	private StopRefreshAction stopRefreshAction = new StopRefreshAction();

	private JMCTreeUpdater treeUpdater;

	/**
	 *  Constructor inits tree
	 *
	 *@param  client  reference to UNICOREpro client
	 */
	public JMCTree(Client client) {
		super(client);

		// root node simply displays "JMC"
		root.setUserObject(res.getString("JMC"));

		treeUpdater = new JMCTreeUpdater(this);

		// popup menu stuff
		buildPopupMenu();

		// initially show cached usites and vsites
		buildNodes();
	}

	/**
	 *  Recursively add contents of a job to the tree
	 *
	 *@param  groupNode  The feature to be added to the JobContents attribute
	 */
	private void addJobContents(ContainerNode groupNode) {
		synchronized (root) {
			GroupContainer container =
				(GroupContainer) groupNode.getActionContainer();
			// build nodes for job group children
			Vector tasks = container.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				ActionContainer ac = (ActionContainer) tasks.elementAt(i);
				if (ac instanceof GroupContainer) {
					// build job group node
					ContainerNode subGroupNode =
						new ContainerNode(ac, new OutcomePanel(client, ac));
					model.insertNodeInto(
						subGroupNode,
						groupNode,
						groupNode.getChildCount());
					// get path to node
					TreePath path =
						new TreePath(model.getPathToRoot(subGroupNode));
					// show path to jobgroup node
					makeVisible(path);

					addJobContents(subGroupNode);
				} else {
					ContainerNode taskNode =
						new ContainerNode(ac, new OutcomePanel(client, ac));
					model.insertNodeInto(
						taskNode,
						groupNode,
						groupNode.getChildCount());
					// get path to node
					TreePath path = new TreePath(model.getPathToRoot(taskNode));
					// show path to jobgroup node
					makeVisible(path);
				}
			}
		}
	}

	/**
	 *  Add a new JobGroup to the selected Vsite. Called by a GetRunningJobs
	 *  request
	 *
	 *@param  jobGroup   jobContainer to be added
	 *@param  vsiteNode  vsite node that is parent to job
	 */
	private void addJobGroup(JobContainer jobGroup, PanelNode vsiteNode) {
		synchronized (root) {
			// test if node is still selected when thread returned
			boolean stillSelected = (hasFocus() && focussedNode == vsiteNode);

			// build job group node
			ContainerNode jobGroupNode =
				new ContainerNode(jobGroup, new OutcomePanel(client, jobGroup));
			if (!nodeExists(jobGroupNode, vsiteNode)) {
				model.insertNodeInto(
					jobGroupNode,
					vsiteNode,
					vsiteNode.getChildCount());
			}

			// show path to jobgroup node
			makeVisible(new TreePath(model.getPathToRoot(jobGroupNode)));

			// add job tasks to tree
			addJobContents(jobGroupNode);

			// select vsite node, only if vsite node was still selected
			// when thread returned
			if (stillSelected) {
				setSelectionPath(new TreePath(model.getPathToRoot(vsiteNode)));
			}
		}
	}

	/**
	 *  Add a new usite to the root node. Called in response to a GetUsites
	 *  request
	 *
	 *@param  usiteNode  The feature to be added to the Usite attribute
	 */
	private void addUsite(PanelNode usiteNode) {
		synchronized (root) {
			usiteNode.setUpToDate();
			if (!nodeExists(usiteNode, root)) {
				model.insertNodeInto(usiteNode, root, root.getChildCount());
			}
			makeVisible(new TreePath(model.getPathToRoot(usiteNode)));
		}
	}

	/**
	 *  Add a new vsite node to a selected usite. Called in respones to a
	 *  GetVsites request
	 *
	 *@param  usiteNode  The feature to be added to the Vsite attribute
	 *@param  vsite      The feature to be added to the Vsite attribute
	 */
	private void addVsite(Vsite vsite, PanelNode usiteNode) {
		synchronized (root) {
			PanelNode vsiteNode = new PanelNode(vsite);
			if (ResourceManager.isSpecialVsite(vsite)) {
				return;
			}
			if (ResourceManager.isVsiteAvailable(vsite)) {
				vsiteNode.setIcon(
					ResourceManager.getIcon(ResourceManager.VSITE));
			} else {
				vsiteNode.setIcon(
					ResourceManager.getIcon(ResourceManager.UNAVAILABLEVSITE));
			}
			if (!nodeExists(vsiteNode, usiteNode)) {
				model.insertNodeInto(
					vsiteNode,
					usiteNode,
					usiteNode.getChildCount());
			} else {
				logger.warning(
					"Found duplicate vsite: " + vsiteNode.getUserObject());
			}
			makeVisible(new TreePath(model.getPathToRoot(vsiteNode)));
		}
	}

	/**
	 *  Build JMC tree with cached usites and vsites
	 */
	private void buildNodes() {
		Vector usites = ResourceManager.getUsites();
		if (usites == null) {
			return;
		}
		for (int i = 0; i < usites.size(); i++) {
			Usite usite = (Usite) usites.elementAt(i);
			PanelNode usiteNode = new PanelNode(usite);
			if (ResourceManager.isUsiteAvailable(usite)) {
				usiteNode.setIcon(
					ResourceManager.getIcon(ResourceManager.USITE));
			} else {
				usiteNode.setIcon(
					ResourceManager.getIcon(ResourceManager.UNAVAILABLEUSITE));
			}

			addUsite(usiteNode);
			Vector vsites = ResourceManager.getVsites(usite);
			if (vsites != null) {
				for (int j = 0; j < vsites.size(); j++) {
					addVsite((Vsite) vsites.elementAt(j), usiteNode);
				}
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	private void buildPopupMenu() {
		popup.removeAll();
		popup.setInvoker(this);

		popup.add(refreshAction);
		popup.add(stopRefreshAction);
		popup.add(fetchOutputAction);
		popup.addSeparator();
		popup.add(removeAction);
		popup.add(abortAction);
		popup.add(holdAction);
		popup.add(resumeAction);
		popup.addSeparator();
		popup.add(copyAction);
		popup.add(collapseAllAction);
		popup.add(showDependenciesAction);
		popup.add(showResourcesAction);

	}

	/**
	 *  Description of the Method
	 */
	private void collapseAll() {
		for (int i = 0; i < this.focussedNode.getChildCount(); i++) {
			PanelNode childNode = (PanelNode) this.focussedNode.getChildAt(i);
			TreePath treePath = new TreePath(childNode.getPath());
			if (!this.isCollapsed(treePath)) {
				this.collapsePath(treePath);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  control  Description of the Parameter
	 */
	private void controlAction(Control control) {
		Vector nodes = getSelectedTopLevelJobNodes();
		for (int i = 0; i < nodes.size(); i++) {
			controlAction(control, (ContainerNode) nodes.elementAt(i));
		}
	}

	/**
	 *  Let the tree updater submit a control action like ABORT, HOLD or RESUME
	 *  control action to be submitted
	 *
	 *@param  control  Description of Parameter
	 *@param  node     Description of the Parameter
	 */
	private void controlAction(Control control, ContainerNode node) {
		if (ResourceManager.isOffline()) {
			return;
		}
		treeUpdater.fireControlAction(node, control);
	}

	/**
	 *  Description of the Method
	 */
	protected void copy() {
		TreePath[] paths = getSelectionPaths();
		TransferableVector copiedNodes =
			new TransferableVector(PanelNode.getFlavor());
		for (int i = 0; i < paths.length; i++) {
			Object pathComponent = paths[i].getLastPathComponent();
			if (!(pathComponent instanceof ContainerNode)) {
				continue;
			}
			ContainerNode node = (ContainerNode) pathComponent;
			// only copy top level jobs
			if (node.isTopLevelJobGroup()) {
				ActionContainer clone =
					(ActionContainer) node.getActionContainer().clone();
				JobGroupPanel panel =
					new JobGroupPanel(client, (JobContainer) clone);
				try {
					copiedNodes.add(new ContainerNode(clone, panel));
				} catch (UnsupportedFlavorException e) {
					logger.log(Level.WARNING, "Could copy node " + node, e);
				}
			}
		}
		clipboard.setContents(copiedNodes, this);
	}

	/**
	 *  Description of the Method
	 */
	protected void cut() {
		// do nothing
	}

	/**
	 *  Remove a job from the tree
	 */
	private void deleteJob() {
		if (ResourceManager.isOffline()) {
			return;
		}

		// now delete nodes
		String jobList = "\n";
		Vector jobNodes = getSelectedTopLevelJobNodes();
		for (int i = 0; i < jobNodes.size(); i++) {
			ContainerNode containerNode =
				(ContainerNode) jobNodes.firstElement();
			JobContainer job =
				(JobContainer) containerNode.getActionContainer();
			PanelNode vsiteNode = (PanelNode) containerNode.getParent();
			Vsite vsite = (Vsite) vsiteNode.getUserObject();
			jobList += jobNodes.elementAt(i) + " at site " + vsite + "\n";
		}
		if (JOptionPane
			.showConfirmDialog(
				client.getContentPane(),
				"Do you really want to delete " + jobList,
				res.getString("DELETE_TITLE"),
				JOptionPane.YES_NO_OPTION)
			== JOptionPane.YES_OPTION) {
			treeUpdater.deleteJob(jobNodes);
		}
	}

	/**
	 *  Description of the Method
	 */
	public void enableDefaultActions() {
		if (focussedNode == null) {
			focussedNode = root;
		}
		boolean enabled =
			(currentTopLevelJobNode != null
				&& !ResourceManager.isOffline()
				&& getSelectionCount() == 1);
		refreshAction.setEnabled(
			!ResourceManager.isOffline() && focussedNode.isUpToDate());
		stopRefreshAction.setEnabled(!focussedNode.isUpToDate());
		removeAction.setEnabled(
			selectionContainsContainerNode() && !ResourceManager.isOffline());
		fetchOutputAction.setEnabled(
			selectionContainsContainerNode() && !ResourceManager.isOffline());
		abortAction.setEnabled(
			selectionContainsContainerNode() && !ResourceManager.isOffline());
		holdAction.setEnabled(
			selectionContainsContainerNode() && !ResourceManager.isOffline());
		resumeAction.setEnabled(
			selectionContainsContainerNode() && !ResourceManager.isOffline());
		copyAction.setEnabled(
			selectionContainsContainerNode() && !ResourceManager.isOffline());
		showDependenciesAction.setEnabled(enabled);
		showResourcesAction.setEnabled(focussedNode.isTask());
		this.collapseAllAction.setEnabled(!this.isCollapsedAll());
	}

	/**
	 *  Fetch the Outcome of a JobGroup
	 */
	private void fetchOutcome() {
		if (ResourceManager.isOffline()) {
			return;
		}
		// only fetch outcome for top level job groups
		Vector jobNodes = getSelectedTopLevelJobNodes();
		for (int i = 0; i < jobNodes.size(); i++) {
			ContainerNode jobNode = (ContainerNode) jobNodes.elementAt(i);
			if (!jobNode.isUpdating()) {
				jobNode.setState(PanelNode.STATE_UPDATING);
				updateNode(jobNode);
			}
			treeUpdater.getOutcomes(jobNode);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	private ContainerNode findExecutingHoldNode(ContainerNode node) {
		if (node.getActionContainer() instanceof HoldJobContainer) {
			if (node
				.getActionContainer()
				.getStatus()
				.equals(AbstractActionStatus.EXECUTING)) {
				return node;
			}
		}
		ContainerNode holdNode = null;
		Enumeration children = node.children();
		while (children.hasMoreElements() && holdNode == null) {
			ContainerNode child = (ContainerNode) children.nextElement();
			holdNode = findExecutingHoldNode(child);
		}
		return holdNode;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void fireChangeEvent(ContainerNode node) {
		firePropertyChange("jmcTreeChanged", node, null);
	}

	/**
	 *  Gets the abortAction attribute of the JMCTree object
	 *
	 *@return    The abortAction value
	 */
	public AbstractAction getAbortAction() {
		return abortAction;
	}

	/**
	 *  Gets the collapseAllAction attribute of the JMCTree object
	 *
	 *@return    The collapseAllAction value
	 */
	public CollapseAllAction getCollapseAllAction() {
		return this.collapseAllAction;
	}

	/**
	 *  Gets the fetchOutputAction attribute of the JMCTree object
	 *
	 *@return    The fetchOutputAction value
	 */
	public AbstractAction getFetchOutputAction() {
		return fetchOutputAction;
	}

	/**
	 *  Gets the holdAction attribute of the JMCTree object
	 *
	 *@return    The holdAction value
	 */
	public AbstractAction getHoldAction() {
		return holdAction;
	}

	/**
	 *  Gets the refreshAction attribute of the JMCTree object
	 *
	 *@return    The refreshAction value
	 */
	public AbstractAction getRefreshAction() {
		return refreshAction;
	}

	/**
	 *  Gets the resumeAction attribute of the JMCTree object
	 *
	 *@return    The resumeAction value
	 */
	public AbstractAction getResumeAction() {
		return resumeAction;
	}

	/**
	 *  Get JMC root node
	 *
	 *@return    root node
	 */
	public PanelNode getRoot() {
		return root;
	}

	/**
	 *  Gets the refreshableNodes attribute of the JMCTree object
	 *
	 *@return    The refreshableNodes value
	 */
	private Vector getSelectedRefreshableNodes() {
		TreePath[] paths = getSelectionPaths();
		Vector nodes = new Vector();
		if (paths != null) {
			for (int i = 0; i < paths.length; i++) {
				PanelNode node = (PanelNode) paths[i].getLastPathComponent();
				nodes.add(node);
			}
		}
		return nodes;
	}

	/**
	 *  Gets the selectedTopLevelJobNodes attribute of the JMCTree object
	 *
	 *@return    The selectedTopLevelJobNodes value
	 */
	private Vector getSelectedTopLevelJobNodes() {
		TreePath[] paths = getSelectionPaths();
		Vector jobNodes = new Vector();
		for (int i = 0; i < paths.length; i++) {
			PanelNode selectedNode =
				(PanelNode) paths[i].getLastPathComponent();
			// check if we are a container
			if (!(selectedNode instanceof ContainerNode)) {
				continue;
			}
			ContainerNode containerNode = (ContainerNode) selectedNode;

			// get the corresponding top level job
			PanelNode parentNode = containerNode;
			while (!((PanelNode) parentNode.getParent()).isVsite()) {
				parentNode = (PanelNode) parentNode.getParent();
			}
			if (!jobNodes.contains(parentNode)) {
				jobNodes.add(parentNode);
			}
		}
		return jobNodes;
	}

	/**
	 *  Gets the showDependenciesAction attribute of the JMCTree object
	 *
	 *@return    The showDependenciesAction value
	 */
	public AbstractAction getShowDependenciesAction() {
		return showDependenciesAction;
	}

	/**
	 *  Gets the showResourcesAction attribute of the JMCTree object
	 *
	 *@return    The showResourcesAction value
	 */
	public AbstractAction getShowResourcesAction() {
		return showResourcesAction;
	}

	/**
	 *  Gets the stopRefreshAction attribute of the JMCTree object
	 *
	 *@return    The stopRefreshAction value
	 */
	public AbstractAction getStopRefreshAction() {
		return stopRefreshAction;
	}

	/**
	 *  Description of the Method
	 */
	private void interruptUpdate() {

		if (ResourceManager.isOffline()) {
			return;
		}
		if (!focussedNode.isUpdating()) {
			// nothing to interrupt
			return;
		}

		// if we stopped refresh on root, stop site detection
		if (focussedNode.isRoot()) {
			client.stopLoadResources();
			return;
		}
		// else stop whatevers running in tree updater
		treeUpdater.interruptUpdate(focussedNode);
	}

	/**
	 *  Gets the collapsedAll attribute of the JMCTree object
	 *
	 *@return    The collapsedAll value
	 */
	private boolean isCollapsedAll() {
		for (int i = 0; i < this.focussedNode.getChildCount(); i++) {
			PanelNode childNode = (PanelNode) this.focussedNode.getChildAt(i);
			if (!this.isCollapsed(new TreePath(childNode.getPath()))) {
				return false;
			}
		}
		return true;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  newNode  Description of Parameter
	 *@param  parent   Description of Parameter
	 *@return          Description of the Returned Value
	 */
	private boolean nodeExists(PanelNode newNode, PanelNode parent) {
		Object object = newNode.getUserObject();
		if (object instanceof GroupContainer) {
			GroupContainer container = (GroupContainer) object;
			for (int i = 0; i < parent.getChildCount(); i++) {
				PanelNode childNode = (PanelNode) parent.getChildAt(i);
				GroupContainer group =
					(GroupContainer) childNode.getUserObject();
				if (group.getIdentifier().equals(container.getIdentifier())) {
					// update outcome in old job node
					//group.setOutcome(container.getOutcome());
					return true;
				}
			}
		}
		return false;
	}

	/**
	 *  Description of the Method
	 */
	protected void paste() {
		// do nothing
	}

	/**
	 *  Fetch new status informations for the selected object
	 */
	public void refresh() {
		Vector nodes = getSelectedRefreshableNodes();

		for (int i = 0; i < nodes.size(); i++) {
			refresh((PanelNode) nodes.elementAt(i));
		}

		// update menus
		enableDefaultActions();

	}

	/**
	 *  Fetch new status informations for the provided object
	 *
	 *@param  node  Description of Parameter
	 */
	public void refresh(PanelNode node) {
		// prevent multiple updates on same node
		// and also updates of a parent node while one of it's children is updating
		boolean isUpdating = node.isUpdating();
		int nrOfChildren = node.getChildCount();
		for (int i = 0; i < nrOfChildren; i++) {
			if (((PanelNode) node.getChildAt(i)).isUpdating()) {
				isUpdating = true;
			}
		}
		// and also updates of the child when the parent is  updating
		PanelNode parent = (PanelNode) focussedNode.getParent();
		if (parent != null) {
			if (parent.isUpdating()) {
				isUpdating = true;
			}
		}

		if (isUpdating) {
			return;
		}

		node.setState(PanelNode.STATE_UPDATING);
		updateNode(node);

		// if root node (=world) was selected get new usites
		if (node.isRoot()) {
			client.loadResources();
			return;
		}

		// selected a usite node
		if (node.isUsite()) {
			Usite usite = (Usite) node.getUserObject();
			if (!ResourceManager.isUsiteAvailable(usite)) {
				UserMessages.info(ResourceManager.getUsiteErrorMessage(usite));
			}
			node.setUpToDate();
			updateNode(node);
			return;
		}

		// vsite node selected
		else if (node.isVsite()) {
			if (ResourceManager.isOffline()) {
				return;
			}

			Vsite vsite = (Vsite) node.getUserObject();
			if (ResourceManager.isVsiteAvailable(vsite)
				&& ResourceManager.vsiteExists(vsite)) {
				// start a thread here...
				treeUpdater.getJobs(node);
			} else {
				UserMessages.info(ResourceManager.getVsiteErrorMessage(vsite));
				node.setUpToDate();
				updateNode(node);
				return;
			}

		}
		// top level job node selected
		else if (((PanelNode) node.getParent()).isVsite()) {
			if (ResourceManager.isOffline()) {
				return;
			}
			// start a thread here...
			treeUpdater.getNewJobStatus((ContainerNode) node);
		}
		// refresh current job group
		else {
			if (ResourceManager.isOffline()) {
				return;
			}
			// start a thread here...
			treeUpdater.getNewJobStatus(currentTopLevelJobNode);
		}
	}

	/**
	 *  Description of the Method
	 */
	protected void remove() {
		deleteJob();
	}

	/**
	 *  Remove a JobGroup from the JMC. Called in response to a DeleteJob
	 *  request
	 *
	 *@param  node  Description of the Parameter
	 */
	void removeSubTree(PanelNode node) {
		synchronized (root) {
			node.removeAllChildren();
			// delete old entries
			model.removeNodeFromParent(node);
			firePropertyChange("jmcTreeDelete", node, null);
			node = null;
		}
	}

	/**
	 *  Sets the requestInfo attribute of the JMCTree object
	 *
	 *@param  message  The new requestInfo value
	 */
	public void setRequestInfo(String message) {
		rootMessage = message;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  runningJobs  Description of the Parameter
	 *@param  vsiteNode    Description of the Parameter
	 */
	void updateRunningJobs(Vector runningJobs, PanelNode vsiteNode) {
		// check, if jobs that exist in the tree have been removed
		// from njs by somebody else
		Vector removeChildren = new Vector();

		//check, if some RunningJobs are destroyed by the admin
		for (int i = 0; i < vsiteNode.getChildCount(); i++) {
			ContainerNode jobNode = (ContainerNode) vsiteNode.getChildAt(i);
			boolean del = true;
			for (int j = 0; j < runningJobs.size(); j++) {
				JobContainer jc = (JobContainer) runningJobs.elementAt(j);
				if (jobNode
					.getActionContainer()
					.getIdentifier()
					.equals(jc.getIdentifier())) {
					del = false;
					break;
				}
			}
			if (del) {
				removeChildren.add(jobNode);
			}
		}

		//all jobs will be elminated from the Job Monitoring Tree, which don't exist anymore
		for (int i = 0; i < removeChildren.size(); i++) {
			removeSubTree((PanelNode) removeChildren.elementAt(i));
		}

		//all submitted jobs, which aren't refreshed yet, are added on the tree.
		for (int i = 0; i < runningJobs.size(); i++) {
			JobContainer jc = (JobContainer) runningJobs.elementAt(i);
			boolean add = true;
			for (int j = 0; j < vsiteNode.getChildCount(); j++) {
				ContainerNode jobNode = (ContainerNode) vsiteNode.getChildAt(j);
				if (jc
					.getIdentifier()
					.equals(jobNode.getActionContainer().getIdentifier())) {
					add = false;
					break;
				}
			}
			if (add) {
				addJobGroup(jc, vsiteNode);
			}
		}
	}

	/**
	 *  React on selection event
	 */
	protected void updateSelection() {
		// get selected objects by calling method of super class
		super.updateSelection();

		if (isRootSelected()) {
			client.setStatusMessage(rootMessage);
		} else {
			client.setStatusMessage("Job Monitoring: " + focussedNode);
		}

		// only refresh node if it a vsite and not up to date
		if (!focussedNode.isUpToDate() && focussedNode.isVsite()) {
			refresh();
		}

		// if site or root node selected, enable menus, set panel and return
		if (focussedNode.isRoot()
			|| focussedNode.isUsite()
			|| focussedNode.isVsite()) {
			// switch main panel to logo
			client.clearDataArea();
			return;
		}

		OutcomePanel outcomePanel = (OutcomePanel) focussedNode.getPanel();
		outcomePanel.updateValues();

		client.addToDataArea(focussedPanel);
	}

	/**
	 *  Description of the Method
	 */
	public void updateUI() {
		super.updateUI();

		setCellRenderer(new ClientCellRenderer());

		if (popup != null) {
			SwingUtilities.updateComponentTreeUI(popup);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usites  Description of the Parameter
	 */
	public void updateUsites(Vector usites) {
		Enumeration usiteNodes = root.children();

		// separate current from outdated nodes
		Vector validUsites = new Vector();
		Vector invalidUsites = new Vector();
		while (usiteNodes.hasMoreElements()) {
			PanelNode usiteNode = (PanelNode) usiteNodes.nextElement();
			Usite oldUsite = (Usite) usiteNode.getUserObject();
			if (!usites.contains(oldUsite)) {
				// usite does not exist anymore -> remove node
				invalidUsites.add(usiteNode);
			} else {
				// old usite still valid
				validUsites.add(oldUsite);
				// reset icon
				usiteNode.setIcon(
					ResourceManager.getIcon(ResourceManager.USITE));
			}
		}

		// remove old usites
		for (int i = 0; i < invalidUsites.size(); i++) {
			removeSubTree((PanelNode) invalidUsites.elementAt(i));
		}

		// add new usites
		for (int i = 0; i < usites.size(); i++) {
			Usite usite = (Usite) usites.elementAt(i);
			if (!validUsites.contains(usite)) {
				PanelNode node = new PanelNode(usite);
				node.setIcon(ResourceManager.getIcon(ResourceManager.USITE));
				addUsite(node);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  vsite  Description of the Parameter
	 */
	public void updateVsiteIcon(Vsite vsite) {
		synchronized (root) {
			Enumeration usiteNodes = root.children();
			while (usiteNodes.hasMoreElements()) {
				PanelNode usiteNode = (PanelNode) usiteNodes.nextElement();
				Enumeration vsiteNodes = usiteNode.children();
				while (vsiteNodes.hasMoreElements()) {
					PanelNode vsiteNode = (PanelNode) vsiteNodes.nextElement();
					Vsite treeVsite = (Vsite) vsiteNode.getUserObject();
					if (vsite.equals(treeVsite)) {
						if (ResourceManager.isVsiteAvailable(vsite)) {
							vsiteNode.setIcon(
								ResourceManager.getIcon(ResourceManager.VSITE));
						} else {
							vsiteNode.setIcon(
								ResourceManager.getIcon(
									ResourceManager.UNAVAILABLEVSITE));
						}
						model.reload(vsiteNode);
					}
				}
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usite   Description of the Parameter
	 *@param  vsites  Description of the Parameter
	 */
	public void updateVsites(Usite usite, Vector vsites) {
		PanelNode usiteNode = null;
		Enumeration usiteNodes = root.children();
		while (usiteNodes.hasMoreElements()) {
			PanelNode node = (PanelNode) usiteNodes.nextElement();
			if (usite.equals(node.getUserObject())) {
				usiteNode = node;
			}
		}
		if (usiteNode == null) {
			return;
		} else if (!ResourceManager.isUsiteAvailable(usite)) {
			usiteNode.setIcon(
				ResourceManager.getIcon(ResourceManager.UNAVAILABLEUSITE));
			return;
		}

		// separate current from outdated nodes
		Enumeration vsiteNodes = usiteNode.children();
		Vector validVsites = new Vector();
		Vector invalidVsites = new Vector();
		while (vsiteNodes.hasMoreElements()) {
			PanelNode vsiteNode = (PanelNode) vsiteNodes.nextElement();
			Vsite oldVsite = (Vsite) vsiteNode.getUserObject();

			if (!vsites.contains(oldVsite)
				|| ResourceManager.isSpecialVsite(oldVsite)) {
				// vsite does not exist anymore -> remove node
				invalidVsites.add(vsiteNode);
			} else {
				// old vsite still valid
				validVsites.add(oldVsite);
			}
		}

		// remove old vsites
		for (int i = 0; i < invalidVsites.size(); i++) {
			removeSubTree((PanelNode) invalidVsites.elementAt(i));
		}

		// add new vsites
		for (int i = 0; i < vsites.size(); i++) {
			if (!validVsites.contains(vsites.elementAt(i))) {
				addVsite((Vsite) vsites.elementAt(i), usiteNode);
			}
		}
	}

}
