package com.pallas.unicore.client.util;

import javax.swing.JFrame;

import com.pallas.unicore.client.panels.ExportJPAPanel;
import com.pallas.unicore.client.panels.HoldJobPanel;
import com.pallas.unicore.client.panels.ImportJPAPanel;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.panels.OperationJPAPanel;
import com.pallas.unicore.client.panels.TransferJPAPanel;
import com.pallas.unicore.container.FileExportContainer;
import com.pallas.unicore.container.FileImportContainer;
import com.pallas.unicore.container.FileOperationContainer;
import com.pallas.unicore.container.FileTransferContainer;
import com.pallas.unicore.container.HoldJobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ClientPluginManager.java,v 1.2 2003/06/30 06:56:38 rrate Exp $
 */
public class ClientPluginManager extends PluginManager {

	/**
	 *  Gets the PanelInstance attribute of the ClientPluginManager object
	 *
	 *@param  container  Description of Parameter
	 *@return            The PanelInstance value
	 */
	public JPAPanel getPanelInstance(TaskContainer container) {
		// first check if we have a plugin container
		if (pluginClasses.size() > 0) {
			Class containerClass = container.getClass();
			for (int i = 0; i < pluginClasses.size(); i++) {
				UnicorePlugable plugable =
					(UnicorePlugable) pluginClasses.elementAt(i);
				if (plugable instanceof TaskPlugable) {
					TaskPlugable taskPlugable = (TaskPlugable) plugable;
					if (taskPlugable.hasContainerClass(container.getClass())) {
						return taskPlugable.getPanelInstance(container);
					}
				}
			}
		}
		// if we came here it must be some standard container
		// user task

		JFrame parent = ResourceManager.getCurrentInstance();

		// import task
		if (container instanceof FileImportContainer) {
			return new ImportJPAPanel(parent, (FileImportContainer) container);
		}
		// export task
		else if (container instanceof FileExportContainer) {
			return new ExportJPAPanel(parent, (FileExportContainer) container);
		}
		// transfer task
		else if (container instanceof FileTransferContainer) {
			return new TransferJPAPanel(
				parent,
				(FileTransferContainer) container);
		}
		// file operation task
		else if (container instanceof FileOperationContainer) {
			return new OperationJPAPanel(
				parent,
				(FileOperationContainer) container);
		}
		// hold job
		else if (container instanceof HoldJobContainer) {
			return new HoldJobPanel(parent, (HoldJobContainer) container);
		}
		// unknown container -> no panel
		return null;
	}

}
