package com.pallas.unicore.client.util;

import java.security.Principal;
/**
 *  A principal represents a distinguished name of an individual which includes
 *  different fields
 *
 *@author     Kirsten Foerster
 *@created    5. September 2002
 *@version    $Id: DistinguishedNameFields.java,v 1.3 2003/06/30 06:56:37 rrate Exp $
 */

public class DistinguishedNameFields {

	private String commonName = "";
	private String country = "";
	private String email = "";
	private String location = "";
	private String org = "";
	private String orgUnit = "";
	private String principal;
	private String state = "";

	/**
	 *  Constructor for the DistinguishedNameFields object
	 *
	 *@param  principal  Description of the Parameter
	 */
	public DistinguishedNameFields(Principal principal) {
		this.principal = principal.toString();
		this.setFields();
	}

	/**
	 *  Gets the commonName attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the common name value with mnemonic;
	 *      otherwise without this mnemonic
	 *@return       The commonName value
	 */
	public String getCommonName(boolean flag) {
		if (!flag && !this.commonName.equals("")) {
			return this.commonName.substring(3, this.commonName.length());
		}
		return this.commonName;
	}

	/**
	 *  Gets the country attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the state value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The country value
	 */
	public String getCountry(boolean flag) {
		if (!flag && !this.country.equals("")) {
			return this.country.substring(2, this.country.length());
		}
		return this.country;
	}

	/**
	 *  Gets the email attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the email value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The email value
	 */
	public String getEmail(boolean flag) {
		if (!flag && !this.email.equals("")) {
			return this.email.substring(6, this.email.length());
		}
		return this.email;
	}

	/**
	 *  Gets the location attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the location value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The location value
	 */
	public String getLocation(boolean flag) {
		if (!flag && !this.location.equals("")) {
			return this.location.substring(2, this.location.length());
		}
		return this.location;
	}

	/**
	 *  Gets the org attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the org value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The org value
	 */
	public String getOrg(boolean flag) {
		if (!flag && !this.org.equals("")) {
			return this.org.substring(2, this.org.length());
		}
		return this.org;
	}

	/**
	 *  Gets the orgUnit attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the orgUnit value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The orgUnit value
	 */
	public String getOrgUnit(boolean flag) {
		if (!flag && !this.orgUnit.equals("")) {
			return this.orgUnit.substring(3, this.orgUnit.length());
		}
		return this.orgUnit;
	}

	/**
	 *  Returns only one information of the principal.
	 *
	 *@param  textFrom  - a string represents the part of the principal you want
	 *      give back
	 *@return           - returns one information of the distinguished name
	 */
	private String getPrincipalField(String textFrom) {
		String text = "";
		String modPrincipal = this.principal.toLowerCase();
		String modTextFrom = textFrom.toLowerCase();
		int indexFrom = modPrincipal.indexOf(modTextFrom);
		int indexTo;

		if (indexFrom != -1) {
			indexTo = modPrincipal.indexOf(",", indexFrom);
			if (indexTo == -1) {
				text
					+= this.principal.substring(
						indexFrom,
						this.principal.length());
			} else {
				text += this.principal.substring(indexFrom, indexTo);
			}
		}

		return text;

	}

	/**
	 *  Gets the state attribute of the DistinguishedNameFields object
	 *
	 *@param  flag  - true, you get the state value with mnemonic; otherwise
	 *      without this mnemonic
	 *@return       The state value
	 */
	public String getState(boolean flag) {
		if (!flag && !this.state.equals("")) {
			return this.state.substring(3, this.state.length());
		}
		return this.state;
	}

	/**
	 *  Set the fields information of a distinguished name.
	 */
	private void setFields() {
		String text = principal.toString();
		this.commonName = this.commonName + this.getPrincipalField("CN");
		this.orgUnit = this.orgUnit + this.getPrincipalField("OU=");
		this.org = this.org + this.getPrincipalField("O=");
		this.location = this.location + this.getPrincipalField("L=");
		this.state = this.state + this.getPrincipalField("ST=");
		this.country = this.country + this.getPrincipalField("C=");
		this.email = this.email + this.getPrincipalField("EMAIL");
		if (email.indexOf("=") >= 0) {
			this.email =
				"EMAIL"
					+ this.email.substring(
						this.email.indexOf("="),
						this.email.length());
		}
	}
}
