package com.pallas.unicore.client.util;

import java.awt.Component;

import javax.swing.JMenuItem;

import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;

/**
 *  Abstract base class for UNICOREpro client extension plugins
 *
 *@author     Ralf Ratering
 *@created    27. Mai 2002
 *@version $Id: ExtensionPlugable.java,v 1.3 2003/06/30 06:56:37 rrate Exp $
 */
public abstract class ExtensionPlugable extends UnicorePlugable {

	/**
	 *  Used to add a plugin controlled item or sub menu to the client's
	 *  extension menu. The client does not attach a listener to this item so it
	 *  is up to the plugin developer to respond to this item. The method
	 *  returns null if no menu item should be added to the clients extension
	 *  menu.
	 *
	 *@return    a menu item that will be added to the client's extension menu
	 */
	public JMenuItem getCustomMenu() {
		return null;
	}

	/**
	 *  Get a Component that will be added to the toolbar that is attached to
	 *  the JPA tree.
	 *
	 *@return    a Component, ideally a button containing an icon
	 */
	public Component getJPAToolBarComponent() {
		return null;
	}

	/**
	 *  Get a Component that will be added to the toolbar that is attached to
	 *  the vsite list in a job group panel.
	 *
	 *@return    a Component, ideally a button containing an icon
	 */
	public Component getVsiteToolBarComponent() {
		return null;
	}

	/**
	 *  This method checks if the plugin can enable special features for a
	 *  vsite. It may for example create a Globus proxy certifcate for a Globus
	 *  enabled Vsite.
	 *
	 *@param  vsite          The vsite of the job
	 *@param  job            The abstract job object
	 *@return                Objects returned.
	 *@exception  Exception  an arbitrary Exception
	 */
	public Object setupSpecialVsiteFeatures(Vsite vsite, AbstractJob job)
		throws Exception {
		return null;
	}

}
