package com.pallas.unicore.client.util;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.swing.ImageIcon;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Class that scans directories for plugins and starts them
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: PluginManager.java,v 1.3 2003/06/30 06:56:36 rrate Exp $
 */
public class PluginManager {

	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.util");

	/**
	 *  Description of the Field
	 */
	protected Hashtable iconTable = new Hashtable();

	/**
	 *  Description of the Field
	 */
	protected Vector knownPlugins = new Vector();
	/**
	 *  Description of the Field
	 */
	protected Vector pluginClasses = new Vector();

	/**
	 *  Description of the Field
	 */
	protected Vector taskPluginClasses = new Vector();

	/**
	 *  Gets the icon attribute of the PluginManager object
	 *
	 *@param  container  Description of the Parameter
	 *@return            The icon value
	 */
	public ImageIcon getIcon(ActionContainer container) {
		ImageIcon icon = (ImageIcon) iconTable.get(container.getClass());
		if (icon == null) {
			icon = ResourceManager.getIcon(ResourceManager.UNKNOWNTASK);
		}
		return icon;
	}

	/**
	 *  Return the initializer classes of all plugins
	 *
	 *@return    The PluginClasses value
	 */
	public Vector getPluginClasses() {
		return pluginClasses;
	}

	/**
	 *  Return all know plugins
	 *
	 *@return    The Plugins value
	 */
	public Vector getPlugins() {
		return knownPlugins;
	}

	/**
	 *  Gets the jPAItems attribute of the PluginManager object
	 *
	 *@return    The jPAItems value
	 */
	public Vector getTaskPlugins() {
		return taskPluginClasses;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  dirname  Description of Parameter
	 */
	private void scanDirectory(String dirname) {
		logger.info("Scanning directory: " + dirname);
		File dir = new File(dirname);

		if (!dir.exists() || !dir.isDirectory()) {
			return;
		}

		String[] plugins = dir.list();
		if (plugins == null) {
			return;
		}

		for (int i = 0; i < plugins.length; i++) {
			String plugin = plugins[i];
			// log.logWrite("Scanning file: "+plugin);
			if (!plugin.toLowerCase().endsWith("plugin.jar")) {
				continue;
			}

			ZipFile zip;
			try {
				zip = new ZipFile(dirname + File.separator + plugin);
			} catch (IOException e) {
				logger.log(
					Level.WARNING,
					"Could not open plugin: " + plugin,
					e);
				continue;
			}

			logger.info("Found plugin jar: " + plugin);

			URL urls[] = new URL[1];
			URLClassLoader urlLoader = null;
			try {
				urls[0] = new URL("file:" + dirname + File.separator + plugin);
				Class self = PluginManager.class;
				urlLoader = new URLClassLoader(urls, self.getClassLoader());
			} catch (MalformedURLException mfue) {
				UserMessages.error(
					"Cannot convert file name to URL: " + plugin);
				continue;
			}

			Enumeration entries = zip.entries();
			while (entries.hasMoreElements()) {
				ZipEntry zipEntry = (ZipEntry) entries.nextElement();
				String name = zipEntry.getName();

				if (!name.toLowerCase().endsWith("plugin.class")) {
					continue;
				}

				boolean duplicate = false;
				for (int j = 0; j < knownPlugins.size(); j++) {
					UnicorePlugin pl =
						(UnicorePlugin) knownPlugins.elementAt(j);
					// assume that packages are separated by '/'s
					String currentClassName = name.replace('/', '.');
					String loadedClassName =
						pl.getPluginClass().getName() + ".class";
					if (loadedClassName.equals(currentClassName)) {
						logger.warning(
							"Plugin class: "
								+ currentClassName
								+ " already loaded");
						duplicate = true;
						continue;
					}
				}
				if (!duplicate) {
					logger.info("Found plugin class: " + name);
					UnicorePlugin unicorePlugin =
						new UnicorePlugin(
							dirname + File.separator + plugin,
							name,
							urls,
							urlLoader);
					if (unicorePlugin.isTrusted()) {
						knownPlugins.add(unicorePlugin);
					}
				}
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  pluginPath  Description of Parameter
	 */
	public void scanPlugins(String pluginPath) {

		String jarDir = ResourceManager.getJarDirectory();

		// first look for plugins in current directory
		scanDirectory(jarDir);

		// first look for plugins in system default directory
		//        scanDirectory(jarDir + "/../plugin");

		//.. now take those from user default plugin dir
		scanDirectory(pluginPath);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  parent  Description of Parameter
	 */
	public void startPlugins(final Client parent) {
		for (int i = 0; i < knownPlugins.size(); i++) {
			final UnicorePlugin plugin =
				(UnicorePlugin) knownPlugins.elementAt(i);
			if (!(plugin.isTrusted())) {
				continue;
			}
			// problem: instantiation of plugins that e.g. overwrite
			// final methods will cause a VerifyError instead of an exception...
			try {
				logger.info("Loading plugin: " + plugin.getPluginName());
				Class pluginClass = plugin.getPluginClass();
				UnicorePlugable plugable =
					(UnicorePlugable) pluginClass.newInstance();
				plugable.startPlugin();
				pluginClasses.add(plugable);
				if (plugable instanceof TaskPlugable) {
					taskPluginClasses.add(plugable);
					TaskPlugable taskPlugable = (TaskPlugable) plugable;
					iconTable.put(
						taskPlugable.getContainerClass(),
						new ImageIcon(taskPlugable.getIconImage()));
				}
			} catch (Error e) {
				UserMessages.error(
					"Cannot start plugin <" + plugin.getPluginName() + ">.",
					e.getMessage());
				logger.log(Level.SEVERE, "", e);
			} catch (Exception e) {
				UserMessages.error(
					"Cannot start plugin <" + plugin.getPluginName() + ">.",
					e.getMessage());
				logger.log(Level.SEVERE, "", e);
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	public void stopPlugins() {
		for (int i = 0; i < pluginClasses.size(); i++) {
			UnicorePlugable plugable =
				(UnicorePlugable) pluginClasses.elementAt(i);
			plugable.stopPlugin();
		}
		knownPlugins.clear();
		pluginClasses.clear();
		taskPluginClasses.clear();
		iconTable.clear();
	}
}
