package com.pallas.unicore.client.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.io.StreamCorruptedException;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: PluginObjectInputStream.java,v 1.2 2003/06/30 06:56:36 rrate Exp $
 */
public class PluginObjectInputStream extends ObjectInputStream {

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.util.ResourceStrings");
	private PluginManager pManager = null;

	/**
	 *  Constructor for the PluginObjectInputStream object
	 *
	 *@param  in                            Description of Parameter
	 *@param  pManager                      Description of Parameter
	 *@exception  IOException               Description of Exception
	 *@exception  StreamCorruptedException  Description of Exception
	 */
	public PluginObjectInputStream(InputStream in, PluginManager pManager)
		throws IOException, StreamCorruptedException {
		super(in);
		this.pManager = pManager;
	}

	/**
	 *  Overrides the standard loader and checks all plugin loaders
	 *
	 *@param  clazz                       Description of Parameter
	 *@return                             Description of the Returned Value
	 *@exception  IOException             Description of Exception
	 *@exception  ClassNotFoundException  Description of Exception
	 */
	protected Class resolveClass(ObjectStreamClass clazz)
		throws IOException, ClassNotFoundException {
		// system classloader
		Class self = PluginObjectInputStream.class;
		ClassLoader loader = self.getClassLoader();
		Vector knownPlugins = pManager.getPlugins();
		if (!knownPlugins.isEmpty()) {
			/*
			 * iterate through the plugins and try to load classes with
			 * the associated classloader
			 */
			for (int i = 0; i < knownPlugins.size(); i++) {
				UnicorePlugin plugin =
					(UnicorePlugin) knownPlugins.elementAt(i);
				loader = plugin.getPluginClassloader();
				try {
					Class retClass =
						Class.forName(clazz.getName(), false, loader);
					return retClass;
				} catch (ClassNotFoundException cnfe) {
				}
			}
		} else {
			// system class loader
			return Class.forName(clazz.getName(), false, loader);
		}
		// finally  class not found
		throw new ClassNotFoundException(res.getString("NOT_FOUND"));
	}

}
