package com.pallas.unicore.client.util;

import javax.help.HelpSet;
import javax.swing.JMenuItem;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Abstract base class for all UNICOREpro plugins.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: UnicorePlugable.java,v 1.3 2003/06/30 06:56:35 rrate Exp $
 */
public abstract class UnicorePlugable {

	/**
	 *  Gets the current Client instance from the ResourceManager
	 *
	 *@return    reference to current Client frame
	 */
	protected Client getClient() {
		return ResourceManager.getCurrentInstance();
	}

	/**
	 *  Return a help set that will be plugged into the client's help. If no
	 *  help set is available return null.
	 *
	 *@return    a javax.help.HelpSet (or null)
	 */
	public HelpSet getHelpSet() {
		return null;
	}

	/**
	 *  Return the plugin info string, containing some information about
	 *  version, author, copyright, etc. This string will be displayed in the
	 *  client's about dialog.
	 *
	 *@return    a string containing some plugin information
	 */
	public abstract String getPluginInfo();

	/**
	 *  Get a menu item for the client's settings menu. Typical use for these
	 *  items is to popup a plugin specific configuration dialog. The client
	 *  does not attach a listener to this item so it is up to the plugin
	 *  developer to respond to this item. The method teturns null if no menu
	 *  item should be added to the clients settings menu.
	 *
	 *@return    a menu item that will be added to the client's settings menu
	 */
	public JMenuItem getSettingsItem() {
		return null;
	}

	/**
	 *  Called to initialize the plugin after the plugin has been loaded.
	 *
	 */
	public abstract void startPlugin();

	/**
	 *  Called before exiting the client. Here some finalization can be done.
	 */
	public abstract void stopPlugin();

}
