package com.pallas.unicore.connection;

import java.io.IOException;

import java.net.*;

import java.util.*;
import java.util.logging.*;

import org.unicore.User;
import org.unicore.Vsite;
import com.pallas.unicore.extensions.Usite;

/**
 *  Description of the Class
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    28. April 2003
 *@version    $Id: ConnectionManager.java,v 1.8 2003/06/27 12:54:43 thomas Exp $
 */
public class ConnectionManager {

    // used for logging
    private static Logger logger = Logger.getLogger(
            "com.pallas.unicore.connection");
    private Hashtable connections = new Hashtable();


    /**
     *  Constructor for the ConnectionManager object
     */
    public ConnectionManager() { }


    /**
     *  Adds a feature to the Connection attribute of the ConnectionManager
     *  object
     *
     *@param  connection  The feature to be added to the Connection attribute
     *@param  vsite       The feature to be added to the Connection attribute
     */
    public void addConnection(Connection connection, Vsite vsite) {
        connections.put(vsite, connection);
    }


    /**
     *  Adds a feature to the Connection attribute of the ConnectionManager
     *  object
     *
     *@param  connection  The feature to be added to the Connection attribute
     *@param  usite       The feature to be added to the Connection attribute
     */
    public void addConnection(Connection connection, Usite usite) {
        connections.put(usite, connection);
    }


    /**
     *  Description of the Method
     *
     *@param  vsite          Description of the Parameter
     *@param  user           Description of the Parameter
     *@return                Description of the Return Value
     *@exception  Exception  Description of the Exception
     */
    public Connection lookupConnection(Vsite vsite, User user)
             throws Exception {
        Connection connection = (Connection)connections.get(vsite);
        if(connection != null) {
            // if user has changed throw away connection
            if(!connection.getUser().equals(user)) {
                removeConnection(vsite);
                connection = null;
            }
        }
        if(connection == null) {
            // new connection
	    if(vsite.getType().getDescription().equals("GS")) {
		    connection = new GSConnection(vsite.getAddress(), user);
	    } else {
		    connection = new Connection(vsite.getAddress(), user);
	    }
            connections.put(vsite, connection);

            logger.info("New connection for vsite: " + vsite.getName() +
                    " user: " + user.getCertificate().getSubjectDN() + " xlogin: " +
                    user.getXlogin() + " project: " + user.getProject());
        }

        logger.info("Connection for vsite: " + vsite.getName() + " user: " +
                user.getCertificate().getSubjectDN() + " xlogin: " + user.getXlogin() +
                " project: " + user.getProject());

        // reset connection values
        connection.setDirectOutput(null);
        connection.setIsAscii(false);

        return connection;
    }


    /**
     *  Description of the Method
     *
     *@param  usite          Description of the Parameter
     *@param  user           Description of the Parameter
     *@return                Description of the Return Value
     *@exception  Exception  Description of the Exception
     */
    public Connection lookupConnection(Usite usite, User user)
             throws Exception {
        Connection connection = (Connection)connections.get(usite);
        if(connection != null) {
            // if user has changed throw away connection
            if(!connection.getUser().equals(user)) {
                removeConnection(connection);
                connection = null;
            }
        }
        if(connection == null) {
            URL url = usite.getAddress();
            connection = new Connection(
                    url.getProtocol() + "://" + url.getHost() +
                    ":" + url.getPort(), user);
            connections.put(usite, connection);
            logger.fine("New connection for usite: " + usite.getName() +
                    " user: " + user.getCertificate().getSubjectDN() + " xlogin: " +
                    user.getXlogin() + " project: " + user.getProject());
        }
        logger.fine("Connection for usite: " + usite.getName() + " user: " +
                user.getCertificate().getSubjectDN() + " xlogin: " + user.getXlogin() +
                " project: " + user.getProject());
        return connection;
    }


    /**
     *  Description of the Method
     *
     *@param  vsite  Description of the Parameter
     *@return        Description of the Return Value
     */
    public Object removeConnection(Vsite vsite) {
        logger.fine("Removing previous connection to " + vsite);
        Connection connection = (Connection)connections.get(vsite);
 		connections.remove(vsite);
        return connection;
    }


    /**
     *  Description of the Method
     */
    public void removeAllConnections() {
        Enumeration connEnum = connections.elements();
        while(connEnum.hasMoreElements()) {
            Connection connection = (Connection)connEnum.nextElement();
			logger.fine("Removing previous connection for user " + connection.getUser());
			try {
				 connection.close();
			}
			 catch(IOException e) {
				 logger.log(Level.INFO, "Could not close connection: " + connection, e);
			}
        }
        connections.clear();
    }


    /**
     *  Description of the Method
     *
     *@param  connection  Description of the Parameter
     */
    private void removeConnection(Connection connection) {
        logger.fine("Removing previous connection for user " + connection.getUser().getCertificate().getSubjectDN());
        try {
            connection.close();
        }
        catch(IOException e) {
            logger.log(Level.INFO, "Could not close connection: " + connection, e);
        }
        connections.remove(connection);
    }
}
