package com.pallas.unicore.connection;

import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.util.logging.Logger;

import javax.swing.JFrame;

import com.pallas.unicore.client.dialogs.LoginDialog;

/**
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ProxyManager.java,v 1.7 2003/07/10 12:25:57 thomas Exp $
 */
public class ProxyManager {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.connection");
	private static JFrame parent;
	private static PasswordAuthentication authentication;

	/**
	 *Constructor for the ProxyManager object
	 */
	private ProxyManager() {
	}

	/**
	 *  Sets the parent attribute of the ProxyManager class
	 *
	 *@param  _parent  The new parent value
	 */
	public static void setParent(JFrame _parent) {
		parent = _parent;
	}

	/**
	 *  Gets the authentication attribute of the ProxyManager class
	 *
	 *@return    The authentication value
	 */
	public static PasswordAuthentication getAuthentication() {
		return authentication;
	}

	/**
	 *  Description of the Method
	 */
	public static void noProxy() {
		System.setProperty("proxySet", "false");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  hostname  Description of the Parameter
	 *@param  port      Description of the Parameter
	 */
	public static void useProxy(String hostname, int port, String noProxy) {
		String portString = new Integer(port).toString();
		System.setProperty("proxySet", "true");
		System.setProperty("http.proxyHost", hostname);
		System.setProperty("http.proxyPort", portString);
		System.setProperty("http.nonProxyHosts", noProxy);
		org.apache.axis.AxisProperties.setProperty(
			"http.proxyHost",
			System.getProperty("http.proxyHost"));
		org.apache.axis.AxisProperties.setProperty(
			"http.proxyPort",
			System.getProperty("http.proxyPort"));
		org.apache.axis.AxisProperties.setProperty(
			"nonProxyHosts",
			System.getProperty("nonProxyHosts"));
		Authenticator.setDefault(new ProxyAuthenticator());
	}
	
	/**
	 *  Description of the Method
	 *
	 *@param  hostname  Description of the Parameter
	 *@param  port      Description of the Parameter
	 */
	public static void useHttpsProxy(String hostname, int port, String noProxy) {
		String portString = new Integer(port).toString();
		System.setProperty("proxySet", "true");
		System.setProperty("https.proxyHost", hostname);
		System.setProperty("https.proxyPort", portString);
		System.setProperty("https.nonProxyHosts", noProxy);
		org.apache.axis.AxisProperties.setProperty(
			"https.proxyHost",
			System.getProperty("https.proxyHost"));
		org.apache.axis.AxisProperties.setProperty(
			"https.proxyPort",
			System.getProperty("https.proxyPort"));
		org.apache.axis.AxisProperties.setProperty(
			"nonProxyHosts",
			System.getProperty("nonProxyHosts"));
		Authenticator.setDefault(new ProxyAuthenticator());
	}	

	private static class ProxyAuthenticator extends Authenticator {
		protected PasswordAuthentication getPasswordAuthentication() {
			logger.info("Getting proxy authorization...");
			LoginDialog proxyDialog =
				new LoginDialog(parent, "UNICOREpro: Proxy Authorization");
			proxyDialog.setVisible(true);
			if ((proxyDialog.getUsername() == null)
				|| (proxyDialog.getPassword() == null)) {
				return null;
			}
			authentication =
				new PasswordAuthentication(
					proxyDialog.getUsername(),
					proxyDialog.getPassword());

			// set up authentication for AXIS
			if (authentication != null) {
				org.apache.axis.AxisProperties.setProperty(
					"http.proxyUser",
					authentication.getUserName());
				org.apache.axis.AxisProperties.setProperty(
					"http.proxyPassword",
					new String(authentication.getPassword()));
				org.apache.axis.AxisProperties.setProperty(
					"https.proxyUser",
					authentication.getUserName());
				org.apache.axis.AxisProperties.setProperty(
					"https.proxyPassword",
					new String(authentication.getPassword()));					
			}

			return authentication;
		}

	}

}
