package com.pallas.unicore.container;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileTest;
import com.pallas.unicore.extensions.ReturnCodeTest;
import com.pallas.unicore.extensions.TimeTest;

/**
 *@author     Ralf Ratering
 *@created    7. Mai 2002
 *@version    $Id: ControlContainer.java,v 1.2 2003/06/30 06:57:25 rrate Exp $
 */
public abstract class ControlContainer extends GroupContainer {
	/**
	 *  Description of the Field
	 */
	protected FileTest fileTest;

	/**
	 *  Description of the Field
	 */
	protected ReturnCodeTest returnCodeTest;
	/**
	 *  Description of the Field
	 */
	protected TimeTest timeTest;

	/**
	 *  Constructor for the ControlContainer object
	 */
	public ControlContainer() {
		returnCodeTest = new ReturnCodeTest();
		fileTest = new FileTest();
		timeTest = new TimeTest();
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public ErrorSet checkContents() {

		ErrorSet err = super.checkContents();

		int enabled = 0;

		// we'll do this later on...
		returnCodeTest.removeAddedFlagsAndDependencies();

		if (returnCodeTest.isEnabled()) {
			enabled++;
			returnCodeTest.checkContents(err, this);
		}
		if (fileTest.isEnabled()) {
			enabled++;
			fileTest.checkContents(err, this);
		}
		if (timeTest.isEnabled()) {
			enabled++;
			timeTest.checkContents(err, this);
		}

		if (enabled == 0) {
			err.add(
				new UError(
					id,
					"All tests are disabled. Please choose a test."));
		} else if (enabled > 1) {
			err.add(
				new UError(
					id,
					"More than one test enabled. Please choose only one test."));
		}
		setErrors(err);
		return err;
	}

	/**
	 *  Gets the fileTest attribute of the IfContainer object
	 *
	 *@return    The fileTest value
	 */
	public FileTest getFileTest() {
		return fileTest;
	}

	/**
	 *  Gets the fileTest attribute of the IfContainer object
	 *
	 *@return    The fileTest value
	 */
	public ReturnCodeTest getReturnCodeTest() {
		return returnCodeTest;
	}

	/**
	 *  Gets the timeTest attribute of the IfContainer object
	 *
	 *@return    The timeTest value
	 */
	public TimeTest getTimeTest() {
		return timeTest;
	}

	/**
	 *  Sets the fileTest attribute of the IfContainer object
	 *
	 *@param  fileTest  The new fileTest value
	 */
	public void setFileTest(FileTest fileTest) {
		this.fileTest = fileTest;
	}

	/**
	 *  Sets the fileTest attribute of the IfContainer object
	 *
	 *@param  returnCodeTest  The new returnCodeTest value
	 */
	public void setReturnCodeTest(ReturnCodeTest returnCodeTest) {
		this.returnCodeTest = returnCodeTest;
	}

	/**
	 *  Sets the timeTest attribute of the IfContainer object
	 *
	 *@param  timeTest  The new timeTest value
	 */
	public void setTimeTest(TimeTest timeTest) {
		this.timeTest = timeTest;
	}

}
