package com.pallas.unicore.container;

import java.io.Serializable;

import org.unicore.ajo.Dependency;

/**
 *  Class representing a dependency between two ActionContainers.
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: DependencyContainer.java,v 1.2 2003/06/30 06:57:24 rrate Exp $
 */
public class DependencyContainer implements Serializable {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "PREDECESSOR", "pred" }, {
			"SUCCESSOR", "succ" }
	};

	final static long serialVersionUID = -1437076470626701799L;

	private ActionContainer pred;
	private ActionContainer succ;

	/**
	 *  Constructor for the DependencyContainer object
	 */
	public DependencyContainer() {
		pred = null;
		succ = null;
	}

	/**
	 *  Constructor for the DependencyContainer object
	 *
	 *@param  pred  Description of Parameter
	 *@param  succ  Description of Parameter
	 */
	public DependencyContainer(ActionContainer pred, ActionContainer succ) {
		this.pred = pred;
		this.succ = succ;
	}

	/**
	 *  Gets the Dependency attribute of the DependencyContainer object
	 *
	 *@return    The Dependency value
	 */
	public Dependency getDependency() {
		return new Dependency(pred.getActionGroup(), succ.getActionGroup());
	}

	/**
	 *  Gets the Pred attribute of the DependencyContainer object
	 *
	 *@return    The Pred value
	 */
	public ActionContainer getPred() {
		return pred;
	}

	/**
	 *  Gets the Succ attribute of the DependencyContainer object
	 *
	 *@return    The Succ value
	 */
	public ActionContainer getSucc() {
		return succ;
	}

	/**
	 *  Sets the Pred attribute of the DependencyContainer object
	 *
	 *@param  pred  The new Pred value
	 */
	public void setPred(ActionContainer pred) {
		this.pred = pred;
	}

	/**
	 *  Sets the Succ attribute of the DependencyContainer object
	 *
	 *@param  succ  The new Succ value
	 */
	public void setSucc(ActionContainer succ) {
		this.succ = succ;
	}

	/**
	 *  Overwrites toString() method from ActionContainer to display a meaningful String for the
	 *  dependency
	 *
	 *@return    Description of the Returned Value
	 */
	public String toString() {
		// pred and succ have their own toString methods, so we can use them
		return new String(pred + " -> " + succ);
	}
}
