package com.pallas.unicore.container;

import java.util.Enumeration;
import java.util.logging.Level;

import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.FileCheck;
import org.unicore.ajo.ProbeTime;
import org.unicore.ajo.RepeatGroup;
import org.unicore.ajo.TestSuccess;
import org.unicore.outcome.FileCheck_Outcome;
import org.unicore.outcome.ProbeTime_Outcome;
import org.unicore.outcome.TestSuccess_Outcome;

import com.pallas.unicore.extensions.ReturnCodeTest;
import com.pallas.unicore.extensions.TestType;
import com.pallas.unicore.extensions.TimeTest;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@created    7. Mai 2002, 22.Mai 2002
 *@version    $Id: DoRepeatContainer.java,v 1.2 2003/06/30 06:57:24 rrate Exp $
 */
public class DoRepeatContainer extends ControlContainer {

	final static long serialVersionUID = -9063565457856815042L;

	/**
	 *  Description of the Method
	 */
	public void buildActionGroup() {
		// build the action group
		RepeatGroup repeatGroup =
			new RepeatGroup(
				getName()
					+ "_RepeatGroup_"
					+ ResourceManager.getNextObjectIdentifier());

		// until mode, while won't work, because
		// decision files do not exist at loop start
		repeatGroup.setUntil();

		buildActionGroup(repeatGroup);

		// get last action for dependency
		AbstractAction lastAction = null;
		Enumeration actions = repeatGroup.getActions();
		while (actions.hasMoreElements()) {
			lastAction = (AbstractAction) actions.nextElement();
		}

		if (returnCodeTest.isEnabled()) {
			ActionContainer testContainer =
				getReturnCodeTest().getTestContainer();
			ActionGroup testGroup = testContainer.getActionGroup();
			TestType type = returnCodeTest.getType();
			if (type.equals(ReturnCodeTest.SUCCESSFUL)
				|| type.equals(ReturnCodeTest.NOT_SUCCESSFUL)) {
				TestSuccess testSuccess =
					new TestSuccess(
						getName()
							+ "_TestSuccess_"
							+ ResourceManager.getNextObjectIdentifier());
				testSuccess.setTarget(testGroup);

				repeatGroup.add(testSuccess);
				try {
					repeatGroup.addDependency(
						new Dependency(testGroup, testSuccess));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}

				repeatGroup.setTestTask(testSuccess);
				if (type.equals(ReturnCodeTest.SUCCESSFUL)) {
					repeatGroup.setTest(TestSuccess_Outcome.SUCCESSFUL);
				} else {
					repeatGroup.setTest(TestSuccess_Outcome.NOT_SUCCESSFUL);
				}
			} else {
				repeatGroup.setTest(
					new Integer(returnCodeTest.getValue()).toString());
				TaskContainer testTask = (TaskContainer) testContainer;
				repeatGroup.setTestTask(testTask.getExecuteTask());
				// add ignore failure here explicitly, because
				// repeatGroup will fail, if not.
				//testTask.getExecuteTask().setPropagateSuccessfulIfNotSuccessful(true);
			}
		} else if (fileTest.isEnabled()) {
			FileCheck fileCheck = fileTest.getFileCheckTask();
			repeatGroup.setTestTask(fileCheck);
			repeatGroup.setTest(FileCheck_Outcome.SATISFIED);
			repeatGroup.add(fileCheck);
			if (lastAction != null) {
				try {
					repeatGroup.addDependency(
						new Dependency(lastAction, fileCheck));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
		} else if (timeTest.isEnabled()) {
			ProbeTime probeTime =
				new ProbeTime(
					getName()
						+ "_ProbeTime_"
						+ ResourceManager.getNextObjectIdentifier());
			probeTime.setTime(getTimeTest().getDate());
			repeatGroup.setTestTask(probeTime);
			if (timeTest.getType().equals(TimeTest.AFTER)) {
				repeatGroup.setTest(ProbeTime_Outcome.LATER);
			}
			repeatGroup.add(probeTime);
			if (lastAction != null) {
				try {
					repeatGroup.addDependency(
						new Dependency(lastAction, probeTime));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
		}

	}
}
