package com.pallas.unicore.container;

import java.io.Serializable;

import org.unicore.sets.OptionSet;

import com.pallas.unicore.container.errorspec.ErrorSet;

/**
 *  Implements a container class for executing commands. This class knows about Context's
 *  environment variables, stdin/stdout/stderr Portfolios.
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: ExecuteContainer.java,v 1.2 2003/06/30 06:57:23 rrate Exp $
 */
public abstract class ExecuteContainer
	extends TaskContainer
	implements Serializable {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "TASKENVIRONMENT", "environment" }, {
			"TASKCOMMANDLINE", "commandLine" }, {
			"EXECUTEVERBOSE", "verbose" }, {
			"SHOWVERSION", "version" }, {
			"REDIRECTSTDOUTW", "redirectStdout" }, {
			"REDIRECTSTDERR", "redirectStderr" }, {
			"REDIRECTSTDIN", "redirectStdin" }, {
			"STDOUTPATH", "stdoutPath" }, {
			"STDERRPATH", "stderrPath" }
	};

	final static long serialVersionUID = 4642089981942904957L;
	/**
	 *  Description of the Field
	 */
	protected String commandLine;

	/**
	 *  Description of the Field
	 */
	protected OptionSet environment;

	/**
	 *  Description of the Field
	 */
	protected String stdoutPath, stderrPath, stdinPath;

	/**
	 *  Description of the Field
	 */
	protected boolean verbose = false,
		version = false,
		redirectStdout,
		redirectStdin,
		redirectStderr;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent container
	 */
	public ExecuteContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Self check the correctness of the values relevant for this container
	 *
	 *@return    Description of the Returned Value
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		// check contexts
		// check environment vars
		// check cmd line
		// check stdout

		// check verbose
		// check version
		setErrors(err);
		return err;
	}

	/**
	 *  Gets the CommandLine attribute of the ExecuteContainer object
	 *
	 *@return    The CommandLine value
	 */
	public String getCommandLine() {
		return commandLine;
	}

	/**
	 *  Gets the Env attribute of the ExecuteContainer object
	 *
	 *@return    The Env value
	 */
	public OptionSet getEnv() {
		return environment;
	}

	/**
	 *  Gets the RedirectStderr attribute of the ExecuteContainer object
	 *
	 *@return    The RedirectStderr value
	 */
	public boolean getRedirectStderr() {
		return redirectStderr;
	}

	/**
	 *  Gets the RedirectStdout attribute of the ExecuteContainer object
	 *
	 *@return    The RedirectStdout value
	 */
	public boolean getRedirectStdout() {
		return redirectStdout;
	}

	/**
	 *  Gets the Stderr attribute of the ExecuteContainer object
	 *
	 *@return    The Stderr value
	 */
	public String getStderr() {
		return stderrPath;
	}

	/**
	 *  Gets the Stdout attribute of the ExecuteContainer object
	 *
	 *@return    The Stdout value
	 */
	public String getStdout() {
		return stdoutPath;
	}

	/**
	 *  Gets the VerboseOn attribute of the ExecuteContainer object
	 *
	 *@return    The VerboseOn value
	 */
	public boolean isVerboseOn() {
		return verbose;
	}

	/**
	 *  Gets the VersionOn attribute of the ExecuteContainer object
	 *
	 *@return    The VersionOn value
	 */
	public boolean isVersionOn() {
		return version;
	}

	/**
	 *  Get commandline arguments
	 *
	 *@param  commandLine  The new CommandLine value
	 */
	public void setCommandLine(String commandLine) {
		this.commandLine = commandLine;
	}

	/**
	 *  Set the environment for this task
	 *
	 *@param  environment  The new Env value
	 */
	public void setEnv(OptionSet environment) {
		this.environment = environment;
	}

	/**
	 *  Sets the RedirectStderr attribute of the ExecuteContainer object
	 *
	 *@param  what  The new RedirectStderr value
	 */
	public void setRedirectStderr(boolean what) {
		this.redirectStderr = what;
	}

	/**
	 *  Sets the RedirectStdout attribute of the ExecuteContainer object
	 *
	 *@param  what  The new RedirectStdout value
	 */
	public void setRedirectStdout(boolean what) {
		this.redirectStdout = what;
	}

	/**
	 *  Sets the Stderr attribute of the ExecuteContainer object
	 *
	 *@param  stderr  The new Stderr value
	 */
	public void setStderr(String stderr) {
		this.stderrPath = stderr;
	}

	/**
	 *  Sets the Stdout attribute of the ExecuteContainer object
	 *
	 *@param  stdout  The new Stdout value
	 */
	public void setStdout(String stdout) {
		this.stdoutPath = stdout;
	}

	/**
	 *  Sets the Verbose attribute of the ExecuteContainer object
	 *
	 *@param  verbose  The new Verbose value
	 */
	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	/**
	 *  Sets the Version attribute of the ExecuteContainer object
	 *
	 *@param  version  The new Version value
	 */
	public void setVersion(boolean version) {
		this.version = version;
	}
}
