package com.pallas.unicore.container;
import java.util.ResourceBundle;

import org.unicore.ajo.ActionGroup;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.ExportFactory;

/**
 *  Container for all export tasks that export files from one uspace to
 *  different locations
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FileExportContainer.java,v 1.2 2003/06/30 06:57:22 rrate Exp $
 */
public class FileExportContainer extends FileContainer {

	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -3891429971924267668L;

	/**
	 *  Constructor builds file container as child of job container
	 *
	 *@param  parentContainer  parent job group
	 */
	public FileExportContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());

		// export files
		ExportFactory exportFactory =
			new ExportFactory(getFileExports(), actionGroup, this);
		exportFactory.build();

		// copy identifier from action group to container to to have a unique id for the njs.
		syncIdentifiers();
	}

	/**
	 *  Add container specific errors to those of super classes
	 *
	 *@return    ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet errorSet = super.checkContents();

		if (fileExports == null || fileExports.length == 0) {
			errorSet.add(
				new UError(getIdentifier(), res.getString("NO_EXPORTS")));
		}

		setErrors(errorSet);
		return errorSet;
	}
}
