package com.pallas.unicore.container;

import java.util.ResourceBundle;

import org.unicore.ajo.ActionGroup;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.ImportFactory;

/**
 *  Container for all import tasks that import files from different locations
 *  into one uspace.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FileImportContainer.java,v 1.2 2003/06/30 06:57:22 rrate Exp $
 */
public class FileImportContainer extends FileContainer {

	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = 6149750275075422934L;

	/**
	 *  Constructor builds file container as child of job container
	 *
	 *@param  parentContainer  parent job group
	 */
	public FileImportContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());

		// import files
		ImportFactory importFactory =
			new ImportFactory(getFileImports(), actionGroup, this);
		importFactory.build();

		// copy identifier from action group to container to to have a unique id for the njs.
		syncIdentifiers();
	}

	/**
	 *  Add container specific errors to those of super classes
	 *
	 *@return    ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet errorSet = super.checkContents();

		if (fileImports == null || fileImports.length == 0) {
			errorSet.add(
				new UError(getIdentifier(), res.getString("NO_IMPORTS")));
		}

		setErrors(errorSet);
		return errorSet;
	}
}
