package com.pallas.unicore.container;

import java.util.ResourceBundle;

import org.unicore.ajo.ActionGroup;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.FileSystemOperationFactory;

/**
 *  Container for all tasks that copy, move or delete files, or make directories
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FileOperationContainer.java,v 1.2 2003/06/30 06:57:21 rrate Exp $
 */
public class FileOperationContainer extends FileContainer {

	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -7752008158209771600L;

	/**
	 *  Constructor builds file container as child of job container
	 *
	 *@param  parentContainer  parent job group
	 */
	public FileOperationContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());

		FileSystemOperationFactory factory =
			new FileSystemOperationFactory(
				fileSystemOperations,
				actionGroup,
				this);
		factory.build();

		// copy identifier from action group to container to to have a unique id for the njs.
		syncIdentifiers();
	}

	/**
	 *  Add container specific errors to those of super classes
	 *
	 *@return    ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		if (fileSystemOperations == null || fileSystemOperations.length == 0) {
			err.add(
				new UError(getIdentifier(), res.getString("NO_OPERATIONS")));
		} else {
			for (int i = 0; i < fileSystemOperations.length; i++) {
				if (!fileSystemOperations[i].isComplete()) {
					err.add(
						new UError(
							id,
							res.getString("INCOMPLETE_FILE_OPERATION")
								+ fileSystemOperations[i]
								+ res.getString("AT_ROW")
								+ (i + 1)));
					// skip other checks if incomplete...
					continue;
				}
				if (fileSystemOperations[i]
					.getStorageItem()
					.equals(FileStorage.NSPACE_STRING)
					&& fileSystemOperations[i].getOperation().equals(
						FileSystemOperation.MAKEDIR)) {
					err.add(
						new UError(
							getIdentifier(),
							res.getString("LOCAL_MKDIR")));
				}
			}
		}

		setErrors(err);
		return err;
	}

}
