package com.pallas.unicore.container;

import java.util.ResourceBundle;
import java.util.Vector;

import org.unicore.ajo.ActionGroup;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.TransferFactory;

/**
 *  Container for all tasks that transfer files from one uspace to another
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FileTransferContainer.java,v 1.2 2003/06/30 06:57:21 rrate Exp $
 */
public class FileTransferContainer extends FileContainer {

	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -7293842589398013001L;

	private Vector additionalDependencies;

	/**
	 *  Constructor builds file container as child of job container
	 *
	 *@param  parentContainer  parent job group
	 */
	public FileTransferContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		// import files
		TransferFactory transferFactory =
			new TransferFactory(getFileTransfers(), actionGroup, this);
		transferFactory.build();

		// copy identifier from action group to container to to have a unique id for the njs.
		syncIdentifiers();
	}

	/**
	 *  Add container specific errors to those of super classes
	 *
	 *@return    ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet errorSet = super.checkContents();

		// remove previous additional dep
		if (additionalDependencies != null) {
			for (int i = 0; i < additionalDependencies.size(); i++) {
				getParentContainer().removeDependency(
					(DependencyContainer) additionalDependencies.elementAt(i));
			}
		}
		additionalDependencies = new Vector();

		if (fileTransfers == null || fileTransfers.length == 0) {
			errorSet.add(
				new UError(getIdentifier(), res.getString("NO_TRANSFERS")));
		}
		// check if destination jobs are existing
		else {
			for (int i = 0; i < fileTransfers.length; i++) {
				JobContainer destJob = fileTransfers[i].getDestinationJob();
				if (!fileTransfers[i].isComplete()) {
					errorSet.add(
						new UError(
							id,
							res.getString("INCOMPLETE_FILE_TRANSFER")
								+ fileTransfers[i]
								+ res.getString("AT_ROW")
								+ (i + 1)));
					// skip other checks if incomplete...
				} else if (!jobExists(destJob)) {
					errorSet.add(
						new UError(
							getIdentifier(),
							res.getString("DESTJOB_MISSING") + destJob));
				}
				// we'll do this later on...
				/*else if(!isPredecessorOf(destJob)) {
				    DependencyContainer dep = new DependencyContainer(this, destJob);
				    additionalDependencies.add(dep);
				    getParentContainer().addDependency(dep);
				}*/
			}
		}
		setErrors(errorSet);
		return errorSet;
	}

	/**
	 *  Gets the jobExisting attribute of the FileTransferContainer object
	 *
	 *@param  job  Description of the Parameter
	 *@return      The jobExisting value
	 */
	private boolean jobExists(JobContainer job) {
		return findTask(findRootContainer(this), job);
	}

}
