package com.pallas.unicore.container;

import java.util.ResourceBundle;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Time;
import org.unicore.ajo.WaitUntil;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.extensions.HoldJobTest;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@author     Sven Haubold
 *@created    7. Mai 2002
 *@version    $Id: HoldJobContainer.java,v 1.2 2003/06/30 06:57:20 rrate Exp $
 */
public class HoldJobContainer extends TaskContainer {

	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");

	final static long serialVersionUID = -9063565457856815042L;

	private HoldJobTest test = new HoldJobTest();

	private WaitUntil waitUntil;

	/**
	 *  Constructor for the HoldJobContainer object
	 *
	 *@param  parentContainer  Description of the Parameter
	 */
	public HoldJobContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Description of the Method
	 */
	public void buildActionGroup() {

		// build the action group
		actionGroup =
			new ActionGroup(
				getName()
					+ "_HOLD_JOB_"
					+ ResourceManager.getNextObjectIdentifier());

		// synchronize identifiers to find corresponding outcomes for
		// container in jmc
		setIdentifier(actionGroup.getId());

		long time = test.getDate().getTime();
		if (!test.isEnabled()) {
			time = 0;
		} else if (test.getType().equals(HoldJobTest.FOREVER)) {
			time = Long.MAX_VALUE;
		}
		waitUntil =
			new WaitUntil(
				getName()
					+ "_WAIT_UNTIL_"
					+ ResourceManager.getNextObjectIdentifier());
		// set ignore failure flag
		// This makes it possible to resume job execution by sending a Control.ABORT Action
		// to the WaitUntil task
		waitUntil.setPropagateSuccessfulIfNotSuccessful(true);
		waitUntil.setTime(new Time(time));
		actionGroup.add(waitUntil);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public ErrorSet checkContents() {

		ErrorSet err = super.checkContents();

		test.checkContents(err, this);

		setErrors(err);
		return err;
	}

	/**
	 *  Gets the timeTest attribute of the HoldJobContainer object
	 *
	 *@return    The timeTest value
	 */
	public HoldJobTest getHoldJobTest() {
		return test;
	}

	/**
	 *  WaitUntil task will be aborted from jmc tree, if the user wants to
	 *  resume job execution.
	 *
	 *@return    The waitUntil value
	 */
	public WaitUntil getWaitUntil() {
		return waitUntil;
	}

	/**
	 *  Sets the timeTest attribute of the HoldJobContainer object
	 *
	 *@param  test  The new holJobTest value
	 */
	public void setHolJobTest(HoldJobTest test) {
		this.test = test;
	}
}
