package com.pallas.unicore.container.errorspec;

import java.util.Vector;

/**
 *  A type-safe container class for <tt>UError</tt> s.
 *
 *@author     Michel Drescher, pallas GmbH
 *@created    17. September 2001
 *@version    $Id: ErrorSet.java,v 1.2 2003/06/30 07:00:21 rrate Exp $
 */
public class ErrorSet {

	/**
	 *  Description of the Field
	 */
	protected Vector set;

	/**
	 *  Creates a new ErrorSet
	 */
	public ErrorSet() {
		set = new Vector();
	}

	/**
	 *  Creates a new ErrorSet
	 *
	 *@param  first  an initial <code>UError</code> entry
	 */
	public ErrorSet(UError first) {
		set = new Vector();
		add(first);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  errorSet  Description of Parameter
	 */
	public void add(ErrorSet errorSet) {
		ErrorSetEnumeration enum = errorSet.elements();
		while (enum.hasMoreElements()) {
			add(enum.nextElement());
		}
	}

	/**
	 *  Add a <tt>UError</tt> to the set.
	 *
	 *@param  thing  The <tt>UError</tt> to be added.
	 */
	public void add(UError thing) {
		set.addElement(thing);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public ErrorSet cloneSet() {
		// Do not use clone as we do not return an Object
		// so compiler will complain (Object sits poorly with rest)
		ErrorSet result = new ErrorSet();
		result.set = (Vector) set.clone();
		return result;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public boolean containsError() {
		return set.size() > 0;
	}

	/**
	 *  Create and return an enumeration over the elements in the <tt>ErrorSet</tt>
	 *
	 *@return    The enumeration.
	 */
	public ErrorSetEnumeration elements() {
		return new ErrorSetEnumeration(this);
	}

	/**
	 *  Returns the element at position <code>pos</code> .
	 *
	 *@param  pos  the position (0 to this.size()-1) in this set
	 *@return      the <code>UError</code> at the specified position.
	 */
	public UError getAt(int pos) {
		return (UError) set.elementAt(pos);
	}

	/**
	 *  Returns the amount of <code>UErrors</code> contained in this set
	 *
	 *@return    the amount of elements as <code>int</code>
	 */
	public int getSize() {
		return set.size();
	}

	/**
	 *  Test if a <tt>UError</tt> is in the set
	 *
	 *@param  thing  the UError instance to be tested
	 *@return        Return true iff the <tt>UError</tt> is in the set
	 */
	public boolean isIn(UError thing) {
		return set.contains(thing);
	}

	/**
	 *  Remove the <tt>UError</tt> from the set.
	 *
	 *@param  thing  The <tt>UError</tt> to be removed.
	 */
	public void remove(UError thing) {
		set.removeElement(thing);
	}

	/**
	 *  Replaces <code>oldError</code> with <code>newError</code> .
	 *
	 *@param  oldError  The UError to be replaced.
	 *@param  newError  The UError instance to be inserted.
	 *@return           True, If the operation succeedes, false, if oldError is not in this set, or if
	 *      oldError or newError are <code>null</code> .
	 */
	public boolean replace(UError oldError, UError newError) {
		if ((oldError == null) || (newError == null)) {
			return false;
		}
		int i = set.indexOf(oldError);
		if (i < 0) {
			return false;
		}
		try {
			set.removeElementAt(i);
			set.insertElementAt(newError, i);
		} catch (ArrayIndexOutOfBoundsException aioob) {
			return false;
		}
		return true;
	}

	/**
	 *  Build String out of set elements separated by newlines
	 *
	 *@return    new Error String
	 */
	public String toString() {
		String message = "";
		for (int i = 0; i < getSize(); i++) {
			message += getAt(i) + "\n";
		}
		return message;
	}

}
