package com.pallas.unicore.extensions;

import java.util.Vector;
import java.util.logging.Level;

import org.unicore.Vsite;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.CopyFile;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.PutPortfolio;
import org.unicore.resources.Spool;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Class builds AJO ActionGroup from an input Array of FileExports
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: ExportFactory.java,v 1.3 2003/06/30 06:58:23 rrate Exp $
 */
public class ExportFactory extends FileOperationFactory {

	/**
	 *  Constructor
	 *
	 *@param  fileOpsArray  input FileExports
	 *@param  actionGroup   input ActionGroup that will be filled with export
	 *      tasks
	 *@param  container     Description of Parameter
	 */
	public ExportFactory(
		FileExport[] fileOpsArray,
		ActionGroup actionGroup,
		TaskContainer container) {
		super(fileOpsArray, actionGroup, container);
	}

	/**
	 *  Go, build all tasks and put them into the ActionGroup
	 */
	public void build() {

		// add additional exports to Vector
		FileExport[] additionalExports = container.getAdditionalExports();
		for (int i = 0; i < additionalExports.length; i++) {
			fileOps.add(additionalExports[i]);
		}

		// export to NSpace
		buildLocalExportTask();

		// collect different storage resources
		Vector storageLocations = new Vector();
		for (int i = 0; i < fileOps.size(); i++) {
			FileExport fileExport = (FileExport) fileOps.get(i);

			// skip local imports
			if (fileExport.isLocal()) {
				continue;
			}
			Object storageItem = fileExport.getStorageItem();
			if (!storageLocations.contains(storageItem)) {
				storageLocations.add(storageItem);
			}
		}
		logger.fine("Found " + storageLocations.size() + " storage locations");

		// loop over different storage locations and build a task for each one
		for (int i = 0; i < storageLocations.size(); i++) {
			buildExportTask((Storage) storageLocations.get(i));
		}
	}

	/**
	 *  Build an export task for one specific storage location in Xspace
	 *
	 *@param  storage  one specific storage location
	 */

	/*
	 *  private void buildExportTask(Storage storage) {
	 *  / build vector of files for a single storage
	 *  Vector storageExports = new Vector();
	 *  Vector nonOverwriteExports = new Vector();
	 *  for (int i = 0; i < fileOps.size(); i++) {
	 *  FileExport fileExport = (FileExport)fileOps.get(i);
	 *  if (fileExport.getStorageItem().equals(storage)) {
	 *  if (fileExport.getOverwritePolicy()) {
	 *  storageExports.add(fileExport);
	 *  } else {
	 *  nonOverwriteExports.add(fileExport);
	 *  }
	 *  }
	 *  }
	 *  logger.fine("Found " + storageExports.size() + " overwrite exports");
	 *  logger.fine(
	 *  "Found " + nonOverwriteExports.size() +
	 *  " non-overwrite exports");
	 *  / exports that overwrite
	 *  constructTask(storageExports, storage, true);
	 *  / non-overwrite exports
	 *  constructTask(nonOverwriteExports, storage, false);
	 *  }
	 */
	/**
	 *  Build an export task for one specific storage location in Xspace
	 *
	 *@param  storage  one specific storage location
	 */
	private void buildExportTask(Storage storage) {
		Vector copyExports = new Vector();
		for (int i = 0; i < fileOps.size(); i++) {
			FileExport fileExport = (FileExport) fileOps.get(i);
			if (fileExport.getStorageItem().equals(storage)) {

				// beginning with NJS 4.0.0 beta10 all exports to Xspace are handled by file copies
				copyExports.add(fileExport);

			}
		}
		constructCopyTask(copyExports, storage);
		logger.fine(
			"Found " + copyExports.size() + " exports handled by copies");
	}

	/**
	 *  This routine copies Portfolios to be exported back to the users
	 *  workstation to the Spool storage resource. The files can be later
	 *  retrieved either automatically or interactively when fetching the
	 *  outcome of the job.
	 */
	private void buildLocalExportTask() {
		Vector spooledFiles = new Vector();

		// for each Nspace export create a MakePortfolio and a PutPortfolio to place them in the Spool area
		for (int i = 0; i < fileOps.size(); i++) {
			FileExport fileExport = (FileExport) fileOps.get(i);
			if (fileExport.isLocal()) {
				MakePortfolio spoolPortfolio =
					new MakePortfolio(createActionName("MP_SPOOL"));
				spooledFiles.add(fileExport);
				spoolPortfolio.addFile(fileExport.getSourceName());
				logger.fine("Nspace export: " + fileExport.getSourceName());
				fileExport.setSpoolPortfolio(spoolPortfolio.getPortfolio());
				PutPortfolio spoolTask =
					new PutPortfolio(createActionName("SP"));
				spoolTask.setSource(spoolPortfolio.getPortfolio());

				//      spoolTask.setTarget(spoolPortfolio.getPortfolio().getId());
				spoolTask.setOverwrite(fileExport.getOverwritePolicy());
				Vsite vsite =
					fileExport
						.getContainer()
						.getParentJobContainer()
						.getVsite();
				NamedResourceSet siteResources =
					ResourceManager.getResourceSet(vsite);
				if (siteResources.isSiteDefault()) {
					logger.severe(
						"Cannot obtain site specific ResourceSet from Vsite= "
							+ vsite.getName());
					return;
				} else {
					logger.info(
						"Found site specific ResourceSet for Vsite= "
							+ vsite.getName());
				}
				Spool spoolResource =
					(Spool) siteResources.findResource(new Spool());
				if (spoolResource == null) {
					logger.severe(
						"Cannot obtain spool space resource from Vsite= "
							+ vsite.getName());
					return;
				} else {
					logger.info(
						"Found spool space resource for Vsite= "
							+ vsite.getName());
				}
				spoolTask.setResources(new ResourceSet(spoolResource));
				fileExport.setSpoolTask(spoolTask);

				// delete portfolio in case of loops
				DeletePortfolio deletePortfolio =
					new DeletePortfolio(
						"_DELETEPORTFOLIO_"
							+ ResourceManager.getNextObjectIdentifier());
				deletePortfolio.setPortfolio(
					spoolPortfolio.getPortfolio().getId());

				// build a sub ActionGroup to remember it in FileExport object
				ActionGroup exportGroup =
					new ActionGroup(createActionName("EXPORTGROUP"));
				exportGroup.add(spoolPortfolio);
				exportGroup.add(spoolTask);
				exportGroup.add(deletePortfolio);
				try {
					exportGroup.addDependency(
						new Dependency(spoolPortfolio, spoolTask));
					exportGroup.addDependency(
						new Dependency(spoolTask, deletePortfolio));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}

				fileExport.setAbstractAction(exportGroup);
				actionGroup.add(exportGroup);
			}
		}
		container.setSpooledFiles(spooledFiles);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  exports  Description of Parameter
	 *@param  storage  Description of Parameter
	 */
	private void constructCopyTask(Vector exports, Storage storage) {

		// nothing to do ?
		if (exports.size() == 0) {
			return;
		}

		// build resource sets for different tasks
		USpace uspaceHome =
			ResourceManager.getResourceSet(container.getVsite()).getUspace();
		if (uspaceHome == null) {
			return;
		}

		// loop over exports and build one copy per export;
		// reason: provides the best outcome details to the user
		// (previous version gathered exports to the same destination dir in a portfolio
		// and a corresponding export task)
		for (int i = 0; i < exports.size(); i++) {
			FileExport fileExport = (FileExport) exports.get(i);

			// create enclosing action group for details panel
			ActionGroup exportGroup =
				new ActionGroup(createActionName("EXPORTGROUP_" + fileExport));
			logger.fine(
				"New ActionGroup: "
					+ exportGroup.getId().getName()
					+ " ID: "
					+ exportGroup.getId());

			// source (including wildcards) and destination are taken from export
			String sourceName = fileExport.getSourceName();
			String destName = fileExport.getDestinationName();

			// resources
			ResourceSet storageSet = new ResourceSet(uspaceHome);
			storageSet.add(storage);

			// set overwrite flag
			boolean overwrite = fileExport.getOverwritePolicy();

			// copy from Uspace to Xspace
			boolean copy_from_uspace = true;

			// >>> AJO4.0
			CopyFile copyFile =
				new CopyFile(
					createActionName("EXPORT_BY_COPYFILE_" + sourceName) + i,
					storageSet,
					sourceName,
					destName,
					overwrite,
					copy_from_uspace);
			logger.fine(
				"New CopyFile: "
					+ copyFile.getId().getName()
					+ " ID: "
					+ copyFile.getId());
			exportGroup.add(copyFile);
			actionGroup.add(exportGroup);

			// remember export actions
			fileExport.setAbstractAction(exportGroup);
		}

	}
}
