package com.pallas.unicore.extensions;

/**
 *  Abstract class representing a file transfer containing a specified storage
 *  location
 *
 *@author     Ralf Ratering
 *@created    13. September 2001
 *@version    $Id: FileStorage.java,v 1.2 2003/06/30 06:58:21 rrate Exp $
 */
public abstract class FileStorage extends FileOperation {

	/**
	 *  Local constant
	 */
	public final static String NSPACE_STRING = "Local";
	final static long serialVersionUID = 23669528063295652L;

	/**
	 *  ASCII or binary mode
	 */
	protected boolean isAscii = false;

	/**
	 *  overwrite policy
	 */
	protected boolean overwrite = true;

	/**
	 *  the storage item may be a String or a Storage resource
	 */
	protected Object storageItem;

	/**
	 *  Constructor including storage item
	 *
	 *@param  storageItem      defining storage location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 */
	public FileStorage(
		Object storageItem,
		String sourceName,
		String destinationName) {
		super(sourceName, destinationName);
		this.storageItem = storageItem;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of Parameter
	 *@return         Description of the Returned Value
	 */
	public boolean equals(Object object) {
		FileStorage storage = (FileStorage) object;
		if (storageItem == null) {
			return super.equals(storage)
				&& (storageItem == storage.getStorageItem());
		} else {
			return super.equals(storage)
				&& storageItem.equals(storage.getStorageItem());
		}
	}

	/**
	 *  Gets the ASCII attribute of the FileExport object
	 *
	 *@return    The OverwritePolicy value
	 */
	public boolean getFileMode() {
		return isAscii;
	}

	/**
	 *  Gets the OverwritePolicy attribute of the FileExport object
	 *
	 *@return    The OverwritePolicy value
	 */
	public boolean getOverwritePolicy() {
		return overwrite;
	}

	/**
	 *@return    storage item as object
	 */
	public Object getStorageItem() {
		return storageItem;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public int hashCode() {
		int result = 17;
		result = 37 * result + super.hashCode();
		result = 37 * result + storageItem.hashCode();
		return result;
	}

	/**
	 *  Check if storage has been set and super class is complete
	 *
	 *@return    true if operation is complete
	 */
	public boolean isComplete() {
		return super.isComplete() && storageItem != null;
	}

	/**
	 *  Test if storageItem is a local one
	 *
	 *@return    true if storage item is local
	 */
	public boolean isLocal() {
		boolean ret = storageItem.equals(NSPACE_STRING);
		return ret;
	}

	/**
	 *  Sets the ASCII attribute of the FileExport object
	 *
	 *@param  isAscii  The new FileMode value
	 */
	public void setFileMode(boolean isAscii) {
		this.isAscii = isAscii;
	}

	/**
	 *  Sets the OverwritePolicy attribute of the FileExport object
	 *
	 *@param  overwrite  The new OverwritePolicy value
	 */
	public void setOverwritePolicy(boolean overwrite) {
		this.overwrite = overwrite;
	}

	/**
	 *@param  storageItem  The new StorageItem value
	 */
	public void setStorageItem(Object storageItem) {
		this.storageItem = storageItem;
	}

}
