package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *  Class representing a file operation (copy, rename, delete, make dir)
 *
 *@author     Ralf Ratering
 *@created    29. August 2001
 *@version    $Id: FileSystemOperation.java,v 1.3 2003/06/30 06:58:20 rrate Exp $
 */
public class FileSystemOperation extends FileStorage {
	/**
	 *  Description of the Field
	 */
	public final static String COPY = "copy to",
		RENAME = "rename to",
		MAKEDIR = "make directory",
		DELETE = "delete",
		CHMOD = "change permissions";
	private static DataFlavor flavor;
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = 4202108145757606227L;

	/**
	 *  Gets the flavor attribute of the FileSystemOperation class
	 *
	 *@return    The flavor value
	 */
	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor =
					new DataFlavor(
						Class.forName(
							"com.pallas.unicore.extensions.FileSystemOperation"),
						"FileSystemOperation");
			} catch (ClassNotFoundException e) {
				logger.log(
					Level.SEVERE,
					"Error getting DataFlavor for com.pallas.unicore.extensions.FileSystemOperatins",
					e);
			}

		}
		return flavor;
	}

	/**
	 *  Gets the Operations attribute of the FileSystemOperation class
	 *
	 *@return    The Operations value
	 */
	public static String[] getOperations() {
		String[] operations = { COPY, DELETE, RENAME, MAKEDIR, CHMOD };
		return operations;
	}

	/**
	 *  String describing uspace operation
	 */
	protected String operation;

	/**
	 *  Constructor including uspace operation
	 *
	 *@param  sourceName       name of source file
	 *@param  operation        String describing uspace operation
	 *@param  destinationName  name of destination file, ignored for delete or
	 *      make dirs
	 *@param  storageItem      Description of Parameter
	 */
	public FileSystemOperation(
		String sourceName,
		String operation,
		String destinationName,
		Object storageItem) {
		super(storageItem, sourceName, destinationName);
		this.operation = operation;
	}

	/**
	 *  Get separator of destination file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getDestinationFileSeparator() {
		return "/";
	}

	/**
	 *@return    operation as String
	 */
	public String getOperation() {
		return operation;
	}

	/**
	 *  Get separator of source file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getSourceFileSeparator() {
		return "/";
	}

	/**
	 *  Gets the subClassFlavor attribute of the FileSystemOperation object
	 *
	 *@return    The subClassFlavor value
	 */
	public DataFlavor getSubClassFlavor() {
		return getFlavor();
	}

	/**
	 *  Gets the Complete attribute of the FileSystemOperation object
	 *
	 *@return    The Complete value
	 */
	public boolean isComplete() {
		// super checks already for empty source, null destination and storage item
		boolean complete = super.isComplete();

		if (operation.equals(RENAME)) {
			// no empty destination allowed
			complete &= !destinationName.equals("");
		} else if (operation.equals(COPY)) {
			complete &= !destinationName.equals("");
		} else if (operation.equals(DELETE)) {
			complete &= destinationName.equals("");
		} else if (operation.equals(MAKEDIR)) {
			complete &= destinationName.equals("");
		} else if (!operation.equals(CHMOD)) {
			complete = false;
		}

		return complete;
	}

	/**
	 *@param  operation  as String
	 */
	public void setOperation(String operation) {
		this.operation = operation;
	}

	/**
	 *@return    String describing operation
	 */
	public String toString() {
		if (operation.equals(DELETE) || operation.equals(MAKEDIR)) {
			return storageItem + ": " + sourceName + " " + operation;
		} else {
			return storageItem
				+ ": "
				+ sourceName
				+ " "
				+ operation
				+ " "
				+ destinationName;
		}
	}

}
