package com.pallas.unicore.extensions;

import java.util.Vector;

import org.unicore.Vsite;
import org.unicore.ajo.FileCheck;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@created    17. Juli 2002
 *@version    $Id: FileTest.java,v 1.2 2003/06/30 06:58:20 rrate Exp $
 */
public class FileTest extends AbstractTest {

	/**
	 *  Description of the Field
	 */
	public final static FileTestType EXISTING =
		new FileTestType(FileTestType.EXISTING),
		NOT_EXISTING = new FileTestType(FileTestType.NOT_EXISTING),
		READABLE = new FileTestType(FileTestType.READABLE),
		WRITABLE = new FileTestType(FileTestType.WRITABLE),
		EXECUTABLE = new FileTestType(FileTestType.EXECUTABLE);

	final static long serialVersionUID = 8558445171865314876L;

	private static Vector types;

	/**
	 *  Gets the types attribute of the FileTest class
	 *
	 *@return    The types value
	 */
	public static Vector getTypes() {
		if (types == null) {
			types = new Vector();
			types.add(DISABLED);
			types.add(EXISTING);
			types.add(NOT_EXISTING);
			types.add(READABLE);
			types.add(WRITABLE);
			types.add(EXECUTABLE);
		}
		return types;
	}

	private Storage storage;

	private String subspacePath;

	/**
	 *  Description of the Method
	 *
	 *@param  errorSet   Description of the Parameter
	 *@param  container  Description of the Parameter
	 *@return            Description of the Return Value
	 */
	public ErrorSet checkContents(
		ErrorSet errorSet,
		ActionContainer container) {
		super.checkContents(errorSet, container);

		Vsite vsite = container.getParentJobContainer().getVsite();
		if (vsite == null) {
			return errorSet;
		}

		NamedResourceSet siteResources = ResourceManager.getResourceSet(vsite);
		if (storage == null) {
			errorSet.add(
				new UError(
					container.getIdentifier(),
					"No storage location selected."));
		} else if (!siteResources.contains(storage)) {
			errorSet.add(
				new UError(
					container.getIdentifier(),
					"Storage location "
						+ storage
						+ " not available at site "
						+ vsite
						+ "."));
		}

		return errorSet;
	}

	/**
	 *  Gets the fileCheckTask attribute of the FileTest object
	 *
	 *@return    The fileCheckTask value
	 */
	public FileCheck getFileCheckTask() {
		FileCheck fileCheck =
			new FileCheck(
				"FileCheck_" + ResourceManager.getNextObjectIdentifier());
		fileCheck.setFileToCheck(getSubspacePath());
		fileCheck.setResources(new ResourceSet(getStorage()));
		if (type.equals(EXISTING)) {
			fileCheck.setFileExists(Boolean.TRUE);
		} else if (type.equals(NOT_EXISTING)) {
			fileCheck.setFileExists(Boolean.FALSE);
		} else if (type.equals(READABLE)) {
			fileCheck.setCanReadFile(Boolean.TRUE);
		} else if (type.equals(WRITABLE)) {
			fileCheck.setCanWriteFile(Boolean.TRUE);
		} else if (type.equals(EXECUTABLE)) {
			fileCheck.setCanExecuteFile(Boolean.TRUE);
		}
		return fileCheck;
	}

	/**
	 *  Gets the selectedLocation attribute of the FileTest object
	 *
	 *@return    The selectedLocation value
	 */
	public Storage getStorage() {
		return storage;
	}

	/**
	 *  Gets the file attribute of the FileTest object
	 *
	 *@return    The file value
	 */
	public String getSubspacePath() {
		return subspacePath;
	}

	/**
	 *  Sets the selectedLocation attribute of the FileTest object
	 *
	 *@param  storage  The new storage value
	 */
	public void setStorage(Storage storage) {
		this.storage = storage;
	}

	/**
	 *  Sets the file attribute of the FileTest object
	 *
	 *@param  subspacePath  The new subspacePath value
	 */
	public void setSubspacePath(String subspacePath) {
		this.subspacePath = subspacePath;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		return "Test if file "
			+ subspacePath
			+ " at "
			+ storage
			+ " is "
			+ type;
	}

}
