package com.pallas.unicore.extensions;
import java.awt.datatransfer.DataFlavor;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.pallas.unicore.container.JobContainer;

/**
 *  Class representing a file transfer between two uspaces
 *
 *@author     Ralf Ratering
 *@created    13. September 2001
 *@version    $Id: FileTransfer.java,v 1.3 2003/06/30 06:58:19 rrate Exp $
 */
public class FileTransfer extends FileOperation {
	private static DataFlavor flavor;
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = 8649468376722199373L;

	/**
	 *  Gets the flavor attribute of the FileTransfer class
	 *
	 *@return    The flavor value
	 */
	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor =
					new DataFlavor(
						Class.forName(
							"com.pallas.unicore.extensions.FileTransfer"),
						"FileTransfer");
			} catch (ClassNotFoundException e) {
				logger.log(
					Level.SEVERE,
					"Error getting DataFlavore for com.pallas.unicore.extensions.FileExport",
					e);
			}
		}
		return flavor;
	}

	/**
	 *  JobContainer defining destination uspace
	 */
	protected JobContainer destinationJob;

	/**
	 *  Constructor including destination job group
	 *
	 *@param  sourceName       name of source file
	 *@param  destinationJob   destination job group
	 *@param  destinationName  name of destination file
	 */
	public FileTransfer(
		String sourceName,
		JobContainer destinationJob,
		String destinationName) {
		super(sourceName, destinationName);
		// sourcename == destination name currently
		this.destinationName = sourceName;
		this.destinationJob = destinationJob;
	}

	/**
	 *  Get separator of destination file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getDestinationFileSeparator() {
		return "/";
	}

	/**
	 *@return    destination JobContainer
	 */
	public JobContainer getDestinationJob() {
		return destinationJob;
	}

	/**
	 *  Get separator of source file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getSourceFileSeparator() {
		return "/";
	}

	/**
	 *  Gets the subClassFlavor attribute of the FileTransfer object
	 *
	 *@return    The subClassFlavor value
	 */
	public DataFlavor getSubClassFlavor() {
		return flavor;
	}

	/**
	 *  Gets the Complete attribute of the FileTransfer object
	 *
	 *@return    The Complete value
	 */
	public boolean isComplete() {
		return super.isComplete() && destinationJob != null;
	}

	/**
	 *@param  destinationJob  The new DestinationJob value
	 */
	public void setDestinationJob(JobContainer destinationJob) {
		this.destinationJob = destinationJob;
	}

	/**
	 *@return    String describing operation
	 */
	public String toString() {
		return sourceName + " -> " + destinationJob + ": " + destinationName;
	}
}
