package com.pallas.unicore.extensions;

import java.util.Vector;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.DependencyContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;

/**
 *@author     Ralf Ratering
 *@created    17. Juli 2002
 *@version    $Id: ReturnCodeTest.java,v 1.2 2003/06/30 06:58:17 rrate Exp $
 */
public class ReturnCodeTest extends AbstractTest implements Cloneable {

	/**
	 *  Description of the Field
	 */
	public final static ReturnCodeTestType EQUAL =
		new ReturnCodeTestType(ReturnCodeTestType.EQUAL),
		NOT_EQUAL = new ReturnCodeTestType(ReturnCodeTestType.NOT_EQUAL),
		GREATER = new ReturnCodeTestType(ReturnCodeTestType.GREATER),
		LESS = new ReturnCodeTestType(ReturnCodeTestType.LESS),
		GREATER_OR_EQUAL =
			new ReturnCodeTestType(ReturnCodeTestType.GREATER_OR_EQUAL),
		LESS_OR_EQUAL =
			new ReturnCodeTestType(ReturnCodeTestType.LESS_OR_EQUAL),
		SUCCESSFUL = new ReturnCodeTestType(ReturnCodeTestType.SUCCESSFUL),
		NOT_SUCCESSFUL =
			new ReturnCodeTestType(ReturnCodeTestType.NOT_SUCCESSFUL);

	final static long serialVersionUID = -2638238924226364124L;

	private static Vector types;

	/**
	 *  Gets the types attribute of the FileTest class
	 *
	 *@return    The types value
	 */
	public static Vector getTypes() {
		if (types == null) {
			types = new Vector();
			types.add(DISABLED);
			types.add(EQUAL);
			types.add(NOT_EQUAL);
			//types.add(GREATER);
			//types.add(LESS);
			//types.add(GREATER_OR_EQUAL);
			//types.add(LESS_OR_EQUAL);
			types.add(SUCCESSFUL);
			types.add(NOT_SUCCESSFUL);
		}
		return types;
	}
	private DependencyContainer addedDependency;
	private ActionContainer addedFlag;
	private ActionContainer testContainer;
	private int value = 0;

	/**
	 *  Description of the Method
	 *
	 *@param  errorSet   Description of the Parameter
	 *@param  container  Description of the Parameter
	 *@return            Description of the Return Value
	 */
	public ErrorSet checkContents(
		ErrorSet errorSet,
		ActionContainer container) {
		super.checkContents(errorSet, container);
		if (!isEnabled()) {
			return errorSet;
		}
		if (testContainer == null) {
			errorSet.add(
				new UError(
					container.getIdentifier(),
					"Missing test task in return code test."));
			return errorSet;
		}
		if (!getType().equals(SUCCESSFUL)
			&& !getType().equals(NOT_SUCCESSFUL)) {
			boolean error = false;
			if (!(testContainer instanceof TaskContainer)) {
				error = true;
			} else {
				TaskContainer task = (TaskContainer) testContainer;
				if (!task.hasExecuteTask()) {
					error = true;
				}
			}
			if (error) {
				errorSet.add(
					new UError(
						container.getIdentifier(),
						testContainer
							+ " in return code test, does not return a value."));
			}
		}

		if (!testContainer.getIgnoreFailure()) {
			// we'll do this later on...
			testContainer.setIgnoreFailure(true);
			addedFlag = testContainer;
			// errorSet.add(new UError(container.getIdentifier(),
			//         "Ignore failure flag of task " + testContainer
			//         + " must be enabled (cf. dependency panel), because " + testContainer
			//         + " will be tested for return code/success in " + container));
		}

		if (container instanceof IfContainer) {
			boolean error = true;
			GroupContainer groupContainer = container.getParentContainer();
			for (int i = 0; i < groupContainer.getDependencies().length; i++) {
				ActionContainer succ =
					groupContainer.getDependencies()[i].getSucc();
				ActionContainer pred =
					groupContainer.getDependencies()[i].getPred();
				if (pred.getIdentifier()
					== getTestContainer().getIdentifier()) {
					if (getTestContainer().isPredecessorOf(container)) {
						error = false;
					}
					break;
				}
			}
			if (error) {
				// we'll do this later on...
				//       addedDependency = new DependencyContainer(getTestContainer(), container);
				//       groupContainer.addDependency(addedDependency);

				errorSet.add(
					new UError(
						container.getIdentifier(),
						testContainer
							+ " must be an ancestor of "
							+ container
							+ " in the dependency graph, because it will be tested for return code/success."));

			}
		}

		return errorSet;
	}

	/**
	 *  Gets the addedDependency attribute of the ReturnCodeTest object
	 *
	 *@return    The addedDependency value
	 */
	public DependencyContainer getAddedDependency() {
		return addedDependency;
	}

	/**
	 *  Gets the addedFlag attribute of the ReturnCodeTest object
	 *
	 *@return    The addedFlag value
	 */
	public ActionContainer getAddedFlag() {
		return addedFlag;
	}

	/**
	 *  Gets the file attribute of the FileTest object
	 *
	 *@return    The file value
	 */
	public ActionContainer getTestContainer() {
		return testContainer;
	}

	/**
	 *  Gets the value attribute of the ReturnCodeTest object
	 *
	 *@return    The value value
	 */
	public int getValue() {
		return this.value;
	}

	/**
	 *  Gets the parentNode attribute of the ReturnCodeTest object
	 *
	 *@param  container1  Description of the Parameter
	 *@param  container2  Description of the Parameter
	 *@return             The parentNode value
	 */
	private boolean isEqual(
		ActionContainer container1,
		ActionContainer container2) {
		return container1.getIdentifier() == container2.getIdentifier();
	}

	/**
	 *  Description of the Method
	 */
	public void removeAddedFlagsAndDependencies() {
		if (addedFlag != null) {
			addedFlag.setIgnoreFailure(false);
			addedFlag = null;
		}
		// for future use...
		/*
		 *  if(addedDependency != null) {
		 *  GroupContainer group = (GroupContainer)addedDependency.getPred().getParentContainer();
		 *  group.removeDependency(addedDependency);
		 *  addedDependency = null;
		 *  }
		 */
	}

	/**
	 *  Sets the addedDependency attribute of the ReturnCodeTest object
	 *
	 *@param  addedDependency  The new addedDependency value
	 */
	public void setAddedDependency(DependencyContainer addedDependency) {
		this.addedDependency = addedDependency;
	}

	/**
	 *  Sets the addedFlag attribute of the ReturnCodeTest object
	 *
	 *@param  addedFlag  The new addedFlag value
	 */
	public void setAddedFlag(ActionContainer addedFlag) {
		this.addedFlag = addedFlag;
	}

	/**
	 *  Sets the file attribute of the FileTest object
	 *
	 *@param  testContainer  The new testContainer value
	 */
	public void setTestContainer(ActionContainer testContainer) {
		this.testContainer = testContainer;
	}

	/**
	 *  Sets the value attribute of the ReturnCodeTest object
	 *
	 *@param  value  The new value value
	 */
	public void setValue(int value) {
		this.value = value;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		return "Container: "
			+ testContainer
			+ ", Test: "
			+ type
			+ ", Value: "
			+ value;
	}

}
