package com.pallas.unicore.extensions;

import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.PutPortfolio;
import org.unicore.idiomatic.Transfer;

import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Class builds AJO ActionGroup from an input Array of FileTransfers
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: TransferFactory.java,v 1.2 2003/06/30 06:58:15 rrate Exp $
 */
public class TransferFactory extends FileOperationFactory {

	/**
	 *  Constructor for the TransferFactory object
	 *
	 *@param  fileOpsArray  Description of the Parameter
	 *@param  actionGroup   Description of the Parameter
	 *@param  container     Description of the Parameter
	 */
	public TransferFactory(
		FileTransfer[] fileOpsArray,
		ActionGroup actionGroup,
		TaskContainer container) {
		super(fileOpsArray, actionGroup, container);
	}

	/**
	 *  Description of the Method
	 */
	public void build() {
		FileTransfer[] transfers = container.getFileTransfers();
		if (transfers.length == 0) {

			// no transfers specified
			return;
		}

		// collect different Destinations and store them with
		// corresponding transfers in Hashtable
		Hashtable transferTable = new Hashtable();
		/*
		 *  for (int i = 0; i < transfers.length; i++) {
		 *  JobContainer destinationJob = transfers[i].getDestinationJob();
		 *  if (!transferTable.containsKey(destinationJob)) {
		 *  Vector jobTransfers = new Vector();
		 *  jobTransfers.add(transfers[i]);
		 *  transferTable.put(destinationJob, jobTransfers);
		 *  } else {
		 *  Vector jobTransfers = (Vector)transferTable.get(destinationJob);
		 *  jobTransfers.add(transfers[i]);
		 *  }
		 *  }
		 *  Enumeration jobEnum = transferTable.keys();
		 *  while (jobEnum.hasMoreElements()) {
		 *  JobContainer job = (JobContainer)jobEnum.nextElement();
		 *  buildTransferTask((Vector)transferTable.get(job), job);
		 *  }
		 */
		for (int i = 0; i < transfers.length; i++) {
			JobContainer destinationJob = transfers[i].getDestinationJob();
			Vector jobTransfers = new Vector();
			jobTransfers.add(transfers[i]);
			buildTransferTask(jobTransfers, destinationJob);
		}
	}

	/**
	 *  Build an import task for one specific storage location in Xspace
	 *
	 *@param  transfers    Description of Parameter
	 *@param  destination  Description of Parameter
	 */
	private void buildTransferTask(
		Vector transfers,
		JobContainer destination) {
		AbstractAction theTransfer = null;

		// build the source portfolio
		MakePortfolio sourcePortfolio =
			new MakePortfolio(
				container.getName()
					+ "_MAKEPORTFOLIO_"
					+ ResourceManager.getNextObjectIdentifier());
		//sourcePortfolio.setPropagateSuccessfulIfNotSuccessful(container.getIgnoreFailure());

		if (destination
			.getVsite()
			.equals(container.getParentJobContainer().getVsite())) {
			theTransfer =
				new PutPortfolio(
					container.getName()
						+ "_PP_TRANS_"
						+ ResourceManager.getNextObjectIdentifier());
		} else {
			theTransfer =
				new Transfer(
					container.getName()
						+ "_TT_TRANS_"
						+ ResourceManager.getNextObjectIdentifier());
			((AbstractJob) theTransfer).setEndorser(destination.getUser());

		}

		// build filename array as input for portfolio
		String[] filenames = new String[transfers.size()];
		ActionGroup dummyGroup =
			new ActionGroup(createActionName("TRANSFER_DUMMY_GROUP"));
		for (int i = 0; i < transfers.size(); i++) {
			FileTransfer fileTransfer = (FileTransfer) transfers.elementAt(i);
			filenames[i] = fileTransfer.getSourceName();

			// remember complete action group in transfer objects
			fileTransfer.setAbstractAction(dummyGroup);
		}
		sourcePortfolio.setFiles(filenames);

		// allow only the same name currently
		MakePortfolio targetPortfolio = sourcePortfolio;
		if (destination
			.getVsite()
			.equals(container.getParentJobContainer().getVsite())) {
			PutPortfolio put = (PutPortfolio) theTransfer;
			put.setSource(sourcePortfolio.getPortfolio());
			put.setTarget(targetPortfolio.getPortfolio().getId());
			put.setTargetUspace((AJOIdentifier) destination.getIdentifier());

		} else {
			Transfer transferTask = (Transfer) theTransfer;
			transferTask.setSourcePortfolio(sourcePortfolio.getPortfolio());
			transferTask.setTargetPortfolio(
				targetPortfolio.getPortfolio().getId());
			transferTask.setTargetUspace(
				(AJOIdentifier) destination.getIdentifier());
			transferTask.setTargetVsite(destination.getVsite());

		}

		// try to delete previous portfolio in case of loops,
		// because NJS will complain about already existing
		// portfolio else
		DeletePortfolio deletePortfolio =
			new DeletePortfolio(
				"_DELETEPORTFOLIO_"
					+ ResourceManager.getNextObjectIdentifier());
		deletePortfolio.setPortfolio(sourcePortfolio.getPortfolio().getId());
		// set ignore failure, because portfolio will not exist yet,
		// if task is not in a loop and also in the first iteration
		deletePortfolio.setPropagateSuccessfulIfNotSuccessful(true);

		dummyGroup.add(sourcePortfolio);
		dummyGroup.add(theTransfer);
		dummyGroup.add(deletePortfolio);
		try {
			dummyGroup.addDependency(
				new Dependency(deletePortfolio, sourcePortfolio));
			dummyGroup.addDependency(
				new Dependency(sourcePortfolio, theTransfer));
		} catch (ActionGroup.InvalidDependencyException ex) {
			logger.log(Level.SEVERE, "Could not add dependency.", ex);
		}
		actionGroup.add(dummyGroup);
	}
}
