package com.pallas.unicore.extensions;

import java.net.URL;

import org.unicore.Unicore;

/**
 *  Class representing a UNICOREpro USite
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: Usite.java,v 1.4 2003/07/09 08:55:27 rrate Exp $
 */
public class Usite extends Unicore implements Comparable {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    24. Mrz 2003
	 */
	public static class Type extends Unicore {
		/**
		 *  Description of the Field
		 */
		public final static Type CLASSIC =
			new Type("Classical UNICORE Site");
		/**
		 *  Description of the Field
		 */
		public final static Type REGISTRY = new Type("Grid Service Registry");
		final static long serialVersionUID = -7208106932332931313L;

		private String description;

		/**
		 *  Create a new Type
		 */
		public Type() {
			this.description = CLASSIC.getDescription();
		}

		/**
		 *  Create a new Type. <p>
		 *
		 *  Note that this is publically available so that the range of
		 *  supported types can be extended by agreement between the clients and
		 *  sites.
		 *
		 *@param  description  Description of the Parameter
		 */
		public Type(String description) {
			this.description = description;
		}

		/**
		 *  Types are equal if their descriptions are equal. <p>
		 *
		 *  Need to do carefully as these cross JVM vboundaries through
		 *  serialisation and so "==" will fail even on statics.
		 *
		 *@param  o  Description of the Parameter
		 *@return    Description of the Return Value
		 */
		public boolean equals(Object o) {
			try {
				return (
					o.getClass() == getClass()
						&& getDescription().equals(((Type) o).getDescription()));
			} catch (NullPointerException nex) {
				return false;
			}
		}

		/**
		 *  Get the decsription of the type
		 *
		 *@return    The description value
		 */
		public String getDescription() {
			return description;
		}

		/**
		 *  Description of the Method
		 *
		 *@return    Description of the Return Value
		 */
		public int hashCode() {
			return getDescription().hashCode();
		}
	}

	final static long serialVersionUID = 6965391670916531770L;
	private String name;
	private Type type;

	private URL url;

	/**
	 *  Constructor for the Usite object
	 *
	 *@param  url   Description of Parameter
	 *@param  name  Description of Parameter
	 */
	public Usite(URL url, String name) {
		this(url, name, Type.CLASSIC);
	}

	/**
	 *  Constructor for the Usite object
	 *
	 *@param  url   Description of the Parameter
	 *@param  name  Description of the Parameter
	 *@param  type  Description of the Parameter
	 */
	public Usite(URL url, String name, Type type) {
		this.url = url;
		this.name = name;
		this.type = type;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public int compareTo(Object object) {
		Usite usite2 = (Usite) object;
		return (name.compareTo(usite2.getName()));
	}

	/**
	 *  Compares two Usites. The result is <code>true</code> if and only if the
	 *  name, URL host and port are equal.
	 *
	 *@param  obj  the Usite to compare against.
	 *@return      <code>true</code> if the URL addresses are the same; <code>false</code>
	 *      otherwise.
	 */

	public boolean equals(Object obj) {
		if (!(obj instanceof Usite)) {
			return false;
		}
		Usite usite2 = (Usite) obj;
		return getAddress().equals(usite2.getAddress());
	}

	/**
	 *  The URL of the UNICOREpro Gateway to the Vsite.
	 *
	 *@return    The Address value
	 */
	public URL getAddress() {
		return url;
	}

	/**
	 *  Gets the Name attribute of the Usite object
	 *
	 *@return    The Name value
	 */
	public String getName() {
		return name;
	}

	/**
	 *  Gets the type attribute of the Usite object
	 *
	 *@return    The type value
	 */
	public Type getType() {
		return type;
	}

	/**
	 *  Always override hashCode when overriding equals.
	 *
	 *@return    Description of the Return Value
	 */
	public int hashCode() {
		int result = 17;
		result = 37 * result + getName().hashCode();
		return result;
	}

	/**
	 *  Sets the Address attribute of the Usite object
	 *
	 *@param  url  The new Address value
	 */
	public void setAddress(URL url) {
		this.url = url;
	}

	/**
	 *  Sets the Name attribute of the Usite object
	 *
	 *@param  name  The new Name value
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 *  Sets the type attribute of the Usite object
	 *
	 *@param  type  The new type value
	 */
	public void setType(Type type) {
		this.type = type;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public String toString() {
		return name;
	}
}
