package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.ResourceBundle;
import java.util.logging.Level;

import org.unicore.AAIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.Control;
import org.unicore.ajo.ControlAction;
import org.unicore.ajo.WaitUntil;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.ControlAction_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.HoldJobContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Thomas Kentemich
 *@created    August 15, 2001
 *@version    $Id: ControlActionService.java,v 1.3 2003/06/30 06:58:02 rrate Exp $
 */
public class ControlActionService extends ObservableNodeThread {

	static ResourceBundle res =
		ResourceBundle.getBundle("com.pallas.unicore.requests.ResourceStrings");

	private Control control;
	private UnicoreResponse response = new UnicoreResponse(0, "<None>");

	/**
	 *@param  control  Description of Parameter
	 *@param  node     Description of the Parameter
	 */
	public ControlActionService(ContainerNode node, Control control) {
		super(node);
		this.control = control;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		try {
			//generating the job
			//first, the " GetJobs" task
			if ((control == null)) {
				throw new Exception("Illegal parameter!!");
			}

			getNode().setState(ActionContainer.STATE_UPDATING);

			ActionContainer ac = (ActionContainer) getNode().getUserObject();
			JobContainer parent = null;
			if (ac.getParentJobContainer() == null) {
				parent = (JobContainer) ac;
			} else {
				parent = ac.getParentJobContainer();
			}

			Vsite vsite = parent.getVsite();
			User user = parent.getUser();

			AAIdentifier id = ac.getIdentifier();

			// for hold tasks do not use the ActionGroup identifier,
			// but instead the WaitUntil id.
			if (ac instanceof HoldJobContainer) {
				HoldJobContainer holdTask = (HoldJobContainer) ac;
				WaitUntil waitUntil = holdTask.getWaitUntil();
				id = waitUntil.getId();
			}

			ControlAction ca =
				new ControlAction(
					servicePrefix + "_Control_(" + id.getName() + ")",
					id,
					control);
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_Control_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					user);
			job.add(ca);

			Reply roReply = nonPolling(job, vsite, user);
			response = roReply.getLastEntry();
			// now analyze outcome
			ControlAction_Outcome cao = null;
			if (!(roReply instanceof RetrieveOutcomeReply)) {
				throw new Exception(
					res.getString("NO_VSITE")
						+ vsite
						+ "\nReason: "
						+ roReply.getLastEntry().getComment());
			} else {
				// deserialize the Outcome Object
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(
							((RetrieveOutcomeReply) roReply)
						.getOutcome()))
						.readObject();

				if (outcome instanceof AbstractJob_Outcome) {

					AbstractJob_Outcome aj_out =
						ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

					cao = (ControlAction_Outcome) aj_out.getOutcome(ca.getId());
					ac.setStatus(cao.getStatus(id));
				} else {
					throw new Exception(res.getString("OUTCOME_ERROR"));
				}
			}
		} catch (Exception e) {
			UserMessages.warning(
				"Executing control action failed.",
				e.getLocalizedMessage());
			logger.log(Level.SEVERE, "Executing control action failed.", e);
		}

		notifyObservers(this, response);
	}

}
