package com.pallas.unicore.requests;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.ActionGroup;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeAck;
import org.unicore.upl.UnicoreResponse;

import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.DeleteAll;

/**
 *@author     Thomas Kentemich
 *@created    August 15, 2001
 *@version    $Id: DeleteJob.java,v 1.3 2003/06/30 06:58:02 rrate Exp $
 */
public class DeleteJob extends ObservableNodeThread {

	private JobContainer jobContainer;
	private UnicoreResponse response = new UnicoreResponse(0, "<None>");

	/**
	 *  Constructor for the DeleteJob object
	 *
	 *@param  node  Description of the Parameter
	 */
	public DeleteJob(ContainerNode node) {
		super(node);
		jobContainer = (JobContainer) node.getActionContainer();
	}

	/**
	 *  Delete all spooled files belonging to this job We currently do not
	 *  support spooled files that persist after the job is deleted
	 *
	 *@param  top  Description of the Parameter
	 */
	private void deleteSpooledFiles(JobContainer top) {
		Vector newNspaceTransfers = top.getNspaceExportTasks();
		Vector subJobs = top.getSubJobs();

		// delete sub job spool files
		for (int j = 0; j < subJobs.size(); j++) {
			deleteSpooledFiles((JobContainer) subJobs.elementAt(j));
		}

		if (newNspaceTransfers.size() == 0) {
			return;
		}

		Vector fileExports = new Vector();
		for (int i = 0; i < newNspaceTransfers.size(); i++) {
			TaskContainer ac = (TaskContainer) newNspaceTransfers.elementAt(i);
			Vector spooledFiles = ac.getSpooledFiles();
			if (spooledFiles != null) {
				for (int j = 0; j < spooledFiles.size(); j++) {
					fileExports.add((FileExport) spooledFiles.elementAt(j));
				}
			}
		}

		DeleteSpoolFiles delete = new DeleteSpoolFiles(top, fileExports);
		delete.start();
	}

	/**
	 *  Description of the Method
	 */
	private void deleteTempFiles() {
		if (jobContainer.getLastUpdateUPLDirectory() != null) {
			try {
				DeleteAll.deleteAll(
					new File(
						Connection.getSessionDir()
							+ File.separator
							+ jobContainer.getLastUpdateUPLDirectory()));
			} catch (IOException ioe) {
			}
		}
		ActionGroup actionGroup = jobContainer.getActionGroup();
		if (actionGroup == null) {
			return;
		}
		findActionGroups(actionGroup);
	}

	/**
	 *  Find an ActionGroup directory and delete it
	 *
	 *@param  actionGroup  containing ExecuteTask
	 */
	private void findActionGroups(ActionGroup actionGroup) {
		Enumeration actions = actionGroup.getActions();
		while (actions.hasMoreElements()) {
			org.unicore.ajo.AbstractAction nextAction =
				(org.unicore.ajo.AbstractAction) (actions.nextElement());
			if (nextAction instanceof ActionGroup) {
				// find action group directories
				String sessionDir =
					Connection.getSessionDir()
						+ File.separator
						+ nextAction.getUPLDirectoryName();
				try {
					logger.info("Deleting: " + sessionDir);
					DeleteAll.deleteAll(new File(sessionDir));
				} catch (IOException e) {
					logger.log(
						Level.SEVERE,
						"Deleting " + sessionDir + " failed.",
						e);
				}
			}

		}
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		try {
			jobContainer = (JobContainer) getNode().getUserObject();

			PanelNode vsiteNode = (PanelNode) getNode().getParent();
			Vsite vsite = null;
			// if we have a vsite node it's safest to use the underlying vsite object
			if (vsiteNode != null) {
				vsite = (Vsite) vsiteNode.getUserObject();
			}
			// service jobs won't have vsite nodes, so take the container's vsite
			else {
				vsite = jobContainer.getVsite();
			}

			// delete the job by acknowleging the outcome
			RetrieveOutcomeAck roAck =
				new RetrieveOutcomeAck(
					(AJOIdentifier) jobContainer.getIdentifier(),
					vsite);

			Connection connection =
				ResourceManager.getConnectionManager().lookupConnection(
					vsite,
					ResourceManager.getUser(vsite));
			Reply roAckReply = connection.sendMessage(roAck, null);
			int returnCode = roAckReply.getLastEntry().getReturnCode();

			response = roAckReply.getLastEntry();
			logger.info("Job deleted: " + jobContainer.getName());
			if (returnCode != 0) {
				logger.severe(
					"DeleteJob Error: "
						+ response.getReturnCode()
						+ " "
						+ response.getComment());
				notifyObservers(this, response);
				return;
			}

			deleteSpooledFiles(jobContainer);
			// Delete all tmp files in the Session directory
			deleteTempFiles();
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Executing delete job failed.", e);

		}
		notifyObservers(this, response);
	}
}
