package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Portfolio;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.DeletePortfolio_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Thomas Kentemich
 *@created    November 7, 2001
 *@version    $Id: DeleteSpoolFiles.java,v 1.3 2003/06/30 06:58:01 rrate Exp $
 */
public class DeleteSpoolFiles extends ObservableRequestThread {
	static ResourceBundle res =
		ResourceBundle.getBundle("com.pallas.unicore.requests.ResourceStrings");
	private AbstractJob_Outcome ajOut;
	private Vector fileExports;
	private JobContainer parent;

	/**
	 *  Description of the Field
	 */
	private final String servicePrefix = ResourceManager.getServicePrefix();

	/**
	 *@param  parent       Description of Parameter
	 *@param  fileExports  Description of the Parameter
	 */
	public DeleteSpoolFiles(JobContainer parent, Vector fileExports) {
		this.parent = parent;
		this.fileExports = fileExports;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		try {
			Vsite vsite = parent.getVsite();
			PutPortfolio_Outcome ppo = null;

			User user = parent.getUser();
			AJOIdentifier id = (AJOIdentifier) parent.getIdentifier();
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_DeleteSpool_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					user);
			for (int i = 0; i < fileExports.size(); i++) {
				FileExport export = (FileExport) (fileExports.elementAt(i));

				/*
				 *  DeletePortfolio deleteSpooledPortfolio = new DeletePortfolio(
				 *  servicePrefix + "_DeletePF_(" +
				 *  id.getName() + ")" +
				 *  ResourceManager.getNextObjectIdentifier());
				 *  logger.info("Deleting Portfolio " + export.getSpoolPortfolio().getId());
				 *  deleteSpooledPortfolio.setPortfolio(export.getSpoolPortfolio().getId());
				 */
				Vsite matchingVsite = ResourceManager.getMatchingVsite(vsite);
				if (matchingVsite == null) {
					logger.severe(
						"Cannot obtain matching Vsite from cache"
							+ vsite.getName());
					return;
				}
				NamedResourceSet siteResources =
					ResourceManager.getResourceSet(matchingVsite);
				if (siteResources.isSiteDefault()) {
					logger.severe(
						"Cannot obtain site specific ResourceSet from Vsite= "
							+ vsite.getName());
					return;
				} else {
					logger.info(
						"Found site specific ResourceSet for Vsite= "
							+ vsite.getName());
				}
				org.unicore.resources.Spool spoolResource =
					(org.unicore.resources.Spool) siteResources.findResource(
						new org.unicore.resources.Spool());
				if (spoolResource == null) {
					logger.severe(
						"Cannot obtain spool space resource from Vsite= "
							+ vsite.getName());
					return;
				} else {
					logger.info(
						"Found spool space resource for Vsite= "
							+ vsite.getName());
				}
				Vector spoolOutcomes =
					OutcomeManager.getOutcomeEntries(export.getSpoolTask());
				for (int j = 0; j < spoolOutcomes.size(); j++) {
					ppo =
						(PutPortfolio_Outcome) ((OutcomeEntry) spoolOutcomes
							.elementAt(j))
							.getOutcome();

					//uplName = ppo.getPortfolio().getName();
					Portfolio pf =
						new Portfolio("", ppo.getPortfolio().getValue());
					DeletePortfolio deleteSpooledPortfolio =
						new DeletePortfolio(
							servicePrefix
								+ "_DeletePF_("
								+ id.getName()
								+ ")"
								+ ResourceManager.getNextObjectIdentifier());
					logger.info("Deleting Portfolio " + pf.getId());
					deleteSpooledPortfolio.setPortfolio(pf.getId());
					deleteSpooledPortfolio.setResources(
						new ResourceSet(spoolResource));
					job.add(deleteSpooledPortfolio);
				}
			}
			Reply roReply = nonPolling(job, vsite, user);

			// now analyze outcome
			if (!(roReply instanceof RetrieveOutcomeReply)) {
				throw new Exception(
					res.getString("NO_VSITE")
						+ vsite
						+ "\nReason: "
						+ roReply.getLastEntry().getComment());
			} else {

				// deserialize the Outcome Object
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(
							((RetrieveOutcomeReply) roReply)
						.getOutcome()))
						.readObject();
				if (outcome instanceof AbstractJob_Outcome) {
					AbstractJob_Outcome ajOut =
						ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
					OutcomeEnumeration oen = ajOut.getOutcomes();
					while (oen.hasMoreElements()) {
						Outcome next = oen.nextElement();
						if (next instanceof DeletePortfolio_Outcome) {
							AbstractActionStatus status = next.getStatus();
							if (status
								.isEquivalent(
									AbstractActionStatus.NOT_SUCCESSFUL)) {
								logger.severe(
									"DeletePortfolio failed: "
										+ next.getReason());
							}
						}
					}
				} else {
					throw new Exception(res.getString("OUTCOME_ERROR"));
				}
			}
		} catch (Exception e) {
			UserMessages.warning(
				"Could not delete spooled files for job " + parent,
				e.getLocalizedMessage());
			logger.log(
				Level.SEVERE,
				"Could not delete spooled files for job " + parent,
				e);
		}
		notifyObservers(this, ajOut);
	}
}
