package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyPortfolioToOutcome;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.ActionGroup.InvalidDependencyException;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.resources.AlternativeUspace;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Get a list of files from another job's uspace
 *
 *@author     Ralf Ratering
 *@version $Id: GetFilesFromUspace.java,v 1.1 2003/07/11 07:46:33 rrate Exp $
 */
public class GetFilesFromUspace extends ObservableRequestThread {
	private AJOIdentifier ajoId;
	private String[] filenames;
	private Vsite vsite;

	/**
 	* Set parameters of request in constructor
 	* @param ajoId 		identifier for job that possesses uspace
 	* @param filenames 	array of filenames to transfer
 	* @param vsite 		destination vsite
 	*/
	public GetFilesFromUspace(AJOIdentifier ajoId, String[] filenames, Vsite vsite) {
		this.ajoId = ajoId;
		this.filenames = filenames;
		this.vsite = vsite;
	}

	/**
 	* Start the request
 	*/
	public void run() {
		AlternativeUspace uspace = new AlternativeUspace();
		uspace.setUspace(ajoId);
		ResourceSet uspaceResourceSet = new ResourceSet(uspace);
		
		MakePortfolio makePortfolio =
				 new MakePortfolio("MakePortfolio");
		makePortfolio.setFiles(filenames);
		makePortfolio.setResources(uspaceResourceSet);
					 
		CopyPortfolioToOutcome copyPortfolio = 
			new CopyPortfolioToOutcome("CopyPortfolio");
		copyPortfolio.setTarget(makePortfolio.getPortfolio().getId());
		copyPortfolio.setResources(uspaceResourceSet);
		
		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "_GetFileFromUSpace_"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				user);
		job.add(makePortfolio);
		job.add(copyPortfolio);
		try {
			job.addDependency(makePortfolio, copyPortfolio);
		}
		catch(InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Could not add dependency.", e);
		}

		Reply reply = nonPolling(job, vsite, user);
		AbstractJob_Outcome ajOut = null;
		
		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning(
				"Virtual site not available: "
					+ vsite
					+ "\nReason: "
					+ reply.getLastEntry().getComment());
			return;
		} 
		else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes)).readObject();
				ajOut =
					ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

			} 
			catch (Exception e) {
				logger.log(Level.SEVERE, "Could not get files from uspace.", e);
			}
		}
		notifyObservers(this, ajOut);
	
	}
	
}
