package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.logging.Level;

import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.GetActionStatus;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.GetActionStatus_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Ralf Ratering
 *@created    May 21, 2003
 *@version    $Id: GetJobStatus.java,v 1.2 2003/06/30 06:58:01 rrate Exp $
 */

public class GetJobStatus extends ObservableRequestThread {
	private AbstractJob job;
	private AbstractActionStatus status;

	/**
	 *  Constructor for the GetJobStatus object
	 *
	 *@param  job  Description of the Parameter
	 */
	public GetJobStatus(AbstractJob job) {
		this.job = job;
	}

	/**
	 *  Gets the status attribute of the GetJobStatus object
	 *
	 *@return    The status value
	 */
	public AbstractJob getJob() {
		return job;
	}

	/**
	 *  Gets the status attribute of the GetJobStatus object
	 *
	 *@return    The status value
	 */
	public AbstractActionStatus getStatus() {
		return status;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {

		Vsite vsite = job.getVsite();

		AbstractJob getStatusJob =
			new AbstractJob(
				ResourceManager.getServicePrefix()
					+ "JobStatus_"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				ResourceManager.getUser(vsite));

		GetActionStatus getStatus =
			new GetActionStatus(
				"GetActionStatus_" + ResourceManager.getNextObjectIdentifier(),
				null,
				job.getAJOId());
		getStatusJob.add(getStatus);

		try {

			Reply reply =
				nonPolling(getStatusJob, vsite, ResourceManager.getUser(vsite));

			if (reply == null || !(reply instanceof RetrieveOutcomeReply)) {
				throw new Exception("Bad NJS reply.");
			} else {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] serializedOutcome = roReply.getOutcome();
				if (serializedOutcome == null) {
					throw new Exception("No job outcome available.");
				}
				AbstractJob_Outcome jobOutcome =
					(AbstractJob_Outcome) new ObjectInputStream(
						new ByteArrayInputStream(serializedOutcome))
						.readObject();
				jobOutcome = ConsignForm.convertFrom(jobOutcome);

				OutcomeEnumeration outcomes = jobOutcome.getOutcomes();
				if (outcomes.hasMoreElements()) {
					GetActionStatus_Outcome statusOutcome =
						(GetActionStatus_Outcome) outcomes.nextElement();
					status = statusOutcome.getActionStatus();
				}

			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could not get status for job " + job, e);
		}

		notifyObservers(this, status);
	}

}
