package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ListDirectory;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.ListDirectory_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.XFile;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.XFileEnumeration;
import org.unicore.sets.XFileSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    12. August 2002
 *@version $Id: GetListing.java,v 1.4 2003/06/30 06:58:00 rrate Exp $
 */
public class GetListing extends ObservableRequestThread {
	private RemoteFile dir;
	private XFileSet xFileSet;

	/**
	 *  Constructor for the GetListings object
	 *
	 *@param  dir  Description of the Parameter
	 */
	public GetListing(RemoteFile dir) {
		this.dir = dir;
	}

	/**
	 *  Gets the directories attribute of the GetListings object
	 *
	 *@return    The directories value
	 */
	public RemoteFile getDirectory() {
		return dir;
	}

	/**
	 *  Gets the remoteFileArray attribute of the GetListing object
	 *
	 *@return    The remoteFileArray value
	 */
	public RemoteFile[] getRemoteFileArray() {
		Vector listing = new Vector();
		XFileEnumeration fileEnum = xFileSet.elements();
		while (fileEnum.hasMoreElements()) {
			XFile xfile = (XFile) fileEnum.nextElement();
			RemoteFile file = new RemoteFile(xfile, dir, dir.getFileSystem());
			listing.add(file);
		}
		RemoteFile[] fileArray = new RemoteFile[listing.size()];
		for (int i = 0; i < listing.size(); i++) {
			fileArray[i] = (RemoteFile) listing.elementAt(i);
		}
		return fileArray;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		Vector listing = new Vector();
		String dirname = dir.getSubspacePath();
		Storage storage = dir.getStorage();
		Vsite vsite = dir.getVsite();
		ListDirectory listDir =
			new ListDirectory(
				"_ListDir_("
					+ vsite.getName()
					+ ", "
					+ dirname
					+ ")"
					+ ResourceManager.getNextObjectIdentifier());
		listDir.setResources(new ResourceSet(storage));
		listDir.setTarget(dirname);

		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "_GetListingsJob_"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				user);
		job.add(listDir);
		logger.fine(
			"Getting listing for " + dirname + " at storage " + storage);

		Reply reply = nonPolling(job, vsite, user);

		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning(
				"Virtual site not available: "
					+ vsite
					+ "\nReason: "
					+ reply.getLastEntry().getComment());
			return;
		} else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes))
						.readObject();
				AbstractJob_Outcome ajOut =
					ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

				Outcome ajOutcome = ajOut.getOutcome(listDir.getId());
				if (ajOutcome instanceof ListDirectory_Outcome) {
					ListDirectory_Outcome listDirOutcome =
						(ListDirectory_Outcome) ajOutcome;
					xFileSet = listDirOutcome.getListing();
				}
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Could not get directory listing ", e);
			}
		}

		if (xFileSet == null) {
			logger.severe(
				"List Directory request did not return a listing for "
					+ dirname
					+ " at storage "
					+ storage);
			xFileSet = new XFileSet();
		}

		// notify all my observers on task end
		notifyObservers(this, xFileSet);
	}
}
