package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ListDirectory;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.ListDirectory_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.XFile;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.XFileEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    12. August 2002
 *@version $Id: GetListings.java,v 1.4 2003/06/30 06:58:00 rrate Exp $
 */
public class GetListings extends ObservableRequestThread {
	private Vector dirNodes;
	private Vector dirs;
	private Vsite vsite;

	/**
	 *  Constructor for the GetListings object
	 *
	 *@param  dirs      Description of the Parameter
	 *@param  vsite     Description of the Parameter
	 *@param  dirNodes  Description of the Parameter
	 */
	public GetListings(Vector dirs, Vector dirNodes, Vsite vsite) {
		this.dirs = dirs;
		this.dirNodes = dirNodes;
		this.vsite = vsite;
	}

	/**
	 *  Constructor for the GetListings object
	 *
	 *@param  dirs   Description of the Parameter
	 *@param  vsite  Description of the Parameter
	 */
	public GetListings(Vector dirs, Vsite vsite) {
		this(dirs, null, vsite);
	}

	/**
	 *  Gets the directories attribute of the GetListings object
	 *
	 *@return    The directories value
	 */
	public Vector getDirectories() {
		return dirs;
	}

	/**
	 *  Gets the directoryNodes attribute of the GetListings object
	 *
	 *@return    The directoryNodes value
	 */
	public Vector getDirectoryNodes() {
		return dirNodes;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		Hashtable table = new Hashtable();
		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "_GetListingsJob_"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				user);
		Vector listDirs = new Vector();
		for (int i = 0; i < dirs.size(); i++) {
			RemoteFile dir = (RemoteFile) dirs.elementAt(i);
			String dirname = dir.getSubspacePath();
			Storage storage = dir.getStorage();
			ListDirectory listDir =
				new ListDirectory(
					"_ListDir_("
						+ vsite.getName()
						+ ", "
						+ dirname
						+ ")"
						+ ResourceManager.getNextObjectIdentifier());
			listDir.setResources(new ResourceSet(storage));
			listDir.setTarget(dirname);
			listDirs.add(listDir);
			job.add(listDir);
			logger.fine(
				"Getting listing for " + dirname + " at storage " + storage);
		}

		Reply reply = nonPolling(job, vsite, user);

		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning(
				"Virtual site not available: "
					+ vsite
					+ "\nReason: "
					+ reply.getLastEntry().getComment());
			return;
		} else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes))
						.readObject();
				AbstractJob_Outcome ajOut =
					ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

				for (int i = 0; i < listDirs.size(); i++) {
					ListDirectory listDir =
						(ListDirectory) listDirs.elementAt(i);
					Outcome ajOutcome = ajOut.getOutcome(listDir.getId());
					if (ajOutcome instanceof ListDirectory_Outcome) {
						ListDirectory_Outcome listDirOutcome =
							(ListDirectory_Outcome) ajOutcome;
						Vector listing = new Vector();
						RemoteFile parent = (RemoteFile) dirs.elementAt(i);
						if (listDirOutcome.getListing() == null) {
							logger.severe(
								"Could not get listing for "
									+ listDir
									+ " at site "
									+ vsite);
						} else {
							XFileEnumeration fileEnum =
								listDirOutcome.getListing().elements();
							while (fileEnum.hasMoreElements()) {
								XFile xfile = (XFile) fileEnum.nextElement();
								RemoteFile file =
									new RemoteFile(
										xfile,
										parent,
										parent.getFileSystem());
								listing.add(file);
							}
						}
						table.put(parent, listing);
					}
				}
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Could not get directory listing ", e);
			}
		}

		// notify all my observers on task end
		notifyObservers(this, table);
	}
}
