package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AAIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.GetActionStatus;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.GetActionStatus_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.ExecuteContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    August 15, 2001
 *@version    $Id: GetNewJobStatus.java,v 1.3 2003/06/30 06:57:39 rrate Exp $
 */
public class GetNewJobStatus extends ObservableNodeThread {

	static ResourceBundle res =
		ResourceBundle.getBundle("com.pallas.unicore.requests.ResourceStrings");

	private Hashtable serialOutcomes = new Hashtable();

	private UserDefaults userDefaults = ResourceManager.getUserDefaults();

	/**
	 *@param  node  Description of the Parameter
	 */
	public GetNewJobStatus(ContainerNode node) {
		super(node);
		logger.setLevel(Level.FINEST);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  jobContainer  Description of the Parameter
	 *@param  vsite         Description of the Parameter
	 *@return               Description of the Return Value
	 */
	private AbstractJob buildJobStatusRequest(
		JobContainer jobContainer,
		Vsite vsite) {
		if (jobContainer.getStatus() != null
			&& jobContainer.getStatus().isEquivalent(AbstractActionStatus.DONE)) {
			return null;
		}
		AbstractJob serviceJob =
			new AbstractJob(
				servicePrefix
					+ "_GetJobStatus"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				ResourceManager.getUser(vsite));

		buildTaskStatusRequest(jobContainer, serviceJob, vsite);
		return serviceJob;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  container   Description of the Parameter
	 *@param  serviceJob  Description of the Parameter
	 *@param  vsite       Description of the Parameter
	 */
	private void buildTaskStatusRequest(
		ActionContainer container,
		AbstractJob serviceJob,
		Vsite vsite) {

		if (container.getStatus() != null
			&& container.getStatus().isEquivalent(AbstractActionStatus.DONE)) {
			return;
		}

		AbstractAction significantTask = container.getActionGroup();
		if (container instanceof ExecuteContainer) {
			ExecuteContainer execCont = (ExecuteContainer) container;
			significantTask = execCont.getExecuteTask();
		}
		GetActionStatus getStatus =
			new GetActionStatus(
				container
					+ "_GetStatus"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				significantTask.getId());
		logger.info(
			"Building status request for task: "
				+ significantTask
				+ " in container "
				+ container);
		serviceJob.add(getStatus);

		if (container instanceof GroupContainer) {
			GroupContainer groupContainer = (GroupContainer) container;
			Vector tasks = groupContainer.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				ActionContainer task = (ActionContainer) tasks.elementAt(i);
				if (task instanceof JobContainer) {
					JobContainer job = (JobContainer) task;
					AbstractJob subJob =
						buildJobStatusRequest(job, job.getVsite());
					if (subJob != null) {
						serviceJob.add(subJob);
					}
				} else {
					buildTaskStatusRequest(task, serviceJob, vsite);
				}
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	public void run() {

		ContainerNode jobNode = (ContainerNode) getNode();
		jobNode.setState(ContainerNode.STATE_UPDATING);

		JobContainer jobContainer = (JobContainer) jobNode.getActionContainer();

		PanelNode vsiteNode = (PanelNode) jobNode.getParent();
		Vsite vsite = null;
		// if we have a vsite node it's safest to use the underlying vsite object
		if (vsiteNode != null) {
			vsite = (Vsite) vsiteNode.getUserObject();
		}
		// service jobs won't have vsite nodes, so take the container's vsite
		else {
			vsite = jobContainer.getVsite();
		}

		AbstractJob statusRequest = buildJobStatusRequest(jobContainer, vsite);

		if (statusRequest == null) {
			notifyObservers(this, jobNode);
			return;
		}
		try {
			Reply reply =
				nonPolling(
					statusRequest,
					vsite,
					ResourceManager.getUser(vsite));

			if (reply == null || !(reply instanceof RetrieveOutcomeReply)) {
				throw new Exception(res.getString("NO_VSITE") + vsite);
			} else {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] serializedOutcome = roReply.getOutcome();
				if (serializedOutcome != null) {
					Outcome outcome =
						(Outcome) new ObjectInputStream(
							new ByteArrayInputStream(serializedOutcome))
							.readObject();
					if (outcome instanceof AbstractJob_Outcome) {
						serializeOutcome(
							ConsignForm.convertFrom(
								(AbstractJob_Outcome) outcome));
						Enumeration enum = serialOutcomes.elements();
						while (enum.hasMoreElements()) {
							GetActionStatus_Outcome next =
								(GetActionStatus_Outcome) enum.nextElement();
							spreadStatusByContainer(next, jobContainer);
						}
					} else {
						throw new Exception(res.getString("OUTCOME_ERROR"));
					}
				}
			}

		} catch (Exception e) {
			UserMessages.warning(
				"Updating status for job " + jobContainer + " failed.",
				e.getLocalizedMessage());
			logger.log(
				Level.SEVERE,
				"Could not get new job statut for job " + jobContainer,
				e);
		}

		notifyObservers(this, jobNode);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  ajOutcome  Description of the Parameter
	 */
	private void serializeOutcome(AbstractJob_Outcome ajOutcome) {
		OutcomeEnumeration oen = ajOutcome.getOutcomes();
		while (oen.hasMoreElements()) {
			Outcome next = oen.nextElement();
			if (next instanceof GetActionStatus_Outcome) {
				if (((GetActionStatus_Outcome) next).getActionIdentifier()
					!= null) {
					serialOutcomes.put(
						((GetActionStatus_Outcome) next).getActionIdentifier(),
						((GetActionStatus_Outcome) next));
				} else {
					logger.severe(
						"GetActionStatus failed for: "
							+ ((GetActionStatus_Outcome) next).getId().getName()
							+ " Reason: "
							+ next.getReason());
				}
			} else if (next instanceof AbstractJob_Outcome) {
				serializeOutcome((AbstractJob_Outcome) next);
			}
		}
	}

	

	/**
	 *  Description of the Method
	 *
	 *@param  statusOutcome   Description of the Parameter
	 *@param  groupContainer  Description of the Parameter
	 *@return                 Description of the Return Value
	 */
	private void spreadStatusByContainer(
		GetActionStatus_Outcome statusOutcome,
		ActionContainer container) {
		
		AAIdentifier id = container.getIdentifier();
		if(container instanceof ExecuteContainer) {
			ExecuteContainer execCont =(ExecuteContainer)container;
			id = execCont.getExecuteTask().getId();
		}
		
		if (statusOutcome
			.getActionIdentifier()
			.equals(id)) {
			container.setStatus(statusOutcome.getActionStatus());
		}
		if (container instanceof GroupContainer) {
			GroupContainer groupContainer = (GroupContainer) container;
			Vector tasks = groupContainer.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				ActionContainer task = (ActionContainer) tasks.elementAt(i);
				spreadStatusByContainer(statusOutcome, task);
			}
		}
	}

}
