package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.ObjectInputStream;
import java.util.ResourceBundle;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyActionOutcome;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcome;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.DeleteAll;
import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Thomas Kentemich
 *@created    August 15, 2001
 *@version    $Id: GetOutcome.java,v 1.3 2003/06/30 06:58:00 rrate Exp $
 */
public class GetOutcome extends ObservableNodeThread {

	static ResourceBundle res =
		ResourceBundle.getBundle("com.pallas.unicore.requests.ResourceStrings");
	private String nspaceString = FileStorage.NSPACE_STRING;

	/**
	 *  Constructor for the GetOutcome object
	 *
	 *@param  node  Description of the Parameter
	 */
	public GetOutcome(ContainerNode node) {
		super(node);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  root  Description of Parameter
	 *@param  jc    Description of Parameter
	 *@return       Description of the Returned Value
	 */
	private AbstractJob assembleRecursiveJobOutcome(
		AbstractJob root,
		JobContainer jc) {
		// must construct an abstract job wrapper; consign it in non-polling mode

		Vsite vsite = jc.getVsite();
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "_JobStatus_"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				ResourceManager.getUser(vsite));
		CopyActionOutcome cao =
			new CopyActionOutcome(
				servicePrefix
					+ "_CopyActionOutcome_"
					+ ResourceManager.getNextObjectIdentifier(),
				root.getId());
		job.add(cao);
		// append the UPL directory of the CopyActionOutcome in order to find the streamed files
		jc.setLastUpdateUPLDirectory(cao.getUPLDirectoryName());
		return job;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		try {
			//generating the job
			//first, the " GetJobs" task
			JobContainer jobContainer =
				(JobContainer) getNode().getUserObject();

			PanelNode vsiteNode = (PanelNode) getNode().getParent();
			Vsite vsite = null;
			// if we have a vsite node it's safest to use the underlying vsite object
			if (vsiteNode != null) {
				vsite = (Vsite) vsiteNode.getUserObject();
			}
			// service jobs won't have vsite nodes, so take the container's vsite
			else {
				vsite = jobContainer.getVsite();
			}
			RetrieveOutcome ro =
				new RetrieveOutcome(
					(AJOIdentifier) jobContainer.getIdentifier(),
					vsite);

			Connection connection =
				ResourceManager.getConnectionManager().lookupConnection(
					vsite,
					ResourceManager.getUser(vsite));
			Reply troReply = connection.sendMessage(ro, null);
			int returnCode = troReply.getLastEntry().getReturnCode();

			// now analyze outcome
			AbstractJob_Outcome aj_out = null;
			if (troReply == null
				|| !(troReply instanceof RetrieveOutcomeReply)) {
				throw new Exception(res.getString("OUTCOME_ERROR"));
			} else {
				if (jobContainer.getLastUpdateUPLDirectory() != null) {
					DeleteAll.deleteAll(
						new File(
							Connection.getSessionDir()
								+ File.separator
								+ jobContainer.getLastUpdateUPLDirectory()));
				}
				RetrieveOutcomeReply roReply =
					(RetrieveOutcomeReply) connection.getReply();

				// deserialize the Outcome Object
				Outcome outcome =
					(Outcome) new ObjectInputStream(new ByteArrayInputStream(roReply
						.getOutcome()))
						.readObject();
				if (outcome instanceof AbstractJob_Outcome) {
					aj_out =
						ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
				} else {
					throw new Exception(res.getString("OUTCOME_ERROR"));
				}
			}

			// if job is still executing send a CopyActionOutcome request to
			// retrieve stdout/stderr as streamed files
			if (jobContainer
				.getStatus()
				.isEquivalent(AbstractActionStatus.EXECUTING)) {
				AbstractJob job =
					assembleRecursiveJobOutcome(
						(AbstractJob) jobContainer.getActionGroup(),
						jobContainer);
				troReply =
					nonPolling(job, vsite, ResourceManager.getUser(vsite));
			}

			// put outcome into outcome manager after streamed files have arrived!
			OutcomeManager.setOutcome(aj_out, jobContainer);

		} catch (Exception e) {
			UserMessages.warning(
				"Getting outcome for job " + getNode() + " failed.",
				e.getLocalizedMessage());
			logger.log(
				Level.SEVERE,
				"Getting outcome for job " + getNode() + " failed.",
				e);
		}

		// notify all my observers on task end
		notifyObservers(this, getNode());
	}

}
