package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyJob;
import org.unicore.ajo.GetJobs;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.CopyJob_Outcome;
import org.unicore.outcome.GetJobs_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.sets.AJOIdentifierEnumeration;
import org.unicore.sets.AJOIdentifierSet;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.VectorSorter;

/**
 *  Class that returns a Vector containing all running jobs on a specific usite
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    August 15, 2001
 *@version    $Id: GetRunningJobs.java,v 1.3 2003/06/30 06:57:59 rrate Exp $
 */
public class GetRunningJobs extends ObservableNodeThread {

	/**
	 *  Description of the Field
	 */
	static ResourceBundle res =
		ResourceBundle.getBundle("com.pallas.unicore.requests.ResourceStrings");
	private Vector knownJobs;

	/**
	 *  Creates new GetJobs
	 *
	 *@param  node  Description of the Parameter
	 */
	public GetRunningJobs(PanelNode node) {
		super(node);
	}

	/**
	 *  Gets the JobCopy attribute of the GetRunningJobs object
	 *
	 *@param  jobIds  Description of Parameter
	 *@return         The AbstractJobs value
	 */
	private Vector getAbstractJobs(AJOIdentifierSet jobIds) throws Exception {
		// build return vector
		Vector abstractJobs = new Vector();

		if (jobIds == null) {
			return abstractJobs;
		}

		// build a job to get AbstractJob objects belonging to ids
		Vsite vsite = (Vsite) getNode().getUserObject();
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "_CopyJobs"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				ResourceManager.getUser(vsite));
		AJOIdentifierEnumeration idEnum = jobIds.elements();
		int jobCount = 0;
		while (idEnum.hasMoreElements()) {
			AJOIdentifier id = (AJOIdentifier) idEnum.nextElement();
			CopyJob cj =
				new CopyJob(
					servicePrefix
						+ "_CopyJob_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					id);
			job.add(cj);
			jobCount++;
		}
		if (jobCount == 0) {
			return abstractJobs;
		}

		// submit the job in non-polling mode
		Reply roReply = nonPolling(job, vsite, ResourceManager.getUser(vsite));

		// now analyze outcome
		if (!(roReply instanceof RetrieveOutcomeReply)) {
			UserMessages.warning(
				res.getString("NO_VSITE") + vsite,
				roReply.getLastEntry().getComment());
		} else {
			Outcome outcome = null;
			// deserialize the Outcome Object
			outcome =
				(Outcome) new ObjectInputStream(
					new ByteArrayInputStream(((RetrieveOutcomeReply) roReply)
					.getOutcome()))
					.readObject();

			if (outcome instanceof AbstractJob_Outcome) {
				// get AbstractJob object from outcome
				AbstractJob_Outcome aj_out =
					ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
				OutcomeEnumeration oen = aj_out.getOutcomes();
				while (oen.hasMoreElements()) {
					Outcome next = oen.nextElement();
					if (next instanceof CopyJob_Outcome) {
						AbstractJob originalJob =
							((CopyJob_Outcome) next).getJob();
						// do not add service jobs
						String jobName = originalJob.getAJOId().getName();
						int startIndex = jobName.indexOf(servicePrefix, 0);
						if (originalJob != null && startIndex == -1) {
							abstractJobs.add(originalJob);
						}
					}
				}
			}
		}

		return abstractJobs;
	}

	/**
	 *  Gets the JobIds attribute of the GetRunningJobs object
	 *
	 *@return    The JobIds value
	 */
	private AJOIdentifierSet getJobIds() throws Exception {
		GetJobs gj =
			new GetJobs(
				servicePrefix
					+ "_GetJobs("
					+ getNode()
					+ ")"
					+ ResourceManager.getNextObjectIdentifier(),
				null);

		Vsite vsite = (Vsite) getNode().getUserObject();
		User user = ResourceManager.getUser(vsite);

		// must construct an abstract job wrapper; consign it in non-polling mode
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "_GetJobs"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				user);
		job.add(gj);

		Reply roReply = nonPolling(job, vsite, user);

		// now analyze outcome
		if (roReply == null) {
			UserMessages.warning(res.getString("NO_VSITE") + vsite);
			return null;
		}

		if (!(roReply instanceof RetrieveOutcomeReply)) {
			UserMessages.warning(
				res.getString("NO_VSITE") + vsite,
				roReply.getLastEntry().getComment());
			return null;
		} else {
			// deserialize the Outcome Object
			Outcome outcome =
				(Outcome) new ObjectInputStream(
					new ByteArrayInputStream(((RetrieveOutcomeReply) roReply)
					.getOutcome()))
					.readObject();

			if (outcome instanceof AbstractJob_Outcome) {

				AbstractJob_Outcome aj_out =
					ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

				GetJobs_Outcome gjo =
					(GetJobs_Outcome) aj_out.getOutcome(gj.getId());

				return gjo.getJobs();
			}
		}
		return null;
	}

	/**
	 *  Gets the KnownJobs attribute of the GetRunningJobs object
	 *
	 *@return    The KnownJobs value
	 */
	public Vector getKnownJobs() {
		return knownJobs;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  job  Description of Parameter
	 *@return      Description of the Returned Value
	 */
	private JobContainer reconstructContainer(AbstractJob job) {
		// build final container
		JobContainer container = JobContainer.decode(job.getExtraInformation());
		if (container != null) {
			container.setIdentifier(job.getId());
		}
		return container;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		try {
			getNode().setState(ContainerNode.STATE_UPDATING);
			// first get ids of jobs known to njs
			AJOIdentifierSet jobIds = getJobIds();

			// get copy of original AbstractJob objects from njs
			Vector abstractJobs = getAbstractJobs(jobIds);

			// reconstruct containers from AbstractJob objects
			knownJobs = new Vector();
			for (int i = 0; i < abstractJobs.size(); i++) {
				AbstractJob job = (AbstractJob) abstractJobs.elementAt(i);
				JobContainer container = reconstructContainer(job);
				// if container is null it could not be decoded
				if (container != null) {
					knownJobs.add(container);
				}
			}

			VectorSorter.sort(knownJobs);
		} catch (Exception e) {
			logger.log(
				Level.SEVERE,
				"Getting AJO Identifiers for " + getNode() + " failed.",
				e);
		}

		// notify all my observers on task end
		notifyObservers(this, getNode());
	}

}
