package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyPortfolioToOutcome;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.CopyPortfolioToOutcome_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Thomas Kentemich
 *@created    August 15, 2001
 *@version    $Id: GetSpooledFiles.java,v 1.3 2003/06/30 06:57:58 rrate Exp $
 */
public class GetSpooledFiles extends ObservableRequestThread {
	static ResourceBundle res =
		ResourceBundle.getBundle("com.pallas.unicore.requests.ResourceStrings");
	private AbstractJob_Outcome ajOut;
	private FileExport export;
	private JobContainer parent;
	private ContainerNode parentNode;
	private final String servicePrefix = ResourceManager.getServicePrefix();

	/**
	 *@param  parent  Description of Parameter
	 *@param  export  Description of Parameter
	 */
	public GetSpooledFiles(ContainerNode parent, FileExport export) {
		this.parentNode = parent;
		this.parent = export.getContainer().getParentJobContainer();
		this.export = export;
	}

	/**
	 *  Constructor for the GetSpooledFiles object
	 *
	 *@param  parentNode  Description of the Parameter
	 *@param  parent      Description of the Parameter
	 *@param  export      Description of the Parameter
	 */
	public GetSpooledFiles(
		ContainerNode parentNode,
		JobContainer parent,
		FileExport export) {
		this.parent = parent;
		this.export = export;
		this.parentNode = parentNode;
	}

	/**
	 *  Return the FileExpprt object associated with this task
	 *
	 *@return    The fileExport value
	 */
	public FileExport getFileExport() {
		return this.export;
	}

	/**
	 *  Return the Container node for the files to be retrieved
	 *
	 *@return    The node value
	 */
	public ContainerNode getNode() {
		return this.parentNode;
	}

	/**
	 *  Description of the Method
	 */
	public void run() {
		try {

			//generating the job
			if ((export == null)) {
				throw new Exception("No file export found in job " + parent);
			}
			parentNode.setState(ContainerNode.STATE_UPDATING);

			//           Vsite vsite = (Vsite)((DefaultMutableTreeNode)parentNode.getParent()).getUserObject();
			Vsite vsite = ResourceManager.getMatchingVsite(parent.getVsite());
			if (vsite == null) {
				logger.severe(
					"Cannot obtain matching Vsite from cache"
						+ vsite.getName());
				return;
			}
			User user = parent.getUser();
			AJOIdentifier id = (AJOIdentifier) parent.getIdentifier();
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_UnSpool_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					user);
			NamedResourceSet siteResources =
				ResourceManager.getResourceSet(vsite);
			if (siteResources.isSiteDefault()) {
				logger.severe(
					"Cannot obtain site specific ResourceSet from Vsite= "
						+ vsite.getName());
				return;
			} else {
				logger.info(
					"Found site specific ResourceSet for Vsite= "
						+ vsite.getName());
			}
			org.unicore.resources.Spool spoolResource =
				(org.unicore.resources.Spool) siteResources.findResource(
					new org.unicore.resources.Spool());
			if (spoolResource == null) {
				logger.severe(
					"Cannot obtain spool space resource from Vsite= "
						+ vsite.getName());
				return;
			} else {
				logger.info(
					"Found spool space resource for Vsite= " + vsite.getName());
			}
			Vector spoolOutcomes =
				OutcomeManager.getOutcomeEntries(export.getSpoolTask());
			if (spoolOutcomes.size() <= 0) {

				// no Outcome
				logger.severe(
					"Cannot find outcome of spool task for file: "
						+ export.getSourceName());
				return;
			}
			for (int i = 0; i < spoolOutcomes.size(); i++) {
				CopyPortfolioToOutcome unspool =
					new CopyPortfolioToOutcome(
						servicePrefix
							+ "_CopyPF_("
							+ id.getName()
							+ ")"
							+ ResourceManager.getNextObjectIdentifier());
				PutPortfolio_Outcome ppo =
					(PutPortfolio_Outcome) ((OutcomeEntry) spoolOutcomes
						.elementAt(i))
						.getOutcome();

				// unspool.setTarget(export.getSpoolPortfolio().getId());
				unspool.setTarget(ppo.getPortfolio());
				unspool.setResources(new ResourceSet(spoolResource));
				job.add(unspool);
			}
			Reply roReply =
				nonPolling(job, vsite, user, export, export.getFileMode());

			// now analyze outcome
			if (!(roReply instanceof RetrieveOutcomeReply)) {
				throw new Exception(
					res.getString("NO_VSITE")
						+ vsite
						+ "\nReason: "
						+ roReply.getLastEntry().getComment());
			} else {

				// deserialize the Outcome Object
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(
							((RetrieveOutcomeReply) roReply)
						.getOutcome()))
						.readObject();
				if (outcome instanceof AbstractJob_Outcome) {
					ajOut =
						ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
					OutcomeEnumeration oen = ajOut.getOutcomes();
					while (oen.hasMoreElements()) {
						Outcome next = oen.nextElement();
						if (next instanceof CopyPortfolioToOutcome_Outcome) {
							AbstractActionStatus status = next.getStatus();
							if (status
								.isEquivalent(
									AbstractActionStatus.NOT_SUCCESSFUL)) {
								logger.severe(
									"CopyPortfolio failed: "
										+ next.getReason());
							}
						}
					}
				} else {
					throw new Exception(res.getString("OUTCOME_ERROR"));
				}
			}
		} catch (Exception e) {
			UserMessages.warning(
				"Could not get spooled files for job " + parent,
				e.getLocalizedMessage());
			logger.log(
				Level.SEVERE,
				"Could not get spooled files for job " + parent,
				e);
		}

		// notify all my observers on task end
		notifyObservers(this, ajOut);
	}
}
