package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.StreamCorruptedException;
import java.security.KeyStoreException;
import java.security.SignatureException;
import java.util.Date;
import java.util.logging.Level;

import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.Outcome;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.ConsignJobReply;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcome;
import org.unicore.upl.RetrieveOutcomeAck;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;

import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.connection.GSConnection;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;

/**
 *@author     Ralf Ratering
 *@created    August 15, 2001
 *@version    $Id: ObservableNodeThread.java,v 1.1 2002/01/18 15:34:44 rrate Exp
 *      $
 */
public abstract class ObservableRequestThread
         extends ObservableThread {
    private boolean interrupted;


    /**
     *  Sets the interrupted attribute of the ConsignAJO object
     *
     *@param  interrupted  The new interrupted value
     */
    public void setInterrupted(boolean interrupted) {
        this.interrupted = interrupted;
    }


    /**
     *  Submit a job in non-polling mode.
     *
     *@param  job    the job to submit
     *@param  vsite  target Vsite
     *@param  user   consigner of the job
     *@return        reply object
     */
    public Reply nonPolling(AbstractJob job, Vsite vsite, User user) {
        Reply reply = null;
        try {
            Connection connection = ResourceManager.getConnectionManager().lookupConnection(
                    vsite,
                    ResourceManager.getUser(vsite));
            reply = doRequest(job, vsite, user, connection);
        }
        catch(KeyStoreException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(StreamCorruptedException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(ClassNotFoundException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(IOException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(SignatureException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(Exception e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        return reply;
    }


    /**
     *  Description of the Method
     *
     *@param  job          the job to submit
     *@param  vsite        target Vsite
     *@param  user         consigner of the job
     *@param  directStore  OutputStream for directly storing streamed files
     *@param  isAscii      Description of the Parameter
     *@return              reply object
     *@return              reply value
     */
    public Reply nonPolling(AbstractJob job, Vsite vsite, User user,
            FileExport directStore, boolean isAscii) {
        Reply reply = null;
        try {

            // CAUTION: This will only work for single files in the stream !!!

//	    Connection connection = ResourceManager.getConnectionManager().lookupConnection(
            //                  vsite,
            //                ResourceManager.getUser(vsite));
            //	    connection.setIsAscii(isAscii);
//	    connection.setDirectOutput(directStore);
            Connection connection = null;
            if(vsite.getType().getDescription().equals("GS")) {
                connection = new GSConnection(vsite.getAddress(),
                        ResourceManager.getUser(vsite), directStore, isAscii);
            }
            else {
                connection = new Connection(vsite.getAddress(),
                        ResourceManager.getUser(vsite), directStore, isAscii);
            }
            reply = doRequest(job, vsite, user, connection);
        }
        catch(KeyStoreException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(StreamCorruptedException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(ClassNotFoundException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(IOException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(SignatureException e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        catch(Exception e) {
            logger.log(Level.SEVERE, "Non-polling request failed.", e);
        }
        return reply;
    }



    /**
     *  Description of the Method
     *
     *@param  job                           the job to submit
     *@param  vsite                         target Vsite
     *@param  user                          consigner of the job
     *@param  connection                    Connection to use
     *@return                               reply object
     *@exception  StreamCorruptedException  Description of Exception
     *@exception  ClassNotFoundException    Description of Exception
     *@exception  IOException               Description of Exception
     *@exception  KeyStoreException         Description of the Exception
     *@exception  SignatureException        Description of the Exception
     *@exception  Exception                 Description of the Exception
     */
    private Reply doRequest(AbstractJob job, Vsite vsite, User user,
            Connection connection)
             throws StreamCorruptedException, ClassNotFoundException,
            IOException, KeyStoreException, SignatureException,
            Exception {
        interrupted = false;

        // Limit the lifetime of Service AJO's default 30 s
        job.setTerminationTime(new Date(System.currentTimeMillis() + connection.getAjoLifetime()));

        ConsignJob cj = JobConverter.convert(job, false, true);
        Reply reply = connection.sendMessage(cj, null);
 

        // request has been switched to polling due to NJS time out
        if(reply instanceof ConsignJobReply) {
			int returnCode = reply.getLastEntry().getReturnCode();
            logger.info(
                    "Switching to polling mode for vsite: " +
                    vsite.getName());
            Outcome result = null;
            while((result == null ||
                    !result.getStatus().isEquivalent(
                    AbstractActionStatus.DONE)) &&
                    !interrupted) {
                try {
                    Thread.currentThread().sleep(5000);
                }
                catch(InterruptedException e) {
                }
                logger.fine("Polling vsite: " + vsite.getName());
                RetrieveOutcome ro = new RetrieveOutcome(job.getAJOId(), vsite);
                reply = connection.sendMessage(ro, null);
                returnCode = reply.getLastEntry().getReturnCode();
                if(returnCode != 0) {
                    throw new IOException("Request failed.");
                }
                if(reply instanceof RetrieveOutcomeReply) {
                    RetrieveOutcomeReply roReply = (RetrieveOutcomeReply)connection.getReply();
                    result = (Outcome)(new ObjectInputStream(new ByteArrayInputStream(roReply.getOutcome()))).readObject();
                }
            }

            // acknowledge outcome
            RetrieveOutcomeAck roa = new RetrieveOutcomeAck(job.getAJOId(), vsite);

            connection.sendMessage(roa, null);
        }
        else if(!(reply instanceof RetrieveOutcomeReply)) {
            UnicoreResponse resp = reply.getLastEntry();
            logger.severe(
                    "Error consigning an AJO:" + resp.getReturnCode() +
                    ", Reason: " + resp.getComment());
        }
        return reply;
    }
}
