package com.pallas.unicore.requests;

import java.util.Vector;

import org.unicore.Vsite;

import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.UserMessages;

/**
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    October 5, 2001
 *@version    $Id: SiteDetector.java,v 1.2 2003/06/30 06:57:56 rrate Exp $
 */
public class SiteDetector extends ObservableThread implements IObserver {
	private static boolean isRunning = false;
	private Vector runningThreads = new Vector();

	/**
	 *  Description of the Method
	 *
	 *@param  theObserved  Description of Parameter
	 *@param  changeCode   Description of Parameter
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {
		// ignore returning threads that have been canceled
		if (!runningThreads.contains(theObserved)) {
			return;
		}

		// pass event to observers
		notifyObservers(theObserved, changeCode);

		if (theObserved instanceof GetUsites) {
			logger.info("Detecting usites finished.");
			Vector usites = (Vector) changeCode;
			for (int i = 0; i < usites.size(); i++) {
				logger.info("Found Usite: " + (Usite) usites.elementAt(i));
				GetVsites getVsites =
					new GetVsites((Usite) usites.elementAt(i));
				runningThreads.add(getVsites);
				getVsites.addObserver(this);
				getVsites.start();
			}
		} else if (theObserved instanceof GetVsites) {
			logger.info(
				"Detecting vsites for "
					+ ((GetVsites) theObserved).getUsite()
					+ " finished.");
			Vector vsites = (Vector) changeCode;
			for (int i = 0; i < vsites.size(); i++) {
				Vsite vsite = (Vsite) vsites.elementAt(i);
				logger.info("Found Vsite: " + vsite);
				if (ResourceManager.isVsiteAvailable(vsite)) {
					GetResources getResources = new GetResources(vsite);
					runningThreads.add(getResources);
					getResources.addObserver(this);
					getResources.start();
				}
			}
		} else if (theObserved instanceof GetResources) {
			logger.info(
				"Returned from getting resource set for vsite: "
					+ ((GetResources) theObserved).getVsite());
		}

		runningThreads.remove(theObserved);

		// tell observer about running threads left
		notifyObservers(this, new Integer(runningThreads.size()));
		isRunning = runningThreads.size() > 0;
	}

	/**
	 *  Main processing method for the SiteDetector object
	 */
	public void run() {
		if (ResourceManager.isOffline()) {
			isRunning = false;
			return;
		}
		if (isRunning) {
			UserMessages.info(
				"Resource update is already running."
					+ "\nPlease wait until last run is finished before re-loading again.");
			return;
		}
		isRunning = true;
		logger.info(
			"Loading usite, vsite and resource information into cache.");
		startThreads();
	}

	/**
	 *  Description of the Method
	 */
	private void startThreads() {
		logger.info("Reading usites from usite page.");
		GetUsites getUsites = new GetUsites();
		runningThreads.add(getUsites);
		getUsites.addObserver(this);
		getUsites.start();
	}

	/**
	 *  Description of the Method
	 */
	public void stopThreads() {
		isRunning = false;
		runningThreads.clear();
		notifyObservers(this, new Integer(runningThreads.size()));
	}

}
