package com.pallas.unicore.resourcemanager;

import java.util.Date;
import java.util.Vector;

import org.unicore.AAIdentifier;
import org.unicore.outcome.AbstractActionStatus;

import com.pallas.unicore.container.ActionContainer;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    4. September 2002
 *@version    $Id: DetailsEntry.java,v 1.2 2003/06/30 06:58:39 rrate Exp $
 */
public class DetailsEntry extends Vector implements Comparable {
	private static int compareIndex = 0;
	private static int secondIndex = 0;

	/**
	 *  Sets the sortingIndex attribute of the RowData object
	 *
	 *@param  _compareIndex  The new compareIndex value
	 *@param  _secondIndex   The new compareIndex value
	 */
	public static void setCompareIndex(int _compareIndex, int _secondIndex) {
		compareIndex = _compareIndex;
		secondIndex = _secondIndex;
	}
	private AAIdentifier action;
	private ActionContainer container;
	private String description;
	private String iteration;
	private String reason;
	private AbstractActionStatus status;
	private Date timeStamp;

	/**
	 *  Constructor for the DetailsEntry object
	 *
	 *@param  timeStamp    Description of the Parameter
	 *@param  status       Description of the Parameter
	 *@param  reason       Description of the Parameter
	 *@param  container    Description of the Parameter
	 *@param  description  Description of the Parameter
	 *@param  iteration    Description of the Parameter
	 *@param  action       Description of the Parameter
	 */
	public DetailsEntry(
		Date timeStamp,
		ActionContainer container,
		String description,
		AbstractActionStatus status,
		String reason,
		String iteration,
		AAIdentifier action) {
		this.timeStamp = timeStamp;
		this.container = container;
		this.description = description;
		this.status = status;
		this.reason = reason;
		this.iteration = iteration;
		this.action = action;

		add(timeStamp);
		add(container);
		add(description);
		add(status);
		add(reason);
		add(iteration);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  i  Description of the Parameter
	 *@return    Description of the Return Value
	 */
	public Comparable comparableAt(int i) {
		if (elementAt(i) instanceof ActionContainer
			|| elementAt(i) instanceof AbstractActionStatus) {
			return elementAt(i).toString();
		}
		return (Comparable) elementAt(i);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public int compareTo(Object object) {
		int value =
			comparableAt(compareIndex).compareTo(
				((DetailsEntry) object).comparableAt(compareIndex));
		if (value == 0 && compareIndex != secondIndex) {
			value =
				comparableAt(secondIndex).compareTo(
					((DetailsEntry) object).comparableAt(secondIndex));
		}
		return value;
	}

	/**
	 *  Gets the container attribute of the DetailsEntry object
	 *
	 *@return    The container value
	 */
	public ActionContainer getContainer() {
		return container;
	}

	/**
	 *  Gets the iteration attribute of the DetailsEntry object
	 *
	 *@return    The iteration value
	 */
	public String getIteration() {
		return iteration;
	}

	/**
	 *  Gets the reason attribute of the DetailsEntry object
	 *
	 *@return    The reason value
	 */
	public String getReason() {
		return reason;
	}

	/**
	 *  Gets the status attribute of the DetailsEntry object
	 *
	 *@return    The status value
	 */
	public AbstractActionStatus getStatus() {
		return status;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		String result = "Name:        " + container.getName() + "\n";
		result += "Description: " + description + "\n";
		result += "Action:      " + action.getName() + "\n";
		result += "Status:      " + status + "\n";
		result += "Time:        " + timeStamp + "\n";
		result += "Iteration:   " + iteration + "\n";
		result += "Reason:      " + reason + "\n";
		return result;
	}

}
