package com.pallas.unicore.resourcemanager;

import java.io.File;
import java.util.Date;

import org.unicore.ajo.AbstractAction;
import org.unicore.outcome.Outcome;

/**
 *  Description of the Class
 *
 *@author     Ralf Ratering
 *@created    4. September 2002
 *@version    $Id: OutcomeEntry.java,v 1.2 2003/06/30 06:58:38 rrate Exp $
 */
public class OutcomeEntry implements Comparable {
	private AbstractAction action;
	private String iteration;
	/**
	 *  Description of the Field
	 */
	protected Outcome outcome;
	private File stdoutFile, stderrFile;

	/**
	 *  Sets the container attribute of the OutcomeEntry object
	 *
	 *@param  action  Description of the Parameter
	 */
	public OutcomeEntry(AbstractAction action) {
		this.action = action;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public int compareTo(Object object) {
		OutcomeEntry entry2 = (OutcomeEntry) object;
		if (outcome == entry2.getOutcome()) {
			return 0;
		}
		if (outcome == null && entry2.getOutcome() != null) {
			return -1;
		}
		if (outcome != null && entry2.getOutcome() == null) {
			return 1;
		}
		Date time1 = outcome.getStatusHistory().getLastTime();
		Date time2 = entry2.getOutcome().getStatusHistory().getLastTime();
		return time1.compareTo(time2);
	}

	/**
	 *  Gets the container attribute of the OutcomeEntry object
	 *
	 *@return    The container value
	 */
	public AbstractAction getAction() {
		return action;
	}

	/**
	 *  Gets the iteration attribute of the OutcomeEntry object
	 *
	 *@return    The iteration value
	 */
	public String getIteration() {
		return iteration;
	}

	/**
	 *  Gets the log attribute of the OutcomeEntry object
	 *
	 *@return    The log value
	 */
	public String getLog() {
		return outcome.getLog().toString();
	}

	/**
	 *  Gets the outcome attribute of the OutcomeEntry object
	 *
	 *@return    The outcome value
	 */
	public Outcome getOutcome() {
		return outcome;
	}

	/**
	 *  Gets the stderrFile attribute of the OutcomeEntry object
	 *
	 *@return    The stderrFile value
	 */
	public File getStderrFile() {
		return stderrFile;
	}

	/**
	 *  Gets the stdoutFile attribute of the OutcomeEntry object
	 *
	 *@return    The stdoutFile value
	 */
	public File getStdoutFile() {
		return stdoutFile;
	}

	/**
	 *  Gets the timeStamp attribute of the OutcomeEntry object
	 *
	 *@return    The timeStamp value
	 */
	public Date getTimeStamp() {
		return outcome.getStatusHistory().getLastTime();
	}

	/**
	 *  Sets the iteration attribute of the OutcomeEntry object
	 *
	 *@param  iteration  The new iteration value
	 */
	public void setIteration(String iteration) {
		this.iteration = iteration;
	}

	/**
	 *  Sets the outcome attribute of the OutcomeEntry object
	 *
	 *@param  outcome  The new outcome value
	 */
	public void setOutcome(Outcome outcome) {
		this.outcome = outcome;
	}

	/**
	 *  Sets the stderrFile attribute of the OutcomeEntry object
	 *
	 *@param  stderrFile  The new stderrFile value
	 */
	public void setStderrFile(File stderrFile) {
		this.stderrFile = stderrFile;
	}

	/**
	 *  Sets the stdoutFile attribute of the OutcomeEntry object
	 *
	 *@param  stdoutFile  The new stdoutFile value
	 */
	public void setStdoutFile(File stdoutFile) {
		this.stdoutFile = stdoutFile;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		return "******** Outcome:   "
			+ outcome
			+ "\n"
			+ "******** Iteration: "
			+ iteration
			+ "\n"
			+ "******** Stdout:    "
			+ stdoutFile
			+ "\n"
			+ "******** Stderr:    "
			+ stdoutFile
			+ "\n";
	}

}
