package com.pallas.unicore.resourcemanager;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.StreamCorruptedException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.net.UnknownHostException;
import java.security.Policy;
import java.security.SecureRandom;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.TimeZone;
import java.util.Vector;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.axis.client.Service;
import org.apache.axis.types.URI;
import org.gridforum.www.ogsi.LocatorType;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.GetResourceDescription;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.GetResourceDescription_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.resources.Resource;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.ListVsites;
import org.unicore.upl.ListVsitesReply;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;
import org.unicore.utility.ConsignForm;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.fujitsu.arcon.upl.ListPorts;
import com.fujitsu.arcon.upl.ListPortsReply;
import com.pallas.ogsa.extensions.ServiceReferenceType;
import com.pallas.ogsa.registry.ISiteRegistry;
import com.pallas.ogsa.registry.SiteRegistryStub;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.SystemDefaults;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.dialogs.DefinePasswordDialog;
import com.pallas.unicore.client.dialogs.UserDefaultsDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.xml.NewXMLFormatter;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.connection.ConnectionManager;
import com.pallas.unicore.connection.URLReader;
import com.pallas.unicore.connection.UnicoreSSLSocketFactory;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.DoNContainer;
import com.pallas.unicore.container.DoRepeatContainer;
import com.pallas.unicore.container.ElseContainer;
import com.pallas.unicore.container.FileContainer;
import com.pallas.unicore.container.HoldJobContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.ServiceContainer;
import com.pallas.unicore.container.ThenContainer;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.requests.SubmitServiceContainer;
import com.pallas.unicore.security.CertificateUtility;
import com.pallas.unicore.security.KeyStoreManager;
import com.pallas.unicore.security.UnicoreSecurityManager;
import com.pallas.unicore.utility.LoggingLevels;
import com.pallas.unicore.utility.UrlEncoder;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.Version;

/**
 *  ResourceManager class keeps track of all global objects as static members.
 *  These objects are made accessible to other classes via static methods, so
 *  that all classes pass objects to each other via the ResourceManager class.
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    August 14, 2001
 *@version    $Id: ResourceManager.java,v 1.37 2003/07/09 12:55:37 thomas Exp $
 */
public class ResourceManager {

	/**
	 *  String as description for resources that have not been sent from a vsite
	 */
	public final static String USER_RESOURCE_DESCRIPTION = "User Resource";

	/**
	 *  logs are written into a new log directory
	 */
	public final static int NEW_LOG_DIRECTORY = 0;

	/**
	 *  logs are written into a file with XML-format
	 */
	public final static int LOG_INTO_XMLFILE = 1;

	/**
	 *  logs are written into a file with simple format
	 */
	public final static int LOG_INTO_PLAINFILE = 2;

	/**
	 *  logs are written into files both with XML-format and with simple format
	 */
	public final static int LOG_INTO_FILES = 3;

	/**
	 *  logs are written to the console
	 */
	public final static int LOG_TO_CONSOLE = 4;

	/**
	 *  Description of the Field
	 */
	public final static String ADD = "add.gif";
	/**
	 *  Description of the Field
	 */
	public final static String ARROW = "arrow.gif";
	/**
	 *  Description of the Field
	 */
	public final static String CHECK = "check.gif";
	/**
	 *  Description of the Field
	 */
	public final static String CLEAR = "clear.gif";
	/**
	 *  Description of the Field
	 */
	public final static String CONNECTION = "estab_conn.gif";
	/**
	 *  Description of the Field
	 */
	public final static String COPY = "copy.gif";
	/**
	 *  Description of the Field
	 */
	public final static String CUT = "cut.gif";
	/**
	 *  Description of the Field
	 */
	public final static String DON = "doN.gif";
	/**
	 *  Description of the Field
	 */
	public final static String DOREPEAT = "doRepeat.gif";
	/**
	 *  Description of the Field
	 */
	public final static String DOWN = "down.gif";
	/**
	 *  Description of the Field
	 */
	public final static String EDIT = "edit.gif";
	/**
	 *  Description of the Field
	 */
	public final static String ELSE = "else.gif";
	/**
	 *  Description of the Field
	 */
	public final static String FETCHOUTPUT = "fetchOutput.gif";
	/**
	 *  Description of the Field
	 */
	public final static String FILEOPERATION = "fileOperation.gif";
	/**
	 *  Description of the Field
	 */
	public final static String HOLDJOB = "holdJob.gif";
	/**
	 *  Description of the Field
	 */
	public final static String IF = "if.gif";
	/**
	 *  Description of the Field
	 */
	public final static String JOBGROUP = "jobGroup.gif";
	/**
	 *  Description of the Field
	 */
	public final static String NEW = "new.gif";
	/**
	 *  Description of the Field
	 */
	public final static String NOCONN = "no_conn.gif";
	/**
	 *  Description of the Field
	 */
	public final static String OPEN = "open.gif";
	/**
	 *  Description of the Field
	 */
	public final static String PASTE = "paste.gif";
	/**
	 *  Description of the Field
	 */
	public final static String REDO = "redo.gif";
	/**
	 *  Description of the Field
	 */
	public final static String REFRESH = "refresh.gif";
	/**
	 *  Description of the Field
	 */
	public final static String REMOVE = "remove.gif";
	/**
	 *  Description of the Field
	 */
	public final static String SAVE = "save.gif";
	/**
	 *  Description of the Field
	 */
	public final static String SAVEAS = "saveAs.gif";
	/**
	 *  Description of the Field
	 */
	public final static String SELECTALL = "selectAll.gif";
	/**
	 *  Description of the Field
	 */
	public final static String SORT = "sort.gif";
	/**
	 *  Description of the Field
	 */
	public final static String STOP = "stop.gif";
	/**
	 *  Description of the Field
	 */
	public final static String SUBMIT = "submit.gif";
	/**
	 *  Description of the Field
	 */
	public final static String THEN = "then.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UNAVAILABLEUSITE = "unavailableUsite.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UNAVAILABLEVSITE = "unavailableVsite.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UNDO = "undo.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UNICORE = "unicore.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UNICOREPROICON = "UNICOREproIcon16x16.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UNKNOWNTASK = "unknownTask.gif";
	/**
	 *  Description of the Field
	 */
	public final static String UP = "up.gif";
	/**
	 *  Description of the Field
	 */
	public final static String USITE = "usite.gif";
	/**
	 *  Description of the Field
	 */
	public final static String VSITE = "vsite.gif";
	/**
	 *  Description of the Field
	 */
	public final static String WAIT = "wait.gif";
	/**
	 *  Description of the Field
	 */
	public final static String X509VERSION1 = "x509Version1.gif";
	/**
	 *  Description of the Field
	 */
	public final static String X509CRITEXTENSION = "x509CritExtension.gif";
	/**
	 *  Description of the Field
	 */
	public final static String X509NONCRITEXTENSION =
		"x509NonCritExtension.gif";
	/**
	 *  Description of the Field
	 */
	public final static String KEY = "key.gif";
	/**
	 *  Description of the Field
	 */
	public final static String NOKEY = "nokey.gif";
	private final static String servicePrefix = "@JMC@";
	private final static String[] imageFilenames =
		{
			ADD,
			ARROW,
			CHECK,
			CLEAR,
			CONNECTION,
			COPY,
			CUT,
			DON,
			DOREPEAT,
			DOWN,
			EDIT,
			ELSE,
			FETCHOUTPUT,
			FILEOPERATION,
			HOLDJOB,
			IF,
			JOBGROUP,
			KEY,
			NEW,
			NOCONN,
			NOKEY,
			OPEN,
			PASTE,
			REDO,
			REFRESH,
			REMOVE,
			SAVE,
			SAVEAS,
			SELECTALL,
			SORT,
			STOP,
			SUBMIT,
			THEN,
			UNAVAILABLEUSITE,
			UNAVAILABLEVSITE,
			UNDO,
			UNICORE,
			UNICOREPROICON,
			UNKNOWNTASK,
			UP,
			USITE,
			VSITE,
			WAIT,
			X509VERSION1,
			X509CRITEXTENSION,
			X509NONCRITEXTENSION };
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.resourcemanager.ResourceStrings");
	private static int logInto = LOG_TO_CONSOLE;

	// used for logging
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.resourcemanager");
	private static boolean offline = false;
	private static Object lock = new Object();

	private static Handler fileXmlHandler = null;
	private static Handler filePlainHandler = null;
	private static Handler consoleHandler = null;
	private static File fileXml = null;
	private static File filePlain = null;

	// Use of path "img"+file.separator not appropriate on Windows
	// when storing img directory in a jar file.
	private static String imagePath = "com/pallas/unicore/img/";
	private static boolean initialized = false;

	// the resource manager delivers an identifier
	private static long objectIdentifier = 0;
	private static ResourceCache resourceCache = new ResourceCache();
	private static LoggingLevels logLevels = new LoggingLevels();
	private static Hashtable userTable = new Hashtable();

	private static SimpleDateFormat dateFormatter =
		new SimpleDateFormat("MM/dd/yyyy"),
		timeFormatter = new SimpleDateFormat("HH:mm:ss"),
		completeFormatter = new SimpleDateFormat("HH:mm:ss MM/dd/yyyy"),
		utcFormatter = new SimpleDateFormat("HH:mm:ss MM/dd/yyyy");
	private static String resourceCacheFilename;

	// reference to current client instance
	private static Client currentInstance;
	private static UnicoreSSLSocketFactory sslFactory;
	private static SecureRandom secureRandom;

	// security objects
	private static KeyStoreManager keystoreManager;
	private static UnicoreSecurityManager securityManager;

	// private static char password[];
	private static String defaultIdentity;

	// get urls with this one...
	private static URLReader urlReader;
	private static SystemDefaults systemDefaults;
	private static UserDefaults userDefaults;
	private static User user;
	private static String project;
	private static String[] usiteNames;
	private static String jarDir;
	private static Image[] image;
	private static Hashtable iconTable;
	private static Hashtable imageTable;
	private static Hashtable filechooserTable;

	// connection stuff
	private static ConnectionManager connectionManager;

	/**
	 *  Constructor for the ResourceManager object
	 */
	private ResourceManager() {
	}

	/**
	 *  Sets the keystoreManager
	 *
	 *@param  manager  The new keystoreManager value
	 */
	public final static void setKeystoreManager(KeyStoreManager manager) {
		keystoreManager = manager;
	}

	/**
	 *  Sets the OfflineMode attribute of the ResourceManager class
	 *
	 *@param  b  The new OfflineMode value
	 */
	public static void setOfflineMode(boolean b) {
		offline = b;
	}

	/**
	 *  Sets the CurrentInstance attribute of the ResourceManager object
	 *
	 *@param  instance  The new CurrentInstance value
	 */
	public static void setCurrentInstance(Client instance) {
		currentInstance = instance;
	}

	/**
	 *  Sets the SSLFactory attribute of the ResourceManager class
	 *
	 *@param  sslfact  The new SSLFactory value
	 */
	public static void setSSLFactory(UnicoreSSLSocketFactory sslfact) {
		sslFactory = sslfact;
	}

	/**
	 *  Sets the secureRandom attribute of the ResourceManager class
	 *
	 *@param  random  The new secureRandom value
	 */
	public static void setSecureRandom(SecureRandom random) {
		secureRandom = random;
	}

	/**
	 *  Sets the user attribute of the ResourceManager class
	 *
	 *@param  user   The new user value
	 *@param  vsite  The new user value
	 */
	public static void setUser(User user, Vsite vsite) {
		userTable.put(vsite, user);
	}

	/**
	 *  Sets the user attribute of the ResourceManager class
	 *
	 *@param  user   The new user value
	 *@param  usite  The new user value
	 */
	public static void setUser(User user, Usite usite) {
		userTable.put(usite, user);
	}

	/**
	 *  Set the relevat security policies for startup-dir, system die and plugin
	 *  dir
	 */
	private static void setAdditionalPolicy() {

		// the directory containing system jar's
		String defaultLibDir = ResourceManager.getJarDirectory();
		defaultLibDir = defaultLibDir + "/*";
		if (!defaultLibDir.startsWith("http")
			&& !defaultLibDir.startsWith("HTTP")
			&& !defaultLibDir.startsWith("file")) {

			// assume file URL
			defaultLibDir = "file:" + defaultLibDir;
		}

		// get medium priority permissions (see unicore.policy)
		ResourceManager.expandPolicy(
			"unicore.system.plugin.dir",
			defaultLibDir);

		// medium trust for plugins from the user plugin dir
		ResourceManager.expandPolicy(
			"unicore.user.plugin.dir",
			"file:" + userDefaults.getPluginDirectory() + "/*");
		ResourceManager.expandPolicy("unicore.dir", Client.getUnicoreDir());
	}

	/**
	 *  Gets the offlineMode attribute of the ResourceManager class
	 *
	 *@return    The offlineMode value
	 */
	public final static boolean getOfflineMode() {

		// offlineMode is  if offline
		return offline;
	}

	/**
	 *  Gets the keystoreManager
	 *
	 *@return    The keystoreManager value
	 */
	public final static KeyStoreManager getKeystoreManager() {
		return keystoreManager;
	}

	/**
	 *  Gets the icon attribute of the ResourceManager class
	 *
	 *@param  name  Description of the Parameter
	 *@return       The icon value
	 */
	public static ImageIcon getIcon(String name) {
		ImageIcon icon = (ImageIcon) iconTable.get(name);
		if (icon == null) {
			icon = (ImageIcon) iconTable.get(UNKNOWNTASK);
		}
		return icon;
	}

	/**
	 *  Gets the icon attribute of the ResourceManager class
	 *
	 *@param  container  Description of the Parameter
	 *@return            The icon value
	 */
	public static ImageIcon getIcon(ActionContainer container) {
		if (container instanceof JobContainer) {
			return getIcon(JOBGROUP);
		} else if (container instanceof DoNContainer) {
			return getIcon(DON);
		} else if (container instanceof DoRepeatContainer) {
			return getIcon(DOREPEAT);
		} else if (container instanceof HoldJobContainer) {
			return getIcon(HOLDJOB);
		} else if (container instanceof IfContainer) {
			return getIcon(IF);
		} else if (container instanceof ThenContainer) {
			return getIcon(THEN);
		} else if (container instanceof ElseContainer) {
			return getIcon(ELSE);
		} else if (container instanceof FileContainer) {
			return getIcon(FILEOPERATION);
		} else if (container instanceof ActionContainer) {

			// if we got here container belongs to a plugin
			return Client.getPluginManager().getIcon(container);
		}
		return getIcon(UNKNOWNTASK);
	}

	/**
	 *  Gets the nextObjectIdentifier attribute of the ResourceManager class
	 *
	 *@return    The nextObjectIdentifier value
	 */
	public static synchronized String getNextObjectIdentifier() {
		return System.currentTimeMillis() + "_" + objectIdentifier++;
	}

	/**
	 *  ResourceManager.java Gets the Offline attribute of the ResourceManager
	 *  class
	 *
	 *@return    The Offline value
	 */
	public static boolean isOffline() {
		return offline;
	}

	/**
	 *  Gets the JarDirectory attribute of the ResourceManager class
	 *
	 *@return    The JarDirectory value
	 */
	public static String getJarDirectory() {
		if (jarDir != null) {
			return jarDir;
		}
		String classFile = "com/pallas/unicore/client/Client.class";

		// dirty, but works....
		URL classDir =
			currentInstance.getClass().getClassLoader().getResource(classFile);

		// if we have been started from a jar, look in jar directory
		// for plugin jars
		if (classDir.toString().startsWith("jar:")) {

			// classDir :=
			// jar:file:/home/rrate/tmp/client31.jar!/com/pallas/unicore/client/resourcemanager/ResourceManager.class
			String tmp = classDir.getFile();

			// cutoff "file:" prefix
			tmp = tmp.substring(tmp.indexOf("file:") + 5);

			// cutoff everything behind "!"
			tmp = tmp.substring(0, tmp.indexOf("!"));

			// cutoff jar name
			jarDir = tmp.substring(0, tmp.lastIndexOf("/"));
			jarDir = UrlEncoder.decodeURL(jarDir);
		}
		// not started from jar -> try to use Jars dir
		else {

			// Assumption classDir :=
			// /home/kdoertel/Unicore/unicore-4.0/classes/com/pallas/unicore/client/Client.class
			String tmp = classDir.getFile();
			String classesDir = "classes/" + classFile;

			// HACK: cutoff everything behind classesDir and add "lib" directory
			if (tmp.endsWith(classesDir)) {
				jarDir = tmp.substring(0, tmp.lastIndexOf(classesDir)) + "lib";
				jarDir = UrlEncoder.decodeURL(jarDir);
			} else {

				// use current working dir
				jarDir = System.getProperty("user.dir");
			}
		}
		return jarDir;
	}

	/**
	 *  Gets the usites attribute of the ResourceManager class
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The usites value
	 */
	public static Usite getUsite(Vsite vsite) {
		return resourceCache.getUsite(vsite);
	}

	/**
	 *  Gets the usites attribute of the ResourceManager class
	 *
	 *@return    The usites value
	 */
	public static Vector getUsites() {
		return resourceCache.getUsites();
	}

	/**
	 *  Gets the vsites attribute of the ResourceManager class
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The vsites value
	 */
	public static Vector getVsites(Usite usite) {
		return resourceCache.getVsites(usite);
	}

	/**
	 *  Gets the resourceSet attribute of the ResourceManager class
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The resourceSet value
	 */
	public static NamedResourceSet getResourceSet(Vsite vsite) {
		return resourceCache.getResourceSet(vsite);
	}

	/**
	 *  Returns the Vsite from the cache matching the characteristics of the
	 *  provided vsite, which may be a different object.
	 *
	 *@param  vsite  Vsite in question
	 *@return        Mathcing vsite or null
	 */
	public static Vsite getMatchingVsite(Vsite vsite) {
		return resourceCache.getMatchingVsite(vsite);
	}

	/**
	 *  Gets the usiteAvailable attribute of the ResourceManager class
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The usiteAvailable value
	 */
	public static boolean isUsiteAvailable(Usite usite) {
		return resourceCache.isUsiteAvailable(usite);
	}

	/**
	 *  Gets the vsiteAvailable attribute of the ResourceManager class
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The vsiteAvailable value
	 */
	public static boolean isVsiteAvailable(Vsite vsite) {
		return resourceCache.isVsiteAvailable(vsite);
	}

	/**
	 *  Gets the specialVsite attribute of the ResourceManager class
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The specialVsite value
	 */
	public static boolean isSpecialVsite(Vsite vsite) {
		return resourceCache.isSpecialVsite(vsite);
	}

	/**
	 *  Gets the usiteErrorMessage attribute of the ResourceManager class
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The usiteErrorMessage value
	 */
	public static String getUsiteErrorMessage(Usite usite) {
		return resourceCache.getUsiteErrorMessage(usite);
	}

	/**
	 *  Gets the vsiteErrorMessage attribute of the ResourceManager class
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The vsiteErrorMessage value
	 */
	public static String getVsiteErrorMessage(Vsite vsite) {
		return resourceCache.getVsiteErrorMessage(vsite);
	}

	/**
	 *  Gets the CurrentInstance attribute of the ResourceManager object
	 *
	 *@return    The CurrentInstance value
	 */
	public static Client getCurrentInstance() {
		return currentInstance;
	}

	/**
	 *  Gets the SystemDefaults attribute of the ResourceManager class
	 *
	 *@return    The SystemDefaults value
	 */
	public static SystemDefaults getSystemDefaults() {
		return systemDefaults;
	}

	/**
	 *  Gets the SSLFactory attribute of the ResourceManager class
	 *
	 *@return    The SSLFactory value
	 */
	public static UnicoreSSLSocketFactory getSSLFactory() {
		return sslFactory;
	}

	/**
	 *  Returns a secure random value
	 *
	 *@return    The secureRandom value
	 */
	public static SecureRandom getSecureRandom() {
		return secureRandom;
	}

	/**
	 *  Gets the SecurityManager attribute of the ResourceManager class
	 *
	 *@return    The SecurityManager value
	 */
	public static UnicoreSecurityManager getSecurityManager() {
		return securityManager;
	}

	/**
	 *  Get reference to UserDefaults
	 *
	 *@return    The UserDefaults value
	 */
	public static UserDefaults getUserDefaults() {
		return userDefaults;
	}

	/**
	 *  Get Prefix for UNICOREpro action Strings
	 *
	 *@return    The Prefix value
	 */
	public static String getServicePrefix() {
		return servicePrefix;
	}

	/**
	 *  Get current Account
	 *
	 *@return    The Project value
	 */
	public static String getProject() {
		return project;
	}

	/**
	 *  Get current User
	 *
	 *@return    The User value
	 */
	public static User getUser() {
		return user;
	}

	/**
	 *  Get current User
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The User value
	 */
	public static User getUser(Vsite vsite) {
		if (vsite == null) {
			return user;
		}

		User returnUser = (User) userTable.get(vsite);
		if (returnUser == null) {
			// try to initialize userTable from UserDefaults
			// which might have not yet happened due to empty ResourceCache
			initUserVsite(vsite);

			if ((returnUser = (User) userTable.get(vsite)) == null) {
				// use identity of corresponding Usite (cf. SetIdentityDialog)
				Usite usite = getUsite(vsite);
				returnUser = getUser(usite);
			}
		}
		return returnUser;
	}

	/**
	 *  Gets the user attribute of the ResourceManager class
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The user value
	 */
	public static User getUser(Usite usite) {
		if (usite == null) {
			return user;
		}

		User returnUser = (User) userTable.get(usite);
		if (returnUser == null) {
			// try to initialize userTable from UserDefaults
			// which might have not yet happened due to empty ResourceCache
			initUserUsite(usite);

			if ((returnUser = (User) userTable.get(usite)) == null) {
				// return default user if nothing special available
				returnUser = user;
			}
		}
		return returnUser;
	}

	/**
	 *  Gets the resourceCache attribute of the ResourceManager class
	 *
	 *@return    The resourceCache value
	 */
	public static ResourceCache getResourceCache() {
		return resourceCache;
	}

	/**
	 *  Gets the connectionManager attribute of the ResourceManager class
	 *
	 *@return    The connectionManager value
	 */
	public static ConnectionManager getConnectionManager() {
		return connectionManager;
	}

	/**
	 *  Gets the image attribute of the ResourceManager class
	 *
	 *@param  name  Description of the Parameter
	 *@return       The image value
	 */
	public static Image getImage(String name) {
		return (Image) imageTable.get(name);
	}

	/**
	 *  Gets a cached SelectorDialog. If the requested dialog does not exist a
	 *  new one will be built.
	 *
	 *@param  vsite         if vsite is null a local SelectorDialog will be
	 *      returned
	 *@param  selectedFile  initially selected File
	 *@return               a SelectorDialog
	 */
	public static SelectorDialog getSelectorDialog(
		File selectedFile,
		Vsite vsite) {
		Object key = vsite;
		if (vsite == null) {
			key = FileStorage.NSPACE_STRING;
		}
		SelectorDialog dialog = (SelectorDialog) filechooserTable.get(key);

		if (dialog == null) {
			dialog = new SelectorDialog(currentInstance, selectedFile, vsite);
			dialog.setSize(800, 600);
			filechooserTable.put(key, dialog);
		} else {
			try {

				// reset chooser settings
				if (!dialog.getLocalEnabled()) {
					dialog.setLocalEnabled(true);
				}
				dialog.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				if (selectedFile != null) {
					dialog.setSelectedFile(selectedFile);
				}

				// remove filters from previous calls
				dialog.removeFileFilters();
				dialog.getSelectorTable().resetValues();
			} catch (FileNotFoundException e) {
				logger.log(Level.SEVERE, "File not found: " + selectedFile, e);
			}
		}
		return dialog;
	}

	/**
	 *  Get a DateFormatter with format "MM/dd/yyyy". Use this method to get a
	 *  format consistent with the rest of the client GUI.
	 *
	 *@return    The dateFormatter value
	 */
	public static SimpleDateFormat getDateFormatter() {
		return dateFormatter;
	}

	/**
	 *  Get a DateFormatter with format "HH:mm:ss". Use this method to get a
	 *  format consistent with the rest of the client GUI.
	 *
	 *@return    The timeFormatter value
	 */
	public static SimpleDateFormat getTimeFormatter() {
		return timeFormatter;
	}

	/**
	 *  Get a DateFormatter with format "HH:mm:ss MM/dd/yyyy". Use this method
	 *  to get a format consistent with the rest of the client GUI.
	 *
	 *@return    The completeFormatter value
	 */
	public static SimpleDateFormat getCompleteFormatter() {
		return completeFormatter;
	}

	/**
	 *  Get a DateFormatter translating local time to UTC. Use this method to
	 *  get a format consistent with the rest of the client GUI.
	 *
	 *@return    The uTCFormatter value
	 */
	public static SimpleDateFormat getUTCFormatter() {
		return utcFormatter;
	}

	/**
	 *  Check if the keystore file already exists. The filename is taken from
	 *  the user defaults.
	 *
	 *@return    true if keystore file exists
	 */
	public static boolean isKeystoreAvailable() {
		String keystoreName = userDefaults.getKeyStoreName();

		File file = new File(keystoreName);

		// if unicorepro keystore exists, return true
		if (file.exists()) {
			return true;
		}
		return false;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  user  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	public static String userInfo(User user) {
		String userinfo = null;
		try {
			userinfo =
				"\nuser alias: "
					+ keystoreManager.getAliasFromCertifcate(
						user.getCertificate())
					+ "\n      cert: "
					+ user.getCertificate().getSubjectDN()
					+ "\n    xlogin: "
					+ user.getXlogin()
					+ "\n   project: "
					+ user.getProject();
		} catch (Exception e) {
			userinfo = "\nuser info not available";
			logger.log(Level.WARNING, userinfo, e);
		}
		return userinfo;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public static boolean vsiteExists(Vsite vsite) {
		return resourceCache.vsiteExists(vsite);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usite  Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public static boolean usiteExists(Usite usite) {
		return resourceCache.usiteExists(usite);
	}

	/**
	 *  Initialization routine tries to load UserDefaults from file
	 *
	 *@param  client  Description of Parameter
	 */
	public static void init(Client client) {
		currentInstance = client;

		// make sure constructor is called only once
		if (initialized) {
			return;
		}

		utcFormatter.setTimeZone(TimeZone.getTimeZone("Etc/UTC"));

		connectionManager = new ConnectionManager();

		// the directory (or jar) where we have been started from
		String classPath = System.getProperty("java.class.path");
		StringTokenizer tokenizer =
			new StringTokenizer(classPath, File.pathSeparator);
		java.net.URL url = null;
		try {
			url = new File(tokenizer.nextToken()).toURL();
			String bas = url.toString();
			ResourceManager.expandPolicy("unicore.startup.dir", url.toString());
		} catch (Exception e) {
			logger.log(
				Level.SEVERE,
				"Could not install security policy for " + url,
				e);
		}
		if (System.getProperty("com.pallas.unicore.security.manager")
			!= null) {
			System.setSecurityManager(new SecurityManager());
		}
		SecurityManager sm = System.getSecurityManager();
		if (sm == null || !(sm instanceof SecurityManager)) {
			initialized = false;

			//        System.exit(-1);
		}
		initialized = true;

		// build system defaults
		if (systemDefaults == null) {
			systemDefaults = new SystemDefaults();
			String filename =
				getJarDirectory() + File.separator + "SystemDefaults.txt";
			logger.info("Loading system defaults from file " + filename);
			if (!systemDefaults.loadFromFile(filename)) {
				UserMessages.warning(
					"Missing system defaults!\nUNICOREpro Client may not work properly!");
			}
		}

		// build new user defaults, these will be loaded from
		// file or newly generated
		if (userDefaults == null) {
			userDefaults = new UserDefaults();
		}
		offline = userDefaults.getStartOffline();
		setAdditionalPolicy();

		// take values from UserDefaults
		redirectLog(
			userDefaults.getRedirect(),
			userDefaults.getLogFormat(),
			logLevels.get(userDefaults.getLogLevel()));
		printVersionInfo();

		// set the required security provider
		// set the SunJSSE for accessing PKCS12 keystores
		//Provider sunjsse = new com.sun.net.ssl.internal.ssl.Provider();
		//Security.addProvider(sunjsse);
		//log.logWrite("Adding new security provider: " + sunjsse.getInfo());
		Security.addProvider(
			new org.bouncycastle.jce.provider.BouncyCastleProvider());
		logger.info("Adding new security provider: ");
		urlReader = new URLReader(currentInstance);

		//       sslFactory = new UnicoreSSLSocketFactory();
		//   sslFactory.setSecureRandom(secureRandom);
		// set system property that allows to read https via the
		// java.net.URL class
		System.setProperty(
			"java.protocol.handler.pkgs",
			"com.sun.net.ssl.internal.www.protocol");

		// this works for JavaWebstart and Windows !!
		URL.setURLStreamHandlerFactory(new URLStreamHandlerFactory() {

			/**
			 *  Description of the Method
			 *
			 *@param  protocol  Description of Parameter
			 *@return           Description of the Returned Value
			 */
			public URLStreamHandler createURLStreamHandler(final String protocol) {
				if (protocol != null && protocol.compareTo("https") == 0) {
					return new com
						.sun
						.net
						.ssl
						.internal
						.www
						.protocol
						.https
						.Handler();
				}
				return null;
			}
		});

		// build new user defaults, these will be loaded from
		// file or newly generated
		//  if(userDefaults == null) {
		//    userDefaults = new UserDefaults();
		//}
		// take values from UserDefaults
		//redirectStdout(userDefaults.getRedirect(), userDefaults.getLogFormat(), logLevels.get(userDefaults.getLogLevel()));

		// build file chooser table as background process
		Thread buildChooser = new Thread() {
			public void run() {
					// build file chooser table
		// containing a local chooser with initially selected job directory
	buildFileChooserTable(userDefaults.getJobDirectory());
			}
		};
		buildChooser.start();

		//        buildFileChooserTable(userDefaults.getJobDirectory());

		defaultIdentity = userDefaults.getDefaultIdentity();
		String unicoreDir = userDefaults.getUnicoreDir();
		resourceCacheFilename =
			unicoreDir + File.separator + "resourceCache.bin";

		// read cached resources
		readCacheFromDisk();
		logger.info("Starting in offline mode: " + offline);
	}

	/**
	 *  Description of the Method
	 */
	public static void initSSLFactory() {

		//        unicoreKeystore.initSSLFactory(sslFactory);
	}

	/**
	 *  Build a new Keystore and generate the user's test certificate
	 *
	 *@param  keystoreName   Description of Parameter
	 *@exception  Exception  Description of the Exception
	 */
	public static void buildTestCertKeystore(String keystoreName)
		throws Exception {
		keystoreManager =
			KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
		// define new password
		DefinePasswordDialog definePasswordDialog =
			new DefinePasswordDialog(
				currentInstance,
				"UNICOREpro: Choose a Password");
		definePasswordDialog.setStatusMessage(
			"Define a password for your new keystore.");
		definePasswordDialog.show();
		char[] passwd = definePasswordDialog.getPassword();
		if (passwd == null) {
			throw new Exception("Building initial keystore aborted by user.");
		}
		keystoreManager.buildEmptyKeystore(keystoreName, passwd, passwd);
		passwd = null;

		Certificate[] caCerts = ServiceManager.getCACertificate();
		X509Certificate userCert =
			(X509Certificate) ServiceManager.getSignedRequest();

		if (userCert == null) {
			logger.log(Level.INFO, "Empty keystore is built.");
			return;
		}

		for (int i = 0; i < caCerts.length; i++) {
			keystoreManager.addTrustedCertificate(caCerts[i], true);
		}

		String alias = CertificateUtility.getCommonName(userCert);
		keystoreManager.replaceKeyCertificate(alias, userCert, true);
		keystoreManager.reWriteKeyStore();

		userDefaults.setDefaultIdentity(alias);
		updateDefaultIdentity();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  keystoreName  Description of the Parameter
	 */
	public static void initKeystoreManager(String keystoreName) {
		if (keystoreManager != null) {
			return;
		}
		keystoreManager =
			KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
	}

	/**
	 *  Build an empty new UNICOREpro keystore
	 *
	 *@param  keystoreName   Description of Parameter
	 *@exception  Exception  Description of the Exception
	 */
	public static void buildEmptyKeystore(String keystoreName)
		throws Exception {

		keystoreManager =
			KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
		// define new password
		DefinePasswordDialog definePasswordDialog =
			new DefinePasswordDialog(
				currentInstance,
				"UNICOREpro: Choose a Password");
		definePasswordDialog.setStatusMessage(
			"Define a password for your new keystore.");
		definePasswordDialog.show();
		char[] passwd = definePasswordDialog.getPassword();
		if (passwd == null) {
			throw new Exception("Building initial keystore aborted by user.");
		}
		keystoreManager.buildEmptyKeystore(keystoreName, passwd, passwd);
		passwd = null;
		File keystoreFile = new File(keystoreName);
		try {
			keystoreManager.reWriteKeyStore(keystoreFile);
		} catch (Exception exp) {
			logger.log(Level.WARNING, "Keystore is not complete. ", exp);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	public static void updateDefaultIdentity() throws Exception {
		updateDefaultIdentity(null);
	}

	/**
	 *  Load the UNICOREpro keystore from file
	 *
	 *@param  keystoreName   Description of Parameter
	 *@param  passwd         Description of Parameter
	 *@return                Description of the Returned Value
	 *@exception  Exception  Description of the Exception
	 */
	public static boolean loadKeystore(String keystoreName, char[] passwd)
		throws Exception {
		try {
			keystoreManager =
				KeyStoreManager.getInstance("SunJCE", "JCEKS", keystoreName);
			keystoreManager.loadKeyStore(passwd, passwd);
		} catch (Exception firstException) {
			if (firstException.getMessage().startsWith("Keystore")) {
				// autsch!! wrong password
				keystoreManager.resetKeystore();
				return false;
			}
			try {
				keystoreManager =
					KeyStoreManager.getInstance("SUN", "JKS", keystoreName);
				keystoreManager.loadKeyStore(passwd, passwd);
			} catch (Exception secondException) {
				if (secondException.getMessage().startsWith("Keystore")) {
					// autsch!! wrong password
					keystoreManager.resetKeystore();
					return false;
				}
				try {
					// try PKCS12 format
					keystoreManager =
						KeyStoreManager.getInstance(
							null,
							"PKCS12",
							keystoreName);
					keystoreManager.loadKeyStore(passwd, passwd);
				} catch (Exception e) {
					UserMessages.warning(
						"Could not open UNICOREpro keystore: " + keystoreName,
						e.getLocalizedMessage());
					logger.log(Level.SEVERE, "Could not load keystore", e);
					if (e.getMessage().startsWith("Keystore")) {
						// autsch!! wrong password
						keystoreManager.resetKeystore();
						return false;
					}
					// if we got here something else but password went wrong...
					throw e;
				}
			}
		}
		try {
			if (userDefaults.getDefaultIdentity() == null
				|| userDefaults.getDefaultIdentity().equals("")) {
				Vector keyEntries = keystoreManager.getCachedKeyEntries();
				if (keyEntries != null && keyEntries.size() == 1) {
					getUserDefaults().setDefaultIdentity(
						(String) keyEntries.elementAt(0));
				}
			}

			updateDefaultIdentity(passwd);
		} catch (Exception e) {
			UserMessages.warning(
				res.getString("NO_DEFAULT_CERT"),
				e.getLocalizedMessage());
			// clear User defaults entry
			userDefaults.getDefaultIdentity();
			logger.log(Level.SEVERE, "Could not set user identity", e);
		}

		// keystore loading ok
		return true;
	}

	/**
	 *  Copy a resource set and return the cloned set
	 *
	 *@param  resourceSet  Description of Parameter
	 *@return              deep copy of resource set
	 */
	public static ResourceSet cloneResourceSet(ResourceSet resourceSet) {
		ResourceSet resourceClones = new ResourceSet();
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			try {
				resourceClones.add((Resource) resource.clone());
			} catch (CloneNotSupportedException e) {
				logger.log(
					Level.SEVERE,
					"Could not clone resource set " + resourceSet,
					e);
			}
		}
		return resourceClones;
	}

	/**
	 *  Load current USites into ResourceManager
	 *
	 *@param  page                    Description of Parameter
	 *@return                         Description of the Return Value
	 *@throws  MalformedURLException  if URL for USites is syntactically
	 *      incorrect
	 *@throws  IOException            if USites could not be loaded
	 */
	public static Vector parseHTML(String page)
		throws MalformedURLException, IOException {
		Vector usiteVector = new Vector();

		// first, search for tag "<dl id="Usites">
		int start = page.toUpperCase().indexOf("<DL ID=\"USITES\"");
		if (start == -1) {
			throw new IOException("Bad usites file: Didn't find start tag '<dl id=\"Usites\">'.");
		}
		String currentString = page.substring(start);

		// loop over usite entries
		while (currentString.toUpperCase().indexOf("<A HREF=") != -1) {

			// look for host address URL
			start = currentString.indexOf("://");
			if (start == -1) {
				throw new IOException("Bad usites file. Didn't find host address.");
			}
			currentString = currentString.substring(start + 3);

			// look for end of URL
			start = currentString.indexOf("\"");
			if (start == -1) {
				throw new IOException("Bad usites file. Didn't find end of host address.");
			}
			try {

				// HACK: Usite requires a URL in constructor, so we re-add
				// the protocol here. This should be removed some day
				URL host =
					new URL("http://" + currentString.substring(0, start));

				// find usite name to be displayed
				start = currentString.indexOf(">");
				currentString = currentString.substring(start);
				start = currentString.indexOf("<");
				String name = currentString.substring(1, start);
				usiteVector.add(new Usite(host, name));
				logger.info("Added Usite " + host);
			} catch (MalformedURLException e) {
				UserMessages.error("Error in Usite URL.", e.getMessage());
			}
		}
		return usiteVector;
	}

	/**
	 *  Get current VSites from USite
	 *
	 *@param  usite  Description of Parameter
	 */
	public static void loadVsites(Usite usite) {
		if (offline) {
			return;
		}
		if (usite.getType().equals(Usite.Type.CLASSIC)) {
			loadClassicalVsites(usite);
		} else if (usite.getType().equals(Usite.Type.REGISTRY)) {
			loadVsitesFromRegistry(usite);
		}
	}

	/**
	 *  Redirect stdout to file
	 *
	 *@param  redirect  Description of Parameter
	 *@param  format    Description of the Parameter
	 *@param  level     Description of the Parameter
	 */
	public static void redirectLog(
		boolean redirect,
		String format,
		Level level) {
		File logDir = new File(userDefaults.getLogDirectory());
		boolean logDirExists = true;
		if (!logDir.exists() && !logDir.isDirectory()) {
			if (JOptionPane
				.showConfirmDialog(
					currentInstance,
					"Logging Directory "
						+ logDir
						+ " does not exist. Create it?",
					"UNICORE: Missing Directory",
					JOptionPane.YES_NO_OPTION)
				== JOptionPane.YES_OPTION) {
				logDir.mkdirs();
			} else {
				logger.warning(
					"Logging Directory "
						+ logDir
						+ " does not exist. Logging to console...");
				userDefaults.setLogFormat(UserDefaultsDialog.PLAIN_FORMAT);
				userDefaults.setRedirect(false);
				redirect = false;
				logDirExists = false;
			}
		}
		if (logDirExists) {
			fileXml =
				new File(
					userDefaults.getLogDirectory()
						+ File.separator
						+ "clientlog.xml");
			filePlain =
				new File(
					userDefaults.getLogDirectory()
						+ File.separator
						+ "clientlog.txt");

		}
		initHandler();

		//sets the level of the loggs
		Logger.getLogger("").setLevel(level);
		if (consoleHandler != null) {
			consoleHandler.setLevel(level);
		}
		if (redirect) {
			redirectLogsIntoFile(format);
		} else {
			redirectLogsToConsole(level);
		}

	}

	/**
	 *  Description of the Method
	 *
	 *@param  filename  Description of the Parameter
	 *@return           Description of the Return Value
	 */
	public static Object readObjectFromFile(String filename) {
		byte[] in = null;
		long tstart = System.currentTimeMillis();
		long timing = tstart;
		try {
			File file = new File(filename);
			FileInputStream f = new FileInputStream(file);
			ObjectInputStream ois =
				new ObjectInputStream(new BufferedInputStream(f));
			Object object = ois.readObject();
			ois.close();
			f.close();
			return object;
		} catch (Exception e) {
			logger.log(
				Level.SEVERE,
				"Could not read object from file " + filename,
				e);
		}
		return null;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object    Description of the Parameter
	 *@param  filename  Description of the Parameter
	 *@return           Description of the Return Value
	 */
	public static boolean writeObjectToFile(Object object, String filename) {
		try {
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bout);
			oos.writeObject(object);
			oos.flush();
			oos.close();
			bout.flush();
			byte[] out = bout.toByteArray();
			bout.close();
			File file = new File(filename);
			FileOutputStream f = new FileOutputStream(file);
			BufferedOutputStream dout =
				new BufferedOutputStream(new DataOutputStream(f));
			dout.write(out, 0, out.length);
			dout.flush();
			dout.close();
			f.flush();
			f.close();
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could not write object " + object, e);
			return false;
		}
		return true;
	}

	/**
	 *  Description of the Method
	 */
	public static void writeCacheToDisk() {
		logger.info("Writing cached information to disk.");
		Thread writer = new Thread() {
			public void run() {
				logger.info(
					"Writing resource cache to file " + resourceCacheFilename);

				/*
				 *  try {
				 *  FileOutputStream file = new FileOutputStream(resourceCacheFilename, false);
				 *  XMLObjectWriter writer = new XMLObjectWriter(file);
				 *  writer.writeObjectXML(resourceCache);
				 *  file.flush();
				 *  file.close();
				 *  }
				 *  catch(Exception e) {
				 *  log.logWrite("Could not write resource cache: " + e.getMessage());
				 *  }
				 */
				writeObjectToFile(resourceCache, resourceCacheFilename);
			}
		};
		writer.start();
	}

	/**
	 *  load images only once
	 */
	public static void loadImages() {
		Class clazz = ResourceManager.class;
		ClassLoader loader = clazz.getClassLoader();
		imageTable = new Hashtable();
		iconTable = new Hashtable();
		for (int i = 0; i < imageFilenames.length; i++) {
			try {
				URL url = loader.getResource(imagePath + imageFilenames[i]);
				logger.fine("Loading image " + url);
				if (url == null) {
					logger.severe(
						"Could not load " + imagePath + imageFilenames[i]);
					continue;
				}
				Image image = Toolkit.getDefaultToolkit().createImage(url);
				if (image != null) {

					// put image into table
					imageTable.put(imageFilenames[i], image);

					// put icon into table
					iconTable.put(imageFilenames[i], new ImageIcon(image));
				}

				// put image into table
				imageTable.put(imageFilenames[i], image);

				// put icon into table
				iconTable.put(imageFilenames[i], new ImageIcon(image));
			} catch (Error e) {
				logger.log(
					Level.SEVERE,
					"Could not load " + imagePath + imageFilenames[i],
					e);
			}
		}
	}

	/**
	 *  Expand a property in the policy file
	 *
	 *@param  property     Which property
	 *@param  newCodebase  The code base for the property
	 */
	public static void expandPolicy(String property, String newCodebase) {
		System.setProperty(property, newCodebase);
		logger.info("Expand: " + property + " to " + newCodebase);
		Policy pol = Policy.getPolicy();
		pol.refresh();
	}

	/**
	 *  Description of the Method
	 */
	public static void initUsers() {
		if (getUsites().isEmpty()) {
			logger.severe(
				"Usites haven't been loaded so far or no usites are available");
			return;
		}
		for (int i = 0; i < getUsites().size(); i++) {
			Usite usite = (Usite) getUsites().elementAt(i);
			initUserUsite(usite);

			Vector vsites = getVsites(usite);
			for (int j = 0; j < vsites.size(); j++) {
				initUserVsite((Vsite) vsites.elementAt(j));
			}
		}
	}

	/**
	 *  Get Usite adresses from HTML or XML page. This method is called by
	 *  getUsites()
	 */
	public static synchronized void loadUsites() {

		// return cached usites in offline mode
		if (offline) {
			return;
		}

		// try to get usite url from user defaults
		String urlStrings = userDefaults.getUsiteURL();
		if (urlStrings == null) {
			logger.severe(
				"Error in UNICORE Sites URL from user defaults: "
					+ userDefaults.getUsiteURL());
			return;
		}

		// build result vector
		Vector newUsites = new Vector();

		// urlStrings is actually a list of URLs separated by "\t" or ","
		StringTokenizer st = new StringTokenizer(urlStrings, "\t,");

		// loop over usite urls
		while (st.hasMoreElements()) {
			String urlString = st.nextToken();

			// read usite page
			String usitePage;
			URL url = null;
			try {
				url = new URL(urlString);
				logger.info("Loading Usite information from: " + url);
				usitePage = urlReader.readURL(url);
			}
			// URL
			catch (java.net.MalformedURLException e) {
				//                logger.log(Level.SEVERE, "Error in Usite URL "+url+".", e);
				UserMessages.error(
					"Error in UNICOREpro site URL " + url + ".",
					e.getMessage());
				continue;
			}

			// proxy or server unknown
			catch (UnknownHostException e) {
				UserMessages.warning(
					"Could not read UNICOREpro sites from URL " + url + ".",
					e.getMessage());
				continue;
			}

			// could not connect
			catch (ConnectException e) {
				UserMessages.warning(
					"Could not read UNICOREpro sites.",
					"Connection to: "
						+ url
						+ " refused."
						+ "\nPlease, check your proxy configuration in menu Settings -> User Defaults -> General."
						+ "\n("
						+ e.getMessage()
						+ ")");
				continue;
			}

			// requestURL
			catch (IOException e) {
				//                logger.log(Level.SEVERE, "Could not request Usite URL "+url+".", e);
				UserMessages.error(
					"Could not request UNICOREpro site URL " + url + ".",
					e.getMessage());
				continue;
			}

			// reading failed, continue with next urlString
			if (usitePage == null) {
				continue;
			}
			try {
				Vector v = parseXML(usitePage);
				newUsites.addAll(v);
			} catch (SAXException saxe) {

				// try html format
				try {
					Vector v = parseHTML(usitePage);
					newUsites.addAll(v);
				} catch (IOException ioe) {
					UserMessages.warning(
						"Could not load UNICOREpro Site Severs from URL:\n<"
							+ urlString
							+ ">\n"
							+ "The file seems to be syntactically incorrect or unreadable.\n"
							+ "Please enter a valid URL in your user defaults.");
					UserMessages.print("Reason: " + ioe.getLocalizedMessage());
				}
			} catch (Exception e) {
				UserMessages.warning(
					"Could not load UNICOREpro Site Severs from URL:\n<"
						+ urlString
						+ ">\n"
						+ "The file seems to be syntactically incorrect or unreadable.\n"
						+ "Please enter a valid URL in your user defaults.");
				UserMessages.print("Reason: " + e.getLocalizedMessage());
			}
		}
		resourceCache.updateUsites(newUsites);
		resourceCache.clearUnavailableUsites();
		resourceCache.clearUnavailableVsites();
		resourceCache.clearSpecialVsites();
	}

	/**
	 *  Update resources of a VSite
	 *
	 *@param  vsite  VSite to connect to
	 */
	public static void loadResources(Vsite vsite) {
		if (offline || !resourceCache.isVsiteAvailable(vsite)) {
			return;
		}
		// use user specified for the Vsite
		User user = getUser(vsite);
		logger.info(
			"Updating resources for vsite "
				+ vsite.getName()
				+ " user "
				+ user);

		//generating the job
		//first, the " GetResourceDescription" task
		GetResourceDescription grd =
			new GetResourceDescription(
				"ResourceDescription(" + vsite + ")",
				null);
		ServiceContainer sc = new ServiceContainer(grd, user, vsite);
		SubmitServiceContainer submit = new SubmitServiceContainer(sc);
		submit.start();

		// wait no longer than 1 min until job has finished...
		int count = 60;
		while (sc.getState() != ActionContainer.STATE_SUBMITTED && count > 0) {
			try {
				Thread.sleep(2000);
				count--;
			} catch (InterruptedException e) {
				logger.log(Level.SEVERE, "Thread interrupted.", e);
			}
		}
		try {
			Reply reply = sc.getSubmissionResult();
			if (reply == null) {
				resourceCache.addUnavailableVsite(
					vsite,
					res.getString("EMPTY_REPLY"));
				return;
			} else if (!(reply instanceof RetrieveOutcomeReply)) {
				resourceCache.addUnavailableVsite(
					vsite,
					"Could not contact virtual site."
						+ " User eventually unknown.");
				logger.warning(
					"Could not contact vsite "
						+ vsite
						+ ": "
						+ reply.getLastEntry().getComment());
				return;
			}
			RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;

			// deserialize the Outcome Object
			Outcome outcome =
				(Outcome) new ObjectInputStream(new ByteArrayInputStream(roReply
					.getOutcome()))
					.readObject();
			if (outcome instanceof AbstractJob_Outcome) {
				AbstractJob_Outcome aj_out =
					ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
				synchronized (lock) {
					GetResourceDescription_Outcome resourceOutcome =
						(GetResourceDescription_Outcome) aj_out.getOutcome(
							sc.getService().getId());
					NamedResourceSet resourceSet =
						new NamedResourceSet(
							"Site Resources for " + vsite,
							resourceOutcome.getResources());
					resourceCache.addResourceSet(vsite, resourceSet);
				}
			}
		} catch (ClassNotFoundException e) {
			resourceCache.addUnavailableVsite(
				vsite,
				res.getString("MISSING_CLASS") + "\nReason: " + e.getMessage());
		} catch (ClassCastException e) {
			resourceCache.addUnavailableVsite(
				vsite,
				res.getString("WRONG_CLASS") + "\nReason: " + e.getMessage());
		} catch (InvalidClassException e) {
			resourceCache.addUnavailableVsite(
				vsite,
				res.getString("INVALID_CLASS") + "\nReason: " + e.getMessage());
		} catch (IOException e) {
			resourceCache.addUnavailableVsite(
				vsite,
				res.getString("IO_EXCEPTION") + "\nReason: " + e.getMessage());
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  selectedFile  Description of the Parameter
	 */
	public static void buildFileChooserTable(String selectedFile) {
		filechooserTable = new Hashtable();
		SelectorDialog dialog =
			new SelectorDialog(currentInstance, new File(selectedFile));
		filechooserTable.put(FileStorage.NSPACE_STRING, dialog);
	}

	/**
	 *  Description of the Method
	 */
	public static void resetFileHandlers() {
		Handler[] handlers = Logger.getLogger("").getHandlers();

		for (int i = 0; i < handlers.length; i++) {
			if (handlers[i] instanceof FileHandler) {
				Handler handler = handlers[i];
				Logger.getLogger("").removeHandler(handlers[i]);
			}
		}

		if (logInto == LOG_INTO_FILES || logInto == LOG_INTO_XMLFILE) {
			appendTailToXmlFile(fileXml.toString());
		}
		filePlainHandler = null;
		fileXmlHandler = null;
		fileXml = null;
		filePlain = null;
		logInto = NEW_LOG_DIRECTORY;
	}

	/**
	 *  Load user for Usite from Userdefaults
	 *
	 *@param  usite  Description of the Parameter
	 */
	private static void initUserUsite(Usite usite) {
		User userUsite = userDefaults.getUser(usite);
		if (userUsite != null) {
			userTable.put(usite, userUsite);
		}
	}

	/**
	 *  Load user for Usite from Userdefaults
	 *
	 *@param  vsite  Description of the Parameter
	 */
	private static void initUserVsite(Vsite vsite) {
		User userVsite = userDefaults.getUser(vsite);
		if (userVsite != null) {
			userTable.put(vsite, userVsite);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usite  Description of the Parameter
	 */
	private static void loadFullVsites(Usite usite) {
		URL url = usite.getAddress();
		logger.info("Loading Vsite description from: " + usite);
		Reply vsitesReply = null;
		try {
			// Use ListVsite to get full Vsites
			ListVsites listVsites = new ListVsites();
			User user = getUser(usite);
			logger.info(
				"Updating list of Vsites for Usite "
					+ usite.getName()
					+ " user "
					+ keystoreManager.getAliasFromCertifcate(
						user.getCertificate())
					+ " (ListVsites)");
			Connection connection =
				connectionManager.lookupConnection(usite, user);
			vsitesReply = connection.sendMessage(listVsites, null);
			int returnCode = vsitesReply.getLastEntry().getReturnCode();

			UnicoreResponse resp = vsitesReply.getLastEntry();
			Vector newVsites = new Vector();
			// Use ListVsite to get full Vsites
			if (!(vsitesReply instanceof ListVsitesReply)) {
				resp = vsitesReply.getLastEntry();
				resourceCache.addUnavailableUsite(
					usite,
					res.getString("NO_GATEWAY")
						+ url.getProtocol()
						+ "://"
						+ url.getHost()
						+ ":"
						+ url.getPort()
						+ res.getString("REASON")
						+ resp.getComment());
			} else {
				// Use ListVsite to get full Vsites
				Vsite[] vsiteArray = ((ListVsitesReply) vsitesReply).getList();

				if (returnCode == 0 && vsiteArray != null) {
					newVsites = resourceCache.updateVsites(usite, vsiteArray);
				} else {
					resp = vsitesReply.getLastEntry();
					resourceCache.addUnavailableUsite(
						usite,
						res.getString("NO_GATEWAY")
							+ url.getProtocol()
							+ "://"
							+ url.getHost()
							+ ":"
							+ url.getPort()
							+ res.getString("REASON")
							+ resp.getComment());
				}
			}

			// put vsite vector into table
			synchronized (lock) {
				resourceCache.addVsites(usite, newVsites);
			}
		} catch (StreamCorruptedException e) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("NO_GATEWAY")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		} catch (ClassNotFoundException e) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("BOGUS_REPLY")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		} catch (ConnectException e) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("CONNECTION_REFUSED")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage()
					+ res.getString("WRONG_ADDRESS"));
		} catch (UnknownHostException e) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("UNKNOWN_HOST")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		} catch (IOException e) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("CONNECTION_REFUSED")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage()
					+ res.getString("AUTHENTICATION")
					+ userInfo(user));
		} catch (Exception e) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("UNKNOWN_HOST")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
		}
		if (vsitesReply == null) {
			resourceCache.addUnavailableUsite(
				usite,
				res.getString("CANNOT_LOAD_VSITES")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort());
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usite  Description of the Parameter
	 */
	private static void loadClassicalVsites(Usite usite) {
		URL url = usite.getAddress();
		logger.info("Loading Vsite description from: " + usite);
		Reply vsitesReply = null;
		try {
			// Use ListPorts to get pure Broker Vsites
			ListPorts listVsites = new ListPorts();
			User user = getUser(usite);
			logger.info(
				"Updating list of Vsites for Usite "
					+ usite.getName()
					+ " user "
					+ keystoreManager.getAliasFromCertifcate(
						user.getCertificate())
					+ "(ListPorts)");
			Connection connection =
				connectionManager.lookupConnection(usite, user);
			vsitesReply = connection.sendMessage(listVsites, null);
			UnicoreResponse resp = vsitesReply.getLastEntry();
			int returnCode = resp.getReturnCode();

			Vector newVsites = new Vector();
			// Use ListPorts to get pure Broker Vsites
			if (!(vsitesReply instanceof ListPortsReply)) {
				logger.warning(
					res.getString("NO_GATEWAY")
						+ url.getProtocol()
						+ "://"
						+ url.getHost()
						+ ":"
						+ url.getPort()
						+ res.getString("REASON")
						+ resp.getComment());
			} else {
				// Use ListPorts to get pure Broker Vsites
				Vsite[] vsiteArray = ((ListPortsReply) vsitesReply).getList();

				if (returnCode == 0 && vsiteArray != null) {
					newVsites = resourceCache.updateVsites(usite, vsiteArray);
					// put vsite vector into table
					synchronized (lock) {
						resourceCache.addVsites(usite, newVsites);
					}
					return;
				} else {
					logger.warning(
						res.getString("NO_GATEWAY")
							+ url.getProtocol()
							+ "://"
							+ url.getHost()
							+ ":"
							+ url.getPort()
							+ res.getString("REASON")
							+ resp.getComment());
				}
			}

			// put empty vsite vector into table
			synchronized (lock) {
				resourceCache.addVsites(usite, newVsites);
			}
			// go on with loadFullVsites
			loadFullVsites(usite);
		} catch (StreamCorruptedException e) {
			logger.warning(
				res.getString("NO_GATEWAY")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
			loadFullVsites(usite);
		} catch (ClassNotFoundException e) {
			logger.warning(
				res.getString("BOGUS_REPLY")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
			loadFullVsites(usite);
		} catch (ConnectException e) {
			logger.warning(
				res.getString("CONNECTION_REFUSED")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage()
					+ res.getString("WRONG_ADDRESS"));
			loadFullVsites(usite);
		} catch (UnknownHostException e) {
			logger.warning(
				res.getString("UNKNOWN_HOST")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
			loadFullVsites(usite);
		} catch (IOException e) {
			logger.warning(
				res.getString("CONNECTION_REFUSED")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage()
					+ res.getString("AUTHENTICATION")
					+ userInfo(user));
			loadFullVsites(usite);
		} catch (Exception e) {
			logger.warning(
				res.getString("UNKNOWN_HOST")
					+ url.getProtocol()
					+ "://"
					+ url.getHost()
					+ ":"
					+ url.getPort()
					+ res.getString("REASON")
					+ e.getLocalizedMessage());
			loadFullVsites(usite);
		} finally {
			if (vsitesReply == null) {
				logger.warning(
					res.getString("CANNOT_LOAD_VSITES")
						+ url.getProtocol()
						+ "://"
						+ url.getHost()
						+ ":"
						+ url.getPort());
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usite  Description of the Parameter
	 */
	private static void loadVsitesFromRegistry(Usite usite) {
		logger.info("Loading Vsite description from registry: " + usite);

		QName uplQname =
			new QName("http://services.ogsa.pallas.com", "SiteRegistry");

		URL url = usite.getAddress();
		LocatorType[] locators = null;

		try {

			System.setProperty(
				"org.apache.axis.components.net.SecureSocketFactory",
				"com.pallas.ogsa.client.SunJSSESocketFactory");
			System.setProperty(
				"axis.socketSecureFactory",
				"com.pallas.ogsa.client.SunJSSESocketFactory");

			System.setProperty("clientauth", "true");

			getKeystoreManager().setupAxisKeymanger(
				getUser(usite).getCertificate());

			Service service = new Service(usite.getAddress(), uplQname);

			ISiteRegistry registry =
				new SiteRegistryStub(usite.getAddress(), service);

			locators = registry.getSites();
		} catch (Exception e) {
			UserMessages.warning(
				"Cannot locate grid service references "
					+ e.getLocalizedMessage());
		}

		if (locators == null) {
			UserMessages.warning("Cannot locate grid service references ");
			return;
		}

		Vsite[] vsiteArray = new Vsite[locators.length];
		for (int i = 0; i < locators.length; i++) {
			ServiceReferenceType reference =
				(ServiceReferenceType) locators[i].getServiceReference();
			String serviceUrl = reference.getWsdlURL();
			URI uri = locators[i].getServiceHandle().getValue();
			String name =
				locators[i].getServiceHandle().getValue().getUserinfo();

			Vsite newVsite = new Vsite(serviceUrl, name);
			newVsite.setAJOSpecificationVersion("4.0.0");
			newVsite.setType(new Vsite.Type("GS"));
			vsiteArray[i] = newVsite;
		}

		Vector registrySites = resourceCache.updateVsites(usite, vsiteArray);
		// put vsite vector into table
		synchronized (lock) {
			resourceCache.addVsites(usite, registrySites);
		}

		return;
	}

	/**
	 *  When user identity has changed from SecPropsEditor update it here
	 *
	 *@param  passwd         Description of the Parameter
	 *@exception  Exception  Description of the Exception
	 */
	private static void updateDefaultIdentity(char[] passwd) throws Exception {
		String defaultAlias = userDefaults.getDefaultIdentity();
		if (defaultAlias == null || defaultAlias.equals("")) {
			return;
		}

		initUsers();

		// update ssl keystore
		keystoreManager.createSSLKeystore(true, passwd, passwd);

		// we must initialize the SSL factory
		//       initializeSSLfactory(passwd);
		user = new User();
		X509Certificate[] chain =
			keystoreManager.getCertificateByAlias(defaultAlias);
		user.setCertificate(chain[0]);
		String email = null;
		String xlogin = null;
		String project = null;
		if ((userDefaults.getUserEmail() != null)
			&& (!userDefaults.getUserEmail().equals(""))) {
			email = userDefaults.getUserEmail();
		}
		if ((userDefaults.getXlogin() != null)
			&& (!userDefaults.getXlogin().equals(""))) {
			xlogin = userDefaults.getXlogin();
		}
		if ((userDefaults.getProject() != null)
			&& (!userDefaults.getProject().equals(""))) {
			project = userDefaults.getProject();
		}
		user.setEmailAddress(email);
		user.setXlogin(xlogin);
		user.setProject(project);

	}

	/**
	 *  Description of the Method
	 */
	private static void readCacheFromDisk() {
		logger.info("Reading cached information from disk.");
		Thread reader = new Thread() {
			public void run() {
				logger.info(
					"Reading resource cache from file: "
						+ resourceCacheFilename);

				/*
				 *  try {
				 *  XMLObjectReader din = new XMLObjectReader(new File(resourceCacheFilename));
				 *  resourceCache = (ResourceCache)din.readObjectXML();
				 *  }
				 *  catch(Exception e) {
				 *  log.logWrite("Could not read resource cache: " + e.getMessage());
				 *  }
				 */
				resourceCache =
					(ResourceCache) readObjectFromFile(resourceCacheFilename);

				// if loading cache failed, build new one
				if (resourceCache == null) {
					resourceCache = new ResourceCache();
				}
			}
		};
		reader.start();
	}

	/**
	 *  Load current USites into ResourceManager
	 *
	 *@param  page                              Description of Parameter
	 *@return                                   Description of the Return Value
	 *@exception  SAXException                  Description of Exception
	 *@exception  ParserConfigurationException  Description of Exception
	 *@throws  MalformedURLException            if URL for USites is
	 *      syntactically incorrect
	 *@throws  IOException                      if USites could not be loaded
	 */
	private static Vector parseXML(String page)
		throws
			MalformedURLException,
			IOException,
			SAXException,
			ParserConfigurationException {
		byte[] xmlBytes = page.getBytes();
		ByteArrayInputStream bis = new ByteArrayInputStream(xmlBytes);
		UsiteSaxHandler handler = new UsiteSaxHandler();

		// Use the validating parser
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);

		// Parse the input
		SAXParser saxParser = factory.newSAXParser();
		saxParser.parse(bis, handler);
		Vector usites = handler.getUsites();

		// memory leak, if we do not set Vecxtor reference to null
		handler.setUsites(null);
		return usites;
	}

	/**
	 *  Description of the Method
	 */
	private static void printVersionInfo() {
		String version = " Running UNICOREpro client version: ";
		version = version + Version.spcVersion();
		version =
			version
				+ " Implementation: "
				+ Package
					.getPackage("com.pallas.unicore.resourcemanager")
					.getImplementationVendor();
		logger.info(version);
		version = " Running AJO version: ";
		version =
			version
				+ Package.getPackage("org.unicore").getSpecificationVersion();
		version =
			version
				+ " Implementation: "
				+ Package.getPackage("org.unicore.ajo").getImplementationVendor();
		logger.info(version);
		version = "JVM version: ";
		version =
			version
				+ System.getProperty("java.vm.version")
				+ " System: "
				+ System.getProperty("os.name");
		logger.info(version);
	}

	/**
	 *  Description of the Method
	 */
	private static void initHandler() {
		if (consoleHandler == null) {
			boolean console = false;
			for (int i = 0;
				i < Logger.getLogger("").getHandlers().length;
				i++) {
				if (Logger.getLogger("").getHandlers()[i]
					instanceof ConsoleHandler) {
					consoleHandler = Logger.getLogger("").getHandlers()[i];
					console = true;
					break;
				}
			}
			if (!console) {
				consoleHandler = new ConsoleHandler();
			}
			consoleHandler.setFormatter(new SimpleFormatter());
			consoleHandler.setLevel(logLevels.get(userDefaults.getLogLevel()));
		}
		if (fileXmlHandler == null) {
			try {
				if (fileXml != null) {
					fileXmlHandler = new FileHandler(fileXml.toString());
					fileXmlHandler.setFormatter(new NewXMLFormatter());
				}
			} catch (IOException e) {
				logger.log(Level.WARNING, "cannot write log into XML-file", e);
			} catch (SecurityException e) {
				logger.log(Level.WARNING, "cannot write log into XML-file", e);
			}

		}
		if (filePlainHandler == null) {
			try {
				if (filePlain != null) {
					filePlainHandler = new FileHandler(filePlain.toString());
					filePlainHandler.setFormatter(new SimpleFormatter());
				}
			} catch (IOException e) {
				logger.log(
					Level.WARNING,
					"cannot write log into simple-file",
					e);
			} catch (SecurityException e) {
				logger.log(
					Level.WARNING,
					"cannot write log into simple-file",
					e);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  level  Description of the Parameter
	 */
	private static void redirectLogsToConsole(Level level) {
		if (logInto == LOG_TO_CONSOLE) {
			return;
		}
		logger.info("Logging to console");
		consoleHandler.setLevel(level);
		Logger.getLogger("").addHandler(consoleHandler);
		switch (logInto) {
			case LOG_INTO_PLAINFILE :
				Logger.getLogger("").removeHandler(filePlainHandler);
				break;
			case LOG_INTO_FILES :
				Logger.getLogger("").removeHandler(filePlainHandler);
			case LOG_INTO_XMLFILE :
				Logger.getLogger("").removeHandler(fileXmlHandler);
				appendTailToXmlFile(fileXml.toString());
		}
		logInto = LOG_TO_CONSOLE;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  format  Description of the Parameter
	 */
	private static void redirectLogsIntoFile(String format) {
		if (UserDefaultsDialog.XML_FORMAT.equals(format)) {
			if (logInto == LOG_INTO_XMLFILE) {
				return;
			}
			logger.info("Logging to XML-file");
			switch (logInto) {
				case LOG_INTO_PLAINFILE :
					Logger.getLogger("").addHandler(fileXmlHandler);
				case LOG_INTO_FILES :
					Logger.getLogger("").removeHandler(filePlainHandler);
					break;
				case LOG_TO_CONSOLE :
					Logger.getLogger("").removeHandler(consoleHandler);
				case NEW_LOG_DIRECTORY :
					Logger.getLogger("").addHandler(fileXmlHandler);
					break;
			}
			logInto = LOG_INTO_XMLFILE;
			return;
		}
		if (UserDefaultsDialog.PLAIN_FORMAT.equals(format)) {
			if (logInto == LOG_INTO_PLAINFILE) {
				return;
			}
			logger.info("Logging to simple file");
			switch (logInto) {
				case LOG_INTO_XMLFILE :
					Logger.getLogger("").addHandler(filePlainHandler);
				case LOG_INTO_FILES :
					Logger.getLogger("").removeHandler(fileXmlHandler);
					appendTailToXmlFile(fileXml.toString());
					break;
				case LOG_TO_CONSOLE :
					Logger.getLogger("").removeHandler(consoleHandler);
				case NEW_LOG_DIRECTORY :
					Logger.getLogger("").addHandler(filePlainHandler);
					break;
			}
			logInto = LOG_INTO_PLAINFILE;
			return;
		}
		if (UserDefaultsDialog.XML_AND_PLAIN_FORMAT.equals(format)) {
			if (logInto == LOG_INTO_FILES) {
				return;
			}
			logger.info("Logging to XML and simple file");
			switch (logInto) {
				case LOG_INTO_PLAINFILE :
					Logger.getLogger("").addHandler(fileXmlHandler);
					break;
				case LOG_INTO_XMLFILE :
					Logger.getLogger("").addHandler(filePlainHandler);
					break;
				case LOG_TO_CONSOLE :
					Logger.getLogger("").removeHandler(consoleHandler);
				case NEW_LOG_DIRECTORY :
					Logger.getLogger("").addHandler(fileXmlHandler);
					Logger.getLogger("").addHandler(filePlainHandler);
					break;
			}
			logInto = LOG_INTO_FILES;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  fileNameXml  string of the xml-file or directory denoted by this
	 *      abstract pathname
	 */
	private static void appendTailToXmlFile(String fileNameXml) {
		try {
			PrintWriter tailOut =
				new PrintWriter(new FileOutputStream(fileNameXml, true));
			String tail = fileXmlHandler.getFormatter().getTail(fileXmlHandler);
			tailOut.print(tail);
			tailOut.close();
		} catch (IOException e) {
			logger.log(Level.WARNING, "Cannot append tail to XML-file ", e);
		}
	}

	/**
	 *  Debugging function
	 *
	 *@param  table  Description of Parameter
	 */
	private void dumpHashtable(Hashtable table) {
		Enumeration e = table.keys();
		Object key;
		Object value;
		while (e.hasMoreElements()) {
			key = e.nextElement();
			value = table.get(key);
			logger.fine("Hash-Key: <" + key + "> value: <" + value + ">");
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     thomas
	 *@created    September 27, 2001
	 */
	private static class UsiteSaxHandler extends DefaultHandler {
		Vector foundSites = new Vector();

		/**
		 *  Constructor for the SaxHandler object
		 */
		public UsiteSaxHandler() {
		}

		/**
		 *  Sets the usites attribute of the UsiteSaxHandler object
		 *
		 *@param  foundSites  The new usites value
		 */
		public void setUsites(Vector foundSites) {
			this.foundSites = foundSites;
		}

		/**
		 *  Gets the Usites attribute of the UsiteSaxHandler object
		 *
		 *@return    The Usites value
		 */
		public Vector getUsites() {
			return foundSites;
		}

		//===========================================================
		// SAX DocumentHandler methods
		//===========================================================

		/**
		 *  Description of the Method
		 *
		 *@exception  SAXException  Description of Exception
		 */
		public void startDocument() throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@exception  SAXException  Description of Exception
		 */
		public void endDocument() throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  namespaceURI      Description of Parameter
		 *@param  lName             Description of Parameter
		 *@param  qName             Description of Parameter
		 *@param  attrs             Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void startElement(
			String namespaceURI,
			String lName,
			String qName,
			Attributes attrs)
			throws SAXException {
			String elementName = lName;
			if ("".equals(elementName)) {
				elementName = qName;

				// namespaceAware = false
			}
			if (elementName.equals("Usite")) {
				if (attrs != null) {
					String host = null;
					String tag = null;
					String port = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("name")) {
							tag = attrs.getValue(i);
						}
						if (aName.equals("address")) {
							host = attrs.getValue(i);
						}
						if (aName.equals("port")) {
							port = attrs.getValue(i);
						}
					}
					if (host != null && tag != null && port != null) {
						try {
							foundSites.add(
								new Usite(
									new URL("http://" + host + ":" + port),
									tag,
									Usite.Type.CLASSIC));
						} catch (MalformedURLException e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}
				}
			} else if (elementName.equals("Registry")) {
				if (attrs != null) {
					String handle = null;
					String description = null;
					String address = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("handle")) {
							handle = attrs.getValue(i);
						}
						if (aName.equals("description")) {
							description = attrs.getValue(i);
						}
						if (aName.equals("address")) {
							address = attrs.getValue(i);
						}
					}
					if (handle != null
						&& description != null
						&& address != null) {
						try {
							foundSites.add(
								new Usite(
									new URL(address),
									handle,
									Usite.Type.REGISTRY));
						} catch (MalformedURLException e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					}
				}
			}
		}

		/**
		 *  Description of the Method
		 *
		 *@param  namespaceURI      Description of Parameter
		 *@param  sName             Description of Parameter
		 *@param  qName             Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void endElement(String namespaceURI, String sName, String qName)
			throws SAXException {
			// qualified name
		}

		/**
		 *  Description of the Method
		 *
		 *@param  buf               Description of Parameter
		 *@param  offset            Description of Parameter
		 *@param  len               Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void characters(char[] buf, int offset, int len)
			throws SAXException {
		}
	}
}
