package com.pallas.unicore.resourcemanager;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.unicore.resources.Memory;
import org.unicore.resources.RunTime;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.extensions.NamedResourceSet;

/**
 *  This class is a container for a set of related ResourceSets (for example all
 *  sets belonging to a JobGroup. It provides functions to manipulate single and
 *  collections of ResourceSets and to store and load the tray.
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    September 28, 2001
 *@version    $Id: ResourceTray.java,v 1.2 2003/06/30 06:58:36 rrate Exp $
 */
public class ResourceTray extends Hashtable {
	final static long serialVersionUID = 2339131310553778886L;

	/**
	 *  The main program for the ResourceTray class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		ResourceTray tray = new ResourceTray("TEST");
		ResourceSet set1 = new ResourceSet();
		Memory cap1 = new Memory("tr1", 20, 10, 0);

		set1.add(cap1);

		String key1 = "key1";
		String key2 = "key2";

		tray.put(set1, key1);
		tray.put(set1, key2);

		cap1.setMaxRequest(50);
		RunTime cap2 = new RunTime("tr2", 50, 10, 0);

		cap1.setMaxRequest(20);
		tray.put(set1, key1);
	}

	// the name of the tray
	private String name;

	/**
	 *  Constructor for the ResourceTray object
	 *
	 */
	public ResourceTray() {
		super();
	}

	/**
	 *  Constructor for the ResourceTray object
	 *
	 *@param  name  Description of Parameter
	 */
	public ResourceTray(String name) {
		this.name = name;
	}

	/**
	 *  Constructor for the ResourceTray object
	 *
	 *@param  initial     Description of Parameter
	 *@param  initialKey  Description of Parameter
	 *@param  name        Description of Parameter
	 */
	public ResourceTray(String name, ResourceSet initial, Object initialKey) {
		this(name);
		put(initialKey, initial);
	}

	/**
	 *  Overwrite get method to Return an empty site default resource set
	 *  instead of null
	 *
	 *@param  key  Description of Parameter
	 *@return      Description of the Returned Value
	 */
	public Object get(Object key) {
		Object value = super.get(key);
		if (value == null) {
			value = new NamedResourceSet();
		}
		return value;
	}

	/**
	 *  Gets the AllResourceSets attribute of the ResourceTray object
	 *
	 *@return    The AllResourceSets value
	 */
	public Vector getAllResourceSets() {
		Vector sets = new Vector();
		Enumeration enum = elements();
		while (enum.hasMoreElements()) {
			NamedResourceSet set = (NamedResourceSet) enum.nextElement();
			if (!sets.contains(set)) {
				sets.add(set);
			}
		}
		return sets;
	}

	/**
	 *  Gets the Name attribute of the ResourceTray object
	 *
	 *@return    The Name value
	 */
	public String getName() {
		return this.name;
	}

	/**
	 *  Sets the Name attribute of the ResourceTray object
	 *
	 *@param  name  The new Name value
	 */
	public void setName(String name) {
		this.name = name;
	}
}
