package com.pallas.unicore.resourcemanager;

import java.io.FileOutputStream;
import java.net.URL;
import java.security.cert.Certificate;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.apache.axis.client.Service;

import com.pallas.ogsa.services.caservice.CAServiceStub;
import com.pallas.ogsa.services.caservice.ICAService;
import com.pallas.ogsa.util.ObjectWrapperBean;
import com.pallas.unicore.client.dialogs.CSRDialog;
import com.pallas.unicore.security.KeyStoreManager;

/**
 *@author     Ralf Ratering
 *@created    7. April 2003
 *@version    $Id: ServiceManager.java,v 1.8 2003/06/30 06:58:36 rrate Exp $
 */
public class ServiceManager {
	// used for logging
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.resourcemanager");

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.resourcemanager.ResourceStrings");

	/**
	 *  Gets the cACertificate attribute of the ServiceManager class
	 *
	 *@return                The cACertificate value
	 *@exception  Exception  Description of the Exception
	 */
	public static Certificate[] getCACertificate() throws Exception {
		logger.info("Getting CA certificate...");

		URL wsdlUrl =
			new URL("http://testgrid.unicorepro.com:5900/unicore/services/CAService?WSDL");
		Service service =
			new Service(
				wsdlUrl,
				new QName("http://services.ogsa.pallas.com", "CAService"));

		ICAService caService = new CAServiceStub(wsdlUrl, service);

		ObjectWrapperBean wrappedCert = caService.getCACertificate();
		Certificate[] cert = (Certificate[]) wrappedCert.deserialize();
		logger.info("Got CA certificates");
		return cert;
	}

	/**
	 *  Description of the Method
	 *
	 *@return                Description of the Return Value
	 *@exception  Exception  Description of the Exception
	 */
	public static Certificate getSignedRequest() throws Exception {
		logger.info("Signing certification request...");

		// let the csr sign by the service
		CSRDialog csrDialog =
			new CSRDialog(
				ResourceManager.getCurrentInstance(),
				res.getString("TEST_CERT_DIALOG_TITLE"));
		csrDialog.show();

		String dnString = csrDialog.getDNString();
		if (dnString == null) {
			System.err.println("Cancel!");
			return null;
		}

		KeyStoreManager keystoreManager = ResourceManager.getKeystoreManager();
		byte[] csr = keystoreManager.generateCSR(dnString, 1024);

		FileOutputStream divert = new FileOutputStream("nextCSR.pem");

		divert.write(csr);
		divert.close();

		URL wsdlUrl =
			new URL("http://testgrid.unicorepro.com:5900/unicore/services/CAService?WSDL");
		Service service =
			new Service(
				wsdlUrl,
				new QName("http://services.ogsa.pallas.com", "CAService"));

		ICAService caService = new CAServiceStub(wsdlUrl, service);

		ObjectWrapperBean wrappedCert = caService.signRequest(csr);
		Certificate cert = (Certificate) wrappedCert.deserialize();
		logger.info("User certificate: " + cert);
		return cert;
	}

	/**
	 *  Constructor for the ServiceManager object
	 */
	private ServiceManager() {
	}

}
