package com.pallas.unicore.security;

import java.security.Signature;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.User;

import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *@author     Thomas Kentemich
 *@created    5. September 2002
 *@version    $Id: AjoSigner.java,v 1.3 2003/06/30 06:58:48 rrate Exp $
 */
public class AjoSigner
	implements org.unicore.utility.ConsignForm.SignatureFactory {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.security");

	/**
	 *  Gets the signer attribute of the AjoSigner class
	 *
	 *@param  user  Description of the Parameter
	 *@return       The signer value
	 */
	public static AjoSigner getSigner(User user) {
		return new AjoSigner(user);
	}

	//private final static AjoSigner ajoSigner = new AjoSigner();

	private User user = null;

	/**
	 *  Constructor for the AjoSigner object
	 *
	 *@param  user  Description of the Parameter
	 */
	private AjoSigner(User user) {

		if (user != null) {
			logger.fine(
				"Generate new job signer for: "
					+ user.getCertificate().getSubjectDN());
		} else {
			logger.fine("Generate new job signer for: <NULL>");
		}

		this.user = user;
	}

	/**
	 *  Implementation of the AJO SignatureFactory interface This is called when
	 *  consigning AJO's in order to sign each sub-AJO with a newly initialized
	 *  Signature object.
	 *
	 *@return    The Instance value
	 */
	public Signature getInstance() {
		Signature signature = null;
		try {
			return ResourceManager.getKeystoreManager().getSignature(
				user.getCertificate());
		} catch (Exception e) {
			logger.log(
				Level.WARNING,
				"Could not get signature for user "
					+ user.getCertificate().getSubjectDN()
					+ " from keystore.",
				e);
		}
		return signature;
	}

}
