package com.pallas.unicore.security;

import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.ajo.AbstractJob;
import org.unicore.upl.ConsignJob;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.util.ExtensionPlugable;

/**
 *
 *@author
 *@created    April 22, 2002
 *@version    $Id: JobConverter.java,v 1.3 2003/06/30 06:58:47 rrate Exp $
 */
public class JobConverter {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.security");

	public final static ConsignJob convert(
		AbstractJob job,
		boolean polling,
		boolean initSpecial)
		throws Exception {
		if (initSpecial) {
			logger.log(
				Level.INFO,
				"Initializing SSO for site " + job.getVsite());
			initSSO(job);
		}
		AbstractJob convertedJob = JobSigner.convertTo(job);
		ConsignJob cj = new ConsignJob();
		cj.setTarget(convertedJob.getAJOId());
		cj.setVsite(convertedJob.getVsite());
		cj.setEndorser(convertedJob.getEndorser());
		cj.setAJO(convertedJob.getConsignForm());
		cj.setSignature(convertedJob.getSignature());
		cj.setPolling(polling);
		return cj;
	}

	/**
	 *  Used to initialize special features of a vsite via a plugin. The plugins
	 *  are asked to provide special stuff for the vsite in question via the
	 *  setupSpecialVsiteFeatures() method. This is use for exmple to add a
	 *  Globus proxy certificate to an AJO.
	 *
	 *@return    Description of the Return Value
	 */
	private static void initSSO(AbstractJob job) {
		Vector plugins = Client.getPluginManager().getPluginClasses();
		for (int i = 0; i < plugins.size(); i++) {
			if (plugins.elementAt(i) instanceof ExtensionPlugable) {
				try {
					ExtensionPlugable plugable =
						(ExtensionPlugable) plugins.elementAt(i);
					Object returnedObject =
						plugable.setupSpecialVsiteFeatures(job.getVsite(), job);
				} catch (Exception e) {
					logger.log(
						Level.WARNING,
						"Initializing vsite features for site "
							+ job.getVsite()
							+ " failed.",
						e);
					return;
				}
			}
		}
		return;
	}
}
