package com.pallas.unicore.utility;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Vector;

/**
 *  Description:
 *
 *@author     Kirsten Foerster
 *@created    12. Mai 2003
 *@version    $Id: VectorComparator.java,v 1.3 2003/06/30 06:59:36 rrate Exp $
 */

public class VectorComparator extends Vector {

	/**
	 *  Constructor for the VectorComparator object
	 */
	public VectorComparator() {
		super();
	}

	/**
	 *  Constructor for the VectorComparator object
	 *
	 *@param  c  Description of the Parameter
	 */
	public VectorComparator(Collection c) {
		super(c);
	}

	/**
	 *  Comparates the current vector to the elements of the transferred
	 *  enumeration.
	 *
	 *@param  enum  Description of the Parameter
	 *@return       Returns a vector which elements are the intersection of the
	 *      current vector and the transferred enumeration.
	 */
	public Vector contains(Enumeration enum) {
		Vector result = new Vector();
		if (enum == null) {
			return result;
		}
		while (enum.hasMoreElements()) {
			Object object = enum.nextElement();
			if (this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 *  Comparates the current vector to the transferred vector.
	 *
	 *@param  vector  Description of the Parameter
	 *@return         Returns a vector which elements are the intersection of
	 *      both vectors.
	 */
	public Vector contains(Vector vector) {
		Vector result = new Vector();
		if (vector == null) {
			return result;
		}
		for (int i = 0; i < vector.size(); i++) {
			Object object = vector.elementAt(i);
			if (this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 *  Comparates the current vector to the elements of the transferred
	 *  enumeration.
	 *
	 *@param  enum  Description of the Parameter
	 *@return       Returns a vector which elements are only components in the
	 *      current vector.
	 */
	public Vector excludes(Enumeration enum) {
		if (enum == null) {
			return this;
		}
		Vector vector = new Vector();
		while (enum.hasMoreElements()) {
			vector.add(enum.nextElement());
		}
		return this.excludes(vector);
	}

	/**
	 *  Comparates the current vector to the transferred vector.
	 *
	 *@param  vector  Description of the Parameter
	 *@return         Returns a vector which elements are only components in the
	 *      current vector
	 */
	public Vector excludes(Vector vector) {
		Vector result = new Vector();
		if (vector == null) {
			return this;
		}
		for (int i = 0; i < this.size(); i++) {
			Object object = this.elementAt(i);
			if (!vector.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 *  Comparates the current vector to the elements of the transferred
	 *  enumeration.
	 *
	 *@param  enum  Description of the Parameter
	 *@return       Returns a vector which elements are only components in the
	 *      transferred enumeration.
	 */
	public Vector isExcluded(Enumeration enum) {
		Vector result = new Vector();
		if (enum == null) {
			return null;
		}
		while (enum.hasMoreElements()) {
			Object object = enum.nextElement();
			if (!this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}

	/**
	 *  Comparates the current vector to the transferred vector.
	 *
	 *@param  vector  Description of the Parameter
	 *@return         Returns a vector which elements are only components in the
	 *      transferred vector
	 */
	public Vector isExcluded(Vector vector) {
		Vector result = new Vector();
		if (vector == null) {
			return null;
		}
		for (int i = 0; i < vector.size(); i++) {
			Object object = vector.elementAt(i);
			if (!this.contains(object)) {
				result.addElement(object);
			}
		}
		return result;
	}
}
