package com.pallas.unicore.utility;

import java.util.Arrays;
import java.util.Vector;

/**
 *  Sort a vector of comparable objects. If objects do not implement the
 *  Comparable interface use their String representation for comparision.
 *
 *@author     Ralf Ratering
 *@created    4. April 2002
 *@version    $Id: VectorSorter.java,v 1.2 2003/06/30 06:59:35 rrate Exp $
 */
public class VectorSorter {
	private static boolean ascending;
	private static StringComparator stringComparator = new StringComparator();
	private final static boolean USE_ARRAY_SORT = true;
	private static boolean useStrings;
	private static Vector vector;

	/**
	 *  Description of the Method
	 */
	private static void arraySort() {
		Object[] result = vector.toArray();
		if (useStrings) {
			Arrays.sort(result, stringComparator);
		} else {
			Arrays.sort(result);
		}
		vector.clear();
		if (ascending) {
			for (int i = 0; i < result.length; i++) {
				vector.add(result[i]);
			}
		} else {
			for (int i = 0; i < result.length; i++) {
				vector.add(result[result.length - i - 1]);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  i       Description of the Parameter
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	private static int compare(int i, Object object) {
		if (useStrings) {
			int value =
				vector.elementAt(i).toString().compareTo(object.toString());
			return (ascending ? value : -value);
		}
		int value = ((Comparable) vector.elementAt(i)).compareTo(object);
		return (ascending ? value : -value);
	}

	/**
	 *  The main program for the StringVectorSorter class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		Vector v = new Vector();

		v.add("afs");
		v.add("sfh");
		v.add("afs");

		v.add("aaatwrzuzthj");
		v.add("dss");
		v.add("afwrths");
		v.add("awrthfs");
		v.add("zhwthtwrhad");
		v.add("aawrha");
		v.add("dswww3456s");
		v.add("afs");
		v.add("afs");
		v.add("zad");
		v.add("gad");
		v.add("fss");
		v.add("afs");

		System.err.println(v);
		VectorSorter.sort(v, false);
		System.err.println(v);

	}

	/**
	 *  Description of the Method
	 *
	 *@param  l  Description of the Parameter
	 *@param  r  Description of the Parameter
	 *@return    Description of the Return Value
	 */
	private static int partition(int l, int r) {
		Object pivot = vector.elementAt((l + r) / 2);
		int i = l - 1;
		int j = r + 1;
		while (true) {
			while (compare(--j, pivot) > 0) {
				;
			}
			while (compare(++i, pivot) < 0) {
				;
			}
			if (i < j) {
				swap(i, j);
			} else {
				return j;
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  l  Description of the Parameter
	 *@param  r  Description of the Parameter
	 */
	private static void quicksort(int l, int r) {
		if (l < r) {
			int q = partition(l, r);
			quicksort(l, q);
			quicksort(q + 1, r);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  v                       Description of the Parameter
	 *@return                         Description of the Return Value
	 *@exception  ClassCastException  Description of the Exception
	 */
	public static Vector sort(Vector v) throws ClassCastException {
		sort(v, true);
		return v;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  _vector                 Description of the Parameter
	 *@param  _ascending              Description of the Parameter
	 *@return                         Description of the Return Value
	 *@exception  ClassCastException  Description of the Exception
	 */
	public static Vector sort(Vector _vector, boolean _ascending)
		throws ClassCastException {
		if (_vector.isEmpty()) {
			return _vector;
		}

		boolean _useStrings = !(_vector.firstElement() instanceof Comparable);

		return sort(_vector, _ascending, _useStrings);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  _vector                 Description of the Parameter
	 *@param  _ascending              Description of the Parameter
	 *@param  _useStrings             Description of the Parameter
	 *@return                         Description of the Return Value
	 *@exception  ClassCastException  Description of the Exception
	 */
	public static Vector sort(
		Vector _vector,
		boolean _ascending,
		boolean _useStrings)
		throws ClassCastException {
		vector = _vector;
		ascending = _ascending;
		useStrings = _useStrings;
		if (vector.isEmpty()) {
			return vector;
		}
		if (USE_ARRAY_SORT) {
			arraySort();
		} else {
			quicksort(0, vector.size() - 1);
		}
		return vector;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  i  Description of the Parameter
	 *@param  j  Description of the Parameter
	 */
	private static void swap(int i, int j) {
		Object temp = vector.elementAt(i);
		vector.setElementAt(vector.elementAt(j), i);
		vector.setElementAt(temp, j);
	}

	/**
	 *  Private Constructor for the VectorSorter object
	 */
	private VectorSorter() {
	}
}
