package com.pallas.util.queues;

import java.security.cert.X509Certificate;
import java.util.logging.Logger;

/**
 *  Description of the Class
 *
 *@author     Thomas Kentemich
 *@created    17. Februar 2003
 *@version    $Id: WorkerThread.java,v 1.3 2003/06/30 06:59:57 rrate Exp $
 */
public class WorkerThread extends Thread {

    /**
     *  Description of the Field
     */
    protected static Logger logger = Logger.getLogger("com.pallas.util.queues");


    /**
     *  Gets the instance attribute of the WorkerThread class
     *
     *@param  group                     Description of the Parameter
     *@param  name                      Description of the Parameter
     *@param  queue                     Description of the Parameter
     *@return                           The instance value
     *@exception  InterruptedException  Description of the Exception
     */
    public static WorkerThread getInstance(ThreadGroup group, String name, RequestQueue queue) throws InterruptedException {
        return new WorkerThread(group, name, queue);
    }

    private X509Certificate peerCertificate;
    private RequestQueue queue;
    private boolean stopped = false;
    private IWork work;


    /**
     *  Constructor for the WorkerThread object
     */
    private WorkerThread() { }


    /**
     *  Constructor for the WorkerThread object
     *
     *@param  group                     Description of the Parameter
     *@param  name                      Description of the Parameter
     *@param  queue                     Description of the Parameter
     *@exception  InterruptedException  Description of the Exception
     */
    private WorkerThread(ThreadGroup group, String name, RequestQueue queue) throws InterruptedException {
        super(group, name);
        this.queue = queue;

    }
    
    public X509Certificate getPeerCertificate() {
	    return peerCertificate;
    }

    
    /**
     *  Main processing method for the WorkerThread object
     */
    public void run() {
        for(; ; ) {
            try {
                work = queue.dequeue();
                if(work instanceof StopObject) {
                    logger.info("Worker " + this.getName() + " got stop signal.");
                    break;
                }
                work.performTask();
            }
            catch(InterruptedException e) {
                logger.severe("WorkerThread interrupted");
            }
        }
        logger.info("Worker " + this.getName() + " stopped.");
    }


    public void setPeerCertificate(X509Certificate peer) {
	    this.peerCertificate = peer;
    }
    
    
    /**
     *  Description of the Method
     */
    public synchronized void stopThread() {
        stopped = true;
    }
}
