package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Requests a report of the Quality of Service that will
 * be provided for the supplied resource request.
 * <p>
 * A Vsite will indicate that it can proivde QoS information by
 * including an instance of {@link org.unicore.resources.QoSCheck}
 * in its Resource Description.
 *
 * Implicit in this Task is the function of it's parent task
 * {@link org.unicore.ajo.CheckResources}, i.e. a successful CheckQoS 
 * implies a successful CheckResources.
 *
 * @since AJO 3.6
 *
 * @see org.unicore.outcome.CheckQoS_Outcome
 * 
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @version CheckQoS.java 1.4 09/09/02
 *
 **/
public final class CheckQoS extends CheckResources {

    static final long serialVersionUID = 2487603748281830821L;

    /**
     * Create a new CheckQoS.
     * <p>
     * @param name The name of the CheckQoS
     * @param extra_information Client supplied extra information about the CheckQoS
     * @param dag DAG of resource sets representing the resource requirements of
     * the job.
     *
     **/
    public CheckQoS(String name, byte[] extra_information, TaskResourceDAG dag) {
        super(name,extra_information, dag);
    }

    /**
     * Create a new CheckQoS.
     * <p>
     * @param name The name of the CheckQoS
     * @param dag DAG of resource sets representing the resource requirements of
     * the job.
     *
     **/
    public CheckQoS(String name, TaskResourceDAG dag) {
        super(name, (byte []) null, dag);
    }

    /**
     * Create a new CheckQoS.
     * <p>
     * @param name The name of the CheckQoS
     *
     **/
    public CheckQoS(String name) {
        this(name,(byte[]) null, (TaskResourceDAG) null);
    }

    public CheckQoS() {
        this("");
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          11 Dec 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
