package org.unicore.ajo;

import org.unicore.AAIdentifier;

/**
 * CopyActionOutcome returns a copy of the target AbstractAction's Outcome.
 * <p>
 * The copy is "shallow" using only the information available on the
 * execution NJS. There is no querying of other NJSs for the Outcome
 * of any child AJOs.
 * However, all the Outcome that is available to the NJS/TSI
 * at the time of execution is returned.
 *<p>
 * @see org.unicore.outcome.Outcome
 * @see org.unicore.outcome.CopyActionOutcome_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version CopyActionOutcome.java 1.6 09/09/02 
 *
 **/
public final class CopyActionOutcome extends QueryService {

    static final long serialVersionUID = 8496942553500194716L;

    /**
     * Create a new CopyActionOutcome.
     * <p>
     * @param name The name of the CopyActionOutcome
     * @param extra_information Client supplied extra information about the CopyActionOutcome
     * @param target_action The AbstractAction whose Outcome is to be copied
     *
     **/
    public CopyActionOutcome( String name,
                              byte[] extra_information,
                              AAIdentifier target_action
                               ) {
        super(name,extra_information);

        this.target_action = target_action;
    }

    /**
     * Create a new CopyActionOutcome.
     * <p>
     * @param name The name of the CopyActionOutcome
     * @param target_action The AbstractAction whose Outcome is to be copied
     *
     **/
    public CopyActionOutcome( String name,
                              AAIdentifier target_action
                               ) {
        this(name,(byte[])null,target_action);
    }

    /**
     * Create a new CopyActionOutcome.
     * <p>
     * @param name The name of the CopyActionOutcome
     * @param target_action The AbstractAction whose Outcome is to be copied
     *
     **/
    public CopyActionOutcome(String name) {
        this(name,(byte[])null,(AAIdentifier) null);
    }

    public CopyActionOutcome() {
        this("");
    }

    private AAIdentifier target_action;
    /**
     * Return the identifier of the action whose Outcome
     * should be returned.
     *
     **/
    public AAIdentifier getActionId() {return target_action;}

    /**
     * Set the identifier of the action whose Outcome
     * should be returned.
     * <p>
     * @since AJO 3
     *
     **/
    public void setActionId(AAIdentifier target_action) {
        this.target_action = target_action;
    }

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           7 Dec 1998 SvdB Constructor to public 
//           5 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
