package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Copy a Portfolio to the Outcome of the task. 
 * <p>
 * The contents of the target Portfolio at the time of
 * the execution of the task are copied into
 * the Outcome of this task. This is the way that files 
 * are returned to a client. 
 *<p>
 * The location of the Portfolio is the Uspace.
 *
 * @deprecated AJO 4.0 Use {@link CopyPortfolioToOutcome}
 *
 * @see Portfolio
 * @see org.unicore.outcome.CopyPortfolio_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version CopyPortfolioTask.java 1.6 06/11/02 
 * 
 **/
public final class CopyPortfolioTask extends FileTransfer {

    static final long serialVersionUID = -6622196043030583533L;

    /**
     * Create a new CopyPortfolioTask.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param target The Portfolio to copy to the Outcome.
     *
     **/
    public CopyPortfolioTask( String name,
                              byte[] extra_information,
                              ResourceSet resources,
                              Portfolio target
                            ) {
        super(name,extra_information,resources);
        this.target = target;
    }

    /**
     * Create a new CopyPortfolioTask.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param target The Portfolio to copy to the Outcome.
     *
     **/
    public CopyPortfolioTask( String name,
                              ResourceSet resources,
                              Portfolio target
                            ) {

        this(name,(byte[]) null, resources, target);

    }

    /**
     * Create a new CopyPortfolioTask with this name
     * and no Portfolio to copy.
     *
     **/
    public CopyPortfolioTask(String name) {
        this(name, (byte[]) null, (ResourceSet) null, (Portfolio) null);
    }

    public CopyPortfolioTask() {
        this("");
    }

    private Portfolio target;
    /**
     * Return the Portfolio to copy.
     *
     **/
    public Portfolio getTarget() {return target;}

    /**
     * Set the Portfolio to copy.
     * <p>
     * This Portfolio must be a Portfolio that is
     * created by a task in the same AJO as this instance
     * of CopyPortfolioTask. The creating task should
     * also have completed execution before this
     * CopyPortfolioTask is executed.
     *
     **/
    public void setTarget(Portfolio target) {this.target = target;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, etc
//          24 Nov 2000 SvdB Removed copy_now field Improved comments
//           9 Jun 2000 SvdB Added copy_now field
//           5 May 2000 SvdB V3.0 
//           5 Mar 1998 SvdB Added setTarget method 
//           7 Dec 1998 SvdB Second release to Unicore 
//           5 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
