package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Copy a spooled Portfolio (does not remove the spooled copy)
 * <p>
 * @see Portfolio
 * @see org.unicore.outcome.CopySpooled_Outcome
 * @see Spool
 * @see UnSpool
 * @see DeleteSpooled
 * 
 * @deprecated AJO 4.0 Use {@link GetPortfolio} with the {@link org.unicore.resources.Spool} resource.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3.5.2
 *
 * @version CopySpooled.java 1.2 05/13/02
 * 
 **/
public final class CopySpooled extends FileTask {

    static final long serialVersionUID = 7269335196024906745L;

    /**
     * Create a new task to copy a spooled Portfolio that
     * requests the given resources and saves the 
     * supplied extra information.
     *
     * @param name The name of the task.
     * @param portfolio The Portfolio to copy
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     *
     **/
    public CopySpooled( String name,
                  Portfolio portfolio,
                  byte[] extra_information,
                  ResourceSet resources
                ) {
        super(name,extra_information,resources);
        this.portfolio = portfolio;
    }

    /**
     * Create a new task to copy a spooled  Portfolio that
     * requests the given resources.
     *
     * @param name The name of the task.
     * @param portfolio The Portfolio to copy
     * @param resources The resources requested by this task.
     *
     **/
    public CopySpooled( String name,
                   Portfolio portfolio,
                   ResourceSet resources
                 ) {

        this(name,portfolio, (byte[]) null,resources);
    }

    /**
     * Create a new CopySpooled with this name and
     * empty values otherwise.
     *
     **/
    public CopySpooled(String name) {
        this(name, (Portfolio) null, (byte[]) null, (ResourceSet) null);
    }

    public CopySpooled() {
        this("");
    }

    private Portfolio portfolio;
    /**
     * Return the spooled Portfolio to be copied by this task.
     *
     **/
    public Portfolio getPortfolio() {
        return portfolio;
    }

    /**
     * Set the spooled Portfolio to be copied by this task.
     **/
    public void setPortfolio(Portfolio portfolio) {this.portfolio = portfolio;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, deprecated
//          01 Nov 2001 SvdB Created. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
