package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Create a directory.
 * <p>
 * The location of the directory is determined by the resources
 * requested (see {@link FileAction})
 * <p>
 *
 * @see org.unicore.outcome.CreateDirectory_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version CreateDirectory.java 1.5 09/09/02  
 * 
 **/
public final class CreateDirectory extends FileAction {

    static final long serialVersionUID = -822774845905410200L;

    /**
     * Create a new CreateDirectory.
     * <p>
     * @param name The name of the CreateDirectory
     * @param extra_information Client supplied extra information about the CreateDirectory
     * @param resources Resources needed to execute the CreateDirectory
     * @param target The name of the directory to create
     *
     **/
    public CreateDirectory( String name,
                       byte[] extra_information,
                       ResourceSet resources,
                       String target
                     ) {

        super(name,extra_information,resources);

        this.target = target;
    }

    /**
     * Create a new CreateDirectory.
     * <p>
     * @param name The name of the CreateDirectory
     * @param resources Resources needed to execute the CreateDirectory
     * @param target The name of the directory to create
     *
     **/
    public CreateDirectory( String name,
                       ResourceSet resources,
                       String target
                     ) {

        this(name,(byte[]) null, resources, target);
    }

    /**
     * Create a new CreateDirectory.
     * <p>
     * @param name The name of the CreateDirectory
     *
     **/
    public CreateDirectory( String name) {
        this(name,(byte[]) null,(ResourceSet) null, (String) null);
    }

    public CreateDirectory() {
        this("");
    }

    private String target;
    /**
     * Return the name of the directory to create.
     *
     **/
    public String getTarget() {return target;}

    /**
     * Set the name of the directory to create.
     *
     **/
    public void setTarget(String target) {this.target = target;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
