package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Delete a file or a directory.
 * <p>
 * If the target is a directory, then the contents
 * are removed recursively.
 * <p>
 * The location of the file is determined by the resources
 * requested (see {@link FileAction})
 *
 * @see org.unicore.outcome.DeleteFile_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version DeleteFile.java 1.7 09/09/02  
 * 
 **/
public final class DeleteFile extends FileAction {

    static final long serialVersionUID = 2982939825948744004L;

    /**
     * Create a new DeleteFile.
     * <p>
     * @param name The name of the DeleteFile
     * @param extra_information Client supplied extra information about the DeleteFile
     * @param resources Resources needed to execute the DeleteFile
     * @param target The name of the file to delete
     *
     **/
    public DeleteFile( String name,
                   byte[] extra_information,
                   ResourceSet resources,
                   String target
                 ) {

        super(name,extra_information,resources);

        this.target = target;
    }

    /**
     * Create a new DeleteFile.
     * <p>
     * @param name The name of the DeleteFile
     * @param resources Resources needed to execute the DeleteFile
     * @param target The name of the file to delete
     *
     **/
    public DeleteFile( String name,
                   ResourceSet resources,
                   String target
                 ) {
        this(name,(byte[]) null, resources, target);
    }

    /**
     * Create a new DeleteFile.
     * <p>
     * @param name The name of the DeleteFile
     *
     **/
    public DeleteFile(String name) {
        this(name,(byte[]) null,(ResourceSet) null, (String) null);
    }

    public DeleteFile() {
        this("");
    }

    private String target;
    /**
     * Return the name of the file to delete.
     *
     **/
    public String getTarget() {return target;}

    /**
     * Set the name of the file to delete.
     *
     **/
    public void setTarget(String target) {this.target = target;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           9 Jun 2000 SvdB Renamed from delete.java
//           5 May 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
