package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

import org.unicore.PortfolioIdentifier;

/**
 * Delete a Portfolio.
 * <p>
 * Removes a Portfolio so that the NJS no longer has any recored of it.
 * This does not delete the files in the Portfolio, neither does it delete
 * and hidden Portfolios with the same identifier.
 * <p>
 * Portfolios can be deleted in the {@link org.unicore.resources.USpace},
 * {@link org.unicore.resources.AlternativeUspace} or {@link org.unicore.resources.Spool}
 * <p>
 * A use of this task is to force revelation of files written
 * to a Uspace by a PutPortfolio i.e. A PutPortfolio followed by a 
 * DeletePortfolio. Another is to deleted Spooled files after their use.
 * <p>
 * @see Portfolio
 * @see org.unicore.outcome.DeletePortfolio_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version DeletePortfolio.java 1.1 05/13/02
 * 
 **/
public final class DeletePortfolio extends FileTask {

    static final long serialVersionUID = 0;

    /**
     * Create a new DeletePortfolio with this name that
     * requests the given resources and saves the 
     * supplied extra information.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param portfolio The Portfolio to delete
     *
     **/
    public DeletePortfolio( String name,
                             byte[] extra_information,
                             ResourceSet resources,
                             PortfolioIdentifier portfolio
                           ) {

        super(name,extra_information,resources);
        this.portfolio = portfolio;
    }

    /**
     * Create a new DeletePortfolio with this name that
     * requests the given resources.
     *
     * @param name The name of the task.
     * @param resources The resources requested by this task.
     * @param portfolio The Portfolio to delete
     *
     **/
    public DeletePortfolio( String name,
                            ResourceSet resources,
                            PortfolioIdentifier portfolio
                              ) {
        this(name,(byte[]) null,resources,portfolio);
    }

    /**
     * Create a new DeletePortfolio with this name and
     * empty values otherwise.
     * <p>
     **/
    public DeletePortfolio(String name) {
        this(name, (byte[]) null, (ResourceSet) null, (PortfolioIdentifier) null);
    }

    public DeletePortfolio() {
        this("");
    }

    private PortfolioIdentifier portfolio;
    /**
     * Return the Portfolio deleted by this task.
     *
     **/
    public PortfolioIdentifier getPortfolio() {
        return portfolio;
    }

    /**
     * Set the Portfolio deleted by this task.
     *
     **/
    public void setPortfolio(PortfolioIdentifier portfolio) {this.portfolio = portfolio;}

}
//
// Change history:
//          22 Apr 2002 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
