package org.unicore.ajo;

import org.unicore.AAIdentifier;

/**
 * Extend the validity of an AbstractAction.
 * <p>
 * @see org.unicore.outcome.ControlAction_Outcome
 * @see Control
 *
 * @author S. van den Berghe (fecit)
 * @author D. Snelling (fecit)
 *
 * @since AJO 4.0
 *
 * @version ExtendTerminationTime.java 1.4 02/10/03 
 *
 **/
public final class ExtendTerminationTime extends ControlService {

    static final long serialVersionUID = 0;

    /**
     * Create a new ExtendTerminationTime.
     * <p>
     * @param name The name of the ExtendTerminationTime
     * @param extra_information Client supplied extra information about the ControlService
     * @param target_action The target AbstractAction for the ExtendTerminationTime
     * @param length The requested extension length
     *
     **/
    public ExtendTerminationTime(String name,
                         byte[] extra_information,
                         AAIdentifier target_action,
                         long length) {
                            
        super(name,extra_information);

        this.target_action = target_action;
        this.length = length;
    }

    /**
     * Create a new ExtendTerminationTime.
     * <p>
     * @param name The name of the ExtendTerminationTime
     * @param target_action The target AbstractAction for the control
     * @param length The requested extension length
     *
     **/
    public ExtendTerminationTime(String name,
                         AAIdentifier target_action,
                         long length) {
        this(name, (byte[]) null, target_action, length);
    }

    /**
     * Create a new ExtendTerminationTime.
     * <p>
     * @param name The name of the ExtendTerminationTime
     *
     **/
    public ExtendTerminationTime(String name) {
        this(name, (byte[]) null, (AAIdentifier) null, 0);
    }

    public ExtendTerminationTime() {
        this("");
    }

    private AAIdentifier target_action;
    /**
     * Get the AbstractAction to extend.
     *
     **/
    public AAIdentifier getActionId() {return target_action;}

    /**
     * Set the AbstractAction to extend.
     *
     **/
    public void setActionId(AAIdentifier target_action) {this.target_action = target_action;}

    private long length;
    /**
     * Get requested extension length, this length is applied to the <em>initial</em>
     * termination time of the AbstractAction, regardless of any previous extensions.
     *
     * @param length The requested extension of the target's initial termination time in milliseconds.
     *
     **/
    public long getLength() {return length;}

    /**
     * Set requested extension length.
     *
     **/
    public void setLength(long length) {this.length = length;}

}
//
// Change history:
//          23 Apr 2002 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
