package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 *
 * FileActions perform some action on files and directories.
 * <p>
 * The files to which the FileActions will be applied are located through the instances of
 * {@link org.unicore.resources.Storage} found in the resources for the FileAction.
 * The default location for a file is the root of the FileAction's Uspace.
 * <p>
 * If a FileAction uses more than one file, then one of these files is <em>always</em>
 * found in the current Upsace directory (Uspace root or the sub-directory specified by
 * the instance of {@link org.unicore.resources.USpace} found in the resources). The other
 * file can be in Xspace (according to the instance of {@link org.unicore.resources.Storage} found in
 * the resources), the default location for this is Uspace root. To apply a FileAction to two files in the Uspace
 * use the defaults and put the required sub-directories in the file names.
 * <p>
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version FileAction.java 1.5 09/09/02 
 * 
 **/
public abstract class FileAction extends AbstractTask {

    static final long serialVersionUID = 3701606850726081182L;
    
    /**
     * Create a new FileAction.
     * <p>
     * @param name The name of the FileAction
     * @param extra_information Client supplied extra information about the FileAction
     * @param resources Resources needed to execute the FileAction
     *
     **/
    protected FileAction( String name,
                          byte[] extra_information,
                          ResourceSet resources
                      ) {
        super(name,extra_information,resources);
    }

}
//
// Change history:
//           6 May 2000 SvdB Created for V3.0 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
