package org.unicore.ajo;

/**
 * A ForGroup contains contains a DAG of actions that will be executed one
 * or more times depending on the iterations parameter.
 * <p>
 * The iterations parameter is examined, if it us greater than 0, then the
 * ActionGroup contained in a RepeatGroup is executed within the same Uspace as 
 * the root AJO of the RepeatGroup (subsequent executions of the ActionGroup 
 * will also take place within the same Uspace).
 * Once all actions in the action group have completed
 * successfully, the iterations parameter decremented by one. If it is zero, the
 * ForGroup terminates with status SUCCESSFUL. Otherwise, the NJS processes
 * the action group as if all statuses were again PENDING.  The ActionGroup returns
 * to the READY status before returning to RUNNING for the next iteration.
 * <p>
 * If the ActionGorup does not complete successfully, then the iterations are
 * stopped and the RepeatGroup gets a final status of NOT_SUCCESSFUL.
 * <p>
 * Note that all actions in one iteration
 * must be complete before the next iteration can start, e.g. this is not a parallel
 * construct.
 * <p>
 * The outcome of a ForGroup is a list of ActionGroup_Outcomes, one for each
 * iteration.
 * <p>
 * @see org.unicore.outcome.ForGroup_Outcome
 *
 * @author D. Snelling (fecit)
 *
 * @since AJO 4.0
 *
 * @version ForGroup.java 1.3 09/09/02
 * 
 **/
public final class ForGroup extends ActionGroup {

    static final long serialVersionUID = 0;

    /**
     * Create a new ForGroup.
     * <p>
     * @param name The name of the For
     * @param extra_information Client supplied extra information about the ForGroup.
     * @param iterations The number of times that the ActionGroup will be repeated
     *
     **/
    public ForGroup( String name,
                        byte[] extra_information,
                        int iterations 
                      ) {
        super(name,extra_information);
        
        this.iterations = iterations;
    }

    /**
     * Create a new ForGroup.
     * <p>
     * @param name The name of the For
     * @param iterations The number of times that the ActionGroup will be repeated
     *
     **/
    public ForGroup( String name,
                        int iterations
                      ) {
        this(name, (byte[]) null, iterations);
    }

    /**
     * Create a new ForGroup.
     * <p>
     * @param name The name of the For
     *
     **/
    public ForGroup(String name) {
        this(name, (byte[]) null, 0);
    }

    public ForGroup() {
        this("");
    }

    private int iterations;

    /**
     * Return the number of times that the ActionGroup will be repeated.
     *
     **/
    public int getIterations() {return iterations;}

    /**
     * Set the repeat count.
     *
     **/
    public void setIterations (int iterations) {this.iterations = iterations;}

}
//
//  Change history:
//       16 Apr 2002  DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
