package org.unicore.ajo;

import org.unicore.AAIdentifier;

/**
 * Get the status of an AbstractAction.
 *
 * @see org.unicore.outcome.AbstractActionStatus
 * @see org.unicore.outcome.Outcome#getStatus
 * @see org.unicore.outcome.GetActionStatus_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version GetActionStatus.java 1.5 09/09/02 
 *
 **/
public final class GetActionStatus extends QueryService {

    static final long serialVersionUID = -8917764727989009820L;

    /**
     * Create a new GetActionStatus.
     * <p>
     * @param name The name of the GetActionStatus
     * @param extra_information Client supplied extra information about the GetActionStatus
     * @param target_action The AbstractAction whose status is wanted
     *
     **/
    public GetActionStatus( String name,
                            byte[] extra_information,
                            AAIdentifier target_action
                          ) {
        super(name, extra_information);

        this.target_action = target_action;
    }

    /**
     * Create a new GetActionStatus.
     * <p>
     * @param name The name of the GetActionStatus
     * @param target_action The AbstractAction whose status is wanted
     *
     **/
    public GetActionStatus( String name, AAIdentifier target_action) {
        this(name, (byte[]) null, target_action);
    }

    /**
     * Create a new GetActionStatus.
     * <p>
     * @param name The name of the GetActionStatus
     *
     **/
    public GetActionStatus(String name) {
        this(name, (byte[]) null, (AAIdentifier) null);
    }

    public GetActionStatus() {
        this("");
    }

    private AAIdentifier target_action;
    /**
     * Get the identifier of the AbstractAction whose status
     * is wanted.
     *
     **/
    public AAIdentifier getActionId() {return target_action;}

    /**
     * Set the identifier of the AbstractAction whose status
     * is wanted.
     *
     **/
    public void setActionId(AAIdentifier target_action) {
        this.target_action = target_action;
    }

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Constructor to public 
//           7 Dec 1998 SvdB Second release to Unicore 
//           5 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
