package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Get a listing of a directory or a file.
 * <p>
 * The location of the directory is determined by the resources
 * requested (see {@link FileAction})
 * 
 * @see org.unicore.outcome.ListDirectory_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3 (AJO 2 in another form)
 *
 * @version ListDirectory.java 1.6 09/09/02 
 *
 **/
public final class ListDirectory extends FileAction {

    static final long serialVersionUID = -7087758942346999363L;

    /**
     * Create a new ListDirectory.
     * <p>
     * @param name The name of the ListDirectory
     * @param extra_information Client supplied extra information about the ListDirectory
     * @param resources Resources needed to execute the ListDirectory
     * @param recursive Provide a recursive listing?
     * @param target The name of the directory to list
     *
     **/
    public ListDirectory(String name,
                         byte[] extra_information,
                         ResourceSet resources,
                         boolean recursive,
                         String target
                           ) {
        super(name, extra_information, resources);

        this.recursive = recursive;
        this.target = target;
    }

    /**
     * Create a new ListDirectory.
     * <p>
     * @param name The name of the ListDirectory
     * @param resources Resources needed to execute the ListDirectory
     * @param recursive Provide a recursive listing?
     * @param target The name of the directory to list
     *
     **/
    public ListDirectory(String name,
                         ResourceSet resources,
                         boolean recursive,
                         String target
                           ) {
        this(name,(byte[]) null, resources, recursive, target);
    }

    /**
     * Create a new ListDirectory.
     * <p>
     * @param name The name of the ListDirectory
     *
     **/
    public ListDirectory(String name) {
        this(name,(byte[]) null,(ResourceSet) null, false, "");
    }

    public ListDirectory() {
        this("");
    }

    private boolean recursive;
    /**
     * List the directory and all subdirectories?
     * <p>
     * Otherwise just list the directory.
     *
     * @deprecated At AJO 4.0. This can lead to arbitrarly long execution times so never used by client
     *
     **/
    public boolean isRecursive() {return recursive;}

    /**
     * Set recursive listing.
     *
     * @deprecated At AJO 4.0. This can lead to arbitrarly long execution times so never used by client
     *
     **/
    public void setRecursive(boolean recursive) {this.recursive = recursive;}

    private String target;
    /**
     * Return the root directory for the listing. 
     *
     **/
    public String getTarget() {return target;}

    /**
     * Set the root directory for the listing. 
     *
     **/
    public void setTarget(String target) {this.target = target;}

}
//
//  Change history:
//          11 Apr 2002 SvdB Deprecated recursive
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB Created for V3.0
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
