package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Move a Fifo to the Outcome of the task. 
 * <p>
 * A {@link org.unicore.upl.RetrieveOutcome} including the outcome
 * of this task will cause the NJS to read from the incarnated Fifo
 * an so the RetrieveOutcome will complete when the application that
 * writes to the incarnated Fifo closes the incarnated Fifo.
 * <p>
 * Note that this operation is a <em>move</em>, so successful
 * execution of this task means that the Fifo is no longer available
 * in the original Uspace.
 * <p>
 * The location of the Fifo is determined by the the
 * Storage resources. These cane be either {@link org.unicore.resources.USpace} or
 * {@link org.unicore.resources.AlternativeUspace}.
 * The default is the root ofthe AJO's Uspace.
 *
 * @see Fifo
 * @see org.unicore.outcome.MoveFifoToOutcome_Outcome
 *
 * @author S. van den Berghe (fecit)
 * @author D. Snelling (fecit)
 *
 * @since AJO 4.0
 *
 * @version MoveFifoToOutcome.java 1.1 05/13/02 
 * 
 **/
public final class MoveFifoToOutcome extends FileTransfer {

    static final long serialVersionUID = 0;

    /**
     * Create a new MoveFifoToOutcome.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param target The Fifo to Move to the Outcome.
     *
     **/
    public MoveFifoToOutcome( String name,
                              byte[] extra_information,
                              ResourceSet resources,
                              Fifo target
                            ) {
        super(name,extra_information,resources);
        this.target = target;
    }

    /**
     * Create a new MoveFifoToOutcome.
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param target The Fifo to Move to the Outcome.
     *
     **/
    public MoveFifoToOutcome( String name,
                              ResourceSet resources,
                              Fifo target
                            ) {

        this(name,(byte[]) null, resources, target);

    }

    /**
     * Create a new MoveFifoToOutcome with this name
     * and no Fifo to move.
     *
     **/
    public MoveFifoToOutcome(String name) {
        this(name, (byte[]) null, (ResourceSet) null, (Fifo) null);
    }

    public MoveFifoToOutcome() {
        this("");
    }

    private Fifo target;
    /**
     * Return the Fifo to Move.
     *
     **/
    public Fifo getTarget() {return target;}

    /**
     * Set the Fifo to Move.
     *
     **/
    public void setTarget(Fifo target) {this.target = target;}

}
//
// Change history:
//          23 Apr 2002 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
