package org.unicore.ajo;

/**
 * Test the time of execution against a specified time.
 * <p>
 * The Outcome of an ProbeTime contains a {@link org.unicore.outcome.Decision}
 * which is {@link org.unicore.outcome.ProbeTime_Outcome}.LATER if the time of execution
 * if later than the test time and {@link org.unicore.outcome.ProbeTime_Outcome}.EARLIER
 * otherwise.
 * 
 * @see If
 * @see RepeatGroup
 * @see org.unicore.outcome.Decision
 * @see org.unicore.outcome.ProbeTime_Outcome
 *
 * @author Sven van den Berghe (Fujitsu Laboratories of Europe)
 *
 * @since AJO 4.0
 *
 * @version ProbeTime.java 1.7 09/18/02
 *
 **/
public class ProbeTime extends QueryService implements Decidable {

    static final long serialVersionUID = 400;

    /**
     * Create a new ProbeTime.
     * <p>
     * @param name The name of the ProbeTime
     * @param extra_information Client supplied extra information about the ProbeTime
     * @param time The test time
     *
     **/
    public ProbeTime(String name, byte[] extra_information, java.util.Date time) {
        super(name,extra_information);
        this.time = time;
    }

    /**
     * Create a new ProbeTime.
     * <p>
     * @param name The name of the ProbeTime
     * @param time The test time
     *
     **/
    public ProbeTime(String name, java.util.Date time) {
        this(name,null,time);
    }

    /**
     * Create a new ProbeTime.
     * <p>
     * @param name The name of the ProbeTime
     *
     **/
    public ProbeTime(String name) {
        this(name,null,null);
    }

    /**
     * Create an instance of ProbeTime.
     *
     **/
    public ProbeTime() {
        this("");
    }

    private java.util.Date time;

    /**
     * Set the test time.
     * <p>
     * If the time is not set, then the ProbeTime ends with a Decision of {@link org.unicore.outcome.ProbeTime_Outcome}.LATER
     *
     **/
    public void setTime(java.util.Date time) {
        this.time = time;
    }

    /**
     * @return The test expression
     *
     **/
    public java.util.Date getTime() {return time;}

}
//
// Change history:
//           8 Aug 2002 SvdB Decidable, removed expression
//          17 Apr 2002 SvdB Created for V4.0 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
