package org.unicore.ajo;

import org.unicore.Unicore;

/**
 * ResourceDependency is an ordering relationship for a DAG of resource
 * sets derived from the DAG of AbstractActions in an
 * {@link org.unicore.ajo.ActionGroup}.
 *
 * @author S. van den Berghe (fecit)
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.6
 *
 * @version ResourceDependency.java 1.4 07/04/02
 * 
 **/
public final class ResourceDependency extends Unicore {

        static final long serialVersionUID = 4349457533337665552L;

        private TaskResource predecessor;
        /**
         * Get the predecessor.
         *
         **/
        public TaskResource getPredecessor() {return predecessor;}

        private TaskResource successor;
        /**
         * Get the successor.
         *
         * @return The second <tt>TaskResource</tt> of the <tt>ResourceDependency</tt>.
         *
         **/
        public TaskResource getSuccessor() {return successor;}

        /**
         * Create a new ResourceDependency.
         *
         * @param pred The predecessor TaskResource
         * @param succ The successor TaskResource
         *
         **/
        public ResourceDependency(TaskResource pred, TaskResource suc) {
            predecessor = pred;
            successor   = suc;
        }

        public ResourceDependency() {
            this(null,null);
        }

        /**
         * ResourceDependencies are the same if they have the same predecessor
         * and successor. 
         *
         **/
        public boolean equals(Object in) {
         return ( in.getClass() ==  this.getClass() &&
             ((ResourceDependency)in).getPredecessor().equals( predecessor ) &&
             ((ResourceDependency)in).getSuccessor().equals( successor ) );
        }


        // ResourceDependencies are likely to be kept in collections and
        // so hashCode will be called. If a ResourceDependency is equal we
        // want the same hashCode. This will clash, but what else
        // can we do?

        public int hashCode() { 
            return predecessor.hashCode() ^ successor.hashCode(); 
        }
}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          28 Jun 2002 DFS Correcte cast in equals()
//          11 Dec 2001 DFS Created by copying Dependency
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
