package org.unicore.ajo;

import org.unicore.AAIdentifier;

/**
 * Test the result of the execution of the test action.
 * <p>
 * The Outcome of an TestSuccess contains a {@link org.unicore.outcome.Decision}
 * which is {@link org.unicore.outcome.TestSuccess_Outcome}.SUCCESSFUL if the test AbstractAction 
 * has an {@link org.unicore.outcome.AbstractActionStatus} equivalent to SUCCESSFUL and is
 * {@link org.unicore.outcome.TestSuccess_Outcome}.NOT_SUCCESSFUL if the test AbstractAction
 * has an {@link org.unicore.outcome.AbstractActionStatus} equivalent to NOT_SUCCESSFUL.
 * <p>
 * If the test AbstractAction has any other status (NOT_DONE, NEVER_RUN or NEVER_TAKEN), then
 * the TestSuccess fails in execution.
 * 
 * @see If
 * @see RepeatGroup
 * @see org.unicore.outcome.Decision
 * @see org.unicore.outcome.TestSuccess_Outcome
 *
 * @author Sven van den Berghe (Fujitsu Laboratories of Europe)
 *
 * @since AJO 4.0
 *
 * @version TestSuccess.java 1.4 11/29/02
 *
 **/
public class TestSuccess extends QueryService implements Decidable {

    static final long serialVersionUID = 400;

    /**
     * Create a new TestSuccess.
     * <p>
     * @param name The name of the TestSuccess
     * @param extra_information Client supplied extra information about the TestSuccess
     * @param AAIdentifier The test AbstractAction
     *
     * @deprecated The target should be given as an AbstractAction
     *
     **/
    public TestSuccess(String name, byte[] extra_information, AAIdentifier test_aai) {
        super(name,extra_information);
        this.test_aa = test_aai;
    }

    /**
     * Create a new TestSuccess.
     * <p>
     * @param name The name of the TestSuccess
     * @param extra_information Client supplied extra information about the TestSuccess
     * @param tset_aa The test AbstractAction
     *
     **/
    public TestSuccess(String name, byte[] extra_information, AbstractAction test_action) {
        super(name,extra_information);
        this.test_action = test_action;
    }

    /**
     * Create a new TestSuccess.
     * <p>
     * @param name The name of the TestSuccess
     * @param AAIdentifier The test AbstractAction
     *
     * @deprecated The target should be given as an AbstractAction
     *
     **/
    public TestSuccess(String name, AAIdentifier test_aai) {
        this(name,null,test_aai);
    }

    /**
     * Create a new TestSuccess.
     * <p>
     * @param name The name of the TestSuccess
     * @param test_aa The test AbstractAction
     *
     **/
    public TestSuccess(String name, AbstractAction test_action) {
        this(name,null,test_action);
    }

    /**
     * Create a new TestSuccess.
     * <p>
     * @param name The name of the TestSuccess
     *
     **/
    public TestSuccess(String name) {
        this(name,null,(AbstractAction)null);
    }

    /**
     * Create a new TestSuccess.
     *
     **/
    public TestSuccess() {
        this("");
    }

    private AAIdentifier test_aa;

    /**
     * Set the test AbstractAction.
     *
     * @deprecated The target should be given as an AbstractAction
     *
     **/
    public void setActionId(AAIdentifier test_aai) {
        this.test_aa = test_aai;
    }

    /**
     * @return The test AbstractAction identifier
     *
     * @deprecated The target should be given as an AbstractAction
     *
     **/
    public AAIdentifier getActionId() {
        if(test_action != null) {
            return test_action.getId();
        }
        else {
            return test_aa;
        }
    }


    private AbstractAction test_action;

    /**
     * Set the test AbstractAction.
     *
     **/
    public void setTarget(AbstractAction test_action) {
        this.test_action = test_action;
    }

    /**
     * @return The test AbstractAction 
     *
     **/
    public AbstractAction getTarget() {return test_action;}

}
//
// Change history:
//          29 Nov 2002 SvdB Corrected field naming to restore compatibility
//          31 Oct 2002 SvdB Should have Action as target, not the Identifier
//          13 Sep 2002 SvdB Created for V4.0 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
