package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Move the files making up a Portfolio from the spool area
 * to the Uspace of the AJO (deletes the spooled copy).
 * <p>
 * @see Portfolio
 * @see org.unicore.outcome.UnSpool_Outcome
 * @see Spool
 * @see DeleteSpooled
 * @see CopySpooled
 *
 * @deprecated AJO 4.0 Use {@link GetPortfolio} with the {@link org.unicore.resources.Spool} resource followed
 *                     bu a {@link DeletePortfolio}
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3.2
 *
 * @version UnSpool.java 1.4 05/13/02
 * 
 **/
public final class UnSpool extends FileTask {

    static final long serialVersionUID = -5808231991662968119L;

    /**
     * Create a new task to unspool the Portfolio that
     * requests the given resources and saves the 
     * supplied extra information.
     *
     * @param name The name of the task.
     * @param portfolio The Portfolio to unspool
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     *
     **/
    public UnSpool( String name,
                  Portfolio portfolio,
                  byte[] extra_information,
                  ResourceSet resources
                ) {
        super(name,extra_information,resources);
        this.portfolio = portfolio;
    }

    /**
     * Create a new task to unspool the Portfolio that
     * requests the given resources.
     *
     * @param name The name of the task.
     * @param portfolio The Portfolio to unspool
     * @param resources The resources requested by this task.
     *
     **/
    public UnSpool ( String name,
                   Portfolio portfolio,
                   ResourceSet resources
                 ) {

        this(name,portfolio, (byte[]) null,resources);
    }

    /**
     * Create a new UnSpool with this name and
     * empty values otherwise.
     *
     **/
    public UnSpool(String name) {
        this(name, (Portfolio) null, (byte[]) null, (ResourceSet) null);
    }

    public UnSpool() {
        this("");
    }

    private Portfolio portfolio;
    /**
     * Return the Portfolio to be unspooled by this task.
     *
     **/
    public Portfolio getPortfolio() {
        return portfolio;
    }

    /**
     * Set the Portfolio to be unspooled by this task.
     **/
    public void setPortfolio(Portfolio portfolio) {this.portfolio = portfolio;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, deprecated
//          30 Jan 2001 SvdB Created. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
