package org.unicore.ajo;

/**
 * Wait until the specified Time has passed, then complete execution.
 * <p>
 * If the specified time has passed, the execution of this action
 * terminates successful immediately. 
 * <p>
 * If the target AbstractAction has passed through the test status, then the
 * offset time is evaluated from the last time that this status change occurred. 
 * Otherwise, the WaitUntil executes until the offset time past the time
 * that the test status is reached (if the target AbstractAction changes status
 * in a way that means that the test status can never be reached, then the
 * WaitUntil fails).
 * <p>
 * If the evaluation of the Time fails (target AbstractAction is not available,
 * target AbstractAction is in a status that means that the test status can never be reached),
 * then the execution of the WaitUntil fails.
 * 
 * @see Time
 * @see org.unicore.outcome.WaitUntil_Outcome
 *
 * @author Sven van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version WaitUntil.java 1.6 09/09/02
 *
 **/
public class WaitUntil extends QueryService {

    static final long serialVersionUID = 0;

    /**
     * Create a new WaitUntil.
     * <p>
     * @param name The name of the WaitUntil
     * @param extra_information Client supplied extra information about the WaitUntil
     * @param time The end time
     *
     **/
    public WaitUntil(String name,
                     byte[] extra_information,
                     Time time
                    ) {
        super(name,extra_information);
        this.time = time;
    }

    /**
     * Create a new WaitUntil.
     * <p>
     * @param name The name of the WaitUntil
     * @param expression end time
     *
     **/
    public WaitUntil(String name,
                     Time time
                    ) {
        this(name,null,time);
    }

    /**
     * Create a new WaitUntil.
     * <p>
     * @param name The name of the WaitUntil
     *
     **/
    public WaitUntil(String name) {
        this(name,null,null);
    }

    /**
     * Create an instance of WaitUntil that terminates execution immediately.
     *
     **/
    public WaitUntil() {
        this("");
    }

    private Time time;
    /**
     * Set the earliest completion time of the WaitUntil.
     *
     **/
    public void setTime(Time time) {
        this.time = time;
    }

    /**
     * @return The earliest time that the WaitUntil will complete execution
     *
     **/
    public Time getTime() {return time;}

}
//
// Change history:
//          26 Jul 2002 SvdB Allowed waits on statuses not yet entered
//          17 Apr 2002 SvdB Created for V4.0 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
