package org.unicore.idiomatic;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.resources.*;

/**
  * Copy a file in this Uspace directly to another Uspace. The source or destination (or
  * both) can be a FIFO rather than a normal file.
  *
  * @author S. van den Berghe (fecit)
  * @author D. Snelling (fecit)
  *
  * @since AJO 4.0
  *
  * @version Stream.java 1.3 04/17/03 <br>
  * 
 **/
public class Stream extends StreamFile {

    /**
     * Create a new Stream  
     *
     * @param name A name for the Stream
     * @param extra_information Storage for data from the creator of this Stream
     * @param sso security object
     * @param endorser The endorser of the AJO (see {@link org.unicore.ajo.AbstractJob#setEndorser})
     * @param source The source of the bytes
     * @param source_storage The location of the fifo 
     * @param target The destination of the bytes
     * @param target_uspace The Uspace to receive the bytes
     * @param target_vsite The destination Vsite
     * @param overwrite Whether the target fifo should overwrite an existing to file
     *
     **/
    public Stream( String name,
                   byte[] extra_information,
                   byte[] sso, 
                   User endorser,
                   String source,
                   USpace source_storage,
                   String target,
                   AlternativeUspace target_uspace,
                   Vsite target_vsite,
                   boolean overwrite
                   ) {
        super(name,extra_information,sso,endorser,source,source_storage,target,target_uspace,target_vsite,overwrite);
    }

    /**
     * Created an uninitialised Stream.
     *
     **/
    public Stream() {
        this("", (byte[]) null, (byte[]) null, (User)null, (String) null,
                  (USpace) null, (String) null, (AlternativeUspace) null,  (Vsite) null, false);
    }

    public void setSourceStorage(USpace source_storage) {
        super._setSourceStorage(source_storage);
    }

    public USpace getSourceStorage() {
        return (USpace) super._getSourceStorage();
    }

    public void setTargetStorage(AlternativeUspace target_storage) {
        super._setTargetStorage(target_storage);
    }

    public AlternativeUspace getTargetStorage() {
        return (AlternativeUspace) super._getTargetStorage();
    }


    static final long serialVersionUID = 401;
}
//
//  Change history:
//          07 Apr 2003 SvdB Rewritten
//          23 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
