package org.unicore.outcome;

import org.unicore.sets.VsiteTaskIDStatusMap;
import org.unicore.sets.VsiteEstimateMap;
import org.unicore.ajo.CheckQoS;

/**
 * The results of executing a {@link org.unicore.ajo.CheckQoS}.
 * <p>
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 * @author A. Bekele (fecit)
 *
 * @since AJO 3.6
 *
 * @version CheckQoS_Outcome.java 1.6 05/13/02
 *
 * @see org.unicore.ajo.CheckQoS
 * 
 **/
public class CheckQoS_Outcome extends CheckResources_Outcome {

    static final long serialVersionUID = -5480592491434030908L;

    private VsiteEstimateMap vsiteEstMap;

    public CheckQoS_Outcome() {
        this(null, null, null, null);
    }
    /**
     * Create a new CheckQoS_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}
     * plus the QoS report.
     *
     * @param source The CheckQoS for which this Outcome will hold the results.
     * @param initial The status of the CheckQoS at the time that the Outcome is created.
     *
     **/
    public CheckQoS_Outcome(CheckQoS    source,
                            AbstractActionStatus initial_status,
                            VsiteTaskIDStatusMap tidStatusMap,
                            VsiteEstimateMap vsiteEstMap
                            ) {
        super(source, initial_status, tidStatusMap);
        this.vsiteEstMap = vsiteEstMap;
    }

    /** 
     * Set the VsiteEstimateMap
     * @param vsiteEstMap VsiteEstimateMap
     *
     **/
    public void setVsiteEstimateMap(VsiteEstimateMap vsiteEstMap){
        this.vsiteEstMap = vsiteEstMap;
    }

    /** 
     * Returns the VsiteEstimateMap for this outcome.
     *
     * This is a map from Vsites to {@link org.unicore.sets.EstimateSet}. Each element of
     * which corresponds to an {@link org.unicore.ajo.AbstractTask} and indicates the Vsite's
     * expected QoS for that task.
     *
     * The QoS includes earliest start time, earliest end time and a cost.
     *
     **/
    public VsiteEstimateMap getVsiteEstimateMap(){
        return this.vsiteEstMap;
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          11 Dec 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
