package org.unicore.outcome;

import org.unicore.sets.VsiteTaskIDStatusMap;
import org.unicore.ajo.CheckResources;

/**
 * The result of executing a {@link org.unicore.ajo.CheckResources}.
 *
 * This class confirms whether or not requested resources by a unicore job
 * are valid at a number of alternative Vsites. It comes as a the reply to
 * a {@link org.unicore.ajo.CheckResources}. Resource requirements of
 * individual tasks in a job are expressed as a {@link org.unicore.ajo.TaskResourceDAG }
 * Accordingly the outcome to this consists of Vsite mappings to TaskIDStatusSets .
 *
 * @see org.unicore.ajo.CheckResources
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 * @author A. Bekele (fecit)
 * 
 * @since AJO 3.6
 *
 * @version CheckResources_Outcome.java 1.10 05/13/02
 * 
 **/
public class CheckResources_Outcome extends AbstractService_Outcome {

    static final long serialVersionUID = 5710252603756810898L;

    private VsiteTaskIDStatusMap tidStatusMap;

    public CheckResources_Outcome() {
        this(null, null, null);
    }

    /**
     * Create a new CheckResources_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome} 
     * plus the indicatior as to whether the resources are available
     * at the Vsite or not.
     *
     * @param source The CheckResources for which this Outcome will hold the results.
     * @param initial The status of the CheckResources at the time that the Outcome is created.
     * @param tidStatusMap Results of the CheckResources
     *
     **/
    public CheckResources_Outcome(CheckResources       source,
                                  AbstractActionStatus initial_status,
                                  VsiteTaskIDStatusMap tidStatusMap
                                      ) {
        super(source, initial_status);
        this.tidStatusMap = tidStatusMap;
    }

    /** 
     * Sets the VsiteTaskIDStatusMap of this outcome
     * @param tidStatusMap Results of the CheckResources
     **/
    public void setVsiteTaskIDStatusMap(VsiteTaskIDStatusMap tidStatusMap){
        this.tidStatusMap = tidStatusMap;
    }
    
    /** 
     * Returns the  VsiteTaskIDStatusMap of this outcome.
     * @return Results of the CheckResources
     **/
    public VsiteTaskIDStatusMap getVsiteTaskIDStatusMap(){
        return this.tidStatusMap;    
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          11 Dec 2001 DFS Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
