package org.unicore.outcome;

import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyJob;

/**
 * The result of executing a {@link org.unicore.ajo.CopyJob} which
 * is a copy of the target AbstractJob.
 *
 * @see org.unicore.ajo.CopyJob
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version CopyJob_Outcome.java 1.5 05/13/02 
 * 
 **/
public class CopyJob_Outcome extends AbstractService_Outcome {

    static final long serialVersionUID = -3864156830193061467L;

    protected AbstractJob job;

    public CopyJob_Outcome() {
        this(null, null);
    }

    /**
     * Create a new CopyJob_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome} 
     * plus a copy of the target AbstractJob.
     *
     * @param source The CopyJob for which this Outcome will hold the results.
     * @param initial The status of the CopyJob at the time that the Outcome is created.
     *
     **/
    public CopyJob_Outcome( CopyJob               source,
                            AbstractActionStatus  initial_status
                         ) {
        super(source, initial_status);
    }

    /**
     * Get the copy of the target AbstractJob.
     *
     **/
    public AbstractJob getJob() {return job;}

    /**
     * Set the copy of the target AbstractJob.
     *
     **/
    public void setJob(AbstractJob a) {job = a;}

}
//
// Change history:
//         18 Mar 2002 SvdB V4 no arg constructor
//         20 Dec 2000 SvdB Removed clone method
//          5 May 2000 SvdB Second release to Unicore
//          7 Dec 1998 SvdB Second release to Unicore
//          6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
