package org.unicore.outcome;

import org.unicore.ajo.CopyPortfolioToOutcome;

/**
 * The result of executing a {@link org.unicore.ajo.CopyPortfolioToOutcome}.
 * <p>
 * The Portfolio copy will come as streamed bytes
 * following the RetrieveOutcomeReply. The target Portfolio
 * files may be large and so placing them in a Java object
 * that will be serialised is inefficient. Consequently,
 * the data are transferred as raw bytes. The instance
 * of CopyPortfolio_Outcome is sent to maintain the rule of
 * every AbstractAction having an Outcome and to transfer the
 * status information about the execution of the CopyPortfolioToOutcome.
 *
 * @see org.unicore.ajo.CopyPortfolioToOutcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version CopyPortfolioToOutcome_Outcome.java 1.1 05/13/02 
 * 
 **/
public class CopyPortfolioToOutcome_Outcome extends AbstractTask_Outcome {

    static final long serialVersionUID = 0;

    public CopyPortfolioToOutcome_Outcome() {
        this(null, null);
    }

    /**
     * Create a new CopyPortfolioToOutcome_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}. However,
     * an instance of this class is always accompanied by UPL streamed
     * bytes which are the contents of the target Portfolio (if the execution was
     * SUCCESSFUL).
     *
     * @param source The CopyPortfolioToOutcome for which this Outcome will hold the results.
     * @param initial The status of the CopyPortfolioToOutcome at the time that the Outcome is created.
     *
     **/
    public CopyPortfolioToOutcome_Outcome( CopyPortfolioToOutcome    source,
                                           AbstractActionStatus initial_status
                               ) {
        super(source, initial_status);
    }

}
//
// Change history:
//          30 Apr 2002 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
