package org.unicore.outcome;

import org.unicore.ajo.CopyPortfolioTask;

/**
 * The result of executing a {@link org.unicore.ajo.CopyPortfolioTask}.
 * <p>
 * The Portfolio copy will come as streamed bytes
 * following the RetrieveOutcomeReply. The target Portfolio
 * files may be large and so placing them in a Java object
 * that will be serialised is inefficient. Consequently,
 * the data are transferred as raw bytes. The instance
 * of CopyPortfolio_Outcome is sent to maintain the rule of
 * every AbstractAction having an Outcome and to transfer the
 * status information about the execution of the CopyPortfolioTask.
 *
 * @see org.unicore.ajo.CopyPortfolioTask
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version CopyPortfolio_Outcome.java 1.5 05/13/02 
 * 
 **/
public class CopyPortfolio_Outcome extends AbstractTask_Outcome {

    static final long serialVersionUID = -7652731053356378624L;

    public CopyPortfolio_Outcome() {
        this(null, null);
    }

    /**
     * Create a new CopyPortfolio_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}. However,
     * an instance of this class is always accompanied by UPL streamed
     * bytes which are the contents of the target Portfolio (if the execution was
     * SUCCESSFUL).
     *
     * @param source The CopyPortfolioTask for which this Outcome will hold the results.
     * @param initial The status of the CopyPortfolioTask at the time that the Outcome is created.
     *
     **/
    public CopyPortfolio_Outcome( CopyPortfolioTask    source,
                                  AbstractActionStatus initial_status
                               ) {
        super(source, initial_status);
    }

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
