package org.unicore.outcome;

/**
 * A Decision is a String produced at run time by AbstractActions implementing
 * the {@link org.unicore.ajo.Decidable} interface. 
 * <p>
 * It reflects the conditions at 
 * the time of execution e.g. the actual time was later or earlier than the
 * test time of a {@link org.unicore.ajo.ProbeTime} or, say, that the incarnation
 * of an {@link org.unicore.ajo.ExecuteTask} could not find a file.
 * <p>
 * The Decision is examined at run time by {@link org.unicore.ajo.If} and
 * {@link org.unicore.ajo.RepeatGroup} and used in the test to determine how the
 * execution of these AbstractActions should progress.
 *
 * @see org.unicore.ajo.Decidable
 *
 * @author D. Snelling (Fujitsu Laboratories of Europe)
 * @author S. van den Berghe (Fujitsu Laboratories of Europe)
 *
 * @since AJO 4.0
 *
 * @version Decision.java 1.2 08/13/02 
 * 
 **/
public interface Decision {

    /**
     * Return the decision of the Decidable's execution
     *
     **/
    public String getDecision();

}
//
// Change history:
//           8 Aug 2002 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
