package org.unicore.outcome;

import org.unicore.ajo.ExecuteTask;

/**
 * The results of executing an {@link org.unicore.ajo.ExecuteTask}.
 * <p>
 * These results include the stdout and stderr of the incarnation of the task. These
 * are transferred as files in the streamed data with the AJO Outcome. By convention
 * the files are named "stdout" and "stderr".
 * <p>
 * The NJS will also record the {@link Decision} produced by the execution of
 * the incarnated executable. The decision can be used by the tests of
 * {@link org.unicore.ajo.If} and {@link org.unicore.ajo.RepeatGroup}
 *
 * @see org.unicore.ajo.ExecuteTask
 *
 * @author S. van den Berghe (Fujitsu Laboratories of Europe)
 *
 * @since AJO 2
 *
 * @version ExecuteTask_Outcome.java 1.8 09/09/02
 * 
 **/
public class ExecuteTask_Outcome extends AbstractTask_Outcome implements Decision {

    static final long serialVersionUID = 400;

    public ExecuteTask_Outcome() {
        this(null,null);
    }

    /**
     * Create a new ExecuteTask_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}.
     * The stdout and stderr from the task's execution (if any) are returned
     * as UPL streamed bytes.
     *
     * @param source The ExecuteTask for which this Outcome will hold the results.
     * @param initial The status of the ExecuteTask at the time that the Outcome is created.
     *
     **/
    public ExecuteTask_Outcome(ExecuteTask source, AbstractActionStatus initial_status) {
        super(source, initial_status);
    }

    private String decision = null;
    /**
     * Return the decision produced by the ExecuteTask's execution.
     * <p>
     * The NJS will incarnate an {@link org.unicore.ajo.ExecuteTask} with an environment
     * variable ($UC_DECISION_FILE) that contains the name of a Uspace file whose contents wiIl
     * be read by the NJS and placed in the Outcome following the execution of the ExecuteTask.
     * <p>
     * The contents of the Decision are arbitrary (they should of course match the test values
     * expected by any instances of {@link org.unicore.ajo.If} or {@link org.unicore.ajo.RepeatGroup} 
     * that examine the Decision).
     * <p>
     * If no Decision file is produced, then this value will be null. An empty Decision file
     * will result in a zero-length String.
     * <p>
     * The Decision file is always read if the ExecuteTask is executed (even if the ExecuteTask
     * incarnation fails in execution).
     *
     * @return The string written to the Decision file by the incarnation of the ExecuteTask
     *         (if any)
     *
     * @since AJO 4.0
     *
     **/
    public String getDecision() {
        return decision;
    }

    /**
     * 
     * @return The ExecuteTask's Decision
     *
     * @since AJO 4.0
     *
     **/
    public void setDecision(String decision) {
        this.decision = decision;
    }

    private int result_code;
    /**
     * @deprecated AJO 4.0 Use Decision
     **/
    public int getResultCode() {return result_code;}

    /**
     * @deprecated AJO 4.0 Use Decision
     *
     **/
    public void setResultCode(int result_code) {this.result_code = result_code;}

}
//
// Change history:
//          08 Jul 2002 SvdB Decision, deprecated ResultCode
//          15 Apr 2002 SvdB Moved reason from UserTask_Outcome
//          18 Mar 2002 SvdB V4 no arg constructor
//          26 Jun 2000 SvdB stdout/stderr now transferred as streamed data 
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
