package org.unicore.outcome;

import org.unicore.ajo.GetPortfolio;
import org.unicore.PortfolioIdentifier;

/**
 * The status and logs of executing a {@link org.unicore.ajo.GetPortfolio}.
 * <p>
 * The getPortfolio fetches a Portfolio from one Uspace to another.
 * There is no data to return to the user so the Outcome reports
 * on the execution of the task.
 *
 * @see org.unicore.ajo.GetPortfolio
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version GetPortfolio_Outcome.java 1.7 09/09/02 
 * 
 **/
public class GetPortfolio_Outcome extends AbstractTask_Outcome {

    static final long serialVersionUID = 4945123491427629058L;

    public GetPortfolio_Outcome() {
        this(null,null);
    }

    /**
     * Create a new GetPortfolio_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}.
     *
     * @param source The GetPortfolio for which this Outcome will hold the results.
     * @param initial The status of the GetPortfolio at the time that the Outcome is created.
     *
     **/
    public GetPortfolio_Outcome(GetPortfolio source, AbstractActionStatus initial) {
        super(source, initial);
    }

    private String[] written_files;
    /**
     * Get the list of file names copied by the GetPortfolio.
     *
     **/
    public String[] getWrittenFiles() {return written_files;}

    /**
     * Set the list of file names copied by the GetPortfolio.
     *
     **/
    public void setWrittenFiles(String[] files) {written_files = files;}

    /**
     * Add a file name to the list of files copied by the GetPortfolio.
     *
     **/
    public void addWrittenFile(String file) {
        if(written_files == null) {
            written_files = new String[1];
            written_files[1] = file;
        }
        else {
            String[] temp = new String[written_files.length+1];
            System.arraycopy(written_files,0,temp,0,written_files.length);
            temp[written_files.length] = file;
            written_files = temp;
        }
    }

    private PortfolioIdentifier portfolio;
    /**
     * The Portfolio that was get
     *
     **/
    public PortfolioIdentifier getPortfolio() {return portfolio;}

    public void setPortfolio(PortfolioIdentifier portfolio) {this.portfolio = portfolio;}
}
//
// Change history:
//          15 Jul 2002 SvdB Added Portfolio and files information
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB Created  
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
