package org.unicore.outcome;

import org.unicore.ajo.ProbeTime;

/**
 * The result of executing a {@link org.unicore.ajo.ProbeTime}.
 *
 *
 * @see org.unicore.ajo.ProbeTime
 *
 * @author D. Snelling (Fujitsu Laboratories of Europe)
 * @author S. van den Berghe (Fujitsu Laboratories of Europe)
 * 
 * @since AJO 4.0
 *
 * @version ProbeTime_Outcome.java 1.2 08/13/02
 * 
 **/
public class ProbeTime_Outcome extends AbstractService_Outcome implements Decision {

    static final long serialVersionUID = 400;

    public static String LATER   = "later";
    public static String EARLIER = "earlier";

    public ProbeTime_Outcome() {
        this(null, null);
    }

    /**
     * Create a new ProbeTime_Outcome.
     * <p>
     * @param source The ProbeTime for which this Outcome will hold the results.
     * @param initial The status of the ProbeTime at the time that the Outcome is created.
     *
     **/
    public ProbeTime_Outcome(ProbeTime source, AbstractActionStatus initial_status) {
        super(source, initial_status);
    }

    private String decision = null;
    /**
     * Return the decision produced by the ProbeTime's execution.
     *
     * @return ProbeTime_Outcome.LATER or ProbeTime_Outcome.EARLIER or
     *         null (never executed, failed in incarnation etc)
     *
     **/
    public String getDecision() {
        return decision;
    }

    /**
     * The time of the ProbeTime's execution was later than the
     * input time.
     *
     **/
    public void later() {
        decision = LATER;
    }

    /**
     * The time of the ProbeTime's execution was earlier than the
     * input time.
     *
     **/
    public void earlier() {
        decision = EARLIER;
    }

}
//
// Change history:
//          08 Aug 2002 SvdB Added Decision
//          13 Jun 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
