package org.unicore.outcome;

import org.unicore.ajo.PutPortfolio;
import org.unicore.PortfolioIdentifier;

/**
 * The status and logs of executing a {@link org.unicore.ajo.PutPortfolio}.
 *
 * @see org.unicore.ajo.PutPortfolio
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version PutPortfolio_Outcome.java 1.7 09/09/02 
 * 
 **/
public class PutPortfolio_Outcome extends AbstractTask_Outcome {

    static final long serialVersionUID = 6802723053800135058L;

    public PutPortfolio_Outcome() {
        this(null,null);
    }

    /**
     * Create a new PutPortfolio_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}.
     *
     * @param source The PutPortfolio for which this Outcome will hold the results.
     * @param initial The status of the PutPortfolio at the time that the Outcome is created.
     *
     **/
    public PutPortfolio_Outcome(PutPortfolio source, AbstractActionStatus initial) {
        super(source, initial);
    }

    private String[] written_files;
    /**
     * Get the list of file names copied by the PutPortfolio.
     *
     **/
    public String[] getWrittenFiles() {return written_files;}

    /**
     * Set the list of file names copied by the PutPortfolio.
     *
     **/
    public void setWrittenFiles(String[] files) {written_files = files;}

    /**
     * Add a file name to the list of files copied by the PutPortfolio.
     *
     **/
    public void addWrittenFile(String file) {
        if(written_files == null) {
            written_files = new String[1];
            written_files[1] = file;
        }
        else {
            String[] temp = new String[written_files.length+1];
            System.arraycopy(written_files,0,temp,0,written_files.length);
            temp[written_files.length] = file;
            written_files = temp;
        }
    }

    private PortfolioIdentifier portfolio;
    /**
     * The Portfolio that was put
     *
     **/
    public PortfolioIdentifier getPortfolio() {return portfolio;}

    public void setPortfolio(PortfolioIdentifier portfolio) {this.portfolio = portfolio;}

}
//
// Change history:
//          15 Jul 2002 SvdB Added Portfolio and files information
//          18 Mar 2002 SvdB V4 no arg constructor
//           5 May 2000 SvdB Created  
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
