package org.unicore.outcome;

import org.unicore.AAIdentifier;
import org.unicore.Unicore;

/**
 * This class is used to indicate the resource requirements status of an Abstract Action.
 * An Abstract Action ID is associated here with a status parameter that represents
 * whether or not the resources requested by this Abstract Action are valid at the Unicore
 * Vsite.
 *
 * @author A. Bekele (fecit)
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.6
 *
 * @version TaskIDStatus.java 1.3 05/13/02
 *
 **/
public class TaskIDStatus extends Unicore {

    static final long serialVersionUID = -3892172886669368048L;

    private AAIdentifier taskID;    
    private boolean taskStatus;
        
    public TaskIDStatus () {
        this(null,false);
    }

    /**
     * Creates new TaskIDStatus object.
     *
     * @param taskID Abstract Action Identifier
     * @param taskStatus Indicates wheather the requested resource requirements for this task can be met.
     *
     **/
    public TaskIDStatus (AAIdentifier taskID, boolean taskStatus) {
        this.taskID = taskID;
        this.taskStatus  = taskStatus;
    }

    /**
     * Returns the Identifier for this Abstract Action
     *
     * @return Abstract Action Identifier
     *
     **/
    public AAIdentifier getTaskID(){
        return this.taskID;
    }

    /** 
     * Returns the status indicating wheather the requested resource
     * requirements for this task can be met.
     *
     * @return status indicator.
     *
     **/
    public boolean getTaskStatus(){
        return this.taskStatus;
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//           6 Dec 2001 DFS Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
