package org.unicore.resources;

/**
 * Resource for access to users home file space at the Vsite.
 * <p>
 * i.e. The storage root file path component is (usually) $HOME
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version Home.java 1.11 09/09/02
 *
 **/
public final class Home extends PathedStorage {

    static final long serialVersionUID = -6714677681388572064L;

    public Home () {
        this("",0.0,0.0,0.0,(String)null);
    }

    /**
     * Create a new Home.
     *
     * @param description A description of the Home
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param subspace The storage subspace
     *
     * @deprecated AJO 4.0 Subspaces are no longer part of the file system model
     *
     **/
    public Home (String description,
                 double maxrequest,
                 double defaultrequest, 
                 double minrequest,
                 SubspaceValue subspace) {
        this(description, maxrequest, defaultrequest, minrequest, (String)null);
        setSubspace(subspace);
    }

    /**
     * Create a new Home.
     *
     * @param description A description of the Home
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param path Path to the subdirectory within the home filesystem (relative to $HOME)
     *
     * @since AJO 4.0
     *
     **/
    public Home (String description,
                 double maxrequest,
                 double defaultrequest, 
                 double minrequest,
                 String path) {
        super(description, maxrequest, defaultrequest, minrequest, path);
        setName("Home");
    }

}
//
// Change history:
//          15 Mar 2002 SvdB To V4 - no subspaces , no arg constructor
//          14 Aug 2000 DFS Added minimum request.
//          21 Jan 2000 DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
