package org.unicore.resources;

/**
 * A storage area that has subdirectories.
 * <p>
 * The complete path to files referred to by AbstractTasks containing
 * instances of PathedStorage has up to 3 components. The first is the
 * "storage root", which is the path to the storage e.g. for Uspaces 
 * this is the (local) path to the pace that Uspaces are stored, for
 * bsolute this is "/". The second component is the path found in the
 * instance of PathedStorage i.e. the subdirectory within the Storage.
 * The last component is the file name from the AbstractTAsk's fields
 * or the incarnated contents of a Portfolio.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @version PathedStorage.java 1.3 06/19/02
 *
 * @since AJO 4.0
 *
 **/
public abstract class PathedStorage extends Storage {

    static final long serialVersionUID = 0;

    protected PathedStorage() {
        this("", 0.0, 0.0, 0.0, null);
    }

    /**
     * Create a new Spool.
     *
     * @param description A description of the Spool
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param path The path to the required directory relative to the storage root
     *
     **/
    protected PathedStorage(String description, 
                    double maxrequest, 
                    double defaultrequest, 
                    double minrequest,
                    String path ) {
        super (description, maxrequest, defaultrequest, minrequest);
        setPath(path);
    }

    private String path = "";

    /**
     *
     * @param path The path to the required directory relative to the storage root
     *
     **/
    public void setPath(String path) {
        if(path == null) {
            this.path = "";
        }
        else {
            this.path = path;
        }
    }

    /**
     * @return The path to the required directory relative to the storage root
     *
     **/
    public String getPath() {return path;}

}
//
// Change history:
//          18 Jun 2002 SvdB No hashcode
//          20 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
