package org.unicore.resources;

/**
 * Resource for somewhere to put and manipulate files.
 * <p>
 * Different types of file storage are represented by the
 * subtypes of Storage, these include some that have meaning
 * within the Unicore model, {@link Spool}, {@link USpace}, {@link AlternativeUspace},
 * and others that are divisions of teh Vsite's Xspace, e.g. {@link Home}.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @version Storage.java 1.12 09/09/02
 *
 **/
public abstract class Storage extends CapacityResource {

    static final long serialVersionUID = -8718475977459202401L;

    protected Storage() {
        this("", 0.0, 0.0, 0.0);
    }

    /**
     * Create a new Storage.
     * <p>
     * Sets the units to "MegaBytes"
     *
     * @param description A description of the Storage
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param subspace The storage subspace
     *
     * @deprecated AJO 4.0 Subspaces are no longer part of the file system model
     *
     **/
    protected Storage (String description, 
                    double maxrequest, 
                    double defaultrequest, 
                    double minrequest,
                    SubspaceValue subspace) {
        super (description, maxrequest, defaultrequest, minrequest);
        setSubspace(subspace);
        this.units    = "MegaBytes";
    }

    /**
     * Create a new Storage.
     * <p>
     * Sets the units to "MegaBytes"
     *
     * @since AJO 4.0
     *
     * @param description A description of the Storage
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     *
     **/
    protected Storage (String description, 
                    double maxrequest, 
                    double defaultrequest, 
                    double minrequest ) {
        super (description, maxrequest, defaultrequest, minrequest);
        this.units    = "MegaBytes";
    }

    private SubspaceValue subspace = SubspaceValue.HOME;
    /**
     * Return the subspace to use
     *
     * @deprecated AJO 4.0 Subspaces are no longer part of the file system model
     *
     **/
    public SubspaceValue getSubspace () { return subspace; }

    /**
     * Set the subspace 
     *
     * @deprecated AJO 4.0 Subspaces are no longer part of the file system model
     *
     **/
    public void setSubspace (SubspaceValue subspace) { 
        if(subspace == null) {
            this.subspace = SubspaceValue.HOME;
        }
        else {
            this.subspace = subspace; 
        }    
    }

    private String name;

    protected void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public String toString() {
        return name;
    }

}
//
// Change history:
//          18 Mar 2002 SvdB V4, no arg, deprecated subspaces, added path
//          20022002     SvdB subspace never null
//          14 Aug 2000 DFS Added minimum request.
//          13 Jul 2000  SvdB override equals
//          21 Jan 2000  DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
