package org.unicore.resources;

/**
 * Resource for StorageServers such as bulk storage and tape archives and 
 * backup facilities as well as convential file systems not modelled by
 * other classes.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version StorageServer.java 1.13 09/09/02 
 *
 **/
public class StorageServer extends PathedStorage {

    static final long serialVersionUID = -1531669629777247147L;

    public StorageServer () {
        this("", 0.0, 0.0, 0.0, (String)null, (String)null);
    }

    /**
     * Create a new StorageServer.
     *
     * @param description A description of the StorageServer
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param subspace The storage subspace
     * @param name Name of the storage server
     *
     * @deprecated AJO 4.0 Subspaces are no longer part of the file system model
     *
     **/
    public StorageServer (String description,
                          double maxrequest,
                          double defaultrequest,
                          double minrequest,
                          SubspaceValue subspace,
                          String server_name) {

        this(description, maxrequest, defaultrequest, minrequest, (String)null, server_name);
        setSubspace(subspace);
    }

    /**
     * Create a new StorageServer.
     *
     * @param description A description of the StorageServer
     * @param maxrequest Maximum amount of storage allowed (in megabytes)
     * @param defaultrequest Amount of storage granted if no specific request received (in megabytes)
     * @param minrequest Minimum amount of storage allowed (in megabytes)
     * @param path The subdirectory within the StorageServer
     * @param name Name of the storage server
     *
     * @since AJO 4.0
     *
     **/
    public StorageServer (String description,
                          double maxrequest,
                          double defaultrequest,
                          double minrequest,
                          String path,
                          String server_name) {

        super(description, maxrequest, defaultrequest, minrequest, path);
        setName(server_name);
    }

    /**
     * Set the name of the storage server.
     *
     **/
    public void setName (String server_name) { 
        if(server_name == null) {
            super.setName("StorageServer");
        }
        else {
            super.setName(server_name); 
        }
    }

    /**
     * StorageServers are equals if they have the same name
     *
     **/
    public boolean equals(Object o) {
        if(o == null) return false;
        if(o.getClass() == getClass()) {
            return ((StorageServer)o).getName().equals(getName());
        }
        else {
            return false;
        }
    }

    /**
     * Return a hash code derived from the name of the StorageServer.
     *
     **/
    public int hashCode() {return getName().hashCode();}
}
//
// Change history:
//          18 Mar 2002 SvdB V4, no arg constructor, no subspaces
//          20022002    SvdB server_name never null
//          14 Aug 2000 DFS Added minimum request.
//          13 Jul 2000 SvdB override equals 
//          21 Jan 2000  DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
