package org.gridschool.unicore.plugins.boltzmann;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.Serializable;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;

import com.pallas.unicore.client.panels.Applyable;
import com.pallas.unicore.client.panels.ImagePanel;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.requests.GetFilesFromUspace;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;

/**
 * Additional outcome panel to visualize boltzmann sample file.
 *
 *@author Ralf Ratering
 *@version    $Id: BoltzmannSamplePanel.java,v 1.1 2003/07/12 16:01:26 rrate Exp $
 */
public class BoltzmannSamplePanel extends JPanel implements Applyable, IObserver {
    static final long serialVersionUID = 9175578907213854759L;

	private static Logger logger =
			Logger.getLogger("org.gridschool.unicore.plugins.boltzmann");

	// filename as written by application into USpace
	private static final String SAMPLE_FILE = "sample.gif";

	// visualize image with client image panel
	private ImagePanel imagePanel = new ImagePanel(true);
						
    /**
     * Constructor
     */
    public BoltzmannSamplePanel() {
    	buildComponents();
    }

	private void buildComponents() {
		JPanel topPanel = new JPanel();

		JButton getSampleButton = new JButton("Get Sample");
		getSampleButton.addActionListener(new GetSampleListener());
		topPanel.add(getSampleButton);

		setLayout(new BorderLayout());
    	add(topPanel, BorderLayout.NORTH);
		add(new JScrollPane(imagePanel), BorderLayout.CENTER);
    }

    /**
     * Method of interface Applyable
     * Unused here
     */
	public void applyValues() {
		logger.info("Applying values in outcome panel...");
	}

	/**
	 * Method of interface Applyable
	 * Called whenever new outcome at client arrives
	 */
	 public void resetValues() {
		logger.info("Resetting values in outcome panel...");
    }

    /**
     * Method of interface Applyable
     * Called whenever panel becomes visible
     */
	public void updateValues() {
		logger.info("Updating values in outcome panel...");
    }
 	
 	/**
  	* Get parent JobContainer from JMC tree
  	* @return parent job container
  	*/
    private JobContainer getJobContainer() {
    	return
	    	ResourceManager.getCurrentInstance().getJMCTree().getCurrentJob();
    }

    
	/**
	* Get plugin container from JMC tree
	* @return plugin container
	*/
    private BoltzmannContainer getBoltzmannContainer() {
		return (BoltzmannContainer)
			ResourceManager.getCurrentInstance().getJMCTree().getFocussedObject();
    }

	/**
	 * 
	 * The Boltzmann application will store intermediate results in
	 * a file called sample.gif. We want to transfer this file to
	 * the Client while the job is still running, so we have to
	 * wrap it inside a Portfolio, because we will use a
	 * CopyPortfolioToOutcome for the transfer.
     */	
    private void getSampleFile() {
    	logger.info("Getting file sample.gif");
    	JobContainer jobContainer = getJobContainer();
	   	AJOIdentifier ajoId = (AJOIdentifier)jobContainer.getIdentifier();
		Vsite vsite = jobContainer.getVsite();
		String[] filenames = {SAMPLE_FILE};
		
		//@TODO send com.pallas.unicore.requests.GetFilesFromUspace request here
	}

	/**
	 * Method from IObserver interface
	 * This class should be observer for the GetFilesFromUspace request
	 * to get a notification when the request returned.
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {
		if (theObserved instanceof GetFilesFromUspace) {
			logger.info("GetFileFromUspace request returned.");
			// SUCCESS! Now get the outcome and implicitly unspool outcome files
			AbstractJob_Outcome outcome = (AbstractJob_Outcome)changeCode;
			logger.info("GetFileFromUspace result: " + outcome.getStatus());
			if(outcome.getStatus().isEquivalent(AbstractActionStatus.SUCCESSFUL)) {
				Vector filenames = OutcomeManager.getStreamedFiles();
				for(int i=0; i<filenames.size(); i++) {
					String filename =  
						Connection.getSessionDir() + 
						File.separator + filenames.elementAt(i);
					logger.info("Filename: " + filename);
					if(filename.endsWith(SAMPLE_FILE)) {
						Image image = Toolkit.getDefaultToolkit().createImage(filename);
						imagePanel.setImage(image);
						imagePanel.repaint();
						return;
					}
				}
			}		
		}
	}

	/**
	 *  Listener for get sample button
	 */
	private class GetSampleListener implements ActionListener, Serializable {
		public void actionPerformed(ActionEvent e) {
			getSampleFile();
		}
	}
}
